/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.dav;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Die Klasse zum Versenden von Daten an bestimmte Objekte. Sie implentiert das ClientSenderInterface welches die
 * Methoden zum Versenden der Daten bereitstellt.
 * 
 * @author Pittner
 * 
 */
public class DaVSender implements ClientSenderInterface
{

  /**
   * Die Connection
   */
  private ClientDavInterface _connection;

  /**
   * Die Sender Rolle
   */
  private SenderRole _role;

  /**
   * Konstruktor der Klasse
   * 
   * @param connection
   * @param role
   */
  public DaVSender(ClientDavInterface connection, SenderRole role)
  {

    if (connection == null || role == null)
      System.err.println(new IllegalArgumentException().toString());

    _connection = connection;
    _role = role;
  }

  /**
   * Das Ergebis der Anfrage wird ermittelt, aufbereitet und versendet
   * 
   * @return true, wenn das Versenden der Daten erfolgreich war
   */
  public Boolean sende(SystemObject so, AttributeGroup atg, Aspect asp, Data data, Short variant)
  {

    if (so == null || atg == null | asp == null)
      System.err.println(new IllegalArgumentException().getMessage());

    DataDescription dd = new DataDescription(atg, asp, variant);

    ResultData resultData = new ResultData(so, dd, _connection.getTime(), data);

    try
    {
      _connection.sendData(resultData);
    }
    catch (DataNotSubscribedException e)
    {
      // TODO Auto-generated catch block
      try
      {
        _connection.subscribeSender(this, resultData.getObject(), dd, _role);
      }
      catch (OneSubscriptionPerSendData e1)
      {
        // TODO Auto-generated catch block
        System.err.println(e1.toString());
      }
      try
      {
        _connection.sendData(resultData);
      }
      catch (DataNotSubscribedException e1)
      {
        // TODO Auto-generated catch block
        System.err.println(e1.toString());
      }
      catch (SendSubscriptionNotConfirmed e1)
      {
        // TODO Auto-generated catch block
        System.err.println(e1.toString());
      }
    }
    catch (SendSubscriptionNotConfirmed e)
    {
      // TODO Auto-generated catch block
      System.err.println(e.toString());
    }
    catch (Exception e)
    {

      System.err.println(e.toString());

    }

    return true;

  }

  /**
   * Abstrakte Methode, die implementiert werden muss: bernimmt das tatschliche Senden der Daten
   */
  public void dataRequest(SystemObject object, DataDescription dataDescription, byte state)
  {
    
  }

  /**
   * Abstrakte Methode, die implementiert werden muss: Zeigt, ob die Sendesteuerung erwnscht ist.
   */
  public boolean isRequestSupported(SystemObject so, DataDescription dd)
  {
    return true;
  }

}
