/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.dobj.editparts;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.util.EList;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.editpolicies.ResizableEditPolicy;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.PlatformUI;

import com.bitctrl.lib.eclipse.draw2d.FigureUtilities2;

import de.bsvrz.buv.plugin.anlagendarstellung.AnlagendarstellungPackage;
import de.bsvrz.buv.plugin.anlagendarstellung.DetektorLVE;
import de.bsvrz.buv.plugin.anlagendarstellung.DoModelMitVerbLinie;
import de.bsvrz.buv.plugin.anlagendarstellung.FahrStreifenModel;
import de.bsvrz.buv.plugin.anlagendarstellung.FahrStreifenModelAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.FahrstreifenFigure;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.LegendeDefinitionVerkehrsLage;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.ILegendeBaustein2;
import de.bsvrz.buv.plugin.anlagendarstellung.model.SituationTyp;
import de.bsvrz.buv.plugin.anlagendarstellung.util.LegendeHelper;
import de.bsvrz.buv.plugin.darstellung.model.Darstellung;
import de.bsvrz.buv.plugin.dobj.internal.DarstellungExtensionPoint;
import de.bsvrz.buv.plugin.dobj.model.ConfiguratedDoTyp;
import de.bsvrz.buv.plugin.dobj.model.DoModel;
import de.bsvrz.buv.plugin.dobj.model.DoTyp;
import de.bsvrz.buv.rw.bitctrl.eclipse.legende.ILegendeBaustein;
import de.bsvrz.buv.rw.bitctrl.eclipse.legende.LegendeBaustein;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.AttributeGroup;

/**
 * Implementierung fr einen {@link DoMitVerbLinieEditPart} der ein Model-Objekt 
 * vom Typ {@link FahrStreifenModelAbstract} verwendet.
 * 
 * @author Pittner
 * 
 */
public class FahrStreifenEditPart extends DoMitVerbLinieEditPart implements ILegendeBaustein2
{

  private SituationTyp _situation;
  private boolean _passiviert = false;
  private Image image;
  private ArrayList<ILegendeBaustein> legendenBausteine;

  @Override
  protected List getModelChildren()
  {
    if (getCastedModel().getKomponenten() != null && getCastedModel().getKomponenten().size() > 0)
    {
      return getCastedModel().getKomponenten();
    }
    else
    {
      ConfiguratedDoTyp doTyp = DarstellungExtensionPoint.getDoTyp("DetektorLveEditPart");
      Darstellung darstellung = (Darstellung)getViewer().getProperty(Darstellung.class.toString());
      EList<DoTyp> doTypen = darstellung.getDoTypen();
      boolean vorhanden = false;
      for (DoTyp typ : doTypen)
      {
        if (typ instanceof ConfiguratedDoTyp)
        {
          ConfiguratedDoTyp co = (ConfiguratedDoTyp)typ;
          if (co.getId().equals(doTyp.getId()))
          {
            vorhanden = true;
            doTyp = co;
          }
        }
      }

      DetektorLVE d1 = (DetektorLVE)doTyp.createModel();
      d1.setLocation(new Point(1, 21));
      getCastedModel().getKomponenten().add(d1);

      if (!vorhanden)
      {
        doTypen.add(doTyp);
      }

      return getCastedModel().getKomponenten();
    }
  }

  @Override
  protected IFigure createFigure()
  {
    DoModelMitVerbLinie model = getModel();    
    Point pos = getModel().getLocation();
    Dimension size = getCastedModel().getSize();
    if (size == null)
    {
      size = new Dimension(14, 50);
      getCastedModel().setSize(size);
    }

    FahrstreifenFigure fig = new FahrstreifenFigure("");
    fig.setSize(size);

//    DoModelMitVerbLinie target = getModel().getTarget();
//
//    if (target != null)
//    {             
//      createTargetConnection(getModel(), target);        
//    }
//
    addToolTipListener(fig);

    return fig;
  }

  @Override
  public void notifyChanged(Notification notification)
  {

    super.notifyChanged(notification);

    final Object notifier = notification.getNotifier();
    final int type = notification.getEventType();
    if (notifier instanceof FahrStreifenModel)
    {
      final int featureID = notification.getFeatureID(DoModel.class);

      switch (type)
      {
        case Notification.SET:
          switch (featureID)
          {
            case AnlagendarstellungPackage.FAHR_STREIFEN_MODEL_ABSTRACT__SIZE:
              refresh();
              break;
            case AnlagendarstellungPackage.FAHR_STREIFEN_MODEL_ABSTRACT__LOCATION:
              refresh();
              break;
          }

          break;
        default:
      }
    }

    if (AnlagendarstellungPackage.Literals.FAHR_STREIFEN_MODEL_ABSTRACT__BEZEICHNUNG.equals(notification.getFeature()))
    {
    }
    else if (AnlagendarstellungPackage.Literals.FAHR_STREIFEN_MODEL_ABSTRACT__KONTEXT.equals(notification.getFeature()))
    {
    }
    else if (AnlagendarstellungPackage.Literals.FAHR_STREIFEN_MODEL_ABSTRACT__DATA_UPDATE.equals(notification
        .getFeature()))
    {
      ResultData result = (ResultData)notification.getNewValue();
      dataUpdateFahrStreifen(result);
    }
    else if (AnlagendarstellungPackage.Literals.FAHR_STREIFEN_MODEL_ABSTRACT__DARSTELLUNG_UPDATE.equals(notification
        .getFeature()))
    {
      String s = (String)notification.getNewValue();
      getFigure().setToolTip(new Label(s));
    }

  }

  @Override
  public void zoomChanged(double zoom)
  {
    super.zoomChanged(zoom);
    getFigure().repaint();
  }

  /**
   * We don't need to explicitly handle refresh visuals because the times when this needs to be done it is handled by
   * the table e.g. handleNameChange()
   */
  protected void refreshVisuals()
  {

    Point pos = getCastedModel().getLocation();
    Dimension size = getCastedModel().getSize();
    getFigure().setLocation(pos);
    if (size != null)
      getFigure().setSize(size);

    getFigure().setBackgroundColor(ColorConstants.gray);
  }

  @Override
  protected void createEditPolicies()
  {
    super.createEditPolicies();
    installEditPolicy(EditPolicy.LAYOUT_ROLE, new ResizableEditPolicy());
  }

  private FahrStreifenModelAbstract getCastedModel()
  {

    FahrStreifenModelAbstract model = null;

    if (getModel() instanceof FahrStreifenModelAbstract)
    {
      model = (FahrStreifenModelAbstract)getModel();
    }

    return model;
  }
  
  protected void dataUpdateFahrStreifen(ResultData result)
  {
    final IFigure inhalt = getFigure();

    AttributeGroup atg = result.getDataDescription().getAttributeGroup();

    Data data = result.getData();
    if (atg.getPid().equals("atg.strfallZustand"))
    {
      if (data != null)
      {

        String s = data.getItem("Situation").asTextValue().getValueText();
        _situation = SituationTyp.setTyp(s);
      }
      else
      {       
        _situation = SituationTyp.setTyp(SituationTyp.KEINE_AUSSAGE.getType());
      }
      situationChange(inhalt);
    }     
    else if ("atg.passivierungFs".equals(atg.getPid()))
    {      
      if (data != null)
      {
        String value = data.getItem("passiviert").asTextValue().getValueText();
        _passiviert = value.equalsIgnoreCase("Ja");
      }
      else
      {     
        _passiviert = false;
      }  
      situationChange(inhalt);
     }
  }
  
  private void situationChange(final IFigure inhalt)
  {
    PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
      @Override
      public void run()
      {
        boolean zeigeWeiss = isPassiviert();
        
        if (zeigeWeiss)
        {
          inhalt.setBackgroundColor(ColorConstants.white);
          inhalt.setBorder(new LineBorder(ColorConstants.white, 1));
        }
        else if (_situation.getType().equals(SituationTyp.STOERUNG.getType()))
        {
          inhalt.setBackgroundColor(ColorConstants.gray);
          inhalt.setBorder(new LineBorder(ColorConstants.gray, 1));
        }
        else if (_situation.getType().equals(SituationTyp.KEINE_AUSSAGE.getType()))
        {
          inhalt.setBackgroundColor(ColorConstants.gray);
          inhalt.setBorder(new LineBorder(ColorConstants.gray, 1));
        }
        else if (_situation.getType().equals(SituationTyp.VERKEHR_FREI.getType()))
        {
          inhalt.setBackgroundColor(ColorConstants.lightGreen);
          inhalt.setBorder(new LineBorder(ColorConstants.lightGreen, 1));
        }
        else if (_situation.getType().equals(SituationTyp.VERKEHR_LEBHAFT.getType()))
        {
          inhalt.setBackgroundColor(ColorConstants.green);
          inhalt.setBorder(new LineBorder(ColorConstants.green, 1));
        }
        else if (_situation.getType().equals(SituationTyp.VERKEHR_DICHT.getType()))
        {
          inhalt.setBackgroundColor(ColorConstants.yellow);
          inhalt.setBorder(new LineBorder(ColorConstants.yellow, 1));
        }
        else if (_situation.getType().equals(SituationTyp.VERKEHR_STOCKEND.getType()))
        {
          inhalt.setBackgroundColor(ColorConstants.red);
          inhalt.setBorder(new LineBorder(ColorConstants.red, 1));
        }
        else if (_situation.getType().equals(SituationTyp.VERKEHR_ZAEHFLIESSEND.getType()))
        {
          inhalt.setBackgroundColor(ColorConstants.orange);
          inhalt.setBorder(new LineBorder(ColorConstants.orange, 1));
        }
        else if (_situation.getType().equals(SituationTyp.STAU.getType()))
        {
          inhalt.setBackgroundColor(ColorConstants.red);
          inhalt.setBorder(new LineBorder(ColorConstants.red, 1));
        }
      }
    });
  }

  
  private boolean isPassiviert()
  {
    // TODO Auto-generated method stub
    return _passiviert;
  }

  /** {@inheritDoc} */
  @Override
  public List<ILegendeBaustein> getBausteine() {
    if (legendenBausteine == null) {
      legendenBausteine = new ArrayList<ILegendeBaustein>();
      
      for (Object listObj : getDefinition())
      {
        legendenBausteine.addAll(LegendeHelper.erzeugeLegendeBausteine(listObj));        
      }
      
    }

    return legendenBausteine;
  }

  @Override
  public Image getIcon()
  {
//    // TODO Auto-generated method stub    
//    System.out.println("FahrStreifenEditPart.getIcon(): " + image);    
//    if (image == null) 
//    {
//      if (getFigure().getSize() != null) {
//        image = FigureUtilities2.createScaledImage(getFigure(), getFigure().getSize(),
//            new Dimension(16, 16));
//      } else {
//        image = FigureUtilities2.createImage(getFigure(), new Dimension(16,
//            16));
//      }
////      System.out.println("FahrStreifenEditPart.getIcon(): " + image.getBounds());
//    }
//    return image;
    return null;
  }

  @Override
  public String getText()
  {
    // TODO Auto-generated method stub
    return "Fahrstreifen";
  }

  @Override
  public String getId()
  {
    // TODO Auto-generated method stub
    return "Fahrstreifen";
  }
  
  @Override
  public List getDefinition()
  {
    List list = new ArrayList();
    int kontext = 0;
    // Hier in diesem Fall hngt die Information des Kontexts am Viewer, da das DoTyp keine Kontextinformationen hat
    Control control = getViewer().getControl();
    if (control != null){
      Object data = control.getData("kontext");
      if (data instanceof Integer)
      {
        kontext = (Integer)data;        
      }
    }    
    list.addAll(LegendeHelper.erzeugeFsLegendeDefinition(getCastedModel(), kontext));
    return list;
  }
  
//  public boolean isStandStreifen()
//  {
//    ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
//    DataModel dataModel = connection.getDataModel();
//    if (connection != null)
//    {
//      SystemObject fs = getModel().getSystemObject();
//      if (fs instanceof ConfigurationObject)
//      { 
//        AttributeGroup atg = dataModel.getAttributeGroup("atg.fahrStreifen");
//        Data data = ((ConfigurationObject)fs).getConfigurationData(atg);
//        if (data != null)
//        {
//          return 6 == data.getItem("Lage").asUnscaledValue().intValue();
//        }
//      } 
//    }
//    
//    return false;
//  }

}
