/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.dobj.editparts;

import java.util.List;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.PointList;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.editpolicies.NonResizableEditPolicy;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.plugin.anlagendarstellung.AnlagendarstellungPackage;
import de.bsvrz.buv.plugin.anlagendarstellung.DoModelMitVerbLinie;
import de.bsvrz.buv.plugin.anlagendarstellung.FreeFormFahrStreifenModel;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.FreeFormFahrStreifenFigure;
import de.bsvrz.buv.plugin.anlagendarstellung.model.SituationTyp;
import de.bsvrz.buv.plugin.dobj.model.DoModel;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.AttributeGroup;

/**
 * Implementierung fr einen {@link DoMitVerbLinieEditPart} der ein Model-Objekt 
 * vom Typ {@link FreeFormFahrStreifenModel} verwendet.
 * 
 * @author Pittner
 * 
 */
public class FreeFormPolygonEditPart extends FahrStreifenEditPart
{
  @Override
  protected IFigure createFigure()
  {
    Point pos = getModel().getLocation();

    PointList pl = getCastedModel().getPunkte();
    if (pl == null)
    {
      pl = new PointList(new int[] { 0, 0, 25, 0, 25, 40, 0, 40 });
      getCastedModel().setPunkte(pl);
    }

    FreeFormFahrStreifenFigure fig = new FreeFormFahrStreifenFigure("", pos, pl);

    RGB rgb = (RGB)getCastedModel().getBackground();

    if (rgb != null)
      fig.setColor(new Color(null, rgb));

    Dimension size = getCastedModel().getSize();
    if (size == null)
    {
      size = fig.getSize();
      getCastedModel().setSize(size);
    }

    addToolTipListener(fig);

    return fig;
  }

  @Override
  public void notifyChanged(Notification notification)
  {
    super.notifyChanged(notification);

    final Object notifier = notification.getNotifier();
    final int type = notification.getEventType();

    FreeFormFahrStreifenFigure fig = (FreeFormFahrStreifenFigure)getFigure();

    if (notifier instanceof FreeFormFahrStreifenFigure)
    {
      final int featureID = notification.getFeatureID(DoModel.class);

      switch (type)
      {
        case Notification.SET:
          switch (featureID)
          {
            case AnlagendarstellungPackage.FAHR_STREIFEN_MODEL_ABSTRACT__LOCATION:
              refresh();
              break;
          }

          break;
        default:
      }
    }

    if (AnlagendarstellungPackage.Literals.FREE_FORM_FAHR_STREIFEN_MODEL__PUNKTE.equals(notification.getFeature()))
    {
      Point pos = getModel().getLocation();
      Dimension size = getCastedModel().getSize();

      PointList pl = (PointList)notification.getNewValue();
      fig.createPointList(pl, pos);
      fig.setSize(size);

      refresh();
    }
    else if (AnlagendarstellungPackage.Literals.FREE_FORM_FAHR_STREIFEN_MODEL__BACKGROUND.equals(notification
        .getFeature()))
    {
      Object obj = notification.getNewValue();
      if (obj instanceof RGB)
      {
        RGB rgb = (RGB)obj;
        ((FreeFormFahrStreifenFigure)getFigure()).setColor(new Color(null, rgb));
      }
      getFigure().repaint();
    }    
  }

  /**
   * We don't need to explicitly handle refresh visuals because the times when this needs to be done it is handled by
   * the table e.g. handleNameChange()
   */
  protected void refreshVisuals()
  {
    Point pos = getModel().getLocation();
    Dimension size = getCastedModel().getSize();

    FreeFormFahrStreifenFigure fig = (FreeFormFahrStreifenFigure)getFigure();
    PointList pl = fig.getPointList();
    fig.createPointList(pl, pos);

    if (size != null)
    {
      fig.setSize(size);
    }

    getFigure().setLocation(pos);
    if (size != null)
      getFigure().setSize(size);

    List list = getChildren();
    if (list != null)
    {
      for (Object object : list)
      {
        if (object instanceof DetektorLveEditPart)
        {
          DetektorLveEditPart ep = (DetektorLveEditPart)object;
          ep.refresh();
        }
      }
    }

    getFigure().setBackgroundColor(ColorConstants.gray);
  }

  @Override
  protected void createEditPolicies()
  {
    super.createEditPolicies();
    installEditPolicy(EditPolicy.LAYOUT_ROLE, new NonResizableEditPolicy());
  }

  private FreeFormFahrStreifenModel getCastedModel()
  {

    FreeFormFahrStreifenModel model = null;

    if (getModel() instanceof FreeFormFahrStreifenModel)
    {
      model = (FreeFormFahrStreifenModel)getModel();
    }

    return model;
  }

  @Override
  public void dataUpdateFahrStreifen(ResultData result)
  {
    FreeFormFahrStreifenModel model = getCastedModel();
    final FreeFormFahrStreifenFigure polygon = (FreeFormFahrStreifenFigure)getFigure();

    AttributeGroup atg = result.getDataDescription().getAttributeGroup();

    if (atg.getPid().equals("atg.strfallZustand"))
    {
      Data data = result.getData();
      if (data != null)
      {
        String s = data.getItem("Situation").asTextValue().getValueText();
        final SituationTyp situation = SituationTyp.setTyp(s);

        PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
          @Override
          public void run()
          {
            Color color = null;

            if (situation.getType().equals(SituationTyp.STOERUNG.getType()))
            {
              color = ColorConstants.white;
            }
            else if (situation.getType().equals(SituationTyp.KEINE_AUSSAGE.getType()))
            {
              color = ColorConstants.gray;
            }
            else if (situation.getType().equals(SituationTyp.VERKEHR_FREI.getType()))
            {
              color = ColorConstants.lightGreen;
            }
            else if (situation.getType().equals(SituationTyp.VERKEHR_LEBHAFT.getType()))
            {
              color = ColorConstants.green;
            }
            else if (situation.getType().equals(SituationTyp.VERKEHR_DICHT.getType()))
            {
              color = ColorConstants.yellow;
            }
            else if (situation.getType().equals(SituationTyp.VERKEHR_STOCKEND.getType()))
            {
              color = ColorConstants.yellow;
            }
            else if (situation.getType().equals(SituationTyp.VERKEHR_ZAEHFLIESSEND.getType()))
            {
              color = ColorConstants.orange;
            }
            else if (situation.getType().equals(SituationTyp.STAU.getType()))
            {
              color = ColorConstants.red;
            }
            else
            {
              color = ColorConstants.gray;
            }
            polygon.setColor(color);
          }
        });

      }
      else
      {
        PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
          @Override
          public void run()
          {
            polygon.setColor(ColorConstants.gray);
          }
        });
      }
    }
  }
}
