/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.dobj.editparts;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.GridData;
import org.eclipse.draw2d.GridLayout;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.gef.LayerConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;

import com.bitctrl.lib.eclipse.emf.gef.commands.SetCommand;

import de.bsvrz.buv.plugin.anlagendarstellung.AnlagendarstellungPackage;
import de.bsvrz.buv.plugin.anlagendarstellung.DoModelMitVerbLinie;
import de.bsvrz.buv.plugin.anlagendarstellung.dav.ImageFactory;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.TabellenMqFigure;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.TabellenMqZeilenFigure;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.LegendeDefinitionDetektor;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.LegendeDefinitionTabelle;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.LegendeDefinitionVerkehrsLage;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.ILegendeBaustein2;
import de.bsvrz.buv.plugin.anlagendarstellung.util.LegendeHelper;
import de.bsvrz.buv.plugin.dobj.model.DoKompositum;
import de.bsvrz.buv.plugin.dobj.model.DobjPackage;
import de.bsvrz.buv.rw.bitctrl.eclipse.legende.ILegendeBaustein;
import de.bsvrz.buv.rw.bitctrl.eclipse.legende.LegendeBaustein;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.NonMutableSet;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Implementierung fr einen {@link DoMitVerbLinieEditPart} der ein Model-Objekt 
 * vom Typ {@link DoModelMitVerbLinie} verwendet.
 * 
 * @author Pittner
 * 
 */
public class TabelleMqB27EditPart extends DoMitVerbLinieEditPart implements LayerConstants, ILegendeBaustein2
{

  private final static Font _font = new Font(null, "Arial", 7, SWT.NORMAL);

  private final static Color colorGray = ColorConstants.gray;

  private final static Color colorBlack = ColorConstants.black;

  private ArrayList<ILegendeBaustein> legendenBausteine;

  @Override
  protected IFigure createFigure()
  {

    DoModelMitVerbLinie model = getModel();
    SystemObject so = model.getSystemObject();

    IFigure fig = createFigureStatisch();

    if (so != null)
    {
      createFigureDynamisch(fig, so);
    }

    addToolTipListener(fig);
    
    return fig;
  }
  
  private Figure createFigureStatisch()
  {
    Figure figure = new Figure();
    GridLayout layout = new GridLayout(7, true);
    layout.horizontalSpacing = 0;
    layout.verticalSpacing = 0;
    figure.setLayoutManager(layout);
    figure.setOpaque(true);

    for (int i = 0; i < 7; i++)
    {
      if (i == 0)
      {
        Label l1 = new Label(" ");
        l1.setFont(_font);
        TabellenMqFigure fig1 = new TabellenMqFigure(l1, false);
        TabellenMqZeilenFigure cf1 = fig1.getListCompartments().get(0);
        TabellenMqZeilenFigure cf2 = fig1.getListCompartments().get(1);
        Label l2 = new Label(" Zeile1 ");
        l2.setFont(_font);
        l2.setForegroundColor(colorGray);
        cf1.add(l2);
        Label l3 = new Label(" Zeile 2 ");
        l3.setFont(_font);
        l3.setForegroundColor(colorGray);
        cf2.add(l3);
        figure.add(fig1);

        GridData gd = new GridData();
        gd.horizontalAlignment = SWT.FILL;
        gd.grabExcessHorizontalSpace = true;

        figure.getLayoutManager().setConstraint(fig1, gd);

      }
      else
      {
        Label l1 = new Label();
        l1.setFont(_font);
        switch (i)
        {
          case 1:
            l1.setText("QKfz");
            break;
          case 2:
            l1.setText("QPkw");
            break;
          case 3:
            l1.setText("QLkw");
            break;
          case 4:
            l1.setText("VKfz");
            break;
          case 5:
            l1.setText("VPkw");
            break;
          case 6:
            l1.setText("VLkw");
            break;
          default:
            l1.setText("");
        }
        l1.setForegroundColor(colorGray);
        TabellenMqFigure fig1 = new TabellenMqFigure(l1, true);
        TabellenMqZeilenFigure cf1 = fig1.getListCompartments().get(0);
        TabellenMqZeilenFigure cf2 = fig1.getListCompartments().get(1);
        Label l2 = new Label("undefiniert");
        l2.setFont(_font);
        l2.setToolTip(new Label("undefiniert"));
        cf1.add(l2);
        Label l3 = new Label("undefiniert");
        l3.setFont(_font);
        l3.setToolTip(new Label("undefiniert"));
        cf2.add(l3);
        figure.add(fig1);

        GridData gd = new GridData();
        gd.horizontalAlignment = SWT.FILL;
        gd.grabExcessHorizontalSpace = true;
        figure.getLayoutManager().setConstraint(fig1, gd);
      }
    }

    figure.setSize(420, 50);
    return figure;
  }

  @Override
  public void notifyChanged(Notification notification)
  {
    super.notifyChanged(notification);

    if (AnlagendarstellungPackage.Literals.TABELLE_MQ_MODEL__DATA_UPDATE.equals(notification.getFeature()))
    {
      ResultData result = (ResultData)notification.getNewValue();
      dataUpdateTabelleMessQuerschnitt(result);
    }
    else if (AnlagendarstellungPackage.Literals.TABELLE_MQ_MODEL__DARSTELLUNG_UPDATE.equals(notification.getFeature()))
    {
      String s = (String)notification.getNewValue();
      getFigure().setToolTip(new Label(s));
    }
    else if (DobjPackage.Literals.DO_MODEL__SYSTEM_OBJECT.equals(notification.getFeature()))
    {
      SystemObject so = (SystemObject)notification.getNewValue();
      createFigureDynamisch(getFigure(), so);
      refresh();
    }
  }

  private void createFigureDynamisch(IFigure figure, SystemObject so)
  {
    if (so != null)
    {
      ConfigurationObject co = (ConfigurationObject)so;
      NonMutableSet set = co.getNonMutableSet("FahrStreifen");
      List<SystemObject> elem = set.getElements();

      SortedMap<String, String> sm = new TreeMap<String, String>();

      for (int i = 0; i < elem.size(); i++)
      {
        SystemObject fs = elem.get(i);
        ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
        AttributeGroup atg = connection.getDataModel().getAttributeGroup("atg.fahrStreifen");
        Data cd = fs.getConfigurationData(atg);

        Integer lage = cd.getItem("Lage").asUnscaledValue().intValue();

        switch (lage)
        {
          case 0:
            sm.put(lage.toString(), " HFS ");
            break;
          case 1:
            sm.put(lage.toString(), " 1FS ");
            break;
          case 2:
            sm.put(lage.toString(), " 2FS ");
            break;
          case 3:
            sm.put(lage.toString(), " 3FS ");
            break;
          case 4:
            sm.put(lage.toString(), " 4FS ");
            break;
          case 5:
            sm.put(lage.toString(), " 5FS ");
            break;
          case 6:
            sm.put(lage.toString(), " 6FS ");
            break;
          case 7:
            sm.put(lage.toString(), " SFS ");
            break;

        }

      }
      sm.put("99", " MQ ");

      List figures = figure.getChildren();

      boolean flag = false;
      for (int i = 0; i < figures.size(); i++)
      {
        if (i > 0)
          flag = true;

        Object obj = figures.get(i);
        if (obj instanceof TabellenMqFigure)
        {
          List<TabellenMqZeilenFigure> listComps = new ArrayList<TabellenMqZeilenFigure>();
          for (Map.Entry<String, String> me : sm.entrySet())
          {
            TabellenMqZeilenFigure cf = new TabellenMqZeilenFigure(flag);
            Label l = new Label();
            l.setForegroundColor(colorBlack);
            cf.add(l);
            listComps.add(cf);
          }

          TabellenMqFigure fig = (TabellenMqFigure)obj;
          fig.setListCompartments(listComps);

          for (Integer j = 0; j < fig.getListCompartments().size(); j++)
          {
            TabellenMqZeilenFigure cf = fig.getListCompartments().get(j);
            Label l1 = (Label)cf.getChildren().get(0);
            l1.setFont(_font);

            if (i == 0)
            {
              String text = sm.get(j.toString());
              l1.setForegroundColor(colorGray);
              l1.setText(text);
            }
            else
            {
              l1.setForegroundColor(colorBlack);
              l1.setText("undefiniert");
            }
          }
          if (i == 0)
          {
            TabellenMqZeilenFigure cf = fig.getListCompartments().get(sm.size() - 1);
            Label l1 = (Label)cf.getChildren().get(0);
            l1.setFont(_font);

            String text = sm.get("99");
            l1.setForegroundColor(colorGray);
            l1.setText(text);
          }

        }
      }

      int offset = 10;
      int height = (int)_font.getFontData()[0].height + 7;
      int y = (height * (sm.size() + 1)) + offset;
      figure.setSize(420, y);
    }
  }

  @Override
  public String getId()
  {
    // TODO Auto-generated method stub
    return getText();
  }

  /** {@inheritDoc} */
  @Override
  public List<ILegendeBaustein> getBausteine() {
    if (legendenBausteine == null) {
      legendenBausteine = new ArrayList<ILegendeBaustein>();
      for (Object listObj : getDefinition())
      {
        legendenBausteine.addAll(LegendeHelper.erzeugeLegendeBausteine(listObj));        
      }
    }

    return legendenBausteine;
  }

  @Override
  public Image getIcon()
  {
    // TODO Auto-generated method stub
    return null;
  }

  @Override
  public String getText()
  {
    // TODO Auto-generated method stub
    return "Tabelle MQ";
  }
  
  @Override
  public List getDefinition()
  {
    // TODO Auto-generated method stub
    List list = new ArrayList();
    list.add(LegendeDefinitionTabelle.class);
    return list;
  }
}
