/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.dobj.properties;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.views.properties.tabbed.ITabbedPropertyConstants;

import com.bitctrl.lib.eclipse.emf.eclipse.model.EclipsePackage;
import com.bitctrl.lib.eclipse.emf.eclipse.model.Named;
import com.bitctrl.lib.eclipse.emf.gef.commands.SetCommand;

/**
 * Sektion zur Darstellung des Namens eines Objekts inerhalb des DOT-Editors innerhalb einer TabbedPropertySheetPage.
 * 
 * @author Pittner
 * @param <T>
 */
public class NameSection extends AbstractDobjSection<Named>
{

  /** das Textfeld zur Eingabe des Namens. */
  private Text nameText;

  @Override
  protected void aktualisiereDaten()
  {
    if (!nameText.getText().equals(getElement().getName()))
    {
      nameText.setText(getElement().getName());
    }
  }

  @Override
  public void createUI(final Composite panel)
  {
    FormData data;

    nameText = getWidgetFactory().createText(panel, "");
    data = new FormData();
    data.left = new FormAttachment(0, AbstractDobjSection.STANDARD_LABEL_WIDTH);
    data.right = new FormAttachment(100, 0);
    data.top = new FormAttachment(0, ITabbedPropertyConstants.VSPACE);
    nameText.setLayoutData(data);

    final CLabel label = getWidgetFactory().createCLabel(panel, "Name:");
    data = new FormData();
    data.left = new FormAttachment(0, 0);
    data.right = new FormAttachment(nameText, -ITabbedPropertyConstants.HSPACE);
    data.top = new FormAttachment(nameText, 0, SWT.CENTER);
    label.setLayoutData(data);

  }

  @Override
  protected void installListeners()
  {
    nameText.addModifyListener(new ModifyListener() {
      @Override
      public void modifyText(final ModifyEvent e)
      {
        if (isAktiv())
        {
          final SetCommand cmd = new SetCommand(getElement(), EclipsePackage.Literals.NAMED__NAME, nameText.getText());
          getCommandStack().execute(cmd);
        }
      }
    });
  }
}
