/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.dobj.properties;

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.ui.views.properties.tabbed.ITabbedPropertyConstants;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetWidgetFactory;

import com.bitctrl.lib.eclipse.emf.gef.commands.SetCommand;
import com.bitctrl.lib.eclipse.emf.gef.model.GefPackage;
import com.bitctrl.lib.eclipse.emf.gef.model.Sized;


/**
 * Sektion zur Darstellung der Gre eines Objekts innerhalb des DOT-Editors innerhalb einer TabbedPropertySheetPage.
 * 
 * @author Pittner
 * @param <T>
 */
public class SizeSection extends AbstractDobjSection<Sized>
{

  /** das Element zur Einstellung der Breite. */
  private Spinner breite;

  /** das Element zur Einstellung der Hhe. */
  private Spinner hoehe;

  @Override
  protected void aktualisiereDaten()
  {
    final Dimension size = getElement().getSize();
    if (size != null)
    {
      if (size.width != breite.getSelection())
      {
        breite.setSelection(size.width);
      }
      if (size.height != hoehe.getSelection())
      {
        hoehe.setSelection(size.height);
      }
    }
    else
    {
      breite.setSelection(0);
      hoehe.setSelection(0);
    }
  }

  @Override
  public void createUI(final Composite parent)
  {
    final TabbedPropertySheetWidgetFactory factory = getWidgetFactory();
    FormData data;
    CLabel label;

    breite = new Spinner(parent, SWT.BORDER);
    breite.setMinimum(0);
    breite.setMaximum(Integer.MAX_VALUE);
    data = new FormData();
    data.left = new FormAttachment(0, AbstractDobjSection.STANDARD_LABEL_WIDTH);
    data.right = new FormAttachment(50, 0);
    data.top = new FormAttachment(0, ITabbedPropertyConstants.VSPACE);
    breite.setLayoutData(data);

    label = factory.createCLabel(parent, "Breite:");
    data = new FormData();
    data.left = new FormAttachment(0, 0);
    data.right = new FormAttachment(breite, -ITabbedPropertyConstants.HSPACE);
    data.top = new FormAttachment(breite, 0, SWT.CENTER);
    label.setLayoutData(data);

    hoehe = new Spinner(parent, SWT.BORDER);
    hoehe.setMinimum(0);
    hoehe.setMaximum(Integer.MAX_VALUE);
    data = new FormData();
    data.left = new FormAttachment(breite, AbstractDobjSection.STANDARD_LABEL_WIDTH);
    data.right = new FormAttachment(100, 0);
    data.top = new FormAttachment(breite, 0, SWT.CENTER);
    hoehe.setLayoutData(data);

    label = factory.createCLabel(parent, "Hhe:");
    data = new FormData();
    data.left = new FormAttachment(50, ITabbedPropertyConstants.HSPACE);
    data.right = new FormAttachment(hoehe, -ITabbedPropertyConstants.HSPACE);
    data.top = new FormAttachment(hoehe, 0, SWT.CENTER);
    label.setLayoutData(data);
  }

  @Override
  protected void installListeners()
  {
    breite.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(final SelectionEvent e)
      {
        versendeDimension();
      }
    });
    hoehe.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(final SelectionEvent e)
      {
        versendeDimension();
      }
    });
  }

  /** versendet ein Kommando zur nderung der Gre. */
  private void versendeDimension()
  {
    if (isAktiv())
    {
      final EStructuralFeature feature;
      feature = GefPackage.Literals.SIZED__SIZE;
      final Dimension dimension = new Dimension(breite.getSelection(), hoehe.getSelection());
      final SetCommand cmd = new SetCommand(getElement(), feature, dimension);
      getCommandStack().execute(cmd);
    }
  }
}
