package de.bsvrz.buv.plugin.anlagendarstellung.dobj.section;

import org.eclipse.core.runtime.Status;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CCombo;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.views.properties.tabbed.ITabbedPropertyConstants;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetWidgetFactory;

import com.bitctrl.lib.eclipse.emf.gef.commands.SetCommand;

import de.bsvrz.buv.plugin.anlagendarstellung.Activator;
import de.bsvrz.buv.plugin.anlagendarstellung.AnlagendarstellungPackage;
import de.bsvrz.buv.plugin.anlagendarstellung.PickerVertical;
import de.bsvrz.buv.plugin.anlagendarstellung.views.ViewZuordnungKilometrierung;
import de.bsvrz.buv.plugin.dobj.properties.AbstractSection;

/**
 * Sektion zum Bearbeiten des {@link PickerVertical}-Daten.
 * 
 * @author Swarco Traffic Systems GmbH
 * 
 */
public class SectionPicker extends AbstractSection<PickerVertical>
{
	/**
	 * Feld zur Erfassung der Breite.
	 */
	private Text _tBreite;

	/**
	 * Feld zur Erfassung der Hhe.
	 */
	private Text _tHoehe;

	/**
	 * Feld zur Erfassung der Distanz zwischen den Pfeilspitzen.
	 */
	private Text _tDistanz;

	/**
	 * Feld zur Erfassung der initialen Y-Position der oberen Pfeilspitze
	 */
	private Text _tYPosPickerTop;

	/**
	 * Feld zur Erfassung der Zuordnung von Kilometrierung zu Pixeln
	 */
	private Text _tKilometrierungZuordnung;

	/**
	 * Checkbox die festlegt, ob Ticks gezeichnet werden sollen oder nicht.
	 */
	private Button _chkPaintTicks = null;

	private boolean _modifiyListenerAktiv = true;

	private String sndId4ViewZuordnungKilometrierung = null;

  private CCombo fahrtrichtung;

	@Override
	public void createControls(Composite parent,
			TabbedPropertySheetPage aTabbedPropertySheetPage)
	{
		super.createControls(parent, aTabbedPropertySheetPage);

		sndId4ViewZuordnungKilometrierung = "" + System.currentTimeMillis();

		final TabbedPropertySheetWidgetFactory factory = getWidgetFactory();
		final Composite container = factory.createFlatFormComposite(parent);

		/*
		 * GUI-Elemente erzeugen
		 */
		CLabel lBreite = factory.createCLabel(container, "Breite:");
		FormData fd = new FormData();
		fd.left = new FormAttachment(0, 0);
		fd.top = new FormAttachment(0, 0);
		lBreite.setLayoutData(fd);

		_tBreite = factory.createText(container, "");
		fd = new FormData();
		fd.left = new FormAttachment(lBreite, 0);
		fd.top = new FormAttachment(0, 0);
		fd.right = new FormAttachment(25, 0);
		_tBreite.setLayoutData(fd);

		CLabel lHoehe = factory.createCLabel(container, "Hhe:");
		fd = new FormData();
		fd.left = new FormAttachment(0, 0);
		fd.top = new FormAttachment(lBreite, 0);
		lHoehe.setLayoutData(fd);

		_tHoehe = factory.createText(container, "");
		fd = new FormData();
		fd.left = new FormAttachment(lHoehe, 0);
		fd.top = new FormAttachment(_tBreite, 0);
		fd.right = new FormAttachment(25, 0);
		_tHoehe.setLayoutData(fd);

		CLabel lDistanz = factory.createCLabel(container, "Distanz:");
		fd = new FormData();
		fd.left = new FormAttachment(0, 0);
		fd.top = new FormAttachment(lHoehe, 0);
		lDistanz.setLayoutData(fd);

		_tDistanz = factory.createText(container, "");
		fd = new FormData();
		fd.left = new FormAttachment(lDistanz, 0);
		fd.top = new FormAttachment(_tHoehe, 0);
		fd.right = new FormAttachment(25, 0);
		_tDistanz.setLayoutData(fd);

		CLabel lYPosPickerTop = factory
				.createCLabel(container, "Y-Pos Picker Top:");
		fd = new FormData();
		fd.left = new FormAttachment(0, 0);
		fd.top = new FormAttachment(lDistanz, 0);
		lYPosPickerTop.setLayoutData(fd);

		_tYPosPickerTop = factory.createText(container, "");
		fd = new FormData();
		fd.left = new FormAttachment(lYPosPickerTop, 0);
		fd.top = new FormAttachment(_tDistanz, 0);
		fd.right = new FormAttachment(25, 0);
		_tYPosPickerTop.setLayoutData(fd);

		CLabel lKilometrierung = factory.createCLabel(container, "Kilometrierung");
		fd = new FormData();
		fd.left = new FormAttachment(0, 0);
		fd.top = new FormAttachment(lYPosPickerTop, 0);
		lKilometrierung.setLayoutData(fd);

		_tKilometrierungZuordnung = factory.createText(container, "");
		_tKilometrierungZuordnung.setEditable(false);
		fd = new FormData();
		fd.left = new FormAttachment(lKilometrierung, 0);
		fd.top = new FormAttachment(_tYPosPickerTop, 0);
		fd.right = new FormAttachment(25, 0);
		_tKilometrierungZuordnung.setLayoutData(fd);

		Button bKM = factory.createButton(container, "Editieren", SWT.PUSH);
		fd = new FormData();
		fd.left = new FormAttachment(_tKilometrierungZuordnung, 0);
		fd.top = new FormAttachment(_tYPosPickerTop, 0);
		fd.right = new FormAttachment(40, 0);
		bKM.setLayoutData(fd);
		bKM.addSelectionListener(new SelectionAdapter()
		{

			@Override
			public void widgetSelected(SelectionEvent e)
			{
				actionKilometrierungYPosEditieren();
			}

		});

		// Ticks (de-)aktivieren
		_chkPaintTicks = factory.createButton(container, "Ticks zeichnen?",
				SWT.CHECK);
		fd = new FormData();
		fd.left = new FormAttachment(0, 0);
		fd.top = new FormAttachment(lKilometrierung, 0);
		_chkPaintTicks.setLayoutData(fd);
		
		/*
		 * Verify-Listener anmelden.
		 */
		_tBreite.addListener(SWT.Verify, getNumberOnlyListener());
		_tHoehe.addListener(SWT.Verify, getNumberOnlyListener());
		_tDistanz.addListener(SWT.Verify, getNumberOnlyListener());
		_tYPosPickerTop.addListener(SWT.Verify, getNumberOnlyListener());

		/*
		 * Modify-Listener
		 */
		_tBreite.addModifyListener(new ModifyListener()
		{

			@Override
			public void modifyText(ModifyEvent e)
			{
				if (isModifiyListenerAktiv())
				{
					PickerVertical element = getElement();

					if (element != null)
					{
						if ((e.getSource() != null) && (e.getSource() instanceof Text))
						{
							Text t = (Text) e.getSource();
							String str = t.getText();
							int asInt = getAsInt(str);

							if (asInt != element.getBreite())
							{
								EStructuralFeature feature = AnlagendarstellungPackage.Literals.PICKER_VERTICAL__BREITE;
								final SetCommand cmd = new SetCommand(getElement(), feature,
										asInt);
								getCommandStack().execute(cmd);
							}
						}
					}
				}
			}
		});

		_tHoehe.addModifyListener(new ModifyListener()
		{

			@Override
			public void modifyText(ModifyEvent e)
			{
				if (isModifiyListenerAktiv())
				{
					PickerVertical element = getElement();

					if (element != null)
					{
						if ((e.getSource() != null) && (e.getSource() instanceof Text))
						{
							Text t = (Text) e.getSource();
							String str = t.getText();
							int asInt = getAsInt(str);

							if (asInt != element.getBreite())
							{
								EStructuralFeature feature = AnlagendarstellungPackage.Literals.PICKER_VERTICAL__HOEHE;
								final SetCommand cmd = new SetCommand(getElement(), feature,
										asInt);
								getCommandStack().execute(cmd);
							}
						}
					}
				}
			}
		});

		_tDistanz.addModifyListener(new ModifyListener()
		{

			@Override
			public void modifyText(ModifyEvent e)
			{
				if (isModifiyListenerAktiv())
				{
					PickerVertical element = getElement();

					if (element != null)
					{
						if ((e.getSource() != null) && (e.getSource() instanceof Text))
						{
							Text t = (Text) e.getSource();
							String str = t.getText();
							int distanzAsInt = getAsInt(str);

							if (distanzAsInt != element.getDistanz())
							{

								EStructuralFeature feature = AnlagendarstellungPackage.Literals.PICKER_VERTICAL__DISTANZ;
								final SetCommand cmd = new SetCommand(getElement(), feature,
										distanzAsInt);
								getCommandStack().execute(cmd);
							}
						}
					}
				}
			}
		});

		_tYPosPickerTop.addModifyListener(new ModifyListener()
		{

			@Override
			public void modifyText(ModifyEvent e)
			{
				if (isModifiyListenerAktiv())
				{
					PickerVertical element = getElement();

					if (element != null)
					{
						if ((e.getSource() != null) && (e.getSource() instanceof Text))
						{
							Text t = (Text) e.getSource();
							String str = t.getText();
							int asInt = getAsInt(str);

							if (asInt != element.getYPosPickerTop())
							{
								EStructuralFeature feature = AnlagendarstellungPackage.Literals.PICKER_VERTICAL__YPOS_PICKER_TOP;
								final SetCommand cmd = new SetCommand(getElement(), feature,
										asInt);
								getCommandStack().execute(cmd);
							}
						}
					}
				}
			}
		});

//		_chkPaintTicks.addSelectionListener(new SelectionListener()
//		{
//
//			@Override
//			public void widgetSelected(SelectionEvent e)
//			{
//				if (isModifiyListenerAktiv())
//				{
//					if (_chkPaintTicks != null)
//					{
//						Boolean selection = new Boolean(_chkPaintTicks.getSelection());
//						EStructuralFeature feature = AnlagendarstellungPackage.Literals.PICKER_VERTICAL__TICKS_ANZEIGEN;
//						final SetCommand cmd = new SetCommand(getElement(), feature,
//								selection);
//						getCommandStack().execute(cmd);
//					}
//				}
//			}
//
//			@Override
//			public void widgetDefaultSelected(SelectionEvent e)
//			{
//			}
//		});
	}

	/**
	 * Validierung der eingegbenen Werte.
	 * 
	 * @param hoehe
	 *          Hhe der Figure
	 * @param yPosPickerTop
	 *          Position des oberen Pickers
	 * @param distance
	 *          Distance zum unteren Picker
	 * @return Liefert <code>true</code> wenn validiert, sonst <code>false</code>.
	 */
	public boolean validate(int hoehe, int yPosPickerTop, int distance)
	{
		boolean validated = false;

		if ((yPosPickerTop >= 0) && (yPosPickerTop <= hoehe))
		{
			if ((yPosPickerTop + distance) <= hoehe)
			{
				validated = true;
			}
		}

		return validated;
	}

	/**
	 * Liefert einen Listener fr {@link SWT#Verify}, der nur die Eingaben der
	 * Zahlen 0-9 zulsst.
	 * 
	 * @return
	 */
	public static Listener getNumberOnlyListener()
	{
		Listener numberOnlyListener = new Listener()
		{
			public void handleEvent(Event e)
			{
				String string = e.text;
				char[] chars = new char[string.length()];
				string.getChars(0, chars.length, chars, 0);
				for (int i = 0; i < chars.length; i++)
				{
					if (!('0' <= chars[i] && chars[i] <= '9'))
					{
						e.doit = false;
						return;
					}
				}
			}
		};

		return numberOnlyListener;
	}

	@Override
	public void refresh()
	{
		PickerVertical pickerVertical = getElement();
		if (pickerVertical != null)
		{
			setModifiyListenerAktiv(false);
			if (_tBreite != null)
			{
				String breiteNeu = "" + pickerVertical.getBreite();
				if (!_tBreite.getText().equals(breiteNeu))
				{
					_tBreite.setText(breiteNeu);
				}
			}

			if (_tHoehe != null)
			{
				String hoeheNeu = "" + pickerVertical.getHoehe();
				if (!_tHoehe.getText().equals(hoeheNeu))
				{
					_tHoehe.setText(hoeheNeu);
				}
			}

			if (_tDistanz != null)
			{
				String distanzNeu = "" + pickerVertical.getDistanz();
				if (!_tDistanz.getText().equals(distanzNeu))
				{
					_tDistanz.setText(distanzNeu);
				}
			}

			if (_tYPosPickerTop != null)
			{
				String yPosPickerTopNeu = "" + pickerVertical.getYPosPickerTop();
				if (!_tYPosPickerTop.getText().equals(yPosPickerTopNeu))
				{
					_tYPosPickerTop.setText(yPosPickerTopNeu);
				}
			}

			if (_tKilometrierungZuordnung != null)
			{
				String kilometrierungZuordnung = pickerVertical
						.getKilometrierungZuordnung();

				// if(kilometrierungZuordnung == null)
				// {
				// /*
				// * NPE vermeiden
				// */
				// kilometrierungZuordnung = "";
				// }
				//								
				// if (!_tKilometrierungZuordnung.getText()
				// .equals(kilometrierungZuordnung))
				// {
				// _tKilometrierungZuordnung.setText(kilometrierungZuordnung);
				// }

				if ((kilometrierungZuordnung != null)
						&& (kilometrierungZuordnung.length() > 0))
				{
					_tKilometrierungZuordnung.setText("Gesetzt");
				} else
				{
					_tKilometrierungZuordnung.setText("Nicht Gesetzt");
				}

			}

//			if (_chkPaintTicks != null)
//			{
//				_chkPaintTicks.setSelection(pickerVertical.isTicksAnzeigen());
//			}
			
			setModifiyListenerAktiv(true);
		}
	}
	private static int getAsInt(String str)
	{
		int re = 0;

		if ((str != null) && (str.length() > 0))
		{
			try
			{
				re = Integer.parseInt(str);
			} catch (NumberFormatException nfe)
			{
			}
		}

		return re;
	}
	private boolean isModifiyListenerAktiv()
	{
		return _modifiyListenerAktiv;
	}
	private void setModifiyListenerAktiv(boolean modifiyListenerAktiv)
	{
		_modifiyListenerAktiv = modifiyListenerAktiv;
	}

	private void actionKilometrierungYPosEditieren()
	{
		CommandStack commandStack = getCommandStack();
		PickerVertical pickerVertical = getElement();

		IWorkbenchWindow window = PlatformUI.getWorkbench()
				.getActiveWorkbenchWindow();

		try
		{
			window.getActivePage().showView(ViewZuordnungKilometrierung.ID,
					sndId4ViewZuordnungKilometrierung, IWorkbenchPage.VIEW_VISIBLE);

			IViewReference reference = PlatformUI.getWorkbench()
					.getActiveWorkbenchWindow().getActivePage()
					.findViewReference(ViewZuordnungKilometrierung.ID,
							sndId4ViewZuordnungKilometrierung);
			IViewPart view = reference.getView(true);
			if (view instanceof ViewZuordnungKilometrierung)
			{
				ViewZuordnungKilometrierung re = (ViewZuordnungKilometrierung) view;
				re.setCommandStackAndModel(commandStack, pickerVertical);
				window.getActivePage().activate(re);
			}

		} catch (PartInitException e)
		{
			String msg = "Die ViewAusgabefenster (ID: "
					+ ViewZuordnungKilometrierung.ID + ", SecId: "
					+ sndId4ViewZuordnungKilometrierung
					+ ") konnte nicht geffnet werden!.";

			Activator.getDefault().getLog().log(
					new Status(Status.WARNING, Activator.PLUGIN_ID, msg, e));
		}

	}

}
