/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.figures;

import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.GridData;
import org.eclipse.draw2d.GridLayout;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.ScaledGraphics;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;

import de.bsvrz.buv.plugin.anlagendarstellung.dav.ImageFactory;
import de.bsvrz.buv.plugin.anlagendarstellung.model.ZusatzElementTyp;

/**
 * Figure, welche den rechten Teil eines AQ eines beliebigen Typs reprsentiert.               
 * 
 * @author Pittner
 * 
 */
public class AnzeigeQuerschnittRechtsFigure extends Figure
{

  protected Label _labelHelligkeit;

  protected Label _labelProgramm;

  protected ZusatzElementTyp _zustandProgramm;

  /**
   * Default - Skalierungsfaktor.
   */
  private double scale = 1.0;

  
  public AnzeigeQuerschnittRechtsFigure(int kontext)
  {
    this(kontext, true);
  }
  
//  public AnzeigeQuerschnittRechtsFigure(int kontext)
//  {
//
//    setOpaque(false);
//
//    GridLayout gl = new GridLayout(1, false);
//    setLayoutManager(gl);
//
//    switch (kontext)
//    {
//      case 0:
//      case 1:
//        createLabelHelligkeit();
//        createLabelProgramm();
//        break;
//      case 2:
//        break;
//      case 3:
//        createLabelProgramm();
//        break;
//      case 4:
//        break;
//
//      default:
//    }
//  }
  
  public AnzeigeQuerschnittRechtsFigure(int kontext, boolean flag)
  {
    
    setOpaque(false);
    
    GridLayout gl = new GridLayout(1, false);
    setLayoutManager(gl);
    
    switch (kontext)
    {
      case 0:
      case 1:
        if (flag)
          createLabelHelligkeit();
        createLabelProgramm();
        break;
      case 2:
        break;
      case 3:
        createLabelProgramm();
        break;
      case 4:
        break;
        
      default:
    }
  }
  
  private void createLabelProgramm()
  {
    _labelProgramm = new Label();
    _labelProgramm.setOpaque(true);
    Image image2 = ImageFactory.getImage("icons/Prg_leer.png");

    _zustandProgramm = ZusatzElementTyp.PROGRAMM_UNDEFINIERT;

    _labelProgramm.setIcon(image2);
    add(_labelProgramm);
    GridData gridData2 = new GridData();
    gridData2.grabExcessVerticalSpace = false;
    gridData2.verticalAlignment = SWT.TOP;
    getLayoutManager().setConstraint(_labelProgramm, gridData2);
  }

  private void createLabelHelligkeit()
  {
    _labelHelligkeit = new Label();
    _labelHelligkeit.setOpaque(true);
    _labelHelligkeit.setText("---");
    add(_labelHelligkeit);
    GridData gridData = new GridData();
    gridData.grabExcessHorizontalSpace = true;
    gridData.horizontalAlignment = SWT.FILL;
    gridData.grabExcessVerticalSpace = false;
    gridData.verticalAlignment = SWT.TOP;
    getLayoutManager().setConstraint(_labelHelligkeit, gridData);
  }

  public void switchLinksRechts(int fr)
  {

    GridData gd1 = null;
    GridData gd2 = null;

    if (_labelHelligkeit != null)
      gd1 = (GridData)getLayoutManager().getConstraint(_labelHelligkeit);
    if (_labelHelligkeit != null)
      gd2 = (GridData)getLayoutManager().getConstraint(_labelProgramm);

    int i1 = 0;
    int i2 = 0;

    if (fr != 1)
    {
      if (gd1 != null)
      {
        gd1.grabExcessVerticalSpace = false;
        gd1.verticalAlignment = SWT.TOP;
      }
      if (gd2 != null)
      {
        gd2.grabExcessVerticalSpace = false;
        gd2.verticalAlignment = SWT.TOP;
      }
      i1 = 0;
      i2 = 1;
    }
    else
    {
      if (gd1 != null)
      {
        gd1.grabExcessVerticalSpace = false;
        gd1.verticalAlignment = SWT.BOTTOM;
      }
      if (gd2 != null)
      {
        gd2.grabExcessVerticalSpace = true;
        gd2.verticalAlignment = SWT.BOTTOM;
      }
      i1 = 1;
      i2 = 0;
    }
    if (_labelHelligkeit != null)
    {
      add(_labelHelligkeit, i1);
      getLayoutManager().setConstraint(_labelHelligkeit, gd1);
    }
    else
    {
      i2 = 0;
    }
    if (_labelProgramm != null)
    {
      add(_labelProgramm, i2);
      getLayoutManager().setConstraint(_labelProgramm, gd2);
    }

  }

  @Override
  protected void paintFigure(final Graphics graphics)
  {
    if (graphics instanceof ScaledGraphics)
    {
      final Dimension size = getSize();

      final Point location = getLocation();
      graphics.translate(location.x + size.width / 2, location.y + size.height / 2);
      graphics.scale(1 / scale);
      graphics.translate(-location.x - size.width / 2, -location.y - size.height / 2);
    }

    super.paintFigure(graphics);
  }

  public Label getLabelHelligkeit()
  {
    return _labelHelligkeit;
  }

  public void setLabelHelligkeit(Label label)
  {
    _labelHelligkeit = label;
  }

  public ZusatzElementTyp getZustandProgramm()
  {
    return _zustandProgramm;
  }

  public void setZustandProgramm(ZusatzElementTyp zustandProgramm)
  {
    _zustandProgramm = zustandProgramm;
  }

  public Label getLabelProgramm()
  {
    return _labelProgramm;
  }

  public void setLabelProgramm(Label label)
  {
    _labelProgramm = label;
  }

  public static String createImagePath(ZusatzElementTyp typ, ZusatzElementTyp betrieb)
  {

    if (typ == null || betrieb == null)
      return null;

    String imagepath = null;

    if (typ.equals(ZusatzElementTyp.PROGRAMM) || typ.equals(ZusatzElementTyp.PROGRAMM_A14))
    {
      if (betrieb.equals(ZusatzElementTyp.PROGRAMM_SBA_HANDPROGRAMM))
      {
        imagepath = "icons/prg1_h.png";
      }
      else if (betrieb.equals(ZusatzElementTyp.PROGRAMM_SBA_SONDERPROGRAMM))
      {
        imagepath = "icons/prg1_s.png";
      }
      else if (betrieb.equals(ZusatzElementTyp.PROGRAMM_SBA_TUNNEL)
          || betrieb.equals(ZusatzElementTyp.PROGRAMM_SBA_TUNNELBETRIEB))
      {
        imagepath = "icons/prg1_t.png";
      }
      else if (betrieb.equals(ZusatzElementTyp.PROGRAMM_SBA_TEST)
          || betrieb.equals(ZusatzElementTyp.PROGRAMM_SBA_TESTBETRIEB))
      {
        imagepath = "icons/prg1_y.png";
      }
      else if (betrieb.equals(ZusatzElementTyp.PROGRAMM_SBA_VORORTBETRIEB))
      {
        imagepath = "icons/prg1_v.png";
      }
      else if (betrieb.equals(ZusatzElementTyp.PROGRAMM_SBA_AUTOMATIKPROGRAMM))
      {
        imagepath = "icons/prg1_a.png";
      }
      else if (betrieb.equals(ZusatzElementTyp.PROGRAMM_SBA_AUTARKBETRIEB))
      {
        imagepath = "icons/prg1_x.png";
      }
      else if (betrieb.equals(ZusatzElementTyp.PROGRAMM_SBA_NOT)
          || betrieb.equals(ZusatzElementTyp.PROGRAMM_SBA_NOTBETRIEB))
      {
        imagepath = "icons/prg1_n.png";
      }
      else if (betrieb.equals(ZusatzElementTyp.PROGRAMM_SBA_HANDPROGRAMM_BLIND))
      {
        if (!typ.equals(ZusatzElementTyp.PROGRAMM_A14))
          imagepath = "icons/prg1_hb.png";
        else
          imagepath = "icons/prg1_h.png";
      }
      else if (betrieb.equals(ZusatzElementTyp.PROGRAMM_SBA_SONDERPROGRAMM_BLIND))
      {
        if (!typ.equals(ZusatzElementTyp.PROGRAMM_A14))
          imagepath = "icons/prg1_sb.png";
        else
          imagepath = "icons/prg1_s.png";
      }
      else if (betrieb.equals(ZusatzElementTyp.PROGRAMM_SBA_AUTOMATIKPROGRAMM_BLIND))
      {
        if (typ.equals(ZusatzElementTyp.PROGRAMM_A14))
          imagepath = "icons/prg1_b1.png";
        else
          imagepath = "icons/prg1_ab.png";
      }
      else
      {
        imagepath = "icons/prg1_leer.png";
      }
    }
    else if (typ.equals(ZusatzElementTyp.HELLIGKEIT))
    {
      if (betrieb.equals(ZusatzElementTyp.HELLIGKEIT_AQ_UNDEFINERT))
      {
        imagepath = "icons/prg1_leer.png";
      }
      else if (betrieb.equals(ZusatzElementTyp.HELLIGKEIT_AQ_GESTOERT))
      {
        imagepath = "icons/prg1_s.png";
      }
      else if (betrieb.equals(ZusatzElementTyp.HELLIGKEIT_AQ_STUFE_0_MANUELL))
      {
        imagepath = "icons/hell0_manuell.png";
      }
      else if (betrieb.equals(ZusatzElementTyp.HELLIGKEIT_AQ_STUFE_0_AUTOMATIK_LOKAL))
      {
        imagepath = "icons/hell0_autom.png";
      }
      else if (betrieb.equals(ZusatzElementTyp.HELLIGKEIT_AQ_STUFE_0_AUTOMATIK_STUFE))
      {
        imagepath = "icons/hell0_stufe.png";
      }
      else
      {
        imagepath = "icons/prg1_leer.png";
      }
    }
    return imagepath;
  }


  /**
   * Bestimmt die Bounds der Figur neu.
   */
  private void updateFigure()
  {
    setBounds(new Rectangle(getLocation(), getSize()));
  }

}
