/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.figures;

import org.eclipse.draw2d.Clickable;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.draw2d.ScalableFigure;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.GridData;

import de.bsvrz.buv.plugin.anlagendarstellung.model.AnzeigeQuerschnitt;

/**
 * Figure, welche den rechten Teil eines generischen AQ eines beliebigen Typs reprsentiert.               
 * 
 * @author Pittner
 * 
 */
public class AnzeigeQuerschnittTypGenericFigure extends Figure implements ScalableFigure
{

  String _zeit = null;

  private Label _labelZeit;

  private Clickable _clickable;

  private AnzeigeElementFigure _anzeige;

  public AnzeigeElementFigure getAnzeige()
  {
    return _anzeige;
  }

  public Clickable getClickable()
  {
    return _clickable;
  }

  public void setClickable(Clickable clickable)
  {
    _clickable = clickable;
  }

  private AnzeigeQuerschnittTypGenericFigure _instanz;

  public AnzeigeQuerschnittTypGenericFigure()
  {
    _instanz = this;
    setBorder(new LineBorder(ColorConstants.orange, 2));
    setBackgroundColor(ColorConstants.gray);
    setOpaque(true);

    updateFigure();   
  }

  public AnzeigeQuerschnittTypGenericFigure(AnzeigeQuerschnitt aq)
  {
    _instanz = this;
    setBorder(new LineBorder(ColorConstants.orange, 2));
    setBackgroundColor(ColorConstants.gray);
    setOpaque(true);

    setLayoutManager(new XYLayout());

    Label l = new Label();
    add(l);
    setConstraint(l, new Rectangle(0, 0, -1, -1));
  }

  /**
   * Die Ausdehnung der Figur.
   */
  private static final Dimension DIM = new Dimension(100, 50);

  /**
   * Default - Skalierungsfaktor.
   */
  private double scale = 1.0;

  private Label _labelName;

  /**
   * Sets the zoom level
   * 
   * @param newZoom
   *          The new zoom level
   */
  public void setScale(double newZoom)
  {
    if (scale == newZoom)
      return;
    scale = newZoom;
    // sorgt dafr, dass die Gre der Figur unabhngig von der Zoomstufe
    // gleich bleibt.
    setSize((int)(DIM.width / scale), (int)(DIM.height / scale));
    // TODO: Die Skalierung des Textes anpassen.

    updateFigure();
    layout();
    repaint();

  }

  /**
   * Bestimmt die Bounds der Figur neu.
   */
  private void updateFigure()
  {
    Dimension newDimension = new Dimension(DIM);
    newDimension.scale(scale);
    setBounds(new Rectangle(getLocation(), newDimension));
  }

  @Override
  public double getScale()
  {
    return scale;
  }

  public AnzeigeQuerschnittTypGenericFigure(String pid, String name, String zeit, int x)
  {
    _instanz = this;
    _zeit = zeit;

    setBorder(new LineBorder(ColorConstants.orange, 2));
    setBackgroundColor(ColorConstants.gray);
    setToolTip(new Label(pid));
    setOpaque(true);

    setLayoutManager(new XYLayout());

    _labelZeit = new Label(_zeit);
    Font f2 = new Font(null, "Arial", 8, SWT.NORMAL);
    _labelZeit.setFont(f2);
    _labelZeit.setToolTip(new Label("Uhrzeit"));
    _labelZeit.setLabelAlignment(GridData.BEGINNING);

    add(_labelZeit);
    setConstraint(_labelZeit, new Rectangle(3, 3, -1, -1));

    _labelName = new Label(name);
    Font f3 = new Font(null, "Arial", 8, SWT.NORMAL);
    _labelName.setFont(f3);
    _labelName.setToolTip(new Label("Name des AQ"));
    _labelName.setLabelAlignment(GridData.END);

    add(_labelName);
    setConstraint(_labelName, new Rectangle(x, 3, -1, -1));

    updateFigure();
  }

  public String getZeit()
  {
    return _zeit;
  }

  public void setZeit(String zeit)
  {
    if (_labelZeit != null)
    {
      _labelZeit.setText(zeit);
      updateFigure();
    }
  }

  public void setName(String name)
  {
    if (_labelName != null)
    {
      _labelName.setText(name);
      updateFigure();
    }
  }

  public Label getLabelZeit()
  {
    return _labelZeit;
  }

  public void setLabelZeit(Label l2)
  {
    _labelZeit = l2;
  }

  @Override
  protected boolean useLocalCoordinates()
  {
    return true;
  }

  public void setExtraText(String newStringValue)
  {
    setName(newStringValue);
    updateFigure();
  }
}
