/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.figures;

import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.ToolbarLayout;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.RGB;

/**
 * Figure, welche die optionale AQ-Bezeichnung reprsentiert.               
 * 
 * @author Pittner
 * 
 */
public class BezeichnungAQFigure extends Label
{
  private int _x;

  private int _y;

  private int _width;

  private int _height;

  private RGB _rgb;

  private String _bezeichnung;

  private String _kilometer;

  private String _abschnitt;

  private String _image;

  private static Font fFett = new Font(null, "Arial", 8, SWT.BOLD);

  private static Font fNorm = new Font(null, "Arial", 8, SWT.NORMAL);

  public BezeichnungAQFigure(int xOrg, int yOrg, String bezeichnung, String abschnitt, String kilometer, int art)
  {
    _x = xOrg;
    _y = yOrg;

    _bezeichnung = bezeichnung;
    _abschnitt = abschnitt;
    _kilometer = kilometer;

    if (art == 0)
      beschrifungNebeneinander();

    if (art == 1)
      beschrifungUntereinander();

    if (art == 2)
      beschrifungEinzeln();

  }

  public BezeichnungAQFigure(int xOrg, int yOrg, String bezeichnung)
  {
    _x = xOrg;
    _y = yOrg;

    _bezeichnung = bezeichnung;

    beschrifung();

  }

  public BezeichnungAQFigure(int x, int y, int width, int height, String bezeichnung)
  {
    _x = x;
    _y = y;
    _width = width;
    _height = height;

    _bezeichnung = bezeichnung;

    beschrifung();

  }

  public void beschrifung()
  {
    ToolbarLayout layout = new ToolbarLayout();
    layout.setVertical(false);

    setOpaque(true);
    setLayoutManager(layout);
    setToolTip(new Label(_bezeichnung));

    setText(_bezeichnung);

    setBounds(new Rectangle(_x, _y, _width, _height));

  }

  public void beschrifungNebeneinander()
  {
    ToolbarLayout layout = new ToolbarLayout();
    layout.setVertical(false);

    setOpaque(true);
    setLayoutManager(layout);
    setToolTip(new Label(_bezeichnung));

    Label l1 = new Label();
    Label l2 = new Label();

    l1.setLabelAlignment(PositionConstants.LEFT);
    l2.setLabelAlignment(PositionConstants.LEFT);

    String s = "";

    if (_abschnitt != null)
      s += " " + _abschnitt + " ";

    s += _kilometer;

    l1.setFont(fFett);
    l1.setText(_bezeichnung);

    l2.setFont(fNorm);
    l2.setText(s);

    add(l1);

    int w2 = 0;
    int h2 = 0;

    if (s.length() > 0)
    {
      add(l2);

      w2 = l2.getPreferredSize().width;
      h2 = l2.getPreferredSize().height;
    }

    int w1 = l1.getPreferredSize().width;
    int h1 = l1.getPreferredSize().height;

    int w = w1 + w2;
    int h = h1;

    if (h2 > h1)
      h = h2;

    setBounds(new Rectangle(_x, _y, w, h));
  }

  public void beschrifungUntereinander()
  {
    ToolbarLayout layout = new ToolbarLayout();

    setOpaque(true);
    setLayoutManager(layout);
    setToolTip(new Label(_bezeichnung));

    Label l1 = new Label();
    Label l2 = new Label();

    l1.setLabelAlignment(PositionConstants.LEFT);
    l2.setLabelAlignment(PositionConstants.LEFT);

    String s = "";

    if (_abschnitt != null)
      s += _abschnitt + " ";

    s += _kilometer;

    l1.setFont(fFett);
    l1.setText(_bezeichnung);

    l2.setFont(fNorm);
    l2.setText(s);

    add(l1);

    int w2 = 0;
    int h2 = 0;

    if (s.length() > 0)
    {
      add(l2);

      w2 = l2.getPreferredSize().width;
      h2 = l2.getPreferredSize().height;
    }

    int w1 = l1.getPreferredSize().width;
    int h1 = l1.getPreferredSize().height;

    int h = h1 + h2;
    int w = w1;

    if (w2 > w1)
      w = w2;

    setBounds(new Rectangle(_x, _y, w, h));
  }

  public void beschrifungEinzeln()
  {
    ToolbarLayout layout = new ToolbarLayout();

    setOpaque(true);
    setLayoutManager(layout);
    setToolTip(new Label(_bezeichnung));
 
    Label l1 = new Label();
    Label l2 = new Label();
    Label l3 = new Label();

    l1.setLabelAlignment(PositionConstants.LEFT);
    l2.setLabelAlignment(PositionConstants.LEFT);
    l3.setLabelAlignment(PositionConstants.LEFT);

    String s = "";

    if (_abschnitt != null)
      s += _abschnitt + " ";

    l1.setFont(fFett);
    l1.setText(_bezeichnung);

    l2.setFont(fNorm);
    l2.setText(s);

    l3.setFont(fNorm);
    l3.setText(_kilometer);

    add(l1);

    int w2 = 0;
    int h2 = 0;

    if (s.length() > 0)
    {
      add(l2);

      w2 = l2.getPreferredSize().width;
      h2 = l2.getPreferredSize().height;
    }

    int w3 = 0;
    int h3 = 0;

    if (_kilometer != null && _kilometer.length() > 0)
    {
      add(l3);

      w3 = l3.getPreferredSize().width;
      h3 = l3.getPreferredSize().height;
    }

    int w1 = l1.getPreferredSize().width;
    int h1 = l1.getPreferredSize().height;

    int h = h1 + h2 + h3;
    int w = w1;

    if (w2 > w1)
      w = w2;
    if (w3 > w)
      w = w3;

    setBounds(new Rectangle(_x, _y, w, h));
  }

  /**
   * Methode liefert image zurck
   * 
   * @return image
   */
  public String getImage()
  {
    return _image;
  }

  /**
   * Methode setzt die Variable image
   * 
   * @param image
   *          zu bernehmender Wert fr image
   */
  public void setImage(String image)
  {
    _image = image;
  }
}
