/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.figures;

import org.eclipse.draw2d.AbstractBorder;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Insets;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.PointList;
import org.eclipse.swt.graphics.Color;

/**
 * Figure, welche eine frei definierbaren Fahrstreifen reprsentiert.               
 * 
 * @author Pittner
 * 
 */
public class FreeFormFahrStreifenFigure extends ColoredPolygon
{

  /**
   * Die Liste der Punktkoordinaten
   */
  private PointList _pl;

  /**
   * Die Location
   */
  private Point _pos;

  /**
   * Getter fuer die Liste der Punktkoordinaten
   * 
   * @return die Liste der Punktkoordinaten
   */
  public PointList getPointList()
  {
    return _pl;
  }

  /**
   * Setter fuer die Liste der Punktkoordinaten
   * 
   * @param pl
   *          die Liste der Punktkoordinaten
   */
  public void setPointList(PointList pl)
  {
    _pl = pl;
  }

  /**
   * Konstruktor.
   * 
   * @param name
   * @param pos
   * @param pl
   */
  public FreeFormFahrStreifenFigure(String name, Point pos, PointList pl)
  {
    super(ColorConstants.gray);

    _pos = pos;

    setOpaque(false);
    FreeFormFahrStreifenFigureBorder border = new FreeFormFahrStreifenFigureBorder(ColorConstants.gray);
    setBorder(border);
    createPointList(pl, pos);
  }

  /**
   * Konstruktor.
   * 
   * @param pointList
   * @param pos
   */
  public void createPointList(PointList pointList, Point pos)
  {
    if (pointList != null)
    {
      _pl = new PointList(pointList.size());
      for (int i = 0; i < pointList.size(); i++)
      {
        _pl.addPoint(pointList.getPoint(i));
      }
    }

    PointList pl = new PointList();
    pl.addAll(_pl);
    pl.translate(pos);

    if (_pl != null)
    {
      setPoints(pl);
    }
  }

  /**
   * Internal Class, implements a Border for {@link FreeFormFahrStreifenFigure}
   * 
   * @author Pittner
   * 
   */
  public class FreeFormFahrStreifenFigureBorder extends AbstractBorder
  {

    private Color color;

    private int width;

    /**
     * @return
     */
    public Color getColor()
    {
      return color;
    }

    /**
     * @param color
     */
    public void setColor(Color color)
    {
      this.color = color;
    }

    /**
     * @return
     */
    public int getWidth()
    {
      return width;
    }

    /**
     * @param width
     */
    public void setWidth(int width)
    {
      this.width = width;
    }

    /**
     * Constructor.
     * 
     * @param c
     * @param w
     */
    public FreeFormFahrStreifenFigureBorder(Color c, int w)
    {
      color = c;
      width = w;

    }

    /**
     * Constructor.
     * 
     * @param c
     */
    public FreeFormFahrStreifenFigureBorder(Color c)
    {
      this(c, 1);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.draw2d.Border#getInsets(org.eclipse.draw2d.IFigure)
     */
    public Insets getInsets(IFigure figure)
    {
      return new Insets(0, 0, 0, 0);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.draw2d.Border#paint(org.eclipse.draw2d.IFigure, org.eclipse.draw2d.Graphics,
     * org.eclipse.draw2d.geometry.Insets)
     */
    public void paint(IFigure figure, Graphics graphics, Insets insets)
    {

      graphics.setForegroundColor(color);
      graphics.setLineWidth(width);

      PointList copy = _pl.getCopy();
      copy.translate(_pos);

      graphics.drawPolygon(copy);
    }

  }

  public void setColor(Color color)
  {
    setFillColor(color);
    setBorder(new FreeFormFahrStreifenFigureBorder(color));
  }
}
