/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.figures;

import org.eclipse.draw2d.AbstractBorder;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Insets;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.PointList;
import org.eclipse.swt.graphics.Color;

/**
 * Figure, welche einen Ufd-Sichtweiten-Sensor reprsentiert.               
 * 
 * @author Pittner
 * 
 */
public class UfdSichtweitenFigure extends ColoredPolygon
{

  private PointList _pl;

  public UfdSichtweitenFigure(String name, Point pos)
  {
    super(ColorConstants.gray);

    setOpaque(false);
    setBorder(new UfdSichtweitenFigureBorder(super.getFillColor()));

    createPointList(pos);
  }

  public void createPointList(Point pos)
  {
    createPointList(pos.x, pos.y + 12);
  }

  public void createPointList(int x, int y)
  {
    int[] pts = new int[] { 0, 0, 12, 0, 6, 6, 24, 6, 18, 0, 30, 0, 30, 9, 21, 12, 21, 18, 9, 18, 9, 12, 0, 9, 0, 0 };

    _pl = null;
    if (pts != null)
    {
      _pl = new PointList(pts.length);
      for (int i = 0; i < pts.length - 1; i += 2)
      {
        int ptx = pts[i];
        int pty = pts[i + 1];
        _pl.addPoint(x + ptx, y + pty);
      }
    }

    if (_pl != null)
      setPoints(_pl);
  }

  public class UfdSichtweitenFigureBorder extends AbstractBorder
  {

    private Color color;

    public UfdSichtweitenFigureBorder(Color c)
    {
      color = c;
    }

    public Insets getInsets(IFigure figure)
    {
      return new Insets(0, 0, 0, 0);
    }

    public void paint(IFigure figure, Graphics graphics, Insets insets)
    {

      graphics.setForegroundColor(color);
      graphics.drawPolygon(_pl);
    }
  }
}
