/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.figures;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.FlowLayout;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.draw2d.ToolbarLayout;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.draw2d.geometry.Rectangle;

/**
 * Figure, welche eine Ufd-Messstelle reprsentiert.               
 * 
 * @author Pittner
 * 
 */
public class UmfeldDatenFigure extends Figure
{

  private IFigure _sichtweite;

  private IFigure _unten;

  private IFigure _oberflaeche;

  private IFigure _niederschlag;

  private Label _label;

  public UmfeldDatenFigure(IFigure umfeld)
  {

    this._sichtweite = umfeld;

    setOpaque(true);
    ToolbarLayout tl = new ToolbarLayout();

    tl.setVertical(false);

    setLayoutManager(tl);
    add(_sichtweite);
  }

  public UmfeldDatenFigure(IFigure sicht, IFigure ober, IFigure nieder)
  {
    this._sichtweite = sicht;
    this._oberflaeche = ober;
    this._niederschlag = nieder;

    setOpaque(false);
    setLayoutManager(new XYLayout());

    _unten = new Figure();
    _unten.setOpaque(true);

    FlowLayout fl = new FlowLayout();
    fl.setMinorSpacing(1);
    _unten.setLayoutManager(fl);
    _unten.add(_oberflaeche);
    _unten.add(_niederschlag);

    _label = new Label();
    _label.setOpaque(true);
    add(_label);
    setConstraint(_label, new Rectangle(0, 0, -1, -1));

    add(_sichtweite);
    setConstraint(_sichtweite, new Rectangle(0, 12, -1, -1));

    add(_unten);
    setConstraint(_unten, new Rectangle(0, 32, -1, -1));

    UmfeldDatenSensorOberflaechenFigure uof = (UmfeldDatenSensorOberflaechenFigure)ober;
    UmfeldDatenSensorNiederschlagFigure unf = (UmfeldDatenSensorNiederschlagFigure)nieder;

    String piduof = uof.getPid();
    String pidunf = unf.getPid();

    Label l1 = new Label(piduof);
    add(l1);
    setConstraint(l1, new Rectangle(0, 52, -1, -1));

    Label l2 = new Label(pidunf);
    add(l2);
    setConstraint(l2, new Rectangle(21, 52, -1, -1));
  }

  public IFigure getSichtweite()
  {
    return _sichtweite;
  }

  public void setSichtweite(IFigure sichtweite)
  {
    _sichtweite = sichtweite;
  }

  public void setSelected(boolean isSelected)
  {
    LineBorder lineBorder = (LineBorder)getBorder();
    if (isSelected)
    {
      lineBorder.setWidth(1);
      lineBorder.setColor(ColorConstants.black);
    }
    else
    {
      lineBorder.setWidth(2);
      lineBorder.setColor(ColorConstants.orange);
    }
  }

  public IFigure getOberflaeche()
  {
    return _oberflaeche;
  }

  public void setOberflaeche(IFigure ober)
  {
    this._oberflaeche = ober;
  }

  public IFigure getNiederschlag()
  {
    return _niederschlag;
  }

  public void setNiederschlag(IFigure nieder)
  {
    this._niederschlag = nieder;
  }

  public IFigure getUnten()
  {
    return _unten;
  }

  public void setUnten(IFigure unten)
  {
    _unten = unten;
  }

  public Label getLabel()
  {
    return _label;
  }

  public void setLabelSichtWeite(String text)
  {

    try
    {
      Integer weite = new Integer(text);

      _label.setText(weite.toString());
      _label.setForegroundColor(ColorConstants.black);
      _label.setBackgroundColor(getBackgroundColor());

    }
    catch (NumberFormatException e)
    {
      _label.setText("          ");
      _label.setForegroundColor(ColorConstants.white);
      _label.setBackgroundColor(ColorConstants.red);     
    }
  }

  public void setSichtStufe(String text)
  {
    try
    {
      Integer stufe = new Integer(text);
      
      if (stufe == 0)
      {
        ((UmfeldDatenSensorSichtweitenFigure)_sichtweite).setFarbe(ColorConstants.red);
      }
      else if (stufe == 1)
        ((UmfeldDatenSensorSichtweitenFigure)_sichtweite).setFarbe(ColorConstants.orange);
      else if (stufe == 2)
        ((UmfeldDatenSensorSichtweitenFigure)_sichtweite).setFarbe(ColorConstants.yellow);
      else if (stufe == 3)
        ((UmfeldDatenSensorSichtweitenFigure)_sichtweite).setFarbe(ColorConstants.green);
      else if (stufe == 4)
        ((UmfeldDatenSensorSichtweitenFigure)_sichtweite).setFarbe(ColorConstants.lightGreen);
      else
        ((UmfeldDatenSensorSichtweitenFigure)_sichtweite).setFarbe(ColorConstants.white);
    }
    catch (NumberFormatException e)
    {      
      ((UmfeldDatenSensorSichtweitenFigure)_sichtweite).setFarbe(ColorConstants.red);
    }

    ((UmfeldDatenSensorSichtweitenFigure)_sichtweite).repaint();
  }
}
