package de.bsvrz.buv.plugin.anlagendarstellung.model;

import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

/**
 * Speichert die Zuordnung zwischen Kilometrierung und Pixeln auf dem Schirm,
 * die durch den Picker dargestellt werden. Des Weiteren stehen diverse
 * Hilfsmethoden zum (De)Serialisieren zur Verfgung<br>
 * <br>
 * Um Lcken zu Vermeiden sollte der BIS-Wert eines Elmentes (KM und Y-Pixel)
 * auf den VON-Wert des Nachfolgelementes (KM und Y-Pixel) gesetzt werden.
 * 
 * @author Swarco Traffic Systems GmbH
 * 
 */
public class KilometrierungZuordnung
{
	/**
	 * Kilometer von
	 */
	private Double _kmVon;

	/**
	 * Kilometer bis
	 */
	private Double _kmBis;

	/**
	 * Pixel zur Abblidung von
	 */
	private Integer _yVon;

	/**
	 * Pixel zur Abbildung bis
	 */
	private Integer _yBis;

	public KilometrierungZuordnung()
	{
	}

	/**
	 * 
	 * @param kmVon
	 *          Kilometer von
	 * @param kmBis
	 *          Kilometer bis
	 * @param yVon
	 *          Pixel zur Abbildung von
	 * @param yBis
	 *          Pixel zur Abbildung bis
	 */
	public KilometrierungZuordnung(Double kmVon, Double kmBis, Integer yVon,
			Integer yBis)
	{
		_kmVon = kmVon;
		_kmBis = kmBis;
		_yVon = yVon;
		_yBis = yBis;
	}
	public Double getKmVon()
	{
		return _kmVon;
	}
	public void setKmVon(Double kmVon)
	{
		_kmVon = kmVon;
	}
	public Double getKmBis()
	{
		return _kmBis;
	}
	public void setKmBis(Double kmBis)
	{
		_kmBis = kmBis;
	}
	public Integer getyVon()
	{
		return _yVon;
	}
	public void setYVon(Integer yVon)
	{
		_yVon = yVon;
	}
	public Integer getyBis()
	{
		return _yBis;
	}
	public void setYBis(Integer yBis)
	{
		_yBis = yBis;
	}

	/**
	 * Serialisierung in einen String.
	 * 
	 * @return
	 */
	public String serToString()
	{

		if ((_kmVon != null) && (_kmBis != null) && (_yVon != null)
				&& (_yBis != null))
		{
			StringBuffer sb = new StringBuffer();
			sb.append("KM");
			sb.append(_kmVon);
			sb.append("-");
			sb.append(_kmBis);
			sb.append("@Y");
			sb.append(_yVon);
			sb.append("-");
			sb.append(_yBis);
			return sb.toString();
		} else
		{
			return null;
		}

	}

	/**
	 * Serialisiert eine Liste von {@link KilometrierungZuordnung} in eine mit
	 * Semikola separierte String-Liste.
	 * 
	 * @param lKmZu
	 * @return
	 */
	public static String serToString(List<KilometrierungZuordnung> lKmZu)
	{
		StringBuffer sb = new StringBuffer();

		if (lKmZu != null)
		{
			for (KilometrierungZuordnung kilometrierungZuordnung : lKmZu)
			{
				if (kilometrierungZuordnung != null)
				{
					String ser = kilometrierungZuordnung.serToString();
					if (ser != null)
					{
						if (sb.length() > 0)
						{
							sb.append(";");
						}
						sb.append(ser);
					}
				}
			}
		}

		return sb.toString();
	}

	/**
	 * Deserialisiert eine {@link KilometrierungZuordnung} aus einem String.
	 * 
	 * @param strKmSer
	 * @return
	 */
	public static KilometrierungZuordnung deserFromString(String strKmSer)
	{
		KilometrierungZuordnung kmDeser = null;

		if (strKmSer != null)
		{
			strKmSer = strKmSer.toUpperCase();
			kmDeser = new KilometrierungZuordnung();

			Double dKmVon = null;
			Double dKmBis = null;
			Integer iYVon = null;
			Integer iYBis = null;

			String[] split = strKmSer.split("@");
			if ((split != null) && (split.length > 1))
			{
				String kmVonBis = split[0];
				String yVonBis = split[1];

				/*
				 * Kilometrierung holen
				 */
				if ((kmVonBis != null) && (kmVonBis.startsWith("KM")))
				{
					kmVonBis = kmVonBis.substring(2, kmVonBis.length());
					if (kmVonBis.contains("-"))
					{
						String[] vonBis = kmVonBis.split("-");
						if (vonBis.length > 1)
						{
							String kmVon = vonBis[0];
							String kmBis = vonBis[1];

							try
							{
								dKmVon = new Double(kmVon);
							} catch (NumberFormatException nfe)
							{
								nfe.printStackTrace();
							}

							try
							{
								dKmBis = new Double(kmBis);
							} catch (NumberFormatException nfe)
							{
								nfe.printStackTrace();
							}

						}
					}
				}

				/*
				 * Y-Positions Zuordnung holen
				 */
				if ((yVonBis != null) && (yVonBis.contains("Y")))
				{
					yVonBis = yVonBis.substring(1, yVonBis.length());
					if (yVonBis.contains("-"))
					{
						String[] vonBis = yVonBis.split("-");
						if (vonBis.length > 1)
						{
							String yVon = vonBis[0];
							String yBis = vonBis[1];

							try
							{
								iYVon = new Integer(yVon);
							} catch (NumberFormatException nfe)
							{
								nfe.printStackTrace();
							}

							try
							{
								iYBis = new Integer(yBis);
							} catch (NumberFormatException nfe)
							{
							  
							  System.out.println("KilometrierungZuordnung.deserFromString(yBis): " + yBis + " -> " + strKmSer);
								nfe.printStackTrace();
							}

						}
					}
				}

			}

			kmDeser.setKmVon(dKmVon);
			kmDeser.setKmBis(dKmBis);
			kmDeser.setYVon(iYVon);
			kmDeser.setYBis(iYBis);

		}

		return kmDeser;
	}

	/**
	 * Deserialisiert eine mit Semikola separierte Strin Liste von
	 * {@link KilometrierungZuordnung} in ein Liste.
	 * 
	 * @param strKmMultiple
	 * @return
	 */
	public static LinkedList<KilometrierungZuordnung> deserFromMultipleString(
			String strKmMultiple)
	{
		LinkedList<KilometrierungZuordnung> list = new LinkedList<KilometrierungZuordnung>();

		if ((strKmMultiple != null) && (strKmMultiple.contains(";")))
		{
			String[] aSplitSer = strKmMultiple.split(";");
			if ((aSplitSer != null) && (aSplitSer.length > 0))
			{
				for (String serKm : aSplitSer)
				{
					if (serKm != null)
					{
						KilometrierungZuordnung km = deserFromString(serKm);
						if (km != null)
						{
							list.add(km);
						}
					}
				}
			}
		}

		return list;
	}

	@Override
	public String toString()
	{
		return "KilometrierungZuordnung [_kmBis=" + _kmBis + ", _kmVon=" + _kmVon
				+ ", _yBis=" + _yBis + ", _yVon=" + _yVon + "]";
	}

	// @Override
	// public int hashCode()
	// {
	// final int prime = 31;
	// int result = 1;
	// result = prime * result + ((_kmBis == null) ? 0 : _kmBis.hashCode());
	// result = prime * result + ((_kmVon == null) ? 0 : _kmVon.hashCode());
	// result = prime * result + ((_yBis == null) ? 0 : _yBis.hashCode());
	// result = prime * result + ((_yVon == null) ? 0 : _yVon.hashCode());
	// return result;
	// }
	//
	// @Override
	// public boolean equals(Object obj)
	// {
	// if (this == obj)
	// return true;
	// if (obj == null)
	// return false;
	// if (getClass() != obj.getClass())
	// return false;
	// KilometrierungZuordnung other = (KilometrierungZuordnung) obj;
	// if (_kmBis == null)
	// {
	// if (other._kmBis != null)
	// return false;
	// } else if (!_kmBis.equals(other._kmBis))
	// return false;
	// if (_kmVon == null)
	// {
	// if (other._kmVon != null)
	// return false;
	// } else if (!_kmVon.equals(other._kmVon))
	// return false;
	// if (_yBis == null)
	// {
	// if (other._yBis != null)
	// return false;
	// } else if (!_yBis.equals(other._yBis))
	// return false;
	// if (_yVon == null)
	// {
	// if (other._yVon != null)
	// return false;
	// } else if (!_yVon.equals(other._yVon))
	// return false;
	// return true;
	// }

	public static boolean isAufsteigend(double von, double bis)
	{
		boolean aufsteigend = false;

		if (von <= bis)
		{
			aufsteigend = true;
		}

		return aufsteigend;
	}

	public static boolean isAufsteigend(int von, int bis)
	{
		boolean aufsteigend = false;

		if (von < bis)
		{
			aufsteigend = true;
		}

		return aufsteigend;
	}

	/**
	 * Validiert eine Liste {@link KilometrierungZuordnung}: Die Werte fr die
	 * Kilometer sowie Y-Postitionen mssen gleichbleibend auf- bzw. absteigend
	 * sein.
	 * 
	 * @param lKilometrierungZuordnung
	 * @return
	 */
	public static boolean validate(
			List<KilometrierungZuordnung> lKilometrierungZuordnung)
	{
		boolean valid = true;

		Boolean aufsteigendKm = null;
		Boolean aufsteigendY = null;

		if ((lKilometrierungZuordnung != null)
				&& (lKilometrierungZuordnung.size() > 0))
		{
			for (KilometrierungZuordnung kilometrierungZuordnung : lKilometrierungZuordnung)
			{
				if (kilometrierungZuordnung != null)
				{
					if ((kilometrierungZuordnung.getKmVon() != null)
							&& (kilometrierungZuordnung.getKmBis() != null))
					{
						if (aufsteigendKm == null)
						{
							aufsteigendKm = isAufsteigend(kilometrierungZuordnung.getKmVon(),
									kilometrierungZuordnung.getKmBis());
						} else if (aufsteigendKm.booleanValue() != isAufsteigend(
								kilometrierungZuordnung.getKmVon(), kilometrierungZuordnung
										.getKmBis()))
						{
							valid = false;
							break;
						}
					}

					if ((kilometrierungZuordnung.getyVon() != null)
							&& (kilometrierungZuordnung.getyBis() != null))
					{
						if (aufsteigendY == null)
						{
							aufsteigendY = isAufsteigend(kilometrierungZuordnung.getyVon(),
									kilometrierungZuordnung.getyBis());
						} else if (aufsteigendY.booleanValue() != isAufsteigend(
								kilometrierungZuordnung.getyVon(), kilometrierungZuordnung
										.getyBis()))
						{
							valid = false;
							break;
						}
					}

				}
			}
		}

		return valid;
	}

	/**
	 * Bestimmt aus einer Liste von {@link KilometrierungZuordnung} diejenige
	 * {@link KilometrierungZuordnung}, in der ein bestimmter Kilometer liegt.
	 * Beim letzten Element wird gem der auf- bzw. absteigenden Richtung
	 * geprft.
	 * 
	 * @param lKmZu
	 *          Liste der {@link KilometrierungZuordnung}.
	 * @param km
	 *          Kilometer fr den die passende {@link KilometrierungZuordnung} zu
	 *          bestimmen ist.
	 * @return Zu dem Kilometer zugehrige {@link KilometrierungZuordnung} oder
	 *         <code>null</code>, falls nicht verfgbar.
	 */
	public static KilometrierungZuordnung bestimmeKilometrierungZuordnungFuerKm(
			List<KilometrierungZuordnung> lKmZu, double km)
	{
		KilometrierungZuordnung re = null;

		if (lKmZu != null)
		{
			/*
			 * Speichert die auf- bzw. absteigende Richtung.
			 */
			Boolean aufsteigend = null;

			for (KilometrierungZuordnung kilometrierungZuordnung : lKmZu)
			{
				if (kilometrierungZuordnung != null)
				{
					if ((kilometrierungZuordnung.getKmVon() != null)
							&& (kilometrierungZuordnung.getKmBis() != null))
					{
						double von = kilometrierungZuordnung.getKmVon().doubleValue();
						double bis = kilometrierungZuordnung.getKmBis().doubleValue();

						if (bis < von)
						{
							/*
							 * Sicherstellen dass von immer kleiner bis ist.
							 */
							double temp = bis;
							bis = von;
							von = temp;

							if (aufsteigend == null)
							{
								aufsteigend = false;
							}
						} else
						{
							aufsteigend = true;
						}

						if ((km >= von) && (km < bis))
						{
							re = kilometrierungZuordnung;
							break;
						}
					}
				}
			}

			/*
			 * Es wurde noch kein passendes Element gefunden. Jetzt prfen wir noch
			 * das letzte Element gem der auf- bzw. absteigenden Richtung.
			 */
			if ((re == null) && (lKmZu.size() > 0))
			{
				/*
				 * Letztes Element prfen
				 */
				KilometrierungZuordnung lastKmZu = lKmZu.get(lKmZu.size() - 1);
				if ((lastKmZu != null) && (aufsteigend != null)
						&& (lastKmZu.getKmVon() != null))
				{
					if (aufsteigend.booleanValue())
					{
						/*
						 * Aufsteigend
						 */
						if (km >= lastKmZu.getKmVon())
						{
							re = lastKmZu;
						}
					} else
					{
						/*
						 * Absteigend
						 */
						if (km <= lastKmZu.getKmVon())
						{
							re = lastKmZu;
						}
					}
				}
			}
		}

		return re;
	}

	public static KilometrierungZuordnung bestimmeKilometrierungZuordnungFuerYPos(
			List<KilometrierungZuordnung> lKmZu, int y)
	{
		KilometrierungZuordnung re = null;

		if (lKmZu != null)
		{
			/*
			 * Speichert die auf- bzw. absteigende Richtung.
			 */
			Boolean aufsteigend = null;

			for (KilometrierungZuordnung kilometrierungZuordnung : lKmZu)
			{
				if (kilometrierungZuordnung != null)
				{
					Integer iYVon = kilometrierungZuordnung.getyVon();
					Integer iYBis = kilometrierungZuordnung.getyBis();

					if ((iYVon != null) && (iYBis != null))
					{
						int yVon = iYVon.intValue();
						int yBis = iYBis.intValue();

						/*
						 * Bestimmen ob Aufsteigend oder Fallend
						 */
						if (yBis < yVon)
						{
							aufsteigend = false;

							int temp = yVon;
							yVon = yBis;
							yBis = temp;

						} else
						{
							aufsteigend = true;
						}

						if ((y >= yVon) && (y < yBis))
						{
							re = kilometrierungZuordnung;
							break;
						}

					}
				}
			}

			/*
			 * Es wurde noch kein passendes Element gefunden. Jetzt prfen wir noch
			 * das letzte Element gem der auf- bzw. absteigenden Richtung.
			 */
			if ((re == null) && (lKmZu.size() > 0))
			{
				/*
				 * Letztes Element prfen
				 */
				KilometrierungZuordnung lastKmZu = lKmZu.get(lKmZu.size() - 1);
				if ((lastKmZu != null) && (aufsteigend != null)
						&& (lastKmZu.getyVon() != null))
				{
					if (aufsteigend.booleanValue())
					{
						/*
						 * Aufsteigend
						 */
						if (y >= lastKmZu.getyVon().intValue())
						{
							re = lastKmZu;
						}
					} else
					{
						/*
						 * Absteigend
						 */
						if (y <= lastKmZu.getyVon().intValue())
						{
							re = lastKmZu;
						}
					}
				}
			}
		}

		return re;
	}

	public static void main(String[] args)
	{
		KilometrierungZuordnung kmOrg1 = new KilometrierungZuordnung(1.0D, 423.4D,
				30, 666);
		KilometrierungZuordnung kmOrg2 = new KilometrierungZuordnung(2.0D, 523.4D,
				50, 766);
		String ser1 = kmOrg1.serToString();
		String ser2 = kmOrg2.serToString();
		KilometrierungZuordnung kmDeser = deserFromString(ser1);

		System.out.println("Original:       " + kmOrg1);
		System.out.println("Serialisiert:   " + ser1);
		System.out.println("Deserialisiert: " + kmDeser);
		System.out.println("Gleich -> " + kmOrg1.equals(kmDeser));
		System.out.println("\nNun als Liste");

		LinkedList<KilometrierungZuordnung> listOrg = new LinkedList<KilometrierungZuordnung>();
		listOrg.add(kmOrg1);
		listOrg.add(kmOrg2);
		String strList = serToString(listOrg);
		LinkedList<KilometrierungZuordnung> lDeser = deserFromMultipleString(strList);

		System.out.println("Liste:        " + listOrg);
		System.out.println("Liste ser.:   " + strList);
		System.out.println("Liste deser.: " + lDeser.toString());

	}

}
