package de.bsvrz.buv.plugin.anlagendarstellung.model;

/**
 * Diese Klasse liefert diverse mathematische Hilfsmethoden.
 * 
 * @author Swarco Traffic Systems GmbH
 * 
 */
public class MathHelper
{
	public static final boolean DEBUG = false;

	/**
	 * Liefert den Fortschritt eines Punktes in Prozent innerhalb eines Bereichs.
	 * 
	 * @param von
	 *          Start-Wert des Bereichs
	 * @param bis
	 *          End-Wert des Bereichs
	 * @param punkt
	 *          Wert des Punktes innerhalb des Bereichs
	 * @return Prozentsatz des Punktes innerhalb des Bereichs.
	 */
	public static double berechneFortschrittProzent(double von, double bis,
			double punkt)
	{
		double p = -1.0d;

		double distanz = bis - von;
		double pNeu = punkt - von;

		p = (pNeu / distanz) * 100.0d;

		if (DEBUG)
		{
			System.out.println("berechneFortschrittProzent von=" + von + " bis="
					+ bis + " Punkt=" + punkt + " => " + p + "%");
		}

		return p;
	}

	/**
	 * Liefert den Fortschritt eines Punktes in Prozent innerhalb eines Bereichs.
	 * 
	 * @param von
	 *          Start-Wert des Bereichs
	 * @param bis
	 *          End-Wert des Bereichs
	 * @param punkt
	 *          Wert des Punktes innerhalb des Bereichs
	 * @return Prozentsatz des Punktes innerhalb des Bereichs.
	 */
	public static int berechneFortschrittProzentAsInt(int von, int bis, int punkt)
	{
		int p = -1;

		p = (int) berechneFortschrittProzent((double) von, (double) bis,
				(double) punkt);

		if (DEBUG)
		{
			System.out.println("\t->Als int: " + p);
		}

		return p;
	}

	/**
	 * Berechnet die absolute Position innerhalb eines Bereichs nach Prozent.
	 * 
	 * @param von
	 *          Start-Wert des Bereichs
	 * @param bis
	 *          End-Wert des Bereichs.
	 * @param prozent
	 *          Prozentuale Position innerhalb des Bereichs
	 * @return Absolute Position innerhalb des Bereichs.
	 */
	public static double berechnePositionNachProzent(double von, double bis,
			double prozent)
	{
		double position = -1;

		double distanz = bis - von;
		double W = distanz * (prozent / 100.0d);

		boolean aufsteigend = (von <= bis);

		position = von + W;

		if (DEBUG)
		{
			System.out.println("von=" + von + " bis=" + bis + " (Aufsteigend="
					+ aufsteigend + ") %=" + prozent + " => " + position
					+ " Pixel ( BEREICHPOS=" + W + ")");
		}

		return position;
	}

	/**
	 * Berechnet die absolute Position innerhalb eines Bereichs nach Prozent.
	 * 
	 * @param von
	 *          Start-Wert des Bereichs
	 * @param bis
	 *          End-Wert des Bereichs.
	 * @param prozent
	 *          Prozentuale Position innerhalb des Bereichs
	 * @return Absolute Position innerhalb des Bereichs.
	 */
	public static int berechnePositionNachProzentAsInt(int von, int bis,
			int prozent)
	{
		int position = -1;

		position = (int) berechnePositionNachProzent((double) von, (double) bis,
				(double) prozent);

		return position;
	}

	public static void main(String[] args)
	{
		berechneFortschrittProzent(0, 100, 20);
		berechneFortschrittProzent(-50, 50, 20);
		berechneFortschrittProzent(100, 0, 20);
		berechneFortschrittProzent(-20, -100, -61);
		berechneFortschrittProzentAsInt(0, 50, 5);

		berechnePositionNachProzent(0, 100, 23.0d);
		berechnePositionNachProzent(0, 50, 23.0d);
		berechnePositionNachProzent(-100, 0, 23.0d);
		berechnePositionNachProzent(50, -50, 25.0d);
		berechnePositionNachProzent(50, -50, 60.0d);
	}

}
