/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.bsvrz.buv.plugin.anlagendarstellung.programmtyp;

import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.Assert;
import org.eclipse.emf.common.util.EList;
import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.plugin.anlagendarstellung.AnlagendarstellungPackage;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeQuerschnittModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.dav.DaVKommunikationDobj;
import de.bsvrz.buv.plugin.anlagendarstellung.model.AnzeigeInhaltTyp;
import de.bsvrz.buv.plugin.anlagendarstellung.util.DaVHelper;
import de.bsvrz.buv.plugin.anlagendarstellung.util.DataHelper;
import de.bsvrz.buv.plugin.darstellung.model.Ansicht;
import de.bsvrz.buv.plugin.darstellung.model.Darstellung;
import de.bsvrz.buv.plugin.darstellung.model.Ebene;
import de.bsvrz.buv.plugin.dobj.model.DoModel;
import de.bsvrz.buv.rw.basislib.login.UrlasserDialog;
import de.bsvrz.buv.rw.basislib.login.UrlasserDialog.UrlasserDialogInfo;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.NonMutableSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

/**
 * View-Klasse, welche die Sicht der Parameter Einzelmassnahmen darstellt
 * 
 * @author Pittner
 * 
 */
public class SonderProgrammTypViewParameter extends ViewPart
{
  public static String ID = "de.bsvrz.buv.plugin.anlagendarstellung.views.sonderProgrammTypViewParameter";

  /**
   * Attributgruppe des DAV, die bercksichtigt wird
   */
  private static final String _attributgruppe = "atg.sswSbaProgrammTyp";

  /**
   * Aspekt des DAV, der bercksichtigt wird
   */
  private static final String _aspektEmpfangen = "asp.parameterSoll";

  /**
   * Aspekt des DAV, der bercksichtigt wird
   */
  private static final String _aspektVersenden = "asp.parameterVorgabe";

  /**
   * Composites
   */
  private Composite _composite;

  private Composite _composite3;

  private Composite _compositeButton3;

  /**
   * Schaltflaechen
   */
  private Button _buttonUbernehmen;

  private Button _buttonOk;

  private Button _buttonAbbrechen;

  private Button _buttonHilfe;

  /**
   * Die Verbindung zum Datenverteiler
   */
  private ClientDavInterface _connection;

  /**
   * Das Register fr die einzelnen Reiter
   */
  private CTabFolder _tabFolder;

  /**
   * Die Secondary View-ID
   */
  private String _secondViewId;

  /**
   * Das Speicherobjekt zum Wiederherstellen des Zustandes nach dem Start der View
   */
  private IMemento _memento;

  @Override
  public void init(IViewSite site, IMemento memento) throws PartInitException
  {
    _memento = memento;
    _secondViewId = site.getSecondaryId();
    super.init(site, memento);
  }

  /**
   * Stellt den Zustand vor dem Beenden das Sicht wieder her
   */
  private void restoreState()
  {
    if (_memento == null)
    {
      return;
    }
    _secondViewId = _memento.getString("pidMemento");
  }

  @Override
  public void saveState(IMemento memento)
  {
    memento.putString("pidMemento", _secondViewId);
    super.saveState(memento);
  }

  @Override
  public void createPartControl(Composite parent)
  {
    _composite = new Composite(parent, SWT.NONE);
    _composite.setLayout(new GridLayout());
    _composite.setLayoutData(new GridData());

    // Fr die F1 Taste
//    PlatformUI.getWorkbench().getHelpSystem().setHelp(_composite,
//        de.bsvrz.buv.plugin.parameter.tsf.Activator.PLUGIN_ID + ".ablauftsf");

    createCompositeTab();
    createCompositeButton();
    restoreState();

    _connection = DaVKommunikationDobj.getInstanz().holeDaVVerbindung();

    if (_connection != null && _secondViewId != null)
    {
      List<SystemObject> list = holeSystemObjekteAusSecondId();

      if (list.size() > 0)
      {
        erzeugeTabItems(list);
      }
    }
  }

  /**
   * Holt die {@code SystemObject} der Ursacheneinheiten aus der SecondaryId.
   * 
   * @return
   */
  private List<SystemObject> holeSystemObjekteAusSecondId()
  {
    List<SystemObject> list = new ArrayList<SystemObject>();
    if (_secondViewId.contains(";"))
    {
      String[] split = _secondViewId.split(";");
      for (String s : split)
      {
        SystemObject so = _connection.getDataModel().getObject(s);
        if (so != null)
          list.add(so);
      }
    }
    else
    {
      SystemObject so = _connection.getDataModel().getObject(_secondViewId);
      if (so != null)
        list.add(so);
    }
    return list;
  }

  /**
   * This method initializes _tabFolder (Bereich ber der ButtonLeiste)
   * 
   */
  private void createCompositeTab()
  {

    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    gridData.verticalAlignment = GridData.FILL;

    _tabFolder = new CTabFolder(_composite, SWT.NONE);
    _tabFolder.setLayout(new GridLayout());
    _tabFolder.setLayoutData(gridData);
    _tabFolder.addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        CTabFolder folder = (CTabFolder)e.getSource();
        CTabItem itemSel = folder.getSelection();
        SystemObject so = (SystemObject)itemSel.getData(itemSel.getText());

        Composite cSel = (Composite)itemSel.getControl();
        Control[] children = cSel.getChildren();

        CompositeParameterProgramme tabComp = null;

        if (children.length == 0)
        {
          GridData gridData = new GridData();
          gridData.horizontalAlignment = GridData.FILL;
          gridData.grabExcessHorizontalSpace = true;
          gridData.grabExcessVerticalSpace = true;
          gridData.verticalAlignment = GridData.FILL;

          ResultData rd = DaVKommunikationDobj.getInstanz().leseDatenDirektVonDaV(so.getPid(), _attributgruppe,
              _aspektEmpfangen);

          tabComp = new CompositeParameterProgramme(getViewSite(), cSel, SWT.NONE, rd);
          tabComp.setLayout(new GridLayout());
          tabComp.setLayoutData(gridData);
        }
        else
        {
          tabComp = (CompositeParameterProgramme)cSel.getChildren()[0];
        }

        cSel.layout();
        if (tabComp != null)
        {
          tabComp.layout();
        }
      }
    });
  }
  
  /**
   * Reiter fr die Liste der Objekte erstellen
   * 
   * @param list
   */
  public void erzeugeTabItems(List<SystemObject> list)
  {

    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.verticalAlignment = GridData.FILL;

    for (SystemObject so : list)
    {
      CTabItem tabItem = new CTabItem(_tabFolder, SWT.NONE);

      String tabName = so.getName();
      tabItem.setText(tabName);
      tabItem.setData(tabName, so);

      Composite c1 = new Composite(_tabFolder, SWT.NONE);
      c1.setLayout(new GridLayout());
      c1.setLayoutData(gridData2);
  
      ResultData rd = DaVKommunikationDobj.getInstanz().leseDatenDirektVonDaV(so.getPid(), _attributgruppe,
          _aspektEmpfangen);
  
      CompositeParameterProgramme tabComp = new CompositeParameterProgramme(getViewSite(), c1, SWT.NONE, rd);
      tabComp.setLayout(new GridLayout());
      tabComp.setLayoutData(gridData2);          
      tabItem.setControl(c1);
    }

    _tabFolder.setSelection(0);
  }

  /**
   * This method initializes _composite3
   * 
   */
  private void createCompositeButton()
  {

    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    _composite3 = new Composite(_composite, SWT.NONE);
    _composite3.setLayout(new GridLayout());
    _composite3.setLayoutData(gridData);

    createCompositeButton3();
  }

  /**
   * This method initializes _compositeButton3
   * 
   */
  private void createCompositeButton3()
  {
    RowLayout rowLayout = new RowLayout();
    rowLayout.justify = false;
    rowLayout.wrap = true;
    rowLayout.pack = false;
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.END;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.CENTER;
    _compositeButton3 = new Composite(_composite3, SWT.NONE);
    _compositeButton3.setLayout(rowLayout);
    _compositeButton3.setLayoutData(gridData);

    _buttonOk = new Button(_compositeButton3, SWT.NONE);
    _buttonOk.setText("Ok");
    _buttonOk.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionSchalten();
      }
    });

    _buttonUbernehmen = new Button(_compositeButton3, SWT.NONE);
    _buttonUbernehmen.setText("bernehmen");
    _buttonUbernehmen.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionUebernehmen();
      }
    });

    _buttonAbbrechen = new Button(_compositeButton3, SWT.NONE);
    _buttonAbbrechen.setText("Abbrechen");
    _buttonAbbrechen.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionAbbrechen();
      }
    });

    _buttonHilfe = new Button(_compositeButton3, SWT.NONE);
    _buttonHilfe.setText("Hilfe");
    _buttonHilfe.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionHilfe();
      }
    });
  }

  /**
   * Daten an DaV senden, danach schliesen
   */
  protected void aktionSchalten()
  {
    if (schaltenUndParametrieren())
    {
      aktionAbbrechen();
    }
    else
    {
      MessageBox mb = new MessageBox(new Shell(Display.getCurrent()), SWT.ICON_INFORMATION | SWT.OK);
      mb.setText("Fehler Urlasser");
      mb.setMessage("Objekt konnte nicht parametriert werden: " + "\nEventuell kein Programm ausgewhlt");
      mb.open();
      return;
    }

  }

  /**
   * Daten an DaV senden
   */
  protected void aktionUebernehmen()
  {
    if (schaltenUndParametrieren())
    {
      // keine weiteren Aktionen
    }
    else
    {
      MessageBox mb = new MessageBox(new Shell(Display.getCurrent()), SWT.ICON_INFORMATION | SWT.OK);
      mb.setText("Fehler Urlasser");
      mb.setMessage("Objekt konnte nicht parametriert werden: " + "\nEventuell keinen AQ ausgewhlt");
      mb.open();
      return;
    }

  }

  /**
   * Methode bernimmt das Versenden der Daten
   * 
   * @return true wenn erfolgreich
   */
  private boolean schaltenUndParametrieren()
  {
    if (_connection != null)
    {
      AttributeGroup atg = _connection.getDataModel().getAttributeGroup(_attributgruppe);
      Aspect asp = _connection.getDataModel().getAspect(_aspektVersenden);

      Map<CompositeParameterProgramme, ResultData> map = new LinkedHashMap<CompositeParameterProgramme, ResultData>();

      CTabItem[] items = _tabFolder.getItems();
      for (CTabItem item : items)
      {
        String tabName = item.getText();
        SystemObject so = (SystemObject)item.getData(tabName);

        Data data = _connection.createData(atg).createModifiableCopy();

        if (!uebernehmeDatenAusDarstellung(item, data))
          return false;

        Composite c = (Composite)item.getControl();
        CompositeParameterProgramme compPar = (CompositeParameterProgramme)c.getChildren()[0];
        
        if (!SonderProgrammTypDataHelper.isEqual(data, compPar.getResultData().getData())) {
        	  long l = new Date().getTime();        	  
        	  
            ResultData rd = new ResultData(so, new DataDescription(atg, asp), l, data);            
            map.put(compPar, rd);	
        }
      }
      
      if (!map.isEmpty()) {
    	  UrlasserDialog uld = new UrlasserDialog(PlatformUI.getWorkbench()
    	          .getDisplay().getActiveShell());
    	             
	      UrlasserDialogInfo anzeigen = uld.anzeigen(map.values());
	      boolean eingabeOk = anzeigen.getEingabeOk();
	      
	      if (eingabeOk && (anzeigen.getFehlerInfo() == null)){
	    	 	for (Map.Entry<CompositeParameterProgramme, ResultData> entry: map.entrySet()) {
		    		entry.getKey().setResultData(entry.getValue());
		    	}
		    	return true;
		    }
		    else {
		    	return false;
	      }      
      }
      else {
    	  return true;
      }
    }
    return false;
  }

  /**
   * Kopiert die Daten aus der Darstellung in das uebergebene Daten-Objekt
   * 
   * @param data
   *          das Ziel in welches die Daten kopiert werden
   */
  private boolean uebernehmeDatenAusDarstellung(CTabItem itemAbschnitt, Data data)
  {
    Composite itemComp = (Composite)itemAbschnitt.getControl();
    CompositeParameterProgramme compPrg = (CompositeParameterProgramme)itemComp.getChildren()[0];

    CompositeParameterBereiche[] compsStufen = compPrg.getCompositeParameterStufen();
    Array arrayPrgs = data.getItem("ProgrammTypen").asArray();
    
    arrayPrgs.setLength(compsStufen.length);
    for (int i = 0; i < arrayPrgs.getLength(); i++)
    {  
      Data itemPrg = arrayPrgs.getItem(i);
      CompositeParameterBereiche comp2 = compsStufen[i];
      
      String textBezeichnung = comp2.getTextBezeichnung().getText();      
      itemPrg.getItem("Bezeichnung").asTextValue().setText(textBezeichnung);
      
      String handPrg = comp2.getRadioButtonHandSchaltung().getSelection() ? "Ja" : "Nein";      
      itemPrg.getItem("Handschaltung").getItem("Handprogramm").asTextValue().setText(handPrg);
      String strPrio = comp2.getComboPrio().getText();      
      itemPrg.getItem("Handschaltung").getItem("Prioritt").asTextValue().setText(strPrio);
      
      String spurSperrung = comp2.getRadioButtonSpurSperrung().getSelection() ? "Ja" : "Nein";      
      itemPrg.getItem("Spursperrung").asTextValue().setText(spurSperrung);      
      
      CompositeParameterDefinitionen[] compsEntfernung = comp2.getCompositeParameterStauBeleg();      
//      System.out.println("compsLength: " + compsEntfernung.length);
      for (int j = 0; j < compsEntfernung.length; j++)
      {
        CompositeParameterDefinitionen comp3 = compsEntfernung[j];        
        
        String bezeichnung = "";
        
        switch(j){
          case 2:
            bezeichnung = "Kernbereich";
            break;
          case 3:
            bezeichnung = "Nachbereich";
            break;
          case 1:
            bezeichnung = "Vorbereich1";
            break;
          case 0:
            bezeichnung = "Vorbereich2";
            break;
        }
        
        Array array = comp3.getAktuelleDaten().getItem("ProgrammTypen").asArray();
        
        int tmp = 0;
        
        if (i < array.getLength()){
          tmp = i;
        }else{
          tmp = array.getLength()-1;
        }
        
        Data dataComp3 = array.getItem(tmp);
        if (bezeichnung.length() > 0){
          Data quelle1 = dataComp3.getItem(bezeichnung).getItem("gesperrt");         
          Data ziel1 = itemPrg.getItem(bezeichnung).getItem("gesperrt");
          ziel1.asArray().setLength(quelle1.asArray().getLength());
          
          DataHelper.kopiereRekursiv(quelle1, ziel1);
          
          Data quelle2 = dataComp3.getItem(bezeichnung).getItem("nichtGesperrt");         
          Data ziel2 = itemPrg.getItem(bezeichnung).getItem("nichtGesperrt");
          ziel2.asArray().setLength(quelle2.asArray().getLength());
          
          DataHelper.kopiereRekursiv(quelle2, ziel2);
        }
      }      
    }
    return true;
  }

  /**
   * Sicht wird geschlossen
   */
  protected void aktionAbbrechen()
  {
    dispose();
  }

  /**
   * Zeigt die Hilfe zum Dialog an.
   * 
   */
  protected void aktionHilfe()
  {
//    String contextId = de.bsvrz.buv.plugin.anl.Activator.PLUGIN_ID + ".einzelmassnahmen";
//    PlatformUI.getWorkbench().getHelpSystem().displayHelp(contextId);
  }

  @Override
  public void dispose()
  {
    super.dispose();    
    for (CTabItem itemSel : _tabFolder.getItems())
    {           
      if (!itemSel.isDisposed()){
        Composite cSel = (Composite)itemSel.getControl();
        if (!cSel.isDisposed()){        
          Control[] children = cSel.getChildren();        
          CompositeParameterProgramme tabComp = null;
          
          if (children.length != 0)
          {
            tabComp = (CompositeParameterProgramme)cSel.getChildren()[0];
            if (!tabComp.isDisposed())
              tabComp.dispose();
            tabComp = null;
          }              
          cSel.dispose();
        }
        cSel = null;  
        itemSel.dispose();
      }
      itemSel = null;
    }
    IWorkbench wb = PlatformUI.getWorkbench();
    if (wb != null){
      IWorkbenchWindow dw = wb.getActiveWorkbenchWindow();
      if (dw != null)
      {
        IWorkbenchPage page = dw.getActivePage();
        if (page != null){
          page.hideView(this);
        }
      }      
    }
  }

  @Override
  public void setFocus()
  {
    // Fr die F1 Taste
    if (_composite != null)
      _composite.setFocus();
  }

  /**
   * Adaptable implementation for {@code GraphicalEditorToView}
   */
  public Object getAdapter(Class adapter)
  {
    // the super implementation handles the rest
    return super.getAdapter(adapter);
  }

  public CTabFolder getTabFolder()
  {
    return _tabFolder;
  }

  public void setTabFolder(CTabFolder tabFolder)
  {
    _tabFolder = tabFolder;
  } 
  
  /**
   * Holt anhand des AnzeigeTyps des bergebenen Anzeige aus dem String des Bildinhaltes {@link AnzeigeInhaltTyp} das
   * entsprechende WVZ-Objekt
   * 
   * @param connection
   *          der DaV-Verbindung
   * @param anzeige
   *          das SystemObject der Anzeige
   * @param bildInhalt
   *          der eindeutige String des WVZ-Inhaltes
   * @return das Objekt vom Typ WVZ-Inhalt
   * 
   */
  public SystemObject holeWVZInhaltVonBildInhaltTsf(SystemObject anzeige, String bildInhalt)
  {
    if (anzeige == null || bildInhalt == null)
      throw new IllegalArgumentException("Uebergabe-Parameter ist null: " + _connection + " - " + bildInhalt);
    
    if (!bildInhalt.equals("DontCare")){
      AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.anzeige");
      Data cd = anzeige.getConfigurationData(atg);
      SystemObject anzeigeTyp = cd.getItem("AnzeigeTyp").asReferenceValue().getSystemObject();
      
      NonMutableSet set = ((ConfigurationObject)anzeigeTyp).getNonMutableSet("WvzInhalt");
      if (set != null)
      {
        List<SystemObject> elements = set.getElements();
        for (SystemObject so : elements)
        {
          atg = _connection.getDataModel().getAttributeGroup("atg.wvzInhalt");
          cd = so.getConfigurationData(atg);
          String s = cd.getItem("Bildinhalt").asTextValue().getText();
          if (s.equals(bildInhalt))
          {
            return so;
          }
        }
      }      
    }else{
      SystemObjectType type = _connection.getDataModel().getType("typ.wvzInhalt");
      List<SystemObject> elements = type.getElements();     
      for (SystemObject so : elements)
      {
        AttributeGroup  atg = _connection.getDataModel().getAttributeGroup("atg.wvzInhalt");
        Data cd = so.getConfigurationData(atg);
        String s = cd.getItem("Bildinhalt").asTextValue().getText();
        if (s.equals(bildInhalt))
        {
//          System.out.println("DontCare: " + so);
          return so;
        }
      }
    }    
    return null;
  }  
}
