/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.bsvrz.buv.plugin.anlagendarstellung.semi;

import java.awt.Toolkit;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.CheckStateChangedEvent;
import org.eclipse.jface.viewers.CheckboxTableViewer;
import org.eclipse.jface.viewers.ICheckStateListener;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.plugin.anlagendarstellung.Activator;
import de.bsvrz.buv.plugin.anlagendarstellung.preferences.DefsPreference;
import de.bsvrz.buv.rw.basislib.login.UrlasserDialog;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Dialog mit dem eine Schaltung bzw. explizite NICHT-Schaltung der UrsachenEnheiten ausgelst werden kann. Es wird
 * hierzu eine MassnahmenAntwortGUI versendet.<br>
 * <br>
 * Der Button Schalten bzw. Nicht Schalten darf nur dann aktiv sein, wenn ein oder mehrere Tabelleintraege
 * mit der Checkbox ausgewhlt wurden. ber das KOntext-Men kann jeder EIntrag einzeln geschaltet werden. 
 * 
 * @author Pittner
 * 
 */
public class DialogSemi extends Dialog
{
  
  /**
   * ID
   */
  public static String ID = "de.bsvrz.buv.plugin.anlagendarstellung.dlgs.dialogSemi";
  
	/**
	 * DEF-Breite
	 */
	private static final int DEF_WIDTH = 600; // 900;

	/**
	 * DEF-Hhe
	 */
	private static final int DEF_HEIGHT = 350; // 250;

	/**
	 * Shell
	 */
	private Shell _shell;

	/**
	 * Instanz des GUI-Managers
	 */
	private DialogSemiGuiManager _dialogSchaltenGuiManager = null;

	/**
	 * Map zur Verwaltung der Anfragen
	 */
	private Map<String, MassnahmenAnfrageGuiData> _map = null;
	
	/**Liste der zu sendenen Antworten
	 * 
	 */
	private List<MassnahmenAnfrageGuiData> _listSend = null;

	/**
	 * Composites
	 */
	private Composite _compAbschnitt;
	private Composite _compCtrl;
	private Composite _compTable;

	
	/**
	 * Buttons
	 */
	private Button _bSchalten;
	private Button _bNichtSchalten;

	/**
	 * Label
	 */
	private Label _lAbschnitt;

  /**
   * Group
   */
  private Group _gruppeTabelleAnzeigen;

  private CheckboxTableViewer _tvAnzeigen;

  
  /**
   * Actions
   */
  private Action _actionSchalten;
  private Action _actionNichtSchalten;

  /**
   * MenueManager fr das Kontext-Men
   */
  private MenuManager _menuMgr;
  
	/**
	 * Konstruktor, bernimmt die Instanz des GUI-Managers
	 * @param parent
	 * @param dialogSchaltenGuiManager
	 *                   der GUI-Manager
	 */
	public DialogSemi(Shell parent,
			DialogSemiGuiManager dialogSchaltenGuiManager)
	{
		super(parent);
		this._dialogSchaltenGuiManager = dialogSchaltenGuiManager;
	}

	/**
	 * ffnet den Dialog zum Schalten des Abschnitts.
	 * 
	 * @param so
	 *          {@link SystemObject} vom Typ <code>typ.tsfAbschnitt</code> des zu
	 *          schaltenden Abschnitts.
	 * @param nurAbbrechenMoeglich
	 *          Flag das anzeigt, das nur noch Abgebrochen werden kann. In diesem
	 *          Fall stehen die anderen Schaltoptionen nicht mehr zur Verfgung.
	 * @return
	 */
	/**
	 * ffnet den Dialog zum Schalten der UrsachenEinheiten.
	 * @param map
	 *         die Map mit den Anfragen, wobei als Key die Pid der UrsachenEinheit dient. 
	 * @return
	 */
	public Object oeffne(Map<String, MassnahmenAnfrageGuiData> map)
	{
		this._map = map;
		
		this._listSend = new ArrayList<MassnahmenAnfrageGuiData>();

		this._shell = ShellOnTop.getShell(getParent(), SWT.TITLE | SWT.BORDER);

		_shell.addDisposeListener(new DisposeListener()
		{
			@Override
			public void widgetDisposed(DisposeEvent e)
			{
				dispose();
			}
		});

		/*
		 * GUI Aufbauen
		 */
		initGui(_shell);
		
		/*
		 * Kontext-Man mit Actions erzeugen
		 */
    createContextMenu();    
    createActions();

    /*
     * Tabelle aktualisieren
     */
    setTableInput(_map);

		/*
		 * Zentrieren
		 */		 
		Toolkit test = java.awt.Toolkit.getDefaultToolkit();

		/*
		 * Auflsung durch 2 subtrahiert durch die Hlfte der in der Shell
		 * angegebenen Horizontalen, damit der Dialog mittig am Bildschirm erscheint
		 */
		int x = test.getScreenSize().width / 2 - DEF_WIDTH / 2;

		/*
		 * Auflsung durch 2 subtrahiert durch die Hlfte der in der Shell
		 * angegebenen Vertikalen
		 */

		int y = test.getScreenSize().height / 2 - DEF_HEIGHT / 2;

		_shell.setBounds(x, y, DEF_WIDTH, DEF_HEIGHT);

//		/*
//		 * Statische Positionierung wenn mglich
//		 */
//		Point location = StaticDialogLocation.getInstance().getDlgLoc(
//				this.getClass().getName(), null);
//		if (location != null)
//		{
//			_shell.setLocation(location);
//		}


		_shell.open();

		Display display = getParent().getDisplay();
		while (!_shell.isDisposed())
			if (!display.readAndDispatch())
				display.sleep();

		return null;
	}

 

	/**
	 * Initialisiert die Darstellung
	 * @param parent
	 */
	public void initGui(Shell parent)
	{
		parent.setText("Semi-Automatik schalten");		
		getCompositeText(parent);
		getCompositeTable(parent);
		getCompButton(parent);

		/*
		 * Layout
		 */
		parent.setLayout(new GridLayout(1, false));

		GridData gdLAbs = new GridData();
		gdLAbs.horizontalSpan = 1;
		gdLAbs.grabExcessHorizontalSpace = true;
		gdLAbs.horizontalAlignment = GridData.FILL;
		getCompositeText(parent).setLayoutData(gdLAbs);

		GridData gdCompInfoText = new GridData();
		gdCompInfoText.grabExcessHorizontalSpace = true;
		gdCompInfoText.grabExcessVerticalSpace = true;
		gdCompInfoText.horizontalAlignment = GridData.FILL;
		gdCompInfoText.verticalAlignment = GridData.FILL;
		getCompositeTable(parent).setLayoutData(gdCompInfoText);

		GridData gdCompCtrl = new GridData();
		gdCompCtrl.grabExcessHorizontalSpace = true;
		gdCompCtrl.horizontalAlignment = GridData.CENTER;
		getCompButton(parent).setLayoutData(gdCompCtrl);
	}

	/**
	 * Initialisiert die Tabelle mit dem entsprechenden Composite
	 * @param parent
	 * @return
	 */
	private Composite getCompositeTable(Composite parent)
	{
		if (_compTable == null)
		{
			_compTable = new Composite(parent, SWT.BORDER);

			GridData gridData = new GridData();
	    gridData.horizontalAlignment = GridData.FILL;
	    gridData.grabExcessVerticalSpace = true;
	    gridData.grabExcessHorizontalSpace = true;
	    gridData.verticalAlignment = GridData.FILL;
	    _compTable.setLayout(new GridLayout());
	    _compTable.setLayoutData(gridData);
	    GridData gridDataTabelle = new GridData();
	    gridDataTabelle.horizontalAlignment = GridData.FILL;
	    gridDataTabelle.grabExcessHorizontalSpace = true;
	    gridDataTabelle.grabExcessVerticalSpace = true;
	    gridDataTabelle.verticalAlignment = GridData.FILL;

	    GridData gridDataGruppeAnzeigequerschnitte = new GridData();
	    gridDataGruppeAnzeigequerschnitte.horizontalAlignment = GridData.FILL;
	    gridDataGruppeAnzeigequerschnitte.grabExcessHorizontalSpace = true;
	    gridDataGruppeAnzeigequerschnitte.grabExcessVerticalSpace = true;
	    gridDataGruppeAnzeigequerschnitte.verticalAlignment = GridData.FILL;
	    _gruppeTabelleAnzeigen = new Group(_compTable, SWT.NONE);
	    _gruppeTabelleAnzeigen.setLayout(new GridLayout());
	    _gruppeTabelleAnzeigen.setLayoutData(gridDataGruppeAnzeigequerschnitte);
	    _gruppeTabelleAnzeigen.setText("UrsachenEinheiten");
	    
	    
	    Table table = new Table(_gruppeTabelleAnzeigen, SWT.FULL_SELECTION | SWT.CHECK);	    
	    _tvAnzeigen = new CheckboxTableViewer (table);
	  
	    TableColumn column1 = new TableColumn(table, SWT.LEFT);
	    column1.setWidth(200);
	    column1.setText("Name");

	    TableColumn column2 = new TableColumn(table, SWT.LEFT);
	    column2.setWidth(80);
	    column2.setText("Applikation");

	    TableColumn column3 = new TableColumn(table, SWT.LEFT);
	    column3.setWidth(80);
	    column3.setText("Algorithmus");

	    TableColumn column4 = new TableColumn(table, SWT.LEFT);
	    column4.setWidth(90);
	    column4.setText("Alternative");

	    _tvAnzeigen.setContentProvider(new ArrayContentProvider());
	    _tvAnzeigen.setLabelProvider(new LabelProvider());
	    _tvAnzeigen.getTable().setLayoutData(gridDataTabelle);
	    _tvAnzeigen.getTable().setLinesVisible(true);
	    _tvAnzeigen.getTable().setHeaderVisible(true);
	    
	    _tvAnzeigen.addCheckStateListener(new ICheckStateListener() {
        
        @Override
        public void checkStateChanged(CheckStateChangedEvent event)
        {         
          if (event != null){
            Object element = event.getElement();
            if (element instanceof MassnahmenAnfrageGuiData)
            {
              MassnahmenAnfrageGuiData data = (MassnahmenAnfrageGuiData)element;
              if (event.getChecked()){
                if (!_listSend.contains(data))
                  _listSend.add(data);               
              }else{
                if (_listSend.contains(data))
                  _listSend.remove(data);
              }               
            }            
          }
          
          if (_listSend.size() > 0){
            _bSchalten.setEnabled(true);
            _bNichtSchalten.setEnabled(true);
          }
          else{
            _bSchalten.setEnabled(false);
            _bNichtSchalten.setEnabled(false);
          }
        }
      });
		}
		return _compTable;
	}
	
	
  /**
   * Erzeugt das Kontext-Men
   */
  private void createContextMenu()
  {
    _menuMgr = new MenuManager("#PopupMenu");
    _menuMgr.setRemoveAllWhenShown(true);
    _menuMgr.addMenuListener(new IMenuListener() {
      public void menuAboutToShow(IMenuManager manager)
      {
        fillContextMenu(manager);
      }
    });

    Menu menu = _menuMgr.createContextMenu(_tvAnzeigen.getControl());
    _tvAnzeigen.getControl().setMenu(menu);
  }

  private void fillContextMenu(IMenuManager manager)
  {
    manager.add(_actionSchalten);
    manager.add(_actionNichtSchalten);
  }
  
  /**
   * Erzeugt die Actions
   */
  private void createActions()
  {
    if (_actionSchalten == null)
    {
      _actionSchalten = new Action() {
        public void run()
        {
          ISelection selection = _tvAnzeigen.getSelection();
          if (selection instanceof StructuredSelection)
          {
            StructuredSelection ss = (StructuredSelection)selection;
            Object object = ss.getFirstElement();
            if (object instanceof MassnahmenAnfrageGuiData)
            {
              Iterator it = ss.iterator();
              while (it.hasNext())
              {
                MassnahmenAnfrageGuiData data = (MassnahmenAnfrageGuiData)it.next();
                if (!_listSend.contains(data))
                  _listSend.add(data);               
                
              }               
              aktionSchalten(true);                          
            }            
          }          
        }
      };
      _actionSchalten.setId(ID + ".actionSchalten");
      
      String ss = "UrsachenEinheit schalten";
      
      _actionSchalten.setText(ss);
      _actionSchalten.setToolTipText(ss);
    }
    if (_actionNichtSchalten == null)
    {
      _actionNichtSchalten = new Action() {
        public void run()
        {
          aktionSchalten(false);          
        }
      };
      _actionNichtSchalten.setId(ID + ".actionNichtSchalten");
      
      String ss = "UrsachenEinheit nicht schalten";
      
      _actionNichtSchalten.setText(ss);
      _actionNichtSchalten.setToolTipText(ss);
    }
     
    _menuMgr.add(_actionSchalten);
    _menuMgr.add(_actionNichtSchalten);    
  }
	

	/**
	 * Initialisiert den Text mit dem entsprechenden Composite
	 * @param parent
	 * @return
	 */
	private Composite getCompositeText(Composite parent)
	{
		if (_compAbschnitt == null)
		{
			_compAbschnitt = new Composite(parent, SWT.BORDER);

			_lAbschnitt = new Label(_compAbschnitt, SWT.NONE);
			_lAbschnitt.setFont(DefsFonts.FONT_ARIAL_12_PKT_BOLD);

			_compAbschnitt.setLayout(new GridLayout(1, false));

			GridData gdLAbschnitt = new GridData();
			gdLAbschnitt.grabExcessHorizontalSpace = true;
			gdLAbschnitt.horizontalAlignment = GridData.CENTER;
			_lAbschnitt.setLayoutData(gdLAbschnitt);
			_lAbschnitt.setText("Vorschlag UrsachenEinheiten");
		}

		return _compAbschnitt;
	}
 
	/**
	 * Getter, liefert die Map mit den Anfragen
	 * @return
	 */
	public Map<String, MassnahmenAnfrageGuiData> getMap()
	{
	  return _map;
	}

	/**
   * Initialisiert die Buttons mit dem entsprechenden Composite
   * @param parent
   * @return
   */
	public Composite getCompButton(Composite parent)
	{
		if (_compCtrl == null)
		{
			_compCtrl = new Composite(parent, SWT.NONE);

			_bSchalten = new Button(_compCtrl, SWT.PUSH);
			_bSchalten.setText("Schalten");
			_bSchalten.setEnabled(false);
			_bSchalten.addSelectionListener(new SelectionAdapter()
			{
				@Override
				public void widgetSelected(SelectionEvent e)
				{
					aktionSchalten(true);
				}
			});

			_bNichtSchalten = new Button(_compCtrl, SWT.PUSH);
			_bNichtSchalten.setText("Nicht schalten");  
      _bNichtSchalten.setEnabled(false);
			_bNichtSchalten.addSelectionListener(new SelectionAdapter()
			{
				@Override
				public void widgetSelected(SelectionEvent e)
				{
					aktionSchalten(false);
				}
			});

			Button bSchliessen = new Button(_compCtrl, SWT.PUSH);
			bSchliessen.setText("Schlieen");
			bSchliessen.addSelectionListener(new SelectionAdapter()
			{
				@Override
				public void widgetSelected(SelectionEvent e)
				{
					if (_shell != null)
					{
						_shell.dispose();
					}
				}
			});

			/*
			 * Layout
			 */
			_compCtrl.setLayout(new RowLayout());
		}

		return _compCtrl;
	}

	/**
	 * Dispose, wird beim {@link DialogSemiGuiManager} ausgefhrt
	 */
	public void dispose()
	{
		if (_dialogSchaltenGuiManager != null)
		{
		  _dialogSchaltenGuiManager.removeDialogSchalten();
		}
	}

  /**
   * Fhrt das Versenden der Schaltung durch. Die Daten werden erzeugt und dem {@link UrlasserDialog} bergeben.
   * @param schalten
   *          Flag, das anzeigt ob Schalten oder Nicht-Schalten ausgefhrt werden soll.
   */
  private void aktionSchalten(boolean schalten)
  {
    if (_listSend.size() > 0)
    {
      ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
      if (connection != null)
      {
        IPreferenceStore store = Activator.getDefault().getPreferenceStore();
        String pidSba = store.getString(DefsPreference.P_STRECKEN_BEEINFLUSSUNGS_ANLAGE);
        if (pidSba != null && pidSba.length() > 0)
        {
          SystemObject object = connection.getDataModel().getObject(pidSba);
          if (object != null)
          {
            AttributeGroup atg = connection.getDataModel().getAttributeGroup("atg.sswSbaMassnahmenAntwortGUI");
            Data data = connection.createData(atg);

            List<ResultData> listResultData = new ArrayList<ResultData>();

            for (MassnahmenAnfrageGuiData tmp : _listSend)
            {
              data.getItem("BenutzerInfo").getItem("nummer").asUnscaledValue().set(tmp.getNummer());
              data.getItem("BenutzerInfo").getItem("absenderAlgorithmus").asTextValue().setText(
                  tmp.getAbsenderAlgorithmus());
              data.getItem("BenutzerInfo").getItem("absenderApplikation").asReferenceValue().setSystemObject(
                  tmp.getAbsenderApplikation());
              
              String strSenden = "Ja";
              if (!schalten)
                strSenden = "Nein";
              
              data.getItem("MassnahmeVersenden").asTextValue().setText(strSenden);

              ResultData rd = new ResultData(object, new DataDescription(atg, tmp.getAspekt()), new Date().getTime(),
                  data);
              listResultData.add(rd);
            }

            UrlasserDialog urlasserDialog = new UrlasserDialog(PlatformUI.getWorkbench().getDisplay().getActiveShell());
            Boolean schlieenOk = (urlasserDialog.anzeigen(listResultData)).getEingabeOk();
          }
        }
      }
    }
  }

  /**
   * Label-Provider, erwartet als Input Instanzen vom Typ {@link MassnahmenAnfrageGuiData}
   * @author Pittner
   *
   */
  private static class LabelProvider implements ITableLabelProvider
  {
    @Override
    public Image getColumnImage(Object element, int columnIndex)
    {
      return null;
    }

    @Override
    public String getColumnText(Object element, int columnIndex)
    {
      String result = null;
      MassnahmenAnfrageGuiData modelObject = (MassnahmenAnfrageGuiData)element;

      switch (columnIndex)
      {
        case 0:
          String name = modelObject.getUrsachenEinheit().getName();
          result = name != null ? name : "";
          break;
        case 1:
          String applikation = modelObject.getAbsenderApplikation().getName();
          result = applikation != null ? applikation : "";
          break;
        case 2:
          String algorithmus = modelObject.getAbsenderAlgorithmus();
          result = algorithmus != null ? algorithmus : "";
          break;
        case 3:
          String alternative = "" + modelObject.getAlternative();
          
          // Umsetzung in eine verstndliche Ausgabe
          if (alternative.equals("-1")){
            alternative = "keine Vorgabe";
          }
          
          result = alternative != null ? alternative : "";
          break;
        default:
          result = "";
          break;
      }

      return result;
    }

    @Override
    public void addListener(ILabelProviderListener listener)
    {
    }

    @Override
    public void dispose()
    {
    }

    @Override
    public boolean isLabelProperty(Object element, String property)
    {
      return false;
    }

    @Override
    public void removeListener(ILabelProviderListener listener)
    {
    }
  }

  /**
   * Aktualisiert die Tabelle mit der Liste der Anfragen
   * @param map
   */
  public void setTableInput(Map<String, MassnahmenAnfrageGuiData> map)
  {
    if (_tvAnzeigen != null)
    {
      _tvAnzeigen.setInput(map.values());
      _tvAnzeigen.refresh();
    }
  }
}
