/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.util;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Display;

import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeQuerschnittModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.DetektorModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.FahrStreifenModelAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.dav.ImageFactory;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.LegendeDefinitionAnzeigeQuerschnitt;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.LegendeDefinitionAnzeigeQuerschnittParam;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.LegendeDefinitionAnzeigeQuerschnittPr;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.LegendeDefinitionAnzeigeQuerschnittProgramm;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.LegendeDefinitionAnzeigeQuerschnittSoll;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.LegendeDefinitionAnzeigeQuerschnittStatus;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.LegendeDefinitionAnzeigeQuerschnittStatusBefehle;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.LegendeDefinitionAnzeigeStatus;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.LegendeDefinitionAnzeigeStatusBefehle;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.LegendeDefinitionDetektor;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.LegendeDefinitionDetektorParam;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.LegendeDefinitionSbaUfdMessStelleNaesseStufe;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.LegendeDefinitionStreckenStation;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.LegendeDefinitionStreckenStationStatus;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.LegendeDefinitionTabelle;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.LegendeDefinitionUfdSensorSichtWeite;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.LegendeDefinitionVerkehrsLage;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.ILegendeBaustein2;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.LegendeDefinitionVerkehrsLageParam;
import de.bsvrz.buv.rw.bitctrl.eclipse.legende.LegendeBaustein;

/**
 * Helfer-Klasse zur Vearbeitung von Legende-Definitionen
 * 
 * @author Pittner
 * 
 */
public class LegendeHelper
{
  
  /**
   * Erzeugt die Legende anhand der bergebenen LegendeDefinition
   * 
   * @param legendeDefinition
   *          die Definition der Legende
   * @return dieList der Bausteine, hier vom Typ {@link ILegendeBaustein2} mit einer Id
   */
  public static List<LegendeBaustein> erzeugeDetektorLegendeDefinition(DetektorModelTypAbstract model, int kontext)
  {
    List list = new ArrayList();
    if (kontext != 5)
    {
      list.add(LegendeDefinitionDetektor.class);
    }else
      list.add(LegendeDefinitionDetektorParam.class);
    
    
    return list;
  }
  
  /**
   * Erzeugt die Legende anhand der bergebenen LegendeDefinition
   * 
   * @param legendeDefinition
   *          die Definition der Legende
   * @return dieList der Bausteine, hier vom Typ {@link ILegendeBaustein2} mit einer Id
   */
  public static List<LegendeBaustein> erzeugeFsLegendeDefinition(FahrStreifenModelAbstract model, int kontext)
  {
    List list = new ArrayList();
    if (kontext != 5)
    {
      list.add(LegendeDefinitionVerkehrsLage.class);
    }else
      list.add(LegendeDefinitionVerkehrsLageParam.class);
    
    
    return list;
  }
  
  /**
   * Erzeugt die Legende anhanf der bergebenen LegendeDefinition
   * 
   * @param legendeDefinition
   *          die Definition der Legende
   * @return dieList der Bausteine, hier vom Typ {@link ILegendeBaustein2} mit einer Id
   */
  public static List<LegendeBaustein> erzeugeAqLegendeDefinition(AnzeigeQuerschnittModelTypAbstract model,
      boolean isPrisma)
  {
    List list = new ArrayList();
    int kontext = 0;
    kontext = model.getKontext();
    if (!isPrisma)
    {
      if (kontext == 0)
        list.add(LegendeDefinitionAnzeigeQuerschnitt.class);
      else if ((kontext == 1) || (kontext == 3) || (kontext == 4))
      {
        list.add(LegendeDefinitionAnzeigeQuerschnittSoll.class);
      }
      else if ((kontext == 5))
      {
        list.add(LegendeDefinitionAnzeigeQuerschnittParam.class);
      }
      else
      {
//        list.add(LegendeDefinitionAnzeigeQuerschnittSoll.class);
        // System.out.println("else1: " + model.getDoTyp().getName() + " : " + kontext);
      }
    }
    else
    {
      if ((kontext != 5))
      {
        list.add(LegendeDefinitionAnzeigeQuerschnittPr.class);
      }
      else
      {
        list.add(LegendeDefinitionAnzeigeQuerschnittParam.class);
      }
    }
    
    if (kontext == 0)
    {
      list.add(LegendeDefinitionAnzeigeQuerschnittStatus.class);
    }
    else if ((kontext == 1) || (kontext == 3) || (kontext == 4))
    {
      list.add(LegendeDefinitionAnzeigeQuerschnittStatusBefehle.class);
    }
    else if ((kontext == 5))
    {
      list.add(LegendeDefinitionAnzeigeQuerschnittStatusBefehle.class);
    }
    else
    {
      // list.add(LegendeDefinitionAnzeigeQuerschnittStatusBefehle.class);
      // System.out.println("else2: " + model.getDoTyp().getName() + " : " + kontext);
    }
    // Nur, wenns keine Parameterdarstellung ist.
    if (kontext != 5)
      list.add(LegendeDefinitionAnzeigeQuerschnittProgramm.class);
    return list;
  }

  /**
   * Erzeugt die Legende anhanf der bergebenen LegendeDefinition
   * 
   * @param legendeDefinition
   *          die Definition der Legende
   * @return dieList der Bausteine, hier vom Typ {@link ILegendeBaustein2} mit einer Id
   */
  public static List<LegendeBaustein> erzeugeAnzeigeLegendeDefinition(AnzeigeQuerschnittModelTypAbstract aq,
      AnzeigeModelTypAbstract model)
  {

    // TODO Auto-generated method stub
    List list = new ArrayList();
    // if (parent instanceof AnzeigeQuerschnittModelTypAbstract)
    {
      // AnzeigeQuerschnittModelTypAbstract aq = (AnzeigeQuerschnittModelTypAbstract)parent;
      int kontext = 0;
      kontext = aq.getKontext();
      if (kontext == 0)
        list.add(LegendeDefinitionAnzeigeStatus.class);
      else if (kontext == 1)
        list.add(LegendeDefinitionAnzeigeStatusBefehle.class);
      else
      {
        list.add(LegendeDefinitionAnzeigeStatusBefehle.class);
        // System.out.println("else3: " + aq.getDoTyp().getName() + " -> " + model.getDoTyp().getName() + " : " +
        // kontext);
      }
    }
    return list;

  }

  /**
   * Erzeugt die Legende anhanf der bergebenen LegendeDefinition
   * 
   * @param legendeDefinition
   *          die Definition der Legende
   * @return dieList der Bausteine, hier vom Typ {@link ILegendeBaustein2} mit einer Id
   */
  public static List<LegendeBaustein> erzeugeLegendeBausteine(Object legendeDefinition)
  {
    List<LegendeBaustein> re = new ArrayList<LegendeBaustein>();

    if (legendeDefinition.equals(LegendeDefinitionAnzeigeQuerschnitt.class))
    {
      for (LegendeDefinitionAnzeigeQuerschnitt status : LegendeDefinitionAnzeigeQuerschnitt.values())
      {
        final Image image = ImageFactory.getImage(status.getPfad());
        if (image != null)
        {
          re.add(new LegendeBaustein(image, status.getName()));
        }
      }
    }
    else if (legendeDefinition.equals(LegendeDefinitionAnzeigeQuerschnittSoll.class))
    {
      for (LegendeDefinitionAnzeigeQuerschnittSoll status : LegendeDefinitionAnzeigeQuerschnittSoll.values())
      {
        final Image image = ImageFactory.getImage(status.getPfad());
        if (image != null)
        {
          re.add(new LegendeBaustein(image, status.getName()));
        }
      }
    }
    else if (legendeDefinition.equals(LegendeDefinitionAnzeigeQuerschnittPr.class))
    {
      for (LegendeDefinitionAnzeigeQuerschnittPr status : LegendeDefinitionAnzeigeQuerschnittPr.values())
      {
        final Image image = ImageFactory.getImage(status.getPfad());
        if (image != null)
        {
          re.add(new LegendeBaustein(image, status.getName()));
        }
      }
    }
    else if (legendeDefinition.equals(LegendeDefinitionAnzeigeQuerschnittParam.class))
    {
      for (LegendeDefinitionAnzeigeQuerschnittParam status : LegendeDefinitionAnzeigeQuerschnittParam.values())
      {
        final Image image = ImageFactory.getImage(status.getPfad());
        if (image != null)
        {
          re.add(new LegendeBaustein(image, status.getName()));
        }
      }
    }
    else if (legendeDefinition.equals(LegendeDefinitionAnzeigeQuerschnittStatus.class))
    {
      for (LegendeDefinitionAnzeigeQuerschnittStatus status : LegendeDefinitionAnzeigeQuerschnittStatus.values())
      {
        final Image image = createColorImage(status.getColor());
        if (image != null)
        {
          re.add(new LegendeBaustein(image, status.getName()));
        }
      }
    }
    else if (legendeDefinition.equals(LegendeDefinitionAnzeigeQuerschnittStatusBefehle.class))
    {
      for (LegendeDefinitionAnzeigeQuerschnittStatusBefehle status : LegendeDefinitionAnzeigeQuerschnittStatusBefehle
          .values())
      {
        final Image image = createColorImage(status.getColor());
        if (image != null)
        {
          re.add(new LegendeBaustein(image, status.getName()));
        }
      }
    }
    else if (legendeDefinition.equals(LegendeDefinitionAnzeigeQuerschnittProgramm.class))
    {
      for (LegendeDefinitionAnzeigeQuerschnittProgramm status : LegendeDefinitionAnzeigeQuerschnittProgramm.values())
      {
        final Image image = ImageFactory.getImage(status.getPfad());
        if (image != null)
        {
          re.add(new LegendeBaustein(image, status.getName()));
        }
      }
    }
    else if (legendeDefinition.equals(LegendeDefinitionDetektor.class))
    {
      for (LegendeDefinitionDetektor status : LegendeDefinitionDetektor.values())
      {
        final Image image = ImageFactory.getImage(status.getPfad());
        if (image != null)
        {
          re.add(new LegendeBaustein(image, status.getName()));
        }
      }
    }
    else if (legendeDefinition.equals(LegendeDefinitionDetektorParam.class))
    {
      for (LegendeDefinitionDetektorParam status : LegendeDefinitionDetektorParam.values())
      {
        final Image image = ImageFactory.getImage(status.getPfad());
        if (image != null)
        {
          re.add(new LegendeBaustein(image, status.getName()));
        }
      }
    }
    else if (legendeDefinition.equals(LegendeDefinitionVerkehrsLage.class))
    {
      for (LegendeDefinitionVerkehrsLage status : LegendeDefinitionVerkehrsLage.values())
      {
        final Image image = createColorImage(status.getColor());
        if (image != null)
        {
          re.add(new LegendeBaustein(image, status.getName()));
        }
      }
    }
    else if (legendeDefinition.equals(LegendeDefinitionVerkehrsLageParam.class))
    {
      for (LegendeDefinitionVerkehrsLageParam status : LegendeDefinitionVerkehrsLageParam.values())
      {
        final Image image = createColorImage(status.getColor());
        if (image != null)
        {
          re.add(new LegendeBaustein(image, status.getName()));
        }
      }
    }
    else if (legendeDefinition.equals(LegendeDefinitionTabelle.class))
    {
      for (LegendeDefinitionTabelle status : LegendeDefinitionTabelle.values())
      {
        final Image image = ImageFactory.getImage(status.getPfad());
        if (image != null)
        {
          re.add(new LegendeBaustein(image, status.getName()));
        }
      }
    }
    else if (legendeDefinition.equals(LegendeDefinitionAnzeigeStatus.class))
    {
      for (LegendeDefinitionAnzeigeStatus status : LegendeDefinitionAnzeigeStatus.values())
      {
        final Image image = createColorImage(status.getColor());
        if (image != null)
        {
          re.add(new LegendeBaustein(image, status.getName()));
        }
      }
    }
    else if (legendeDefinition.equals(LegendeDefinitionAnzeigeStatusBefehle.class))
    {
      for (LegendeDefinitionAnzeigeStatusBefehle status : LegendeDefinitionAnzeigeStatusBefehle.values())
      {
        final Image image = createColorImage(status.getColor());
        if (image != null)
        {
          re.add(new LegendeBaustein(image, status.getName()));
        }
      }
    }
    else if (legendeDefinition.equals(LegendeDefinitionStreckenStation.class))
    {
      for (LegendeDefinitionStreckenStation status : LegendeDefinitionStreckenStation.values())
      {
        final Image image = ImageFactory.getImage(status.getPfad());
        if (image != null)
        {
          re.add(new LegendeBaustein(image, status.getName()));
        }
      }
    }
    else if (legendeDefinition.equals(LegendeDefinitionStreckenStationStatus.class))
    {
      for (LegendeDefinitionStreckenStationStatus status : LegendeDefinitionStreckenStationStatus.values())
      {
        final Image image = createColorImage(status.getColor());
        if (image != null)
        {
          re.add(new LegendeBaustein(image, status.getName()));
        }
      }
    }
    else if (legendeDefinition.equals(LegendeDefinitionUfdSensorSichtWeite.class))
    {
      for (LegendeDefinitionUfdSensorSichtWeite status : LegendeDefinitionUfdSensorSichtWeite.values())
      {
        final Image image = ImageFactory.getImage(status.getPfad());
        if (image != null)
        {
          re.add(new LegendeBaustein(image, status.getName()));
        }
      }
    }
    else if (legendeDefinition.equals(LegendeDefinitionSbaUfdMessStelleNaesseStufe.class))
    {
      for (LegendeDefinitionSbaUfdMessStelleNaesseStufe status : LegendeDefinitionSbaUfdMessStelleNaesseStufe.values())
      {
        final Image image = ImageFactory.getImage(status.getPfad());
        if (image != null)
        {
          re.add(new LegendeBaustein(image, status.getName()));
        }
      }
    }
    return re;
  }

  /**
   * Erzeugt aus der {@link Color} ein SWT-Image mit fester Groesse
   * 
   * @param color
   * @return das {@link Image}
   */
  private static Image createColorImage(final Color color)
  {
    final Display display = Display.getDefault();
    final Image image = new Image(display, 16, 16);
    final GC gc = new GC(image);
    gc.setBackground(color);
    gc.fillRectangle(image.getBounds());
    gc.dispose();
    return image;
  }

}
