/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.views;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;

import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.Preferences.IPropertyChangeListener;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.jface.viewers.AbstractTreeViewer;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableFontProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.ITreeViewerListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeExpansionEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerFilter;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeColumn;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.internal.WorkbenchPage;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.plugin.anlagendarstellung.Activator;
import de.bsvrz.buv.plugin.anlagendarstellung.dav.DaVKommunikationDobj;
import de.bsvrz.buv.plugin.anlagendarstellung.dav.ImageFactory;
import de.bsvrz.buv.plugin.anlagendarstellung.sonderprogramm.Programm;
import de.bsvrz.buv.plugin.anlagendarstellung.sonderprogramm.ProgrammHierarchie;
import de.bsvrz.buv.plugin.anlagendarstellung.sonderprogramm.Programmart;
import de.bsvrz.buv.plugin.anlagendarstellung.sonderprogramm.SonderProgramm;
import de.bsvrz.buv.plugin.anlagendarstellung.sonderprogramm.Wanderbaustelle;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

/**
 * Sicht zum Auswhlen von Sonderprogrammen.
 * 
 * @author Pittner
 * 
 */
/**
 * @author Pittner
 * 
 */
public class AuswahlSonderProgrammView extends ViewPart //implements PropertyChangeListener
{

  public static String ID = "de.bsvrz.buv.plugin.anlagendarstellung.auswahlSonderProgrammView";

  /**
   * Composites
   */
  private Composite _parent;

  private Composite compositeTop;

  /**
   * Die Instanz der Sonderprogramm-Sicht
   */
  private ZustandsDarstellungSonderProgramm _contentsView;

  /**
   * Maps, zum Speichern der Selektion bzw. der Expansion im TreeTable
   */
  private Map<String, Object> _mapSelection = new HashMap<String, Object>();

  private Map<String, Object[]> _mapExpandiert = new HashMap<String, Object[]>();

  /**
   * Die TreeViewer
   */
  private TreeViewer _tvProgrammTyp;

  /**
   * String-Definitionen
   */
  private final String _alle = "Alle Sonderprogramm-Typen";

  private final String _defaultText = "Suchkriterium eingeben";

  /**
   * Die Combo zum Auswahl der SonderprogrammTypen
   */
  private Combo _comboObjTypFilter;

  private Combo _comboObjTypFilter2;

  private Combo _comboObjTypFilter21;

  /**
   * Die Group
   */
  private Group _grpAuswahl;

  /**
   * Das Textfeld zur Eingabe des Filters
   */
  private Text _textNamen;

  private CTabFolder _folder;

  private Composite compositeTop1;

  private Composite compositeTop2;

  private TreeViewer _tvWanderBaustelle;

  private Text _textNamen2;

  private ClientDavInterface _connection;

  private Composite compositeTop3;

  private TreeViewer _tvProgrammHierarchie;

  private Text _textNamen21;

  /**
   * 
   */
  private HashMap<String, ProgrammHierarchie> _mapHierarchie;

  private SystemObject _systemObject;

  /**
   * Attributgruppe des DAV, die bercksichtigt wird
   */
  private static final String _attributgruppe = "atg.sswSbaSonderprogrammHierarchie";

  private static final String _attributgruppe2 = "atg.sswSbaSonderprogramm";

  /**
   * Aspekt des DAV, der bercksichtigt wird
   */
  private static final String _aspekt = "asp.parameterSoll";

  // private TreeViewer _tvFilter3;

  private static final Image CHECKED = Activator.getImageDescriptor("icons/checkbox_yes.png").createImage();

  private static final Image UNCHECKED = Activator.getImageDescriptor("icons/checkbox_no.png").createImage();

  /**
   * Das Format der nderungszeit
   */
  private final static SimpleDateFormat _sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss");

  private Action _aktionWbsEditieren;

  private Action _aktionPrgKopieren;

  private Action _aktionPrgLoeschen;

  private Action _aktionPrgAendern;
  
  private String secondaryId = null;

  @Override
  public void createPartControl(Composite parent)
  {
    _parent = parent;

    _connection = DaVKommunikationDobj.getInstanz().holeDaVVerbindung();

    // Fr die F1 Taste
    PlatformUI.getWorkbench().getHelpSystem().setHelp(_parent, Activator.PLUGIN_ID + ".auswahlSonderProgramm");

    parent.setLayout(new GridLayout());

    createCompositeTop(parent);
    erzeugeToolbar();

    _folder.setSelection(0);
    
//    IWorkbenchPage page = getSite().getPage();
//    if (page != null)
//    {
//      IViewReference ref = page.findViewReference(ZustandsDarstellungSonderProgramm.ID, "NO");
//
//      if (ref != null)
//      {
//        IViewPart view = ref.getView(true);
//        ZustandsDarstellungSonderProgramm spNo = (ZustandsDarstellungSonderProgramm)view;
//        spNo.addPropertyChangeListener(this);
//      }
//      
//      IViewReference ref2 = page.findViewReference(ZustandsDarstellungSonderProgramm.ID, "SW");
//      
//      if (ref2 != null)
//      {
//        IViewPart view2 = ref2.getView(true);
//        ZustandsDarstellungSonderProgramm spSw = (ZustandsDarstellungSonderProgramm)view2;
//        spSw.addPropertyChangeListener(this);
//      }
//    }
    
  }

  @Override
  public void setFocus()
  {
    // Fr die F1 Taste
    if (_parent != null)
      _parent.setFocus();

  }

  /**
   * Erzeugt das CompositeTop
   * 
   * @param parent
   */
  private void createCompositeTop(Composite parent)
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessVerticalSpace = true;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    compositeTop = new Composite(parent, SWT.NONE);
    compositeTop.setLayout(new GridLayout());
    compositeTop.setLayoutData(gridData);

    _folder = new CTabFolder(compositeTop, SWT.NONE);
    _folder.setLayout(new FillLayout());
    _folder.setLayoutData(gridData);
    _folder.addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        // TODO Auto-generated method stub
        _mapExpandiert.clear();

        int index = _folder.getSelectionIndex();

        switch (index)
        {
          case 1:
            wanderBaustellenInListeAnzeigen(_contentsView);
            break;
          case 2:
            programmHierarchieInListeAnzeigen(_contentsView);
            break;
          default:
        }
      }
    });

    CTabItem tab = new CTabItem(_folder, SWT.NONE);
    tab.setText("Auswahl Programm-Typ");

    compositeTop1 = new Composite(_folder, SWT.NONE);
    compositeTop1.setLayout(new GridLayout());
    compositeTop1.setLayoutData(gridData);

    tab.setControl(compositeTop1);
    createGroupTreeProgrammTyp(compositeTop1);

    CTabItem tab2 = new CTabItem(_folder, SWT.NONE);
    tab2.setText("Auswahl Wanderbaustelle");

    compositeTop2 = new Composite(_folder, SWT.NONE);
    compositeTop2.setLayout(new GridLayout());
    compositeTop2.setLayoutData(gridData);

    tab2.setControl(compositeTop2);
    createGroupTreeWanderBaustelle(compositeTop2);

    CTabItem tab3 = new CTabItem(_folder, SWT.NONE);
    tab3.setText("Auswahl Hierarchie");

    compositeTop3 = new Composite(_folder, SWT.NONE);
    compositeTop3.setLayout(new GridLayout());
    compositeTop3.setLayoutData(gridData);

    tab3.setControl(compositeTop3);
    createGroupTreeProgrammHierarchie(compositeTop3);

  }

  /**
   * Schliesst die View
   */
  protected void aktionAbbrechen()
  {
    PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().hideView(this);
  }

  /**
   * Zeigt die Hilfe zum Dialog an.
   * 
   */
  private void aktionHilfe()
  {
    String contextId = Activator.PLUGIN_ID + ".auswahlSonderProgramm";
    PlatformUI.getWorkbench().getHelpSystem().displayHelp(contextId);
  }

  /**
   * Setzt den Inhalt der Tabelle mit der aktuellen Sonderprogramm-View
   * 
   * @param contents
   *          die View der Sonderprogramme
   */
  public void setContent(ZustandsDarstellungSonderProgramm contents, boolean neueDaten)
  {
    setContent(contents, neueDaten, null);
  }

  /**
   * Setzt den Inhalt der Tabelle mit der aktuellen Sonderprogramm-View
   * 
   * @param contents
   *          die View der Sonderprogramme
   */
  public void setContent(ZustandsDarstellungSonderProgramm contents, boolean neueDaten, String pid)
  {
//    System.out.println("AuswahlSonderProgrammView.setContent(): " + pid + " -> " + neueDaten);
    if (contents != null)
    {
      String secIdNeu = contents.getViewSite().getSecondaryId();
      if (_contentsView != null)
      {
        String secId = _contentsView.getViewSite().getSecondaryId();
        if (secId.equals(secIdNeu))
        {
          if (neueDaten)
          {
            sonderProgrammeInListeAnzeigen(contents);
            expansionWiederHerstellen(secId);
            selektionWiederHerstellen(secId, pid);
            setGroupAuswahlText();
          }
        }
        else
        {
          if (!neueDaten)
          {
            sonderProgrammeInListeAnzeigen(contents);
            expansionWiederHerstellen(secIdNeu);
            selektionWiederHerstellen(secIdNeu, pid);
            setGroupAuswahlText();
          }
        }
      }
      else
      {
        sonderProgrammeInListeAnzeigen(contents);
      }
    }
    else
    {
      this._contentsView = contents;
      _comboObjTypFilter.removeAll();
      _tvProgrammTyp.getTree().removeAll();
      _tvProgrammTyp.setInput(null);
      _comboObjTypFilter2.removeAll();
      _tvWanderBaustelle.getTree().removeAll();
      _tvWanderBaustelle.setInput(null);
      _comboObjTypFilter21.removeAll();
      _tvProgrammHierarchie.getTree().removeAll();
      _tvProgrammHierarchie.setInput(null);
    }
  }

  /**
   * Hilfsmethode, holt die entsprechende ProgrammTyp-Bezeichnung aus den DaV-Daten (atg.sonderProgramm) eines
   * Sonderprogrammes
   * 
   * @param secId
   *          die secondary-Id der View
   * @param result
   *          die DaV-Daten
   * @return
   */
  private Programmart holeProgrammArtVonResultData(String secId, ResultData result)
  {
    List<Programmart> listeProgramme = getListeProgramme();
    for (Programmart programmart : listeProgramme)
    {
      if (result.hasData())
      {
        Data data = result.getData();
        String text = data.getItem("typBezeichnung").asTextValue().getText();
        if (programmart.bezeichnung.equals(text))
        {
          return programmart;
        }
      }
    }

    return null;

  }

  /**
   * Hilfsmethode, holt die entsprechende ProgrammTyp-Bezeichnung aus den DaV-Daten (atg.sonderProgramm) eines
   * Sonderprogrammes
   * 
   * @param secId
   *          die secondary-Id der View
   * @param result
   *          die DaV-Daten
   * @return
   */
  private Wanderbaustelle holeWanderBaustelleVonResultData(String secId, ResultData result)
  {
    List<Wanderbaustelle> listeProgramme = getListeWanderbaustellen2();
    for (Wanderbaustelle programmart : listeProgramme)
    {
      if (result.hasData())
      {
        Data data = result.getData();
        String text = data.getItem("typBezeichnung").asTextValue().getText();
        if (programmart.bezeichnung.equals(text))
        {
          return programmart;
        }
      }
    }

    return null;

  }

  /**
   * Setzt den Namen der Sicht Sonderprogramme der Gruppe Auswahl
   */
  private void setGroupAuswahlText()
  {
    if (_contentsView != null)
    {
      String partName = _contentsView.getPartName();
      if (_grpAuswahl != null && !_grpAuswahl.isDisposed() && partName != null)
      {
        _grpAuswahl.setText("Auswahl " + partName);
      }
    }
  }

  /**
   * Wiederherstellen der Selektion der Tabelle anhand der SecondaryViewId
   * 
   * @param secId
   */
  private void selektionWiederHerstellen(String secId, String pid)
  {
    if (_mapSelection.containsKey(secId))
    {

      Object obj = _mapSelection.get(secId);
      int index = _folder.getSelectionIndex();
      if (pid != null)
      {
        if (_contentsView.getSonderProgrammMap().containsKey(pid))
        {
          ResultData result = _contentsView.getSonderProgrammMap().get(pid);
          if (index == 0)
          {
            Programmart parent = holeProgrammArtVonResultData(secId, result);
            if (parent != null)
              obj = new Programm(parent, result);
          }
          else if (index == 1)
          {
            Wanderbaustelle parent = holeWanderBaustelleVonResultData(secId, result);
            if (parent != null)
              obj = new Programm(parent, result);
          }

        }
      }

      if (index == 0)
      {
        List<Programmart> input = (List<Programmart>)_tvProgrammTyp.getInput();
        if (input != null)
        {
          for (Programmart programmart : input)
          {
            if (obj instanceof Programmart)
            {
              Programmart art = (Programmart)obj;

              if (art.bezeichnung.equals(programmart.bezeichnung))
              {
                _tvProgrammTyp.setSelection(new StructuredSelection(programmart), true);
                _tvProgrammTyp.refresh();
                return;
              }

            }
            else if (obj instanceof Programm)
            {
              Programm prg = (Programm)obj;
              Collection<Programm> programme = programmart.getProgramme();
              for (Programm programm : programme)
              {
                String pid1 = programm.result.getObject().getPid();
                String pid2 = prg.result.getObject().getPid();

                if (pid1.equals(pid2))
                {
                  _tvProgrammTyp.setSelection(new StructuredSelection(programm), true);
                  _tvProgrammTyp.refresh();
                  return;
                }
              }
            }
          }
        }
      }
      else if (index == 1)
      {
        List<Wanderbaustelle> input2 = (List<Wanderbaustelle>)_tvWanderBaustelle.getInput();
        if (input2 != null)
        {
          for (Wanderbaustelle wanderbaustelle : input2)
          {
            if (obj instanceof Wanderbaustelle)
            {
              Wanderbaustelle art = (Wanderbaustelle)obj;

              if (art.bezeichnung.equals(wanderbaustelle.bezeichnung))
              {
                _tvWanderBaustelle.setSelection(new StructuredSelection(wanderbaustelle), true);
                _tvWanderBaustelle.refresh();
                return;
              }

            }
            else if (obj instanceof Programm)
            {
              Programm prg = (Programm)obj;
              Collection<Programm> programme = wanderbaustelle.getProgramme();
              for (Programm programm : programme)
              {
                String pid1 = programm.result.getObject().getPid();
                String pid2 = prg.result.getObject().getPid();

                if (pid1.equals(pid2))
                {
                  _tvWanderBaustelle.setSelection(new StructuredSelection(programm), true);
                  _tvWanderBaustelle.refresh();
                  return;
                }
              }
            }
          }
        }
      }
    }
  }

  /**
   * Wiederherstellen der Expansion der Tabelle anhand der SecondaryViewId
   * 
   * @param secId
   */
  private void expansionWiederHerstellen(String secId)
  {
    if (_mapExpandiert.containsKey(secId))
    {
      Object[] objects = _mapExpandiert.get(secId);

      ArrayList<Object> listNeu = new ArrayList<Object>();
      for (Object object : objects)
      {
        if (object instanceof Programmart)
        {
          Programmart art = (Programmart)object;
          List<Programmart> input = (List<Programmart>)_tvProgrammTyp.getInput();
          if (input != null)
          {
            for (Programmart programmart : input)
            {
              String pid1 = programmart.toString();
              String pid2 = art.toString();

              if (pid1.equals(pid2))
              {
                listNeu.add(programmart);
              }
            }
          }
          if (_tvProgrammTyp != null && !_tvProgrammTyp.getTree().isDisposed())
          {
            _tvProgrammTyp.setExpandedElements(listNeu.toArray());
            _tvProgrammTyp.refresh();
          }
        }
        if (object instanceof Wanderbaustelle)
        {
          Wanderbaustelle art = (Wanderbaustelle)object;
          List<Wanderbaustelle> input = (List<Wanderbaustelle>)_tvWanderBaustelle.getInput();
          if (input != null)
          {
            for (Wanderbaustelle wanderbaustelle : input)
            {
              String pid1 = wanderbaustelle.toString();
              String pid2 = art.toString();

              if (pid1.equals(pid2))
              {
                listNeu.add(wanderbaustelle);
              }
            }
          }
          if (_tvWanderBaustelle != null && !_tvWanderBaustelle.getTree().isDisposed())
          {
            _tvWanderBaustelle.setExpandedElements(listNeu.toArray());
            _tvWanderBaustelle.refresh();
          }
        }
        if (object instanceof ProgrammHierarchie)
        {
          ProgrammHierarchie art = (ProgrammHierarchie)object;
          List<ProgrammHierarchie> input = (List<ProgrammHierarchie>)_tvProgrammHierarchie.getInput();
          if (input != null)
          {
            for (ProgrammHierarchie wanderbaustelle : input)
            {
              String pid1 = wanderbaustelle.toString();
              String pid2 = art.toString();

              if (pid1.equals(pid2))
              {
                listNeu.add(wanderbaustelle);
              }
            }
          }
          if (_tvProgrammHierarchie != null && !_tvProgrammHierarchie.getTree().isDisposed())
          {
            _tvProgrammHierarchie.setExpandedElements(listNeu.toArray());
            _tvProgrammHierarchie.refresh();
          }
        }
      }
    }
  }

  @Override
  public void dispose()
  {
    // TODO Auto-generated method stub
    super.dispose();
    

    IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
    IWorkbenchPage page = dw.getActivePage();
    if (page != null)
    {
      IViewReference[] refs = page.getViewReferences();
      if (refs != null)
      {
        for (IViewReference ref : refs)
        {
          IViewPart view = ref.getView(true);
          if (view instanceof ZustandsDarstellungSonderProgramm)
          {
            ((ZustandsDarstellungSonderProgramm)view).aktualisiereSichtKeineSelektion();
//            ((ZustandsDarstellungSonderProgramm)view).removePropertyChangeListener(this);
          }
        }
      }
    }
  }

  /**
   * Anzeigenn der Sonderprogramme in der Tabelle
   * 
   * @param contents
   *          die Instanz der SonderprogrammView
   */
  private void sonderProgrammeInListeAnzeigen(ZustandsDarstellungSonderProgramm contents)
  {
    this._contentsView = contents;
    if (_tvProgrammTyp != null && !_tvProgrammTyp.getTree().isDisposed())
    {
      List<Programmart> input = getListeProgramme();      
      _tvProgrammTyp.setInput(input);
    }
    fuelleComboProgrammTyp();
    wanderBaustellenInListeAnzeigen(contents);
  }

  /**
   * Anzeigenn der Sonderprogramme in der Tabelle
   * 
   * @param contents
   *          die Instanz der SonderprogrammView
   */
  private void wanderBaustellenInListeAnzeigen(ZustandsDarstellungSonderProgramm contents)
  {
    this._contentsView = contents;
    if (_tvWanderBaustelle != null && !_tvWanderBaustelle.getTree().isDisposed())
    {
      List<Wanderbaustelle> input = getListeWanderbaustellen2();
      _tvWanderBaustelle.setInput(input);
    }
    fuelleComboWanderBaustelle();
    programmHierarchieInListeAnzeigen(contents);
  }

  /**
   * Anzeigenn der Sonderprogramme in der Tabelle
   * 
   * @param contents
   *          die Instanz der SonderprogrammView
   */
  private void programmHierarchieInListeAnzeigen(ZustandsDarstellungSonderProgramm contents)
  {
    this._contentsView = contents;
    if (_tvProgrammHierarchie != null && !_tvProgrammHierarchie.getTree().isDisposed())
    {
      SystemObjectType type = _connection.getDataModel().getType("typ.sswSbaStreckenBeeinflussungsAnlageRichtung");
      if (type != null)
      {
        List<SystemObject> elements = type.getElements();
        for (SystemObject so : elements)
        {
          AttributeGroup atg = _connection.getDataModel().getAttributeGroup(
              "atg.sswSbaStreckenBeeinflussungsAnlageRichtung");
          Data cd = so.getConfigurationData(atg);
          if (cd != null)
          {
            String fr = cd.getItem("Fahrtrichtung").asTextValue().getValueText();
            String secId = contents.getViewSite().getSecondaryId();
            if (secId.equals(fr))
            {
              _systemObject = so;
              List<ProgrammHierarchie> input = getListeProgrammHierarchie(so);
              initMapHierarchie(input);
              _tvProgrammHierarchie.setInput(input);
              break;
            }
          }
        }
      }
    }
    fuelleComboProgrammHierarchie();
  }

  /**
   * Liefert die Liste der Sonderprogramme der aktuellen Instanz der SonderprogrammView
   * 
   * @return
   */
  public List<Programmart> getListeProgramme()
  {
    SortedMap<String, Collection<ResultData>> typenMap = new TreeMap<String, Collection<ResultData>>();

    if (_contentsView != null)
    {
      Map<String, ResultData> sonderProgrammMap = _contentsView.getSonderProgrammMap();

      Collection<ResultData> values = sonderProgrammMap.values();

      for (ResultData resultData : values)
      {
        Data dta = resultData.getData();
        
        if (dta != null)
        {
          String name = dta.getItem("typBezeichnung").asTextValue().getText();
          if (name != null && name.length() > 0)
          {
            Collection<ResultData> list = null;
            if (!typenMap.containsKey(name))
            {
              list = new ArrayList<ResultData>();
            }
            else
            {
              list = typenMap.get(name);
            }
            list.add(resultData);
            typenMap.put(name, list);            
          }
        }
      }
    }

    List<Programmart> input = new ArrayList<Programmart>();
    for (Map.Entry<String, Collection<ResultData>> me : typenMap.entrySet())
    {
      input.add(new Programmart(me.getKey(), me.getValue()));
    }
    return input;
  }

  /**
   * Liefert die Liste der Sonderprogramme der aktuellen Instanz der SonderprogrammView
   * 
   * @return
   */
  public List<Wanderbaustelle> getListeWanderbaustellen()
  {
    if (_contentsView != null)
    {
      Map<String, List<ResultData>> wanderBaustellen = _contentsView.getWanderBaustellen();
      List<Wanderbaustelle> input = new ArrayList<Wanderbaustelle>();
      for (Map.Entry<String, List<ResultData>> me : wanderBaustellen.entrySet())
      {
        input.add(new Wanderbaustelle(me.getKey(), me.getValue()));
      }
      return input;
    }
    return null;
  }

  /**
   * Liefert die Liste der Sonderprogramme der aktuellen Instanz der SonderprogrammView
   * 
   * @return
   */
  public List<Wanderbaustelle> getListeWanderbaustellen2()
  {
    ClientDavInterface connection = DaVKommunikationDobj.getInstanz().holeDaVVerbindung();

    if (connection != null)
    {
      AttributeGroup atg1 = connection.getDataModel().getAttributeGroup("atg.sswSbaWanderbaustelle");
      AttributeGroup atg2 = connection.getDataModel().getAttributeGroup("atg.sswSbaSonderprogramm");
      Aspect asp = connection.getDataModel().getAspect("asp.parameterSoll");

      SystemObjectType type = connection.getDataModel().getType("typ.sswSbaStreckenBeeinflussungsAnlageRichtung");
      List<SystemObject> elements = type.getElements();

      List<Wanderbaustelle> ret = new ArrayList<Wanderbaustelle>();

      for (SystemObject systemObject : elements)
      {
        Data cd = systemObject.getConfigurationData(systemObject.getDataModel().getAttributeGroup(
            "atg.sswSbaStreckenBeeinflussungsAnlageRichtung"));
        String strFr = cd.getItem("Fahrtrichtung").asTextValue().getValueText();

        if (strFr.equals(_contentsView.getViewSite().getSecondaryId()))
        {

          DataDescription dd = new DataDescription(atg1, asp);
          ResultData rd = connection.getData(systemObject, dd, 1000);
          if (rd != null && rd.hasData())
          {
            Data data = rd.getData();
            Array array = data.getItem("Wanderbaustelle").asArray();
            List<Wanderbaustelle> input = new ArrayList<Wanderbaustelle>();
            for (int i = 0; i < array.getLength(); i++)
            {
              String name = array.getItem(i).getItem("Name").asTextValue().getValueText();
              SystemObject[] sps = array.getItem(i).getItem("Sonderprogramme").asReferenceArray()
                  .getSystemObjectArray();

              List<ResultData> list = new ArrayList<ResultData>();
              for (SystemObject sp : sps)
              {
                if (sp.isValid())
                {
                  ResultData rd2 = connection.getData(sp, new DataDescription(atg2, asp), 1000);
                  list.add(rd2);
                }
              }
              ret.add(new Wanderbaustelle(name, list));
            }
          }
        }
      }
      return ret;
    }
    return null;
  }

  /**
   * Liefert die Liste der Sonderprogramme der aktuellen Instanz der SonderprogrammView
   * 
   * @return
   */
  public List<ProgrammHierarchie> getListeProgrammHierarchie(SystemObject systemObject)
  {
    ClientDavInterface connection = DaVKommunikationDobj.getInstanz().holeDaVVerbindung();

    if (connection != null)
    {
      AttributeGroup atg1 = connection.getDataModel().getAttributeGroup(_attributgruppe);
      Aspect asp = connection.getDataModel().getAspect(_aspekt);

      DataDescription dd = new DataDescription(atg1, asp);
      ResultData rd = connection.getData(systemObject, dd, 1000);
      if (rd != null && rd.hasData())
      {
        Data data = rd.getData();
        Data dataNeu = data.createModifiableCopy();
        List<ProgrammHierarchie> ret = new ArrayList<ProgrammHierarchie>();
        bestimmeProgrammHierarchie(connection, null, dataNeu, ret);
        return ret;
      }
    }
    return null;
  }

  /**
   * @param connection
   * @param parent
   * @param data
   * @param ret
   */
  void bestimmeProgrammHierarchie(ClientDavInterface connection, ProgrammHierarchie parent, Data data,
      List<ProgrammHierarchie> ret)
  {

    AttributeGroup atg = connection.getDataModel().getAttributeGroup(_attributgruppe2);
    Aspect asp = connection.getDataModel().getAspect(_aspekt);

    Array array = data.getItem("Men").asArray();
    for (int i = 0; i < array.getLength(); i++)
    {
      String name = array.getItem(i).getItem("Name").asTextValue().getValueText();
      String[] menues = array.getItem(i).getItem("UnterMen").asTextArray().getTextArray();
      SystemObject[] sps = array.getItem(i).getItem("Sonderprogramme").asReferenceArray().getSystemObjectArray();

      List<ResultData> list = new ArrayList<ResultData>();
      for (SystemObject sp : sps)
      {
        if (sp.isValid())
        {
          ResultData rd2 = connection.getData(sp, new DataDescription(atg, asp), 1000);
          list.add(rd2);
        }
      }
      String parentString = getParentString(connection, data, name);
      ProgrammHierarchie programmHierarchie = new ProgrammHierarchie(parentString, name, menues, list);
      ret.add(programmHierarchie);
    }
  }

  /**
   * @param connection
   * @param data
   * @param name
   * @return
   */
  private String getParentString(ClientDavInterface connection, Data data, String name)
  {

    AttributeGroup atg = connection.getDataModel().getAttributeGroup("atg.sswSbaSonderprogramm");
    Aspect asp = connection.getDataModel().getAspect("asp.parameterSoll");

    Array array = data.getItem("Men").asArray();
    for (int i = 0; i < array.getLength(); i++)
    {
      String[] parentMenues = array.getItem(i).getItem("UnterMen").asTextArray().getTextArray();
      String[] menues = array.getItem(i).getItem("UnterMen").asTextArray().getTextArray();
      SystemObject[] sps = array.getItem(i).getItem("Sonderprogramme").asReferenceArray().getSystemObjectArray();

      List<ResultData> list = new ArrayList<ResultData>();
      for (SystemObject sp : sps)
      {
        if (sp.isValid())
        {
          ResultData rd2 = connection.getData(sp, new DataDescription(atg, asp), 1000);
          list.add(rd2);
        }
      }
      for (String string : parentMenues)
      {
        if (string.equals(name))
        {
          return string;
        }
      }
    }
    return null;
  }

  /**
   * Erzeugt die Gruppe Auswahl mit dem TreeViewer
   * 
   * @param parent
   */
  private void createGroupTreeProgrammTyp(Composite parent)
  {
    GridData gd = new GridData();
    gd.horizontalAlignment = GridData.FILL;
    gd.grabExcessHorizontalSpace = true;
    gd.verticalAlignment = GridData.FILL;
    gd.grabExcessVerticalSpace = true;

    erzeugeComboProgrammTyp(parent);
    erzeugeFilterProgrammTyp(parent);

    Tree filterTree = new Tree(parent, SWT.FULL_SELECTION | SWT.BORDER);
    filterTree.setHeaderVisible(true);
    filterTree.setLinesVisible(true);
    filterTree.setLayoutData(gd);
    _tvProgrammTyp = new TreeViewer(filterTree);

    TreeColumn column0 = new TreeColumn(filterTree, SWT.LEFT);
    column0.setText("Status");
    column0.setWidth(80);

    TreeColumn column1 = new TreeColumn(filterTree, SWT.LEFT);
    column1.setText("Programm-Typ/-Name");
    column1.setWidth(230);

    TreeColumn column2 = new TreeColumn(filterTree, SWT.LEFT);
    column2.setText("nderungszeitpunkt");
    column2.setWidth(130);

    TreeColumn column3 = new TreeColumn(filterTree, SWT.LEFT);
    column3.setText("Stand");
    column3.setWidth(100);

    TreeColumn column4 = new TreeColumn(filterTree, SWT.LEFT);
    column4.setText("Simulation");
    column4.setWidth(100);

    TreeColumn column5 = new TreeColumn(filterTree, SWT.LEFT);
    column5.setText("Benutzer");
    column5.setWidth(100);

    TreeColumn column6 = new TreeColumn(filterTree, SWT.LEFT);
    column6.setText("Ursache");
    column6.setWidth(100);

    TreeColumn column7 = new TreeColumn(filterTree, SWT.LEFT);
    column7.setText("Veranlasser");
    column7.setWidth(100);

    _tvProgrammTyp.setContentProvider(new ProgrammTypContentProvider());
    _tvProgrammTyp.setLabelProvider(new ProgrammTypLabelProvider());

    _tvProgrammTyp.addSelectionChangedListener(new ISelectionChangedListener() {
      public void selectionChanged(SelectionChangedEvent event)
      {
        aktionSelektion(event);
      }
    });

    _tvProgrammTyp.addTreeListener(new ITreeViewerListener() {

      @Override
      public void treeExpanded(TreeExpansionEvent event)
      {
        Object element = event.getElement();
        if (element instanceof Programmart)
        {
          sichereExpandierteObjekte(element, true);
        }

      }

      @Override
      public void treeCollapsed(TreeExpansionEvent event)
      {
        Object element = event.getElement();
        if (element instanceof Programmart)
        {
          sichereExpandierteObjekte(element, false);
        }
      }
    });

    TreeColumn[] columns = filterTree.getColumns();
    for (final TreeColumn tc : columns)
    {
      tc.addSelectionListener(new SelectionAdapter() {
        @Override
        public void widgetSelected(SelectionEvent e)
        {
          Object source = e.getSource();
          if (source instanceof TreeColumn)
          {
            TreeColumn column = (TreeColumn)tc;

            String text = column.getText();
            if (column.getData() != null)
            {
              Boolean b = (Boolean)column.getData();
              if (b)
                aktionSortierung(SWT.UP, text);
              else
                aktionSortierung(SWT.DOWN, text);

              column.setData(!(Boolean)column.getData());
            }
            else
            {
              column.setData(true);
            }
          }
        }
      });
    }
    
    final MenuManager menuMgr = new MenuManager();
    Menu menu = menuMgr.createContextMenu(_tvProgrammTyp.getControl());
    _tvProgrammTyp.getControl().setMenu(menu);
    getSite().registerContextMenu(menuMgr, _tvProgrammTyp);
    
    menuMgr.addMenuListener(new IMenuListener() {
      @Override
      public void menuAboutToShow(IMenuManager manager)
      {        
        if (_tvProgrammTyp.getSelection().isEmpty())
        {
          return;
        }

        if (_tvProgrammTyp.getSelection() instanceof IStructuredSelection)
        {
          IStructuredSelection ss = (IStructuredSelection)_tvProgrammTyp.getSelection();
          Object firstElement = ss.getFirstElement();
          if (firstElement instanceof Programm)
          {
            menuMgr.add(_aktionPrgKopieren);
            menuMgr.add(_aktionPrgAendern);            
            menuMgr.add(_aktionPrgLoeschen);
          }
        }
      }
    });    
    menuMgr.setRemoveAllWhenShown(true);   
    _aktionPrgKopieren = new Action("Sonderprogramm kopieren") {
      public void run()
      {
        _contentsView.aktionSonderProgrammKopieren(); 
      }
    };
    _aktionPrgLoeschen = new Action("Sonderprogramm lschen") {
      public void run()
      {
        _contentsView.aktionLoeschen();
      }
    };
    _aktionPrgAendern = new Action("Programm-Typ ndern") {
      public void run()
      {
        _contentsView.aktionProgrammTypAendern(); 
      }
    };
  }

  /**
   * Erzeugt die Gruppe Auswahl mit dem TreeViewer
   * 
   * @param parent
   */
  private void createGroupTreeWanderBaustelle(Composite parent)
  {
    GridData gd = new GridData();
    gd.horizontalAlignment = GridData.FILL;
    gd.grabExcessHorizontalSpace = true;
    gd.verticalAlignment = GridData.FILL;
    gd.grabExcessVerticalSpace = true;

    erzeugeComboWanderBaustelle(parent);
    erzeugeFilterWanderBaustelle(parent);

    Tree filterTree = new Tree(parent, SWT.FULL_SELECTION | SWT.BORDER);
    filterTree.setHeaderVisible(true);
    filterTree.setLinesVisible(true);
    filterTree.setLayoutData(gd);
    _tvWanderBaustelle = new TreeViewer(filterTree);

    TreeColumn column0 = new TreeColumn(filterTree, SWT.LEFT);
    column0.setText("Status");
    column0.setWidth(80);

    TreeColumn column1 = new TreeColumn(filterTree, SWT.LEFT);
    column1.setText("Wanderbaustelle");
    column1.setWidth(230);

    TreeColumn column00 = new TreeColumn(filterTree, SWT.LEFT);
    column00.setText("Stand");
    column00.setWidth(80);

    TreeColumn column01 = new TreeColumn(filterTree, SWT.LEFT);
    column01.setText("Simulation");
    column01.setWidth(80);

    _tvWanderBaustelle.setContentProvider(new WanderBaustelleContentProvider());
    _tvWanderBaustelle.setLabelProvider(new WanderBaustelleLabelProvider());
    _tvWanderBaustelle.addSelectionChangedListener(new ISelectionChangedListener() {
      public void selectionChanged(SelectionChangedEvent event)
      {
        aktionSelektion(event);
      }
    });

    _tvWanderBaustelle.addTreeListener(new ITreeViewerListener() {

      @Override
      public void treeExpanded(TreeExpansionEvent event)
      {
        Object element = event.getElement();
        if (element instanceof Wanderbaustelle)
        {
          sichereExpandierteObjekte(element, true);
        }

      }

      @Override
      public void treeCollapsed(TreeExpansionEvent event)
      {
        Object element = event.getElement();
        if (element instanceof Wanderbaustelle)
        {
          sichereExpandierteObjekte(element, false);
        }
      }

    });

    TreeColumn[] columns = filterTree.getColumns();
    for (final TreeColumn tc : columns)
    {
      tc.addSelectionListener(new SelectionAdapter() {
        @Override
        public void widgetSelected(SelectionEvent e)
        {
          Object source = e.getSource();
          if (source instanceof TreeColumn)
          {
            TreeColumn column = (TreeColumn)tc;

            String text = column.getText();
            if (column.getData() != null)
            {
              Boolean b = (Boolean)column.getData();
              if (b)
                aktionSortierung(SWT.UP, text);
              else
                aktionSortierung(SWT.DOWN, text);

              column.setData(!(Boolean)column.getData());
            }
            else
            {
              column.setData(true);
            }
          }
        }
      });
    }

    final MenuManager menuMgr = new MenuManager();
    Menu menu = menuMgr.createContextMenu(_tvWanderBaustelle.getControl());
    _tvWanderBaustelle.getControl().setMenu(menu);
    getSite().registerContextMenu(menuMgr, _tvWanderBaustelle);
    
    menuMgr.addMenuListener(new IMenuListener() {
      @Override
      public void menuAboutToShow(IMenuManager manager)
      {        
        if (_tvWanderBaustelle.getSelection().isEmpty())
        {
          return;
        }

        if (_tvWanderBaustelle.getSelection() instanceof IStructuredSelection)
        {
          IStructuredSelection ss = (IStructuredSelection)_tvWanderBaustelle.getSelection();
          Object firstElement = ss.getFirstElement();
          if (firstElement instanceof Wanderbaustelle)
          {
            menuMgr.add(_aktionWbsEditieren);
          }
        }
      }
    });    
    menuMgr.setRemoveAllWhenShown(true);   
    _aktionWbsEditieren = new Action("Wanderbaustelle editieren") {
      public void run()
      {
        ISelection selection = _tvWanderBaustelle.getSelection();
        if (selection instanceof StructuredSelection)
        {
          StructuredSelection ss = (StructuredSelection)selection;
          Object firstElement = ss.getFirstElement();
          if (firstElement instanceof Wanderbaustelle)
          {
            Wanderbaustelle wb = (Wanderbaustelle)firstElement;
            aktionWanderbaustelleEditieren(wb);    
          }
        } 
      }
    };
  }

  /**
   * Erzeugt die Gruppe Auswahl mit dem TreeViewer
   * 
   * @param parent
   */
  private void createGroupTreeProgrammHierarchie(Composite parent)
  {
    GridData gd = new GridData();
    gd.horizontalAlignment = GridData.FILL;
    gd.grabExcessHorizontalSpace = true;
    gd.verticalAlignment = GridData.FILL;
    gd.grabExcessVerticalSpace = true;

    erzeugeComboProgrammHierarchie(parent);
    erzeugeFilterProgrammHierarchie(parent);

    Tree filterTree = new Tree(parent, SWT.FULL_SELECTION | SWT.BORDER);
    filterTree.setHeaderVisible(true);
    filterTree.setLinesVisible(true);
    filterTree.setLayoutData(gd);
    _tvProgrammHierarchie = new TreeViewer(filterTree);

    TreeColumn column0 = new TreeColumn(filterTree, SWT.LEFT);
    column0.setText("Status");
    column0.setWidth(80);

    TreeColumn column1 = new TreeColumn(filterTree, SWT.LEFT);
    column1.setText("ProgrammHierarchie");
    column1.setWidth(230);

    TreeColumn column2 = new TreeColumn(filterTree, SWT.LEFT);
    column2.setText("nderungszeitpunkt");
    column2.setWidth(130);

    TreeColumn column3 = new TreeColumn(filterTree, SWT.LEFT);
    column3.setText("Stand");
    column3.setWidth(100);

    TreeColumn column4 = new TreeColumn(filterTree, SWT.LEFT);
    column4.setText("Simulation");
    column4.setWidth(100);

    TreeColumn column5 = new TreeColumn(filterTree, SWT.LEFT);
    column5.setText("Benutzer");
    column5.setWidth(100);

    TreeColumn column6 = new TreeColumn(filterTree, SWT.LEFT);
    column6.setText("Ursache");
    column6.setWidth(100);

    TreeColumn column7 = new TreeColumn(filterTree, SWT.LEFT);
    column7.setText("Veranlasser");
    column7.setWidth(100);

    _tvProgrammHierarchie.setContentProvider(new ProgrammHierarchieContentProvider());
    _tvProgrammHierarchie.setLabelProvider(new ProgrammHierarchieLabelProvider());
    _tvProgrammHierarchie.addSelectionChangedListener(new ISelectionChangedListener() {
      public void selectionChanged(SelectionChangedEvent event)
      {
        aktionSelektion(event);
      }
    });

    _tvProgrammHierarchie.addTreeListener(new ITreeViewerListener() {

      @Override
      public void treeExpanded(TreeExpansionEvent event)
      {
        Object element = event.getElement();
        if (element instanceof ProgrammHierarchie)
        {
          sichereExpandierteObjekte(element, true);
        }

      }

      @Override
      public void treeCollapsed(TreeExpansionEvent event)
      {
        Object element = event.getElement();
        if (element instanceof ProgrammHierarchie)
        {
          sichereExpandierteObjekte(element, false);
        }
      }

    });

    TreeColumn[] columns = filterTree.getColumns();
    for (final TreeColumn tc : columns)
    {
      tc.addSelectionListener(new SelectionAdapter() {
        @Override
        public void widgetSelected(SelectionEvent e)
        {
          Object source = e.getSource();
          if (source instanceof TreeColumn)
          {
            TreeColumn column = (TreeColumn)tc;

            String text = column.getText();
            if (column.getData() != null)
            {
              Boolean b = (Boolean)column.getData();
              if (b)
                aktionSortierung(SWT.UP, text);
              else
                aktionSortierung(SWT.DOWN, text);

              column.setData(!(Boolean)column.getData());
            }
            else
            {
              column.setData(true);
            }
          }
        }
      });
    }

    _tvProgrammHierarchie.getTree().addMouseListener(new MouseAdapter() {

      @Override
      public void mouseUp(MouseEvent e)
      {
        // TODO Auto-generated method stub
      }

      @Override
      public void mouseDown(MouseEvent e)
      {
        if (e.button == 3)
        {
          MenuManager menuMgr = new MenuManager();

          ISelection selection = _tvProgrammHierarchie.getSelection();
          if (selection instanceof StructuredSelection)
          {
            StructuredSelection ss = (StructuredSelection)selection;
            Object firstElement = ss.getFirstElement();
            if (firstElement instanceof Wanderbaustelle)
            {

              final Wanderbaustelle wbs = (Wanderbaustelle)firstElement;

              Menu menu = menuMgr.createContextMenu(_tvProgrammHierarchie.getControl());
              _tvProgrammHierarchie.getControl().setMenu(menu);
              getSite().registerContextMenu(menuMgr, _tvProgrammHierarchie);

              menuMgr.add(new Action("ProgrammHierarchie editieren") {
                public void run()
                {

                  IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
                  IWorkbenchPage page = dw.getActivePage();
                  try
                  {
                    page.showView(AuswahlWanderBaustelleView.ID, wbs.bezeichnung, IWorkbenchPage.VIEW_ACTIVATE);

                    IViewReference ref = page.findViewReference(AuswahlWanderBaustelleView.ID, wbs.bezeichnung);
                    ((WorkbenchPage)page).getActivePerspective().getPresentation().detachPart(ref);
                    IWorkbenchPart viewPart = ref.getPart(false);
                    ((AuswahlWanderBaustelleView)viewPart).setContentsView(_contentsView);

                    Shell shell = viewPart.getSite().getShell();

                    Display display = PlatformUI.getWorkbench().getDisplay();
                    Point p = display.getCursorLocation();

                    shell.setLocation(p.x - 10, p.y - 10);
                    shell.setSize(520, 315);
                  }
                  catch (PartInitException e)
                  {
                    e.printStackTrace();
                  }
                };
              });
            }
          }
        }
      }

      @Override
      public void mouseDoubleClick(MouseEvent e)
      {
        // TODO Auto-generated method stub
      }
    });
  }

  /**
   * Erzeugt die Combo zur Auswahl der ProgrammTypen
   * 
   * @param parent
   */
  private void erzeugeComboProgrammTyp(Composite parent)
  {
    GridData gdTableItem = new GridData();
    gdTableItem.horizontalAlignment = GridData.FILL;
    gdTableItem.grabExcessHorizontalSpace = true;
    gdTableItem.verticalAlignment = GridData.CENTER;

    _comboObjTypFilter = new Combo(parent, SWT.BORDER);
    _comboObjTypFilter.setLayoutData(gdTableItem);

    _comboObjTypFilter.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        bestimmeGefilterteProgrammTypen();
      }
    });
  }

  /**
   * Erzeugt die Combo zur Auswahl der WanderBaustellen
   * 
   * @param parent
   */
  private void erzeugeComboWanderBaustelle(Composite parent)
  {
    GridData gdTableItem = new GridData();
    gdTableItem.horizontalAlignment = GridData.FILL;
    gdTableItem.grabExcessHorizontalSpace = true;
    gdTableItem.verticalAlignment = GridData.CENTER;

    _comboObjTypFilter2 = new Combo(parent, SWT.BORDER);
    _comboObjTypFilter2.setLayoutData(gdTableItem);

    _comboObjTypFilter2.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        bestimmeGefilterteWanderBaustelle();
      }
    });
  }

  /**
   * Erzeugt die Combo zur Auswahl der ProgrammHierarchien
   * 
   * @param parent
   */
  private void erzeugeComboProgrammHierarchie(Composite parent)
  {
    GridData gdTableItem = new GridData();
    gdTableItem.horizontalAlignment = GridData.FILL;
    gdTableItem.grabExcessHorizontalSpace = true;
    gdTableItem.verticalAlignment = GridData.CENTER;

    _comboObjTypFilter21 = new Combo(parent, SWT.BORDER);
    _comboObjTypFilter21.setLayoutData(gdTableItem);

    _comboObjTypFilter21.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        bestimmeGefilterteProgrammHierarchie();
      }
    });
  }

  /**
   * Erzeugt das Textfeld zur Filtern der ProgrammTypen
   * 
   * @param parent
   */
  private void erzeugeFilterProgrammTyp(Composite parent)
  {
    GridData gdTableItem = new GridData();
    gdTableItem.horizontalAlignment = GridData.FILL;
    gdTableItem.grabExcessHorizontalSpace = true;
    gdTableItem.verticalAlignment = GridData.CENTER;

    _textNamen = new Text(parent, SWT.BORDER);
    _textNamen.setLayoutData(gdTableItem);
    _textNamen.setText(_defaultText);
    _textNamen.addFocusListener(new FocusListener() {
      @Override
      public void focusLost(FocusEvent e)
      {
        if (_textNamen.getText().isEmpty())
        {
          _textNamen.setText(_defaultText);
        }
      }

      @Override
      public void focusGained(FocusEvent e)
      {
        if (_textNamen.getText().equals(_defaultText))
        {
          _textNamen.setText("");
        }

      }
    });

    _textNamen.addModifyListener(new ModifyListener() {
      @Override
      public void modifyText(ModifyEvent e)
      {
        String text = _textNamen.getText();
        if (text.length() > 0 && !text.equals(_defaultText))
          aktionFilterProgrammTyp(text);
        else
        {
          bestimmeGefilterteProgrammTypen();
          aktionFilterProgrammTyp(".*");
        }
      }
    });
  }

  /**
   * Erzeugt das Textfeld zur Filtern der Wanderbaustellen
   * 
   * @param parent
   */
  private void erzeugeFilterWanderBaustelle(Composite parent)
  {
    GridData gdTableItem = new GridData();
    gdTableItem.horizontalAlignment = GridData.FILL;
    gdTableItem.grabExcessHorizontalSpace = true;
    gdTableItem.verticalAlignment = GridData.CENTER;

    _textNamen2 = new Text(parent, SWT.BORDER);
    _textNamen2.setLayoutData(gdTableItem);
    _textNamen2.setText(_defaultText);
    _textNamen2.addFocusListener(new FocusListener() {
      @Override
      public void focusLost(FocusEvent e)
      {
        if (_textNamen2.getText().isEmpty())
        {
          _textNamen2.setText(_defaultText);
        }
      }

      @Override
      public void focusGained(FocusEvent e)
      {
        if (_textNamen2.getText().equals(_defaultText))
        {
          _textNamen2.setText("");
        }

      }
    });

    _textNamen2.addModifyListener(new ModifyListener() {
      @Override
      public void modifyText(ModifyEvent e)
      {
        String text = _textNamen2.getText();
        if (text.length() > 0 && !text.equals(_defaultText))
          aktionFilterWanderBaustelle(text);
        else
        {
          bestimmeGefilterteWanderBaustelle();
          aktionFilterWanderBaustelle(".*");
        }
      }
    });
  }

  /**
   * Erzeugt das Textfeld zur Filtern der ProgrammHierarchien
   * 
   * @param parent
   */
  private void erzeugeFilterProgrammHierarchie(Composite parent)
  {
    GridData gdTableItem = new GridData();
    gdTableItem.horizontalAlignment = GridData.FILL;
    gdTableItem.grabExcessHorizontalSpace = true;
    gdTableItem.verticalAlignment = GridData.CENTER;

    _textNamen21 = new Text(parent, SWT.BORDER);
    _textNamen21.setLayoutData(gdTableItem);
    _textNamen21.setText(_defaultText);
    _textNamen21.addFocusListener(new FocusListener() {
      @Override
      public void focusLost(FocusEvent e)
      {
        if (_textNamen21.getText().isEmpty())
        {
          _textNamen21.setText(_defaultText);
        }
      }

      @Override
      public void focusGained(FocusEvent e)
      {
        if (_textNamen21.getText().equals(_defaultText))
        {
          _textNamen21.setText("");
        }

      }
    });

    _textNamen21.addModifyListener(new ModifyListener() {
      @Override
      public void modifyText(ModifyEvent e)
      {
        String text = _textNamen21.getText();
        if (text.length() > 0 && !text.equals(_defaultText))
          aktionFilterProgrammHierarchie(text);
        else
        {
          bestimmeGefilterteProgrammHierarchie();
          aktionFilterProgrammHierarchie(".*");
        }
      }
    });
  }

  /**
   * Setzt den Filter des Baums der ProgrammTypen auf den angegebenen Ausdruck - wobei aus Performance-Grnden die
   * contains-Methode mit verwendet wird
   * 
   * @param ausdruck
   *          Ausdruck
   */
  private void aktionFilterProgrammTyp(final String ausdruck)
  {
    // ViewFilter setzen
    final ViewerFilter textFilter = new ViewerFilter() {

      @Override
      public boolean select(Viewer viewer, Object parentElement, Object element)
      {
        // String als Matcher fr den Vergleich setzen und Ergebnis abfragen
        if (element instanceof Programm)
        {
          Programm prg = (Programm)element;
          if (prg.toString().contains(ausdruck))
            return true;
          else if (prg.toString().matches(ausdruck))
            return true;

        }
        else if (element instanceof Programmart)
        {
          Collection<Programm> programme = ((Programmart)element).getProgramme();
          for (Programm programm : programme)
          {
            if (select(viewer, element, programm))
              return true;
          }
        }
        return false;
      }
    };

    _tvProgrammTyp.getTree().setRedraw(false);
    // alten Filter lschen
    if (_tvProgrammTyp.getFilters().length > 0)
      _tvProgrammTyp.removeFilter(_tvProgrammTyp.getFilters()[0]);

    // neuen Filter einfgen und Ebenen entsprechend expandieren
    if (ausdruck.length() > 0)
    {
      _tvProgrammTyp.addFilter(textFilter);
      List<Programmart> input = (List<Programmart>)_tvProgrammTyp.getInput();
      if (input != null)
      {
        for (Programmart programmart : input)
        {
          if (_mapExpandiert.containsKey(getViewSite().getSecondaryId()))
          {
            Object[] objects = _mapExpandiert.get(getViewSite().getSecondaryId());
            for (Object object : objects)
            {
              if (object instanceof Programmart)
              {
                Programmart art = (Programmart)object;
                if (programmart.toString().equals(art.toString()))
                {
                  _tvProgrammTyp.expandToLevel(programmart, 1);
                }
              }
            }
          }
        }
      }
    }
    _tvProgrammTyp.getTree().setRedraw(true);
  }

  /**
   * Setzt den Filter des Baums der WanderBaustellen auf den angegebenen Ausdruck - wobei aus Performance-Grnden die
   * contains-Methode mit verwendet wird
   * 
   * @param ausdruck
   *          Ausdruck
   */
  private void aktionFilterWanderBaustelle(final String ausdruck)
  {
    // ViewFilter setzen
    final ViewerFilter textFilter = new ViewerFilter() {

      @Override
      public boolean select(Viewer viewer, Object parentElement, Object element)
      {
        // String als Matcher fr den Vergleich setzen und Ergebnis abfragen
        if (element instanceof Programm)
        {
          Programm prg = (Programm)element;
          if (prg.toString().contains(ausdruck))
            return true;
          else if (prg.toString().matches(ausdruck))
            return true;

        }
        else if (element instanceof Wanderbaustelle)
        {
          Collection<Programm> programme = ((Wanderbaustelle)element).getProgramme();
          for (Programm programm : programme)
          {
            if (select(viewer, element, programm))
              return true;
          }
        }
        return false;
      }
    };

    _tvWanderBaustelle.getTree().setRedraw(false);
    // alten Filter lschen
    if (_tvWanderBaustelle.getFilters().length > 0)
      _tvWanderBaustelle.removeFilter(_tvWanderBaustelle.getFilters()[0]);

    // neuen Filter einfgen und Ebenen entsprechend expandieren
    if (ausdruck.length() > 0)
    {
      _tvWanderBaustelle.addFilter(textFilter);
      List<Wanderbaustelle> input = (List<Wanderbaustelle>)_tvWanderBaustelle.getInput();
      if (input != null)
      {
        for (Wanderbaustelle programmart : input)
        {
          if (_mapExpandiert.containsKey(getViewSite().getSecondaryId()))
          {
            Object[] objects = _mapExpandiert.get(getViewSite().getSecondaryId());
            for (Object object : objects)
            {
              if (object instanceof Programmart)
              {
                Programmart art = (Programmart)object;
                if (programmart.toString().equals(art.toString()))
                {
                  _tvWanderBaustelle.expandToLevel(programmart, 1);
                }
              }
            }
          }
        }
      }
    }
    _tvWanderBaustelle.getTree().setRedraw(true);
  }

  /**
   * Setzt den Filter des Baums der ProgrammHierarchien auf den angegebenen Ausdruck - wobei aus Performance-Grnden die
   * contains-Methode mit verwendet wird
   * 
   * @param ausdruck
   *          Ausdruck
   */
  private void aktionFilterProgrammHierarchie(final String ausdruck)
  {
    // ViewFilter setzen
    final ViewerFilter textFilter = new ViewerFilter() {

      @Override
      public boolean select(Viewer viewer, Object parentElement, Object element)
      {
        // String als Matcher fr den Vergleich setzen und Ergebnis abfragen
        if (element instanceof Programm)
        {
          Programm prg = (Programm)element;
          if (prg.toString().contains(ausdruck))
            return true;
          else if (prg.toString().matches(ausdruck))
            return true;

        }
        else if (element instanceof ProgrammHierarchie)
        {
          Collection<Programm> programme = ((ProgrammHierarchie)element).getProgramme();
          for (Programm programm : programme)
          {
            if (select(viewer, element, programm))
              return true;
          }
        }
        return false;
      }
    };

    _tvProgrammHierarchie.getTree().setRedraw(false);
    // alten Filter lschen
    if (_tvProgrammHierarchie.getFilters().length > 0)
      _tvProgrammHierarchie.removeFilter(_tvProgrammHierarchie.getFilters()[0]);

    // neuen Filter einfgen und Ebenen entsprechend expandieren
    if (ausdruck.length() > 0)
    {
      _tvProgrammHierarchie.addFilter(textFilter);
      List<ProgrammHierarchie> input = (List<ProgrammHierarchie>)_tvProgrammHierarchie.getInput();
      if (input != null)
      {
        for (ProgrammHierarchie programmart : input)
        {
          if (_mapExpandiert.containsKey(getViewSite().getSecondaryId()))
          {
            Object[] objects = _mapExpandiert.get(getViewSite().getSecondaryId());
            for (Object object : objects)
            {
              if (object instanceof ProgrammHierarchie)
              {
                ProgrammHierarchie art = (ProgrammHierarchie)object;
                if (programmart.toString().equals(art.toString()))
                {
                  _tvProgrammHierarchie.expandToLevel(programmart, 1);
                }
              }
            }
          }
        }
      }
    }
    _tvProgrammHierarchie.getTree().setRedraw(true);
  }

  /**
   * Setzt die Sortierung des Baums entsprechend auf- oder abwrts unter Bercksichtigung des gegebenen Comparators
   * 
   * @param up
   *          Richtung der Sortierung (SWT.UP/SWT.DOWN)
   */
  private void aktionSortierung(int up, final String column)
  {
    if (up == SWT.UP)
    {
      _tvProgrammTyp.setSorter(new ViewerSorter() {
        public int compare(Viewer viewer, Object e1, Object e2)
        {
          int ret = 0;

          if (e1 instanceof Programmart)
          {
            ret = -1;
          }
          else if (e1 instanceof Programm)
          {
            Programm prg1 = ((Programm)e1);
            Programm prg2 = ((Programm)e2);
            ret = compareProgramm(column, prg1, prg2);
          }
          return ret;
        }
      });
    }
    else
    {
      _tvProgrammTyp.setSorter(new ViewerSorter() {
        public int compare(Viewer viewer, Object e1, Object e2)
        {
          int ret = 0;

          if (e1 instanceof Programmart)
          {
            ret = -1;
          }
          else if (e1 instanceof Programm)
          {
            Programm prg1 = ((Programm)e1);
            Programm prg2 = ((Programm)e2);
            ret = compareProgramm(column, prg2, prg1);
          }
          return ret;
        }
      });
    }
  }

  /**
   * Fllt die Liste der Sonderprogramme.
   */
  public void bestimmeGefilterteProgrammTypen()
  {
    List<Programmart> arten = getListeProgramme();
    List<Programmart> input = new ArrayList<Programmart>();
    input.addAll(arten);

    int index = _comboObjTypFilter.getSelectionIndex();
    String nameTyp = _comboObjTypFilter.getItem(index);

    List<Programmart> gefilterteProgrammArten = new ArrayList<Programmart>();

    /*
     * Nach Typen filtern
     */
    if (nameTyp.equals(_alle))
    {
      for (Programmart art : input)
        gefilterteProgrammArten.add(art);
    }
    else
    {
      for (Programmart art : input)
      {
        if (art.toString().equals(nameTyp))
          gefilterteProgrammArten.add(art);
      }
    }

    /*
     * Nach Namen filtern
     */
    List<Programmart> gefilterteDatensaetzeNamen = new ArrayList<Programmart>();

    String filterName = _textNamen.getText();
    if (filterName.isEmpty() || filterName.equals(_defaultText))
    {
      for (Programmart art : gefilterteProgrammArten)
      {
        gefilterteDatensaetzeNamen.add(art);
      }
    }
    else
    {
      for (Programmart art : gefilterteProgrammArten)
      {
        List<Programm> gefilterteProgrammeNamen = new ArrayList<Programm>();
        Collection<Programm> programme = art.getProgramme();
        for (Programm prg : programme)
        {
          try
          {
            if (prg.toString().contains(filterName))
              gefilterteProgrammeNamen.add(prg);
            else if (prg.toString().matches(filterName))
              gefilterteProgrammeNamen.add(prg);
          }
          catch (Exception e)
          {
            e.printStackTrace();
          }
        }

        art.setProgramme(gefilterteProgrammeNamen);

        if (art.getProgramme().size() > 0)
        {
          gefilterteDatensaetzeNamen.add(art);
        }

      }
    }
    _tvProgrammTyp.setInput(gefilterteDatensaetzeNamen);
  }

  /**
   * Fllt die Liste der WanderBaustellen.
   */
  public void bestimmeGefilterteWanderBaustelle()
  {
    List<Wanderbaustelle> arten = getListeWanderbaustellen2();
    List<Wanderbaustelle> input = new ArrayList<Wanderbaustelle>();
    input.addAll(arten);

    int index = _comboObjTypFilter2.getSelectionIndex();
    String nameTyp = _comboObjTypFilter2.getItem(index);

    List<Wanderbaustelle> gefilterteProgrammArten = new ArrayList<Wanderbaustelle>();

    /*
     * Nach Typen filtern
     */
    if (nameTyp.equals(_alle))
    {
      for (Wanderbaustelle art : input)
        gefilterteProgrammArten.add(art);
    }
    else
    {
      for (Wanderbaustelle art : input)
      {
        if (art.toString().equals(nameTyp))
          gefilterteProgrammArten.add(art);
      }
    }

    /*
     * Nach Namen filtern
     */
    List<Wanderbaustelle> gefilterteDatensaetzeNamen = new ArrayList<Wanderbaustelle>();

    String filterName = _textNamen2.getText();
    if (filterName.isEmpty() || filterName.equals(_defaultText))
    {
      for (Wanderbaustelle art : gefilterteProgrammArten)
      {
        gefilterteDatensaetzeNamen.add(art);
      }
    }
    else
    {
      for (Wanderbaustelle art : gefilterteProgrammArten)
      {
        List<Programm> gefilterteProgrammeNamen = new ArrayList<Programm>();
        Collection<Programm> programme = art.getProgramme();
        for (Programm prg : programme)
        {
          try
          {
            if (prg.toString().contains(filterName))
              gefilterteProgrammeNamen.add(prg);
            else if (prg.toString().matches(filterName))
              gefilterteProgrammeNamen.add(prg);
          }
          catch (Exception e)
          {
            e.printStackTrace();
          }
        }

        art.setProgramme(gefilterteProgrammeNamen);

        if (art.getProgramme().size() > 0)
        {
          gefilterteDatensaetzeNamen.add(art);
        }

      }
    }
    _tvWanderBaustelle.setInput(gefilterteDatensaetzeNamen);
  }

  /**
   * Fllt die Liste der Sonderprogramme.
   */
  public void bestimmeGefilterteProgrammHierarchie()
  {
    List<ProgrammHierarchie> arten = getListeProgrammHierarchie(_systemObject);
    List<ProgrammHierarchie> input = new ArrayList<ProgrammHierarchie>();
    input.addAll(arten);

    int index = _comboObjTypFilter21.getSelectionIndex();
    String nameTyp = _comboObjTypFilter21.getItem(index);

    List<ProgrammHierarchie> gefilterteProgrammArten = new ArrayList<ProgrammHierarchie>();

    /*
     * Nach Typen filtern
     */
    if (nameTyp.equals(_alle))
    {
      for (ProgrammHierarchie art : input)
        gefilterteProgrammArten.add(art);
    }
    else
    {
      for (ProgrammHierarchie art : input)
      {
        if (art.toString().equals(nameTyp))
          gefilterteProgrammArten.add(art);
      }
    }

    /*
     * Nach Namen filtern
     */
    List<ProgrammHierarchie> gefilterteDatensaetzeNamen = new ArrayList<ProgrammHierarchie>();

    String filterName = _textNamen2.getText();
    if (filterName.isEmpty() || filterName.equals(_defaultText))
    {
      for (ProgrammHierarchie art : gefilterteProgrammArten)
      {
        gefilterteDatensaetzeNamen.add(art);
      }
    }
    else
    {
      for (ProgrammHierarchie art : gefilterteProgrammArten)
      {
        List<Programm> gefilterteProgrammeNamen = new ArrayList<Programm>();
        Collection<Programm> programme = art.getProgramme();
        for (Programm prg : programme)
        {
          try
          {
            if (prg.toString().contains(filterName))
              gefilterteProgrammeNamen.add(prg);
            else if (prg.toString().matches(filterName))
              gefilterteProgrammeNamen.add(prg);
          }
          catch (Exception e)
          {
            e.printStackTrace();
          }
        }

        art.setProgramme(gefilterteProgrammeNamen);

        if (art.getProgramme().size() > 0)
        {
          gefilterteDatensaetzeNamen.add(art);
        }

      }
    }
    _tvProgrammHierarchie.setInput(gefilterteDatensaetzeNamen);
  }

  /**
   * Alle Anzeigen auf beliebigen Inhalt setzten
   */
  private void schaltbildZuruecksetzen()
  {
    if (_contentsView != null)
    {
      _contentsView.aktualisiereSichtKeineSelektion();
    }
  }

  /**
   * Sichert die Expandierten Objekte in einer Map
   * 
   * @param element
   * 
   * @param flag
   */
  private void sichereExpandierteObjekte(Object element, boolean flag)
  {
    List<Object> list = new ArrayList<Object>();
    Object[] objs = null;

    int index = _folder.getSelectionIndex();

    switch (index)
    {
      case 0:
        objs = _tvProgrammTyp.getExpandedElements();
        break;
      case 1:
        objs = _tvWanderBaustelle.getExpandedElements();
        break;
      case 2:
        objs = _tvProgrammHierarchie.getExpandedElements();
        break;
      default:
    }

    for (Object object : objs)
    {
      list.add(object);
    }

    boolean inBearbeitung = false;

    if (flag)
    {
      if (!list.contains(element))
      {
        inBearbeitung = true;
        list.add(element);
      }
    }
    else
    {
      if (list.contains(element) && !inBearbeitung)
      {
        list.remove(element);
      }
    }
    _mapExpandiert.put(_contentsView.getViewSite().getSecondaryId(), list.toArray());
  }

  /**
   * Der Content-Provider
   * 
   * @author Pittner
   * 
   */
  private class ProgrammTypContentProvider implements ITreeContentProvider
  {

    public Object[] getChildren(Object parentElement)
    {

      if (parentElement instanceof List)
        return ((List<?>)parentElement).toArray();

      if (parentElement instanceof Programmart)
        return ((Programmart)parentElement).getProgramme().toArray();

      return new Object[0];
    }

    public Object getParent(Object element)
    {

      if (element instanceof Programm)
      {
        return ((Programm)element).getParent();
      }

      return null;
    }

    public boolean hasChildren(Object element)
    {
      if (element instanceof List)
        return ((List<?>)element).size() > 0;
      if (element instanceof Programmart)
        return ((Programmart)element).getProgramme().size() > 0;

      return false;
    }

    public Object[] getElements(Object input)
    {
      // input ist das, was in setInput(..) gesetzt wurde.
      return getChildren(input);
    }

    public void dispose()
    {
    }

    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
    {
    }
  }

  /**
   * Der Content-Provider
   * 
   * @author Pittner
   * 
   */
  private class WanderBaustelleContentProvider implements ITreeContentProvider
  {

    public Object[] getChildren(Object parentElement)
    {

      if (parentElement instanceof List)
        return ((List<?>)parentElement).toArray();

      if (parentElement instanceof Wanderbaustelle)
        return ((Wanderbaustelle)parentElement).getProgramme().toArray();

      return new Object[0];
    }

    public Object getParent(Object element)
    {

      if (element instanceof Programm)
      {
        return ((Programm)element).getParent();
      }

      return null;
    }

    public boolean hasChildren(Object element)
    {
      if (element instanceof List)
        return ((List<?>)element).size() > 0;
      if (element instanceof Wanderbaustelle)
        return ((Wanderbaustelle)element).getProgramme().size() > 0;

      return false;
    }

    public Object[] getElements(Object input)
    {
      // input ist das, was in setInput(..) gesetzt wurde.
      return getChildren(input);
    }

    public void dispose()
    {
    }

    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
    {
    }
  }

  /**
   * Der Label-Provider des Trees
   * 
   * @author Pittner
   * 
   */
  private class ProgrammTypLabelProvider implements ITableLabelProvider, ITableFontProvider
  // , IColorProvider
  {

    public Image getColumnImage(Object element, int columnIndex)
    {
      Image re = null;

      switch (columnIndex)
      {
        case 0:
          if (element instanceof Programm)
          {
            String simu = ((Programm)element).getSimulation();
            if (simu.contains("Ja"))
            {
              re = ImageFactory.getImage("icons/sp_simu.png");
            }
            String stand = ((Programm)element).getStand();
            if (stand.equals("Aktiv"))
            {
              re = ImageFactory.getImage("icons/sp_aktiv.png");
            }
            else if (stand.equals("Pause"))
            {
              re = ImageFactory.getImage("icons/sp_pause.png");
            }
          }
          break;
        default:
      }

      return re;
    }

    public String getColumnText(Object element, int columnIndex)
    {

      switch (columnIndex)
      {
        case 1:
          return element.toString();
        case 2:
          if (element instanceof Programm)
            return ((Programm)element).getZeitpunkt();
        case 3:
          if (element instanceof Programm)
            return ((Programm)element).getStand();
        case 4:
          if (element instanceof Programm)
            return ((Programm)element).getSimulation();
        case 5:
          if (element instanceof Programm)
            return ((Programm)element).getBenutzer();
        case 6:
          if (element instanceof Programm)
            return ((Programm)element).getUrsache();
        case 7:
          if (element instanceof Programm)
            return ((Programm)element).getVeranlasser();
      }
      return null;
    }

    public void addListener(ILabelProviderListener listener)
    {
    }

    public void dispose()
    {
    }

    public boolean isLabelProperty(Object element, String property)
    {
      return false;
    }

    public void removeListener(ILabelProviderListener listener)
    {
    }

    @Override
    public Font getFont(Object element, int columnIndex)
    {
      switch (columnIndex)
      {
        case 1:
          if (element instanceof Programmart)
          {
            FontData fontData = Display.getCurrent().getSystemFont().getFontData()[0];
            String name = fontData.getName();
            return JFaceResources.getFontRegistry().getBold(name);
          }
          if (element instanceof Programm)
          {
            String stand = ((Programm)element).getStand();
            if (stand.equals("Aktiv"))
            {
              FontData fontData = Display.getCurrent().getSystemFont().getFontData()[0];
              String name = fontData.getName();
              return JFaceResources.getFontRegistry().getItalic(name);
            }
          }

      }
      return null;
    }
  }

  /**
   * Der Label-Provider des Trees
   * 
   * @author Pittner
   * 
   */
  private class WanderBaustelleLabelProvider implements ITableLabelProvider, ITableFontProvider
  {

    public Image getColumnImage(Object element, int columnIndex)
    {
      Image re = null;
      switch (columnIndex)
      {
        case 0:
          if (element instanceof Programm)
          {

            String simu = ((Programm)element).getSimulation();
            if (simu.contains("Ja"))
            {
              re = ImageFactory.getImage("icons/sp_simu.png");
            }
            String stand = ((Programm)element).getStand();
            if (stand.equals("Aktiv"))
            {
              re = ImageFactory.getImage("icons/sp_aktiv.png");
            }
            else if (stand.equals("Pause"))
            {
              re = ImageFactory.getImage("icons/sp_pause.png");
            }
          }
          break;
        default:
      }

      return re;
    }

    public String getColumnText(Object element, int columnIndex)
    {

      switch (columnIndex)
      {
        case 1:
          return element.toString();
        case 2:
          if (element instanceof Programm)
            return ((Programm)element).getStand();
        case 3:
          if (element instanceof Programm)
            return ((Programm)element).getSimulation();

      }
      return null;
    }

    public void addListener(ILabelProviderListener listener)
    {
    }

    public void dispose()
    {
    }

    public boolean isLabelProperty(Object element, String property)
    {
      return false;
    }

    public void removeListener(ILabelProviderListener listener)
    {
    }

    @Override
    public Font getFont(Object element, int columnIndex)
    {
      switch (columnIndex)
      {
        case 1:
          if (element instanceof Wanderbaustelle)
          {
            FontData fontData = Display.getCurrent().getSystemFont().getFontData()[0];
            String name = fontData.getName();
            return JFaceResources.getFontRegistry().getBold(name);
          }
          if (element instanceof Programm)
          {
            String stand = ((Programm)element).getStand();
            if (stand.equals("Aktiv"))
            {
              FontData fontData = Display.getCurrent().getSystemFont().getFontData()[0];
              String name = fontData.getName();
              return JFaceResources.getFontRegistry().getItalic(name);
            }
          }
      }
      return null;
    }
  }

  /**
   * Der Label-Provider fr den Tab ProgrammHierarchie
   * 
   * @author Pittner
   * 
   */
  private class ProgrammHierarchieLabelProvider implements ITableLabelProvider, ITableFontProvider
  // , IColorProvider
  {

    public Image getColumnImage(Object element, int columnIndex)
    {
      Image re = null;
      switch (columnIndex)
      {
        case 0:
          if (element instanceof Programm)
          {

            String simu = ((Programm)element).getSimulation();
            if (simu.contains("Ja"))
            {
              re = ImageFactory.getImage("icons/sp_simu.png");
            }
            String stand = ((Programm)element).getStand();
            if (stand.equals("Aktiv"))
            {
              re = ImageFactory.getImage("icons/sp_aktiv.png");
            }
            else if (stand.equals("Pause"))
            {
              re = ImageFactory.getImage("icons/sp_pause.png");
            }
          }
          break;
        default:
      }

      return re;
    }

    public String getColumnText(Object element, int columnIndex)
    {

      switch (columnIndex)
      {
        case 1:
          return element.toString();
        case 2:
          if (element instanceof Programm)
            return ((Programm)element).getZeitpunkt();
        case 3:
          if (element instanceof Programm)
            return ((Programm)element).getStand();
        case 4:
          if (element instanceof Programm)
            return ((Programm)element).getSimulation();
        case 5:
          if (element instanceof Programm)
            return ((Programm)element).getBenutzer();
        case 6:
          if (element instanceof Programm)
            return ((Programm)element).getUrsache();
        case 7:
          if (element instanceof Programm)
            return ((Programm)element).getVeranlasser();

      }
      return null;
    }

    public void addListener(ILabelProviderListener listener)
    {
    }

    public void dispose()
    {
    }

    public boolean isLabelProperty(Object element, String property)
    {
      return false;
    }

    public void removeListener(ILabelProviderListener listener)
    {
    }

    @Override
    public Font getFont(Object element, int columnIndex)
    {
      switch (columnIndex)
      {
        case 1:
          if (element instanceof ProgrammHierarchie)
          {
            FontData fontData = Display.getCurrent().getSystemFont().getFontData()[0];
            String name = fontData.getName();
            return JFaceResources.getFontRegistry().getBold(name);
          }
          if (element instanceof Programm)
          {
            String stand = ((Programm)element).getStand();
            if (stand.equals("Aktiv"))
            {
              FontData fontData = Display.getCurrent().getSystemFont().getFontData()[0];
              String name = fontData.getName();
              return JFaceResources.getFontRegistry().getItalic(name);
            }
          }

      }
      return null;
    }
  }

  /**
   * Hilfs-Methode, initialisiert und fllt die Map mit den ProgrammHierarchien
   * 
   * @param list
   */
  private void initMapHierarchie(Collection<ProgrammHierarchie> list)
  {
    if (_mapHierarchie == null)
    {
      _mapHierarchie = new HashMap<String, ProgrammHierarchie>();
    }
    _mapHierarchie.clear();
    for (ProgrammHierarchie ph : list)
    {
      _mapHierarchie.put(ph.bezeichnung, ph);
    }
  }

  /**
   * Der Content-Provider fr den Tab ProgrammHierarchie
   * 
   * @author Pittner
   * 
   */
  private class ProgrammHierarchieContentProvider implements ITreeContentProvider
  {

    public Object[] getChildren(Object parentElement)
    {

      if (parentElement instanceof List)
        return ((List<?>)parentElement).toArray();

      if (parentElement instanceof ProgrammHierarchie)
      {
        List<Object> list = new ArrayList<Object>();

        list.addAll(((ProgrammHierarchie)parentElement).getProgramme());

        Collection<String> menues = ((ProgrammHierarchie)parentElement).getUntermenues();
        for (String name : menues)
        {
          if (_mapHierarchie.containsKey(name))
          {
            list.add(_mapHierarchie.get(name));
          }
        }
        return list.toArray();
      }
      return new Object[0];
    }

    public Object getParent(Object element)
    {

      if (element instanceof Programm)
      {
        return ((Programm)element).getParent();
      }

      if (element instanceof ProgrammHierarchie)
      {
        String parent = ((ProgrammHierarchie)element).getParentString();

        if (_mapHierarchie.containsKey(parent))
        {
          return _mapHierarchie.get(parent);
        }
      }
      return null;
    }

    public boolean hasChildren(Object element)
    {
      if (element instanceof List)
        return ((List<?>)element).size() > 0;
      if (element instanceof ProgrammHierarchie)
      {
        boolean b1 = ((ProgrammHierarchie)element).getProgramme().size() > 0;
        boolean b2 = ((ProgrammHierarchie)element).getUntermenues().size() > 0;
        return b1 | b2;
      }
      return false;
    }

    public Object[] getElements(Object input)
    {
      return getChildren(input);
    }

    public void dispose()
    {
    }

    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
    {
    }
  }

  /**
   * Liefert die Toolbar mit den Kontroll-Aktionen zur SonderprogrammAuswahl
   */
  private void erzeugeToolbar()
  {
    // selektierte Baumknoten verstecken
    IAction actionCollapse = new Action("Collapse") {
      public void run()
      {
        actionCollapseActualSelection();
      };
    };
    actionCollapse.setToolTipText("Kindknoten verstecken");
    actionCollapse.setImageDescriptor(Activator.getImageDescriptor("icons/tree_collapse.png"));

    // selektierte Baumknoten expandieren
    IAction actionExpand = new Action("Expand") {
      public void run()
      {
        actionExpandActualSelection();
      };
    };
    actionExpand.setToolTipText("Kindknoten anzeigen");
    actionExpand.setImageDescriptor(Activator.getImageDescriptor("icons/tree_expand.png"));

    // alle Kindknoten verstecken
    IAction actionCollapseAll = new Action("CollapseAll") {
      public void run()
      {
        if (_tvProgrammTyp != null && _folder.getSelectionIndex() == 0)
        {
          _tvProgrammTyp.getTree().setRedraw(false);
          _tvProgrammTyp.collapseAll();
          _tvProgrammTyp.getTree().setRedraw(true);
        }
        else if (_tvWanderBaustelle != null && _folder.getSelectionIndex() == 1)
        {
          _tvWanderBaustelle.getTree().setRedraw(false);
          _tvWanderBaustelle.collapseAll();
          _tvWanderBaustelle.getTree().setRedraw(true);
        }
        else if (_tvProgrammHierarchie != null && _folder.getSelectionIndex() == 2)
        {
          _tvProgrammHierarchie.getTree().setRedraw(false);
          _tvProgrammHierarchie.collapseAll();
          _tvProgrammHierarchie.getTree().setRedraw(true);
        }
      };
    };
    actionCollapseAll.setToolTipText("alle Kindknoten verstecken");
    actionCollapseAll.setImageDescriptor(Activator.getImageDescriptor("icons/tree_collapseAll.png"));

    // alle Kindknoten anzeigen
    IAction actionExpandAll = new Action("ExpandAll") {
      public void run()
      {
        // Informiert den Baum mit dem Neuzeichnen bis zur Fertigstellung zu
        // warten
        if (_tvProgrammTyp != null && _folder.getSelectionIndex() == 0)
        {
          _tvProgrammTyp.getTree().setRedraw(false);
          _tvProgrammTyp.expandAll();
          _tvProgrammTyp.getTree().setRedraw(true);
        }
        else if (_tvWanderBaustelle != null && _folder.getSelectionIndex() == 1)
        {
          _tvWanderBaustelle.getTree().setRedraw(false);
          _tvWanderBaustelle.expandAll();
          _tvWanderBaustelle.getTree().setRedraw(true);
        }
        else if (_tvProgrammHierarchie != null && _folder.getSelectionIndex() == 2)
        {
          _tvProgrammHierarchie.getTree().setRedraw(false);
          _tvProgrammHierarchie.expandAll();
          _tvProgrammHierarchie.getTree().setRedraw(true);
        }
      };
    };
    actionExpandAll.setToolTipText("alle Kindknoten anzeigen");
    actionExpandAll.setImageDescriptor(Activator.getImageDescriptor("icons/tree_expandAll.png"));

    // ToolBar erzeugen
    IToolBarManager toolBarManager = getViewSite().getActionBars().getToolBarManager();
    toolBarManager.add(actionCollapse);
    toolBarManager.add(actionExpand);
    toolBarManager.add(actionCollapseAll);
    toolBarManager.add(actionExpandAll);

  }

  /**
   * Kollabiert die aktuelle Selektion im Baum fr die ausgewaehlte Ansicht
   */
  private void actionCollapseActualSelection()
  {
    if (_tvProgrammTyp != null && _folder.getSelectionIndex() == 0)
    {
      LinkedList<Object> expandedElem = new LinkedList<Object>();

      ISelection selection = _tvProgrammTyp.getSelection();
      if (selection instanceof StructuredSelection)
      {
        StructuredSelection ss = (StructuredSelection)selection;

        Iterator itExpElem = ss.iterator();
        while (itExpElem.hasNext())
        {
          Object obj = itExpElem.next();

          if (obj instanceof Programmart)
          {
            Programmart elem = (Programmart)obj;
            if (elem.getProgramme().size() > 0)
            {
              if (!expandedElem.contains(elem))
              {
                expandedElem.add(elem);
              }
            }
          }
          else if (obj instanceof Programm)
          {
            Programm elem = (Programm)obj;
            Object parent = elem.getParent();
            if (!expandedElem.contains(parent))
            {
              expandedElem.add(parent);
            }
          }
        }
      }

      if (expandedElem.size() > 0)
      {
        _tvProgrammTyp.getTree().setRedraw(false);
        Iterator<Object> it = expandedElem.iterator();
        while (it.hasNext())
        {
          Object elem = it.next();
          _tvProgrammTyp.collapseToLevel(elem, AbstractTreeViewer.ALL_LEVELS);
        }
        _tvProgrammTyp.getTree().setRedraw(true);
      }
    }
    else if (_tvWanderBaustelle != null && _folder.getSelectionIndex() == 1)
    {
      LinkedList<Object> expandedElem = new LinkedList<Object>();

      ISelection selection = _tvWanderBaustelle.getSelection();
      if (selection instanceof StructuredSelection)
      {
        StructuredSelection ss = (StructuredSelection)selection;

        Iterator itExpElem = ss.iterator();
        while (itExpElem.hasNext())
        {
          Object obj = itExpElem.next();

          if (obj instanceof Wanderbaustelle)
          {
            Wanderbaustelle elem = (Wanderbaustelle)obj;
            if (elem.getProgramme().size() > 0)
            {
              if (!expandedElem.contains(elem))
              {
                expandedElem.add(elem);
              }
            }
          }
          else if (obj instanceof Programm)
          {
            Programm elem = (Programm)obj;
            Object parent = elem.getParent();
            if (!expandedElem.contains(parent))
            {
              expandedElem.add(parent);
            }
          }
        }
      }

      if (expandedElem.size() > 0)
      {
        _tvWanderBaustelle.getTree().setRedraw(false);
        Iterator<Object> it = expandedElem.iterator();
        while (it.hasNext())
        {
          Object elem = it.next();
          _tvWanderBaustelle.collapseToLevel(elem, AbstractTreeViewer.ALL_LEVELS);
        }
        _tvWanderBaustelle.getTree().setRedraw(true);
      }
    }
  }

  /**
   * Expandiert die aktuelle Selektion im Baum fr die ausgewaehlte Ansicht
   */
  private void actionExpandActualSelection()
  {
    if (_tvProgrammTyp != null && _folder.getSelectionIndex() == 0)
    {
      LinkedList<Object> collapsedNodes = new LinkedList<Object>();

      ISelection selection = _tvProgrammTyp.getSelection();
      if (selection instanceof StructuredSelection)
      {
        StructuredSelection ss = (StructuredSelection)selection;

        @SuppressWarnings("unchecked")
        Iterator itColBest = ss.iterator();
        while (itColBest.hasNext())
        {
          Object obj = itColBest.next();

          if (obj instanceof Programmart)
          {
            Programmart elem = (Programmart)obj;
            if (elem.getProgramme().size() > 0)
            {
              collapsedNodes.add(elem);
            }
          }
        }
      }

      if (collapsedNodes.size() > 0)
      {
        _tvProgrammTyp.getTree().setRedraw(false);
        Iterator<Object> it = collapsedNodes.iterator();
        while (it.hasNext())
        {
          Object colNode = it.next();
          _tvProgrammTyp.expandToLevel(colNode, AbstractTreeViewer.ALL_LEVELS);
        }
        _tvProgrammTyp.getTree().setRedraw(true);
      }
    }
    else if (_tvWanderBaustelle != null && _folder.getSelectionIndex() == 1)
    {
      LinkedList<Object> collapsedNodes = new LinkedList<Object>();

      ISelection selection = _tvWanderBaustelle.getSelection();
      if (selection instanceof StructuredSelection)
      {
        StructuredSelection ss = (StructuredSelection)selection;

        @SuppressWarnings("unchecked")
        Iterator itColBest = ss.iterator();
        while (itColBest.hasNext())
        {
          Object obj = itColBest.next();

          if (obj instanceof Wanderbaustelle)
          {
            Wanderbaustelle elem = (Wanderbaustelle)obj;
            if (elem.getProgramme().size() > 0)
            {
              collapsedNodes.add(elem);
            }
          }
        }
      }

      if (collapsedNodes.size() > 0)
      {
        _tvWanderBaustelle.getTree().setRedraw(false);
        Iterator<Object> it = collapsedNodes.iterator();
        while (it.hasNext())
        {
          Object colNode = it.next();
          _tvWanderBaustelle.expandToLevel(colNode, AbstractTreeViewer.ALL_LEVELS);
        }
        _tvWanderBaustelle.getTree().setRedraw(true);
      }
    }
    else if (_tvProgrammHierarchie != null && _folder.getSelectionIndex() == 2)
    {
      LinkedList<Object> collapsedNodes = new LinkedList<Object>();

      ISelection selection = _tvProgrammHierarchie.getSelection();
      if (selection instanceof StructuredSelection)
      {
        StructuredSelection ss = (StructuredSelection)selection;

        @SuppressWarnings("unchecked")
        Iterator itColBest = ss.iterator();
        while (itColBest.hasNext())
        {
          Object obj = itColBest.next();

          if (obj instanceof ProgrammHierarchie)
          {
            ProgrammHierarchie elem = (ProgrammHierarchie)obj;
            if (elem.getProgramme().size() > 0)
            {
              collapsedNodes.add(elem);
            }
          }
        }
      }

      if (collapsedNodes.size() > 0)
      {
        _tvProgrammHierarchie.getTree().setRedraw(false);
        Iterator<Object> it = collapsedNodes.iterator();
        while (it.hasNext())
        {
          Object colNode = it.next();
          _tvProgrammHierarchie.expandToLevel(colNode, AbstractTreeViewer.ALL_LEVELS);
        }
        _tvProgrammHierarchie.getTree().setRedraw(true);
      }
    }
  }

  /**
   * Fllt die Combo mit den ProgrammTypen
   */
  private void fuelleComboProgrammTyp()
  {

    List<Programmart> listeProgramme = getListeProgramme();

    List<String> sortList = new ArrayList<String>();

    for (Programmart art : listeProgramme)
    {
      sortList.add(art.toString());
    }

    Collections.sort(sortList);
    sortList.add(0, _alle);

    if (_comboObjTypFilter != null && !_comboObjTypFilter.isDisposed())
    {
      _comboObjTypFilter.removeAll();
      for (String s : sortList)
      {
        _comboObjTypFilter.add(s);
      }
      _comboObjTypFilter.select(0);
    }
  }

  /**
   * Fllt die Combo mit den Wanderbaustellen
   */
  private void fuelleComboWanderBaustelle()
  {

    List<Wanderbaustelle> listeProgramme = getListeWanderbaustellen2();

    List<String> sortList = new ArrayList<String>();

    for (Wanderbaustelle art : listeProgramme)
    {
      sortList.add(art.toString());
    }

    // Collections.sort(sortList);
    sortList.add(0, _alle);

    if (_comboObjTypFilter2 != null && !_comboObjTypFilter2.isDisposed())
    {
      _comboObjTypFilter2.removeAll();
      for (String s : sortList)
      {
        _comboObjTypFilter2.add(s);
      }
      _comboObjTypFilter2.select(0);
    }
  }

  /**
   * Fllt die Combo mit den ProgrammHierarchien
   */
  private void fuelleComboProgrammHierarchie()
  {

    List<ProgrammHierarchie> listeProgramme = getListeProgrammHierarchie(_systemObject);

    List<String> sortList = new ArrayList<String>();

    for (ProgrammHierarchie art : listeProgramme)
    {
      sortList.add(art.toString());
    }

    // Collections.sort(sortList);
    sortList.add(0, _alle);

    if (_comboObjTypFilter21 != null && !_comboObjTypFilter21.isDisposed())
    {
      _comboObjTypFilter21.removeAll();
      for (String s : sortList)
      {
        _comboObjTypFilter21.add(s);
      }
      _comboObjTypFilter21.select(0);
    }
  }

  /**
   * Vergleicht die Einzelnen Programmbestandteile fuer die Sortierung
   * 
   * @param column
   *          der Text der Spalte
   * @param prg1
   *          ein Sonderprogramm
   * @param prg2
   *          ein Sonderprogramm
   * 
   * @return Ergebnis fuer die Sortierung
   * 
   */
  private int compareProgramm(final String column, Programm prg1, Programm prg2)
  {
    int ret = 0;

    if (column.equals("Programm-Typ/-Name"))
    {
      ret = prg2.toString().compareTo(prg1.toString());
    }
    else if (column.equals("nderungszeitpunkt"))
    {
      try
      {
        Long l1 = _sdf.parse(prg1.getZeitpunkt()).getTime();
        Long l2 = _sdf.parse(prg2.getZeitpunkt()).getTime();
        ret = l1.compareTo(l2);
      }
      catch (ParseException e)

      {
        // TODO Auto-generated catch block
        e.printStackTrace();
      }
    }
    else if (column.equals("Stand"))
    {
      ret = prg1.getStand().compareTo(prg2.getStand());
    }
    else if (column.equals("Simulation"))
    {
      ret = prg1.getSimulation().compareTo(prg2.getSimulation());
    }
    else if (column.equals("Benutzer"))
    {
      ret = prg1.getBenutzer().compareTo(prg2.getBenutzer());
    }
    else if (column.equals("Ursache"))
    {
      ret = prg1.getUrsache().compareTo(prg2.getUrsache());
    }
    else if (column.equals("Veranlasser"))
    {
      ret = prg1.getVeranlasser().compareTo(prg2.getVeranlasser());
    }
    else
    {
      System.err.println("else: " + column);
    }
    return ret;
  }

  public ZustandsDarstellungSonderProgramm getContentsView()
  {
    return _contentsView;
  }

  public TreeViewer getTvFilter()
  {
    return _tvProgrammTyp;
  }

  public void setTvFilter(TreeViewer tvFilter)
  {
    _tvProgrammTyp = tvFilter;
  }

  private void aktionSelektion(SelectionChangedEvent event)
  {
    boolean reset = false;

    IStructuredSelection selection = (IStructuredSelection)event.getSelection();
    if (!selection.isEmpty())
    {
      Object firstElement = selection.getFirstElement();
      if (firstElement instanceof Programm)
      {
        Programm prg = (Programm)firstElement;

        ResultData rd = prg.getResult();

        if (_contentsView != null && _connection != null)
        {

          SonderProgramm ds = _contentsView.fuelleDatenStruktur(_connection, rd);
          if (ds != null)
          {
            _contentsView.aktualisiereSicht(ds);
          }
          _mapSelection.put(_contentsView.getViewSite().getSecondaryId(), prg);
        }
      }
      else if (firstElement instanceof Programmart)
      {
        Programmart art = (Programmart)firstElement;
        _mapSelection.put(_contentsView.getViewSite().getSecondaryId(), art);
        reset = true;
      }
      else if (firstElement instanceof Wanderbaustelle)
      {
        Wanderbaustelle art = (Wanderbaustelle)firstElement;
        _mapSelection.put(_contentsView.getViewSite().getSecondaryId(), art);
      }
      else if (firstElement instanceof ProgrammHierarchie)
      {
        ProgrammHierarchie art = (ProgrammHierarchie)firstElement;
        _mapSelection.put(_contentsView.getViewSite().getSecondaryId(), art);
      }
      else
      {
        reset = true;
      }
    }
    else
    {
      // Schaltbild zuruecksetzen
      reset = true;
    }

    if (reset)
      schaltbildZuruecksetzen();
  }

  private void aktionWanderbaustelleEditieren(Wanderbaustelle wbs)
  {
    IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
    IWorkbenchPage page = dw.getActivePage();
    try
    {
      page.showView(AuswahlWanderBaustelleView.ID, wbs.bezeichnung, IWorkbenchPage.VIEW_ACTIVATE);

      IViewReference ref = page.findViewReference(AuswahlWanderBaustelleView.ID, wbs.bezeichnung);
      ((WorkbenchPage)page).getActivePerspective().getPresentation().detachPart(ref);
      IWorkbenchPart viewPart = ref.getPart(false);
      ((AuswahlWanderBaustelleView)viewPart).setContentsView(_contentsView);

      Shell shell = viewPart.getSite().getShell();

      Display display = PlatformUI.getWorkbench().getDisplay();
      Point p = display.getCursorLocation();

      shell.setLocation(p.x - 10, p.y - 10);
      shell.setSize(520, 315);

      if (dw != null)
      {
        /*
         * Positionierung des Detached-Windows zentriert auf der Shell des Workbench-Window.
         */
        Point sizeWB = dw.getShell().getSize();
        Point sizeSH = shell.getSize();

        int x = ((int)(((double)sizeWB.x - (double)sizeSH.x) / 2.0d)) + dw.getShell().getLocation().x;
        int y = ((int)(((double)sizeWB.y - (double)sizeSH.y) / 2.0d)) + dw.getShell().getLocation().y;

        shell.setLocation(x, y);
      }
      else
      {
        IllegalArgumentException iae = new IllegalArgumentException(
            "Zentrieren des Fensters nicht mglich, das aktive WorkbenchWindow kann nicht bestimmt werden!");
        Activator
            .getDefault()
            .getLog()
            .log(
                new Status(
                    Status.WARNING,
                    Activator.PLUGIN_ID,
                    "Zentrieren des Fensters nicht mglich, das aktive WorkbenchWindow kann nicht bestimmt werden!",
                    iae));
      }

    }
    catch (PartInitException e)
    {
      e.printStackTrace();
    }
  }

//  @Override
//  public void propertyChange(PropertyChangeEvent evt)
//  {
//    // TODO Auto-generated method stub
//    if (evt != null)
//    {
//      String prop = evt.getPropertyName();
//      
//      boolean init = false;
//      if (prop.equals(ZustandsDarstellungSonderProgramm.PROP_SONDER_PROGRAMM_INIT_NO)){
//        secondaryId = "NO";
//        init = true;
//      }
//      else if (prop.equals(ZustandsDarstellungSonderProgramm.PROP_SONDER_PROGRAMM_INIT_SW)){
//        secondaryId = "SW";
//        init = true;
//      }
//     
//      boolean neueDaten = false;     
//      if (init){        
//        _comboObjTypFilter.removeAll();
//        _tvProgrammTyp.getTree().removeAll();
//        _tvProgrammTyp.setInput(null);
//        _comboObjTypFilter2.removeAll();
//        _tvWanderBaustelle.getTree().removeAll();
//        _tvWanderBaustelle.setInput(null);
//        _comboObjTypFilter21.removeAll();
//        _tvProgrammHierarchie.getTree().removeAll();
//        _tvProgrammHierarchie.setInput(null);
//        _contentsView = (ZustandsDarstellungSonderProgramm)evt.getSource();
//        neueDaten = true;
//      }
//      
//      
//      if (prop.equals(ZustandsDarstellungSonderProgramm.PROP_SONDER_PROGRAMM_DATA_NO)){
//        if (secondaryId.equals("NO")){
//          neueDaten = true;
//        }
//      }
//      else if (prop.equals(ZustandsDarstellungSonderProgramm.PROP_SONDER_PROGRAMM_DATA_SW)){
//        if (secondaryId.equals("SW")){
//          neueDaten = true;
//        }
//      }
//      if (neueDaten){
//        sonderProgrammeInListeAnzeigen(_contentsView);
//        expansionWiederHerstellen(secondaryId);
//        if (!init)
//          selektionWiederHerstellen(secondaryId, ((SystemObject)evt.getNewValue()).getPid());
//        setGroupAuswahlText();        
//      }
//    }   
//  }
  
}
