/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.views;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;

import org.eclipse.core.runtime.Assert;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DragSource;
import org.eclipse.swt.dnd.DragSourceAdapter;
import org.eclipse.swt.dnd.DragSourceEvent;
import org.eclipse.swt.dnd.DropTarget;
import org.eclipse.swt.dnd.DropTargetAdapter;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.ISelectionService;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import com.bitctrl.lib.eclipse.emf.gef.commands.SetCommand;

import de.bsvrz.buv.plugin.anlagendarstellung.Activator;
import de.bsvrz.buv.plugin.anlagendarstellung.AnlagendarstellungPackage;
import de.bsvrz.buv.plugin.anlagendarstellung.DoModelMitVerbLinie;
import de.bsvrz.buv.plugin.anlagendarstellung.editor.GraphicalEditorToView;
import de.bsvrz.buv.plugin.anlagendarstellung.einstellungen.SbaAnsichtProvider;
import de.bsvrz.buv.plugin.anlagendarstellung.einstellungen.SbaEinstellungen;
import de.bsvrz.buv.plugin.anlagendarstellung.preferences.DefsPreference;
import de.bsvrz.buv.plugin.darstellung.editparts.AnsichtEditPart;
import de.bsvrz.buv.plugin.darstellung.model.Ansicht;
import de.bsvrz.buv.plugin.darstellung.model.Darstellung;
import de.bsvrz.buv.plugin.darstellung.model.DarstellungsSpalte;
import de.bsvrz.buv.plugin.darstellung.model.StilisierteDarstellung;
import de.bsvrz.buv.plugin.dobj.model.DoKompositum;
import de.bsvrz.buv.plugin.dobj.model.DoModel;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Sicht zum Auswhlen von Spalten einer {@link Ansicht}.
 * 
 * 
 * @author Pittner
 * 
 */
public class AuswahlSpaltenView extends ViewPart
{

  public static String ID = "de.bsvrz.buv.plugin.anlagendarstellung.auswahlSpaltenView";

  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();

  private Composite _parent;

  private Composite compositeTop;

  private Group _gruppeUebersicht;

  private Table _tabelleSpalten;

  private TableViewer _tableViewerSpalten;

  private EditPart _contents;

  private Button _btnUp;

  private Button _btnDown;

  private TableItem _selSpalte;

  protected GraphicalEditorToView _part;

  @Override
  public void createPartControl(Composite parent)
  {
    _parent = parent;

    // Fr die F1 Taste
    PlatformUI.getWorkbench().getHelpSystem().setHelp( _parent, Activator.PLUGIN_ID + ".auswahlSpalten" );

    parent.setLayout(new GridLayout());
    createCompositeTop(parent);

    erzeugeButtonUnten(parent);
  
    List<DarstellungsSpalte> sortList = new ArrayList<DarstellungsSpalte>();

    if (_contents != null)
    {
      AnsichtEditPart contents = (AnsichtEditPart)_contents;

      Ansicht ansicht = contents.getModel();
      StilisierteDarstellung darstellung = (StilisierteDarstellung)ansicht.getDarstellung();

      EList<DarstellungsSpalte> spalten = darstellung.getSpalten();
      for (DarstellungsSpalte d : spalten)
      {
        sortList.add(d);
      }
      _tableViewerSpalten.setInput(sortList);

    }

    ISelectionService selectionService = getSite().getWorkbenchWindow().getSelectionService();
    selectionService.addSelectionListener(new ISelectionListener() {
      public void selectionChanged(IWorkbenchPart part, ISelection selection)
      {
        if (!(selection instanceof IStructuredSelection))
          return;

        if (part instanceof GraphicalEditorToView)
        {
          GraphicalEditorToView view = (GraphicalEditorToView)part;
          _part = view;
          GraphicalViewer viewer = (GraphicalViewer)view.getAdapter(GraphicalViewer.class);
          EditPart contents = viewer.getContents();
          if (contents instanceof AnsichtEditPart)
          {
            AnsichtEditPart ansichtEditPart = (AnsichtEditPart)contents;
            setContent(ansichtEditPart);
          }
        }
      }
    });

  }

  /**
   * Initialisierungen fr die zustzlichen rechten Buttons
   * 
   * @param parent
   *          Elternteil zu dem diese Komponenten hinzugefgt werden sollen
   */
  private void erzeugeButtonRechts(Composite parent)
  {
    // Buttons fr die Positionierung
    Composite compRechts = new Composite(parent, SWT.NONE);
    RowLayout rowLayout = new RowLayout(SWT.VERTICAL);
    rowLayout.pack = true;
    compRechts.setLayout(rowLayout);

    GridData data = new GridData(SWT.FILL, SWT.FILL, true, true);
    data.verticalAlignment = SWT.BEGINNING;
    compRechts.setLayoutData(data);

    _btnUp = new Button(compRechts, SWT.ARROW | SWT.UP);
    _btnUp.setEnabled(false);
    _btnUp.addSelectionListener(new SelectionAdapter() {
      // Selektiertes Element nach oben schieben
      public void widgetSelected(SelectionEvent event)
      {
        if (_selSpalte != null)
        {
          TableItem[] tia = _tabelleSpalten.getItems();

          for (int i = 0; i < tia.length; i++)
          {
            if (tia[i].equals(_selSpalte) && (i != 0))
            {
              String selText1 = _selSpalte.getText(1);              
              boolean selCheck = _selSpalte.getChecked();
              Object selData = _selSpalte.getData();

              tia[i].setText(1, tia[i - 1].getText(1));
              tia[i].setChecked(tia[i - 1].getChecked());
              tia[i].setData(tia[i - 1].getData());

              tia[i - 1].setText(1, selText1);
              tia[i - 1].setChecked(selCheck);
              tia[i - 1].setData(selData);

              _tabelleSpalten.setSelection(i - 1);
              _selSpalte = tia[i - 1];

              aktionMove(_selSpalte.getText(1), i, _selSpalte);
              _tabelleSpalten.redraw();
              break;
            }
          }

        }

      }
    });
    _btnDown = new Button(compRechts, SWT.ARROW | SWT.DOWN);
    _btnDown.setEnabled(false);
    _btnDown.addSelectionListener(new SelectionAdapter() {
      // Selektiertes Element nach unten schieben
      public void widgetSelected(SelectionEvent event)
      {
        if (_selSpalte != null)
        {
          TableItem[] tia = _tabelleSpalten.getItems();

          for (int i = 0; i < tia.length; i++)
          {
            if (tia[i].equals(_selSpalte) && (i != tia.length - 1))
            {
              String selText1 = _selSpalte.getText(1);              
              boolean selCheck = _selSpalte.getChecked();
              Object selData = _selSpalte.getData();

              tia[i].setText(1, tia[i + 1].getText(1));
              tia[i].setChecked(tia[i + 1].getChecked());
              tia[i].setData(tia[i + 1].getData());

              tia[i + 1].setText(1, selText1);
              tia[i + 1].setChecked(selCheck);
              tia[i + 1].setData(selData);

              _tabelleSpalten.setSelection(i + 1);
              _selSpalte = tia[i + 1];

              aktionMove(_selSpalte.getText(1), i + 2, _selSpalte);
              _tabelleSpalten.redraw();
              break;
            }
          }
        }
      }
    });
  }

  /**
   * Initialisierungen fr die zustzlichen Buttons unten
   * 
   * @param parent
   *          Elternteil zu dem diese Komponenten hinzugefgt werden sollen
   */
  private void erzeugeButtonUnten(Composite parent)
  {
    // Buttons fr die Aktivierung und Deaktivierung
    Composite compUnten = new Composite(parent, SWT.NONE);
    RowLayout rowLayout = new RowLayout();
    rowLayout.pack = false;
    compUnten.setLayout(rowLayout);

    GridData data = new GridData(SWT.FILL, SWT.FILL, true, true);
    data.verticalAlignment = SWT.BEGINNING;
    compUnten.setLayoutData(data);

    Button btnAktivAll = new Button(compUnten, SWT.PUSH);
    btnAktivAll.setText("Alle aktivieren");
    btnAktivAll.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event)
      {
        TableItem[] tia = _tabelleSpalten.getItems();

        for (int i = 0; i < tia.length; i++)
        {
          tia[i].setChecked(true);
          aktionCheck(tia[i]);
        }

      }
    });
    Button btnDeaktivAll = new Button(compUnten, SWT.PUSH);
    btnDeaktivAll.setText(" Alle deaktivieren ");
    btnDeaktivAll.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event)
      {
        TableItem[] tia = _tabelleSpalten.getItems();

        for (int i = 0; i < tia.length; i++)
        {
          tia[i].setChecked(false);
          aktionCheck(tia[i]);
        }
      }
    });
    Button btnSichern = new Button(compUnten, SWT.PUSH);
    btnSichern.setText(" Einstellungen sichern");
    btnSichern.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event)
      {
        if (_part instanceof GraphicalEditorToView)
        {
          GraphicalEditorToView view = (GraphicalEditorToView)_part;
          view.einstellungenSichern();
        }
      }
    });
    Button btnDefault = new Button(compUnten, SWT.PUSH);
    btnDefault.setText(" Default herstellen ");
    btnDefault.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event)
      {
        GraphicalEditorToView view = (GraphicalEditorToView)_part;
        view.restoreFromPrefs();
        
        Object adapter = view.getAdapter(GraphicalViewer.class);
        if (adapter instanceof GraphicalViewer)
        {
          GraphicalViewer viewer = (GraphicalViewer)adapter;
          AnsichtEditPart contents = (AnsichtEditPart)viewer.getContents();
          setContent(contents);
        }
      }
    });
  }

  public TableViewer getTableViewerSpalten()
  {
    return _tableViewerSpalten;
  }

  @Override
  public void setFocus()
  {
    // Fr die F1 Taste
    if (_parent != null)
      _parent.setFocus();

  }

  /**
   * Erzeugt gruppeUebersicht
   * 
   */
  private void createCompositeTop(Composite parent)
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessVerticalSpace = true;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    compositeTop = new Composite(parent, SWT.NONE);
    compositeTop.setLayout(new GridLayout());
    compositeTop.setLayoutData(gridData);
    GridData gridDataTabelle = new GridData();
    gridDataTabelle.horizontalAlignment = GridData.FILL;
    gridDataTabelle.grabExcessHorizontalSpace = true;
    gridDataTabelle.grabExcessVerticalSpace = true;
    gridDataTabelle.verticalAlignment = GridData.FILL;
    GridData gridDataGruppeUebersicht = new GridData();
    gridDataGruppeUebersicht.horizontalAlignment = GridData.FILL;
    gridDataGruppeUebersicht.grabExcessHorizontalSpace = true;
    gridDataGruppeUebersicht.grabExcessVerticalSpace = true;
    gridDataGruppeUebersicht.verticalAlignment = GridData.FILL;
    _gruppeUebersicht = new Group(compositeTop, SWT.NONE);
    _gruppeUebersicht.setLayout(new GridLayout(2, false));
    _gruppeUebersicht.setLayoutData(gridDataGruppeUebersicht);
    _gruppeUebersicht.setText("Auswahl Spalten:");
    _tabelleSpalten = new Table(_gruppeUebersicht, SWT.BORDER | SWT.FULL_SELECTION | SWT.CHECK);
    _tabelleSpalten.setHeaderVisible(true);
    _tabelleSpalten.setLayoutData(gridDataTabelle);
    _tabelleSpalten.setLinesVisible(true);
    _tableViewerSpalten = new TableViewer(_tabelleSpalten);

    TableColumn tcSicht = new TableColumn(_tabelleSpalten, SWT.LEFT);
    tcSicht.setText("Sichtbar");
    tcSicht.setWidth(75);

    TableColumn tcTyp = new TableColumn(_tabelleSpalten, SWT.LEFT);
    tcTyp.setText("Name");
    tcTyp.setWidth(200);

    _tableViewerSpalten.getTable().addListener(SWT.Selection, new Listener() {
      public void handleEvent(Event event)
      {
        if (event.detail == SWT.CHECK)
        {
          TableItem ti = (TableItem)event.item;
          aktionCheck(ti);
        }
      }
    });

    _tableViewerSpalten.addDoubleClickListener(new IDoubleClickListener() {

      @Override
      public void doubleClick(DoubleClickEvent event)
      {
        TableItem[] selection = _tableViewerSpalten.getTable().getSelection();

        if (selection != null)
        {
          for (TableItem ti : selection)
          {
            DarstellungsSpalte e = (DarstellungsSpalte)ti.getData();
            e.setVisible(!e.isVisible());
            ti.setChecked(e.isVisible());
          }
        }
      }
    });

    _tableViewerSpalten.addSelectionChangedListener(new ISelectionChangedListener() {

      @Override
      public void selectionChanged(SelectionChangedEvent event)
      {
        // TODO Auto-generated method stub
        TableItem[] selection = _tableViewerSpalten.getTable().getSelection();
        if (selection != null && selection.length > 0)
        {
          // Selektion merken
          _selSpalte = selection[0];

          // entsprechende Up-Down-Buttons de-/aktivieren
          TableItem[] tia = _tabelleSpalten.getItems();

          if (tia[0].equals(_selSpalte))
            _btnUp.setEnabled(false);
          else
            _btnUp.setEnabled(true);

          if (tia[tia.length - 1].equals(_selSpalte))
            _btnDown.setEnabled(false);
          else
            _btnDown.setEnabled(true);
        }
      }
    });

    Transfer[] types = new Transfer[] { TextTransfer.getInstance() };
    DragSource source = new DragSource(_tableViewerSpalten.getTable(), DND.DROP_MOVE | DND.DROP_COPY);
    source.setTransfer(types);

    source.addDragListener(new DragSourceAdapter() {
      public void dragSetData(DragSourceEvent event)
      {

        DragSource ds = (DragSource)event.widget;
        Table table = (Table)ds.getControl();
        TableItem[] selection = table.getSelection();

        StringBuffer buff = new StringBuffer();
        for (int i = 0, n = selection.length; i < n; i++)
        {
          String text = selection[i].getText(1);
          buff.append(text);
        }
        event.data = buff.toString();
      }

      @Override
      public void dragFinished(DragSourceEvent event)
      {
        if (event.detail == DND.DROP_COPY)
        {
          DragSource ds = (DragSource)event.widget;
          Table table = (Table)ds.getControl();
          TableItem[] items = table.getSelection();
          for (int i = 0; i < items.length; i++)
          {
            items[i].dispose();
          }
          // table.redraw();
        }
      }

    });

    // Create the drop target
    DropTarget target = new DropTarget(_tableViewerSpalten.getTable(), DND.DROP_MOVE | DND.DROP_COPY | DND.DROP_DEFAULT);
    target.setTransfer(types);
    target.addDropListener(new DropTargetAdapter() {
      public void dragEnter(DropTargetEvent event)
      {
        if (event.detail == DND.DROP_DEFAULT)
        {
          event.detail = (event.operations & DND.DROP_COPY) != 0 ? DND.DROP_COPY : DND.DROP_NONE;
        }

        // Allow dropping text only
        for (int i = 0, n = event.dataTypes.length; i < n; i++)
        {
          if (TextTransfer.getInstance().isSupportedType(event.dataTypes[i]))
          {
            event.currentDataType = event.dataTypes[i];
          }
        }
      }

      public void dragOver(DropTargetEvent event)
      {
        event.feedback = DND.FEEDBACK_SELECT | DND.FEEDBACK_SCROLL;
      }

      public void drop(DropTargetEvent event)
      {
        if (TextTransfer.getInstance().isSupportedType(event.currentDataType))
        {
          // Get the dropped data
          DropTarget target = (DropTarget)event.widget;
          Table table = (Table)target.getControl();
          String data = (String)event.data;

          Point p = event.display.map(null, table, event.x, event.y);
          TableItem dropItem = table.getItem(p);
          int index = dropItem == null ? table.getItemCount() : table.indexOf(dropItem);

          TableItem item = new TableItem(table, SWT.NONE, index);
          aktionMove(data, index, item);
          table.redraw();
        }
      }
    });

    erzeugeButtonRechts(_gruppeUebersicht);

  }

  protected void aktionAbbrechen()
  {
    PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().hideView(this);
  }

  /**
   * Zeigt die Hilfe zum Dialog an.
   * 
   */
  private void aktionHilfe()
  {
     String contextId = Activator.PLUGIN_ID + ".auswahlSpalten";
     _debug.fine("ContextID: " + contextId);
     PlatformUI.getWorkbench().getHelpSystem().displayHelp(contextId);
  }

  public void setContent(AnsichtEditPart contents)
  {
    if (_tableViewerSpalten != null && !_tableViewerSpalten.getTable().isDisposed())
      _tableViewerSpalten.getTable().removeAll();

    if (contents != null)
    {
      this._contents = contents;

      Ansicht ansicht = contents.getModel();
      Darstellung darstellung = (Darstellung)ansicht.getDarstellung();
      if (darstellung instanceof StilisierteDarstellung)
      {
        StilisierteDarstellung stilisierteDarstellung = (StilisierteDarstellung)darstellung;
        EList<DarstellungsSpalte> spalten = stilisierteDarstellung.getSpalten();

        if (!_tableViewerSpalten.getTable().isDisposed())
        {
          for (DarstellungsSpalte d : spalten)
          {
            TableItem ti = new TableItem(_tableViewerSpalten.getTable(), SWT.NONE);
            ti.setChecked(d.isVisible());
            ti.setText(1, d.getName());
            ti.setData(d);
          }
        }
      }
    }
    else
    {
//      System.err.println("contents ist null2");
      _tabelleSpalten.removeAll();
    }

  }

  /**
   * Gibt den Command Stack zurck.
   * 
   * @return der Command Stack, niemals <code>null</code>.
   */
  protected CommandStack getCommandStack()
  {
    CommandStack stack = null;
    if (_contents != null)
    {
      stack = _contents.getViewer().getEditDomain().getCommandStack();
      Assert.isNotNull(stack, "Command Stack nicht verfgbar.");
    }
    return stack;
  }

  private void aktionCheck(TableItem ti)
  {
    DarstellungsSpalte spalte = (DarstellungsSpalte)ti.getData();
    if (spalte != null && ti != null)
    {
      boolean visible = ti.getChecked();
      spalte.setVisible(visible);
      EList<DoModel> doObjekte = spalte.getDoObjekte();
      for (DoModel doModel : doObjekte)
      {
        if (doModel instanceof DoModelMitVerbLinie)
        {
          DoModelMitVerbLinie model = (DoModelMitVerbLinie)doModel;
          EStructuralFeature feature = AnlagendarstellungPackage.Literals.DO_MODEL_MIT_VERB_LINIE__TARGET;
          final SetCommand cmd = new SetCommand(model, feature, model.getTarget());
          getCommandStack().execute(cmd); 
        }
      }
    }
  }

  private void aktionMove(String data, int index, TableItem item)
  {
    if (_contents != null)
    {
      AnsichtEditPart contents = (AnsichtEditPart)_contents;

      Ansicht ansicht = contents.getModel();
      StilisierteDarstellung darstellung = (StilisierteDarstellung)ansicht.getDarstellung();

      EList<DarstellungsSpalte> spalten = darstellung.getSpalten();

      DarstellungsSpalte spaltenZumVerschieben = null;

      for (DarstellungsSpalte d : spalten)
      {
        if (d.getName().equals(data))
        {
          item.setData(d);
          item.setChecked(d.isVisible());
          item.setText(1, d.getName());
          spaltenZumVerschieben = d;
          break;
        }
      }

      if (spaltenZumVerschieben != null)
      {
        if (index > 0)
          index--;
        spalten.move(index, spaltenZumVerschieben);
      }
    }
  }
}
