/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.views;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.plugin.anlagendarstellung.Activator;
import de.bsvrz.buv.plugin.anlagendarstellung.daVDataUpdateListener.DaVDataUpdateEvent;
import de.bsvrz.buv.plugin.anlagendarstellung.daVDataUpdateListener.DaVDataUpdateListener;
import de.bsvrz.buv.plugin.anlagendarstellung.daVDataUpdateListener.IDaVDataUpdateListener;
import de.bsvrz.buv.plugin.anlagendarstellung.dav.DaVKommunikationDobj;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.NonMutableSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

/**
 * Sicht zur Anzeige von Detektor-Daten.
 * 
 * @author Pittner
 * 
 */ 
public class DetektorLveView extends ViewPart implements IDaVDataUpdateListener
{

  public static final String ID = "de.bsvrz.buv.plugin.anlagendarstellung.detektorlveanzeige";

  private CTabFolder _tabFolder = null;

  private Composite _parent;

  private Composite _composite = null;

  private IMemento _memento = null;

  private String _secondViewId;

  private ClientDavInterface _connection;

  private TableViewer _tv1;

  private TableViewer _tv2;

  private TableViewer _tv3;

  private Label _l1;

  private Label _l2;

  private Label _l3;

  private SimpleDateFormat _sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss");

  private SystemObject _de;

  private int _selection = 0;

  @Override
  public void createPartControl(Composite parent)
  {
    _parent = parent;

    // Fr die F1 Taste
    PlatformUI.getWorkbench().getHelpSystem().setHelp(_parent, Activator.PLUGIN_ID + ".zuflussParameter");

    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessVerticalSpace = true;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    GridLayout gridLayout = new GridLayout();
    gridLayout.numColumns = 1;
    _composite = new Composite(parent, SWT.NONE);
    _composite.setLayout(gridLayout);
    _composite.setLayoutData(gridData);

    restoreState();

    _de = erzeugeCompositeTop();

    createCompositeTab();
    erzeugeTabItems();

    _tabFolder.setSelection(_selection);

    erzeugeDAVanmeldeDaten(_de);
  }

  private SystemObject erzeugeCompositeTop()
  {
    _connection = DaVVerbindung.getInstanz().getDaVVerbindung();
    SystemObject so = _connection.getDataModel().getObject(_secondViewId);

    ConfigurationObject co = (ConfigurationObject)so;
    AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.fahrStreifen");
    Data data = co.getConfigurationData(atg);

    SystemObject de = data.getItem("FahrStreifenQuelle").asReferenceValue().getSystemObject();

    co = (ConfigurationObject)de;
    atg = _connection.getDataModel().getAttributeGroup("atg.de");
    data = co.getConfigurationData(atg);
    String deKanal = data.getItem("DEKanal").asTextValue().getValueText();

    setPartName(so.getName());

    Font boldFont = JFaceResources.getFontRegistry().getBold(JFaceResources.DEFAULT_FONT);

    Composite c01 = new Composite(_composite, SWT.NONE);
    c01.setLayout(new GridLayout(2, false));
    c01.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));

    SystemObject sm = bestimmeSmVonDe(de);

    Label l11 = new Label(c01, SWT.NONE);
    l11.setText("KnNr / DE : ");
    l11.setFont(boldFont);
    co = (ConfigurationObject)sm;
    atg = _connection.getDataModel().getAttributeGroup("atg.gert");
    data = co.getConfigurationData(atg);
    // String nummer = data.getItem("KnotenNummer").asTextValue().getValueText();

    String nummer = "";

    long l = data.getItem("KnotenNummer").asUnscaledValue().longValue();
    if (l > 0)
    {
      Long l1 = l / 256L;
      Long l2 = l % 256L;

      nummer = l1 + " - " + l2;
    }

    Label l12 = new Label(c01, SWT.NONE);
    if (nummer != null)
    {
      l12.setText(nummer + " / " + deKanal);
    }  

    Label l01 = new Label(c01, SWT.NONE);
    l01.setText("Name        : ");
    l01.setFont(boldFont);
    Label l02 = new Label(c01, SWT.NONE);
    if (de != null)
    {
      l02.setText(de.getName());
    }
    else
    {
      l02.setText("");
    }
    return de;
  }

  private void erzeugeTabelle1(Composite parent)
  {

    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessVerticalSpace = true;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.FILL;

    _tv1 = new TableViewer(parent, SWT.HIDE_SELECTION);

    TableViewerColumn column1 = new TableViewerColumn(_tv1, SWT.NONE);
    column1.getColumn().setWidth(100);
    column1.getColumn().setText("Attribut");

    TableViewerColumn column2 = new TableViewerColumn(_tv1, SWT.NONE);
    column2.getColumn().setWidth(100);
    column2.getColumn().setText("Ein");

    _tv1.setContentProvider(new ArrayContentProvider());
    _tv1.setLabelProvider(new TableLabelProvider());
    _tv1.getTable().setLayoutData(gridData);
    _tv1.getTable().setLinesVisible(true);
    _tv1.getTable().setHeaderVisible(false);
  }

  private void erzeugeTabelle2(Composite parent)
  {

    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessVerticalSpace = true;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.FILL;

    _tv2 = new TableViewer(parent, SWT.HIDE_SELECTION);

    TableViewerColumn column1 = new TableViewerColumn(_tv2, SWT.NONE);
    column1.getColumn().setWidth(100);
    column1.getColumn().setText("1");

    TableViewerColumn column2 = new TableViewerColumn(_tv2, SWT.NONE);
    column2.getColumn().setWidth(100);
    column2.getColumn().setText("2");

    _tv2.setContentProvider(new ArrayContentProvider());
    _tv2.setLabelProvider(new TableLabelProvider());
    _tv2.getTable().setLayoutData(gridData);
    _tv2.getTable().setLinesVisible(true);
    _tv2.getTable().setHeaderVisible(false);

  }

  private void erzeugeTabelle3(Composite parent)
  {

    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessVerticalSpace = true;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.FILL;

    _tv3 = new TableViewer(parent, SWT.HIDE_SELECTION);

    TableViewerColumn column1 = new TableViewerColumn(_tv3, SWT.NONE);
    column1.getColumn().setWidth(100);
    column1.getColumn().setText("Attribut");

    TableViewerColumn column2 = new TableViewerColumn(_tv3, SWT.NONE);
    column2.getColumn().setWidth(100);
    column2.getColumn().setText("Ein");

    _tv3.setContentProvider(new ArrayContentProvider());
    _tv3.setLabelProvider(new TableLabelProvider());
    _tv3.getTable().setLayoutData(gridData);
    _tv3.getTable().setLinesVisible(true);
    _tv3.getTable().setHeaderVisible(false);

  }

  public class TableLabelProvider implements ITableLabelProvider
  {

    @Override
    public Image getColumnImage(Object element, int columnIndex)
    {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public String getColumnText(Object element, int columnIndex)
    {

      String returnString = "undefiniert";

      Data data = (Data)element;

      switch (columnIndex)
      {
        case 0:
          returnString = data.getName();
          break;
        case 1:
          returnString = data.asTextValue().getText();
          break;
        default:
      }
      return returnString;
    }

    @Override
    public void addListener(ILabelProviderListener listener)
    {
      // TODO Auto-generated method stub

    }

    @Override
    public void dispose()
    {
      // TODO Auto-generated method stub

    }

    @Override
    public boolean isLabelProperty(Object element, String property)
    {
      // TODO Auto-generated method stub
      return false;
    }

    @Override
    public void removeListener(ILabelProviderListener listener)
    {
      // TODO Auto-generated method stub

    }
  }

  /**
   * Holt das zu einem EAK gehrende Steuermodul
   * 
   * @param de
   *          SystemObjekt des EAK zu dem das SM bestimmt wird
   * @return SystemObjekt das SM welches bestimmt wurde
   */
  private SystemObject bestimmeSmVonEak(SystemObject eak)
  {
    if (eak == null)
      throw new IllegalArgumentException();

    SystemObjectType typeSm = _connection.getDataModel().getType("typ.steuerModul");
    List<SystemObject> sms = typeSm.getElements();
    for (SystemObject sm : sms)
    {
      ConfigurationObject csm = (ConfigurationObject)sm;
      NonMutableSet set2 = csm.getNonMutableSet("Eak");
      List<SystemObject> listeEAK = set2.getElements();

      for (SystemObject soEAK : listeEAK)
      {
        if (soEAK.equals(eak))
        {
          return sm;
        }
      }
    }
    return null;
  }

  /**
   * Holt das zu einer De gehrende Steuermodul
   * 
   * @param de
   *          SystemObjekt der De zu der die Uz bestimmt wird
   * @return SystemObjekt der Uz die bestimmt wurde
   */
  private SystemObject bestimmeSmVonDe(SystemObject de)
  {
    if (de == null)
      throw new IllegalArgumentException();

    SystemObjectType typeEak = _connection.getDataModel().getType("typ.eak");
    List<SystemObject> eaks = typeEak.getElements();
    for (SystemObject eak : eaks)
    {
      ConfigurationObject ceak = (ConfigurationObject)eak;
      NonMutableSet set1 = ceak.getNonMutableSet("De");

      if (set1 != null)
      {
        List<SystemObject> listeDE = set1.getElements();

        for (SystemObject soDE : listeDE)
        {
          if (soDE.equals(de))
          {
            return bestimmeSmVonEak(eak);
          }
        }
      }
    }
    return null;
  }

  private void erzeugeDAVanmeldeDaten(SystemObject systemObject)
  {

    DaVDataUpdateListener.getInstanz().addDaVDataUpdateListener(this);

    String pidAtg1 = "atg.tlsLveErgebnisMeldungVersion0Bis4";
    String pidAtg2 = "atg.tlsLveBetriebsParameter";
    String pidAtg3 = "atg.tlsGloDeFehler";

    List<String> soList = new ArrayList<String>();

    soList.add(systemObject.getPid());

    DaVKommunikationDobj.getInstanz().anmeldenAufDatenMQDaten(soList, pidAtg1, "asp.tlsAntwort");
    DaVKommunikationDobj.getInstanz().anmeldenAufDatenMQDaten(soList, pidAtg2, "asp.parameterSoll");
    DaVKommunikationDobj.getInstanz().anmeldenAufDatenMQDaten(soList, pidAtg3, "asp.tlsAntwort");
  }

  @Override
  public void setFocus()
  {
    // Fr die F1 Taste
    if (_parent != null)
      _parent.setFocus();
  }

  /**
   * This method initializes _composite
   * 
   */
  private void createCompositeTab()
  {
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.FILL;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.grabExcessVerticalSpace = true;
    gridData3.verticalAlignment = GridData.FILL;

    _tabFolder = new CTabFolder(_composite, SWT.NONE);
    _tabFolder.setLayout(new GridLayout());
    _tabFolder.setLayoutData(gridData3);
  }

  public void erzeugeTabItems()
  {

    CTabItem tabItem = new CTabItem(_tabFolder, SWT.NONE);
    tabItem.setText("Verkehrserfassung");
    tabItem.setData("atg.tlsLveErgebnisMeldungVersion0Bis4");

    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    gridData.verticalAlignment = GridData.FILL;

    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;

    Composite c1 = new Composite(_tabFolder, SWT.NONE);
    c1.setLayout(new GridLayout());
    c1.setLayoutData(gridData);

    _l1 = new Label(c1, SWT.NONE);
    _l1.setLayoutData(gridData2);
    _l1.setText("--.--.---- --:--:--");

    erzeugeTabelle1(c1);
    tabItem.setControl(c1);

    CTabItem tabItem2 = new CTabItem(_tabFolder, SWT.NONE);
    tabItem2.setText("Betriebsparameter");
    tabItem2.setData("atg.tlsLveBetriebsParameter");

    Composite c2 = new Composite(_tabFolder, SWT.NONE);
    c2.setLayout(new GridLayout());
    c2.setLayoutData(gridData);

    _l2 = new Label(c2, SWT.NONE);
    _l2.setLayoutData(gridData2);
    _l2.setText("--.--.---- --:--:--");

    erzeugeTabelle2(c2);

    tabItem2.setControl(c2);

    CTabItem tabItem3 = new CTabItem(_tabFolder, SWT.NONE);
    tabItem3.setText("DE-Fehler");
    tabItem3.setData("atg.tlsGloDeFehler");

    Composite c3 = new Composite(_tabFolder, SWT.NONE);
    c3.setLayout(new GridLayout());
    c3.setLayoutData(gridData);

    _l3 = new Label(c3, SWT.NONE);
    _l3.setLayoutData(gridData2);
    _l3.setText("--.--.---- --:--:--");

    erzeugeTabelle3(c3);
    tabItem3.setControl(c3);

    _tabFolder.setSelection(0);
  }

  @Override
  public void init(IViewSite site, IMemento memento) throws PartInitException
  {
    _memento = memento;
    _secondViewId = site.getSecondaryId();
    super.init(site, memento);
  }

  private void restoreState()
  {

    if (_memento == null)
    {
      return;
    }

    _secondViewId = _memento.getString("pidDetektorLve");
    _selection = _memento.getInteger("selektion");

  }

  @Override
  public void saveState(IMemento memento)
  {
    memento.putString("pidDetektorLve", _secondViewId);

    memento.putInteger("selektion", _tabFolder.getSelectionIndex());
    super.saveState(memento);
  }

  @Override
  public void daVDataUpdate(DaVDataUpdateEvent e)
  {
    if (e != null && e.getResult().getObject().equals(_de))
    {
      verarbeiteDavDaten(e);
    }
  }

  /**
   * @param e
   */
  private void verarbeiteDavDaten(DaVDataUpdateEvent e)
  {
    List<Data> list = new ArrayList<Data>();
    String pidAtg = e.getResult().getDataDescription().getAttributeGroup().getPid();
    String time = _sdf.format(new Date(e.getResult().getDataTime())) + " Uhr";

    Data data = e.getResult().getData();
    if (data != null)
    {
      Iterator it = data.iterator();
      while (it.hasNext())
      {
        Data plainData = (Data)it.next();
        if (plainData.isPlain())
        {
          list.add(plainData);
        }
      }
      if (pidAtg.equals("atg.tlsLveErgebnisMeldungVersion0Bis4"))
      {
        _l1.setText(time);
        _tv1.setInput(list);
        TableColumn[] columns = _tv1.getTable().getColumns();
        for (TableColumn tc : columns)
        {
          tc.pack();
        }
      }
      else if (pidAtg.equals("atg.tlsLveBetriebsParameter"))
      {
        _l2.setText(time);
        _tv2.setInput(list);
        TableColumn[] columns = _tv2.getTable().getColumns();
        for (TableColumn tc : columns)
        {
          tc.pack();
        }
      }
      else if (pidAtg.equals("atg.tlsGloDeFehler"))
      {
        _l3.setText(time);
        _tv3.setInput(list);
        TableColumn[] columns = _tv3.getTable().getColumns();
        for (TableColumn tc : columns)
        {
          tc.pack();
        }
      }
    }
    else
    {
      if (pidAtg.equals("atg.tlsLveErgebnisMeldungVersion0Bis4"))
      {
        _l1.setText(time + " (keine Daten)");
        _tv1.setInput(new ArrayList<Data>());
      }
      else if (pidAtg.equals("atg.tlsLveBetriebsParameter"))
      {
        _l2.setText(time + " (keine Daten)");
        _tv2.setInput(list);
      }
      else if (pidAtg.equals("atg.tlsGloDeFehler"))
      {
        _l3.setText(time + " (keine Daten)");
        _tv3.setInput(list);
      }
    }

  }

  @Override
  public void dispose()
  {
    // super.dispose();
    DaVDataUpdateListener.getInstanz().removeDaVDataUpdateListener(this);
    IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
    if (page != null)
    {
      page.hideView(this);
    }
  }
}
