/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.views;

/**
 * Liefert die Datenstruktur fr die Sicht {@link UmfeldDatenTabelleView}. <br>
 * 
 * @author Pittner
 * 
 */
public class UmfeldDatenTabelleDaten
{
  public static String KEINE_DATEN = "keine Daten";

  public static String KEINE_QUELLE = "keine Quelle";

  public static String UNDEFINERT = "---";

  private String _wert = null;

  private String _stufe = null;

  private Long _wertZeit = null;

  private Long _stufeZeit = null;

  private int _index = -1;

  private boolean _stufeVorhanden = false;

  private boolean _keinWertVorhanden = false;;

  public UmfeldDatenTabelleDaten()
  {
    _wert = UmfeldDatenTabelleDaten.UNDEFINERT;
    _stufe = UmfeldDatenTabelleDaten.UNDEFINERT;
    _wertZeit = 0L;
    _stufeZeit = 0L;
  }

  public String getWert()
  {
    return _wert;
  }

  public void setWert(String wert)
  {
    _wert = wert;
  }

  public String getStufe()
  {
    return _stufe;
  }

  public void setStufe(String stufe)
  {
    _stufe = stufe;
  }

  public boolean hasDaten()
  {
    boolean hasDaten = false;

    if (!isKeinWertVorhanden())
    {
      if (isStufeVorhanden())
      {
        if (_wert != null && !(_stufe.equals(UNDEFINERT) || _wert.equals(KEINE_DATEN) || _wert.equals(KEINE_QUELLE)))
          hasDaten = true;
        else
        {
          if (_wert != null && !(_wert.equals(KEINE_DATEN) || _wert.equals(KEINE_QUELLE)))
            hasDaten = true;
        }
      }
    }
    else if (isStufeVorhanden())
    {
      if (_stufe != null && !(_stufe.equals(UNDEFINERT) || _stufe.equals(KEINE_DATEN) || _stufe.equals(KEINE_QUELLE)))
        hasDaten = true;
    }
    return hasDaten;
  }

  public boolean hasVollstaendigeDaten()
  {
    boolean hasDatenWert = false;
    boolean hasDatenStufe = false;

    if (!isKeinWertVorhanden())
    {
      if (_wert != null && !(_wert.equals(UNDEFINERT) || _wert.equals(KEINE_DATEN) || _wert.equals(KEINE_QUELLE)))
        hasDatenWert = true;
    }
    else
      hasDatenWert = true;

    if (isStufeVorhanden())
    {
      if (!isKeinWertVorhanden())
      {
        if (_stufe != null && !(_wert.equals(UNDEFINERT) || _stufe.equals(KEINE_DATEN) || _stufe.equals(KEINE_QUELLE)))
          hasDatenStufe = true;
      }
      else
      {
        if (_stufe != null && !(_stufe.equals(KEINE_DATEN) || _stufe.equals(KEINE_QUELLE)))
          hasDatenStufe = true;
      }

    }
    else
      hasDatenStufe = true;

    return hasDatenWert && hasDatenStufe;
  }

  @Override
  public String toString()
  {
    String text = "";
    if (isStufeVorhanden())
    {
      if (isKeinWertVorhanden())
      {
        if (_stufe != null)
          text = " \n" + _stufe;
        else
          text = " \n ";
      }
      else
      {
        if (_wert != null && _stufe != null)
          text = _wert + "\n" + _stufe;
        else if (_wert != null && _stufe == null)
          text = _wert + "\n ";
        else if (_wert == null && _stufe != null)
          text = " \n" + _stufe;
        else
          text = " \n ";
      }
    }
    else
    {
      if (_wert != null)
        text = _wert;
      else
        text = " \n ";
    }
    return text;
  }

  public int getIndex()
  {
    return _index;
  }

  public void setIndex(int index)
  {
    index = _index;
  }

  public Long getWertZeit()
  {
    return _wertZeit;
  }

  public void setWertZeit(Long wertZeit)
  {
    _wertZeit = wertZeit;
  }

  public Long getStufeZeit()
  {
    return _stufeZeit;
  }

  public void setStufeZeit(Long stufeZeit)
  {
    _stufeZeit = stufeZeit;
  }

  public void setStufeVorhanden(boolean stufeVorhanden)
  {
    _stufeVorhanden = stufeVorhanden;
  }

  public void setKeinWertVorhanden(boolean keinWertVorhanden)
  {
    _keinWertVorhanden = keinWertVorhanden;
  }

  public boolean isStufeVorhanden()
  {
    return _stufeVorhanden;
  }

  public boolean isKeinWertVorhanden()
  {
    return _keinWertVorhanden;
  }
}
