/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/*
 * Rahmenwerk-Plug-in "Baustellen- und Unfalleingabe"
 * Copyright (C) 2009 BitCtrl Systems GmbH 
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */
package de.bsvrz.buv.plugin.baueditor.sts.actions;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IActionDelegate2;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IViewActionDelegate;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.plugin.baueditor.sts.BauEditorPlugin;
import de.bsvrz.buv.plugin.baueditor.sts.EngstellenVerwaltungPluginFunktion;
import de.bsvrz.buv.plugin.baueditor.sts.FunktionsBerechtigungsListener;
import de.bsvrz.buv.plugin.baueditor.sts.RahmenwerkSelektionsVerteiler;

/**
 * Spezielle Aktion, die im Rahmenwerk sichtbar ist und ueber den
 * <code>RahmenwerkSelektionsVerteiler</code> separat mit Selections von einem
 * dort registrierten <code>ISelectionProvider</code> versorgt werden kann.<br>
 * <b>Achtung:</b> Hierbei handelt es sich um einen Workaround, bis das
 * Rahmenwerk selbst die Aktionen von registrierten
 * <code>ISelectionProvider</code>n an Menues und Toolbars weiterreicht.
 * 
 * @author BitCtrl Systems GmbH, Thomas Thierfelder, Andreas Meiner
 * @version $Id: EngstellenVerwaltungPluginAktion.java,v 1.2 2014/11/14 07:29:41 Pittner Exp $
 */
public abstract class EngstellenVerwaltungPluginAktion extends Action implements
		IObjectActionDelegate, IActionDelegate2, IViewActionDelegate,
		FunktionsBerechtigungsListener, IWorkbenchWindowActionDelegate {

	/**
	 * Die Aktion, die von dem Delegate tatschlich ausgefhrt wird.
	 */
	private IAction ausgefuehrteAktion;

	/**
	 * Die mit dieser Aktion asoziierte Funktion.
	 */
	private final EngstellenVerwaltungPluginFunktion funktion;

	/**
	 * Aktuell ausgewhltes/selektiertes Element.
	 */
	private Object selektiertesElement;

	/**
	 * Aktueller Ziel-WorkbenchPart.
	 */
	private IWorkbenchPart workbenchPart;

	/**
	 * Konstruktor.
	 * 
	 * @param name
	 *            der Name der Aktion, wird fr den Namen sowie den Tooltip
	 *            verwandt.
	 * @param imageDescriptor
	 *            der ImageDescriptor
	 * @param funktion
	 *            die mit dieser Aktion asoziierte Funktion, nicht
	 *            <code>null</code>
	 */
	public EngstellenVerwaltungPluginAktion(final String name,
			final ImageDescriptor imageDescriptor,
			final EngstellenVerwaltungPluginFunktion funktion) {
		super(name);

		this.funktion = funktion;

		setToolTipText(name);
		setId(this.getClass().getName());
		setImageDescriptor(imageDescriptor);
		RahmenwerkSelektionsVerteiler.addRahmenwerkAktion(this);
		funktion.addFunktionsBerechtigungsListener(this);
		setEnabled(funktion.isFreigegeben());
	}

	@Override
	public void init(final IWorkbenchWindow window) {
		// tut nix
	}

	@Override
	public void setActivePart(final IAction action,
			final IWorkbenchPart targetPart) {
		workbenchPart = targetPart;
	}

	@Override
	public void dispose() {
		funktion.removeFunktionsBerechtigungsListener(this);
	}

	@Override
	public void run(final IAction action) {
		runWithEvent(null);
	}

	@Override
	public void runWithEvent(final Event event) {
		runWithEvent(event, selektiertesElement, workbenchPart);
	}

	/**
	 * Subklassen mssen diese Methode implementieren.
	 * 
	 * @param event
	 *            das Event
	 * @param element
	 *            das aktuell ausgewhlte/selektierte Element
	 * @param targetPart
	 *            das akutelle Ziel-WorkbenchPart
	 * 
	 * @return den Status dieser Methode,
	 *         {@link org.eclipse.core.runtime.Status#OK_STATUS} oder
	 *         {@link org.eclipse.core.runtime.Status#CANCEL_STATUS}
	 */
	public abstract IStatus runWithEvent(Event event, Object element,
			IWorkbenchPart targetPart);

	@Override
	public void berechtigungsAenderung() {
		setEnabled(ausgefuehrteAktion, isEnabled());
	}

	@Override
	public void init(final IAction action) {
		ausgefuehrteAktion = action;
	}

	@Override
	public void init(final IViewPart viewPart) {
		workbenchPart = viewPart;
	}

	@Override
	public void runWithEvent(final IAction action, final Event event) {
		runWithEvent(event);
	}

	@Override
	public void selectionChanged(final IAction action,
			final ISelection selection) {
		if (selection instanceof IStructuredSelection) {
			selektiertesElement = ((IStructuredSelection) selection)
					.getFirstElement();
		}
		setEnabled(action, isEnabled());
	}

	/**
	 * Setzt den Enable-Zustand dieser Aktion.
	 * 
	 * @param action
	 *            die Workbenchaktion, die mit dieser Aktion assoziiert ist oder
	 *            <code>null</code>, wenn es sich um eine Aktion handelt, die
	 *            nur im Rahmenwerk registriert ist.
	 * @param enabled
	 *            enabled?
	 */
	void setEnabled(final IAction action, final boolean enabled) {
		if (action == null) {
			this.setEnabled(enabled);
		} else {
			action.setEnabled(enabled);
		}
	}

	@Override
	public final boolean isEnabled() {
		return funktion.isFreigegeben() && isEnabled(selektiertesElement);
	}

	/**
	 * Liefert <code>true</code> zurck, wenn diese Aktion fr das selektierte
	 * Element ausfhrbar sein soll, ansonsten <code>false</code>.
	 * 
	 * @param element
	 *            das aktuell ausgewhlte/selektierte Element
	 * 
	 * @return <code>true</code>, wenn diese Aktion fr das selektierte Element
	 *         ausfhrbar sein soll, ansonsten <code>false</code>
	 */
	protected boolean isEnabled(final Object element) {
		return true;
	}

	/**
	 * Liefert die aktuelle Shell zurck.
	 * 
	 * @return die aktuelle Shell
	 */
	protected Shell getShell() {
		if (workbenchPart != null) {
			return workbenchPart.getSite().getShell();
		}
		return PlatformUI.getWorkbench().getDisplay().getActiveShell();
	}

	/**
	 * Hilfsmethode, welche einen Editor mit der gegebenen ID und dem gegebenen
	 * EditorInput ffnet.
	 * 
	 * @param editorInput
	 *            der EditorInput
	 * @param editorId
	 *            die Editor-ID
	 * @param wbPart
	 *            der aktuelle WorkbenchPart
	 */
	protected void oeffneEditor(final IEditorInput editorInput,
			final String editorId, final IWorkbenchPart wbPart) {
		try {
			getActivePage(wbPart).openEditor(editorInput, editorId);
		} catch (final PartInitException e) {
			BauEditorPlugin.getDefault().getLogger()
					.warning("Editor konnte nicht geffnet werden.", e);
			MessageDialog.openInformation(wbPart.getSite().getShell(),
					"Information",
					"Es sind Probleme beim ffnen des Editors aufgetreten."
							+ "\n\nBitte versuchen Sie es erneut.");
		}
	}

	/**
	 * Liefert die aktive {@link IWorkbenchPage} zurck.
	 * 
	 * @param wbPart
	 *            der {@link org.eclipse.ui.part.WorkbenchPart} auf dem diese
	 *            Aktion ausgefhrt wurde, oder <code>null</code>
	 * 
	 * @return die aktive {@link IWorkbenchPage}
	 */
	private IWorkbenchPage getActivePage(final IWorkbenchPart wbPart) {
		if (wbPart != null) {
			wbPart.getSite().getPage();
		}
		return PlatformUI.getWorkbench().getActiveWorkbenchWindow()
				.getActivePage();
	}
}
