/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/*
 * Rahmenwerk-Plug-in "Baustellen- und Unfalleingabe"
 * Copyright (C) 2009 BitCtrl Systems GmbH 
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */
package de.bsvrz.buv.plugin.baueditor.sts.dialogs;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.layout.GridLayoutFactory;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.sys.funclib.bitctrl.modell.att.Zeitstempel;
import de.bsvrz.sys.funclib.bitctrl.modell.tmereigniskalenderglobal.objekte.Ereignis;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.Baustelle;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.Situation;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.Unfall;

/**
 * Dialog zum Besttigen einer Lschung einer Engstelle.
 * 
 * @author BitCtrl Systems GmbH, Andreas Meissner
 * @version $Id: BestaetigeLoeschungDialog.java,v 1.2 2014/11/14 07:29:43 Pittner Exp $
 */
public class BestaetigeLoeschungDialog extends Dialog {

	/** Die zu lschende Engstelle. */
	private final Situation situation;

	/**
	 * Checkbox zur Definition, ob das dazughrige Ereignis mitgelscht werden
	 * soll.
	 */
	private Button loescheEreignisButton;

	/** Image: Warnung. */
	private final Image warningImage;

	/**
	 * Boolean, welcher angibt, ob im Dialog vor dem Lschen eines dazugehrigen
	 * Ereignisses gewarnt werden soll. Es wird gewarnt, wenn die zeitliche
	 * Gltigkeit des Ereignisse in der Vergangenheit liegt.
	 */
	private boolean warneVorLoeschung;

	/**
	 * Boolean, welcher angibt, ob das dazugehrige Ereignis gelscht werden
	 * soll.
	 */
	private boolean loescheEreignis;

	/**
	 * Der Konstruktor.
	 * 
	 * @param shell
	 *            die Shell
	 * @param situation
	 *            die zu loeschende Situation
	 * @param ereignis
	 *            das zur Situation gehrende Ereignis
	 */
	public BestaetigeLoeschungDialog(final Shell shell,
			final Situation situation, final Ereignis ereignis) {
		super(shell);

		warningImage = PlatformUI.getWorkbench().getSharedImages().getImage(
				ISharedImages.IMG_OBJS_WARN_TSK);
		this.situation = situation;
		loescheEreignis = false;

		final Zeitstempel beginnZeitlicheGueltigkeit = ereignis
				.getPdEreignisParameter().getDatum()
				.getBeginnZeitlicheGueltigkeit();
		warneVorLoeschung = false;
		if (beginnZeitlicheGueltigkeit != null) {
			warneVorLoeschung = beginnZeitlicheGueltigkeit.getTime() < System
					.currentTimeMillis();
		}
	}

	@Override
	protected Control createDialogArea(final Composite parent) {

		// Composite
		final Composite composite = (Composite) super.createDialogArea(parent);
		GridLayoutFactory.fillDefaults().numColumns(2).equalWidth(false)
				.extendedMargins(10, 15, 10, 10).spacing(5, 5).applyTo(
						composite);

		// Frage Image
		final Label logo = new Label(composite, SWT.NONE);
		logo.setImage(parent.getDisplay().getSystemImage(SWT.ICON_QUESTION));
		GridDataFactory.fillDefaults().align(SWT.BEGINNING, SWT.BEGINNING)
				.applyTo(logo);

		// Main composite
		final Composite main = new Composite(composite, SWT.NONE);
		GridLayoutFactory.fillDefaults().numColumns(8).equalWidth(false)
				.applyTo(main);
		GridDataFactory.fillDefaults().indent(8, 0).applyTo(main);

		// Frage Label
		final Label frageLabel = new Label(main, SWT.NONE);
		frageLabel
				.setText("Soll die ausgewhlte Engstelle tatschlich gelscht werden?\t\t");
		GridDataFactory.fillDefaults().span(8, 1).applyTo(frageLabel);

		final Label dummyLabel = new Label(main, SWT.NONE);
		dummyLabel.setText(""); //$NON-NLS-1$
		GridDataFactory.fillDefaults().grab(true, false);

		// Loesche Ereignis - Checkbox
		loescheEreignisButton = new Button(composite, SWT.CHECK);
		loescheEreignisButton.setText("Lsche dazugehriges Ereignis");
		loescheEreignisButton.setSelection(!warneVorLoeschung);
		GridDataFactory.fillDefaults().span(2, 1).indent(60, 0).applyTo(
				loescheEreignisButton);

		// Warnung Composite
		if (warneVorLoeschung) {
			final Composite warningComposite = new Composite(composite,
					SWT.NONE);
			GridLayoutFactory.fillDefaults().spacing(0, 0).applyTo(
					warningComposite);
			GridDataFactory.fillDefaults().span(2, 1).indent(20, 0).grab(true,
					false).applyTo(warningComposite);

			final CLabel warningLabel = new CLabel(warningComposite, SWT.NONE);
			warningLabel.setImage(warningImage);
			warningLabel.setForeground(ColorConstants.red);
			warningLabel
					.setText("Das Lschen eines Ereignisses in der Vergangenheit\n"
							+ "kann zu Inkonsistenzen fhren.");
		}

		return composite;
	}

	@Override
	protected void okPressed() {
		loescheEreignis = loescheEreignisButton.getSelection();
		super.okPressed();
	}

	/**
	 * Liefert <code>true</code> zurck, wenn neben der Engstelle auch das
	 * dazugehrige Ereignis gelscht werden soll.
	 * 
	 * @return <code>true</code>, wenn neben der Engstelle auch das dazugehrige
	 *         Ereignis gelscht werden soll
	 */
	public boolean loescheEreignis() {
		return loescheEreignis;
	}

	@Override
	protected void configureShell(final Shell newShell) {
		super.configureShell(newShell);
		if (situation instanceof Baustelle) {
			newShell.setText("Baustelle lschen");
		} else if (situation instanceof Unfall) {
			newShell.setText("Unfall lschen");
		}
	}
}
