/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/*
 * Rahmenwerk-Plug-in "Baustellen- und Unfalleingabe"
 * Copyright (C) 2009 BitCtrl Systems GmbH 
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */
package de.bsvrz.buv.plugin.baueditor.sts.editors;

import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;
import org.eclipse.ui.forms.widgets.TableWrapData;

import de.bsvrz.buv.rw.bitctrl.startup.StartupPlugin;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.Situation;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.Unfall;

/**
 * Der Unfalleditor.
 * 
 * @author BitCtrl Systems GmbH, Andreas Meissner
 * @version $Id: UnfallEditor.java,v 1.2 2014/11/14 07:29:40 Pittner Exp $
 */
public class UnfallEditor extends EngstellenEditor {

	/**
	 * Die eindeutige ID des Unfalleditors.
	 */
	public static final String ID = "de.bsvrz.buv.plugin.baueditor.editor.unfall"; //$NON-NLS-1$

	@Override
	public void init(final IEditorSite site, final IEditorInput editorInput)
			throws PartInitException {

		if (!(editorInput instanceof UnfallEditorInput)) {
			throw new PartInitException(
					"BaustellenEditor muss ein UnfallEditorInput bergeben werden.");
		}

		super.init(site, editorInput);
	}

	@Override
	protected void erzeugeKopfBereich(final FormToolkit toolkit,
			final Composite parent) {

		// Name Textfeld
		final Text nameText = erzeugeNamensTextFeld(toolkit, parent);
		nameText.addFocusListener(getFocusListener(null));
		nameText.setLayoutData(new TableWrapData(TableWrapData.FILL_GRAB,
				TableWrapData.FILL, 1, 4));

		// Dummy Label
		final Label dummyLabel = toolkit.createLabel(parent, "    ");
		dummyLabel.setBackground(null);

		// Ereignis-Definition
		final Composite ereignisComposite = erzeugeEreignisKopfbereichZeile(
				toolkit, parent);
		ereignisComposite.setLayoutData(new TableWrapData(
				TableWrapData.FILL_GRAB, TableWrapData.FILL, 1, 3));
	}

	@Override
	protected void erzeugeSections(final FormToolkit toolkit,
			final Composite composite) {

		final Section lageSection = erzeugeLageSection(toolkit, composite);
		lageSection.setLayoutData(new TableWrapData(TableWrapData.FILL_GRAB,
				TableWrapData.FILL, 1, 2));

		final Control segmenteControl = erzeugeSegmenteSection(toolkit,
				composite);
		segmenteControl.setLayoutData(new TableWrapData(TableWrapData.RIGHT,
				TableWrapData.FILL_GRAB, 3, 1));

		final Section dauerSection = erzeugeDauerSection(toolkit, composite);
		dauerSection.setLayoutData(new TableWrapData(TableWrapData.FILL,
				TableWrapData.FILL));

		final Section fahrstreifenSection = erzeugeFahrstreifenSection(toolkit,
				composite);
		fahrstreifenSection.setLayoutData(new TableWrapData(
				TableWrapData.FILL_GRAB, TableWrapData.FILL));

		final Section infoSection = erzeugeInfoSection(toolkit, composite);
		infoSection.setLayoutData(new TableWrapData(TableWrapData.FILL_GRAB,
				TableWrapData.FILL, 1, 2));
	}

	@Override
	protected String getLoeschInfoText(final String nameEngstelle) {
		return "Der Unfall '"
				+ nameEngstelle
				+ "' wurde im Datenverteiler gelscht, sodass der dazugehrige Editor geschlossen werden muss.";
	}

	@Override
	protected String getPIDPraefix() {
		return "unfall."; //$NON-NLS-1$
	}

	@Override
	protected Class<? extends Situation> getEngstellenClass() {
		return Unfall.class;
	}

	@Override
	protected boolean fuegeZuMengeHinzu(final Situation situation) {
		assert situation instanceof Unfall;
		return StartupPlugin.getDefault().getDefaultNetz().getUnfaelle().add(
				(Unfall) situation);
	}

	@Override
	protected String getEreignisBeschreibung(final String nameEngstelle) {
		return "Ereignis zum Unfall '" + nameEngstelle + "'";
	}
}
