/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.bsvrz.buv.plugin.parameter.tsf.ablauf;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.gef.editparts.ScalableRootEditPart;
import org.eclipse.gef.editparts.ZoomManager;
import org.eclipse.gef.ui.parts.ScrollingGraphicalViewer;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchPartSite;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.dialogs.ElementListSelectionDialog;
import org.eclipse.ui.internal.ActionDescriptor;
import org.eclipse.ui.internal.PluginAction;
import org.eclipse.ui.internal.registry.IWorkbenchRegistryConstants;

import com.bitctrl.lib.eclipse.emf.gef.commands.SetCommand;
import com.sun.xml.internal.txw2.IllegalAnnotationException;

import de.bsvrz.buv.plugin.anlagendarstellung.AnlagendarstellungPackage;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeQuerschnittModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.VerhaltenDoTyp;
import de.bsvrz.buv.plugin.anlagendarstellung.editor.GraphicalEditorToView;
import de.bsvrz.buv.plugin.anlagendarstellung.einstellungen.SbaAnsichtProvider;
import de.bsvrz.buv.plugin.anlagendarstellung.model.AnzeigeInhaltTyp;
import de.bsvrz.buv.plugin.anlagendarstellung.preferences.DefsPreference;
import de.bsvrz.buv.plugin.anlagendarstellung.util.DaVHelper;
import de.bsvrz.buv.plugin.anlagendarstellung.util.DoModelHelper;
import de.bsvrz.buv.plugin.anlagendarstellung.views.SbaScalableRootEditPart;
import de.bsvrz.buv.plugin.darstellung.editparts.AnsichtEditPart;
import de.bsvrz.buv.plugin.darstellung.editparts.DarstellungEditPartFactory;
import de.bsvrz.buv.plugin.darstellung.model.Ansicht;
import de.bsvrz.buv.plugin.darstellung.model.Darstellung;
import de.bsvrz.buv.plugin.darstellung.model.Ebene;
import de.bsvrz.buv.plugin.darstellung.model.StilisierteDarstellung;
import de.bsvrz.buv.plugin.dobj.editparts.DoModelEditPart;
import de.bsvrz.buv.plugin.dobj.model.DoModel;
import de.bsvrz.buv.plugin.dobj.model.DoTyp;
import de.bsvrz.buv.plugin.parameter.tsf.DaVKommunikation;
import de.bsvrz.buv.plugin.parameter.tsf.zeitdauer.MyCompositeZeitDauer;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

/**
 * Die Klasse realisert die Darstellung und Bearbeitung des Schaltbild-Parameters (TSF) einer
 * Ursacheneinheit. Diese wird in einer Registerkarte der Composites {@code CompositeParameterStufen}} verwendet.
 * 
 * @author Pittner
 * 
 */
public class CompositeParameterDarstellung extends Composite
{
  /**
   * Die Verbindung zum Datenverteiler
   */
  private ClientDavInterface _connection;

  /**
   * Die geteilte Ansicht
   */
  private SashForm _sf1;

  /**
   * Composites
   */
  private Composite _composite1;
  private Composite _composite12;
  private Composite _composite2;
//  private Composite _composite21;
  private Composite _compositeButton1;


  /**
   * Die GEF-Viewer
   */
  private final GraphicalViewer graphicalViewerBeginn = new ScrollingGraphicalViewer();
  private final GraphicalViewer graphicalViewerEnde = new ScrollingGraphicalViewer();
 
  /**
   * Das Register
   */  
  private CTabFolder _tabFolder;

  /**
   * Der DaV-Datensatz
   */  
  private ResultData _resultData;

  /**
   * Das aktuelle Programm
   */  
  private int _indexPrg;
  
  /**
   * Die aktuelle Stufe
   */  
  private int _indexStufe;

  /**
   * Texts
   */
  private Text darstellungParameterKommentar;  
  private Text darstellungParameterText;  
  
  /**
   * Button
   */  
  private Button parameterSelectButton;

  /**
   * Die Zeitlwhler
   */  
  private MyCompositeZeitDauer _compositeDelayBeginn;
  private MyCompositeZeitDauer _compositeDelaySchaltBild;
  private MyCompositeZeitDauer _compositeTimeout;

  /**
   * Spinner
   */  
  private Spinner _spinnerVideoPrg;

  private MenuManager _kontextMenuMgr1;
  private MenuManager _kontextMenuMgr2;

  private IViewSite _viewSite;
  
  /**
   * Die Register-Bezeichnungen
   */  
  public final static String REGISTER_BEGINN_STUFE = "Beginn Stufe";
  public final static String REGISTER_ENDE_STUFE = "Ende Stufe";
  

  /**
   * Konstruktor
   * 
   * @param parent
   *          das Elternobjekt
   * @param style
   *          der Stil wie das {@code Composite} dargestellt werden soll
   * @param so
   *          das Systemobjekt
   */
  public CompositeParameterDarstellung(IViewSite viewSite, Composite parent, int style, int indexPrg, int indexStufe, ResultData rd)
  {
    
    super(parent, style);    
    _viewSite = viewSite;
    _connection = DaVKommunikation.getInstanz().holeDaVVerbindung();
    
    _resultData = rd;
    _indexStufe = indexStufe;
    _indexPrg = indexPrg;
    
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.verticalAlignment = GridData.FILL;
    
    setLayoutData(gridData1);
    setLayout(new GridLayout());
    
    createInhaltTab();
    
    registriereActions();
    erzeugeKontextMenue1();
    erzeugeKontextMenue2();
    
    aktualisiereDarstellungAusDaVDaten2();
  }

  /**
   * Initialisiert den Inhalt des {@code TabItem}
   * 
   * @param tabItemComposite
   *          das Eltern Objekt
   * @param so
   *          die Ursacheneinheit
   */
  private void createInhaltTab()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.verticalAlignment = GridData.FILL;

    _sf1 = new SashForm(this, SWT.HORIZONTAL | SWT.BORDER);
    _sf1.setLayoutData(gridData1);

    // rechtes und linkes Composite in der Sash-Form
    createComposite1();
    createComposite2();

    _sf1.setWeights(new int[] { 75, 85 });   
  }

  /**
   * This method initializes _composite1
   * 
   */
  private void createComposite1()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    _composite1 = new Composite(_sf1, SWT.NONE);
    _composite1.setLayout(new GridLayout());
    _composite1.setLayoutData(gridData1);

    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.FILL;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.grabExcessVerticalSpace = true;
    gridData3.verticalAlignment = GridData.FILL;
    _composite12 = new Composite(_composite1, SWT.NONE);
    _composite12.setLayout(new GridLayout(2, false));
    _composite12.setLayoutData(gridData3);

    createCompositeDelay();

  }
  
  /**
   * This method initializes _composite12
   * 
   */
  private void createCompositeDelay()
  {
    GridLayout rowLayout = new GridLayout(3, false);
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    gridData.grabExcessVerticalSpace = true;
    
    _compositeButton1 = new Composite(_composite12, SWT.NONE);
    _compositeButton1.setLayoutData(gridData);
    _compositeButton1.setLayout(rowLayout);
    
    Label label0 = new Label(_compositeButton1, SWT.None);
    label0.setText("Kommentar ");
    
    darstellungParameterKommentar = new Text(_compositeButton1, SWT.BORDER);   
    GridData gdComment = new GridData();
    gdComment.horizontalAlignment = GridData.FILL;
    gdComment.grabExcessHorizontalSpace = true;
    gdComment.verticalAlignment = GridData.CENTER;
    gdComment.horizontalSpan = 2;
    darstellungParameterKommentar.setLayoutData(gdComment);
         
    Label label1 = new Label(_compositeButton1, SWT.None);
    label1.setText("Teilabschnitt ");
 
    darstellungParameterText = new Text(_compositeButton1, SWT.BORDER | SWT.READ_ONLY);   
    GridData gdText = new GridData();
    gdText.horizontalAlignment = GridData.FILL;
    gdText.grabExcessHorizontalSpace = true;
    gdText.verticalAlignment = GridData.CENTER;
    darstellungParameterText.setLayoutData(gdText);
    
    parameterSelectButton = new Button(_compositeButton1, SWT.PUSH);
    parameterSelectButton.setText("Teilabschnitt");
    GridData gdBtn = new GridData();
    gdBtn.horizontalAlignment = GridData.FILL;
    gdBtn.grabExcessHorizontalSpace = true;
    gdBtn.verticalAlignment = GridData.CENTER;
    parameterSelectButton.setLayoutData(gdBtn);
    
    parameterSelectButton.addSelectionListener(new SelectionAdapter() {
      
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        // TODO Auto-generated method stub
        super.widgetSelected(e);                
        String s = null;
        
        final List<Object> contents = new ArrayList<Object>();        
        SystemObjectType type = _connection.getDataModel().getType("typ.sswSbaTsfTeilAbschnitt");
        List<SystemObject> elements = type.getElements();
        for (SystemObject so : elements)
        {
          contents.add(so);
        }

        if (contents.isEmpty())
        {
          MessageDialog.openInformation(e.widget.getDisplay().getActiveShell(), "Information",
          "Es gibt kein anderer Teilabschnitt auf den referenziert werden kann.");
          return;
        }
        
        final String REMOVE_RELATIV_ZU = "<Referenz lschen>";
        contents.add(0, REMOVE_RELATIV_ZU);
        
        final ElementListSelectionDialog dialog = new ElementListSelectionDialog(
            e.widget.getDisplay().getActiveShell(), new SystemObjectLabelProvider());
        dialog.setTitle("Teilabschnitt auswhlen");
        dialog
        .setMessage("Bitte whlen Sie eine anderes Objekt zur Auswahl des Teilabschnittes.");
        dialog.setMultipleSelection(false);
        dialog.setElements(contents.toArray());
        
        if (s != null)
        {
          dialog.setInitialElementSelections(Collections.singletonList(s));
        }
        
        if (dialog.open() == Window.OK)
        {
          Object value = null;
          if (REMOVE_RELATIV_ZU.equals(dialog.getFirstResult()))
          {
            value = null;
          }
          else
          {
            value = dialog.getFirstResult();
          }
          
          if (value != null){
            SystemObject so = (SystemObject)value; 
            darstellungParameterText.setText(so.getPidOrId());
          }else{
            darstellungParameterText.setText("");            
          }
        }        
      }      
    });
    
    Label label4 = new Label(_compositeButton1, SWT.None);
    label4.setText("Videoprogramm");    
    _spinnerVideoPrg = new Spinner(_compositeButton1, SWT.BORDER);    
    // zum Ausfllen der Zelle
    GridData gdSpinner = new GridData();
    gdSpinner.horizontalAlignment = GridData.FILL;
    gdSpinner.grabExcessHorizontalSpace = true;
    gdSpinner.verticalAlignment = GridData.CENTER;
    _spinnerVideoPrg.setLayoutData(gdSpinner);    
    _spinnerVideoPrg.setMaximum(Integer.MAX_VALUE);
    Label filler4 = new Label(_compositeButton1, SWT.None);
    
    Label label2 = new Label(_compositeButton1, SWT.None);
    label2.setText("Delayzeit Beginn Stufe");    
    _compositeDelayBeginn = new MyCompositeZeitDauer(_compositeButton1, SWT.NONE, new Date().getTime(), true);
    _compositeDelayBeginn.setBackground( _compositeButton1.getBackground() );
    // zum Ausfllen der Zelle
    GridData gdDauer = new GridData();
    gdDauer.horizontalAlignment = GridData.FILL;
    gdDauer.grabExcessHorizontalSpace = true;
    gdDauer.verticalAlignment = GridData.CENTER;
    _compositeDelayBeginn.setLayoutData(gdDauer);
    _compositeDelayBeginn.setZeitDauer(0, 0, 0, 0, 0, 0, 0);    
    Label filler2 = new Label(_compositeButton1, SWT.None);
    
    Label label3 = new Label(_compositeButton1, SWT.None);
    label3.setText("Delayzeit Schaltbilder");    
    _compositeDelaySchaltBild = new MyCompositeZeitDauer(_compositeButton1, SWT.NONE, new Date().getTime(), true);
    _compositeDelaySchaltBild.setBackground( _compositeButton1.getBackground() );
    // zum Ausfllen der Zelle
    GridData gdDauer2 = new GridData();
    gdDauer2.horizontalAlignment = GridData.FILL;
    gdDauer2.grabExcessHorizontalSpace = true;
    gdDauer2.verticalAlignment = GridData.CENTER;
    _compositeDelaySchaltBild.setLayoutData(gdDauer2);
    _compositeDelaySchaltBild.setZeitDauer(0, 0, 0, 0, 0, 0, 0);    
    Label filler3 = new Label(_compositeButton1, SWT.None);    
    
    Label label5 = new Label(_compositeButton1, SWT.None);
    label5.setText("Timeoutzeit");    
    _compositeTimeout = new MyCompositeZeitDauer(_compositeButton1, SWT.NONE, new Date().getTime(), true);
    _compositeTimeout.setBackground( _compositeButton1.getBackground() );
    // zum Ausfllen der Zelle
    GridData gdDauer3 = new GridData();
    gdDauer3.horizontalAlignment = GridData.FILL;
    gdDauer3.grabExcessHorizontalSpace = true;
    gdDauer3.verticalAlignment = GridData.CENTER;
    _compositeTimeout.setLayoutData(gdDauer3);
    _compositeTimeout.setZeitDauer(0, 0, 0, 0, 0, 0, 0);    
    Label filler5 = new Label(_compositeButton1, SWT.None);
  }

  /**
   * This method initializes _composite2
   * 
   */
  private void createComposite2()
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    _composite2 = new Composite(_sf1, SWT.NONE);
    _composite2.setLayout(new GridLayout());
    _composite2.setLayoutData(gridData);

    createCompositeTab();
  }
  
  
  private class SystemObjectLabelProvider extends LabelProvider{
	  
	  @Override
	public String getText(Object element) {
		// TODO Auto-generated method stub
		  
		if (element instanceof SystemObject) {
			SystemObject so = (SystemObject) element;
			return so.getNameOrPidOrId();
		}  
		  
		return super.getText(element);
	}
	  
	  
  }
  
  /**
   * Methode zum Anlegen des Composite mit den Reitern
   */
  private void createCompositeTab()
  {
    GridData gridDataGroup = new GridData();
    gridDataGroup.horizontalAlignment = GridData.FILL;
    gridDataGroup.grabExcessHorizontalSpace = true;
    gridDataGroup.grabExcessVerticalSpace = true;
    gridDataGroup.verticalAlignment = GridData.FILL;

    GridData gridDataTabFolder = new GridData();
    gridDataTabFolder.horizontalAlignment = GridData.FILL;
    gridDataTabFolder.grabExcessHorizontalSpace = true;
    gridDataTabFolder.grabExcessVerticalSpace = true;
    gridDataTabFolder.verticalAlignment = GridData.FILL;
    gridDataTabFolder.heightHint = 300;

    _tabFolder = new CTabFolder(_composite2, SWT.NONE);
    _tabFolder.setLayout(new GridLayout());
    _tabFolder.setLayoutData(gridDataTabFolder);
    _tabFolder.addSelectionListener(new SelectionListener() {

      @Override
      public void widgetDefaultSelected(SelectionEvent e)
      {
      }

      @Override
      public void widgetSelected(SelectionEvent e)
      {        
        CTabItem itemSel = _tabFolder.getSelection();        
        Composite cSel = (Composite)itemSel.getControl();        
        Control[] children = cSel.getChildren();
        
        
        if (children.length == 0){
//          System.out.println("Reiter neu: " + itemSel.getText());
//          ScrollingGraphicalViewer gv = createGraphicalViewer(cSel, itemSel.getText());
//          setGraphicalViewer(gv, itemSel.getText());
//          aktualisiereDarstellungAusDaVDaten();
//          
//          cSel.redraw();
//          cSel.layout(true);
        }     
        
        _tabFolder.redraw();
        _tabFolder.layout(true);
        
      }
    });
    
    
    CTabItem tabItem1 = new CTabItem( _tabFolder, SWT.NONE );

    tabItem1.setText( REGISTER_BEGINN_STUFE );

    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.verticalAlignment = GridData.FILL;

    Composite c1 = new Composite( _tabFolder, SWT.NONE );
    c1.setLayout( new GridLayout() );
    c1.setLayoutData( gridData1 );
    
    tabItem1.setControl( c1 );
    
    createGraphicalViewerBeginn(c1); 
        
    CTabItem tabItem2 = new CTabItem( _tabFolder, SWT.NONE );    

    tabItem2.setText( REGISTER_ENDE_STUFE );
    
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.verticalAlignment = GridData.FILL;
    
    Composite c2 = new Composite( _tabFolder, SWT.NONE );
    c2.setLayout( new GridLayout() );
    c2.setLayoutData( gridData2 );
    
    tabItem2.setControl( c2 );
    
    createGraphicalViewerEnde(c2); 
    
    _tabFolder.setSelection( 0 );
    _tabFolder.redraw();
    _tabFolder.layout(true);
    
  }

  private void createGraphicalViewerBeginn(Composite c)
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    Composite composite = new Composite(c, SWT.NONE);
    composite.setLayout(new FillLayout());
    composite.setLayoutData(gridData);

    graphicalViewerBeginn.createControl(composite);
    // XXX: Wichtig - Damit wei der Viewer welchen EditPart er fr ein
    // Modell Objekt benutzen soll.
    graphicalViewerBeginn.setEditDomain(new EditDomain());
    graphicalViewerBeginn.setRootEditPart(new SbaScalableRootEditPart());
    graphicalViewerBeginn.setEditPartFactory(new DarstellungEditPartFactory());
    
//    IPreferenceStore store = Activator.getDefault().getPreferenceStore();
//    String name = store.getString(DefsPreference.P_DARSTELLUNG_PARAMETER);    
//    if (name != null && name.length() > 0){
//      graphicalViewerBeginn.setContents(EcoreUtil.copy(SbaEinstellungen.INSTANCE.getAnsicht(name)));    
//    }
    
    Ansicht ansicht = SbaAnsichtProvider.getInstanz().getAnsichtKopie(DefsPreference.P_DARSTELLUNG_PARAMETER);
    if (ansicht != null)
      graphicalViewerBeginn.setContents(ansicht);    
    
    getCommandStack(REGISTER_BEGINN_STUFE).setUndoLimit(1);
    
    final ScalableRootEditPart rootEditPart = (ScalableRootEditPart)graphicalViewerBeginn.getRootEditPart();

//    final ZoomManager zoomManager = rootEditPart.getZoomManager();
//    zoomManager.setZoomLevels(new double[] { 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0, 10.0 });
//    graphicalViewerBeginn.setProperty(MouseWheelHandler.KeyGenerator.getKey(SWT.NONE), MouseWheelZoomHandler.SINGLETON);

    graphicalViewerBeginn.getControl().addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDoubleClick(MouseEvent e)
      {     
        
//        System.out
//            .println("CompositeParameterDarstellung.createGraphicalViewerBeginn(...).new MouseAdapter() {...}.mouseDoubleClick()");
        
        ISelection selection = graphicalViewerBeginn.getSelection();

        CTabItem tabItem = _tabFolder.getItem(0);
        if (tabItem != null)
        {
          Data data = _resultData.getData();
          if (data != null){
            Data copy = data.createModifiableCopy();
            Array arrayPrg = copy.getItem("Programme").asArray();
            if (arrayPrg != null && arrayPrg.getLength() > _indexPrg){
              Data itemPrg = arrayPrg.getItem(_indexPrg);
              if (itemPrg != null){
                Array arrayStf = itemPrg.getItem("Stufen").asArray();  
                if (arrayStf != null){
//                  System.out.println("Stf: " + arrayStf.getLength() + " -> " + _indexStufe);                    
                  if (arrayStf.getLength() <= _indexStufe){
//                    System.out.println("---> else(Stf): " + arrayStf.getLength() + " -> " + _indexStufe);                    
                    arrayStf.setLength(_indexStufe + 1);
                  }                  
                  Data itemStufe = arrayStf.getItem(_indexStufe);
                  Data item = itemStufe.getItem("BefehleBeginnStufe");
                  if (item != null)
                    DoModelHelper.holeDoppelKlickAktionAusViewerSelection(selection, item);                  
                }
              }
            }else{
//              System.out.println("else(Prg): " + arrayPrg.getLength() + " -> " + _indexPrg);              
            }
          }else{
//           System.out.println("Data" + _resultData); 
          }
        }else{
//          System.out.println("Tab" + _tabFolder.getItemCount());
        }
      }     
    });
  }
  
  private void createGraphicalViewerEnde(Composite c)
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    Composite composite = new Composite(c, SWT.NONE);
    composite.setLayout(new FillLayout());
    composite.setLayoutData(gridData);
    
    graphicalViewerEnde.createControl(composite);
    // XXX: Wichtig - Damit wei der Viewer welchen EditPart er fr ein
    // Modell Objekt benutzen soll.
    graphicalViewerEnde.setEditDomain(new EditDomain());
    graphicalViewerEnde.setRootEditPart(new SbaScalableRootEditPart());
    graphicalViewerEnde.setEditPartFactory(new DarstellungEditPartFactory());
    
//    IPreferenceStore store = Activator.getDefault().getPreferenceStore();
//    String name = store.getString(DefsPreference.P_DARSTELLUNG_PARAMETER);    
//    if (name != null && name.length() > 0){
//      graphicalViewerEnde.setContents(EcoreUtil.copy(SbaEinstellungen.INSTANCE.getAnsicht(name)));    
//    }
    
    Ansicht ansicht = SbaAnsichtProvider.getInstanz().getAnsichtKopie(DefsPreference.P_DARSTELLUNG_PARAMETER);    
    if (ansicht != null)
      graphicalViewerEnde.setContents(ansicht);    
    
    getCommandStack(REGISTER_ENDE_STUFE).setUndoLimit(1);
    
    final ScalableRootEditPart rootEditPart = (ScalableRootEditPart)graphicalViewerBeginn.getRootEditPart();

//    final ZoomManager zoomManager = rootEditPart.getZoomManager();
//    zoomManager.setZoomLevels(new double[] { 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0, 10.0 });
//    graphicalViewerEnde.setProperty(MouseWheelHandler.KeyGenerator.getKey(SWT.NONE), MouseWheelZoomHandler.SINGLETON);

    graphicalViewerEnde.getControl().addMouseListener(new MouseAdapter() {
      
      @Override
      public void mouseDoubleClick(MouseEvent e)
      {     
        ISelection selection = graphicalViewerEnde.getSelection();

        CTabItem tabItem = _tabFolder.getItem(1);
        if (tabItem != null)
        {
          Data data = _resultData.getData();
          if (data != null){
            Data copy = data.createModifiableCopy();
            Array arrayPrg = copy.getItem("Programme").asArray();
            if (arrayPrg != null && arrayPrg.getLength() >= _indexPrg){
              Data itemPrg = arrayPrg.getItem(_indexPrg);
              if (itemPrg != null){
                Array arrayStf = itemPrg.getItem("Stufen").asArray();                
                if (arrayStf != null && arrayStf.getLength() > _indexStufe){
                }else{
                  arrayStf.setLength(_indexStufe);
//                  System.out.println("else(Stf): " + arrayStf.getLength() + " -> " + _indexStufe);
                }
                Data itemStufe = arrayStf.getItem(_indexStufe);
                Data item = itemStufe.getItem("BefehleEndeStufe");
                if (item != null)
                  DoModelHelper.holeDoppelKlickAktionAusViewerSelection(selection, item);

              }
            }
          }
        }
      }     
    });
  }
  
  /**
   * Holt das {@code DoModel} aus dem Parameter atg.sswSbaEinzelMassnahme und benachrichtigt den {@code GraphicalViewer}
   * zur Darstellung der Aenderung
   * 
   * @param die
   *          Daten aus der Atg
   */
  public void aktualisiereDarstellungAusDaVDaten2()
  { 
    if (_resultData != null)
    {
      
      CTabItem[] items = _tabFolder.getItems();
      for (int loc = 0; loc < items.length; loc++)
      {
        CTabItem item = items[loc];
        String tabName = item.getText();
        String strItemBefehle = null;
        
//        System.out.println(tabName + " - " + _indexPrg + " - " + _indexStufe);
        
        if (tabName.equals(REGISTER_BEGINN_STUFE))
          strItemBefehle = "BefehleBeginnStufe";
        else if (tabName.equals(REGISTER_ENDE_STUFE))
          strItemBefehle = "BefehleEndeStufe";
        else{
          System.err.println("else: " + tabName);
          return;
        }
        
        setzeAlleAnzeigen(tabName, AnzeigeInhaltTyp.BELIEBIG_BI);
        
        GraphicalViewer gv = getGraphicalViewer(tabName);
        Data data = _resultData.getData();
        
        Array arrayPrg = data.getItem("Programme").asArray();
        
        if (arrayPrg.getLength() > _indexPrg)
        {
          Data itemPrg = arrayPrg.getItem(_indexPrg);
          if (itemPrg != null)
          {
            Array arrayStufen = itemPrg.getItem("Stufen").asArray();
            if (arrayStufen.getLength() > _indexStufe)
            {
              Data itemStufe = arrayStufen.getItem(_indexStufe);
              if (itemStufe != null)
              {
                String str = itemStufe.getItem("Kommentar").asTextValue().getValueText();
                if (str != null)
                  darstellungParameterKommentar.setText(str);              
                SystemObject ta = itemStufe.getItem("Teilabschnitt").asReferenceValue().getSystemObject();
                if (ta != null)
                {
                  darstellungParameterText.setText(ta.getPidOrId());
                }
                long t1 = itemStufe.getItem("DelayzeitBeginnStufe").asTimeValue().getMillis();
                _compositeDelayBeginn.setZeitDauerInBezugAufAnfangsZeitpunkt(t1);
                long t2 = itemStufe.getItem("DelayzeitSchaltbilder").asTimeValue().getMillis();
                _compositeDelaySchaltBild.setZeitDauerInBezugAufAnfangsZeitpunkt(t2);
                int ivp = (int)itemStufe.getItem("Videoprogramm").asUnscaledValue().longValue();
                _spinnerVideoPrg.setSelection(ivp);
                long t3 = itemStufe.getItem("Timeoutzeit").asTimeValue().getMillis();
                _compositeTimeout.setZeitDauerInBezugAufAnfangsZeitpunkt(t3);
                
                Ansicht diagram = (Ansicht)gv.getContents().getModel();   
                EList<Ebene> ebenen = diagram.getDarstellung().getEbenen();
                for (Ebene ebene : ebenen)
                {
                  EList<DoModel> figures = ebene.getDoObjekte();
                  Array array = itemStufe.getItem(strItemBefehle).asArray();
                  for (int i = 0; i < array.getLength(); i++)
                  {
                    Data item2 = array.getItem(i);
                    SystemObject soAnzeige = item2.getItem("Anzeige").asReferenceValue().getSystemObject();
                    int notwendig = item2.getItem("notwendig").asTextValue().getValueText().equals("Ja") ? 1 : 0;
                    String prio = item2.getItem("Prioritt").asTextValue().getValueText();
                    int blinken = item2.getItem("Schaltbild").getItem("Blinken").asTextValue().getValueText().equals("Ja") ? 1 : 0;
                    String takt = item2.getItem("Schaltbild").getItem("Blinktakt").asTextValue().getText();
                    if (soAnzeige != null)
                    {
                      SystemObject soAnzeigeInhalt = item2.getItem("Schaltbild").getItem("AnzeigeInhalt")
                      .asReferenceValue().getSystemObject();
                      
                      if (soAnzeigeInhalt == null){
                        System.out.println("NULL -> " + soAnzeige.getPid() + " : " + soAnzeigeInhalt);
                      }
                      
                      String bi = DaVHelper.holeBildInhaltVonWVZInhalt(_connection, soAnzeigeInhalt);
                      AnzeigeInhaltTyp inhaltTyp = AnzeigeInhaltTyp.setBildInhalt(bi, _resultData.getDataTime());
                                            
                      for (DoModel doModel : figures)
                      {
                        if (doModel instanceof AnzeigeQuerschnittModelTypAbstract)
                        {
                          AnzeigeQuerschnittModelTypAbstract aq = (AnzeigeQuerschnittModelTypAbstract)doModel;
                          EList<AnzeigeModelTypAbstract> anzeigen = aq.getKomponenten();
                          for (AnzeigeModelTypAbstract anzeige : anzeigen)
                          {
                            if (anzeige.getSystemObject().getPid().equals(soAnzeige.getPid())){
                              anzeige.setNotwendig(notwendig);
                              anzeige.setPrioritaet(prio);
                              anzeige.setBlinken(blinken);
                              if (takt.contains(",")){
                                String taktNeu = takt.replace(",", ".");
                                anzeige.setBlinktakt(taktNeu);
                              }
                              else 
                                anzeige.setBlinktakt(takt);
                              EStructuralFeature feature = AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE;
                              final SetCommand cmd = new SetCommand(anzeige, feature, inhaltTyp);
                              getCommandStack(tabName).execute(cmd);
                            }
                          }
                        }
                      }
                    }
                  }
                }
              }
            }else{              
              System.out.println("else Stufe: " + arrayStufen.getLength() + " - "  + _indexStufe);
            }
          }
        }else{
          System.out.println("else Prg: " + arrayPrg.getLength() + " - " + _indexPrg);
        }
      } 
    }
  }

  /**
   * Gibt den Command Stack zurck.
   * 
   * @return der Command Stack, niemals <code>null</code>.
   */
  protected CommandStack getCommandStack(String tabName)
  {

    CommandStack stack = getGraphicalViewer(tabName).getEditDomain().getCommandStack();

    Assert.isNotNull(stack, "Command Stack nicht verfgbar.");
    return stack;
  }

  /**
   * Holt den GEF-Viewer
   * 
   * @return viewer
   */
  public GraphicalViewer getGraphicalViewer(String tabName)
  {
     if (tabName.equals(REGISTER_BEGINN_STUFE))
       return graphicalViewerBeginn;
     else if (tabName.equals(REGISTER_ENDE_STUFE))
       return graphicalViewerEnde;
     else
       throw new IllegalAnnotationException("Keine Viewer fr Tab: " + tabName);
  }
  
  public int getIndexPrg()
  {
    return _indexPrg;
  }
  
  public MyCompositeZeitDauer getCompositeDelayBeginn()
  {
    return _compositeDelayBeginn;
  }

  public MyCompositeZeitDauer getCompositeDelaySchaltBild()
  {
    return _compositeDelaySchaltBild;
  }

  public MyCompositeZeitDauer getCompositeTimeout()
  {
    return _compositeTimeout;
  }

  public Spinner getSpinnerVideoPrg()
  {
    return _spinnerVideoPrg;
  }
  
  public CTabFolder getTabFolder()
  {
    return _tabFolder;
  }

  public void setTabFolder(CTabFolder tabFolder)
  {
    _tabFolder = tabFolder;
  }

  public Text getDarstellungParameterText()
  {
    return darstellungParameterText;
  }
    
  public Text getDarstellungParameterKommentar()
  {
    return darstellungParameterKommentar;
  }
  
  public ResultData getResultData()
  {
    return _resultData;
  }

  public void setResultData(ResultData resultData)
  {
    _resultData = resultData;
  }
  
  /**
   * Vorbelegung aller Anzeigen mit einer Inhalt
   * @param inhalt
   */
  public void setzeAlleAnzeigen(String tabName, AnzeigeInhaltTyp  inhalt)
  {
    AnsichtEditPart contents = (AnsichtEditPart)getGraphicalViewer(tabName).getContents();

    Ansicht ansicht = contents.getModel();

    StilisierteDarstellung darstellung = (StilisierteDarstellung)ansicht.getDarstellung();

    EList<Ebene> ebenen = darstellung.getEbenen();
    for (Ebene ebene : ebenen)
    {
      EList<DoModel> dos = ebene.getDoObjekte();
      for (DoModel dom : dos)
      {
        if (dom instanceof AnzeigeQuerschnittModelTypAbstract)
        {
          AnzeigeQuerschnittModelTypAbstract model = (AnzeigeQuerschnittModelTypAbstract)dom;
          EList<AnzeigeModelTypAbstract> komponenten = model.getKomponenten();
          for (AnzeigeModelTypAbstract anzeige : komponenten)
          {
            EStructuralFeature feature = AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE;
            final SetCommand cmd = new SetCommand(anzeige, feature, inhalt);
            getCommandStack(tabName).execute(cmd);
          }
        }
      }
    }
  }
  
  /**
   * Registriert die Actions zum Aufruf der Outline-View
   */
  private void registriereActions()
  {   
      IMenuManager menuManager = _viewSite.getActionBars().getMenuManager();    
     
      AnsichtEditPart contents = (AnsichtEditPart)getGraphicalViewer(REGISTER_BEGINN_STUFE).getContents();
      Ansicht ansicht = contents.getModel();
      Darstellung darstellung = ansicht.getDarstellung();
      EList<Double> stufen = darstellung.getZoomStufen();
      
      ScalableRootEditPart rootEditPart1 = (ScalableRootEditPart)getGraphicalViewer(REGISTER_BEGINN_STUFE).getRootEditPart();
      final ZoomManager zoomManager1 = rootEditPart1.getZoomManager();
      ScalableRootEditPart rootEditPart2 = (ScalableRootEditPart)getGraphicalViewer(REGISTER_ENDE_STUFE).getRootEditPart();
      final ZoomManager zoomManager2 = rootEditPart2.getZoomManager();
      
      double[] dd = new double[stufen.size()];
      
      for (int i = 0; i < dd.length; i++)
      {
        dd[i] = stufen.get(i);      
      }
      
      zoomManager1.setZoomLevels(dd);
      zoomManager2.setZoomLevels(dd);
      
      for (final Double stufe : stufen)
      {
        Action zoomAction = new Action() {
          public void run()
          {                          
            zoomManager1.setZoom(stufe);            
            zoomManager2.setZoom(stufe);
          }
        };
        zoomAction.setText("Zoomstufe " + stufe);
        menuManager.add(zoomAction);
      }      
    }
  
  
  /**
   * Liefert das KontextMenue zum Navigator
   */
  private void erzeugeKontextMenue1()
  {
    _kontextMenuMgr1 = new MenuManager("#ParameterTsfAblaufMenu1");     
    _kontextMenuMgr1.addMenuListener(new IMenuListener()
    {
      public void menuAboutToShow(IMenuManager manager)
      {
        IStructuredSelection ss = (IStructuredSelection)getGraphicalViewer(REGISTER_BEGINN_STUFE).getSelection();
         
        if (!ss.isEmpty())
        {
          Object obj = ss.getFirstElement();

          _kontextMenuMgr1.removeAll();

          if (obj instanceof DoModelEditPart)
          {
            DoModelEditPart part = (DoModelEditPart)obj;
            DoModel model = (DoModel)part.getModel();
            DoTyp doTyp = model.getDoTyp();
            if (doTyp instanceof VerhaltenDoTyp)
            {
              VerhaltenDoTyp coDoTyp = (VerhaltenDoTyp)doTyp;

              int kontext = -1;

              if (model instanceof AnzeigeQuerschnittModelTypAbstract)
              {
                AnzeigeQuerschnittModelTypAbstract aq = (AnzeigeQuerschnittModelTypAbstract)model;
                kontext = aq.getKontext();
              }
              else if (model instanceof AnzeigeModelTypAbstract)
              {
                AnzeigeModelTypAbstract a = (AnzeigeModelTypAbstract)model;
                EObject parent = a.eContainer();
                if (parent != null)
                {
                  if (parent instanceof AnzeigeQuerschnittModelTypAbstract)
                  {
                    AnzeigeQuerschnittModelTypAbstract aq = (AnzeigeQuerschnittModelTypAbstract)parent;
                    kontext = aq.getKontext();
                  }
                }
              }

              String id = null;

              switch (kontext)
              {
                case 0:
                  id = coDoTyp.getKontextMenueIdIstZustand();
                  break;
                case 1:
                  id = coDoTyp.getKontextMenueIdSollZustand();
                  break;
                case 2:
                  id = coDoTyp.getKontextMenueIdHandSchaltung();
                  break;
                case 3:
                  id = coDoTyp.getKontextMenueIdSimulation();
                  break;
                case 4:
                  id = coDoTyp.getKontextMenueIdTrichter();
                  break;
                case 5:
                  id = coDoTyp.getKontextMenueIdParameter();
                  break;

                default:
                  break;
              }
              if (id != null)
                leseActionSets1(id);
            }
          }
        }
      }
    });

    // Men erzeugen - Viewer wird dabei dem Men bekannt gemacht
    Menu menu = _kontextMenuMgr1.createContextMenu(getGraphicalViewer(REGISTER_BEGINN_STUFE).getControl());    
    // Das Men dem Baum bekannt machen
    getGraphicalViewer(REGISTER_BEGINN_STUFE).getControl().setMenu(menu);
    
    
    
    _viewSite.registerContextMenu(_kontextMenuMgr1, getGraphicalViewer(REGISTER_BEGINN_STUFE));

  }
  
  /**
   * Liest die Aktionen unter Bercksichtigung des aktuellen WorkbenchWindow ein.<br>
   * Die Aktionen werden so erzeugt, dass die Actions am SelectionProvider des aktuellen EclipseWindows hngen. <br>
   * Evtl. muss noch das aktuelle Workbenchwindow beim speichern der Aktionen bercksichtigt werden. Dies ist fr ein
   * Window aber NICHT ntig.
   */
  @SuppressWarnings("restriction")
  public void leseActionSets1(String id)
  {
    IExtensionRegistry reg = Platform.getExtensionRegistry();
    IConfigurationElement[] aConfigurationElementsFor = reg.getConfigurationElementsFor("org.eclipse.ui.actionSets");
    IWorkbenchWindow window = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
    for (IConfigurationElement confEle : aConfigurationElementsFor)
    {
      String contributor = confEle.getContributor().getName();            
//      if (Activator.PLUGIN_ID.equals(contributor))
      {     
        IConfigurationElement[] iceAction = confEle.getChildren("action");
        for (IConfigurationElement element : iceAction)
        {          
          String att = element.getAttribute("id");  
          if (att.equals(id)){            
            boolean pullDownStyle = false;
            String style = element.getAttribute(IWorkbenchRegistryConstants.ATT_STYLE);
            if (style != null)
            {
              pullDownStyle = style.equals(ActionDescriptor.STYLE_PULLDOWN);
            } else
            {
              String pulldown = element.getAttribute(ActionDescriptor.STYLE_PULLDOWN);
              pullDownStyle = pulldown != null && pulldown.equals("true");
            }
            
            ActionDescriptor desc = null;
            if (pullDownStyle)
            {
              desc = new ActionDescriptor(element,
                  ActionDescriptor.T_WORKBENCH_PULLDOWN, window);
            } else
            {
              desc = new ActionDescriptor(element, ActionDescriptor.T_WORKBENCH, window);
            }           
            PluginAction action = desc.getAction();                
            _kontextMenuMgr1.add(action);            
          }  
        }        
      }      
    }
  }
  
  /**
   * Liefert das KontextMenue zum Navigator
   */
  private void erzeugeKontextMenue2()
  {
    _kontextMenuMgr2 = new MenuManager("#ParameterTsfAblaufMenu2");     
    _kontextMenuMgr2.addMenuListener(new IMenuListener()
    {
      public void menuAboutToShow(IMenuManager manager)
      {
        IStructuredSelection ss = (IStructuredSelection)getGraphicalViewer(REGISTER_ENDE_STUFE).getSelection();
        
        if (!ss.isEmpty())
        {
          Object obj = ss.getFirstElement();
          
          _kontextMenuMgr2.removeAll();
          
          if (obj instanceof DoModelEditPart)
          {
            DoModelEditPart part = (DoModelEditPart)obj;
            DoModel model = (DoModel)part.getModel();
            DoTyp doTyp = model.getDoTyp();
            if (doTyp instanceof VerhaltenDoTyp)
            {
              VerhaltenDoTyp coDoTyp = (VerhaltenDoTyp)doTyp;
              
              int kontext = -1;
              
              if (model instanceof AnzeigeQuerschnittModelTypAbstract)
              {
                AnzeigeQuerschnittModelTypAbstract aq = (AnzeigeQuerschnittModelTypAbstract)model;
                kontext = aq.getKontext();
              }
              else if (model instanceof AnzeigeModelTypAbstract)
              {
                AnzeigeModelTypAbstract a = (AnzeigeModelTypAbstract)model;
                EObject parent = a.eContainer();
                if (parent != null)
                {
                  if (parent instanceof AnzeigeQuerschnittModelTypAbstract)
                  {
                    AnzeigeQuerschnittModelTypAbstract aq = (AnzeigeQuerschnittModelTypAbstract)parent;
                    kontext = aq.getKontext();
                  }
                }
              }
              
              String id = null;
              
              switch (kontext)
              {
                case 0:
                  id = coDoTyp.getKontextMenueIdIstZustand();
                  break;
                case 1:
                  id = coDoTyp.getKontextMenueIdSollZustand();
                  break;
                case 2:
                  id = coDoTyp.getKontextMenueIdHandSchaltung();
                  break;
                case 3:
                  id = coDoTyp.getKontextMenueIdSimulation();
                  break;
                case 4:
                  id = coDoTyp.getKontextMenueIdTrichter();
                  break;
                case 5:
                  id = coDoTyp.getKontextMenueIdParameter();
                  break;
                  
                default:
                  break;
              }
              if (id != null)
                leseActionSets2(id);
            }
          }
        }
      }
    });
    
    // Men erzeugen - Viewer wird dabei dem Men bekannt gemacht
    Menu menu = _kontextMenuMgr2.createContextMenu(getGraphicalViewer(REGISTER_ENDE_STUFE).getControl());    
    // Das Men dem Baum bekannt machen
    getGraphicalViewer(REGISTER_ENDE_STUFE).getControl().setMenu(menu);
    _viewSite.registerContextMenu(_kontextMenuMgr2, getGraphicalViewer(REGISTER_ENDE_STUFE));
    
  }
  
  /**
   * Liest die Aktionen unter Bercksichtigung des aktuellen WorkbenchWindow ein.<br>
   * Die Aktionen werden so erzeugt, dass die Actions am SelectionProvider des aktuellen EclipseWindows hngen. <br>
   * Evtl. muss noch das aktuelle Workbenchwindow beim speichern der Aktionen bercksichtigt werden. Dies ist fr ein
   * Window aber NICHT ntig.
   */
  @SuppressWarnings("restriction")
  public void leseActionSets2(String id)
  {
    IExtensionRegistry reg = Platform.getExtensionRegistry();
    IConfigurationElement[] aConfigurationElementsFor = reg.getConfigurationElementsFor("org.eclipse.ui.actionSets");
    IWorkbenchWindow window = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
    for (IConfigurationElement confEle : aConfigurationElementsFor)
    {
      String contributor = confEle.getContributor().getName();            
//      if (Activator.PLUGIN_ID.equals(contributor))
      {     
        IConfigurationElement[] iceAction = confEle.getChildren("action");
        for (IConfigurationElement element : iceAction)
        {          
          String att = element.getAttribute("id");  
          if (att.equals(id)){            
            boolean pullDownStyle = false;
            String style = element.getAttribute(IWorkbenchRegistryConstants.ATT_STYLE);
            if (style != null)
            {
              pullDownStyle = style.equals(ActionDescriptor.STYLE_PULLDOWN);
            } else
            {
              String pulldown = element.getAttribute(ActionDescriptor.STYLE_PULLDOWN);
              pullDownStyle = pulldown != null && pulldown.equals("true");
            }
            
            ActionDescriptor desc = null;
            if (pullDownStyle)
            {
              desc = new ActionDescriptor(element,
                  ActionDescriptor.T_WORKBENCH_PULLDOWN, window);
            } else
            {
              desc = new ActionDescriptor(element, ActionDescriptor.T_WORKBENCH, window);
            }           
            PluginAction action = desc.getAction();                
            _kontextMenuMgr2.add(action);            
          }  
        }        
      }      
    }
  }
  
//  public IViewSite getViewSite(){
//    IWorkbench wb = PlatformUI.getWorkbench();
//    if (wb != null){
//      IWorkbenchWindow dw = wb.getActiveWorkbenchWindow();
//      if (dw != null){
//        IWorkbenchPage page = dw.getActivePage();
//        if (page != null){
//          IWorkbenchPart part = page.getActivePart();
//          IWorkbenchPartSite wbSite = part.getSite();
////          return wbSite;
//          if (wbSite instanceof IViewSite)
//          {
//            IViewSite viewSite = (IViewSite)wbSite;
//            return (IViewSite)viewSite;                      
//          }
//        }
//      }
//    }
//    return null;
//  }
//  
//  public IWorkbenchPartSite getWorkbenchPartSite(){
//    IWorkbench wb = PlatformUI.getWorkbench();
//    if (wb != null){
//      IWorkbenchWindow dw = wb.getActiveWorkbenchWindow();
//      if (dw != null){
//        IWorkbenchPage page = dw.getActivePage();
//        if (page != null){
//          IWorkbenchPart part = page.getActivePart();
//          IWorkbenchPartSite wbSite = part.getSite();
//          return wbSite;
////          if (wbSite instanceof IViewSite)
////          {
////            IViewSite viewSite = (IViewSite)wbSite;
////            return (IViewSite)viewSite;                      
////          }
//        }
//      }
//    }
//    return null;
//  }
  
  @Override
  public void dispose()
  {
    System.err.println("---> CompositeParameterDarstellung.dispose()");
    super.dispose();
    if (!_sf1.isDisposed())
      _sf1.dispose();
    _sf1 = null;
  }
  
}
