/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.plugin.parameter.tsf.mqZuordnung;

import java.util.IllegalFormatException;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.basislib.ort.FilterAsbOrtsangabe;
import de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer;
import de.bsvrz.buv.rw.basislib.ort.FilterKnoten;
import de.bsvrz.buv.rw.basislib.ort.FilterStrassenSegmentOffset;
import de.bsvrz.buv.rw.basislib.ort.OrtsangabeEnumeratorAuswahl;
import de.bsvrz.buv.rw.basislib.ort.FilterKnoten.Richtung;
import de.bsvrz.buv.rw.basislib.ort.hilfsklassen.EnumAsbVerkehrsRichtung;
import de.bsvrz.buv.rw.basislib.ort.hilfsklassen.Strasse;
import de.bsvrz.buv.rw.basislib.ort.hilfsklassen.StrassenKnoten;
import de.bsvrz.buv.rw.basislib.ort.hilfsklassen.StrassenSegment;
import de.bsvrz.buv.rw.basislib.ort.hilfsklassen.AsbOrtsangabe.AsbKnoten;
import de.bsvrz.buv.rw.basislib.ort.hilfsklassen.AsbOrtsangabe.AsbKnotenKombiniert;
import de.bsvrz.buv.rw.basislib.ort.hilfsklassen.AsbOrtsangabe.BereichsInfoAsbKnotenKombiniert;
import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.sys.funclib.debug.Debug;

public class OrtsangabeComposite extends Composite
{
	/**
	 * Der Debugger
	 */
	private Debug _debug = Debug.getLogger();

	private TabFolder _pultOrdner = null;

	private Composite _compositeStrassenSegment = null;

	private Label _labelStrassenSegmentStrasse = null;

	private Label _labelStrassenSegmentStrassenSegment = null;

	private Label _labelStrassenSegmentKilometer = null;

	private Label _labelStrassenSegmentStrasseFueller = null;

	private Label _labelStrassenSegmentStrasseLaengenwarnung = null;

	private Combo _comboStrassenSegmentStrasse = null;

	/**
	 * {@link ComboViewer} fr Ortstangabe ber Straensegment - Straenauswahl
	 */
	private ComboViewer _cvStrassenSegmentStrasse = null;

	private Combo _comboStrassenSegmentStrassenSegment = null;

	/**
	 * {@link ComboViewer} fr Ortstangabe ber Straensegment - StraenSegment
	 * auswahl
	 */
	private ComboViewer _cvStrassenSegmentStrassenSegment = null;

	private Text _textStrassenSegmentKilometer = null;

	private Composite _compositeBetriebskilometer = null;

	private Label _labelBetriebskilometerStrasse = null;

	private Label _labelBetriebskilometerRichtung = null;

	private Label _labelBetriebskilometerBetriebskilometer = null;

	private Label _labelBetriebskilometerBlocknummer = null;

	/**
	 * Dieses Label signalisiert, dass der angegebene Betriebskilometer nicht im
	 * LCL-Segment dieser Strae liegt und wird auch nur dann VISIBLE gesetzt.
	 */
	private Label _labelBetriebskilometerWarnung;

	private Combo _comboBetriebskilometerStrae = null;

	/**
	 * ComboViewer der Straen der Betriebskilometer
	 */
	private ComboViewer _cvBetriebsKilometerStrasse;

	private Combo _comboBetriebskilometerRichtung = null;

	private ComboViewer _cvBetriebsKilometerRichtung;

	private Text _textBetriebskilometerBetriebskilometer = null;

	/**
	 * Listener fr die Selektionen in der ComboBox Strassen der BetriebsKilometer
	 */
	private ISelectionChangedListener _listenerBetriebsKilometerStrasse;

	private ISelectionChangedListener _listenerBetriebskilometerRichtung;

	/**
	 * Listener, der die Eingabe im Feld Betriebskilometer darauf hin prft, ob
	 * nur Zahlen eingetragen wurden.
	 */
	private Listener _listenerTextBetriebskilometerBetriebskilometer = null;

	/**
	 * Listener, der die Eingabe im Feld Betriebskilometer Auswerter
	 */
	private ModifyListener _modifyListenerTextBetriebskilometerBetriebskilometer = null;

	private Combo _comboBetriebskilometerBlocknummer = null;

	/**
	 * ComboViewer fr die BlockNummern
	 */
	private ComboViewer _cvBetriebsKilometerBlockNummer = null;

	/**
	 * Composite, auf dem die ASB-Ortsangabe aufgebaut wird.
	 */
	private Composite _compositeAsbOrtsangabe = null;

	private Label _labelAsbStrasse = null;

	private Label _labelAsbVonNetzknoten = null;

	private Label _labelAsbNachNetzknoten = null;

	private Label _labelAsbStationierung = null;

	/**
	 * Label zum Anzeigen, dass sich der angegebene Stationierungswert nicht im
	 * gltigen Bereich befindet.
	 */
	private Label _labelAsbStationierungAusserhalbBereich = null;

	/**
	 * Label Stationierungsrichtung der ASB-Ortsangabe
	 */
	private Label _labelAsbStationierungStationierungsRichtung = null;

	/**
	 * ComboBox zur Auswahl der Strae bei der ASB-Stationierung
	 */
	private Combo _comboAsbStrasse = null;

	/**
	 * {@link ComboViewer} zur Auswahl der Strae (ASB)
	 */
	private ComboViewer _cvAsbStrasse = null;

	/**
	 * ComboBox zur Auswahl des Anfangsknotens bei der ASB-Stationierung
	 */
	private Combo _comboAsbAnfangsKnoten = null;

	/**
	 * {@link ComboViewer} zur Auswahl des Von-Netzknoten/Astes (ASB)
	 */
	private ComboViewer _cvAsbAnfangsKnoten = null;

	/**
	 * ComboBox zur Auswahl des Endknotens bei der ASB-Stationierung
	 */
	private Combo _comboAsbEndKnoten = null;

	/**
	 * {@link ComboViewer} zur Auswahl des Nach-Netzknoten/Astes (ASB)
	 */
	private ComboViewer _cvAsbEndKnoten = null;

	private Combo _comboAsbStationierungsrichtung = null;
	private ComboViewer _cvAsbStationierungsrichtung = null;

	/**
	 * Textfeld zum Eingeben der ASB-Stationierung in Metern.
	 */
	private Text _textAsbStationierung = null;

	private Composite _compositeKnotenOffset = null;

	private Composite _compositeKnotenOffsetOben = null;

	private Group _gruppeKnotenOffsetUnten = null;

	private Label _labelKnotenOffsetStrasse = null;

	private Label _labelKnotenOffsetKnoten = null;

	private Label _labelKnotenOffsetKilometer = null;

	private Button _radioButtonKnotenOffsetPositiv = null;

	private Button _radioButtonKnotenOffsetNegativ = null;

	private Combo _comboKnotenOffsetStrasse = null;

	/**
	 * {@link ComboViewer} fr comboKnotenOffsetStrasse
	 */
	private ComboViewer _cvKnotenOffsetStrasse = null;

	private Combo _comboKnotenOffsetKnoten = null;

	/**
	 * {@link ComboViewer} fr comboKnotenOffsetKnoten
	 */
	private ComboViewer _cvKnotenOffsetKnoten = null;

	private Text _textKnotenOffsetKilometer = null;

	/*
	 * TabBezeichner
	 */
	/**
	 * TabBezeichner Straensegment-Offset
	 */
	private final static String _bezStrassenSegmentOffset = "Straensegment-Offset";

	/**
	 * TabBezeichner Betriebskilometer
	 */
	private final static String _bezBetriebskilometer = "Betriebskilometer";

	/**
	 * TabBezeichner ASB-Ortsangabe
	 */
	private final static String _bezAsbOrtsangabe = "ASB-Ortsangabe";

	/**
	 * TabBezeichner Knoten-Offset
	 */
	private final static String _bezKnotenOffset = "Knoten-Offset";

	/**
	 * Composite zur Darstellung des Straensegments
	 */
	private Composite _pultStrassenSegment = null;

	/**
	 * Composite zur Darstellung der Betriebskilometerauswahl
	 */
	private Composite _pultBetriebskilometer = null;

	/**
	 * Composite zur Orstangabe ber ASB
	 */
	private Composite _pultAsbOrtsangabe = null;

	/**
	 * Composite zur Orstangabe ber einen Knoten
	 */
	private Composite _pultKnotenOffset = null;

	/**
	 * Speichert, welche Art der Ortsangabe gewhlt wurde.
	 */
	private OrtsangabeEnumeratorAuswahl _auswahl = OrtsangabeEnumeratorAuswahl.strassensegment;

	/**
	 * Filter fr die Angabe ber Straensegment & Offset.
	 */
	private FilterStrassenSegmentOffset _filterStrassenSegmentOffset = null;

	/**
	 * Filter fr die Angabe ber ASB.
	 */
	private FilterAsbOrtsangabe _filterAsbOrtsangabe = null;

	/**
	 * Filter fr die Angabe ber Betriebskilometer.
	 */
	private FilterBetriebskilometer _filterBetriebskilometer = null;

	/**
	 * Filter fr die Angabe ber Knoten.
	 */
	private FilterKnoten _filterKnoten = null;

	/**
	 * Verbindung zum DaV
	 */
	private ClientDavInterface _connection = null;

  private String _title;

	/**
	 * Liefert ein Composite zur Erfassung von Ortsangaben.
	 * 
	 * @param parent
	 *          Eltern-Composite fr die Darstellung
	 * @param style
	 *          SWT-Stil
	 * @param connection
	 *          Verbindung zum DaV.
	 */
	public OrtsangabeComposite(Composite parent, int style,
			ClientDavInterface connection, String title)
	{
		super(parent, style);
		this._connection = connection;
		this._title = title;
		initialize();
	}

	private void initialize()
	{
		setSize(new Point(300, 200));
		GridLayout layout = new GridLayout();
		setLayout(layout);

//		if (_connection != null)
//		{
//			erzeugePultOrdner(this);
//		} else
//		{
//			Label labelInfo = new Label(this, SWT.NONE);
//			labelInfo.setForeground(new Color(Display.getCurrent(), 255, 0, 0));
//			labelInfo.setText("Es wurde keine Verbindung zum DaV bergeben!");
//			erzeugePultOrdner(this);
//		}
//
//		fuellePultordner();
		
		Composite erzeugeCompositeGroupStart = erzeugeCompositeGroup(this, _title);
		erzeugeCompositeGroupStart.setLayout(new GridLayout());
		erzeugeCompositeStrassenSegment(erzeugeCompositeGroupStart);
//		Composite erzeugeCompositeGroupEnde = erzeugeCompositeGroup(this, "Ende-Offset");
//		erzeugeCompositeGroupEnde.setLayout(new GridLayout());
//		erzeugeCompositeStrassenSegment(erzeugeCompositeGroupEnde);
		

		/*
		 * "Selektieren" des ersten Tabs, damit sich die Strassen in diesem
		 * aktualisieren.
		 */
		selektiereAuswahlArt(_bezStrassenSegmentOffset);

		this.pack(true);
	}

	/**
	 * Erzeugung des pultOrdner zur Auswahl der Ortsangaben
	 * 
	 * @param elternComposite
	 *          {@link Composite}, auf dessen Basis die Darstellung erfolgen
	 *          soll.
	 */
	private void erzeugePultOrdner(Composite elternComposite)
	{
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = GridData.FILL;
		_pultOrdner = new TabFolder(elternComposite, SWT.NONE);
		_pultOrdner.setLayoutData(gridData);

		_pultOrdner.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				String selektion = ((TabItem) (e.item)).getText();

				_debug.fine("Selektiertes Tab: " + selektion);

				selektiereAuswahlArt(selektion);

			}
		});
	}

//	/**
//	 * Selektiert einen {@link TabFolder} an Hand einer Instanz des Enumerators
//	 * {@link OrtsangabeEnumeratorAuswahl}.
//	 * 
//	 * @param auswahlArt
//	 *          {@link OrtsangabeEnumeratorAuswahl}
//	 */
//	private void selektierePultOrdner(OrtsangabeEnumeratorAuswahl auswahlArt)
//	{
//		/*
//		 * Selektion des PultOrdners
//		 */
//		switch (auswahlArt)
//		{
//		case strassensegment:
//			_pultOrdner.setSelection(0);
//			selektiereAuswahlArt(_bezStrassenSegmentOffset);
//			break;
//		case betriebskilometer:
//			_pultOrdner.setSelection(1);
//			selektiereAuswahlArt(_bezBetriebskilometer);
//			break;
//		case asborstangabe:
//			_pultOrdner.setSelection(2);
//			selektiereAuswahlArt(_bezAsbOrtsangabe);
//			break;
//		case knotenoffset:
//			_pultOrdner.setSelection(3);
//			selektiereAuswahlArt(_bezKnotenOffset);
//			break;
//		default:
//			_debug.fine("IMMER AUSGEFHRT");
//			break;
//		}
//
//	}

	/**
	 * Bestimmt nach dem als Parameter bergebenen Tab die Auswahlart, die sich
	 * mit {@link OrtsangabeComposite#getAuswahlArt()} erfragen lsst. <br>
	 * 
	 * Die Filter werden dementsprechend selektiert.<br>
	 * 
	 * Diese Methode wird vom {@link SelectionAdapter} der Methode
	 * {@link OrtsangabeComposite#erzeugePultOrdner(Composite)} aufgerufen.
	 * 
	 * @param selektion
	 */
	private void selektiereAuswahlArt(String selektion)
	{
		/*
		 * Straensegment-Offset
		 */
		if (selektion.equals(_bezStrassenSegmentOffset))
		{
			aktionenStrassenSegmentOffset();

		}

		/*
		 * Betriebskilometer
		 */
		if (selektion.equals(_bezBetriebskilometer))
		{
			aktionenBetriebskilometer();

		}

		/*
		 * ASB-Ortsangabe
		 */
		if (selektion.equals(_bezAsbOrtsangabe))
		{
			// _auswahl = OrtsangabeEnumeratorAuswahl.asborstangabe;
			aktionenAsbOrtsangabe();
		}

		/*
		 * Knoten-Offset
		 */
		if (selektion.equals(_bezKnotenOffset))
		{
			aktionenKnotenOffset();

		}
	}

	/**
	 * Versorgt die Elemente des Tabs Straensegment-Offset mit den notwendigen
	 * Funktionalitten.
	 */
	private void aktionenStrassenSegmentOffset()
	{
		_auswahl = OrtsangabeEnumeratorAuswahl.strassensegment;
		if ((_filterStrassenSegmentOffset == null) && (_connection != null))
		{
			_filterStrassenSegmentOffset = new FilterStrassenSegmentOffset(
					(ClientDavConnection) _connection);

			/*
			 * ComboViwer Straen
			 */
			if (_cvStrassenSegmentStrasse == null)
			{
				_cvStrassenSegmentStrasse = new ComboViewer(
						_comboStrassenSegmentStrasse);
				_cvStrassenSegmentStrasse
						.setContentProvider(new ArrayContentProvider());
				_cvStrassenSegmentStrasse.setLabelProvider(new LabelProvider());
			}

			_cvStrassenSegmentStrasse.setInput(_filterStrassenSegmentOffset
					.getStrassen());

			/*
			 * SelectionListener Straen
			 */
			ISelectionChangedListener selectionChangedListenerStrasse = new ISelectionChangedListener()
			{

				@Override
				public void selectionChanged(SelectionChangedEvent event)
				{
					ISelection selection = event.getSelection();
					if (selection instanceof StructuredSelection)
					{
						StructuredSelection ss = (StructuredSelection) selection;
						Object[] objects = ss.toArray();
						if (objects.length > 0)
						{
							Object object = objects[0];
							if (object instanceof Strasse)
							{
								Strasse strasse = (Strasse) object;
								_filterStrassenSegmentOffset.setPidSelektierteStrasse(strasse
										.getPid());
								_cvStrassenSegmentStrassenSegment
										.setInput(_filterStrassenSegmentOffset
												.getStrassenSegmenteGefiltert(strasse.getPid()));

								// LinkedList<StrassenSegment> strassenSegmenteGefiltert =
								// _filterStrassenSegmentOffset
								// .getStrassenSegmenteGefiltert(strasse.getPid());

							}
						}
					}
				}
			};

			_cvStrassenSegmentStrasse
					.addSelectionChangedListener(selectionChangedListenerStrasse);

			/*
			 * ComboViewer StraenSegmente
			 */

			if (_cvStrassenSegmentStrassenSegment == null)
			{
				_cvStrassenSegmentStrassenSegment = new ComboViewer(
						_comboStrassenSegmentStrassenSegment);
				_cvStrassenSegmentStrassenSegment
						.setContentProvider(new ArrayContentProvider());
				_cvStrassenSegmentStrassenSegment.setLabelProvider(new LabelProvider());
			}

			/*
			 * SelectionListener StraenSegmente
			 */
			ISelectionChangedListener selectionChangedListenerStrassenSegmente = new ISelectionChangedListener()
			{

				@Override
				public void selectionChanged(SelectionChangedEvent event)
				{
					ISelection selection = event.getSelection();
					if (selection instanceof StructuredSelection)
					{
						StructuredSelection ss = (StructuredSelection) selection;
						Object[] objects = ss.toArray();
						if (objects.length > 0)
						{
							Object object = objects[0];
							if (object instanceof StrassenSegment)
							{
								StrassenSegment strassenSegment = (StrassenSegment) object;
								_filterStrassenSegmentOffset
										.setPidSelektiertesStrassenSegment(strassenSegment.getPid());
							}
						}
					}

				}

			};

			_cvStrassenSegmentStrassenSegment
					.addSelectionChangedListener(selectionChangedListenerStrassenSegmente);

			/*
			 * TEXTFELD
			 */
			Listener listenerEingabe = new Listener()
			{

				@Override
				public void handleEvent(Event e)
				{
					String string = e.text;
					char[] chars = new char[string.length()];
					string.getChars(0, chars.length, chars, 0);
					for (int i = 0; i < chars.length; i++)
					{
						if (!(('0' <= chars[i] && chars[i] <= '9') || (',' == chars[i])))
						{
							e.doit = false;
							return;
						} else
						{
							// Es wird die Eingabe mehrerer Kommas verhindert
							String text = _textStrassenSegmentKilometer.getText() + string;
							int first = text.indexOf(",");
							int last = text.lastIndexOf(",");
							_debug.fine(text + " -> " + first + " : " + last);
							if (first != last)
							{
								e.doit = false;
								return;
							}
							// System.out.println(textBetriebskilometerBetriebskilometer
							// .getText());
						}
					}

				}

			};

			_textStrassenSegmentKilometer.addListener(SWT.Verify, listenerEingabe);

			_textStrassenSegmentKilometer.addModifyListener(new ModifyListener()
			{

				public void modifyText(ModifyEvent e)
				{
					// Long l = new Long(0);
					Float f = new Float(0.0);

					if (_textStrassenSegmentKilometer.getText().length() > 0)
					{
						// l = Long.valueOf(textStrassenSegmentKilometer.getText());

						String text = _textStrassenSegmentKilometer.getText();

						f = Float.valueOf(text.replace(",", "."));

						// _filterStrassenSegmentOffset.setKm(l);
						_filterStrassenSegmentOffset.setKm(f);
					} else
					{
						// l = new Long(0);
						f = new Float(0.0);
					}

					// _debug.finest(getClass().getName() + " LONG: " + l.toString());
					_debug.finest(getClass().getName() + " FLOAT: " + f.toString());

					if (_filterStrassenSegmentOffset != null)
					{
						ISelection selection = _cvStrassenSegmentStrassenSegment
								.getSelection();
						if (selection instanceof StructuredSelection)
						{
							StructuredSelection s = (StructuredSelection) selection;
							Object firstElement = s.getFirstElement();
							if (firstElement instanceof StrassenSegment)
							{
								StrassenSegment strassenSegment = (StrassenSegment) firstElement;
								// if (_filterStrassenSegmentOffset.pruefeKilometerSegment(l,
								// strassenSegment.getPid()))
								if (_filterStrassenSegmentOffset.pruefeKilometerSegment(f,
										strassenSegment.getPid()))
								{
									// _filterStrassenSegmentOffset.setKm(l);
									_filterStrassenSegmentOffset.setKm(f);
									_labelStrassenSegmentStrasseLaengenwarnung.setText(" ");
									_labelStrassenSegmentStrasseLaengenwarnung.pack(true);
								} else
								{
									_debug.fine(getClass().getName()
											+ " => KM passt nicht ins Segment");

									_labelStrassenSegmentStrasseLaengenwarnung
											.setForeground(Display.getDefault().getSystemColor(
													SWT.COLOR_RED));
									_labelStrassenSegmentStrasseLaengenwarnung
											.setText("Die Maximallnge des Segments wurde berschritten!");
									_labelStrassenSegmentStrasseLaengenwarnung.pack(true);
								}
							}
						}

					}

				}

			});

		}
	}

	/**
	 * Versorgt die Elemente des Tabs Betriebskilometer mit den notwendigen
	 * Funktionalitten.
	 */
	private void aktionenBetriebskilometer()
	{
		_auswahl = OrtsangabeEnumeratorAuswahl.betriebskilometer;

		if (_filterBetriebskilometer == null)
		{
			_filterBetriebskilometer = new FilterBetriebskilometer(
					(ClientDavConnection) _connection);
		}

		/*
		 * ComboViewer fr die Strae
		 */
		if (_cvBetriebsKilometerStrasse == null)
		{
			_cvBetriebsKilometerStrasse = new ComboViewer(
					_comboBetriebskilometerStrae);
			_cvBetriebsKilometerStrasse
					.setContentProvider(new ArrayContentProvider());
			_cvBetriebsKilometerStrasse.setLabelProvider(new LabelProvider());

			LinkedList<Strasse> strassen = _filterBetriebskilometer.getStrassen();
			_cvBetriebsKilometerStrasse.setInput(strassen);
		}

		/*
		 * Betriebskilometer fr die Strae erstellen
		 */
		if (_listenerBetriebsKilometerStrasse == null)
		{

			_listenerBetriebsKilometerStrasse = new ISelectionChangedListener()
			{
				@Override
				public void selectionChanged(SelectionChangedEvent event)
				{
					ISelection selection = event.getSelection();
					if (selection instanceof StructuredSelection)
					{
						StructuredSelection ss = (StructuredSelection) selection;
						@SuppressWarnings("unchecked")
						List list = ss.toList();
						if (list.size() > 0)
						{
							Object object = list.get(0);
							_debug.fine("Object: " + object.getClass().getName());
							if (object instanceof Strasse)
							{
								Strasse strasseBk = (Strasse) object;
								_filterBetriebskilometer.setSelektierteStrasse(strasseBk
										.getPid());

								_cvBetriebsKilometerRichtung.setSelection(null);

								if ((strasseBk.getNummer() % 2) == 0)
								{
									// Gerade == OST + West
									LinkedList<de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung> richtung = new LinkedList<de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung>();

									de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung west = de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung.WEST;
									de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung ost = de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung.OST;

									richtung.add(west);
									richtung.add(ost);
									_cvBetriebsKilometerRichtung.setInput(richtung);
								} else
								{
									// Ungerade == NORD + SD

									LinkedList<de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung> richtung = new LinkedList<de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung>();

									de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung nord = de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung.NORD;
									de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung sued = de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung.SD;

									richtung.add(nord);
									richtung.add(sued);

									_cvBetriebsKilometerRichtung.setInput(richtung);
								}

								_textBetriebskilometerBetriebskilometer.setText("");
								_cvBetriebsKilometerBlockNummer.setSelection(null);

							}
						}
					}
				}

			};

			_cvBetriebsKilometerStrasse
					.addSelectionChangedListener(_listenerBetriebsKilometerStrasse);
		}

		/*
		 * ComboViewer fr die Richtung
		 */

		if (_cvBetriebsKilometerRichtung == null)
		{
			_cvBetriebsKilometerRichtung = new ComboViewer(
					_comboBetriebskilometerRichtung);
			_cvBetriebsKilometerRichtung
					.setContentProvider(new ArrayContentProvider());
			_cvBetriebsKilometerRichtung.setLabelProvider(new LabelProvider());

			LinkedList<de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung> richtung = new LinkedList<de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung>();

			de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung nord = de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung.NORD;
			de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung sued = de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung.SD;
			de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung west = de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung.WEST;
			de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung ost = de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung.OST;

			richtung.add(nord);
			richtung.add(sued);
			richtung.add(west);
			richtung.add(ost);
			_cvBetriebsKilometerRichtung.setInput(richtung);
		}

		/*
		 * Listener fr nderungen in der ComboBox Richtung erstellen
		 */
		if (_listenerBetriebskilometerRichtung == null)
		{
			_listenerBetriebskilometerRichtung = new ISelectionChangedListener()
			{

				@Override
				public void selectionChanged(SelectionChangedEvent event)
				{
					ISelection selection = event.getSelection();
					if (selection instanceof StructuredSelection)
					{
						StructuredSelection ss = (StructuredSelection) selection;
						Object[] objects = ss.toArray();
						if (objects.length > 0)
						{
							Object object = objects[0];
							if (object instanceof de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung)
							{
								de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung ri = (de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung) object;
								// _debug.fine("Richtung: " + ri.toString());
								_filterBetriebskilometer.setSelektierteRichtung(ri);
							}
						}
					}

				}

			};

			_cvBetriebsKilometerRichtung
					.addSelectionChangedListener(_listenerBetriebskilometerRichtung);
		}

		/*
		 * Prfung der Eingabe der Betriebskilometer. Es knnen nur ZAHLEN
		 * eingegeben werden.
		 */
		if (_listenerTextBetriebskilometerBetriebskilometer == null)
		{
			_listenerTextBetriebskilometerBetriebskilometer = new Listener()
			{

				@Override
				public void handleEvent(Event e)
				{
					String string = e.text;
					char[] chars = new char[string.length()];
					string.getChars(0, chars.length, chars, 0);
					for (int i = 0; i < chars.length; i++)
					{
						if (!('0' <= chars[i] && chars[i] <= '9'))
						{
							e.doit = false;
							return;
						} else
						{
							// System.out.println(textBetriebskilometerBetriebskilometer
							// .getText());
						}
					}

				}

			};

			_textBetriebskilometerBetriebskilometer.addListener(SWT.Verify,
					_listenerTextBetriebskilometerBetriebskilometer);

		}

		/*
		 * ComboViewer fr die Combobox BetriebsKilometerBlockNummer
		 */
		if (_cvBetriebsKilometerBlockNummer == null)
		{
			_cvBetriebsKilometerBlockNummer = new ComboViewer(
					_comboBetriebskilometerBlocknummer);
			_cvBetriebsKilometerBlockNummer
					.setContentProvider(new ArrayContentProvider());
			_cvBetriebsKilometerBlockNummer.setLabelProvider(new LabelProvider());

			_cvBetriebsKilometerBlockNummer
					.addSelectionChangedListener(new ISelectionChangedListener()
					{

						@Override
						public void selectionChanged(SelectionChangedEvent event)
						{
							ISelection selection = event.getSelection();
							if (selection instanceof StructuredSelection)
							{
								StructuredSelection structuredSelection = (StructuredSelection) selection;
								Object object = structuredSelection.getFirstElement();
								if (object != null)
								{
									if (object instanceof String)
									{
										String wert = (String) object;
										_filterBetriebskilometer.setSelektierteBlockNummer(wert);

										// Ausblenden der Warnung, dass man etwas selektieren muss.
										_labelBetriebskilometerWarnung.setVisible(false);
									}
								}
							}

						}

					});

		}

		/*
		 * Listener zur bernahme des Genderten Textes im Feld Betriebskilometer
		 */
		if (_modifyListenerTextBetriebskilometerBetriebskilometer == null)
		{
			_modifyListenerTextBetriebskilometerBetriebskilometer = new ModifyListener()
			{

				@Override
				public void modifyText(ModifyEvent e)
				{
					String text = _textBetriebskilometerBetriebskilometer.getText();

					if ((text != null) && (text.length() > 0))
					{
						long bkWert = Long.valueOf(text);

						_filterBetriebskilometer.setBetriebskilometer(bkWert);

						LinkedList<String> vorhandeneBlockNummern = _filterBetriebskilometer
								.getVorhandeneBlockNummern();

						_cvBetriebsKilometerBlockNummer.setInput(vorhandeneBlockNummern);
						if (_comboBetriebskilometerBlocknummer.getItemCount() == 1)
						{

							_cvBetriebsKilometerBlockNummer
									.setSelection(new StructuredSelection(
											_cvBetriebsKilometerBlockNummer.getElementAt(0)));
						}

						// Einblenden des Warnhinweises, wenn der Betriebskilometer nicht
						// existiert.
						if (_comboBetriebskilometerBlocknummer.getItemCount() < 1)
						{
							_comboBetriebskilometerBlocknummer.setEnabled(false);
							_labelBetriebskilometerWarnung
									.setText("Der angegebene Betriebskilometer ist im LCL-Segment der Strae nicht vorhanden!");
							_labelBetriebskilometerWarnung.setVisible(true);

						} else if (_comboBetriebskilometerBlocknummer.getItemCount() > 1)
						{
							_comboBetriebskilometerBlocknummer.setEnabled(true);
							_labelBetriebskilometerWarnung
									.setText("Es stehen mehrere Blocknummern zur Auswahl. Sie mssen einen auswhlen.");
							_labelBetriebskilometerWarnung.setVisible(true);
						} else
						{
							_comboBetriebskilometerBlocknummer.setEnabled(false);
							_labelBetriebskilometerWarnung.setVisible(false);
						}
					}
				}

			};

			_textBetriebskilometerBetriebskilometer
					.addModifyListener(_modifyListenerTextBetriebskilometerBetriebskilometer);
		}

	}

	/**
	 * Versorgt die Elemente des Tabs ASB-Ortsangabe mit den notwendigen
	 * Funktionalitten.
	 */
	private void aktionenAsbOrtsangabe()
	{
		_auswahl = OrtsangabeEnumeratorAuswahl.asborstangabe;

		if (_filterAsbOrtsangabe == null)
		{
			_filterAsbOrtsangabe = new FilterAsbOrtsangabe(
					(ClientDavConnection) _connection);
		}

		/*
		 * ComboViewer fr ASB Strae
		 */
		if (_cvAsbStrasse == null)
		{
			_cvAsbStrasse = new ComboViewer(_comboAsbStrasse);
			_cvAsbStrasse.setContentProvider(new ArrayContentProvider());
			_cvAsbStrasse.setLabelProvider(new LabelProvider());

			LinkedList<Strasse> strassen = _filterAsbOrtsangabe.getStrassen();
			_cvAsbStrasse.setInput(strassen);
		}

		/*
		 * Listener fr den ComboViewer der Strae
		 */
		ISelectionChangedListener listenerAsbStrasse = new ISelectionChangedListener()
		{

			@Override
			public void selectionChanged(SelectionChangedEvent event)
			{
				ISelection selection = event.getSelection();
				if (selection instanceof StructuredSelection)
				{
					StructuredSelection ss = (StructuredSelection) selection;
					@SuppressWarnings("unchecked")
					List list = ss.toList();
					if (list.size() > 0)
					{
						Object object = list.get(0);
						if (object instanceof Strasse)
						{
							Strasse strasseAsb = (Strasse) object;
							_debug.fine("PID: " + strasseAsb.getPid());
							_filterAsbOrtsangabe.setSelektierteStrasse(strasseAsb.getPid());

							LinkedList<AsbKnotenKombiniert> verfuegbareAsbKnoten = _filterAsbOrtsangabe
									.getAsbOrtsangabe().getVonAuswahlfeldKombiniert();
							// LinkedList<AsbKnoten> verfuegbareAsbKnoten =
							// _filterAsbOrtsangabe
							// .getAsbOrtsangabe().getVerfuegbareAsbKnoten();

							_cvAsbAnfangsKnoten.setInput(verfuegbareAsbKnoten);

							_cvAsbAnfangsKnoten.setSelection(null);
							_cvAsbEndKnoten.setSelection(null);
							_textAsbStationierung.setText("");
							_labelAsbStationierungAusserhalbBereich.setVisible(false);
						}
					}
				}

			}

		};

		_cvAsbStrasse.addSelectionChangedListener(listenerAsbStrasse);

		/*
		 * VON-Netrknoten/Ast
		 */
		if (_cvAsbAnfangsKnoten == null)
		{
			_cvAsbAnfangsKnoten = new ComboViewer(_comboAsbAnfangsKnoten);
			_cvAsbAnfangsKnoten.setContentProvider(new ArrayContentProvider());
			_cvAsbAnfangsKnoten.setLabelProvider(new LabelProvider());
		}

		ISelectionChangedListener listenerAsbVon = new ISelectionChangedListener()
		{

			@Override
			public void selectionChanged(SelectionChangedEvent event)
			{
				ISelection selection = event.getSelection();
				if (selection instanceof StructuredSelection)
				{
					StructuredSelection ss = (StructuredSelection) selection;
					@SuppressWarnings("unchecked")
					List list = ss.toList();
					if (list.size() > 0)
					{
						Object object = list.get(0);

						// if (object instanceof AsbKnoten)
						// {
						// AsbKnoten asbVonKnoten = (AsbKnoten) object;
						//
						// _filterAsbOrtsangabe.setAnfangsKnoten(asbVonKnoten.getKnoten());
						// _filterAsbOrtsangabe.setAnfangsKnotenTmcRichtung(asbVonKnoten
						// .getTmcRichtung());
						//
						// LinkedList<AsbKnoten> erreichbareAsbKnoten = _filterAsbOrtsangabe
						// .getAsbOrtsangabe()
						// .bestimmeErreichbareAsbKnoten(asbVonKnoten);
						//
						// _cvAsbEndKnoten.setInput(erreichbareAsbKnoten);
						// _cvAsbEndKnoten.setSelection(null);
						// _textAsbStationierung.setText("");
						//
						// }
						if (object instanceof AsbKnotenKombiniert)
						{
							AsbKnotenKombiniert asbVonKnotenKombiniert = (AsbKnotenKombiniert) object;
							_filterAsbOrtsangabe.setAnfangsKnoten(asbVonKnotenKombiniert
									.getKnoten());

							LinkedList<AsbKnotenKombiniert> erreichbareAsbKnoten = _filterAsbOrtsangabe
									.getAsbOrtsangabe().bestimmeErreichbareAsbKnotenKombiniert(
											asbVonKnotenKombiniert).getListeKombinierteKnoten();

							_cvAsbEndKnoten.setInput(erreichbareAsbKnoten);
							_cvAsbEndKnoten.setSelection(null);
							_textAsbStationierung.setText("");
							_labelAsbStationierungAusserhalbBereich.setVisible(false);
						}
					}
				}
			}

		};

		_cvAsbAnfangsKnoten.addSelectionChangedListener(listenerAsbVon);

		/*
		 * Nach-Netzknoten/Ast
		 */
		if (_cvAsbEndKnoten == null)
		{
			_cvAsbEndKnoten = new ComboViewer(_comboAsbEndKnoten);
			_cvAsbEndKnoten.setContentProvider(new ArrayContentProvider());
			_cvAsbEndKnoten.setLabelProvider(new LabelProvider());
		}

		ISelectionChangedListener listenerAsbNach = new ISelectionChangedListener()
		{

			@Override
			public void selectionChanged(SelectionChangedEvent event)
			{
				ISelection selection = event.getSelection();
				if (selection instanceof StructuredSelection)
				{
					StructuredSelection ss = (StructuredSelection) selection;
					@SuppressWarnings("unchecked")
					List list = ss.toList();
					if (list.size() > 0)
					{
						Object object = list.get(0);

						if (object instanceof AsbKnotenKombiniert)
						{
							AsbKnotenKombiniert endKnoten = (AsbKnotenKombiniert) object;

							_filterAsbOrtsangabe.setEndKnoten(endKnoten.getKnoten());
							_textAsbStationierung.setText("");
							_labelAsbStationierungAusserhalbBereich.setVisible(false);
						}

					}
				}
			}

		};

		_cvAsbEndKnoten.addSelectionChangedListener(listenerAsbNach);

		/*
		 * Sicherstellen, dass im TextFeld textAsbStationierung nur Zahlen
		 * eingegeben werden
		 */
		Listener listenerTextAsbStationierung = new Listener()
		{

			@Override
			public void handleEvent(Event e)
			{
				String string = e.text;
				char[] chars = new char[string.length()];
				string.getChars(0, chars.length, chars, 0);
				for (int i = 0; i < chars.length; i++)
				{
					if (!('0' <= chars[i] && chars[i] <= '9'))
					{
						e.doit = false;
						return;
					}
				}

			}
		};

		_textAsbStationierung.addListener(SWT.Verify, listenerTextAsbStationierung);

		ModifyListener modifyListenerTextAsbStationierung = new ModifyListener()
		{

			@Override
			public void modifyText(ModifyEvent e)
			{
				// String text = _textAsbStationierung.getText();
				pruefeAsbStationierungWaehrendTextfeldEingabe();

				// if ((text != null) && (text.length() > 0))
				// {
				//
				// try
				// {
				// long wert = Long.valueOf(text);
				//
				// AsbKnotenKombiniert asbKnotenAnfangStationierung = null;
				// AsbKnotenKombiniert asbKnotenEndeStationierung = null;
				//
				// StructuredSelection ssAnfang = (StructuredSelection)
				// _cvAsbAnfangsKnoten
				// .getSelection();
				// Object firstElementAnfang = ssAnfang.getFirstElement();
				// if ((firstElementAnfang != null)
				// && (firstElementAnfang instanceof AsbKnotenKombiniert))
				// {
				// asbKnotenAnfangStationierung = (AsbKnotenKombiniert)
				// firstElementAnfang;
				// }
				//
				// StructuredSelection ssEnde = (StructuredSelection) _cvAsbEndKnoten
				// .getSelection();
				// Object firstElementEnde = ssEnde.getFirstElement();
				// if ((firstElementEnde != null)
				// && (firstElementEnde instanceof AsbKnotenKombiniert))
				// {
				// asbKnotenEndeStationierung = (AsbKnotenKombiniert) firstElementEnde;
				// }
				//
				// /*
				// * Prfen des Wertes im Stationierungsfeld, wenn Anfangs und
				// * Endknoten ausgewhlt wurden.
				// */
				// if ((asbKnotenAnfangStationierung != null)
				// && (asbKnotenEndeStationierung != null))
				// {
				// BereichsInfoAsbKnotenKombiniert stationierungKombiniert =
				// _filterAsbOrtsangabe
				// .getAsbOrtsangabe().pruefeAsbStationierungKombiniert(
				// asbKnotenAnfangStationierung, asbKnotenEndeStationierung,
				// wert);
				//
				// if (!stationierungKombiniert.istInBereich())
				// {
				// _labelAsbStationierungAusserhalbBereich.setVisible(true);
				// } else
				// {
				// _filterAsbOrtsangabe.setStationierung(wert);
				// _labelAsbStationierungAusserhalbBereich.setVisible(false);
				// }
				//
				// }
				//
				// } catch (IllegalFormatException ife)
				// {
				// ife.printStackTrace();
				// }
				//
				// }
			}
		};

		_textAsbStationierung.addModifyListener(modifyListenerTextAsbStationierung);

		// if (_cvAsbStrasse == null)
		// {
		// _cvAsbStrasse = new ComboViewer(_comboAsbStrasse);
		// _cvAsbStrasse.setContentProvider(new ArrayContentProvider());
		// _cvAsbStrasse.setLabelProvider(new LabelProvider());
		//
		// LinkedList<Strasse> strassen = _filterAsbOrtsangabe.getStrassen();
		// _cvAsbStrasse.setInput(strassen);
		// }

		if (_cvAsbStationierungsrichtung == null)
		{
			_cvAsbStationierungsrichtung = new ComboViewer(
					_comboAsbStationierungsrichtung);
			_cvAsbStationierungsrichtung
					.setContentProvider(new ArrayContentProvider());
			_cvAsbStationierungsrichtung.setLabelProvider(new LabelProvider());
			LinkedList<EnumAsbVerkehrsRichtung> asbVerkehrsRichtungen = new LinkedList<EnumAsbVerkehrsRichtung>();
			asbVerkehrsRichtungen
					.add(EnumAsbVerkehrsRichtung.INSTATIONIERUNGSRICHTUNG);
			asbVerkehrsRichtungen
					.add(EnumAsbVerkehrsRichtung.GEGENSTATIONIERUNGSRICHTUNG);
			asbVerkehrsRichtungen.add(EnumAsbVerkehrsRichtung.BEIDERICHTUNGEN);
			asbVerkehrsRichtungen.add(EnumAsbVerkehrsRichtung.UNBEKANNT);
			_cvAsbStationierungsrichtung.setInput(asbVerkehrsRichtungen);

			_cvAsbStationierungsrichtung.setSelection(new StructuredSelection(
					EnumAsbVerkehrsRichtung.UNBEKANNT));

			ISelectionChangedListener listenerAsbStationierung = new ISelectionChangedListener()
			{
				@Override
				public void selectionChanged(SelectionChangedEvent event)
				{
					ISelection selection = event.getSelection();
					if (selection instanceof StructuredSelection)
					{
						StructuredSelection ss = (StructuredSelection) selection;
						Object firstElement = ss.getFirstElement();
						if (firstElement instanceof EnumAsbVerkehrsRichtung)
						{
							EnumAsbVerkehrsRichtung richtung = (EnumAsbVerkehrsRichtung) firstElement;
							_filterAsbOrtsangabe.setAsbVerkehrsRichtung(richtung);
							pruefeAsbStationierung(richtung);

						}
					}
				}

			};
			_cvAsbStationierungsrichtung
					.addSelectionChangedListener(listenerAsbStationierung);

		}
	}

	private void pruefeAsbStationierungWaehrendTextfeldEingabe()
	{
		ISelection selection = _cvAsbStationierungsrichtung.getSelection();
		if (selection instanceof StructuredSelection)
		{
			StructuredSelection ss = (StructuredSelection) selection;
			Object firstElement = ss.getFirstElement();
			if (firstElement instanceof EnumAsbVerkehrsRichtung)
			{
				EnumAsbVerkehrsRichtung richtung = (EnumAsbVerkehrsRichtung) firstElement;
				pruefeAsbStationierung(richtung);
			}
		}

	}

	private void pruefeAsbStationierung(EnumAsbVerkehrsRichtung richtung)
	{
		String text = _textAsbStationierung.getText();

		if ((text != null) && (text.length() > 0))
		{

			try
			{
				long wert = Long.valueOf(text);

				AsbKnotenKombiniert asbKnotenAnfangStationierung = null;
				AsbKnotenKombiniert asbKnotenEndeStationierung = null;

				StructuredSelection ssAnfang = (StructuredSelection) _cvAsbAnfangsKnoten
						.getSelection();
				Object firstElementAnfang = ssAnfang.getFirstElement();
				if ((firstElementAnfang != null)
						&& (firstElementAnfang instanceof AsbKnotenKombiniert))
				{
					asbKnotenAnfangStationierung = (AsbKnotenKombiniert) firstElementAnfang;
				}

				StructuredSelection ssEnde = (StructuredSelection) _cvAsbEndKnoten
						.getSelection();
				Object firstElementEnde = ssEnde.getFirstElement();
				if ((firstElementEnde != null)
						&& (firstElementEnde instanceof AsbKnotenKombiniert))
				{
					asbKnotenEndeStationierung = (AsbKnotenKombiniert) firstElementEnde;
				}

				/*
				 * Prfen des Wertes im Stationierungsfeld, wenn Anfangs und Endknoten
				 * ausgewhlt wurden.
				 */
				if ((asbKnotenAnfangStationierung != null)
						&& (asbKnotenEndeStationierung != null))
				{
					BereichsInfoAsbKnotenKombiniert bereichsInfoStationierungKombiniert = _filterAsbOrtsangabe
							.getAsbOrtsangabe().pruefeAsbStationierungKombiniert(
									asbKnotenAnfangStationierung, asbKnotenEndeStationierung,
									wert);

					if (richtung.equals(EnumAsbVerkehrsRichtung.BEIDERICHTUNGEN))
					{
						boolean[] erg = bereichsInfoStationierungKombiniert
								.istInBereichGefiltert(richtung);
						if (erg.length == 2)
						{
							boolean inStatRichtung = erg[0];
							boolean gegenStatRichung = erg[1];

							if (inStatRichtung || gegenStatRichung)
							{
								_filterAsbOrtsangabe.setStationierung(wert);
								_labelAsbStationierungAusserhalbBereich.setVisible(false);
							} else
							{
//								long[] minMax = FilterAsbOrtsangabe.ermittleMinMax(
//										bereichsInfoStationierungKombiniert
//												.getBereichsInfoAsbKnotenTmcPos(),
//										bereichsInfoStationierungKombiniert
//												.getBereichsInfoAsbKnotenTmcNeg(), richtung);

							  long[] minMax = null;
							  
								if (minMax != null)
								{
									_labelAsbStationierungAusserhalbBereich
											.setText("Die Stationierung muss zwischen " + minMax[0]
													+ " und " + minMax[1] + " m liegen!");

								} else
								{
									_labelAsbStationierungAusserhalbBereich
											.setText("Bereich fr Stationierung nicht ermittelbar!");
								}

								_labelAsbStationierungAusserhalbBereich.pack();
								_labelAsbStationierungAusserhalbBereich.setVisible(true);

							}

						}
					} else if ((richtung
							.equals(EnumAsbVerkehrsRichtung.INSTATIONIERUNGSRICHTUNG))
							|| (richtung
									.equals(EnumAsbVerkehrsRichtung.GEGENSTATIONIERUNGSRICHTUNG)))
					{
						auswertungRichtung(bereichsInfoStationierungKombiniert, richtung,
								wert);

					} else if (richtung.equals(EnumAsbVerkehrsRichtung.UNBEKANNT))
					{
						if (bereichsInfoStationierungKombiniert.istInBereich())
						{
							_filterAsbOrtsangabe.setStationierung(wert);
							_labelAsbStationierungAusserhalbBereich.setVisible(false);
						} else
						{
//							long[] minMax = FilterAsbOrtsangabe.ermittleMinMax(
//									bereichsInfoStationierungKombiniert
//											.getBereichsInfoAsbKnotenTmcPos(),
//									bereichsInfoStationierungKombiniert
//											.getBereichsInfoAsbKnotenTmcNeg(), richtung);
							
							long[] minMax = null;

							if (minMax != null)
							{
								_labelAsbStationierungAusserhalbBereich
										.setText("Die Stationierung muss zwischen " + minMax[0]
												+ " und " + minMax[1] + " m liegen!");

							} else
							{
								_labelAsbStationierungAusserhalbBereich
										.setText("Bereich fr Stationierung nicht ermittelbar!");
							}

							_labelAsbStationierungAusserhalbBereich.pack();
							_labelAsbStationierungAusserhalbBereich.setVisible(true);

						}

					}

				}

			} catch (IllegalFormatException ife)
			{
				ife.printStackTrace();
			}

		}
	}

	/**
	 * bernimmt die Auswertung der Richtung fr
	 * {@link EnumAsbVerkehrsRichtung#INSTATIONIERUNGSRICHTUNG} und
	 * {@link EnumAsbVerkehrsRichtung#GEGENSTATIONIERUNGSRICHTUNG}.
	 * 
	 * @param bereichsInfoStationierungKombiniert
	 *          Kombinierte {@link BereichsInfoAsbKnotenKombiniert} fr die
	 *          {@link AsbKnoten} in positiver und negativer TMC-Richtung.
	 * @param richtung
	 *          Zu prfende {@link EnumAsbVerkehrsRichtung}. Funktioniert nur fr
	 *          {@link EnumAsbVerkehrsRichtung#INSTATIONIERUNGSRICHTUNG} und
	 *          {@link EnumAsbVerkehrsRichtung#GEGENSTATIONIERUNGSRICHTUNG}.
	 * @param wertStationierung
	 *          Wert der Stationierung in Metern.
	 */
	private void auswertungRichtung(
			BereichsInfoAsbKnotenKombiniert bereichsInfoStationierungKombiniert,
			EnumAsbVerkehrsRichtung richtung, long wertStationierung)
	{
//		String msgKeineBestimmung = "";

		boolean[] erg = bereichsInfoStationierungKombiniert
				.istInBereichGefiltert(richtung);
		if (erg.length == 2)
		{
			boolean inBereichTmcPos = erg[0];
			boolean inBereichTmcNeg = erg[1];

			if (inBereichTmcPos || inBereichTmcNeg)
			{
				_filterAsbOrtsangabe.setStationierung(wertStationierung);
				_labelAsbStationierungAusserhalbBereich.setVisible(false);
			} else
			{
//				AsbKnoten asbKnotenVonPos = null;
//				AsbKnoten asbKnotenBisPos = null;
//				AsbKnoten asbKnotenVonNeg = null;
//				AsbKnoten asbKnotenBisNeg = null;
//
//				if (bereichsInfoStationierungKombiniert
//						.getBereichsInfoAsbKnotenTmcPos() != null)
//				{
//					asbKnotenVonPos = bereichsInfoStationierungKombiniert
//							.getBereichsInfoAsbKnotenTmcPos().getAsbKnotenVon();
//					asbKnotenBisPos = bereichsInfoStationierungKombiniert
//							.getBereichsInfoAsbKnotenTmcPos().getAsbKnotenBis();
//				}
//
//				if (bereichsInfoStationierungKombiniert
//						.getBereichsInfoAsbKnotenTmcNeg() != null)
//				{
//					asbKnotenVonNeg = bereichsInfoStationierungKombiniert
//							.getBereichsInfoAsbKnotenTmcNeg().getAsbKnotenVon();
//					asbKnotenBisNeg = bereichsInfoStationierungKombiniert
//							.getBereichsInfoAsbKnotenTmcNeg().getAsbKnotenBis();
//				}
//
//				String a = "";
//
//				if ((asbKnotenVonPos != null) && (asbKnotenBisPos != null)
//						&& (asbKnotenVonNeg != null) && (asbKnotenBisNeg != null))
//				{
//					String out = "======START======\n";
//					out = out + "asbKnotenVonPos: "
//							+ asbKnotenVonPos.getAsbVerkehrsRichtung() + "\n";
//					out = out + "asbKnotenBisPos: "
//							+ asbKnotenBisPos.getAsbVerkehrsRichtung() + "\n";
//					out = out + "asbKnotenVonNeg: "
//							+ asbKnotenVonNeg.getAsbVerkehrsRichtung() + "\n";
//					out = out + "asbKnotenVonNeg: "
//							+ asbKnotenVonNeg.getAsbVerkehrsRichtung() + "\n";
//					out = out + "=================\n";
//					System.out.println(out);
//
//					// Knoten in positiver TMC-Richtung haben unterschiedliche
//					// ASB-Verkehrsrichtung
//					if (!asbKnotenVonPos.getAsbVerkehrsRichtung().equals(
//							asbKnotenBisPos.getAsbVerkehrsRichtung()))
//					{
//						msgKeineBestimmung = msgKeineBestimmung
//								+ "Die Verkehrsrichtung der AsbKnoten in positiver TMC Richtung ist nicht gleich ("
//								+ asbKnotenVonPos + "->" + asbKnotenBisPos + ").\n";
//					}
//					// Knoten in negativer TMC-Richtung haben unterschiedliche
//					// ASB-Verkehrsrichtungen
//					if (!asbKnotenVonNeg.getAsbVerkehrsRichtung().equals(
//							asbKnotenBisNeg.getAsbVerkehrsRichtung()))
//					{
//						msgKeineBestimmung = msgKeineBestimmung
//								+ "Die Verkehrsrichtung der AsbKnoten in negativer TMC Richtung ist nicht gleich ("
//								+ asbKnotenVonNeg + "->" + asbKnotenBisNeg + ").\n";
//					}// Die Richtung der Knoten in der positiven und der negativen
//					// TMC-Richtung ist identisch.
//					else if ((asbKnotenVonPos.getAsbVerkehrsRichtung()
//							.equals(asbKnotenVonNeg.getAsbVerkehrsRichtung()))
//							|| (asbKnotenBisPos.getAsbVerkehrsRichtung()
//									.equals(asbKnotenBisNeg.getAsbVerkehrsRichtung())))
//					{
//						msgKeineBestimmung = msgKeineBestimmung
//								+ "Die Verkehrsrichtung der AsbKnoten in positiver und negativer TMC Richtung ist identisch!\n";
//					}
//
//					if ((msgKeineBestimmung != null) && (msgKeineBestimmung.length() > 0))
//					{
//						msgKeineBestimmung = "ASB-Ortsangabe - Keine Bestimmung mglich:\n"
//								+ msgKeineBestimmung;
//						msgKeineBestimmung = msgKeineBestimmung + "Stationierung: "
//								+ wertStationierung + " m.\n";
//						msgKeineBestimmung = msgKeineBestimmung + "asbKnotenVonPos: "
//								+ asbKnotenVonPos + "\n";
//						msgKeineBestimmung = msgKeineBestimmung + "asbKnotenBisPos: "
//								+ asbKnotenBisPos + "\n";
//						msgKeineBestimmung = msgKeineBestimmung + "asbKnotenVonNeg: "
//								+ asbKnotenVonNeg + "\n";
//						msgKeineBestimmung = msgKeineBestimmung + "asbKnotenBisNeg: "
//								+ asbKnotenBisNeg + "\n";
//						msgKeineBestimmung = msgKeineBestimmung
//								+ _filterAsbOrtsangabe.toString() + "\n";
//						System.out.println(msgKeineBestimmung);
//					}
//				} else
//				{
//					String msg = "";
//					msg = msg
//							+ "Das Ergebnis fr einen AsbKnoten der Auswertung ist null\n";
//					msg = msg + _filterAsbOrtsangabe.toString();
//					Activator.getDefault().getLog().log(
//							new Status(Status.INFO, Activator.PLUGIN_ID, msg));
//				}

				if (!inBereichTmcPos && !inBereichTmcNeg)
				{
//					long[] minMax = FilterAsbOrtsangabe.ermittleMinMax(
//							bereichsInfoStationierungKombiniert
//									.getBereichsInfoAsbKnotenTmcPos(),
//							bereichsInfoStationierungKombiniert
//									.getBereichsInfoAsbKnotenTmcNeg(), richtung);
					
					long[] minMax = null;

					String labelText = null;
					if (minMax != null)
					{
						labelText = "Die Stationierung muss zwischen " + minMax[0]
								+ " und " + minMax[1] + " m liegen!";
					} else
					{
						// labelText = "Es ist keine Bereichsbestimmung mglich, siehe
						// Log.";
						// Activator.getDefault().getLog()
						// .log(
						// new Status(Status.INFO, Activator.PLUGIN_ID,
						// msgKeineBestimmung));

						labelText = "Es ist keine Bereichsbestimmung mglich da keine Infos in \""
								+ richtung + "\" vorhanden.";

					}

					_labelAsbStationierungAusserhalbBereich.setText(labelText);
					_labelAsbStationierungAusserhalbBereich.pack();
					_labelAsbStationierungAusserhalbBereich.setVisible(true);
				}

				String info = "";
				info = info + "inBereichTmcPos: " + inBereichTmcPos + "\n";
				info = info + "inBereichTmcNeg: " + inBereichTmcNeg + "\n";

				System.out.println(info);

			}

		}
	}

	/**
	 * Versorgt die Elemente des Tabs Knoten-Offset mit den notwendigen
	 * Funktionalitten.
	 */
	private void aktionenKnotenOffset()
	{
		_auswahl = OrtsangabeEnumeratorAuswahl.knotenoffset;

		if ((_filterKnoten == null) && (_connection != null))
		{
			_filterKnoten = new FilterKnoten((ClientDavConnection) _connection);
		}

		/*
		 * Straen versorgen
		 */
		if (_cvKnotenOffsetStrasse == null)
		{

			_cvKnotenOffsetStrasse = new ComboViewer(_comboKnotenOffsetStrasse);
			_cvKnotenOffsetStrasse.setContentProvider(new ArrayContentProvider());
			_cvKnotenOffsetStrasse.setLabelProvider(new LabelProvider());

			_cvKnotenOffsetStrasse.setInput(_filterKnoten.getStrassen());

		}

		ISelectionChangedListener listenerKnotenOffsetStrasse = new ISelectionChangedListener()
		{

			@Override
			public void selectionChanged(SelectionChangedEvent event)
			{
				ISelection selection = event.getSelection();
				if (selection instanceof StructuredSelection)
				{
					StructuredSelection ss = (StructuredSelection) selection;
					@SuppressWarnings("unchecked")
					List list = ss.toList();
					if (list.size() > 0)
					{
						Object object = list.get(0);
						if (object instanceof Strasse)
						{
							Strasse strasseKnoten = (Strasse) object;
							_filterKnoten.setPidStrasse(strasseKnoten.getPid());

							if (_cvKnotenOffsetKnoten != null)
							{
								_cvKnotenOffsetKnoten.setInput(_filterKnoten
										.getKnoten(strasseKnoten.getPid()));
							}

						}
					}
				}

			}

		};

		_cvKnotenOffsetStrasse
				.addSelectionChangedListener(listenerKnotenOffsetStrasse);

		/*
		 * Knoten versorgen
		 */
		if (_cvKnotenOffsetKnoten == null)
		{
			_cvKnotenOffsetKnoten = new ComboViewer(_comboKnotenOffsetKnoten);

			_cvKnotenOffsetKnoten.setContentProvider(new ArrayContentProvider());
			_cvKnotenOffsetKnoten.setLabelProvider(new LabelProvider());
		}

		ISelectionChangedListener listenerKnotenOffsetKnoten = new ISelectionChangedListener()
		{

			@Override
			public void selectionChanged(SelectionChangedEvent event)
			{
				ISelection selection = event.getSelection();
				if (selection instanceof StructuredSelection)
				{
					StructuredSelection ss = (StructuredSelection) selection;
					@SuppressWarnings("unchecked")
					List list = ss.toList();
					if (list.size() > 0)
					{
						Object object = list.get(0);
						if (object instanceof StrassenKnoten)
						{
							StrassenKnoten knoten = (StrassenKnoten) object;
							_filterKnoten.setPidKnoten(knoten.getPid());
						}
					}
				}
			}

		};

		_cvKnotenOffsetKnoten
				.addSelectionChangedListener(listenerKnotenOffsetKnoten);

		/*
		 * Erfassen der Kilometer
		 */
		_textKnotenOffsetKilometer.addListener(SWT.Verify, new Listener()
		{

			@Override
			public void handleEvent(Event event)
			{

				String text = event.text;
				if (text.length() > 0)
				{
					char c = text.charAt(text.length() - 1);
					if (c != '.')
					{
						if (!('0' <= c && c <= '9'))
						{
							event.doit = false;
							return;
						}
					}

					/*
					 * Prfen auf mehrfaches vorkommen von "."
					 */
					if (c == '.')
					{
						String textFeld = _textKnotenOffsetKilometer.getText();

						int count = 0;
						for (int i = 0; i < textFeld.length(); i++)
						{
							_debug.fine("T: " + textFeld.charAt(i));

							if (textFeld.charAt(i) == '.')
							{
								count++;
							}
						}

						_debug.fine("C: " + count);

						if (count >= 1)
						{
							event.doit = false;
							return;
						}
					}

				}
			}

		});

		_textKnotenOffsetKilometer.addModifyListener(new ModifyListener()
		{

			@Override
			public void modifyText(ModifyEvent e)
			{
				String text = _textKnotenOffsetKilometer.getText();
				Float fl = Float.valueOf(text);
				_filterKnoten.setKm(fl);
			}

		});

		/*
		 * Listener fr die Radiobuttons
		 */
		_radioButtonKnotenOffsetPositiv.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				if (_radioButtonKnotenOffsetPositiv.getSelection())
				{

					_debug.fine(getClass().getName() + " => POSITIV SELECTED");

					if (_filterKnoten != null)
					{
						_filterKnoten.setRichtung(Richtung.positiv);
						_debug
								.fine(getClass().getName()
										+ " => in FilterKnoten wurde Richtugn auf Richtung.positiv gesetzt.");

					}
				}
			}
		});

		_radioButtonKnotenOffsetNegativ.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				if (_radioButtonKnotenOffsetNegativ.getSelection())
				{
					_debug.fine(getClass().getName() + " => NEGATIV SELECTED");

					if (_filterKnoten != null)
					{
						_filterKnoten.setRichtung(Richtung.negativ);

						_debug
								.fine(getClass().getName()
										+ " => in FilterKnoten wurde Richtugn auf Richtung.negativ gesetzt.");

					}
				}

			}
		});

	}

	/**
	 * Erzeugung von compositeStrassenSegment
	 * 
	 */
	private void erzeugeCompositeStrassenSegment()
	{
		GridData gridData4 = new GridData();
		gridData4.horizontalAlignment = GridData.FILL;
		gridData4.grabExcessHorizontalSpace = true;
		gridData4.verticalAlignment = GridData.CENTER;
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		GridData gridData1 = new GridData();
		gridData1.horizontalAlignment = GridData.FILL;
		gridData1.grabExcessHorizontalSpace = true;
		gridData1.grabExcessVerticalSpace = true;
		gridData1.verticalAlignment = GridData.FILL;
		_compositeStrassenSegment = new Composite(_pultStrassenSegment, SWT.NONE);
		_compositeStrassenSegment.setLayoutData(gridData1);
		_compositeStrassenSegment.setLayout(gridLayout);
		_labelStrassenSegmentStrasse = new Label(_compositeStrassenSegment,
				SWT.NONE);
		_labelStrassenSegmentStrasse.setText("Strae:");
		erzeugeComboStrassenSegmentStrasse();
		_labelStrassenSegmentStrassenSegment = new Label(_compositeStrassenSegment,
				SWT.NONE);
		_labelStrassenSegmentStrassenSegment.setText("Straensegment:");
		erzeugeComboStrassenSegmentStrassenSegment();
		_labelStrassenSegmentKilometer = new Label(_compositeStrassenSegment,
				SWT.NONE);
		_labelStrassenSegmentKilometer.setText("Kilometer:");
		_textStrassenSegmentKilometer = new Text(_compositeStrassenSegment,
				SWT.BORDER);
		_textStrassenSegmentKilometer.setLayoutData(gridData4);

		_labelStrassenSegmentStrasseFueller = new Label(_compositeStrassenSegment,
				SWT.NONE);
		_labelStrassenSegmentStrasseFueller.setText("");

		_labelStrassenSegmentStrasseLaengenwarnung = new Label(
				_compositeStrassenSegment, SWT.NONE);
		_labelStrassenSegmentStrasseLaengenwarnung.setLayoutData(gridData1);

	}
	
	/**
	 * Erzeugung von compositeStrassenSegment
	 * 
	 */
	private void erzeugeCompositeStrassenSegment(Composite parent)
	{
	  GridData gridData4 = new GridData();
	  gridData4.horizontalAlignment = GridData.FILL;
	  gridData4.grabExcessHorizontalSpace = true;
	  gridData4.verticalAlignment = GridData.CENTER;
	  GridLayout gridLayout = new GridLayout();
	  gridLayout.numColumns = 2;
	  GridData gridData1 = new GridData();
	  gridData1.horizontalAlignment = GridData.FILL;
	  gridData1.grabExcessHorizontalSpace = true;
	  gridData1.grabExcessVerticalSpace = true;
	  gridData1.verticalAlignment = GridData.FILL;
	  _compositeStrassenSegment = new Composite(parent, SWT.NONE);
	  _compositeStrassenSegment.setLayoutData(gridData1);
	  _compositeStrassenSegment.setLayout(gridLayout);
	  _labelStrassenSegmentStrasse = new Label(_compositeStrassenSegment,
	      SWT.NONE);
	  _labelStrassenSegmentStrasse.setText("Strae:");
	  erzeugeComboStrassenSegmentStrasse();
	  _labelStrassenSegmentStrassenSegment = new Label(_compositeStrassenSegment,
	      SWT.NONE);
	  _labelStrassenSegmentStrassenSegment.setText("Straensegment:");
	  erzeugeComboStrassenSegmentStrassenSegment();
	  _labelStrassenSegmentKilometer = new Label(_compositeStrassenSegment,
	      SWT.NONE);
	  _labelStrassenSegmentKilometer.setText("Offset(km):");
	  _textStrassenSegmentKilometer = new Text(_compositeStrassenSegment,
	      SWT.BORDER);
	  _textStrassenSegmentKilometer.setLayoutData(gridData4);
	  
	  _labelStrassenSegmentStrasseFueller = new Label(_compositeStrassenSegment,
	      SWT.NONE);
	  _labelStrassenSegmentStrasseFueller.setText("");
	  
	  _labelStrassenSegmentStrasseLaengenwarnung = new Label(
	      _compositeStrassenSegment, SWT.NONE);
	  _labelStrassenSegmentStrasseLaengenwarnung.setLayoutData(gridData1);
	  
	}

	/**
	 * Erzeugung von comboStrassenSegmentStrasse
	 * 
	 */
	private void erzeugeComboStrassenSegmentStrasse()
	{
		GridData gridData2 = new GridData();
		gridData2.horizontalAlignment = GridData.FILL;
		gridData2.grabExcessHorizontalSpace = true;
		gridData2.verticalAlignment = GridData.CENTER;
		_comboStrassenSegmentStrasse = new Combo(_compositeStrassenSegment,
				SWT.NONE);
		_comboStrassenSegmentStrasse.setLayoutData(gridData2);

	}

	/**
	 * Erzeugung von comboStrassenSegmentStrassenSegment
	 * 
	 */
	private void erzeugeComboStrassenSegmentStrassenSegment()
	{
		GridData gridData3 = new GridData();
		gridData3.horizontalAlignment = GridData.FILL;
		gridData3.grabExcessHorizontalSpace = true;
		gridData3.verticalAlignment = GridData.CENTER;
		_comboStrassenSegmentStrassenSegment = new Combo(_compositeStrassenSegment,
				SWT.NONE);
		_comboStrassenSegmentStrassenSegment.setLayoutData(gridData3);

	}

	/**
	 * Erzeugung von compositeBetriebskilometer
	 * 
	 */
	private void erzeugeCompositeBetriebskilometer()
	{
		GridData gridData8 = new GridData();
		gridData8.horizontalAlignment = GridData.FILL;
		gridData8.grabExcessHorizontalSpace = true;
		gridData8.verticalAlignment = GridData.CENTER;
		GridLayout gridLayout1 = new GridLayout();
		gridLayout1.numColumns = 2;
		GridData gridData5 = new GridData();
		gridData5.horizontalAlignment = GridData.FILL;
		gridData5.grabExcessHorizontalSpace = true;
		gridData5.grabExcessVerticalSpace = true;
		gridData5.verticalAlignment = GridData.FILL;
		_compositeBetriebskilometer = new Composite(_pultBetriebskilometer,
				SWT.NONE);
		_compositeBetriebskilometer.setLayoutData(gridData5);
		_compositeBetriebskilometer.setLayout(gridLayout1);
		_labelBetriebskilometerStrasse = new Label(_compositeBetriebskilometer,
				SWT.NONE);
		_labelBetriebskilometerStrasse.setText("Strae:");
		erzeugeComboBetriebskilometerStrae();
		_labelBetriebskilometerRichtung = new Label(_compositeBetriebskilometer,
				SWT.NONE);
		_labelBetriebskilometerRichtung.setText("Richtung:");
		erzeugeComboBetriebskilometerRichtung();
		_labelBetriebskilometerBetriebskilometer = new Label(
				_compositeBetriebskilometer, SWT.NONE);
		_labelBetriebskilometerBetriebskilometer.setText("Betriebskilometer:");
		_textBetriebskilometerBetriebskilometer = new Text(
				_compositeBetriebskilometer, SWT.BORDER);
		_textBetriebskilometerBetriebskilometer.setLayoutData(gridData8);
		_textBetriebskilometerBetriebskilometer
				.setToolTipText("Hier knnen die Betriebskilometer in Metern eingegeben werden.");

		_labelBetriebskilometerBlocknummer = new Label(_compositeBetriebskilometer,
				SWT.NONE);
		_labelBetriebskilometerBlocknummer.setText("Blocknummer:");
		erzeugeComboBetriebskilometerBlocknummer();

		Label dummyLabel = new Label(_compositeBetriebskilometer, SWT.NONE);
		dummyLabel.setVisible(false);

		_labelBetriebskilometerWarnung = new Label(_compositeBetriebskilometer,
				SWT.NONE);

		_labelBetriebskilometerWarnung.setForeground(new Color(
				Display.getCurrent(), 255, 0, 0));
		_labelBetriebskilometerWarnung
				.setText("Der angegebene Betriebskilometer ist im LCL-Segment der Strae nicht vorhanden!");
		_labelBetriebskilometerWarnung.setVisible(false);

	}

	/**
	 * Erzeugung von comboBetriebskilometerStrae
	 * 
	 */
	private void erzeugeComboBetriebskilometerStrae()
	{
		GridData gridData6 = new GridData();
		gridData6.horizontalAlignment = GridData.FILL;
		gridData6.grabExcessHorizontalSpace = true;
		gridData6.verticalAlignment = GridData.CENTER;
		_comboBetriebskilometerStrae = new Combo(_compositeBetriebskilometer,
				SWT.NONE);
		_comboBetriebskilometerStrae.setLayoutData(gridData6);

	}

	/**
	 * Erzeugung von comboBetriebskilometerRichtung
	 * 
	 */
	private void erzeugeComboBetriebskilometerRichtung()
	{
		GridData gridData7 = new GridData();
		gridData7.horizontalAlignment = GridData.FILL;
		gridData7.grabExcessHorizontalSpace = true;
		gridData7.verticalAlignment = GridData.CENTER;
		_comboBetriebskilometerRichtung = new Combo(_compositeBetriebskilometer,
				SWT.NONE);
		_comboBetriebskilometerRichtung.setLayoutData(gridData7);
	}

	/**
	 * Erzeugung von comboBetriebskilometerBlocknummer
	 * 
	 */
	private void erzeugeComboBetriebskilometerBlocknummer()
	{
		GridData gridData9 = new GridData();
		gridData9.horizontalAlignment = GridData.FILL;
		gridData9.grabExcessHorizontalSpace = true;
		gridData9.verticalAlignment = GridData.CENTER;
		_comboBetriebskilometerBlocknummer = new Combo(_compositeBetriebskilometer,
				SWT.NONE);
		_comboBetriebskilometerBlocknummer.setLayoutData(gridData9);
	}

	/**
	 * Erzeugung von compositeAsbOrtsangabe
	 * 
	 */
	private void erzeugeCompositeAsbOrtsangabe()
	{
		GridData gridDataTextAsbStationierung = new GridData();
		gridDataTextAsbStationierung.horizontalAlignment = GridData.FILL;
		gridDataTextAsbStationierung.grabExcessHorizontalSpace = true;
		gridDataTextAsbStationierung.verticalAlignment = GridData.CENTER;
		GridLayout gridLayoutAsbOrtsangabe = new GridLayout();
		gridLayoutAsbOrtsangabe.numColumns = 2;
		GridData gridDataAsbOrtsangabe = new GridData();
		gridDataAsbOrtsangabe.horizontalAlignment = GridData.FILL;
		gridDataAsbOrtsangabe.grabExcessHorizontalSpace = true;
		gridDataAsbOrtsangabe.grabExcessVerticalSpace = true;
		gridDataAsbOrtsangabe.verticalAlignment = GridData.FILL;
		_compositeAsbOrtsangabe = new Composite(_pultAsbOrtsangabe, SWT.NONE);
		_compositeAsbOrtsangabe.setLayoutData(gridDataAsbOrtsangabe);
		_compositeAsbOrtsangabe.setLayout(gridLayoutAsbOrtsangabe);
		_labelAsbStrasse = new Label(_compositeAsbOrtsangabe, SWT.NONE);
		_labelAsbStrasse.setText("Strae:");
		erzeugeComboAsbStrasse();
		_labelAsbVonNetzknoten = new Label(_compositeAsbOrtsangabe, SWT.NONE);
		_labelAsbVonNetzknoten.setText("Von Netzknoten/Ast:");
		erzeugeComboAsbAnfangsKnoten();
		_labelAsbNachNetzknoten = new Label(_compositeAsbOrtsangabe, SWT.NONE);
		_labelAsbNachNetzknoten.setText("Nach Netzknoten/Ast:");
		erzeugeComboAsbEndKnoten();
		_labelAsbStationierung = new Label(_compositeAsbOrtsangabe, SWT.NONE);
		_labelAsbStationierung.setText("Stationierung:");

		_textAsbStationierung = new Text(_compositeAsbOrtsangabe, SWT.BORDER);
		_textAsbStationierung.setLayoutData(gridDataTextAsbStationierung);
		_textAsbStationierung
				.setToolTipText("Hier knnen Sie einen Wert fr die Stationierung "
						+ "eingeben (in Metern). Dieser wird\ndaraufhin geprft, ob er sich im gltigen "
						+ "Bereich der zuvor ausgewhlten Knoten befindet.");

		_labelAsbStationierungStationierungsRichtung = new Label(
				_compositeAsbOrtsangabe, SWT.NONE);
		_labelAsbStationierungStationierungsRichtung
				.setText("Stationierungsrichtung:");
		erzeugeComboAsbStationierung();

		/*
		 * Dummy-Label zum fllen des freien Platzes
		 */
		@SuppressWarnings("unused")
		Label labelAsbDummy = new Label(_compositeAsbOrtsangabe, SWT.NONE);

		/*
		 * Label zur Anzeige, dass sich die Stationierung auerhalb des gltigen
		 * Bereiches befindet.
		 */
		_labelAsbStationierungAusserhalbBereich = new Label(
				_compositeAsbOrtsangabe, SWT.NONE);
		_labelAsbStationierungAusserhalbBereich.setForeground(new Color(PlatformUI
				.getWorkbench().getDisplay(), 255, 0, 0));
		_labelAsbStationierungAusserhalbBereich
				.setText("Die angegebene Stationierung liegt auerhalb des gltigen Bereichs!");
		_labelAsbStationierungAusserhalbBereich.setVisible(false);

		/*
		 * TabListe setzen
		 */
		Control[] tabList = new Control[] { _comboAsbStrasse,
				_comboAsbAnfangsKnoten, _comboAsbEndKnoten, _textAsbStationierung };
		_compositeAsbOrtsangabe.setTabList(tabList);
	}

	/**
	 * Erzeugung von comboAsbStrasse
	 * 
	 */
	private void erzeugeComboAsbStrasse()
	{
		GridData gridDataComboAsbStrasse = new GridData();
		gridDataComboAsbStrasse.horizontalAlignment = GridData.FILL;
		gridDataComboAsbStrasse.grabExcessHorizontalSpace = true;
		gridDataComboAsbStrasse.verticalAlignment = GridData.CENTER;
		_comboAsbStrasse = new Combo(_compositeAsbOrtsangabe, SWT.NONE);
		_comboAsbStrasse.setToolTipText("Hier knnen Sie eine Strae auswhlen.");
		_comboAsbStrasse.setLayoutData(gridDataComboAsbStrasse);
	}

	/**
	 * Erzeugung von comboAsbVonNetzknoten
	 * 
	 */
	private void erzeugeComboAsbAnfangsKnoten()
	{
		GridData gridDataAsbAnfangsKnoten = new GridData();
		gridDataAsbAnfangsKnoten.horizontalAlignment = GridData.FILL;
		gridDataAsbAnfangsKnoten.grabExcessHorizontalSpace = true;
		gridDataAsbAnfangsKnoten.verticalAlignment = GridData.CENTER;
		_comboAsbAnfangsKnoten = new Combo(_compositeAsbOrtsangabe, SWT.READ_ONLY);
		_comboAsbAnfangsKnoten.setLayoutData(gridDataAsbAnfangsKnoten);
		_comboAsbAnfangsKnoten
				.setToolTipText("Hier knnen Sie einen Anfangs Knoten auswhlen.");
	}

	/**
	 * Erzeugung von comboNachNetzknoten
	 * 
	 */
	private void erzeugeComboAsbEndKnoten()
	{
		GridData gridDataComboAsbEndknoten = new GridData();
		gridDataComboAsbEndknoten.horizontalAlignment = GridData.FILL;
		gridDataComboAsbEndknoten.grabExcessHorizontalSpace = true;
		gridDataComboAsbEndknoten.verticalAlignment = GridData.CENTER;
		_comboAsbEndKnoten = new Combo(_compositeAsbOrtsangabe, SWT.READ_ONLY);
		_comboAsbEndKnoten.setLayoutData(gridDataComboAsbEndknoten);
		_comboAsbEndKnoten
				.setToolTipText("Hier knnen Sie einen End Knoten auswhlen.");
	}

	private void erzeugeComboAsbStationierung()
	{
		GridData gridDataComboAsbStationierung = new GridData();
		gridDataComboAsbStationierung.horizontalAlignment = GridData.FILL;
		gridDataComboAsbStationierung.grabExcessHorizontalSpace = true;
		gridDataComboAsbStationierung.verticalAlignment = GridData.CENTER;

		_comboAsbStationierungsrichtung = new Combo(_compositeAsbOrtsangabe,
				SWT.READ_ONLY);
		_comboAsbStationierungsrichtung
				.setLayoutData(gridDataComboAsbStationierung);
	}

	/**
	 * Erzeugung von compositeKnotenOffset
	 * 
	 */
	private void erzeugeCompositeKnotenOffset()
	{
		GridData gridData15 = new GridData();
		gridData15.horizontalAlignment = GridData.FILL;
		gridData15.grabExcessHorizontalSpace = true;
		gridData15.grabExcessVerticalSpace = true;
		gridData15.verticalAlignment = GridData.BEGINNING;
		_compositeKnotenOffset = new Composite(_pultKnotenOffset, SWT.NONE);
		_compositeKnotenOffset.setLayout(new GridLayout());
		erzeugeCompositeKnotenOffsetOben();
		_compositeKnotenOffset.setLayoutData(gridData15);
		erzeugeGruppeKnotenOffsetUnten();
	}

	/**
	 * Erzeugung von compositeKnotenOffsetOben
	 * 
	 */
	private void erzeugeCompositeKnotenOffsetOben()
	{
		GridData gridData19 = new GridData();
		gridData19.horizontalAlignment = GridData.FILL;
		gridData19.grabExcessHorizontalSpace = true;
		gridData19.verticalAlignment = GridData.CENTER;
		GridLayout gridLayout4 = new GridLayout();
		gridLayout4.numColumns = 2;
		GridData gridData16 = new GridData();
		gridData16.horizontalAlignment = GridData.FILL;
		gridData16.grabExcessHorizontalSpace = true;
		gridData16.grabExcessVerticalSpace = true;
		gridData16.verticalAlignment = GridData.FILL;
		_compositeKnotenOffsetOben = new Composite(_compositeKnotenOffset, SWT.NONE);
		_compositeKnotenOffsetOben.setLayoutData(gridData16);
		_compositeKnotenOffsetOben.setLayout(gridLayout4);
		_labelKnotenOffsetStrasse = new Label(_compositeKnotenOffsetOben, SWT.NONE);
		_labelKnotenOffsetStrasse.setText("Strae:");
		erzeugeComboKnotenOffsetStrasse();
		_labelKnotenOffsetKnoten = new Label(_compositeKnotenOffsetOben, SWT.NONE);
		_labelKnotenOffsetKnoten.setText("Knoten:");
		erzeugeComboKnotenOffsetKnoten();
		_labelKnotenOffsetKilometer = new Label(_compositeKnotenOffsetOben,
				SWT.NONE);
		_labelKnotenOffsetKilometer.setText("Kilometer:");
		_textKnotenOffsetKilometer = new Text(_compositeKnotenOffsetOben,
				SWT.BORDER);
		_textKnotenOffsetKilometer.setLayoutData(gridData19);
	}

	/**
	 * Erzeugung von gruppeKnotenOffsetUnten
	 * 
	 */
	private void erzeugeGruppeKnotenOffsetUnten()
	{
		_gruppeKnotenOffsetUnten = new Group(_compositeKnotenOffset, SWT.NONE);
		_gruppeKnotenOffsetUnten.setLayout(new GridLayout());
		_gruppeKnotenOffsetUnten.setText("Richtung");
		_radioButtonKnotenOffsetPositiv = new Button(_gruppeKnotenOffsetUnten,
				SWT.RADIO);
		_radioButtonKnotenOffsetPositiv.setText("positiv");
		_radioButtonKnotenOffsetPositiv.setSelection(true);
		_radioButtonKnotenOffsetNegativ = new Button(_gruppeKnotenOffsetUnten,
				SWT.RADIO);
		_radioButtonKnotenOffsetNegativ.setText("negativ");

	}

	/**
	 * Erzeugung von comboKnotenOffsetStrasse
	 * 
	 */
	private void erzeugeComboKnotenOffsetStrasse()
	{
		GridData gridData17 = new GridData();
		gridData17.horizontalAlignment = GridData.FILL;
		gridData17.grabExcessHorizontalSpace = true;
		gridData17.verticalAlignment = GridData.CENTER;
		_comboKnotenOffsetStrasse = new Combo(_compositeKnotenOffsetOben, SWT.NONE);
		_comboKnotenOffsetStrasse.setLayoutData(gridData17);
	}

	/**
	 * Erzeugung von comboKnotenOffsetKnoten
	 * 
	 */
	private void erzeugeComboKnotenOffsetKnoten()
	{
		GridData gridData18 = new GridData();
		gridData18.horizontalAlignment = GridData.FILL;
		gridData18.grabExcessHorizontalSpace = true;
		gridData18.verticalAlignment = GridData.CENTER;
		_comboKnotenOffsetKnoten = new Combo(_compositeKnotenOffsetOben, SWT.NONE);
		_comboKnotenOffsetKnoten.setLayoutData(gridData18);
	}

//	/**
//	 * Fllen des {@link TabFolder} mit den jeweiligen {@link TabItem}s der
//	 * verschiedenen Mglichkeiten zur Ortsauswahl.
//	 * 
//	 */
//	private void fuellePultordner()
//	{
//		GridLayout pultLayout = new GridLayout();
//
//		_pultStrassenSegment = erzeugeSeite(_pultOrdner, _bezStrassenSegmentOffset);
//		_pultStrassenSegment.setLayout(pultLayout);
//		erzeugeCompositeStrassenSegment();
//
//		_pultBetriebskilometer = erzeugeSeite(_pultOrdner, _bezBetriebskilometer);
//		_pultBetriebskilometer.setLayout(pultLayout);
//		erzeugeCompositeBetriebskilometer();
//
//		_pultAsbOrtsangabe = erzeugeSeite(_pultOrdner, _bezAsbOrtsangabe);
//		_pultAsbOrtsangabe.setLayout(pultLayout);
//		erzeugeCompositeAsbOrtsangabe();
//
//		_pultKnotenOffset = erzeugeSeite(_pultOrdner, _bezKnotenOffset);
//		_pultKnotenOffset.setLayout(pultLayout);
//		erzeugeCompositeKnotenOffset();
//
//	}

	/**
	 * Liefert die Art der Ortsauswahl als Instanz des Enumerator
	 * {@link OrtsangabeEnumeratorAuswahl}.
	 * 
	 * @return Art der Ortsauswahl
	 */
	public OrtsangabeEnumeratorAuswahl getAuswahlArt()
	{
		return _auswahl;
	}

	/**
	 * Liefert das Strassensegment
	 * 
	 * @return {@link FilterStrassenSegmentOffsetALT}
	 */
	public FilterStrassenSegmentOffset getFilterStrassenSegmentOffset()
	{
		return _filterStrassenSegmentOffset;
	}

	/**
	 * Liefert die ASB-Ortsangabe
	 * 
	 * @return {@link FilterAsbOrtsangabe}
	 */
	public FilterAsbOrtsangabe getFilterAsbOrtsangabe()
	{
		return _filterAsbOrtsangabe;
	}

	/**
	 * Liefert die Betriebskilometer
	 * 
	 * @return {@link FilterBetriebskilometer}
	 */
	public FilterBetriebskilometer getFilterBetriebskilometer()
	{
		return _filterBetriebskilometer;
	}

	/**
	 * Liefert den Knoten
	 * 
	 * @return {@link FilterKnotenAlt}
	 */
	public FilterKnoten getFilterKnoten()
	{
		return _filterKnoten;
	}

	/**
	 * Setzt die Angabe via StraenSegment
	 * 
	 * @param pidSelektierteStrasse
	 * @param pidStrassenSegment
	 * @param km
	 * @throws IllegalArgumentException
	 */
	public void setStrassenSegment(String pidSelektierteStrasse,
			String pidStrassenSegment, long km) throws IllegalArgumentException
	{
//		selektierePultOrdner(OrtsangabeEnumeratorAuswahl.strassensegment);

		/*
		 * Strae selektieren
		 */
		boolean selektierteStrasseGefunden = false;
		int anzStrassen = _comboStrassenSegmentStrasse.getItemCount();

		for (int i = 0; i < anzStrassen; i++)
		{
			Object object = _cvStrassenSegmentStrasse.getElementAt(i);
			if (object instanceof Strasse)
			{
				Strasse strasse = (Strasse) object;
				if (strasse.getPid().equals(pidSelektierteStrasse))
				{

					selektierteStrasseGefunden = true;
					_cvStrassenSegmentStrasse.setSelection(new StructuredSelection(
							strasse));
					_filterStrassenSegmentOffset.setPidSelektierteStrasse(strasse
							.getPid());
					break;
				}
			}
		}

		if (!selektierteStrasseGefunden)
		{
			throw (new IllegalArgumentException(getClass().getName()
					+ "::setBetriebsKilomert \n\t=> Die Strae mit der PID "
					+ pidSelektierteStrasse + " wurde nicht gefunden!"));
		}

		/*
		 * StraenSegment selektieren
		 */
		boolean selektiertesStrasseSegmentGefunden = false;
		int anzSegmente = _comboStrassenSegmentStrassenSegment.getItemCount();
		for (int i = 0; i < anzSegmente; i++)
		{
			Object object = _cvStrassenSegmentStrassenSegment.getElementAt(i);
			if (object instanceof StrassenSegment)
			{
				StrassenSegment strassenSegment = (StrassenSegment) object;
				if (strassenSegment.getPid().equals(pidStrassenSegment))
				{
					selektiertesStrasseSegmentGefunden = true;
					_cvStrassenSegmentStrassenSegment
							.setSelection(new StructuredSelection(strassenSegment));
					_filterStrassenSegmentOffset
							.setPidSelektiertesStrassenSegment(strassenSegment.getPid());
					break;
				}

			}
		}

		if (!selektiertesStrasseSegmentGefunden)
		{
			throw new IllegalArgumentException(getClass().getName()
					+ "::setStraenSegment \n\t=> Die Strae mit der PID "
					+ pidSelektierteStrasse + " wurde nicht gefunden!");
		}

		// Feld setzen
		// textStrassenSegmentKilometer.setText("" +
		// _filterStrassenSegmentOffset.getKm());

		// _filterStrassenSegmentOffset.getKm();

		float fKm = (float) km / 1000;
		String sKm = "" + fKm;
		sKm = sKm.replace(".", ",");

		_textStrassenSegmentKilometer.setText(sKm);
		_filterStrassenSegmentOffset.setKm(km);

	}

	/**
	 * Methode zum Setzen der Daten eines Betriebskilometers.
	 * 
	 * @param pidSelektierteStrasse
	 *          PID der selektierten {@link Strasse} des Betriebskilometers.
	 * @param himmelsRichtung
	 *          Himmelsrichtung
	 *          {@link de.bsvrz.buv.rw.basislib.ort.FilterBetriebskilometer.Richtung}
	 * @param betriebsKilometer
	 *          Betriebskilometer in Metern
	 * @param selektierterBlock
	 *          Selektierter Block des Betriebskilometers. Wurde kein BlockNummer
	 *          selektiert, ist dies mit "KeineAngabe" sichtbar zu machen. Wird
	 *          <code>null</code> bergeben, wird automatisch auf "KeineAngabe"
	 *          gesetzt.
	 * @throws IllegalArgumentException
	 *           Wird geworfen, wenn keine Strae mit der angegebenen PID
	 *           existiert oder wenn der angegebene Betriebskilometer auf dieser
	 *           Strae nicht existiert oder wenn eine BlockNummer angegeben
	 *           wurde, die nicht existiert.
	 */
	public void setBetriebskilometer(String pidSelektierteStrasse,
			FilterBetriebskilometer.Richtung himmelsRichtung, long betriebsKilometer,
			String selektierterBlock) throws IllegalArgumentException
	{
		// Tab selektieren
//		selektierePultOrdner(OrtsangabeEnumeratorAuswahl.betriebskilometer);

		/*
		 * Strasse selektieren
		 */
		boolean selektierteStrasseGefunden = false;
		int anzStrassen = _comboBetriebskilometerStrae.getItemCount();
		for (int i = 0; i < anzStrassen; i++)
		{
			Object object = _cvBetriebsKilometerStrasse.getElementAt(i);
			if (object instanceof Strasse)
			{
				Strasse strasse = (Strasse) object;
				if (strasse.getPid().equals(pidSelektierteStrasse))
				{
					selektierteStrasseGefunden = true;
					_cvBetriebsKilometerStrasse.setSelection(new StructuredSelection(
							strasse));
					break;
				}
			}
		}
		if (!selektierteStrasseGefunden)
		{
			throw (new IllegalArgumentException(getClass().getName()
					+ "::setBetriebsKilomert \n\t=> Die Strae mit der PID "
					+ pidSelektierteStrasse + " wurde nicht gefunden!"));
		}

		/*
		 * Himmelsrichtung selektieren
		 */
		int anzRichtungen = _comboBetriebskilometerRichtung.getItemCount();
		for (int i = 0; i < anzRichtungen; i++)
		{
			Object object = _cvBetriebsKilometerRichtung.getElementAt(i);
			if (object instanceof FilterBetriebskilometer.Richtung)
			{
				FilterBetriebskilometer.Richtung richtung = (FilterBetriebskilometer.Richtung) object;
				if (richtung.equals(himmelsRichtung))
				{
					_cvBetriebsKilometerRichtung.setSelection(new StructuredSelection(
							richtung));
					break;
				}
			}
		}

		/*
		 * Betriebskilometer eintragen
		 */
		_textBetriebskilometerBetriebskilometer.setText("" + betriebsKilometer);
		if (_filterBetriebskilometer.getVorhandeneBlockNummern().size() == 0)
		{
			throw (new IllegalArgumentException(
					"::setBetriebsKilomert \n\t=> Der Betriebskilometerwert "
							+ betriebsKilometer + " m ist nicht vorhanden!"));

		}

		/*
		 * BlockNummer auswhlen
		 */
		boolean blockNummerGefunden = false;
		int anzBloecke = _comboBetriebskilometerBlocknummer.getItemCount();
		if (anzBloecke > 0)
		{
			for (int i = 0; i < anzBloecke; i++)
			{
				Object object = _cvBetriebsKilometerBlockNummer.getElementAt(i);
				if (object instanceof String)
				{
					String block = (String) object;
					if (block.equalsIgnoreCase(selektierterBlock))
					{
						blockNummerGefunden = true;
						_cvBetriebsKilometerBlockNummer
								.setSelection(new StructuredSelection(block));
						break;
					}
				}
			}
		}
		if (!blockNummerGefunden)
		{
			throw (new IllegalArgumentException(getClass().getName()
					+ "::setBetriebsKilomert \n\t=> Die BlockNummer " + selektierterBlock
					+ " wurde nicht gefunden!"));
		}

	}

	public void setAsb(String pidSelektierteStrasse, String anfangsKnoten,
			String endKnoten, long stationierung,
			EnumAsbVerkehrsRichtung asbVerkehrsRichtung)
			throws IllegalArgumentException
	{
		/*
		 * Tab selektieren
		 */
//		selektierePultOrdner(OrtsangabeEnumeratorAuswahl.asborstangabe);

		/*
		 * Strasse selektieren
		 */
		boolean selektierteStrasseGefunden = false;
		int anzStrassen = _comboAsbStrasse.getItemCount();
		for (int i = 0; i < anzStrassen; i++)
		{
			Object object = _cvAsbStrasse.getElementAt(i);
			if (object instanceof Strasse)
			{
				Strasse strasse = (Strasse) object;
				if (strasse.getPid().equals(pidSelektierteStrasse))
				{
					selektierteStrasseGefunden = true;
					_cvAsbStrasse.setSelection(new StructuredSelection(strasse));
					break;
				}
			}
		}
		if (!selektierteStrasseGefunden)
		{
			throw (new IllegalArgumentException(getClass().getName()
					+ "::setAsb \n\t=> Die Strae mit der PID " + pidSelektierteStrasse
					+ " wurde nicht gefunden!"));
		}

		/*
		 * Anfangsknoten selektieren
		 */
		boolean anfangsKnotenGefunden = false;
		int anzAnfangsKnoten = _comboAsbAnfangsKnoten.getItemCount();
		for (int i = 0; i < anzAnfangsKnoten; i++)
		{
			Object object = _cvAsbAnfangsKnoten.getElementAt(i);
			// if (object instanceof AsbKnoten)
			// {
			// AsbKnoten knoten = (AsbKnoten) object;
			// if ((knoten.getKnoten().equals(anfangsKnoten))
			// && (knoten.getTmcRichtung().equals(richtungAnfangsKnoten)))
			// {
			// anfangsKnotenGefunden = true;
			// _cvAsbAnfangsKnoten.setSelection(new StructuredSelection(knoten));
			// break;
			// }
			// }
			if (object instanceof AsbKnotenKombiniert)
			{
				AsbKnotenKombiniert knoten = (AsbKnotenKombiniert) object;
				if (knoten.getKnoten().equals(anfangsKnoten))
				{
					anfangsKnotenGefunden = true;
					_cvAsbAnfangsKnoten.setSelection(new StructuredSelection(knoten));
					break;
				}
			}
		}
		if (!anfangsKnotenGefunden)
		{
			throw (new IllegalArgumentException(getClass().getName()
					+ "::setAsb \n\t=> Der Anfangsknoten " + anfangsKnoten
					+ " wurde nicht gefunden!"));
		}

		/*
		 * Endknoten selektieren
		 */
		boolean endKnotenGefunden = false;
		int anzEndKnoten = _comboAsbEndKnoten.getItemCount();
		for (int i = 0; i < anzEndKnoten; i++)
		{
			Object object = _cvAsbEndKnoten.getElementAt(i);
			// if (object instanceof AsbKnoten)
			// {
			// AsbKnoten knoten = (AsbKnoten) object;
			// if ((knoten.getKnoten().equals(endKnoten))
			// && (knoten.getTmcRichtung().equals(richtungEndKnoten)))
			// {
			// endKnotenGefunden = true;
			// _cvAsbEndKnoten.setSelection(new StructuredSelection(knoten));
			// break;
			// }
			// }
			if (object instanceof AsbKnotenKombiniert)
			{
				AsbKnotenKombiniert knoten = (AsbKnotenKombiniert) object;
				if (knoten.getKnoten().equals(endKnoten))
				{
					endKnotenGefunden = true;
					_cvAsbEndKnoten.setSelection(new StructuredSelection(knoten));
					break;
				}
			}
		}
		if (!endKnotenGefunden)
		{
			throw (new IllegalArgumentException(getClass().getName()
					+ "::setAsb \n\t=> Der Endknoten " + endKnoten
					+ " wurde nicht gefunden!"));
		}

		if (stationierung >= 0)
		{
			_textAsbStationierung.setText("" + stationierung);
		}

		/*
		 * Verkehrsrichtung
		 */
		for (int i = 0; i < _comboAsbStationierungsrichtung.getItemCount(); i++)
		{
			Object elementAt = _cvAsbStationierungsrichtung.getElementAt(i);
			if (elementAt instanceof EnumAsbVerkehrsRichtung)
			{
				EnumAsbVerkehrsRichtung richtung = (EnumAsbVerkehrsRichtung) elementAt;
				if (richtung.equals(asbVerkehrsRichtung))
				{
					_cvAsbStationierungsrichtung.setSelection(new StructuredSelection(
							richtung));
					break;
				}
			}
		}

	}

	public void setAsb(String pidSelektierteStrasse, String anfangsKnoten,
			String endKnoten, long stationierung)
	{
		setAsb(pidSelektierteStrasse, anfangsKnoten, endKnoten, stationierung, null);
	}

	/**
	 * Methode zum Setzen der Daten eines Knoten-Offsets
	 * 
	 * @param pidSelektierteStrasse
	 *          PID der selektierten {@link Strasse}.
	 * @param pidSelektierterKnoten
	 *          PID des selektierten {@link StrassenKnoten}s.
	 * @param km
	 *          Kilometer.
	 * @param selektierteRichtung
	 *          {@link Richtung}.
	 * @throws IllegalArgumentException
	 *           Wird geworfen, wenn die angegebene {@link Strasse} oder der
	 *           angegebene {@link StrassenKnoten} nicht existiert.
	 */
	public void setKnotenOffset(String pidSelektierteStrasse,
			String pidSelektierterKnoten, float km, Richtung selektierteRichtung)
	{
		// Tab selektieren
//		selektierePultOrdner(OrtsangabeEnumeratorAuswahl.knotenoffset);

		/*
		 * Strae selektieren
		 */
		boolean selektierteStrasseGefunden = false;
		int anzStrassen = _comboKnotenOffsetStrasse.getItemCount();
		for (int i = 0; i < anzStrassen; i++)
		{
			Object object = _cvKnotenOffsetStrasse.getElementAt(i);
			if (object instanceof Strasse)
			{
				Strasse strasse = (Strasse) object;
				if (strasse.getPid().equals(pidSelektierteStrasse))
				{
					selektierteStrasseGefunden = true;
					_cvKnotenOffsetStrasse.setSelection(new StructuredSelection(strasse));
					break;
				}
			}
		}
		if (!selektierteStrasseGefunden)
		{
			throw (new IllegalArgumentException(getClass().getName()
					+ "::setKnotenOffset \n\t=> Die Strae mit der PID "
					+ pidSelektierteStrasse + " wurde nicht gefunden!"));
		}

		/*
		 * Knoten selektieren
		 */
		boolean selektierterKnotenGefunden = false;
		int anzKnoten = _comboKnotenOffsetKnoten.getItemCount();
		for (int i = 0; i < anzKnoten; i++)
		{
			Object object = _cvKnotenOffsetKnoten.getElementAt(i);
			if (object instanceof StrassenKnoten)
			{
				StrassenKnoten knoten = (StrassenKnoten) object;
				if (knoten.getPid().equals(pidSelektierterKnoten))
				{
					selektierterKnotenGefunden = true;
					_cvKnotenOffsetKnoten.setSelection(new StructuredSelection(knoten));
					break;
				}
			}
		}
		if (!selektierterKnotenGefunden)
		{
			throw (new IllegalArgumentException(getClass().getName()
					+ "::setKnotenOffset \n\t=> Der Knoten mit der PID "
					+ pidSelektierterKnoten + " wurde nicht gefunden!"));
		}

		/*
		 * Textfeld Kilometer setzen
		 */
		_textKnotenOffsetKilometer.setText("" + km);

		/*
		 * Richtung setzen
		 */
		if (selektierteRichtung.equals(Richtung.positiv))
		{
			_radioButtonKnotenOffsetNegativ.setSelection(false);
			_radioButtonKnotenOffsetPositiv.setSelection(true);
		} else if (selektierteRichtung.equals(Richtung.negativ))
		{
			_radioButtonKnotenOffsetPositiv.setSelection(false);
			_radioButtonKnotenOffsetNegativ.setSelection(true);
		}

	}

//	/**
//	 * Erzeugen der Seiten fr den {@link TabFolder}.
//	 * 
//	 * @param folder
//	 *          {@link TabFolder}, zu dem die Seiten hinzugefgt werden sollen.
//	 * @param label
//	 *          Bezeichnung der Seite im Reiter.
//	 * @return
//	 */
//	private static Composite erzeugeSeite(TabFolder folder, String label)
//	{
//		// TabItem erzeugen + TabText setzen
//		TabItem tab = new TabItem(folder, SWT.NONE);
//		tab.setText(label);
//
//		// Composite erzeugen
//		Composite page = new Composite(folder, SWT.NONE);
//
//		// Composite als Control dem Tab zuordnen
//		tab.setControl(page);
//		return page;
//	}
	
	/**
	 * Erzeugen der Seiten fr den {@link TabFolder}.
	 * 
	 * @param folder
	 *          {@link TabFolder}, zu dem die Seiten hinzugefgt werden sollen.
	 * @param label
	 *          Bezeichnung der Seite im Reiter.
	 * @return
	 */
	private static Composite erzeugeCompositeGroup(Composite parent, String label)
	{
	  // TabItem erzeugen + TabText setzen
	  Group group = new Group(parent, SWT.NONE);
	  group.setText(label);
	  
	  // Composite erzeugen
//	  Composite page = new Composite(parent, SWT.NONE);
	  	
	  return group;
	}

}
