package de.bsvrz.buv.plugin.parameter.ufd.haqparam;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;

import de.bsvrz.buv.plugin.parameter.ufd.common.AbstractDataContainerComposite;
import de.bsvrz.buv.plugin.parameter.ufd.dav.DaVKommunikation;
import de.bsvrz.buv.plugin.parameter.ufd.dav.Tools;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;


public class HAQZuordnungSensorComposite extends AbstractDataContainerComposite
{
  private List<SystemObject> _sensoren = new ArrayList<SystemObject>();
  
  private Combo _comboSensor;
  
  public HAQZuordnungSensorComposite(Composite parent, int style) {
    super(parent, style);
  }

  @Override
  public void initialize() {
    ClientDavInterface connection = DaVKommunikation.getInstanz().holeDaVVerbindung();
    if (connection != null) {
      DataModel dataModel = connection.getDataModel();
      
      SystemObjectType type = dataModel.getType("typ.umfeldDatenSensor");
      
      if (type != null) {
        for (SystemObject so : type.getElements())
        {
          if (so.isValid() && !so.getPid().startsWith("swarco.pua")) {
            _sensoren.add(so);
          }  
        }  
        
        Collections.sort(_sensoren, new Comparator<SystemObject>() {

          @Override
          public int compare(SystemObject arg0, SystemObject arg1)
          {
            String name0 = Tools.getName(arg0);
            String name1 = Tools.getName(arg1);
            
            return name0.compareTo(name1);
          }
          
        });
      }
      
      _sensoren.add(0, null);
    }
      
    super.initialize();
  }
  
  private String getEntryText(SystemObject so) {
    if (so != null) {
      return so.toString();
    }
    else {
      return "undefiniert";
    }
  }
  
  protected void createControls()
  {
    GridLayout gridLayout = new GridLayout(2, false);
    GridData gridData = new GridData();
//    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
  
    Group groupParameter = new Group(this, SWT.NONE);
    groupParameter.setLayout(gridLayout);
    groupParameter.setLayoutData(gridData);
    groupParameter.setText("ZuordnungSensor");
    
    Label label = new Label(groupParameter, SWT.NONE);
    label.setText("Sensor:");
    
    gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    
    _comboSensor = new Combo(groupParameter, SWT.READ_ONLY);
    _comboSensor.setLayoutData(gridData);
    
    for (SystemObject so: _sensoren) {
      _comboSensor.add(getEntryText(so));
    }
    _comboSensor.setVisibleItemCount(10);
  }

  @Override
  public Data getUpdate()
  {
    if (!isInitialized())
      return null;
    
    Data result = _data.createModifiableCopy();
    
    int index = _comboSensor.getSelectionIndex();
    if (index >= 0 && index < _sensoren.size()) {
      result.getItem("Sensor").asReferenceValue().setSystemObject(_sensoren.get(index));
    }
    return result;
  }

  @Override
  public void setUpdate(Data data)
  {
    if (!isInitialized())
      return;
    
    SystemObject so = data.getItem("Sensor").asReferenceValue().getSystemObject();
    String text = getEntryText(so);
    if (Arrays.asList(_comboSensor.getItems()).contains(text)) {
      _comboSensor.setText(text);  
    }
    else {
      _comboSensor.setText("_Undefiniert_");
    }
    
  }

}
