/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.pua.basis;

import java.text.SimpleDateFormat;
import java.util.Collections;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;
import java.util.Map.Entry;

import de.bsvrz.buv.plugin.pua.basis.pua.ProtokolleUndAuswertungen;
import de.bsvrz.buv.plugin.pua.basis.pua.daten.ProtokollInformationen;
import de.bsvrz.dav.daf.main.archive.ArchiveDataKind;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.losb.util.Tuple;

/**
 * Methode liefert die Informationen die zu einem Protokoll gehren als einzelne Elemente oder als String
 * 
 * @author Swarco Traffic Systems GmbH
 */
public class ProtokollInfo
{
  /**
   * Maximale Anzahl von Elementen die in Form eines Strings ausgegeben werden
   */
  private int _maxAnzahl = 5;

  /**
   * Protokollinformation
   */
  private ProtokollInformationen m_info;

  /**
   * Instanz der aufrufenden PUA
   */
  private ProtokolleUndAuswertungen m_pua;

  /**
   * angewhlte Objekte
   */
  private Vector<SystemObject> m_objekte = new Vector<SystemObject>();

  /**
   * Zeitbereiche
   */
  private Vector<String> m_zeitbereiche = new Vector<String>();

  /**
   * Archivdatenart
   */
  private ArchiveDataKind[] m_archivDatenArt;

  /**
   * Max. Anzahl von Objekten die als String hintereinander ausgegeben werden
   */
  private int m_maxAnzahlObjekte;

  /**
   * Max. Anzahl von Zeitbereichen die als String hintereinander ausgegeben werden
   */
  private int m_maxAnzahlZeitbereiche;

  /**
   * Skriptname
   */
  private String m_scriptName;

  /**
   * Protokoll ID
   */
  private long m_protokollId = 0l;

  /**
   * Informationstext
   */
  private String m_informationsText;
  
  /**
   * Konstruktor der Klasse
   * 
   * @param info
   *          Protokollinformation
   * @param protokolleUndAuswertungen
   *          Instanz der PUA
   * @param stream
   *          Stream mit dem Protokoll
   */
  public ProtokollInfo(ProtokollInformationen info, ProtokolleUndAuswertungen protokolleUndAuswertungen)
  {
    m_info = info;

    m_pua = protokolleUndAuswertungen;
    
    m_informationsText = info.getInformationsText();

    bestimmeInfo();

    bestimmeZeitbereiche();

    bestimmeArchivDatenArt();
  }

  /**
   * Methode bestimmt die bentigten Informationen
   */
  private void bestimmeInfo()
  {
    bestimmeObjekte();
  }

  /**
   * Methode bestimmt die angewhlten Objekte
   */
  private void bestimmeObjekte()
  {
    Map<String, String[]> psObjekte = new Hashtable<String, String[]>();
    if (m_info == null)
      return;

    psObjekte = m_info.getPseudoObjekte();

    Iterator it = psObjekte.entrySet().iterator();
    while (it.hasNext())
    {
      Map.Entry entry = (Entry)it.next();

      String pidObjekte[] = (String[])entry.getValue();
      for (int i = 0; i < pidObjekte.length; ++i)
      {
        SystemObject objekt = null;

        try
        {
          objekt = m_pua.getClientDaVConnection().getDataModel().getObject(pidObjekte[i]);
        }
        catch (Exception e)
        {
        }

        if (objekt != null)
          m_objekte.add(objekt);
      }
    }
  }

  /**
   * Methode liefert die angewhlten Systemobjekte
   * 
   * @return angewhlten Systemobjekte
   */
  public Vector<SystemObject> getObjekte()
  {
    return m_objekte;
  }

  /**
   * Methode liefert die angewhlten Systemobjekte als String.
   * 
   * @return angewhlten Systemobjekte als String
   */
  public String getObjekteAsString()
  {
    return getObjekteAsString(-1);
  }

  /**
   * Methode liefert die angewhlten Systemobjekte als String. Es werden max. <anzahl> Objekte angezeigt. Bei mehr
   * Objekten werden 3 Punkte ("...") angehngt
   * 
   * @param anzahl
   *          Max. Anzahl auszugebender Objekt, < 0 alle Objekte werden ausgegeben
   * @return angewhlten Systemobjekte als String
   */
  public String getObjekteAsString(int anzahl)
  {
    int anz = m_objekte.size();

    if (anzahl >= 0)
    {
      if (anz > anzahl)
        anz = anzahl;
    }

    String s = "";
    for (int i = 0; i < anz; ++i)
    {
      s += m_objekte.get(i).getName();
      if (m_objekte.size() > 1)
      {
        if (i < (m_objekte.size() - 1))
          s += ",  ";
      }
    }

    if (m_objekte.size() > anz)
      s += "...";

    m_maxAnzahlObjekte = anz;

    return s;
  }

  /**
   * Methode prft ob mehr angewhlte Objekte vorhanden sind wie als String ausgegeben werden
   * 
   * @return mehr angewhlte Objekte vorhanden
   */
  public boolean isZusaetzlicheObjekteVorhanden()
  {
    return m_objekte.size() > m_maxAnzahlObjekte;
  }

  /**
   * Methode bestimmt die angeforderten Zeitbereiche
   */
  private void bestimmeZeitbereiche()
  {
    List<Tuple<Long, Long>> zb = new Vector<Tuple<Long, Long>>();

    if (m_info == null)
      return;

    zb = m_info.getIntervalle();
    for (int i = 0; i < zb.size(); ++i)
    {
      Tuple<Long, Long> t = zb.get(i);

      long start = t.first;
      long end = t.last;

      SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss");

      try
      {
        String tmp = sdf.format(start) + " - " + sdf.format(end);
        m_zeitbereiche.add(tmp);
      }
      catch (Exception e)
      {
      }
    }
  }

  /**
   * Methode liefert die angeforderten Zeitbereiche
   * 
   * @return angeforderten Zeitbereiche
   */
  public Vector<String> getZeitbereiche()
  {
    return m_zeitbereiche;
  }

  /**
   * Methode liefert die angewhlten Zeitbereiche als String.
   * 
   * @return angewhlte Zeitbereiche als String
   */
  public String getZeitbereicheAsString()
  {
    return getZeitbereicheAsString(-1);
  }

  /**
   * Methode liefert die angewhlten Zeitbereiche als String. Es werden max. <anzahl> Zeitbereiche angezeigt. Bei mehr
   * Zeitbereichen werden 3 Punkte ("...") angehngt
   * 
   * @param anzahl
   *          Max. Anzahl auszugebender Zeitbereiche, < 0 alle Zeitbereiche werden ausgegeben
   * @return angewhlte Zeitbereiche als String
   */
  public String getZeitbereicheAsString(int anzahl)
  {
    int anz = m_zeitbereiche.size();

    if (anzahl >= 0)
    {
      if (anz > anzahl)
        anz = anzahl;
    }

    String s = "";
    for (int i = 0; i < anz; ++i)
    {
      s += m_zeitbereiche.get(i);
      if (m_zeitbereiche.size() > 1)
      {
        if (i < (m_zeitbereiche.size() - 1))
          s += ",  ";
      }
    }

    if (m_zeitbereiche.size() > anz)
      s += "...";

    m_maxAnzahlZeitbereiche = anz;

    return s;
  }

  /**
   * Methode prft ob mehr angewhlte Zeitbereiche vorhanden sind wie als String ausgegeben werden
   * 
   * @return mehr angewhlte Zeitbereiche vorhanden
   */
  public boolean isZusaetzlicheZeitbereicheVorhanden()
  {
    return m_zeitbereiche.size() > m_maxAnzahlZeitbereiche;
  }

  /**
   * Methode bestimme die Archivdatenart
   */
  private void bestimmeArchivDatenArt()
  {
    m_archivDatenArt = new ArchiveDataKind[0];
    if (m_info == null)
      return;

    m_archivDatenArt = m_info.getArchivDatenArt().toArray(new ArchiveDataKind[0]);
        
//    m_archivDatenArt = m_info.getArchivDatenArt();
  }

  /**
   * Methode liefert die Archivdatenart
   * 
   * @return Archivdatenart
   */
  public ArchiveDataKind[] getArchivDatenArt()
  {
    if (m_info == null)
      return null;

    return m_archivDatenArt;
  }

  /**
   * Methode liefert die angewhlten Archivdatenart als String. Es werden max. 5 Arten angezeigt. Bei mehr Arten werden
   * 3 Punkte ("...") angehngt
   * 
   * @return angewhlte Archivdatenarten als String
   */
  public String getArchivDatenArtAsString()
  {
    int anz = m_archivDatenArt.length;
    if (anz > _maxAnzahl)
      anz = _maxAnzahl;

    String s = "";
    for (int i = 0; i < anz; ++i)
    {
      ArchiveDataKind a = m_archivDatenArt[i];
      s += a.toString();
      if (m_archivDatenArt.length > 1)
      {
        if (i < (m_archivDatenArt.length - 1))
          s += ",  ";
      }
    }

    if (m_archivDatenArt.length > _maxAnzahl)
      s += "...";

    return s;
  }

  /**
   * Methode liefert die angeforderte Simulationsvariante
   * 
   * @return angeforderte Simulationsvariante
   */
  public int getSimulationsVariante()
  {
    if (m_info == null)
      return 0;

    return m_info.getSimlationsVariante();
  }

  /**
   * Methode zum Setzen des Skriptnamens
   * 
   * @param scriptName
   *          Skriptname
   */
  public void setScriptName(String scriptName)
  {
    m_scriptName = scriptName;
  }

  /**
   * liefert die Variable scriptName zurck
   * 
   * @return Variable scriptName
   */
  public String getScriptName()
  {
    if (m_info == null)
      return "";

    return m_scriptName;
  }

  /**
   * Methose zum Setzen der Protokoll-ID
   * 
   * @param protokollId
   *          Protokoll-ID
   */
  public void setProtokollId(long protokollId)
  {
    m_protokollId = protokollId;
  }

  /**
   * liefert die Variable protokollId zurck
   * 
   * @return Variable protokollId zurck
   */
  public long getProtokollId()
  {
    return m_protokollId;
  }
  
  /**
   * liefert die Variable informationsText zurck
   * 
   * @return Variable informationsText
   */
  public String getInformationsText()
  {
    if (m_info == null)
      return "";

    return m_informationsText;
  }
  
  public String getInformationsTextBereinigt() {
  	return m_info.getInformationsTextBereinigt();
  }
  
  public boolean isOeffentlich() {
	  return m_info.isOeffentlich();
  }
  
  public boolean isAenderungsProtokoll() {
	  return m_info.isAenderungsProtokoll();
  }
  
  public Map<String, String> getBindungen() {
	  return Collections.unmodifiableMap(m_info.getBindungen());
  }

}
