package de.bsvrz.buv.plugin.pua.basis.combinedChart;

import java.util.HashSet;
import java.util.Set;

import org.eclipse.core.runtime.Status;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.plugin.pua.basis.Activator;
import de.bsvrz.buv.plugin.pua.basis.views.GanglinieSicht;

public class KombinierteGanglinieAuswahlView extends ViewPart implements IChartAvailableListener
{
  
  public final static String ID = "de.bsvrz.buv.plugin.pua.basis.combinedChart.kombinierteGanglinienAuswahlView";
  
  public final static String HELP_ID = Activator.PLUGIN_ID + ".kombinierteGanglinienAuswahl";
  
  private static long INSTANCE_COUNT = 0;
  
  private KombinierteGanglinieAuswahlComposite _combinedChartComposite;
  
  private Button _buttonKombinierteGanglinie;

  private Button _buttonSchliessen;

  private Button _buttonHilfe;

  @Override
  public void createPartControl(Composite parent)
  {
   
    Composite composite = new Composite(parent, SWT.NONE);
    composite.setLayout(new GridLayout());
    
    //Fr die F1 Taste
    PlatformUI.getWorkbench().getHelpSystem().setHelp(composite, HELP_ID);
            
    _combinedChartComposite = new KombinierteGanglinieAuswahlComposite(composite, SWT.NONE);
    _combinedChartComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
  
    ProtokollAusgabeManager.getInstance().addProtokollAusgabeListener(_combinedChartComposite);
   
    Composite compositeButton = new Composite( composite, SWT.NONE );
    compositeButton.setLayoutData( new GridData(SWT.END, SWT.CENTER, true, false) );
    
    RowLayout rowLayout = new RowLayout();
    rowLayout.pack = false;
    compositeButton.setLayout( rowLayout );

    _buttonKombinierteGanglinie = new Button( compositeButton, SWT.NONE );
    _buttonKombinierteGanglinie.setEnabled(false);
    _buttonKombinierteGanglinie.setText( "Ganglinie" );
    _buttonKombinierteGanglinie.addSelectionListener( new SelectionAdapter()
    {
      @Override
      public void widgetSelected( SelectionEvent e )
      {
        aktionGanglinie();
      }
    } );

    _buttonSchliessen = new Button( compositeButton, SWT.NONE );
    _buttonSchliessen.setText( "Schlieen" );
    _buttonSchliessen.addSelectionListener( new SelectionAdapter()
    {

      @Override
      public void widgetSelected( SelectionEvent e )
      {
        aktionSchliessen();
      }

    } );
    _buttonHilfe = new Button( compositeButton, SWT.NONE );
    _buttonHilfe.setText( "Hilfe" );
    _buttonHilfe.addSelectionListener( new SelectionAdapter()
    {

      @Override
      public void widgetSelected( SelectionEvent e )
      {
        aktionHilfe();
      }

    } );
    
    _combinedChartComposite.addChartAvailableListener(this);
  }
  
  @Override
  public void dispose()
  {
    if (_combinedChartComposite != null) {
      _combinedChartComposite.removeChartAvailableListener(this);
      ProtokollAusgabeManager.getInstance().removeProtokollAusgabeListener(_combinedChartComposite);
    }
    super.dispose();
  }

  @Override
  public void setFocus()
  {
    if (_combinedChartComposite != null) {
      _combinedChartComposite.getParent().setFocus();
    }
  }
  
  private void aktionGanglinie()
  {
    if (_combinedChartComposite != null && _combinedChartComposite.checkSelection()) {
      IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
      IWorkbenchPage page = dw.getActivePage();
      try
      {
        String secondaryId = "" + GanglinieSicht.INSTANCE_COUNT;
               
//        System.out.println(GanglinieSicht.ID + " : " + secondaryId);
        
        page.showView(GanglinieSicht.ID, secondaryId, IWorkbenchPage.VIEW_VISIBLE);
        IViewReference ref = page.findViewReference(GanglinieSicht.ID, secondaryId);
        IViewPart vPart = ref.getView(true);
        if (vPart instanceof GanglinieSicht)
        {
          GanglinieSicht view = (GanglinieSicht)vPart;

          view.erzeugeChart(_combinedChartComposite.erzeugeChartInput(), " kombiniert - " + 
              ++INSTANCE_COUNT);
        }
        GanglinieSicht.INSTANCE_COUNT++;
      }
      catch (PartInitException e)
      {
        String msg = "Fehler beim ffnen der Sicht ID \"" + GanglinieSicht.ID + "\"";
        Activator.getDefault().getLog().log(new Status(Status.ERROR, Activator.PLUGIN_ID, msg, e));
        System.err.println(msg);
        e.printStackTrace(System.err);
      }
    }
   
  }
  
  private void aktionSchliessen()
  {
    getViewSite().getPage().hideView(this);
  }
  
  /**
   * Zeigt die Hilfe an.
   * 
   */
  private void aktionHilfe()
  {
    PlatformUI.getWorkbench().getHelpSystem().displayHelp(HELP_ID);
  }

  @Override
  public void chartAvailable(boolean available)
  {
    _buttonKombinierteGanglinie.setEnabled(available);
  }

}
