package de.bsvrz.buv.plugin.pua.basis.combinedChart;

import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.Map;

import org.eclipse.core.runtime.Status;

import de.bsvrz.buv.plugin.pua.basis.Activator;

/**
 * Registrieren und Deregistrieren von ausgegebenenen PUA-Protokollen
 *  
 * @author Sans
 *
 */
public class ProtokollAusgabeManager
{
  
  private Map<String, IProtokollAusgabe> _protokollAusgabeMap = new LinkedHashMap<String, IProtokollAusgabe>();
  
  private Collection<IProtokollAusgabeListener> _listeners = new LinkedHashSet<IProtokollAusgabeListener>();
  
  private ProtokollAusgabeManager() {
    // prevent Instantiation
  }
  
  private static class Loader {
    final static ProtokollAusgabeManager INSTANCE = new ProtokollAusgabeManager();
  }
  
  /**
   * 
   * @return the single instance of this manager
   */
  public static ProtokollAusgabeManager getInstance() {
    return Loader.INSTANCE;
  }
  
  public synchronized void register(IProtokollAusgabe pa) {
    if (!_protokollAusgabeMap.containsKey(pa.getId())) {
      _protokollAusgabeMap.put(pa.getId(), pa);
      fireUpdateProtokollAusgaben();
      System.out.println("Register ProtokollAusgabe " + pa.getId());
    }
    else {
      String msg = "ProtokollAusgabe " + pa.getId() + " ist bereits registriert";
      Activator.getDefault().getLog().log(new Status(Status.WARNING, Activator.PLUGIN_ID, msg));
      System.err.println(msg);
    }
  }
  
  public synchronized void unregister(IProtokollAusgabe pa) {
    if (_protokollAusgabeMap.remove(pa.getId()) != null) {
      fireUpdateProtokollAusgaben();
      System.out.println("Unregister ProtokollAusgabe " + pa.getId());
    }
  }
  
  private void fireUpdateProtokollAusgaben(IProtokollAusgabeListener listener) {
    Collection<IProtokollAusgabe> values = _protokollAusgabeMap.values();
    listener.updateProtokollAusgaben(values.toArray(new IProtokollAusgabe[values.size()]));
  }
  
  private void fireUpdateProtokollAusgaben() {
    for (IProtokollAusgabeListener listener: _listeners) {
      fireUpdateProtokollAusgaben(listener);
    }
  }
  
  public synchronized void addProtokollAusgabeListener(IProtokollAusgabeListener listener) {
    if (_listeners.add(listener)) {
      fireUpdateProtokollAusgaben(listener);
    }
  }
  
  public synchronized void removeProtokollAusgabeListener(IProtokollAusgabeListener listener) {
    _listeners.remove(listener);
  }
  
}
