/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.pua.basis.dav;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.swt.widgets.Display;

import de.bsvrz.buv.plugin.pua.basis.DaVKommunikation;
import de.bsvrz.buv.plugin.pua.basis.archivDataUpdateListener.ArchiveDataUpdateEvent;
import de.bsvrz.buv.plugin.pua.basis.archivDataUpdateListener.ArchiveDataUpdateListener;
import de.bsvrz.buv.plugin.pua.basis.archivDataUpdateListener.IArchiveDataUpdateListener;
import de.bsvrz.buv.rw.basislib.kalender.KalenderZeitFilterDatenStrukturZeitGruppe;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.archive.ArchiveData;
import de.bsvrz.dav.daf.main.archive.ArchiveDataQueryResult;
import de.bsvrz.dav.daf.main.archive.ArchiveDataSpecification;
import de.bsvrz.dav.daf.main.archive.ArchiveDataStream;
import de.bsvrz.dav.daf.main.archive.ArchiveQueryPriority;
import de.bsvrz.dav.daf.main.archive.ArchiveRequestManager;
import de.bsvrz.dav.daf.main.archive.ArchiveTimeSpecification;
import de.bsvrz.dav.daf.main.archive.TimingType;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Hauptklasse der Applikation ArchiveRequest.<br>
 * <br>
 * Die Applikation TimeSync liest DE-Objekte nach einem bestimmten Muster aus der Datenverteiler- Konfiguration aus, und
 * sendet an die dem Muster entsprechenden Objekte die aktuelle Uhrzeit (Zeitsynchronisation). Bei jedem Lauf wird eine
 * Log-Datei erzeugt, der Name der Datei besteht aus TimeSync_'Datum'_'Uhrzeit'.log. <br>
 * <br>
 * Folgende Startparameter sind mglich:<br>
 * <br>
 * DAF-Startparameter wie in SSB beschrieben und durch StandardApplication verarbeitet:<br>
 * <br>
 * Beispiele:<br>
 * -datenverteiler=rechnerDav:8083<br>
 * -debugLevelStdErrText=FINE<br>
 * -benutzer=Tester<br>
 * -authentifizierung=C:\kernsoftware-system-20-03-2006\passwd<br>
 * <br>
 * Spezielle Startparameter:<br>
 * <br>
 * -uhrzeit=Uhrzeit an welcher die Zeitsynchronisation durgefuehrt wird <br>
 * -wochentag=Wird hier ein Wert zwischen 0 (Sonnatag) und 7 (Samstag) eingetragen wird die Zeitsynchronisation nur an
 * diesem Wochentag durchgefhrt. Bei keinem Eintrag wird taeglich an der bei dem Parameter '-uhrzeit' eingestellten
 * Zeit die Synchronisation durchgefuehrt. <br>
 * -bereiche=zu verwendende Konfigurationsbereiche durch ':' getrennt (optional, bei keiner Angabe Verwendung aller
 * vorhandenen Konfigurationsbereiche) <br>
 * -datei=bei Wert 0 keine Abfrage der Datenverteiler-Konfiguration, es werden nur die in Datei eingetragenen Objekte
 * zur Zeitsynchronisation verwendet. Wird kein Wert angeben findet trotzdem eine Abfrage statt. <br>
 * Beispiele:<br>
 * -uhrzeit=14:07<br>
 * -wochentag=2<br>
 * -bereiche=kb.objekte.VerkehrMqFs.uz.Dambach -datei=0 <br>
 * 
 * @version $Log: ArchiveRequest.java,v $
 * @version Revision 1.3  2012/12/21 09:56:27  Pittner
 * @version Update Pittner 20121221 License
 * @version Committed on the Free edition of March Hare Software CVSNT Server.
 * @version Upgrade to CVS Suite for more features and support:
 * @version http://march-hare.com/cvsnt/
 * @version
 * @version Revision 1.2  2012/02/07 13:26:53  Sans
 * @version Sicherheitskopie 20111216
 * @version
 * @version
 * @version
 * @version
 * @version Committed on the Free edition of March Hare Software CVSNT Server.
 * @version Upgrade to CVS Suite for more features and support:
 * @version http://march-hare.com/cvsnt/
 * @version
 * @version Revision 1.1  2011/10/27 15:46:51  Sans
 * @version Initiales Check-in 20111027
 * @version Committed on the Free edition of March Hare Software CVSNT Server.
 * @version Upgrade to CVS Suite for more features and support:
 * @version http://march-hare.com/cvsnt/
 * @version
 * @version Revision 1.1  2011/10/13 10:01:28  Sans
 * @version Initiales Check-in 20111013
 * @version Committed on the Free edition of March Hare Software CVSNT Server.
 * @version Upgrade to CVS Suite for more features and support:
 * @version http://march-hare.com/cvsnt/
 * @version
 * @version Revision 1.1  2011/08/31 16:59:14  Pittner
 * @version Ersterstellung Pittner 20110831
 * @version Committed on the Free edition of March Hare Software CVSNT Server.
 * @version Upgrade to CVS Suite for more features and support:
 * @version http://march-hare.com/cvsnt/
 * @version
 * @version Revision 1.2 2010/10/29 17:11:04 Pittner
 * @version Update Pittner 20101029
 * @version Committed on the Free edition of March Hare Software CVSNT Server.
 * @version Upgrade to CVS Suite for more features and support:
 * @version http://march-hare.com/cvsnt/
 * @version
 * @version Revision 1.1 2010/10/28 16:55:56 Pittner
 * @version Update Pittner 20101028
 * @version Committed on the Free edition of March Hare Software CVSNT Server.
 * @version Upgrade to CVS Suite for more features and support:
 * @version http://march-hare.com/cvsnt/
 * @version
 * @version Revision 1.6 2009/10/14 15:12:26 HofmannJ
 * @version Ausgabe fr Langzeitdaten hinzugefgt
 * @version
 * @version Revision 1.5 2009/09/15 12:31:36 HofmannJ
 * @version Dokumentation ergnzt.
 * @version
 * @version Revision 1.4 2009/09/15 10:00:49 HofmannJ
 * @version Streams der Archivanfrage werden bei Schlieen der entsprechenden Views Ordnungsgem in der Klasse
 *          ArchiveRequest geschlossen. Methode abort() wird auf jedem Stream aufgerufen.
 * @version
 * @version Revision 1.3 2009/09/10 11:32:51 HofmannJ
 * @version Jedes Auswertungsergebnis-View hat jetzt ihren eigenen Datenlieferanten. Oberflche blockiert nicht beim
 *          ffnen.
 * @version
 * @version Revision 1.2 2009/08/07 11:18:18 ObertM
 * @version Datensicherung
 * @version
 * @version Revision 1.1 2009/08/04 13:04:58 ObertM
 * @version Datensicherung
 * @version
 * @version Revision 1.2 2009/07/30 10:49:05 Pittner
 * @version Update Pittner
 * @version
 * @version Revision 1.1 2009/07/30 07:57:27 Pittner
 * @version Update Pittner
 * @version
 * @version Revision 1.2 2009/07/30 07:47:23 Pittner
 * @version Update Pittner
 * @version
 * @version Revision 1.1 2009/07/29 14:29:55 Pittner
 * @version Update Pittner
 * @version
 * @version Revision 1.2 2007/03/01 15:11:28 Pittner
 * @version Datensicherung
 * @version
 * @version Revision 1.1 2007/03/01 13:37:45 Pittner
 * @version Datensicherung
 * @version
 * @version Revision 1.3 2007/02/27 14:55:54 Pittner
 * @version Datensicherung
 * @version
 * @version Revision 1.2 2006/12/05 09:36:05 Drapp
 * @version *** empty log message ***
 * @version
 * @version Revision 1.1 2006/11/28 08:42:51 Drapp
 * @version Testeinstellung
 * @version
 * @version Revision 0.10 2006/10/06 14:56:08 Pittner
 * @version Datensicherung
 * 
 * @version $Revision: 1.3 $ / $Date: 2012/12/21 09:56:27 $ / ($Author: Pittner $)
 * 
 * @author Dambach-Werke GmbH
 * @author Timo Pittner
 * 
 */
public class ArchiveRequest
{
  /**
   * DebugLogger fr Debug-Ausgaben
   */
  private static Debug _debug;

  /**
   * Verbindung zum Datenverteiler
   */
  public ClientDavInterface _connection;

  /**
   * Speichert den Zugriff auf das Archivsystem.
   */
  private ArchiveRequestManager _archiveRequestManager;

  /**
   * Liste der {@link IArchiveDataUpdateListener}, die ber einen neue DaV-Daten informiert werden mchten.
   */
  private LinkedList<IArchiveDataUpdateListener> _archiveDataUpdateListener = new LinkedList<IArchiveDataUpdateListener>();

  /**
   * Flag das anzeigt, ob der Thread luft oder nicht. Wird diese Variable zur Laufzeit des Threads von
   * <code>true</code> auf <code>false</code> gesetzt, so beendet sich der Thread und die Archiv-Streams werden mittel
   * abort() geschlossen. Andere Archivanfragen knnen dadurch direkt loslaufen.
   */
  private boolean _laufeThread = false;

  public ArchiveRequest()
  {
    _debug = Debug.getLogger();
  }

  /**
   * Fgt einen Listener zur Liste der DaVDataUpdate-Listener hinzu.<br>
   * Diese Methode ist Threadsicher.
   * 
   * @param listener
   *          Hinzuzufgender Listener.
   */
  public void addArchiveDataUpdateListener(IArchiveDataUpdateListener listener)
  {
    synchronized (_archiveDataUpdateListener)
    {
      if (!_archiveDataUpdateListener.contains(listener))
      {
        _archiveDataUpdateListener.add(listener);
      }
    }
  }

  /**
   * Entfernt einen {@link IArchiveDataUpdateListener} von der Liste der Zuhrer.<br>
   * Diese Methode ist Threadsicher.
   * 
   * @param IArchiveDataUpdateListener
   *          Zu entfernender {@link IArchiveDataUpdateListener}.
   */
  public void removeArchiveDataUpdateListener(IArchiveDataUpdateListener listener)
  {
    synchronized (_archiveDataUpdateListener)
    {
      if (_archiveDataUpdateListener.contains(listener))
      {
        _archiveDataUpdateListener.remove(listener);
      }
    }
  }

  /**
   * Liefert eine Liste aller registrierten Listener.<br>
   * Diese Mehtode ist Threadischer.
   * 
   * @return Liste aller registrierten Listener.
   */
  private LinkedList<IArchiveDataUpdateListener> getListeners()
  {
    LinkedList<IArchiveDataUpdateListener> re = new LinkedList<IArchiveDataUpdateListener>();

    synchronized (_archiveDataUpdateListener)
    {
      for (IArchiveDataUpdateListener archiveDataUpdateListener : _archiveDataUpdateListener)
      {
        re.add(archiveDataUpdateListener);
      }
    }

    return re;
  }

  /**
   * Benachrichtigt alle registrierten {@link IArchiveDataUpdateListener} darber, dass neue Daten vorhanden sind.<br>
   * Diese Methode ist Threadsicher.
   * 
   */
  public void notifyArchiveDataUpdateListener(ArchiveData result)
  {

    ArchiveDataUpdateEvent event = new ArchiveDataUpdateEvent(this, result);

    Iterator<IArchiveDataUpdateListener> it = getListeners().iterator();
    while (it.hasNext())
    {
      IArchiveDataUpdateListener listener = it.next();
      listener.archiveDataUpdate(event);
    }
  }

  public void startRequest(List<AnfrageDaten> listeAnfrageDaten)
  {
    Date d = new Date();
    _connection = DaVKommunikation.getInstanz().holeDaVVerbindung();

    _archiveRequestManager = _connection.getArchive();

    List<ArchiveDataSpecification> archiveDataSpecifications = new LinkedList<ArchiveDataSpecification>();

    ArchiveQueryPriority prio = null;
    TimingType timingType = null;

    for (AnfrageDaten anfrageDaten : listeAnfrageDaten)
    {
      if (prio == null)
        prio = anfrageDaten.getArchiveQueryPriority();

      if (timingType == null)
        timingType = anfrageDaten.getTimingType();

      List<SystemObject> objekte = anfrageDaten.getObjekte();
      List<KalenderZeitFilterDatenStrukturZeitGruppe> zeitgruppen = anfrageDaten.getZeitgruppen();

      for (SystemObject systemObject : objekte)
      {

        for (KalenderZeitFilterDatenStrukturZeitGruppe zeitGruppenZeile : zeitgruppen)
        {
          d.setTime(zeitGruppenZeile.getStartzeitpunkt());
          d.setTime(zeitGruppenZeile.getEndzeitpunkt());

          ArchiveTimeSpecification archiveTimeSpecification = new ArchiveTimeSpecification(
              anfrageDaten.getTimingType(), anfrageDaten.getStartRelative(), zeitGruppenZeile.getStartzeitpunkt(),
              zeitGruppenZeile.getEndzeitpunkt());

          archiveDataSpecifications.add(new ArchiveDataSpecification(archiveTimeSpecification, anfrageDaten
              .getArchiveDataKindCombination(), anfrageDaten.getArchiveOrder(), anfrageDaten.getArchiveRequestOption(),
              anfrageDaten.getDataDescription(), systemObject));
        }
      }
    }

    // Anfrag starten
    final ArchiveDataQueryResult queryResult = _archiveRequestManager.request(prio, archiveDataSpecifications);
    try
    {
      if (queryResult.isRequestSuccessful())
      {
        _debug.info("Archivanfrage konnte erfolgreich bearbeitet werden.");
        ArchiveDataStream[] archiveDataStream = queryResult.getStreams();
        Thread archiveThread = new Thread(new RequestThread(archiveDataStream, timingType), "Archivanfrage "
            + new Date().toString());
        _laufeThread = true;
        archiveThread.start();
      }
      else
      {
        _debug.warning("Eine Archivanfrage konnte nicht bearbeitet werden, Fehler: " + queryResult.getErrorMessage());

      }
    }
    catch (InterruptedException ex)
    {
      _debug.warning(ex.getMessage());
    }
  }

  public void startRequest(AnfrageDaten anfrageDaten)
  {
    Date d = new Date();    
    _connection = DaVKommunikation.getInstanz().holeDaVVerbindung();
    _archiveRequestManager = _connection.getArchive();

    List<ArchiveDataSpecification> archiveDataSpecifications = new LinkedList<ArchiveDataSpecification>();

    List<SystemObject> objekte = anfrageDaten.getObjekte();
    List<KalenderZeitFilterDatenStrukturZeitGruppe> zeitgruppen = anfrageDaten.getZeitgruppen();

    for (SystemObject systemObject : objekte)
    {
      for (KalenderZeitFilterDatenStrukturZeitGruppe zeitGruppenZeile : zeitgruppen)
      {
        d.setTime(zeitGruppenZeile.getStartzeitpunkt());
        d.setTime(zeitGruppenZeile.getEndzeitpunkt());
        
        ArchiveTimeSpecification archiveTimeSpecification = new ArchiveTimeSpecification(anfrageDaten.getTimingType(),
            anfrageDaten.getStartRelative(), zeitGruppenZeile.getStartzeitpunkt(), zeitGruppenZeile.getEndzeitpunkt());

        archiveDataSpecifications.add(new ArchiveDataSpecification(archiveTimeSpecification, anfrageDaten
            .getArchiveDataKindCombination(), anfrageDaten.getArchiveOrder(), anfrageDaten.getArchiveRequestOption(),
            anfrageDaten.getDataDescription(), systemObject));

      }

    }

    // Anfrag starten
    final ArchiveDataQueryResult queryResult = _archiveRequestManager.request(anfrageDaten.getArchiveQueryPriority(),
        archiveDataSpecifications);
    try
    {
      if (queryResult.isRequestSuccessful())
      {
        _debug.info("Archivanfrage konnte erfolgreich bearbeitet werden.");        
        ArchiveDataStream[] archiveDataStream = queryResult.getStreams();
        Thread archiveThread = new Thread(new RequestThread(archiveDataStream, anfrageDaten.getTimingType()),
            "Archivanfrage " + new Date().toString());
        _laufeThread = true;

        archiveThread.start();

      }
      else
      {
        _debug.warning("Eine Archivanfrage konnte nicht bearbeitet werden, Fehler: " + queryResult.getErrorMessage());

      }
    }
    catch (InterruptedException ex)
    {
      _debug.warning(ex.getMessage());
    }
  }

  /**
   * Stoppt den Thread zur Auswertung des Streams des Archivs. Alle geffneten Streams werden mittels abort()
   * geschlossen. Diese Methode sollte z.B. dann, wenn ein Fenster welches auf Daten aus diesem Thread angewiesen ist
   * geschlossen wird, aufgerufen werden.
   */
  public void stopRequestThread()
  {
    if (_laufeThread)
    {
      _laufeThread = false;
    }
  }

  /**
   * Diese Klasse verarbeitet die erhaltenen Streams einer Archivanfrage und stellt die Datenstze in einer
   * Online-Tabelle dar.
   */
  private final class RequestThread implements Runnable
  {

    private final ArchiveDataStream[] _archiveDataStreams;

    public RequestThread(ArchiveDataStream[] archiveDataStreams, TimingType timingType)
    {
      _archiveDataStreams = archiveDataStreams;
    }

    public void run()
    {

      // TBD STH Sortierung nach Datenidentifikation - momentan wird
      // ein Objekt nach dem anderen abgearbeitet
      for (int i = 0; i < _archiveDataStreams.length; i++)
      {
        /*
         * Threadabbruch? Schleife beenden.
         */
        if (!_laufeThread)
        {
          break;
        }

        try
        {
          ArchiveDataStream dataStream = _archiveDataStreams[i];

          ArchiveData archiveData = dataStream.take();
          while (archiveData != null)
          {
            if (!_laufeThread)
            {
              break;
            }

            if (archiveData.getData() != null)
            {

              final ArchiveData fad = archiveData;

              Display.getDefault().asyncExec(new Runnable() {
                public void run()
                {
                  ArchiveDataUpdateListener.getInstanz().notifyArchiveDataUpdateListener(fad);

                  // Benachrichtigen aller nur diesem Thread bekannten Listener.
                  notifyArchiveDataUpdateListener(fad);
                }
              });

            }
            archiveData = dataStream.take();
          }

        }
        catch (InterruptedException ex)
        {
          _debug.error(ex.getMessage());

        }
        catch (IOException ex)
        {
          _debug.error(ex.getMessage());

        }
        finally
        {

          _debug.info("Keine weiteren Archivdatenstze fr diese Datenidentifikation.");
        }
      }

      /*
       * Bei vorzeitigem Abbruch alle Streams schlieen.
       */
      if (!_laufeThread)
      {
        for (ArchiveDataStream stream : _archiveDataStreams)
        {
          stream.abort();
        }
      }

      // Date d = new Date();
      // System.out.println("...finished Request at: " + d.toString());

      Display.getDefault().asyncExec(new Runnable() {
        public void run()
        {
          ArchiveDataUpdateListener.getInstanz().notifyArchiveDataUpdateListener(null);

          // Benachrichtigen aller diesem Thread bekannten Listener.
          notifyArchiveDataUpdateListener(null);
        }
      });

    }
  }

}
