/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.pua.basis.expose;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import de.bsvrz.buv.plugin.pua.basis.DaVKommunikation;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

/**
 * Default ObjektProvider
 * 
 * @author Sans
 *
 */
public class ObjektProviderDefault implements IObjektProvider {
	
	/**
	 * 
	 * @see de.bsvrz.buv.plugin.pua.basis.expose.IObjektProvider#getGuiObjekte(java.lang.String)
	 * @return alle Objekte des bergebenenen Typs, gefiltert gemss bergabeparametern
	 */
	public List<SystemObject> getObjekte(String pidSystemObjectType, boolean ausfilternDE255,
			String umsetzungsModulFilterText) {
		List<SystemObject> result = new ArrayList<SystemObject>();
		ClientDavInterface connection = DaVKommunikation.getInstanz().holeDaVVerbindung();
		if (connection != null) {
			DataModel dataModel= connection.getDataModel();
			SystemObjectType typ = dataModel.getType(pidSystemObjectType);
			AttributeGroup atg = dataModel.getAttributeGroup("atg.de");
			for (SystemObject so: typ.getElements())
			{
				boolean hinzufuegen = true;
				if (ausfilternDE255) {
					Data data = so.getConfigurationData(atg);
					hinzufuegen = (data == null || data.getUnscaledValue("DEKanal").intValue() != 255);
				}
				if (hinzufuegen && umsetzungsModulFilterText != null && 
						!umsetzungsModulFilterText.isEmpty()) {
					Data data = so.getConfigurationData(atg);
					hinzufuegen = (data == null || 
						data.getTextValue("UmsetzungsModul").getText().contains(umsetzungsModulFilterText));
				}
				if (hinzufuegen) {
					result.add(so);
				}
			}
		}
		return result;
	}
	
	public SystemObject getHauptObjekt(List<SystemObject> objekte) {
		return objekte.isEmpty() ? null : objekte.get(0);
	}
	
	/**
	 * 
	 * @see de.bsvrz.buv.plugin.pua.basis.expose.IObjektProvider#getPuaObjekte(java.util.Collection) 
	 * @return die in <code>objekte</code> bergebenen Objekte unter dem Pseudoobjekt-Alias "mq0"
	 */
	public Map<String, String[]> getPseudoObjekte(List<SystemObject> objekte) {
		Map<String, String[]> result = new HashMap<String, String[]>();
		String[] pids = new String[objekte.size()];
		for (int i = 0; i < pids.length; i += 1) {
			pids[i] = objekte.get(i).getPid();
		}
		result.put("mq0", pids);
		return result;
	}
	
	public boolean isSpeichernSelektierteObjekte() {
		return false;
	}
	
}
