/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.pua.basis.font;
import java.awt.image.BufferedImage;
import java.util.BitSet;
import java.util.HashMap;

public class VisuAnzeigeFontsZeichen
{
  /**
   * Pid des Zeichen.
   */
  private String m_zeichenname;    
  /**
   * Bitmap des Zeichen.
   */
  private String m_bitmap[];
  /**
   * Hhe der Bitmap des Zeichen.
   */
  private int m_hoehe;    
  /**
   * Breite der Bitmap des Zeichen.
   */
  private int m_breite;
  /**
   * 
   */
  private HashMap<Integer, BitSet> m_bitmapBitSet = new HashMap<Integer, BitSet>();
  /**
   * Bitmap als Image.
   */
  private BufferedImage m_image;
  /**
   * ASCII-Code des Zeichen
   */
  private int m_asciiCode;
  /**
   * UNI-Code des Zeichen
   */
  private int m_uniCode;  
  
  /**
   * 
   */
  VisuAnzeigeFontsZeichen()
  {
  }

  /**
   * @return Returns the bitmap.
   */
  public String[] getBitmap()
  {
    return m_bitmap;
  }

  /**
   * @param bitmap The bitmap to set.
   */
  public void setBitmap(String[] bitmap)
  {
    m_bitmap = bitmap;
    
    m_bitmapBitSet.clear();
    
    for (int i = 0; i < m_bitmap.length; i++)
    {
      String zeile = m_bitmap[i];
      int laenge = zeile.length();
      
      BitSet bitSet = new BitSet(laenge);
      
      for (int j = 0; j < laenge; j++)
      {
//        char zeichen = zeile.charAt(j);
        
        if (zeile.charAt(j) == 'x')
        {
          bitSet.set(j);
        }
      }
      
      m_bitmapBitSet.put(i, bitSet);
    }
  }

  /**
   * @return Returns the zeichenname.
   */
  public String getZeichenname()
  {
    return m_zeichenname;
  }

  /**
   * @param zeichenname The zeichenname to set.
   */
  public void setZeichenname(String zeichenname)
  {
    m_zeichenname = zeichenname;
  }

  /**
   * @return Returns the breite.
   */
  public int getBreite()
  {
    return m_breite;
  }

  /**
   * @param breite The breite to set.
   */
  public void setBreite(int breite)
  {
    m_breite = breite;
  }

  /**
   * @return Returns the hoehe.
   */
  public int getHoehe()
  {
    return m_hoehe;
  }

  /**
   * @param hoehe The hoehe to set.
   */
  public void setHoehe(int hoehe)
  {
    m_hoehe = hoehe;
  }

  /**
   * 
   * 
   * @return Returns the bitmap.
   */
  public BitSet getBitmapBitSetZeile(int zeilenIndex)
  {
    if ((zeilenIndex >= 0) && (zeilenIndex < getHoehe()))
    {
      return (BitSet)m_bitmapBitSet.get(zeilenIndex);
    }
    
    return null;
  }
  
  /**
   * 
   */
  public void erstelleBitmap()
  {
    BufferedImage bitmap = new BufferedImage(getBreite(), getHoehe(), BufferedImage.TYPE_BYTE_BINARY);

    int rgb = 0xFFFFFF;

    BitSet bitSet;
    
    for (int i = 0; i < getHoehe(); i++)
    {
      bitSet = (BitSet)m_bitmapBitSet.get(i);
      
      for (int j = 0; j < bitSet.size(); j++)
      {
        if (bitSet.get(j))
        {
          bitmap.setRGB(j, i, rgb);
        }
      }
    }
    
    m_image = bitmap;
  }
  
  /**
   * @return
   */
  public BufferedImage getBufferedImage()
  {
    return m_image; 
  }

  /**
   * @return Returns the asciiCode.
   */
  public int getAsciiCode()
  {
    return m_asciiCode;
  }

  /**
   * @param asciiCode The asciiCode to set.
   */
  public void setAsciiCode(int asciiCode)
  {
    m_asciiCode = asciiCode;
  }

  /**
   * @return Returns the uniCode.
   */
  public int getUniCode()
  {
    int ergebnis;
    
    ergebnis = m_uniCode;
    
    if (ergebnis == 0) // UNI-Code nicht versorgt, da gleicher Wert wie ASCII-Code
    {
      ergebnis = m_asciiCode;
    }
    
    return ergebnis;
  }

  /**
   * @param uniCode The uniCode to set.
   */
  public void setUniCode(int uniCode)
  {
    m_uniCode = uniCode;
  }
}
