/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.pua.basis.sba.xml;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.util.Collection;
import java.util.Vector;

import org.jdom.Attribute;
import org.jdom.DocType;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.output.XMLOutputter;

import de.bsvrz.buv.plugin.pua.basis.ProtokollInfo;
import de.bsvrz.dav.daf.main.archive.ArchiveDataKind;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Klasse zum Erzeugen einer XML-Struktur entsprechend der tabelle.dtd
 * 
 * @author Dambach Werke GmbH
 */

public class XMLPrinter
{
  private static final String XmlEncoding = "ISO-8859-1";

  /**
   * Wurzelelement (JDOM) mit der StartStopp XML Struktur
   */
  private Element m_root = null;

  /**
   * Konstruktor der Klasse
   */
  public XMLPrinter()
  {
    m_root = erzeugeTabelle();
  }

  /**
   * Methode zum Erzeugen des XML-Elements <Tabelle> der Tabelle
   * 
   * @return erzeugtes Element
   */
  public Element erzeugeTabelle()
  {
    Element em = new Element(XmlDefs.ELE_TABELLE);

    return em;
  }

  /**
   * Methode fgt das Element Info hinzu
   * 
   * @param info
   *          Protokollinformation
   */
  public void addInfo(ProtokollInfo info, Collection<SystemObject> objekte)
  {
    Element em = new Element(XmlDefs.ELE_INFO);
    
    em.addContent(addInfoSkriptName(info));
    em.addContent(addProtokollId(info));
    em.addContent(addInformationstext(info));
    em.addContent(addInfoObjekte(objekte));
    em.addContent(addInfoZeitbereiche(info));
    em.addContent(addInfoArchivdatenart(info));
    em.addContent(addInfoSimulationsvariante(info));

    m_root.addContent(em);
  }

  /**
   * Methode fgt das Element Protokoll Id dem Infoanteil hinzu
   * 
   * @param info
   *          Protokollinformation
   */
  private Element addProtokollId(ProtokollInfo info)
  {
    Element em = new Element(XmlDefs.ELE_PROTOKOLLID);

    Attribute wert = new Attribute(XmlDefs.ATT_WERT, "" + info.getProtokollId());

    em.setAttribute(wert);

    return em;
  }

  /**
   * Methode fgt das Element Informationstext dem Infoanteil hinzu
   * 
   * @param info
   *          Protokollinformation
   */
  private Element addInformationstext(ProtokollInfo info)
  {
    Element em = new Element(XmlDefs.ELE_INFORMATIONSTEXT);

    Attribute wert = new Attribute(XmlDefs.ATT_WERT, "" + info.getInformationsTextBereinigt());

    em.setAttribute(wert);

    return em;
  }
  
  /**
   * Methode fgt das Element Skriptname dem Infoanteil hinzu
   * 
   * @param info
   *          Protokollinformation
   */
  private Element addInfoSkriptName(ProtokollInfo info)
  {
    Element em = new Element(XmlDefs.ELE_SKRIPTNAME);

    Attribute wert = new Attribute(XmlDefs.ATT_NAME, "" + info.getScriptName());

    em.setAttribute(wert);

    return em;
  }

  /**
   * Methode fgt das Element Objekte dem Infoanteil hinzu
   * 
   * @param info
   *          Protokollinformation
   */
  private Element addInfoObjekte(ProtokollInfo info)
  {
    Element em = new Element(XmlDefs.ELE_OBJEKTE);

    Vector<SystemObject> objekte = info.getObjekte();
    for (SystemObject o : objekte)
    {
      Element em1 = new Element(XmlDefs.ELE_OBJEKT);

      Attribute name = new Attribute(XmlDefs.ATT_NAME, o.getName());
      Attribute pid = new Attribute(XmlDefs.ATT_PID, o.getPid());

      em1.setAttribute(name);
      em1.setAttribute(pid);

      em.addContent(em1);
    }

    return em;
  }
  
  private Element addInfoObjekte(Collection<SystemObject> objekte)
  {
    Element em = new Element(XmlDefs.ELE_OBJEKTE);

    for (SystemObject o : objekte)
    {
      Element em1 = new Element(XmlDefs.ELE_OBJEKT);

      Attribute name = new Attribute(XmlDefs.ATT_NAME, o.getName());
      Attribute pid = new Attribute(XmlDefs.ATT_PID, o.getPid());

      em1.setAttribute(name);
      em1.setAttribute(pid);

      em.addContent(em1);
    }

    return em;
  }

  /**
   * Methode fgt das Element Zeitbereiche dem Infoanteil hinzu
   * 
   * @param info
   *          Protokollinformation
   */
  private Element addInfoZeitbereiche(ProtokollInfo info)
  {
    Element em = new Element(XmlDefs.ELE_ZEITBEREICHE);

    Vector<String> zeitbereiche = info.getZeitbereiche();
    for (String zb : zeitbereiche)
    {
      Element em1 = new Element(XmlDefs.ELE_ZEITBEREICH);

      String[] s = new String[2];
      s = zb.split("-");

      Attribute von = new Attribute(XmlDefs.ATT_VON, s[0].trim());
      Attribute bis = new Attribute(XmlDefs.ATT_BIS, s[1].trim());

      em1.setAttribute(von);
      em1.setAttribute(bis);

      em.addContent(em1);
    }

    return em;
  }

  /**
   * Methode fgt das Element Archivdatenart dem Infoanteil hinzu
   * 
   * @param info
   *          Protokollinformation
   */
  private Element addInfoArchivdatenart(ProtokollInfo info)
  {
    Element em = new Element(XmlDefs.ELE_ARCHIVDATENARTEN);

    ArchiveDataKind[] arten = info.getArchivDatenArt();
    for (ArchiveDataKind adk : arten)
    {
      Element em1 = new Element(XmlDefs.ELE_ARCHIVDATENART);

      Attribute art = new Attribute(XmlDefs.ATT_ART, adk.toString());

      em1.setAttribute(art);

      em.addContent(em1);
    }

    return em;
  }

  /**
   * Methode fgt das Element Simulationsvariante dem Infoanteil hinzu
   * 
   * @param info
   *          Protokollinformation
   */
  private Element addInfoSimulationsvariante(ProtokollInfo info)
  {
    Element em = new Element(XmlDefs.ELE_SIMULATIONSVARIANTE);

    Attribute wert = new Attribute(XmlDefs.ATT_WERT, "" + info.getSimulationsVariante());

    em.setAttribute(wert);

    return em;
  }

  /**
   * Methode zum Erzeugen des XML-Elements <Kopf> der Tabelle
   * 
   * @return erzeugtes Element
   */
  public Element addKopf(String daten[])
  {
    Element em = new Element(XmlDefs.ELE_KOPF);

    for (String s : daten)
    {
      Element em1 = new Element(XmlDefs.ELE_SPALTENTITEL);

      Attribute a1 = new Attribute(XmlDefs.ATT_BESCHRIFTUNG, s);
      em1.setAttribute(a1);

      em.addContent(em1);
    }

    m_root.addContent(em);
    return em;
  }

  /**
   * Methode zum Erzeugen des XML-Elements <Zeile> der Tabelle
   * 
   * @return erzeugtes Element
   */
  public Element addZeile(String daten[])
  {
    Element em = new Element(XmlDefs.ELE_ZEILE);

    for (String s : daten)
    {
      Element em1 = new Element(XmlDefs.ELE_SPALTE);

      Attribute a1 = new Attribute(XmlDefs.ATT_WERT, s);
      em1.setAttribute(a1);

      em.addContent(em1);
    }

    m_root.addContent(em);
    return em;
  }

  /**
   * @return liefert die Klassenvariable m_root zurck
   */
  public Element getRoot()
  {
    return m_root;
  }

  /**
   * Liefert eine formatierte Ausgabe der erzeugten XML-Struktur
   * 
   * @param rootElement
   *          Wurzelelement des Dokumentes, wird detached.
   * @param ignoreWhitespace
   *          Wenn <code>true</code>, dann werden WhiteSpaces nicht beachtet.
   * @param xmlDocTypeElementName
   *          Name des Wurzelelementes
   * @param xmlDocTypePublicId
   *          Public ID
   * @param xmlDocSystemId
   *          DTD
   * @return Formatierte XML-Struktur
   */
  public static String ausgabeXmlFormatiert(Element rootElement, boolean ignoreWhitespace,
      String xmlDocTypeElementName, String xmlDocTypePublicId, String xmlDocSystemId)
  {
    String returnString = null;
    rootElement.detach();

    Document doc = new Document(rootElement);

    if ((xmlDocTypeElementName != null) && (xmlDocTypePublicId != null) && (xmlDocSystemId != null))
    {
      DocType doctype = new DocType(xmlDocTypeElementName, xmlDocTypePublicId, xmlDocSystemId);
      doc.setDocType(doctype);

    }

    XMLOutputter out = new XMLOutputter();

    // XMLOutputter out = new XMLOutputter("\t", true);
    // out.setEncoding(XmlEncoding);

    returnString = out.outputString(doc);

    return returnString;
  }

  /**
   * Schreibt einen String in eine Datei
   * 
   * @param file
   *          Kompletter Pfad zur Datei, in die geschrieben werden soll
   * @param schreibString
   *          String, der in die Datei geschrieben werden soll
   * @param ueberschreiben
   *          true == berschreiben der Datei, falls schon vorhanden
   * @return Liefert <code>true</code>, wenn die Datei erfolgreich geschrieben werden konnte, sonst <code>false</code>.
   * @throws IllegalArgumentException
   *           Wir geworfen, wenn die Datei existiert und nicht berschrieben werden kann
   */
  public static boolean schreibeString(File file, String schreibString, boolean ueberschreiben)
      throws IllegalArgumentException
  {
    boolean erfolgreich = false;

    // File existiert, auf berschreiben prfen
    if (file.exists() && ueberschreiben)
    {
      try
      {
        PrintWriter pw = new PrintWriter(file);
        pw.print(schreibString);
        pw.close();
      }
      catch (FileNotFoundException e)
      {
        e.printStackTrace();
      }
    }
    // File existiert nicht und kann einfach angelegt werden
    else if (!file.exists())
    {
      try
      {
        PrintWriter pw = new PrintWriter(file);
        pw.print(schreibString);
        pw.close();
        erfolgreich = true;
      }
      catch (FileNotFoundException e)
      {
        e.printStackTrace();
      }
    }
    // File existiert und ueberschreiben == flase
    else
    {
      throw new IllegalArgumentException("Die Datei " + file.getPath() + " existiert und ueberschreiben == flase");
    }

    return erfolgreich;
  }

  /**
   * Methode schreibt die formatierte XML-Struktur in eine Datei
   * 
   * @param file
   *          Datei in die geschrieben werden soll
   * @return war Schreiben erfolgreich
   */
  public boolean schreibeDateiFormatiert(File file)
  {
    String s = ausgabeXmlFormatiert(getRoot(), true, XmlDefs.xmlDocTypeElementName, XmlDefs.xmlDocTypePublicId,
        XmlDefs.xmlDocSystemId);

    return schreibeString(file, s, true);
  }

  @Override
  public String toString()
  {
    String s = ausgabeXmlFormatiert(getRoot(), true, XmlDefs.xmlDocTypeElementName, XmlDefs.xmlDocTypePublicId,
        XmlDefs.xmlDocSystemId);

    return s;
  }
}
