/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.pua.basis.views;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.SortedMap;
import java.util.StringTokenizer;
import java.util.TreeMap;
import java.util.Vector;
import java.util.logging.Filter;

import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.plugin.pua.basis.Activator;
import de.bsvrz.buv.plugin.pua.basis.expose.IPIGanglinienKonvertierung;
import de.bsvrz.buv.plugin.pua.basis.expose.IProjektInfo;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.AttributeType;
import de.bsvrz.dav.daf.main.config.DoubleAttributeType;
import de.bsvrz.dav.daf.main.config.IntegerAttributeType;
import de.bsvrz.dav.daf.main.config.IntegerValueRange;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.pua.prot.client.dataobject.Column;
import de.bsvrz.pua.prot.client.dataobject.ProtocolAdministrationData;
import de.bsvrz.pua.prot.client.dataobject.Column.ColumnType;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.losb.util.Tuple;

/**
 * In diesem Dialog werden die Zur Auswertung relevanten Attribute definiert.
 * 
 * <br>
 * <br>
 * Die Klasse erfllt die technische Anforderungen des Segments Bedienung und Visualisierung (BUV): <br>
 * <ul>
 * <li>TAnf-Vi-BW-C3-9</li>
 * </ul>
 * 
 * @version $Revision: 1.24 $ / $Date: 2014/02/18 15:31:14 $ / ($Author: Sans $)
 * @author HofmannJ
 * 
 */
public class GanglinieAuswahlDialog extends Dialog
{
  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();

  /**
   * Filterverwaltungsdialog
   */
  private Shell _shell = null; // @jve:decl-index=0:visual-constraint="144,59"

  /**
   * Composites
   */
  private Composite compositeTop = null;

  private Composite compositeBottom = null;

  private Composite composite = null;

  private Composite composite1 = null;

  private Composite composite2 = null;

  private Composite composite3 = null;

  private Composite composite4 = null;

  private Composite composite5 = null;

  private Composite composite6 = null;

  private Composite composite7 = null;

  private Composite composite8 = null;

  private Composite composite9 = null;

  /**
   * Buttons
   */
  private Button buttonOk = null;

  private Button buttonSchliessen = null;

  private Button buttonHilfe = null;

  private Button buttonAdd = null;

  private Button buttonRemove = null;

  /**
   * Groups
   */
  private Group group = null;

  private Group group1 = null;

  private Group group2 = null;

  private Group group3 = null;

  /**
   * TableViewers, Tables
   */
  private TableViewer _tvBetriebsMeldungen1;

  private TableViewer _tvBetriebsMeldungen2;

  private TableViewer _tvBetriebsMeldungen3;

  private TableViewer _tvBetriebsMeldungen4;
  
  private IProjektInfo _projektInfo;

  private Table _table;

  private java.util.List<String> listObjekte;

  private java.util.List<String> listSpalten;

  private String _zusatzInfo;
  
  private Tuple<Long, Long> _gesamtIntervall;

  /**
   * Konstruktor fr neue Instanzen der Klasse {@link GanglinieAuswahlDialog}.
   * 
   * @param parent
   *          Eltern-Shell, die fr die Darstellung benutzt werden soll.
   */
  public GanglinieAuswahlDialog(Shell parent)
  {
    super(parent);
  }

  /**
   * ffnet den Dialog {@link GanglinieAuswahlDialog}.
   * 
   * @param filterSpeicher
   *          Instanz der Klasse {@link FilterSpeicher}, die die Liste aller {@link Filter} enthlt.
   * 
   * @return Liefert <code>true</code>, wenn sich etwas im {@link FilterSpeicher} gendert hat, sonst <code>false</code>
   *         ;
   */
  public boolean oeffne()
  {

    // Neue Shell anlegen, Wait & Dispose, evtl. Rckgabewerte.
    Shell parent = getParent();
    // _shell = new Shell(parent, SWT.TITLE | SWT.APPLICATION_MODAL | SWT.MIN | SWT.MAX | SWT.CLOSE);
    _shell = new Shell();
    _shell.setText("Ganglinie Auswahl");
    _shell.setSize(new Point(400, 520));
    _shell.setLayout(new GridLayout());

    // Fr die F1 Taste
    PlatformUI.getWorkbench().getHelpSystem().setHelp(_shell, Activator.PLUGIN_ID + ".auswahlGanglinie");

    createCompositeTop();
    createCompositeBot();

    _shell.open();

    Display display = parent.getDisplay();

    while (!_shell.isDisposed())
      if (!display.readAndDispatch())
        display.sleep();

    return true;
  }

  /**
   * ffnet den Dialog {@link GanglinieAuswahlDialog}.
   * 
   * @param filterSpeicher
   *          Instanz der Klasse {@link FilterSpeicher}, die die Liste aller {@link Filter} enthlt.
   * 
   * @return Liefert <code>true</code>, wenn sich etwas im {@link FilterSpeicher} gendert hat, sonst <code>false</code>
   *         ;
   */
  @Deprecated
  public boolean oeffne(/* ClientDavInterface connection, */ProtocolAdministrationData pad, Table table, 
		  Collection<SystemObject> objekte, Collection<String> spalten, String zusatzInfo)
  {
    // Neue Shell anlegen, Wait & Dispose, evtl. Rckgabewerte.
    Shell parent = getParent();
    _shell = new Shell(parent, SWT.TITLE | SWT.APPLICATION_MODAL | SWT.MIN | SWT.MAX | SWT.CLOSE);
    // _shell = new Shell();
    _shell.setText("Ganglinie Auswahl");
    _shell.setSize(new Point(450, 550));
    _shell.setLayout(new GridLayout());

    // Fr die F1 Taste
    PlatformUI.getWorkbench().getHelpSystem().setHelp(_shell, Activator.PLUGIN_ID + ".auswahlGanglinie");

    createCompositeTop();
    createCompositeBot();

    _shell.open();

    _table = table;
    _zusatzInfo = zusatzInfo;

    fuelleMitDaten(objekte, spalten);

    if (_tvBetriebsMeldungen1.getTable().getItemCount() == 1)
    {
      _tvBetriebsMeldungen1.getTable().setSelection(0);
    }

    updateOkButton();

    Display display = parent.getDisplay();

    while (!_shell.isDisposed())
      if (!display.readAndDispatch())
        display.sleep();

    return true;
  }

  /**
   * ffnet den Dialog {@link GanglinieAuswahlDialog}.
   * 
   * @param filterSpeicher
   *          Instanz der Klasse {@link FilterSpeicher}, die die Liste aller {@link Filter} enthlt.
   * 
   * @return Liefert <code>true</code>, wenn sich etwas im {@link FilterSpeicher} gendert hat, sonst <code>false</code>
   *         ;
   */
  public boolean oeffne(IProjektInfo projektInfo, Table table, Collection<SystemObject> objekte, String zusatzInfo, Tuple<Long, Long> gesamtIntervall)
  {
    // Neue Shell anlegen, Wait & Dispose, evtl. Rckgabewerte.
    Shell parent = getParent();
    _shell = new Shell(parent, SWT.TITLE | SWT.APPLICATION_MODAL | SWT.MIN | SWT.MAX | SWT.CLOSE);
    // _shell = new Shell();
    _shell.setText("Ganglinie Auswahl");
    _shell.setSize(new Point(450, 550));
    _shell.setLayout(new GridLayout());

    // Fr die F1 Taste
    PlatformUI.getWorkbench().getHelpSystem().setHelp(_shell, Activator.PLUGIN_ID + ".auswahlGanglinie");

    createCompositeTop();
    createCompositeBot();

    _shell.open();

    _projektInfo = projektInfo;
    _table = table;
    _zusatzInfo = zusatzInfo;
    _gesamtIntervall = gesamtIntervall;

    fuelleMitDaten(objekte, projektInfo.getAusgabeTabelleSpalten());

    if (_tvBetriebsMeldungen1.getTable().getItemCount() == 1)
    {
      _tvBetriebsMeldungen1.getTable().setSelection(0);
    }

    updateOkButton();

    Display display = parent.getDisplay();

    while (!_shell.isDisposed())
      if (!display.readAndDispatch())
        display.sleep();

    return true;
  }
  
  
  private void bestimmeOberTypen(SystemObjectType typ, List<SystemObjectType> list)
  {
    list.add(typ);
    List<SystemObjectType> superTypes = typ.getSuperTypes();

    for (SystemObjectType superTyp : superTypes)
    {
      bestimmeOberTypen(superTyp, list);
    }

  }

  private String getEinheit(Data dataZeiger)
  {
    String ret = "";

    AttributeType attributTyp = dataZeiger.getAttributeType();

    if (dataZeiger.getName().equals("Fahrzeuganzahl"))
    {
      attributTyp = dataZeiger.getAttributeType();
    }

    if (attributTyp instanceof IntegerAttributeType)
    {
      IntegerValueRange integerValueRange = ((IntegerAttributeType)attributTyp).getRange();

      if (integerValueRange != null)
      {
        ret = integerValueRange.getUnit();
      }
    }
    else if (attributTyp instanceof DoubleAttributeType)
    {
      ret = ((DoubleAttributeType)attributTyp).getUnit();
    }

    return (ret);
  }

  @SuppressWarnings("unchecked")
  private void doAttributhierarchieAufloesen(Data dataZeiger, String kopfzeileBisher, List<String> kopfzeilen)
  {
    if (dataZeiger.isPlain())
    {
      String einheit = getEinheit(dataZeiger);
      String caption = kopfzeileBisher;
      if (einheit.length() > 0)
      {
        caption += " [" + einheit + "]";
      }

      if (!kopfzeilen.contains(caption))
      {
        kopfzeilen.add(caption);
      }
    }
    else
    {
      if (dataZeiger.isList())
      {
        Iterator<Data> it = dataZeiger.iterator();
        while (it.hasNext())
        {
          Data data = it.next();
          doAttributhierarchieAufloesen(data, kopfzeileBisher + "." + data.getName(), kopfzeilen);
        }
      }
      else
      {
        throw new RuntimeException("nicht behandelte Auspraegung von Administrationsdaten");
      }
    }
  }

  /**
   * Fllt den Dialog mit Daten
   * 
   * @param connection
   * @param adminData
   */
  private void fuelleMitDaten(Collection<SystemObject> objekte, Collection<String> spalten)
	{
		listObjekte = new ArrayList<String>();
	    for (SystemObject so : objekte)
	    {
	    	listObjekte.add(so.getNameOrPidOrId());
		}
		Collections.sort(listObjekte);
		_tvBetriebsMeldungen1.setInput(listObjekte);
		
		listSpalten = new ArrayList<String>();
		listSpalten.addAll(spalten);
		Collections.sort(listSpalten);
		_tvBetriebsMeldungen2.setInput(listSpalten);
	}

  @Deprecated
  private void fuelleMitDaten(ClientDavInterface connection, ProtocolAdministrationData adminData,
      Collection<SystemObject> objekte)
  {
    listObjekte = new ArrayList<String>();

    // for (int i = 0; i < adminData.dids.length; i++)
    // {
    // for (int j = 0; j < adminData.dids[i].length; j++)
    // {
    // long id = adminData.dids[i][j];
    //
    // SystemObject so = connection.getDataModel().getObject(id);
    // SystemObjectType sot = so.getType();
    // ArrayList<SystemObjectType> typen = new ArrayList<SystemObjectType>();
    // bestimmeOberTypen(sot, typen);
    // for (SystemObjectType st : typen)
    // {
    // if (st.getPid().equals("typ.de") || st.getPid().equals("typ.messQuerschnittAllgemein")
    // || st.getPid().equals("typ.anzeigeQuerschnitt") || st.getPid().equals("typ.umfeldDatenSensor")
    // || st.getPid().equals("umfeldDatenMessStelle"))
    // sot = st;
    // }
    //        
    // if (sot.getPid().equals("typ.informationsKanal") || sot.getPid().equals("typ.messQuerschnittAllgemein")
    // || sot.getPid().equals("typ.de") || sot.getPid().equals("typ.fahrStreifen")
    // || sot.getPid().equals("typ.anzeigeQuerschnitt") || sot.getPid().equals("typ.umfeldDatenSensor")
    // || sot.getPid().equals("umfeldDatenMessStelle"))
    // {
    // listObjekte.add(so.getPid());
    // }
    //
    // }
    // }

    // if (listObjekte != null)
    // {
    //
    // Collections.sort(listObjekte);
    // _tvBetriebsMeldungen1.setInput(listObjekte);
    // }

    listSpalten = new ArrayList<String>();

    // int anzahlObjekte = listObjekte.size();

    // if (anzahlObjekte > 0)
    // {
    for (int i = 0; i < adminData.columns.length; /* i += anzahlObjekte */i += 1)
    {
      String caption = adminData.columns[i].getCaption();
      Column col = adminData.columns[i];
      int index = adminData.columnToDataId[i];

      long idObj = adminData.dids[index][0];
      SystemObject so0 = connection.getDataModel().getObject(idObj);
      String pid = so0.getPid();
      if (!listObjekte.contains(pid))
      {
        listObjekte.add(pid);
      }

      long idAtg = adminData.dids[index][1];
      SystemObject so1 = connection.getDataModel().getObject(idAtg);
      String attr = col.getAttributeName();
      AttributeGroup atg = (AttributeGroup)so1;
      Data data = connection.createData(atg);

      switch (col.getType())
      {
        case REAL_ATTRIBUTE:
        case ATTRIBUTE_GROUP:
          // case TEMP_ATTRIBUTE:
          break;
        default:
          throw new RuntimeException("unbehandelter Attributtyp in der Spalte");
      }

      Data dataZeiger = data;
      if (col.getType() == ColumnType.REAL_ATTRIBUTE /* || col.getType() == ColumnType.TEMP_ATTRIBUTE */)
      {
        StringTokenizer zerleger = new StringTokenizer(attr, ".");
        while (zerleger.hasMoreTokens())
        {
          String attributEbene = zerleger.nextToken(); // .split("\\[")[0]; // bei Feldern, den Index vernachlssigen
          if (attributEbene.contains("["))
          {
            attributEbene = attributEbene.split("\\[")[0];
          }
          dataZeiger = dataZeiger.getItem(attributEbene);
          if (zerleger.hasMoreTokens() && dataZeiger.isArray())
          {
            // Falls dataZeiger auf ein Array zeigt und dieses Array noch
            // durch ein weiteres Attribut qualifiziert ist:
            // erzeuge 1 Array-Element zur weitern Auswertung
            dataZeiger.asArray().setLength(1);
            dataZeiger = dataZeiger.asArray().getItem(0);
          }
        }
      }
      doAttributhierarchieAufloesen(dataZeiger, caption, listSpalten);
    }
    // }
    // else
    // {
    // System.err.println("Anzahl Objekte ist null!!!");
    // }

    // if (listObjekte != null)
    // {
    //
    // Collections.sort(listObjekte);
    // _tvBetriebsMeldungen1.setInput(listObjekte);
    // }

    List<String> namenObjekte = new ArrayList<String>();
    for (SystemObject so : objekte)
    {
      namenObjekte.add(so.getNameOrPidOrId());
    }
    Collections.sort(namenObjekte);
    _tvBetriebsMeldungen1.setInput(namenObjekte);

    if (listSpalten != null)
    {

      Collections.sort(listSpalten);

      _tvBetriebsMeldungen2.setInput(listSpalten);
    }

  }

  /**
   * Schliet den Verwaltungsdialog {@link GanglinieAuswahlDialog}.
   * 
   */
  private void aktionSchliessen()
  {
    _shell.dispose();
  }

  /**
   * Zeigt die Hilfe zum Dialog an.
   * 
   */
  private void aktionHilfe()
  {
    String contextId = Activator.PLUGIN_ID + ".auswahlGanglinie";
    _debug.fine("ContextID: " + contextId);
    PlatformUI.getWorkbench().getHelpSystem().displayHelp(contextId);
  }

  /**
   * This method initializes composite
   * 
   */
  private void createCompositeTop()
  {
    GridLayout gridLayout = new GridLayout();
    gridLayout.numColumns = 1;
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.FILL;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.grabExcessVerticalSpace = true;
    gridData3.verticalAlignment = GridData.FILL;
    compositeTop = new Composite(_shell, SWT.NONE);
    compositeTop.setLayoutData(gridData3);
    compositeTop.setLayout(gridLayout);
    createComposite();
  }

  /**
   * This method initializes compositeBot
   * 
   */
  private void createCompositeBot()
  {
    RowLayout rowLayout = new RowLayout();
    rowLayout.pack = false;
    rowLayout.fill = false;
    rowLayout.wrap = false;
    GridData gridData4 = new GridData();
    gridData4.horizontalAlignment = GridData.END;
    gridData4.verticalAlignment = GridData.CENTER;
    compositeBottom = new Composite(_shell, SWT.NONE);
    compositeBottom.setLayoutData(gridData4);
    compositeBottom.setLayout(rowLayout);
    buttonOk = new Button(compositeBottom, SWT.NONE);
    buttonOk.setText("OK");
    buttonOk.addMouseListener(new org.eclipse.swt.events.MouseAdapter() {
      public void mouseDown(org.eclipse.swt.events.MouseEvent e)
      {
        aktionGanglinie();
      }
    });

    buttonSchliessen = new Button(compositeBottom, SWT.NONE);
    buttonSchliessen.setText("Abbrechen");
    buttonSchliessen.addMouseListener(new org.eclipse.swt.events.MouseAdapter() {
      public void mouseDown(org.eclipse.swt.events.MouseEvent e)
      {
        aktionSchliessen();
      }
    });

    buttonHilfe = new Button(compositeBottom, SWT.NONE);
    buttonHilfe.setText("Hilfe");
    buttonHilfe.addMouseListener(new org.eclipse.swt.events.MouseAdapter() {
      public void mouseDown(org.eclipse.swt.events.MouseEvent e)
      {
        aktionHilfe();
      }
    });
  }

  /**
   * This method initializes composite
   * 
   */
  private void createComposite()
  {
    GridLayout gridLayout1 = new GridLayout();
    gridLayout1.numColumns = 1;
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.verticalAlignment = GridData.FILL;
    composite = new Composite(compositeTop, SWT.NONE);
    composite.setLayout(gridLayout1);
    createComposite1();
    composite.setLayoutData(gridData1);
    createComposite3();
  }

  /**
   * This method initializes composite1
   * 
   */
  private void createComposite1()
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.CENTER;
    composite1 = new Composite(composite, SWT.NONE);
    composite1.setLayout(new GridLayout());
    createGroup2();
    composite1.setLayoutData(gridData);
  }

  /**
   * This method initializes composite3
   * 
   */
  private void createComposite3()
  {
    GridLayout gridLayout2 = new GridLayout();
    gridLayout2.numColumns = 3;
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.verticalAlignment = GridData.FILL;
    composite3 = new Composite(composite, SWT.NONE);
    composite3.setLayout(gridLayout2);
    createComposite2();
    composite3.setLayoutData(gridData2);
    createComposite4();
    createComposite5();
  }

  /**
   * This method initializes group2
   * 
   */
  private void createGroup2()
  {
    GridData gridData8 = new GridData();
    gridData8.horizontalAlignment = GridData.FILL;
    gridData8.grabExcessHorizontalSpace = true;
    gridData8.grabExcessVerticalSpace = true;
    gridData8.verticalAlignment = GridData.FILL;
    group2 = new Group(composite1, SWT.NONE);
    group2.setLayout(new GridLayout());
    group2.setText("Objektauswahl");
    createComposite6();
    group2.setLayoutData(gridData8);
  }

  /**
   * This method initializes composite6
   * 
   */
  private void createComposite6()
  {
    GridData gridData13 = new GridData();
    gridData13.horizontalAlignment = GridData.FILL;
    gridData13.grabExcessHorizontalSpace = true;
    gridData13.grabExcessVerticalSpace = true;
    gridData13.verticalAlignment = GridData.FILL;
    composite6 = new Composite(group2, SWT.NONE);
    composite6.setLayout(new FillLayout());
    composite6.setLayoutData(gridData13);
    erzeugeTabelle1();
  }

  /**
   * This method initializes composite7
   * 
   */
  private void createComposite7()
  {
    GridData gridData6 = new GridData();
    gridData6.horizontalAlignment = GridData.FILL;
    gridData6.grabExcessHorizontalSpace = true;
    gridData6.grabExcessVerticalSpace = true;
    gridData6.verticalAlignment = GridData.FILL;
    composite7 = new Composite(group, SWT.NONE);
    composite7.setLayout(new FillLayout());
    composite7.setLayoutData(gridData6);
    erzeugeTabelle2();
  }

  /**
   * This method initializes composite8
   * 
   */
  private void createComposite8()
  {
    GridData gridData14 = new GridData();
    gridData14.horizontalAlignment = GridData.FILL;
    gridData14.grabExcessHorizontalSpace = true;
    gridData14.grabExcessVerticalSpace = true;
    gridData14.verticalAlignment = GridData.FILL;
    composite8 = new Composite(group3, SWT.NONE);
    composite8.setLayout(new FillLayout());
    composite8.setLayoutData(gridData14);
    erzeugeTabelle3();
  }

  /**
   * This method initializes composite9
   * 
   */
  private void createComposite9()
  {
    GridData gridData15 = new GridData();
    gridData15.horizontalAlignment = GridData.FILL;
    gridData15.grabExcessHorizontalSpace = true;
    gridData15.grabExcessVerticalSpace = true;
    gridData15.verticalAlignment = GridData.FILL;
    composite9 = new Composite(group1, SWT.NONE);
    composite9.setLayout(new FillLayout());
    composite9.setLayoutData(gridData15);
    erzeugeTabelle4();
  }

  private void erzeugeTabelle1()
  {
    _tvBetriebsMeldungen1 = new TableViewer(composite6, SWT.BORDER | SWT.FULL_SELECTION | SWT.MULTI | SWT.H_SCROLL
        | SWT.V_SCROLL);
    // _tvBetriebsMeldungen1.setData("name", "viewer1");

    final TableColumn tc0 = new TableColumn(_tvBetriebsMeldungen1.getTable(), SWT.LEFT);
    // tc0.setText("Pid");
    tc0.setWidth(300);

    _tvBetriebsMeldungen1.setContentProvider(new TabelleInhaltsVersorger());
    _tvBetriebsMeldungen1.setLabelProvider(new TabelleDarstellungsVersorger());

    _tvBetriebsMeldungen1.addSelectionChangedListener(new ISelectionChangedListener() {
      public void selectionChanged(SelectionChangedEvent event)
      {
        IStructuredSelection selection = (IStructuredSelection)event.getSelection();
        Object firstElement = selection.getFirstElement();

        if (firstElement instanceof SystemObject)
        {
        }

        updateOkButton();
      }
    });

  }

  private void erzeugeTabelle2()
  {
    _tvBetriebsMeldungen2 = new TableViewer(composite7, SWT.BORDER | SWT.FULL_SELECTION | SWT.MULTI | SWT.H_SCROLL
        | SWT.V_SCROLL);
    // _tvBetriebsMeldungen2.setData("name", "viewer1");

    final TableColumn tc0 = new TableColumn(_tvBetriebsMeldungen2.getTable(), SWT.LEFT);
    tc0.setWidth(300);

    _tvBetriebsMeldungen2.setContentProvider(new TabelleInhaltsVersorger());
    _tvBetriebsMeldungen2.setLabelProvider(new TabelleDarstellungsVersorger());

    _tvBetriebsMeldungen2.addSelectionChangedListener(new ISelectionChangedListener() {
      public void selectionChanged(SelectionChangedEvent event)
      {
        ISelection selection = event.getSelection();
        buttonAdd.setEnabled(!selection.isEmpty());
      }
    });

  }

  private void erzeugeTabelle3()
  {
    _tvBetriebsMeldungen3 = new TableViewer(composite8, SWT.BORDER | SWT.FULL_SELECTION | SWT.H_SCROLL
        | SWT.V_SCROLL);
    // _tvBetriebsMeldungen3.setData("name", "viewer1");

    final TableColumn tc0 = new TableColumn(_tvBetriebsMeldungen3.getTable(), SWT.LEFT);
    // tc0.setText("Pid");
    tc0.setWidth(300);

    _tvBetriebsMeldungen3.setContentProvider(new TabelleInhaltsVersorger());
    _tvBetriebsMeldungen3.setLabelProvider(new TabelleDarstellungsVersorger());

  

  }

  private void erzeugeTabelle4()
  {
    _tvBetriebsMeldungen4 = new TableViewer(composite9, SWT.BORDER | SWT.FULL_SELECTION | SWT.MULTI | SWT.H_SCROLL
        | SWT.V_SCROLL);

    final TableColumn tc0 = new TableColumn(_tvBetriebsMeldungen4.getTable(), SWT.LEFT);
    tc0.setWidth(300);

    _tvBetriebsMeldungen4.setContentProvider(new TabelleInhaltsVersorger());
    _tvBetriebsMeldungen4.setLabelProvider(new TabelleDarstellungsVersorger());
    
    _tvBetriebsMeldungen4.addSelectionChangedListener(new ISelectionChangedListener() {
      public void selectionChanged(SelectionChangedEvent event)
      {
        ISelection selection = event.getSelection();
        buttonRemove.setEnabled(!selection.isEmpty());
      } 
    });
  }

  /**
   * This method initializes composite2
   * 
   */
  private void createComposite2()
  {
    GridLayout gridLayout3 = new GridLayout();
    gridLayout3.numColumns = 1;
    GridData gridData5 = new GridData();
    gridData5.horizontalAlignment = GridData.FILL;
    gridData5.grabExcessHorizontalSpace = true;
    gridData5.grabExcessVerticalSpace = true;
    gridData5.verticalAlignment = GridData.FILL;
    composite2 = new Composite(composite3, SWT.NONE);
    createGroup();
    composite2.setLayout(gridLayout3);
    composite2.setLayoutData(gridData5);
    createGroup3();
  }

  /**
   * This method initializes group
   * 
   */
  private void createGroup()
  {
    GridData gridData9 = new GridData();
    gridData9.horizontalAlignment = GridData.FILL;
    gridData9.grabExcessHorizontalSpace = true;
    gridData9.grabExcessVerticalSpace = true;
    gridData9.verticalAlignment = GridData.FILL;
    group = new Group(composite2, SWT.NONE);
    group.setLayout(new GridLayout());
    group.setText("Verfgbare Spalten");
    createComposite7();
    group.setLayoutData(gridData9);
  }

  /**
   * This method initializes group3
   * 
   */
  private void createGroup3()
  {
    GridData gridData10 = new GridData();
    gridData10.horizontalAlignment = GridData.FILL;
    gridData10.grabExcessHorizontalSpace = true;
    gridData10.grabExcessVerticalSpace = true;
    gridData10.verticalAlignment = GridData.FILL;
    group3 = new Group(composite2, SWT.NONE);
    group3.setLayout(new GridLayout());
    group3.setText("Nicht mehr verfgbare Spalten");
    createComposite8();
    group3.setLayoutData(gridData10);
  }

  /**
   * This method initializes composite4
   * 
   */
  private void createComposite4()
  {
    GridLayout gridLayout4 = new GridLayout();
    gridLayout4.verticalSpacing = 5;
    gridLayout4.horizontalSpacing = 5;
    GridData gridData12 = new GridData();
    gridData12.horizontalAlignment = GridData.CENTER;
    gridData12.grabExcessHorizontalSpace = false;
    gridData12.grabExcessVerticalSpace = false;
    gridData12.heightHint = -1;
    gridData12.verticalAlignment = GridData.CENTER;
    composite4 = new Composite(composite3, SWT.NONE);
    composite4.setLayoutData(gridData12);
    composite4.setLayout(gridLayout4);
    buttonAdd = new Button(composite4, SWT.NONE);
    buttonAdd.setEnabled(false);
    buttonAdd.setText(">");
    buttonAdd.addMouseListener(new MouseAdapter() {
      public void mouseDown(MouseEvent e)
      {
        aktionZuAuswahlHinzufuegen();
      }
    });

    buttonRemove = new Button(composite4, SWT.NONE);
    buttonRemove.setEnabled(false);
    buttonRemove.setText("<");
    buttonRemove.addMouseListener(new org.eclipse.swt.events.MouseAdapter() {
      public void mouseDown(org.eclipse.swt.events.MouseEvent e)
      {
        aktionVonAuswahlEntfernen();
      }
    });

  }

  /**
   * This method initializes composite5
   * 
   */
  private void createComposite5()
  {
    GridData gridData7 = new GridData();
    gridData7.horizontalAlignment = GridData.FILL;
    gridData7.grabExcessHorizontalSpace = true;
    gridData7.grabExcessVerticalSpace = true;
    gridData7.verticalAlignment = GridData.FILL;
    composite5 = new Composite(composite3, SWT.NONE);
    composite5.setLayout(new GridLayout());
    createGroup1();
    composite5.setLayoutData(gridData7);
  }

  /**
   * This method initializes group1
   * 
   */
  private void createGroup1()
  {
    GridData gridData11 = new GridData();
    gridData11.horizontalAlignment = GridData.FILL;
    gridData11.grabExcessHorizontalSpace = true;
    gridData11.grabExcessVerticalSpace = true;
    gridData11.verticalAlignment = GridData.FILL;
    group1 = new Group(composite5, SWT.NONE);
    group1.setLayout(new GridLayout());
    group1.setText("Ausgewhlte Spalten");
    createComposite9();
    group1.setLayoutData(gridData11);
  }

  private class TabelleInhaltsVersorger implements IStructuredContentProvider
  {
    public Object[] getElements(Object inputElement)
    {
      return ((java.util.List)inputElement).toArray();
    }

    public void dispose()
    {
    }

    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
    {

    }

  }

  private class TabelleDarstellungsVersorger implements ITableLabelProvider
  {

    public Image getColumnImage(Object element, int columnIndex)
    {
      return null;
    }

    public String getColumnText(Object object, int columnIndex)
    {
      String returnString = null;

      switch (columnIndex)
      {
        case 0:
          returnString = (String)object;
          break;

        default:
          returnString = "";
      }

      return returnString;
    }

    public void addListener(ILabelProviderListener listener)
    {
    }

    public void dispose()
    {
    }

    public boolean isLabelProperty(Object element, String property)
    {
      return false;
    }

    public void removeListener(ILabelProviderListener listener)
    {
    }

  }

  /* *
   * Fgt die Selektierten Spalten zur Auswahl hinzu. / private void aktionZuAuswahlHinzufuegen() { IStructuredSelection
   * sel = (IStructuredSelection)_tvBetriebsMeldungen2.getSelection(); if (sel != null) { Object[] selectedValues =
   * sel.toArray(); for (Object selVal : selectedValues) { String s1 = (String)selVal;
   * 
   * TableItem[] items = _tvBetriebsMeldungen2.getTable().getItems(); for (TableItem ti : items) { String s2 =
   * ti.getText();
   * 
   * // System.out.println(s1 + " : " + s2);
   * 
   * if (s1.equals(s2)) { _tvBetriebsMeldungen4.add(selVal); _tvBetriebsMeldungen2.remove(selVal); break; } }
   * 
   * List<String> input = (List<String>)_tvBetriebsMeldungen4.getInput();
   * 
   * if (input != null) { Collections.sort(input); _tvBetriebsMeldungen4.setInput(input);
   * 
   * }
   * 
   * verfuegbarkeitFestlegen(); } verfuegbarkeitFestlegen(); } else System.out.println("sel ist null!"); }
   */

  /* *
   * Entfernt die selektierten Objekte von der Auswahl. / private void aktionVonAuswahlEntfernen() {
   * IStructuredSelection sel = (IStructuredSelection)_tvBetriebsMeldungen4.getSelection(); if (sel != null) { Object[]
   * selectedValues = sel.toArray();
   * 
   * for (Object selectedValue : selectedValues) { _tvBetriebsMeldungen2.add(selectedValue);
   * _tvBetriebsMeldungen4.remove(selectedValue); }
   * 
   * List<String> input = (List<String>)_tvBetriebsMeldungen2.getInput();
   * 
   * if (input != null) { Collections.sort(input); _tvBetriebsMeldungen2.setInput(input); }
   * 
   * verfuegbarkeitFestlegen(); } }
   */

  /* *
   * Legt die Verfgbarkeit von Elementen fest. Jede Y-Achse kann natrlich nur mit einer Einheit belegt sein. Sind z.B.
   * schon Elemente mit den Einheiten km/h und Fz/h ausgewhlt, knnen nur noch Elemente dieses Typs zur Auswahl
   * hinzugefgt werden. Elemente mit anderen Einheiten sind dann nicht mehr verfgbar. Diese Methode erledigt die
   * notwendigen Aktionen zur Bestimmung der verfgbaren und nicht mehr verfgbaren Elemente. / private void
   * verfuegbarkeitFestlegen() { String _einheit1 = null; String _einheit2 = null; String _einheit3 = null; String
   * _einheit4 = null;
   * 
   * TableItem[] items = _tvBetriebsMeldungen4.getTable().getItems(); for (TableItem ti : items) { String siw =
   * ti.getText();
   * 
   * int i1 = siw.indexOf("["); int i2 = siw.indexOf("]");
   * 
   * if (i1 >= 0 && i2 >= 0) { String einheit = siw.substring(i1, i2 + 1);
   * 
   * if (_einheit1 == null) { _einheit1 = einheit; } else if ((_einheit2 == null) && (!_einheit1.equals(einheit))) {
   * _einheit2 = einheit; break; } else if ((_einheit3 == null) && (!_einheit1.endsWith(einheit)) &&
   * (!_einheit2.endsWith(einheit))) { _einheit3 = einheit; break; } else if ((_einheit4 == null) &&
   * (!_einheit1.endsWith(einheit)) && (!_einheit2.endsWith(einheit)) && (!_einheit3.endsWith(einheit))) { _einheit4 =
   * einheit; break; } } }
   * 
   * if ((_einheit1 != null) && (_einheit2 != null)) // && (_einheit3 != null) && (_einheit4 != null)) {
   * LinkedList<Object> lMove = new LinkedList<Object>();
   * 
   * TableItem[] items2 = _tvBetriebsMeldungen2.getTable().getItems(); for (TableItem ti : items2) { String siw1 =
   * ti.getText();
   * 
   * String si = ""; if (si.contains("[") && si.contains("]")) si = siw1.substring(siw1.indexOf("["),
   * siw1.indexOf("]")); else si = siw1;
   * 
   * if (!(( (!si.equals(_einheit1)) && (si.equals(_einheit2)) || ((!si.equals(_einheit2)) && (si.equals(_einheit1))
   * )))) { lMove.add(siw1); }
   * 
   * }
   * 
   * if (lMove.size() > 0) { for (Object obj : lMove) { _tvBetriebsMeldungen2.remove(obj);
   * _tvBetriebsMeldungen3.add(obj); } }
   * 
   * List<String> input = (List<String>)_tvBetriebsMeldungen3.getInput();
   * 
   * if (input != null) { Collections.sort(input); _tvBetriebsMeldungen3.setInput(input); }
   * 
   * } else { LinkedList<Object> lMove = new LinkedList<Object>();
   * 
   * TableItem[] items2 = _tvBetriebsMeldungen3.getTable().getItems(); for (TableItem ti : items2) { String siw1 =
   * ti.getText(); lMove.add(siw1); }
   * 
   * if (lMove.size() > 0) { for (Object obj : lMove) { _tvBetriebsMeldungen2.add(obj);
   * _tvBetriebsMeldungen3.remove(obj); }
   * 
   * List<String> input = (List<String>)_tvBetriebsMeldungen2.getInput();
   * 
   * if (input != null) { Collections.sort(input); _tvBetriebsMeldungen2.setInput(input); }
   * 
   * } } }
   */

  private HashSet<String> getSelectedUnits()
  {
    HashSet<String> ret = new HashSet<String>();

    TableItem[] items = _tvBetriebsMeldungen4.getTable().getItems();
    for (TableItem tableItem : items)
    {
//      String item = tableItem.getText();
//      int kla = item.indexOf("[");
//      int klz = item.indexOf("]");
//      if (kla >= 0 && klz >= kla)
//      {
//        ret.add(item.substring(kla, klz + 1));
//      }
      
      // Einheitenlose Spalten werden als null - leere Einheit - hinzugefgt
      ret.add(getUnit(tableItem.getText()));
    }

    return (ret);
  }

  private String getUnit(String item)
  {
    String ret = null;

    int kla = item.indexOf("[");
    int klz = item.indexOf("]");
    if (kla >= 0 && klz >= kla)
    {
      ret = item.substring(kla, klz + 1);
    }

    return (ret);
  }

  /**
   * Fgt die Selektierten Spalten zur Auswahl hinzu.
   */
  @SuppressWarnings("unchecked")
  private void aktionZuAuswahlHinzufuegen()
  {
    IStructuredSelection selection = (IStructuredSelection)_tvBetriebsMeldungen2.getSelection();
    if (selection != null)
    {
      Object[] selectedValues = selection.toArray();
      if (selectedValues.length > 0)
      {
        // bisher ausgewaehlte einheiten bestimmen
        HashSet<String> selectedUnits = getSelectedUnits();

        boolean enlarged = false;
        for (Object selectedValue : selectedValues)
        {
          String selectedItem = (String)selectedValue;
          if (selectedUnits.contains(getUnit(selectedItem)))
          {
            // hinzuzufuegendes element hat eine bereits ausgewaehlte einheit
            _tvBetriebsMeldungen4.add(selectedValue);
            _tvBetriebsMeldungen2.remove(selectedValue);
            enlarged = true;
          }
          else if (selectedUnits.size() < GanglinieSicht.MAX_Y_UNITS_COUNT)
          {
            // noch nicht das maximum aller auswaehlbaren einheiten erreicht
            selectedUnits.add(getUnit(selectedItem));
            _tvBetriebsMeldungen4.add(selectedValue);
            _tvBetriebsMeldungen2.remove(selectedValue);
            enlarged = true;
          }
        }

        updateOkButton();

        if (enlarged)
        {
          // zur tabelle der ausgewaehlten elemente wurde etwas hinzugefuegt -> inhalt sortieren
          Vector<String> itemList = new Vector<String>();
          TableItem[] items = _tvBetriebsMeldungen4.getTable().getItems();
          for (TableItem tableItem : items)
          {
            itemList.add(tableItem.getText());
          }
          Collections.sort(itemList);
          _tvBetriebsMeldungen4.getTable().removeAll();
          for (String item : itemList)
          {
            _tvBetriebsMeldungen4.add(item);
          }

          if (selectedUnits.size() >= GanglinieSicht.MAX_Y_UNITS_COUNT)
          {
            // maximale anzahl einheiten ausgewaehlt -> evtl. nicht mehr passende elemente von
            // auwaehlbare elemente nach nicht auswaehlbare elemente verschieben
            items = _tvBetriebsMeldungen2.getTable().getItems();
            for (TableItem tableItem : items)
            {
              if (!selectedUnits.contains(getUnit(tableItem.getText())))
              {
                _tvBetriebsMeldungen3.add(tableItem.getText());
                _tvBetriebsMeldungen2.remove(tableItem.getText());
              }
            }
          }
        }
      }
    }
  }

  /**
   * Entfernt die selektierten Objekte von der Auswahl.
   */
  private void aktionVonAuswahlEntfernen()
  {
    IStructuredSelection selection = (IStructuredSelection)_tvBetriebsMeldungen4.getSelection();
    if (selection != null)
    {
      Object[] selectedValues = selection.toArray();

      boolean enlarged = false;
      for (Object selectedValue : selectedValues)
      {
        _tvBetriebsMeldungen2.add(selectedValue);
        _tvBetriebsMeldungen4.remove(selectedValue);
        enlarged = true;
      }

      updateOkButton();

      if (enlarged)
      {
        // jetzt ausgewaehlte einheiten bestimmen
        HashSet<String> selectedUnits = getSelectedUnits();

        if (selectedUnits.size() < GanglinieSicht.MAX_Y_UNITS_COUNT)
        {
          // alle elements aus tabelle nicht auswaehlbare elemente nach auswaehlbare elemente hinzufuegen
          TableItem[] items = _tvBetriebsMeldungen3.getTable().getItems();
          for (TableItem tableItem : items)
          {
            _tvBetriebsMeldungen2.add(tableItem.getText());
            _tvBetriebsMeldungen3.remove(tableItem.getText());
          }
        }

        // zur tabelle der auswaehlbaren elemente wurde etwas hinzugefuegt -> inhalt sortieren
        Vector<String> itemList = new Vector<String>();
        TableItem[] items = _tvBetriebsMeldungen2.getTable().getItems();
        for (TableItem tableItem : items)
        {
          itemList.add(tableItem.getText());
        }
        Collections.sort(itemList);
        _tvBetriebsMeldungen2.getTable().removeAll();
        for (String item : itemList)
        {
          _tvBetriebsMeldungen2.add(item);
        }
      }
    }
  }

  /**
   * Erzeugt die Ganglinie durch Aufruf der Sicht
   */
  protected void aktionGanglinie()
  {
    List<String> namenSelektierteObjekte = null;
    if (_tvBetriebsMeldungen1 != null)
    {
      IStructuredSelection selection = (IStructuredSelection)_tvBetriebsMeldungen1.getSelection();
      int selektierteObjekte = selection.size();
      int selektierteSpalten = _tvBetriebsMeldungen4.getTable().getItemCount();
      if (selektierteObjekte * selektierteSpalten > GanglinieSicht.MAX_GANGLINIEN)
      {
        MessageBox mb = new MessageBox(PlatformUI.getWorkbench().getDisplay().getActiveShell(), SWT.ICON_WARNING
            | SWT.OK);
        mb.setText("Zuviele Objekte und Spalten selektiert");
        mb.setMessage("Es knnen Objekte und Spalten fr maximal " + GanglinieSicht.MAX_GANGLINIEN + " Ganglinien selektiert werden."
            + "\nAktuell sind " + selektierteObjekte + " Objekte x " + selektierteSpalten + " Spalten fr "
            + (selektierteObjekte * selektierteSpalten) + " Ganglienien selektiert.");
        mb.open();
        return;
      }
      else
      {
        namenSelektierteObjekte = new ArrayList<String>();
        Iterator it = selection.iterator();
        while (it.hasNext())
        {
          Object obj = it.next();

          // if (obj instanceof String) {
          // namenSelektierteObjekte.add(((SystemObject)obj).getName());
          // }

          namenSelektierteObjekte.add(obj.toString());
        }
      }
    }
    if (_table != null && _tvBetriebsMeldungen4 != null && namenSelektierteObjekte != null)
    {
      IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
      IWorkbenchPage page = dw.getActivePage();
      try
      {
        String secondaryId = "" + GanglinieSicht.INSTANCE_COUNT;
               
//        System.out.println(GanglinieSicht.ID + " : " + secondaryId);
        
        page.showView(GanglinieSicht.ID, secondaryId, IWorkbenchPage.VIEW_VISIBLE);
        IViewReference ref = page.findViewReference(GanglinieSicht.ID, secondaryId);
        IViewPart vPart = ref.getView(true);
        if (vPart instanceof GanglinieSicht)
        {
          GanglinieSicht view = (GanglinieSicht)vPart;

          Set<String> selektierteObjekte = new HashSet<String>();
          selektierteObjekte.addAll(namenSelektierteObjekte);
          view.erzeugeChart(erzeugeChartInput(_table, _tvBetriebsMeldungen4.getTable().getItems(), 
              selektierteObjekte, _projektInfo, _gesamtIntervall), _zusatzInfo);
        }
        GanglinieSicht.INSTANCE_COUNT++;
      }
      catch (PartInitException e1)
      {
        // TODO Auto-generated catch block
        e1.printStackTrace();
      }
    }
    _shell.dispose();
  }

  /**
   * Hilfsmethode, erzeugt die Datenstruktur fr die Chart
   * 
   * @param table
   * @param attribut
   * @return
   */
  private Map<String, Map<String, SortedMap<Long, String>>> erzeugeChartInput(Table table, TableItem[] items,
      Collection<String> selektierteObjekte, IProjektInfo projektInfo, Tuple<Long, Long> gesamtIntervall)
  {
    Map<String, Map<String, SortedMap<Long, String>>> ret = new TreeMap<String, Map<String, SortedMap<Long, String>>>();

    // map fuer alle attribute mit spalte in der tabelle anlegen
    Integer platzhalter = new Integer(0);
    SortedMap<String, Integer> spaltenNachschlagewerk = new TreeMap<String, Integer>();
    for (int i = 0; i < items.length; i++)
    {
      spaltenNachschlagewerk.put(items[i].getText(), platzhalter);
    }

    // spalteninformationen in der attribute-map setzen
    TableColumn[] spalten = table.getColumns();
    for (int i = 0; i < spalten.length; i++)
    {
      if (spaltenNachschlagewerk.containsKey(spalten[i].getText()))
      {
        spaltenNachschlagewerk.put(spalten[i].getText(), new Integer(i));
      }
    }

    // ueber alle attribute laufen
    for (Map.Entry<String, Integer> mapEntry : spaltenNachschlagewerk.entrySet())
    {
      // attribut und spalte ermitteln
      String attribut = mapEntry.getKey();
      int spalte = mapEntry.getValue().intValue();
      Map<String, SortedMap<Long, String>> objektNachschlagewerk = new TreeMap<String, SortedMap<Long, String>>();

      // ueber alle zeilen der tabelle laufen
      int itemCount = table.getItemCount();
      for (int i = 0; i < itemCount; i++)
      {
        TableItem tableItem = table.getItem(i);
        // objekt ermittlen
        String objekt = tableItem.getText(1);

        if (!selektierteObjekte.contains(objekt))
        {
          continue;
        }

        // zeitstempel ermitteln
        Long zeitstempel = null;
        // TableItem tableItem = table.getItem(i);
//        try
//        {
//          zeitstempel = new Long(GanglinieSicht.SDF.parse(tableItem.getText(0)).getTime());
//        }
//        catch (Exception e)
//        {
//          continue;
//        }
        // Hole Zeitstempel aus Daten beim TableItem mit Key TIMESTAMP_KEY
        Object data = tableItem.getData(ProtokollAusgabeSicht.TIMESTAMP_KEY);
        if (data instanceof Long) {
          zeitstempel = (Long)data;
        }
        else {
          continue;
        }

        // // objekt ermittlen
        // String objekt = tableItem.getText(1);

        // wert ermitteln
        String wert = tableItem.getText(spalte);
        
        if (projektInfo instanceof IPIGanglinienKonvertierung && ((IPIGanglinienKonvertierung)projektInfo).isAttributKonvertierbar(attribut)) {
          if (((IPIGanglinienKonvertierung)projektInfo).getZuIgnorierendeWerte().contains(wert)) {
            continue;
          }
          String konvWert = ((IPIGanglinienKonvertierung)projektInfo).getWertKonvertiert(attribut, wert, objekt);
          if (konvWert != null) {
            wert = konvWert;  
          }
        }
        
        wert.replace(',', '.');

        // die liste der zeitstempel und werte ermitteln
        SortedMap<Long, String> zeitstempelWertListe = objektNachschlagewerk.get(objekt);
        if (zeitstempelWertListe == null)
        {
          // liste noch nicht vorhanden -> neu anlegen und im objekt nachschlagewerk ablegen
          zeitstempelWertListe = new TreeMap<Long, String>();
          objektNachschlagewerk.put(objekt, zeitstempelWertListe);
        }
        // zeitstempel und wert in die liste eintragen
        zeitstempelWertListe.put(zeitstempel, wert);
      }
      
      if (projektInfo instanceof IPIGanglinienKonvertierung && ((IPIGanglinienKonvertierung)projektInfo).isAttributKonvertierbar(attribut)) {
        
        long milliesTreppenfunktionErgaenzen = ((IPIGanglinienKonvertierung)projektInfo).getMilliesTreppenfunktionErgaenzen();
        if (milliesTreppenfunktionErgaenzen > 0) {
          // Werte fr 'Treppenfunktion' ergnzen
          for ( SortedMap<Long, String> zeitstempelWertListe: objektNachschlagewerk.values()) {
            
            Long letzterZeitstempel = null;
            String letzterWert = null;
            String letzerGeaenderterWert = null;
                           
            Map<Long, String> ergaenzungsMap = new HashMap<Long, String>();
            
            for (Long zeitstempel: zeitstempelWertListe.keySet()) {
              
              String wert = zeitstempelWertListe.get(zeitstempel);
              
              if (letzerGeaenderterWert == null || !letzerGeaenderterWert.equals(wert)) {
                if (letzerGeaenderterWert != null && letzterZeitstempel != null && letzterWert != null) {
                  if (zeitstempel - letzterZeitstempel > milliesTreppenfunktionErgaenzen) {
                    ergaenzungsMap.put(zeitstempel - milliesTreppenfunktionErgaenzen, letzerGeaenderterWert);
                  }
                }
                letzerGeaenderterWert = wert;
              }
              letzterWert = wert;
              letzterZeitstempel = zeitstempel;
            }
            
            // Ergnze Werte fr Treppenfunktion
            zeitstempelWertListe.putAll(ergaenzungsMap);
            
            // Anfang auffllen
            if (((IPIGanglinienKonvertierung)projektInfo).anfangAuffuellen()) {
              Integer letzterWertVorAnfang = null;
              Collection<Long> zeitstempelVorAnfang = new LinkedList<Long>();
              
              for (Long zeitstempel: zeitstempelWertListe.keySet()) {
                
                if (gesamtIntervall == null ||  zeitstempel >= gesamtIntervall.first) {
                  break;
                }
                
                String wert = zeitstempelWertListe.get(zeitstempel);
                zeitstempelVorAnfang.add(zeitstempel);
                try {
                  letzterWertVorAnfang = Integer.parseInt(wert);
                }
                catch (Exception e) {
                  // tue nichts
                }
              }
              
              for (Long zeitstempel: zeitstempelVorAnfang) {
                zeitstempelWertListe.remove(zeitstempel);
              }
              
              if (letzterWertVorAnfang != null && !zeitstempelWertListe.containsKey(gesamtIntervall.first)) {
                zeitstempelWertListe.put(gesamtIntervall.first, "" + letzterWertVorAnfang);
              }
            }
            
            // Ende auffllen
            if (((IPIGanglinienKonvertierung)projektInfo).endeAuffuellen()) {
              Long zeitstempelEnde = null;
              Integer letzterNumerischerWert = null;
              
              for (Long zeitstempel: zeitstempelWertListe.keySet()) {
                
                String wert = zeitstempelWertListe.get(zeitstempel);
                zeitstempelEnde = zeitstempel;
                try {
                  letzterNumerischerWert = Integer.parseInt(wert);
                }
                catch (Exception e) {
                  // tue nichts
                }
              }
                                
              if (gesamtIntervall != null && zeitstempelEnde != null && gesamtIntervall.last > zeitstempelEnde) {
                if (letzterNumerischerWert != null /* && !zeitstempelWertListe.containsKey(gesamtIntervall.last) */) {
                  zeitstempelWertListe.put(gesamtIntervall.last, "" + letzterNumerischerWert);  
                }
              }  
            }
          }  
        }
      }
      
      if (projektInfo instanceof IPIGanglinienKonvertierung) {
        attribut = ((IPIGanglinienKonvertierung)projektInfo).getAttributKonvertiert(attribut);
      }

      ret.put(attribut, objektNachschlagewerk);
    }

    return ret;
  }

  private void updateOkButton()
  {
    buttonOk.setEnabled((_tvBetriebsMeldungen1 == null || !_tvBetriebsMeldungen1.getSelection().isEmpty())
        && (_tvBetriebsMeldungen4 == null || _tvBetriebsMeldungen4.getTable().getItemCount() > 0));
  }
}
