/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.pua.basis.views;

import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;

import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;

/**
 * Klasse zur Anzeige der Daten aller Messquerschnitte als View
 * @author ObertM
 */
public class TableSort
{
  /**
   * Enum zur Definition der Sortiermglichkeiten
   * @author ObertM
   */
  enum SORT
  {
    STRING,
    TIME,
  };

  private boolean _sortFlag = true;
  
  private TableViewer _tv;

  /**
   * Konstruktor der Klasse
   */
  public TableSort(TableViewer tv)
  {
    _tv = tv;
  }

  /**
   * Basisklasse fr alle weiteren Comparatoren...
   * 
   */

  class BaseComapator implements Comparable {

    String[] _textBuffer;

    int _oldRowNum;

    int _colPos;

    boolean _sortFlag;

    public BaseComapator(String[] textBuffer, int oldRowNum, int colPos) {
      _textBuffer = textBuffer;
      _oldRowNum = oldRowNum;
      _colPos = colPos;
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Comparable#compareTo(java.lang.Object)
     */
    public int compareTo(Object o) {

      return 0;
    }
  }

  /**
   * 
   * @author Administrator Eigener Comparator der das Vergleichen der
   *         einzelnen Tabellenzeilen vornimmt.
   */

  class StringComparator extends BaseComapator implements Comparable {

    /**
     * Konstruktor ...
     * 
     * @param textBuffer :
     *            Der Text der Zeile
     * @param rowNum :
     *            Die Zeilennr der Tabellenzeile
     * @param sortFlag :
     *            Aufsteigend false, Absteigend true
     * @param colPos :
     *            Die Spalte nach der Sortiert werden soll
     */

    public StringComparator(String[] textBuffer, int rowNum, int colPos) {
      super(textBuffer, rowNum, colPos);
    }

    public final int compareTo(Object arg0) {
      int ret = _textBuffer[_colPos]
          .compareTo(((StringComparator) arg0)._textBuffer[_colPos]);
      return _sortFlag ? ret : ret * -1;
    }
  }
  
  /**
   * TimeComparator kmmert sich um das Vergleichen von Datumsangaben.
   * 
   */

  class TimeComparator extends BaseComapator implements Comparable {

    long _longValue;

    public TimeComparator(String[] textBuffer, int rowNum, int colPos) {
      super(textBuffer, rowNum, colPos);
      try {
        SimpleDateFormat sdf = new SimpleDateFormat(
            "dd.MM.yyyy HH:mm:ss");
        Date d = sdf.parse(textBuffer[colPos]);

        _longValue = d.getTime();

        // _debug.fine(textBuffer[colPos] + " : " + _longValue);

      } catch (Exception e) {
        // TODO: handle exception
        e.printStackTrace();
      }
    }

    public final int compareTo(Object o) {

      long _longValToCompare = ((TimeComparator) o)._longValue;
      int ret = _longValue < _longValToCompare ? -1
          : (_longValue == _longValToCompare ? 0 : 1);

      return _sortFlag ? ret : ret * -1;
    }

  }

  /**
   * Hier werden die Daten der Spalten des "original" TableItem[] in einen
   * MyComperatorArray umkopiert und darauf hin wird dieser mittels der
   * Array.sort(Object[] o) Methode sortiert. Da sich der MyComperator die
   * ursprngliche Position in der Tabelle gemerkt hat, kann man nun ganz
   * einfach die entsprechenden Werte im Original Array tauschen und schon ist
   * man fertig.
   * 
   * @param tableItems
   */
  protected final void sortTable(Table table, TableItem[] tableItems,
      int colPos, BaseComapator[][] comparables, boolean sortFlag) {

    // SortFlag bei allen richtig setzen ... leider kann ich bei inneren
    // keine static Variablen anlegen ansonsten wre das hier unntig...
    for (int i = 0; i < comparables[colPos].length; i++) {
      comparables[colPos][i]._sortFlag = sortFlag;
    }

    Arrays.sort(comparables[colPos]);

    String[] tmp;
    String[] textBuffer = null;
    textBuffer = new String[comparables.length];
    for (int i = 0; i < comparables[colPos].length; i++) {
      int rowNum = comparables[colPos][i]._oldRowNum;
      tmp = comparables[colPos][i]._textBuffer;
      tableItems[rowNum].setText(comparables[colPos][rowNum]._textBuffer);
      tableItems[i].setText(tmp);
    }
  }

  public void aendereSortierung ()
  {
    _sortFlag = !_sortFlag;
    
  }
  
  /**
   * Methode sortiert die Tabelle an Hand der Daten in der bergebenen Spalte
   * @param colPos  Spalte
   */
  protected void sortiere (int colPos)
  {
    sortiere(colPos, SORT.STRING);
  }
  
  /**
   * Methode sortiert die Tabelle an Hand der Daten in der bergebenen Spalte
   * @param colPos  Spalte
   * @param sortierer Sortierkennung
   */
  public void sortiere(int colPos, SORT sortierer)
  {
    if (colPos == -1)  // bisher nicht sortiert
      return;
    
    Table table = _tv.getTable();
    
    BaseComapator[][] comparables = new BaseComapator[table.getColumnCount()][table.getItemCount()];

    for (int i = 0; i < table.getItemCount(); i++) 
    {
      TableItem item = table.getItem(i);

      String[] strings = new String[table.getColumnCount()];
      for (int j = 0; j < table.getColumnCount(); j++) 
      {
        String text = item.getText(j);
        strings[j] = text;
      }

      switch (sortierer)
      {
        case STRING:
        {
          comparables[colPos][i] = new StringComparator(strings, i, colPos);
          break;
        }

        case TIME:
        {
          comparables[colPos][i] = new TimeComparator(strings, i, colPos);
          break;
        }
      }
    }

    sortTable(table, table.getItems(), colPos, comparables, _sortFlag);
  }
}
