package de.bsvrz.buv.plugin.pua.basis.views;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IContributor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.plugin.pua.basis.Activator;
import de.bsvrz.buv.plugin.pua.basis.DaVKommunikation;
import de.bsvrz.buv.plugin.pua.basis.pua.ProtokolleUndAuswertungen;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.pua.prot.util.ProtocolRequestResult;
import de.bsvrz.sys.funclib.losb.datk.AtlBase;

public class UpdateScriptsView extends ViewPart {

	public static final String ID = "de.bsvrz.buv.plugin.pua.views.updateScriptsView";
	
	Composite _composite;
	
	Table _table;
	
	List<IConfigurationElement> _scripts = new ArrayList<IConfigurationElement>();
	
	@Override
	public void createPartControl(Composite parent) {
		
		GridData gdComposite = new GridData();
		gdComposite.verticalAlignment = GridData.FILL;
		gdComposite.horizontalAlignment = GridData.FILL;
		gdComposite.grabExcessVerticalSpace = true;
		gdComposite.grabExcessHorizontalSpace = true;
		    
		_composite = new Composite(parent, SWT.NONE);
		_composite.setLayout(new GridLayout());
		_composite.setLayoutData(gdComposite);
		
		GridData gdTable = new GridData();
		gdTable.verticalAlignment = GridData.FILL;
		gdTable.horizontalAlignment = GridData.FILL;
		gdTable.grabExcessVerticalSpace = true;
		gdTable.grabExcessHorizontalSpace = true;
		 
		_table = new Table(_composite, SWT.CHECK | SWT.BORDER | SWT.V_SCROLL | SWT.H_SCROLL);
		_table.setLayoutData(gdTable);
		 
		IConfigurationElement[] configurationElementsFor = 
			Platform.getExtensionRegistry().getConfigurationElementsFor("de.bsvrz.buv.plugin.pua.basis.skriptInfos");
		 
		for (IConfigurationElement ce : configurationElementsFor) {
					  
			if("skriptInfos".equals(ce.getName())) {
				  
				for (IConfigurationElement ch : ce.getChildren()) {
					  
					if("skriptInfo".equals(ch.getName())) {
						  
						if ("true".equals(ch.getAttribute("Erstellen"))) {
							  
							String name = ch.getAttribute("Name");
							if (!"".equals(name)) {
								_scripts.add(ch);
								 TableItem item = new TableItem(_table, SWT.NONE);
							     item.setText(name);
							}
						}
					}
				}
			}
		}
		  
    	RowLayout rowLayout = new RowLayout();
		rowLayout.pack = false;
		GridData gdCompositeButton = new GridData();
		gdCompositeButton.horizontalAlignment = GridData.END;
		gdCompositeButton.grabExcessHorizontalSpace = true;
		gdCompositeButton.verticalAlignment = GridData.CENTER;
		Composite compositeBottom = new Composite(_composite, SWT.NONE);
		compositeBottom.setLayoutData(gdCompositeButton);
		compositeBottom.setLayout(rowLayout);

		Button buttonAktualisieren = new Button(compositeBottom, SWT.NONE);
	    buttonAktualisieren.setText("Aktualisieren");
		buttonAktualisieren.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				aktionAktualisieren();
			}
			
		});

		Button buttonAbbrechen = new Button(compositeBottom, SWT.NONE);
		buttonAbbrechen.setText("Abbrechen");
		buttonAbbrechen.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				aktionAbbrechen();
			}
			
		});
	
		
	}

	
	private void aktionAktualisieren() {

		ClientDavInterface connection = DaVKommunikation.getInstanz().holeDaVVerbindung();
		if (connection != null) {
			ProtokolleUndAuswertungen pua = new ProtokolleUndAuswertungen(connection);
			
			TableItem[] items = _table.getItems();
			int index = 0;
			int fehler = 0;
			int gesamt = 0;
			
			for (IConfigurationElement ch: _scripts) {
				if (items[index++].getChecked()) {
					gesamt++;
					IContributor plugin = ch.getContributor();
					String skriptDatei = ch.getAttribute("Skriptdatei");
					InputStream inputStream = null;
					String urlText = "platform:/plugin/" + plugin.getName() +
						(skriptDatei.startsWith("/") ? "" : "/") + skriptDatei;
					  
					try {
						URL url = new URL(urlText);
						inputStream = url.openConnection().getInputStream();
						BufferedReader source = new BufferedReader(new InputStreamReader(inputStream));
						StringBuffer buffer = new StringBuffer();
						String line;
						    
						while ((line = source.readLine()) != null) {
							buffer.append(line + "\n");
						}
						
						pua.erstelleSkript(ch.getAttribute("Pid"), ch.getAttribute("Name"),
							new AtlBase(ch.getAttribute("Name"), ch.getAttribute("Beschreibung"),
								ch.getAttribute("Autor"), ch.getAttribute("Status"), 
						    	ch.getAttribute("Datum")), buffer.toString());
						    
						ProtocolRequestResult result = pua.getRequestResult();
		     		  	if (result.isRequestSuccessful()) {
		     		  	      String msg = "Erfolgreich erstellt: " + skriptDatei;
		     			  	  Activator.getDefault().getLog().log(new Status(Status.INFO, Activator.PLUGIN_ID, msg));
		     			  	  System.err.println(msg);
		     		  	}
		     		  	else {
		     		  		  fehler++;
		     		  		  Status status = new Status(Status.ERROR, Activator.PLUGIN_ID, result.getErrorMsg()); 
		     		  		  String msg = "Fehler bei Erstellung: " + skriptDatei;
		     			  	  Activator.getDefault().getLog().log(new MultiStatus(Activator.PLUGIN_ID, Status.INFO, 
		     			  	      new IStatus[] { status }, msg, null));
		     			  	  System.err.println(msg);
		     			  	  System.err.println(result.getErrorMsg());
		     		    }
				  } 
				  catch (MalformedURLException e) {
					  fehler++;
					  String msg = "Ungltige URL \"" + urlText + "\"";
			  	      Activator.getDefault().getLog().log(new Status(Status.ERROR, Activator.PLUGIN_ID, msg, e));
			  	      System.err.println(msg);
			  	      e.printStackTrace(System.err);
				  }
				  catch (IOException e) {
					  fehler++;
					  String msg = "I/O-Fehler beim Lesen von \"" + skriptDatei + "\" als Bundle-Eintrag";
			  	      Activator.getDefault().getLog().log(new Status(Status.ERROR, Activator.PLUGIN_ID, msg, e));
			  	      System.err.println(msg);
			  	      e.printStackTrace(System.err);
				  }
				  catch (RuntimeException e) {
					  fehler++;
					  String msg = "Fehler beim Bearbeiten der URL \"" + urlText + "\"";
			  	      Activator.getDefault().getLog().log(new Status(Status.ERROR, Activator.PLUGIN_ID, msg, e));
			  	      System.err.println(msg);
			  	      e.printStackTrace(System.err);
				  }
				  finally {
					  if (inputStream != null) {
						  try {
							inputStream.close();
						  }
						  catch (IOException e) {
							e.printStackTrace();
						  }
					  }
				   }
				}
			}
			
			MessageBox mb = new MessageBox( new Shell( Display.getCurrent() ), SWT.ICON_INFORMATION | SWT.OK);
			mb.setText( "PUA-Skripte aktualisieren" );
			mb.setMessage((gesamt - fehler) + "/" + gesamt + " PUA-Skripte erstellt - Details siehe Error Log");
			mb.open();
		}
		else {
			MessageBox mb = new MessageBox( new Shell( Display.getCurrent() ), SWT.ICON_WARNING | SWT.OK);
			mb.setText( "PUA-Skripte aktualisieren" );
			mb.setMessage("Es knnen keine Skripte erstellt werden, da keine DaV-Verbindung besteht");
			mb.open();
		}	 
	}
		 
	private void aktionAbbrechen() {
		getViewSite().getPage().hideView(this);
	}
	
    @Override
	public void setFocus()
	{
    	//Fr die F1 Taste
	    if (_composite != null)
	    	_composite.setFocus();
    }

}
