/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/* Segment 13 Bedienung und Visualisierung, Plug-In Streckenprofil 
 * Copyright (C) 2009-2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information: 
 * BitCtrl Systems GmbH 
 * Weienfelser Strae 67 
 * 04229 Leipzig 
 * Phone: +49 341-490670 
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.streckenprofil.sts.a8;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.ImageRegistry;
import org.osgi.framework.BundleContext;

import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.StreckenprofilEditorModel;
import de.bsvrz.buv.rw.basislib.berechtigung.DaVKommunikation;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungenSpeicher;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungenWaehler;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungenWaehler.EinstellungsArt;
import de.bsvrz.buv.rw.bitctrl.AbstractDAVPlugin;
import de.bsvrz.sys.funclib.bitctrl.daf.DavProvider;
import de.bsvrz.sys.funclib.bitctrl.modell.tmrechner.objekte.Rechner;
import de.bsvrz.sys.funclib.bitctrl.modell.tmvewsimulationglobal.attribute.AtlStartInfo;

/**
 * The activator class controls the plug-in life cycle.
 * 
 * @author BitCtrl Systems GmbH, Enrico Schnepel
 */
public class Activator extends AbstractDAVPlugin implements
		PropertyChangeListener {

	/**
	 * Praefix fr Dateien, die ein {@link StreckenprofilEditorModel} enthalten.
	 */
	public static final String FILE_EXTENSION = "streckenprofil";

	/**
	 * Konstante des allgemeinen Pluginverzeichnisses.
	 */
	private static final String PLATFORM_PLUGIN = "platform:/plugin/";

	/**
	 * Plugin - ID.
	 */
	public static final String PLUGIN_ID = "de.bsvrz.buv.plugin.streckenprofil.sts.a8";

	/**
	 * Konstante fr das Icon des Selektionstransfers.
	 */
	public static final String SELEKTIONSTRANFER_GIF = "/icons/selektionstranfer.gif";

	/**
	 * Konstante fr das Icon zum anhalten der Offline Simulation.
	 */
	public static final String ICON_SIM_STOP_GIF = "/icons/icon_sim_stop.gif";

	/**
	 * Konstante fr das Icon zum pausieren der Offline Simulation.
	 */
	public static final String ICON_SIM_PAUSE_GIF = "/icons/icon_sim_pause.gif";

	/**
	 * Konstante fr das Icon zum starten der Offline Simulation.
	 */
	public static final String ICONS_ICON_SIM_START_GIF = "/icons/icon_sim_start.gif";

	/**
	 * Konstante fr die bildliche Darstellung statischer Beschilderung.
	 */
	public static final String GESCHWINDIGKEITSBESCHRAENKUNG_10_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/beschraenkung10.png";

	/**
	 * Konstante fr die bildliche Darstellung statischer Beschilderung.
	 */
	public static final String GESCHWINDIGKEITSBESCHRAENKUNG_20_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/beschraenkung20.png";

	/**
	 * Konstante fr die bildliche Darstellung statischer Beschilderung.
	 */
	public static final String GESCHWINDIGKEITSBESCHRAENKUNG_30_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/beschraenkung30.png";

	/**
	 * Konstante fr die bildliche Darstellung statischer Beschilderung.
	 */
	public static final String GESCHWINDIGKEITSBESCHRAENKUNG_40_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/beschraenkung40.png";

	/**
	 * Konstante fr die bildliche Darstellung statischer Beschilderung.
	 */
	public static final String GESCHWINDIGKEITSBESCHRAENKUNG_50_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/beschraenkung50.png";

	/**
	 * Konstante fr die bildliche Darstellung statischer Beschilderung.
	 */
	public static final String GESCHWINDIGKEITSBESCHRAENKUNG_70_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/beschraenkung70.png";

	/**
	 * Konstante fr die bildliche Darstellung statischer Beschilderung.
	 */
	public static final String GESCHWINDIGKEITSBESCHRAENKUNG_80_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/beschraenkung80.png";

	/**
	 * Konstante fr die bildliche Darstellung statischer Beschilderung.
	 */
	public static final String UEBERHOLVERBOT_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/ueberholverbot.png";

	/**
	 * Konstante fr die bildliche Darstellung statischer Beschilderung.
	 */
	public static final String UEBERHOLVERBOT_LKW_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/ueberholverbotLKW.png";

	/**
	 * Konstante fr die bildliche Darstellung statischer Beschilderung.
	 */
	public static final String UEBERHOLVERBOT_BEI_NAESSE_80_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/ueberholverbotBeiNaesse.png";

	/**
	 * Konstante fr die bildliche Darstellung statischer Beschilderung.
	 */
	public static final String GESCHWINDIGKEITSBESCHRAENKUNG_60_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/beschraenkung60.png";

	/**
	 * Konstante fr die bildliche Darstellung statischer Beschilderung.
	 */
	public static final String GESCHWINDIGKEITSBESCHRAENKUNG_100_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/beschraenkung100.png";

	/**
	 * Konstante fr die bildliche Darstellung statischer Beschilderung.
	 */
	public static final String GESCHWINDIGKEITSBESCHRAENKUNG_120_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/beschraenkung120.png";

	/**
	 * Konstante fr die bildliche Darstellung statischer Beschilderung.
	 */
	public static final String GESCHWINDIGKEITSBESCHRAENKUNG_130_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/beschraenkung130.png";

	/**
	 * Konstante fr die bildliche Darstellung statischer Beschilderung.
	 */
	public static final String BESCHRAENKUNG_ENDE_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/ende_beschraenkung.png";

	/**
	 * Konstante fr die bildliche Darstellung statischer Beschilderung.
	 */
	public static final String ACHSENPOSITION_LINKS_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/AxisSide_LEFT.png";

	/**
	 * Konstante fr die bildliche Darstellung statischer Beschilderung.
	 */
	public static final String ACHSENPOSITION_RECHTS_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/AxisSide_RIGHT.png";

	/**
	 * Konstante fr ein Bild.
	 */
	public static final String SERIESTYP_BALKEN_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/SeriesType_BAR.png";

	/**
	 * Konstante fr ein Bild.
	 */
	public static final String SERIESTYP_LINIE_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/SeriesType_LINE.png";

	/**
	 * Konstante fr ein Bild.
	 */
	public static final String SERIESTYP_BALKEN_GESPAPELT_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/SeriesType_BAR.png";

	/**
	 * Konstante fr ein Bild.
	 */
	public static final String LINESTYLE_DASH_DOTTED_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/LineStyle_DASH_DOTTED_LITERAL.png";

	/**
	 * Konstante fr ein Bild.
	 */
	public static final String LINESTYLE_DASHED_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/LineStyle_DASHED_LITERAL.png";

	/**
	 * Konstante fr ein Bild.
	 */
	public static final String LINESTYLE_DOTTED_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/LineStyle_DOTTED_LITERAL.png";

	/**
	 * Konstante fr ein Bild.
	 */
	public static final String LINESTYLE_SOLID_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/LineStyle_SOLID_LITERAL.png";

	/**
	 * Konstante fr ein Bild.
	 */
	public static final String LINETHICKNESS_NORMAL_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/LineThickness_NORMAL.png";

	/**
	 * Konstante fr ein Bild.
	 */
	public static final String LINETHICKNESS_THICK_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/LineThickness_THICK.png";

	/**
	 * Konstante fr ein Bild.
	 */
	public static final String LINETHICKNESS_THICKER_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/LineThickness_THICKER.png";

	/**
	 * Konstante fr ein Bild.
	 */
	public static final String LINETHICKNESS_THIN_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/LineThickness_THIN.png";

	/**
	 * Konstante fr ein Bild.
	 */
	public static final String MARKERTYPE_BOX_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/MarkerType_BOX_LITERAL.png";

	/**
	 * Konstante fr ein Bild.
	 */
	public static final String MARKERTYPE_CIRCLE_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/MarkerType_CIRCLE_LITERAL.png";

	/**
	 * Konstante fr ein Bild.
	 */
	public static final String MARKERTYP_COLUMN_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/MarkerType_COLUMN_LITERAL.png";

	/**
	 * Konstante fr ein Bild.
	 */
	public static final String MARKERTYPE_CROSS_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/MarkerType_CROSS_LITERAL.png";

	/**
	 * Konstante fr ein Bild.
	 */
	public static final String MARKERTYPE_CROSSHAIR_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/MarkerType_CROSSHAIR_LITERAL.png";

	/**
	 * Konstante fr ein Bild.
	 */
	public static final String MARKERTYPE_DIAMOND_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/MarkerType_DIAMOND_LITERAL.png";

	/**
	 * Konstante fr ein Bild.
	 */
	public static final String MARKERTYPE_ELLIPSE_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/MarkerType_ELLIPSE_LITERAL.png";

	/**
	 * Konstante fr ein Bild.
	 */
	public static final String MARKERTYPE_FOUR_DIAMONDS_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/MarkerType_FOUR_DIAMONDS_LITERAL.png";

	/**
	 * Konstante fr ein Bild.
	 */
	public static final String MARKERTYPE_HEXAGON_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/MarkerType_HEXAGON_LITERAL.png";

	/**
	 * Konstante fr ein Bild.
	 */
	public static final String MARKERTYPE_ICON_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/transparent.gif";

	/**
	 * Konstante fr ein Bild.
	 */
	public static final String MARKERTYPE_NABLA_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/MarkerType_NABLA_LITERAL.png";

	/**
	 * Konstante fr ein Bild.
	 */
	public static final String MARKERTYPE_RECTANGLE_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/MarkerType_RECTANGLE_LITERAL.png";

	/**
	 * Konstante fr ein Bild.
	 */
	public static final String MARKERTYPE_SEMI_CIRCLE_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/MarkerType_SEMI_CIRCLE_LITERAL.png";

	/**
	 * Konstante fr ein Bild.
	 */
	public static final String MARKERTYPE_STAR_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/MarkerType_STAR_LITERAL.png";

	/**
	 * Konstante fr ein Bild.
	 */
	public static final String MARKERTYPE_TRIANGLE_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/MarkerType_TRIANGLE_LITERAL.png";

	/**
	 * Konstante fr ein Bild.
	 */
	public static final String MARKERTYPE_TRANSPARENT_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/transparent.gif";

	/**
	 * Konstante fr die bildliche Darstellung Geschwindigkeitsbeschrnkung an
	 * Anzeigequerschnitten.
	 */
	public static final String DYNAMISCHE_GESCHWINDIGKEITSBESCHRAENKUNG_20_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/dynamischeBeschraenkung20.png";

	/**
	 * Konstante fr die bildliche Darstellung Geschwindigkeitsbeschrnkung an
	 * Anzeigequerschnitten.
	 */
	public static final String DYNAMISCHE_GESCHWINDIGKEITSBESCHRAENKUNG_30_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/dynamischeBeschraenkung30.png";

	/**
	 * Konstante fr die bildliche Darstellung Geschwindigkeitsbeschrnkung an
	 * Anzeigequerschnitten.
	 */
	public static final String DYNAMISCHE_GESCHWINDIGKEITSBESCHRAENKUNG_40_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/dynamischeBeschraenkung40.png";

	/**
	 * Konstante fr die bildliche Darstellung Geschwindigkeitsbeschrnkung an
	 * Anzeigequerschnitten.
	 */
	public static final String DYNAMISCHE_GESCHWINDIGKEITSBESCHRAENKUNG_50_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/dynamischeBeschraenkung50.png";

	/**
	 * Konstante fr die bildliche Darstellung Geschwindigkeitsbeschrnkung an
	 * Anzeigequerschnitten.
	 */
	public static final String DYNAMISCHE_GESCHWINDIGKEITSBESCHRAENKUNG_60_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/dynamischeBeschraenkung60.png";

	/**
	 * Konstante fr die bildliche Darstellung Geschwindigkeitsbeschrnkung an
	 * Anzeigequerschnitten.
	 */
	public static final String DYNAMISCHE_GESCHWINDIGKEITSBESCHRAENKUNG_70_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/dynamischeBeschraenkung70.png";

	/**
	 * Konstante fr die bildliche Darstellung Geschwindigkeitsbeschrnkung an
	 * Anzeigequerschnitten.
	 */
	public static final String DYNAMISCHE_GESCHWINDIGKEITSBESCHRAENKUNG_80_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/dynamischeBeschraenkung80.png";

	/**
	 * Konstante fr die bildliche Darstellung Geschwindigkeitsbeschrnkung an
	 * Anzeigequerschnitten.
	 */
	public static final String DYNAMISCHE_GESCHWINDIGKEITSBESCHRAENKUNG_90_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/dynamischeBeschraenkung90.png";

	/**
	 * Konstante fr die bildliche Darstellung Geschwindigkeitsbeschrnkung an
	 * Anzeigequerschnitten.
	 */
	public static final String DYNAMISCHE_GESCHWINDIGKEITSBESCHRAENKUNG_100_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/dynamischeBeschraenkung100.png";

	/**
	 * Konstante fr die bildliche Darstellung Geschwindigkeitsbeschrnkung an
	 * Anzeigequerschnitten.
	 */
	public static final String DYNAMISCHE_GESCHWINDIGKEITSBESCHRAENKUNG_110_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/dynamischeBeschraenkung110.png";

	/**
	 * Konstante fr die bildliche Darstellung Geschwindigkeitsbeschrnkung an
	 * Anzeigequerschnitten.
	 */
	public static final String DYNAMISCHE_GESCHWINDIGKEITSBESCHRAENKUNG_120_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/dynamischeBeschraenkung120.png";

	/**
	 * Konstante fr die bildliche Darstellung Geschwindigkeitsbeschrnkung an
	 * Anzeigequerschnitten.
	 */
	public static final String DYNAMISCHE_GESCHWINDIGKEITSBESCHRAENKUNG_130_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/dynamischeBeschraenkung130.png";

	/**
	 * Konstante fr die bildliche Darstellung Geschwindigkeitsbeschrnkung an
	 * Anzeigequerschnitten.
	 */
	public static final String DYNAMISCHE_GESCHWINDIGKEITSBESCHRAENKUNG_20_ENDE_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/dynamischeBeschraenkung20_ende.png";

	/**
	 * Konstante fr die bildliche Darstellung Geschwindigkeitsbeschrnkung an
	 * Anzeigequerschnitten.
	 */
	public static final String DYNAMISCHE_GESCHWINDIGKEITSBESCHRAENKUNG_30_ENDE_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/dynamischeBeschraenkung30_ende.png";

	/**
	 * Konstante fr die bildliche Darstellung Geschwindigkeitsbeschrnkung an
	 * Anzeigequerschnitten.
	 */
	public static final String DYNAMISCHE_GESCHWINDIGKEITSBESCHRAENKUNG_40_ENDE_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/dynamischeBeschraenkung40_ende.png";

	/**
	 * Konstante fr die bildliche Darstellung Geschwindigkeitsbeschrnkung an
	 * Anzeigequerschnitten.
	 */
	public static final String DYNAMISCHE_GESCHWINDIGKEITSBESCHRAENKUNG_50_ENDE_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/dynamischeBeschraenkung50_ende.png";

	/**
	 * Konstante fr die bildliche Darstellung Geschwindigkeitsbeschrnkung an
	 * Anzeigequerschnitten.
	 */
	public static final String DYNAMISCHE_GESCHWINDIGKEITSBESCHRAENKUNG_60_ENDE_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/dynamischeBeschraenkung60_ende.png";

	/**
	 * Konstante fr die bildliche Darstellung Geschwindigkeitsbeschrnkung an
	 * Anzeigequerschnitten.
	 */
	public static final String DYNAMISCHE_GESCHWINDIGKEITSBESCHRAENKUNG_70_ENDE_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/dynamischeBeschraenkung70_ende.png";

	/**
	 * Konstante fr die bildliche Darstellung Geschwindigkeitsbeschrnkung an
	 * Anzeigequerschnitten.
	 */
	public static final String DYNAMISCHE_GESCHWINDIGKEITSBESCHRAENKUNG_80_ENDE_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/dynamischeBeschraenkung80_ende.png";

	/**
	 * Konstante fr die bildliche Darstellung Geschwindigkeitsbeschrnkung an
	 * Anzeigequerschnitten.
	 */
	public static final String DYNAMISCHE_GESCHWINDIGKEITSBESCHRAENKUNG_90_ENDE_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/dynamischeBeschraenkung90_ende.png";

	/**
	 * Konstante fr die bildliche Darstellung Geschwindigkeitsbeschrnkung an
	 * Anzeigequerschnitten.
	 */
	public static final String DYNAMISCHE_GESCHWINDIGKEITSBESCHRAENKUNG_100_ENDE_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/dynamischeBeschraenkung100_ende.png";

	/**
	 * Konstante fr die bildliche Darstellung Geschwindigkeitsbeschrnkung an
	 * Anzeigequerschnitten.
	 */
	public static final String DYNAMISCHE_GESCHWINDIGKEITSBESCHRAENKUNG_110_ENDE_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/dynamischeBeschraenkung110_ende.png";

	/**
	 * Konstante fr die bildliche Darstellung Geschwindigkeitsbeschrnkung an
	 * Anzeigequerschnitten.
	 */
	public static final String DYNAMISCHE_GESCHWINDIGKEITSBESCHRAENKUNG_120_ENDE_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/dynamischeBeschraenkung120_ende.png";

	/**
	 * Konstante fr die bildliche Darstellung Geschwindigkeitsbeschrnkung an
	 * Anzeigequerschnitten.
	 */
	public static final String DYNAMISCHE_GESCHWINDIGKEITSBESCHRAENKUNG_130_ENDE_ICON_MARKER_URL = PLATFORM_PLUGIN
			+ PLUGIN_ID + "/icons/dynamischeBeschraenkung130_ende.png";

	/**
	 * Konstante fr das Icon zum Zoomen.
	 */
	public static final String ICON_ZOOM_IN = PLATFORM_PLUGIN + PLUGIN_ID
			+ "/icons/zoomIn.gif";

	/**
	 * Konstante fr das Icon zum Zoomen.
	 */
	public static final String ICON_ZOOM_OUT = PLATFORM_PLUGIN + PLUGIN_ID
			+ "/icons/zoomOut.gif";

	/**
	 * Konstante fr das Aufklappen.
	 */
	public static final String ICON_EXPAND_ALL = PLATFORM_PLUGIN + PLUGIN_ID
			+ "/icons/expandall.gif";

	/**
	 * The shared instance.
	 */
	private static Activator plugin;

	/**
	 * The constructor.
	 */
	public Activator() {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.ui.plugin.AbstractUIPlugin#start(org.osgi.framework.BundleContext
	 * )
	 */
	@Override
	public void start(final BundleContext context) throws Exception {
		super.start(context);
		plugin = this;
		DaVKommunikation.getInstanz().hinzufuegenOberflaechenBenutzerZuhoerer(
				StreckenprofilBerechtigungsVerwaltung.getInstanz());
		getObjektFactory().addPropertyChangeListener(this);
		if (getObjektFactory().isVerbunden()) {
			startOnline();
		} else {
			startOffline();
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.ui.plugin.AbstractUIPlugin#stop(org.osgi.framework.BundleContext
	 * )
	 */
	@Override
	public void stop(final BundleContext context) throws Exception {
		DaVKommunikation.getInstanz().entferneOberflaechenBenutzerZuhoerer(
				StreckenprofilBerechtigungsVerwaltung.getInstanz());
		plugin = null;
		super.stop(context);
	}

	/**
	 * Returns the shared instance.
	 * 
	 * @return the shared instance
	 */
	public static Activator getDefault() {
		return plugin;
	}

	/**
	 * Rahmenwerk - "Properties" fr Simulation ermitteln.
	 * 
	 * @return Liefert die {@link AtlStartInfo}, zum Starten der Offline -
	 *         Simulation.
	 */
	public AtlStartInfo ermittleSimulationStartInfo() {
		Activator.getDefault();
		final String rechnerPid = Activator.PLUGIN_ID
				+ ".simulation.rechnerpid";
		final String startStoppLeer = Activator.PLUGIN_ID + ".startstoppskript";
		final EinstellungenWaehler esw = EinstellungenWaehler.getInstanz();
		final EinstellungenSpeicher esp = esw
				.getEinstellungen(EinstellungsArt.allgemeinLokal);
		final String rechnerPidParameter = esp.getParameter(rechnerPid);
		final String startStoppLeerParameter = esp.getParameter(startStoppLeer);
		if (rechnerPidParameter != null && startStoppLeerParameter != null) {
			final AtlStartInfo info = new AtlStartInfo();
			info.setRechner((Rechner) Activator.getDefault().getObjektFactory()
					.getModellobjekt(rechnerPidParameter));
			info.setStartskript(startStoppLeerParameter);
			return info;
		} else {
			return null;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.ui.plugin.AbstractUIPlugin#initializeImageRegistry(org.eclipse
	 * .jface.resource.ImageRegistry)
	 */
	@Override
	protected void initializeImageRegistry(final ImageRegistry reg) {
		super.initializeImageRegistry(reg);

		reg.put(ICONS_ICON_SIM_START_GIF,
				getImageDescriptor(ICONS_ICON_SIM_START_GIF));
		reg.put(ICON_SIM_STOP_GIF, getImageDescriptor(ICON_SIM_STOP_GIF));
		reg.put(SELEKTIONSTRANFER_GIF,
				getImageDescriptor(SELEKTIONSTRANFER_GIF));
		reg.put(ICON_ZOOM_IN, getImageDescriptor(ICON_ZOOM_IN));
		reg.put(ICON_ZOOM_OUT, getImageDescriptor(ICON_ZOOM_OUT));
		reg.put(ICON_SIM_PAUSE_GIF, getImageDescriptor(ICON_SIM_PAUSE_GIF));
		reg.put(ICON_EXPAND_ALL, getImageDescriptor(ICON_EXPAND_ALL));
	}

	/**
	 * (Neu-)Start des Plug-in bei vorhandener Verbindung zum DaV.
	 */
	private void startOnline() {
		Activator
				.getDefault()
				.getLogger()
				.info("Initialisiere die Berechtigungsverwaltung"
						+ " des Plug-in Streckenprofil");
		DaVKommunikation.getInstanz().hinzufuegenOberflaechenBenutzerZuhoerer(
				StreckenprofilBerechtigungsVerwaltung.getInstanz());
		StreckenprofilBerechtigungsVerwaltung.getInstanz()
				.aktualisiereBerechtigungen();
	}

	/**
	 * (Neu-)Start des Plug-in bei fehlender Verbindung zum DaV.
	 */
	private void startOffline() {
		DaVKommunikation.getInstanz().entferneOberflaechenBenutzerZuhoerer(
				StreckenprofilBerechtigungsVerwaltung.getInstanz());
	}

	@Override
	public void propertyChange(final PropertyChangeEvent evt) {
		if (DavProvider.PROP_VERBUNDEN.equals(evt.getPropertyName())) {
			if (Boolean.TRUE.equals(evt.getNewValue())) {
				startOnline();
			} else {
				startOffline();
			}
		}
	}
}
