/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.bsvrz.buv.plugin.streckenprofil.sts.a8.actions;

import java.awt.image.BufferedImage;
import java.awt.image.DirectColorModel;
import java.awt.image.WritableRaster;
import java.io.File;
import java.lang.ref.WeakReference;

import javax.swing.SwingUtilities;

import net.sf.paperclips.GridPrint;
import net.sf.paperclips.PagePrint;
import net.sf.paperclips.PaperClips;
import net.sf.paperclips.Print;
import net.sf.paperclips.PrintIterator;
import net.sf.paperclips.PrintJob;
import net.sf.paperclips.PrintPiece;
import net.sf.paperclips.ScalePrint;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.ActionContributionItem;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuCreator;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Device;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.PaletteData;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.graphics.Transform;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IViewActionDelegate;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.PlatformUI;

import com.swarco.sts.chart.control.TChart;

import de.bsvrz.buv.plugin.streckenprofil.sts.a8.util.ImageFactory;
import de.bsvrz.buv.rw.basislib.seiteneinstellungen.DefsSeiteneinstellungen;
import de.bsvrz.buv.rw.basislib.seiteneinstellungen.druck.vorschau.DruckvorschauDialog;
import de.bsvrz.buv.rw.basislib.seiteneinstellungen.druck.vorschau.DruckvorschauDialog.SeitenDekoration;

public class ExportAktion implements IViewActionDelegate, IMenuCreator
{
  public final static int EX_PDF_A4_HOCHFORMAT = 0; 
  public final static int EX_PDF_A4_QUERFORMAT = 1;
  public final static int EX_PRINT_VIEW = 2;

  private ChartAktionenZiel chartAktionenZiel = null;
  private IAction aktion = null;
  private Menu menu = null;
  private UnterExportAktion exPdfHochAktion = null;
  private UnterExportAktion exPdfQuerAktion = null;
  private UnterExportAktion exPrintViewAktion = null;
  
  /**
   * Die Singleton-Instanz der Image-Verwaltung
   */
  private ImageFactory _imageFactory = ImageFactory.getInstanz();
  
  public ExportAktion()
  {
    super();
  }
  
  public ExportAktion(ChartAktionenZiel chartAktionenZiel)
  {
    super();
    
    this.chartAktionenZiel = chartAktionenZiel;
  }

  @Override
  public void init(IViewPart sicht)
  {
    if(sicht instanceof ChartAktionenZiel)
    {
      this.chartAktionenZiel = (ChartAktionenZiel)sicht;
    }
  }

  @Override
  public void run(IAction aktion)
  {
  }

  @Override
  public void selectionChanged(IAction aktion, ISelection auswahl)
  {
    if(aktion != this.aktion)
    {
      aktion.setMenuCreator(this);
      this.aktion = aktion;
    }
  }

  public void dispose()
  {
    if(menu != null)
    {
      menu.dispose();
      menu = null;
    }
//    if(exPdfHochAktion != null)
//    {
//      exPdfHochAktion.dispose();
//      exPdfHochAktion = null;
//    }
//    if(exPdfQuerAktion != null)
//    {
//      exPdfQuerAktion.dispose();
//      exPdfQuerAktion = null;
//    }
//    if(exPrintViewAktion != null)
//    {
//      exPrintViewAktion.dispose();
//      exPrintViewAktion = null;
//    }
  }

  public Menu getMenu(Control vorgaenger)
  {
    dispose();
    menu = new Menu(vorgaenger);

    if(chartAktionenZiel != null)
    {
      TChart chart = chartAktionenZiel.getChart();
      if(chart != null)
      {
        exPdfHochAktion = new UnterExportAktion(vorgaenger.getDisplay(), "Export PDF (A4, Hochformat)", ExportAktion.EX_PDF_A4_HOCHFORMAT);
        exPdfQuerAktion = new UnterExportAktion(vorgaenger.getDisplay(), "Export PDF (A4, Querformat)", ExportAktion.EX_PDF_A4_QUERFORMAT);
        exPrintViewAktion = new UnterExportAktion(vorgaenger.getDisplay(), "Drucken", ExportAktion.EX_PRINT_VIEW);
        hinzufuegenAktionZuMenu(menu, exPdfHochAktion);
        hinzufuegenAktionZuMenu(menu, exPdfQuerAktion);
        hinzufuegenAktionZuMenu(menu, exPrintViewAktion);
      }
    }

    return (menu);
  }


  public Menu getMenu(Menu vorgaenger)
  {
    return (null);
  }

  private void hinzufuegenAktionZuMenu(Menu menu, IAction aktion)
  {
    ActionContributionItem element = new ActionContributionItem(aktion);
    element.fill(menu, -1);
  }
  
  public void doExportAktion(final int type)
  {
    if(type == ExportAktion.EX_PRINT_VIEW)
    {
      printChart();
    }
    else
    {
      FileDialog dateifenster = new FileDialog(PlatformUI.getWorkbench().getDisplay().getActiveShell(), SWT.SAVE);
      dateifenster.setText("Ganglinie als PDF exportieren");
      dateifenster.setFilterNames(new String[] { "PDF Datei" });
      dateifenster.setFilterExtensions(new String[] { "*.pdf" });
      String dateipfadundname = dateifenster.open();
      if (dateipfadundname != null && dateipfadundname.length() > 0)
      {
        final File datei = new File(dateipfadundname);
        if (datei.getAbsoluteFile().toString().endsWith(".pdf"))
        {
          de.bsvrz.buv.rw.basislib.Activator activator = de.bsvrz.buv.rw.basislib.Activator.getDefault();
          IPreferenceStore preferenceStore = activator.getPreferenceStore();
          final String kopfzeileLinks = preferenceStore.getString(DefsSeiteneinstellungen.kopfzeileLinks);
          final String kopfzeileMitte = preferenceStore.getString(DefsSeiteneinstellungen.kopfzeileMitte);
          final String kopfzeileRechts = preferenceStore.getString(DefsSeiteneinstellungen.kopfzeileRechts);
          final String fusszeileLinks = preferenceStore.getString(DefsSeiteneinstellungen.fusszeileLinks);
          final String fusszeileMitte = preferenceStore.getString(DefsSeiteneinstellungen.fusszeileMitte);
          final String fusszeileRechts = preferenceStore.getString(DefsSeiteneinstellungen.fusszeileRechts);

          final TChart chart = chartAktionenZiel.getChart();
          if(chart != null)
          {
            SwingUtilities.invokeLater(new Runnable()
            {
              public void run()
              {
                try
                {
                  switch(type)
                  {
                    case ExportAktion.EX_PDF_A4_HOCHFORMAT:
                    {
                      chart.exportToPdf(TChart.PDF_A4_PORTRAIT, datei, new String[]{kopfzeileLinks, kopfzeileMitte, kopfzeileRechts}, new String[]{fusszeileLinks, fusszeileMitte, fusszeileRechts});
                      break;
                    }
                    case ExportAktion.EX_PDF_A4_QUERFORMAT:
                    {
                      chart.exportToPdf(TChart.PDF_A4_LANDSCAPE, datei, new String[]{kopfzeileLinks, kopfzeileMitte, kopfzeileRechts}, new String[]{fusszeileLinks, fusszeileMitte, fusszeileRechts});
                      break;
                    }
                  }
                }
                catch(Throwable t)
                {
                  t.printStackTrace();
                }
                
                chart.repaint(false);
              }
            });
          }
        }
      }
    }
  }

  private class UnterExportAktion extends Action
  {
    private Image symbol = null;
    private int type = -1;
    
    public UnterExportAktion(Display display, String text, int type)
    {
      super(text, Action.AS_PUSH_BUTTON);
      this.type = type;
      
      String imageName = null;
      switch(type)
      {
        case ExportAktion.EX_PDF_A4_HOCHFORMAT:
        {
//          imageName = "export_pdf_a4_pt.gif";
          symbol = _imageFactory.getImage("EXPORT_A4_PT");
          break;
        }
        case ExportAktion.EX_PDF_A4_QUERFORMAT:
        {
//          imageName = "export_pdf_a4_ls.gif";
          symbol = _imageFactory.getImage("EXPORT_A4_LS");
          break;
        }
        case ExportAktion.EX_PRINT_VIEW:
        {
//          imageName = "printview_tsk.gif";
          symbol = _imageFactory.getImage("PRINTVIEW_TSK");
        }
      }
      
//      if(imageName != null)
//      {
//        try
//        {
//          // TODO RALF HORST Frage?
//          // Ist das eine gltige Methode um im Rahmenwerk ein Image aus dem '/icons'-Verzeichnis zu laden?
//          Bundle bundle = Platform.getBundle(Activator.PLUGIN_ID);
//          Path pfad = new Path("icons/" + imageName);
//          URL fileURL = FileLocator.find(bundle, pfad, null);
//          InputStream in = fileURL.openStream();
//          symbol = new Image(display, in);
//          in.close();
//          setImageDescriptor(ImageDescriptor.createFromImage(symbol));
//        }
//        catch(Exception e)
//        {
//          e.printStackTrace();
//        }
//      }
      
      if (symbol != null){
        setImageDescriptor(ImageDescriptor.createFromImage(symbol));
      }
    }
    
//    public void dispose()
//    {
//      if(symbol != null)
//      {
//        symbol.dispose();
//        symbol = null;
//      }
//    }
    
    public void run()
    {
      doExportAktion(type);
    }
  }
  
  // Ab hier implementierung Drucken
  // -------------------------------
  private void printChart()
  {
    ChartPrint chartPrint = new ChartPrint();

    GridPrint gridPrint = new GridPrint("d");
    gridPrint.add(chartPrint);

    ScalePrint scalePrint = new ScalePrint(gridPrint);

    PagePrint pagePrint = new PagePrint(scalePrint);
    de.bsvrz.buv.rw.basislib.Activator activator = de.bsvrz.buv.rw.basislib.Activator.getDefault();
    IPreferenceStore preferenceStore = activator.getPreferenceStore();
    pagePrint.setHeader(new SeitenDekoration(preferenceStore.getString(DefsSeiteneinstellungen.kopfzeileLinks),
                                             preferenceStore.getString(DefsSeiteneinstellungen.kopfzeileMitte), 
                                             preferenceStore.getString(DefsSeiteneinstellungen.kopfzeileRechts), 
                                             true, 
                                             "Ganglinie"));
    pagePrint.setHeaderGap(5);
    pagePrint.setFooter(new SeitenDekoration(preferenceStore.getString(DefsSeiteneinstellungen.fusszeileLinks),
                                             preferenceStore.getString(DefsSeiteneinstellungen.fusszeileMitte),
                                             preferenceStore.getString(DefsSeiteneinstellungen.fusszeileRechts), 
                                             false));
    pagePrint.setFooterGap(5);

    PrintJob printJob = new PrintJob("Ganglinie Ausdruck", pagePrint);
    chartPrint.setPrintJob(printJob);
    DruckvorschauDialog vorschau = new DruckvorschauDialog(PlatformUI.getWorkbench().getDisplay().getActiveShell());
    vorschau.oeffne(printJob);
  }
  
  private class ChartPrint implements Print
  {
    private WeakReference<PrintJob> printJob = null;
    
    public PrintIterator iterator(Device device, GC gc)
    {
      PrintJob printJob = getPrintJob();
      float leftMargin = (float)printJob.getMargins().left * ((float)gc.getDevice().getDPI().x / 72.0f); 
      float rightMargin = (float)printJob.getMargins().right * ((float)gc.getDevice().getDPI().x / 72.0f);
      float topMargin = (float)printJob.getMargins().top * ((float)gc.getDevice().getDPI().y / 72.0f);
      float bottomMargin = (float)printJob.getMargins().bottom * ((float)gc.getDevice().getDPI().y / 72.0f);
      float width = gc.getDevice().getClientArea().width;
      float height = gc.getDevice().getClientArea().height;

      int pageWidth = 0;
      int pageHeight = 0;
      if(printJob.getOrientation() == PaperClips.ORIENTATION_DEFAULT || printJob.getOrientation() == PaperClips.ORIENTATION_PORTRAIT)
      {
        pageWidth = (int)((float)width - leftMargin - rightMargin);
        pageHeight = (int)((float)height - topMargin - bottomMargin);
      }
      else
      {
        pageWidth = (int)((float)height - leftMargin - rightMargin);
        pageHeight = (int)((float)width - topMargin - bottomMargin);
      }
      
      return (new InternalChartPrintIterator(pageWidth, pageHeight));
    }

    private void setPrintJob(PrintJob printJob)
    {
      this.printJob = new WeakReference<PrintJob>(printJob);
    }
    
    private PrintJob getPrintJob()
    {
      return (printJob.get());
    }
    
    private class InternalChartPrintIterator implements PrintIterator
    {
      private int pageWidth = 0;
      private int pageHeight = 0;
      private boolean hasNext = true;
      private ChartPiece chartPiece = null;
      
      private InternalChartPrintIterator(int pageWidth, int pageHeight)
      {
        this.pageWidth = pageWidth;
        this.pageHeight = pageHeight;
        chartPiece = new ChartPiece(pageWidth, pageHeight);
      }
      
      public PrintIterator copy()
      {
        InternalChartPrintIterator ret = new InternalChartPrintIterator(pageWidth, pageHeight);
        ret.hasNext = hasNext;
        
        return (ret);
      }

      public boolean hasNext()
      {
        return (hasNext);
      }

      public Point minimumSize()
      {
        return (chartPiece.getSize());
      }

      public PrintPiece next(int width, int height)
      {
        hasNext = false;
        return (chartPiece);
      }

      public Point preferredSize()
      {
        return (chartPiece.getSize());
      }
    }

    private class ChartPiece implements PrintPiece
    {
      private int pageWidth = 0;
      private int pageHeight = 0;
      private TChart chart = chartAktionenZiel.getChart();
      
      private ChartPiece(int pageWidth, int pageHeight)
      {
        this.pageWidth = pageWidth;
        this.pageHeight = pageHeight;
      }
      
      public void dispose()
      {
      }

      public Point getSize()
      {
        return new Point(pageWidth, pageHeight);
      }

      private boolean isPreview()
      {
        boolean ret = true;
        
        try
        {
          throw new RuntimeException();
        }
        catch(Throwable t)
        {
          StackTraceElement[] stackTrace = t.getStackTrace();
          for(int i = 0; i < stackTrace.length; i++)
          {
            if(stackTrace[i].toString().indexOf("net.sf.paperclips.PaperClips.printPages") != -1)
            {
              ret = false;
            }
          }
        }
        
        return (ret);
      }
      
      private Image convertToSWTImage(Device device, BufferedImage bufferedImage) 
      {
        Image ret = null;
        
        if(bufferedImage.getColorModel() instanceof DirectColorModel)
        {
          DirectColorModel colorModel = (DirectColorModel) bufferedImage.getColorModel();
          PaletteData palette = new PaletteData(colorModel.getRedMask(), colorModel.getGreenMask(), colorModel.getBlueMask());
          ImageData data = new ImageData(bufferedImage.getWidth(), bufferedImage.getHeight(), colorModel.getPixelSize(), palette);
          WritableRaster raster = bufferedImage.getRaster();
          int[] pixelArray = new int[3];
          RGB rgb = new RGB(0, 0, 0);
          for(int y = 0; y < data.height; y++) 
          {
            for(int x = 0; x < data.width; x++) 
            {
              raster.getPixel(x, y, pixelArray);
              rgb.red = pixelArray[0];
              rgb.green = pixelArray[1];
              rgb.blue = pixelArray[2];
              int pixel = palette.getPixel(rgb);
              data.setPixel(x, y, pixel);
            }
          }
          ret = new Image(device, data);
        }
        
        return (ret);
      } 
      
      public void paint(GC gc, int x, int y)
      {
        float dpiPreview = 216.0f;
        float dpiPrint = 600.0f;
        float previewLimit = 1000.0f;
        float printLimit = 16000.0f;
        float dpi = Float.NaN;
        float limit = Float.NaN;
        
        int tileWidth = 0;
        int tileHeight = 0;
        int totalWidth = 0;
        int totalHeight = 0;
        float scaleX = Float.NaN;
        float scaleY = Float.NaN;
        Point printerDPI = gc.getDevice().getDPI();

        if(isPreview())
        {
          dpi = dpiPreview;
          limit = previewLimit;
        }
        else
        {
          dpi = dpiPrint;
          limit = printLimit;
        }

        float scaleXTmp = printerDPI.x / dpi;
        float scaleYTmp = printerDPI.y / dpi;
        float scaleTmp = Math.min(scaleXTmp, scaleYTmp);
        float totalWidthTmp = (float)pageWidth / scaleTmp;
        float totalHeightTmp = (float)pageHeight / scaleTmp;
        if(Math.max(totalWidthTmp, totalHeightTmp) > limit)
        {
          scaleTmp = scaleTmp * Math.max(totalWidthTmp, totalHeightTmp) / limit;
          if(totalWidthTmp > totalHeightTmp)
          {
            totalHeightTmp *= (limit / totalWidthTmp);
            totalWidthTmp = limit;
          }
          else
          {
            totalWidthTmp *= (limit / totalHeightTmp);
            totalHeightTmp = limit;
          }
        }
        tileWidth = (int)Math.min(1500, totalWidthTmp);
        tileHeight = (int)Math.min(1500, totalHeightTmp);
        totalWidth = (int)totalWidthTmp;
        totalHeight = (int)totalHeightTmp;
        if(scaleXTmp < scaleYTmp)
        {
          scaleX = scaleTmp;
          scaleY = scaleTmp * (scaleYTmp / scaleXTmp);
        }
        else
        {
          scaleX = scaleTmp * (scaleXTmp / scaleYTmp);
          scaleY = scaleTmp;
        }
        
        Transform transformOrig = new Transform(gc.getDevice());
        gc.getTransform(transformOrig);

        float[] elements = new float[6];
        transformOrig.getElements(elements);
        Transform transformNew = new Transform(gc.getDevice(), elements);
        transformNew.scale(scaleX, scaleY);
        gc.setTransform(transformNew);

        java.awt.Dimension resultingSize = new java.awt.Dimension();
        java.awt.Point index = new java.awt.Point();
        
        // TODO RALF HORST Frage? 
        // Hier ist das eigentliche Drucken. Das hochaufgelste Chart wird Kachel fr Kachel in
        // den GC gezeichnet. Lsst sich das drucken eleganter lsen als hier (mit Sicherheit)?
        // Wie komme ich an den PrintJob (auf richtige Art und Weise und nicht ber eine WeakReference)?
        // Wie kann ich beim zeichnen Unterscheiden, ob ich in den Preview zeichne oder auf den Drucker?
        // Das Drucken funktioniert so nicht stabil (Querformat auf FreePdf funktioniert bei mir 
        // nicht -> Programm hngt sich auf. Sonst geht mit allen Druckern.
        BufferedImage tile = chart.getFirstImage(tileWidth, tileHeight, totalWidth, totalHeight, resultingSize, index);
        int xOffset = Math.round((pageWidth - resultingSize.width * scaleX) / 2.0f / scaleX);
        while(tile != null)
        {
          Image tileImage = convertToSWTImage(gc.getDevice(), tile);
          if(tileImage != null)
          {
            gc.drawImage(tileImage, xOffset + index.x * tileWidth, index.y * tileHeight);
            tileImage.dispose();
          }
          tile.flush();
          tile = chart.getNextImage(index);
        }
        
        gc.setTransform(transformOrig);
      }
    }
  }
}
