/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/* Segment 13 Bedienung und Visualisierung, Plug-In Streckenprofil 
 * Copyright (C) 2009-2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information: 
 * BitCtrl Systems GmbH 
 * Weienfelser Strae 67 
 * 04229 Leipzig 
 * Phone: +49 341-490670 
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.streckenprofil.sts.a8.actions;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.nebula.widgets.grid.Grid;
import org.eclipse.nebula.widgets.grid.GridColumn;
import org.eclipse.nebula.widgets.grid.GridColumnGroup;
import org.eclipse.nebula.widgets.grid.GridItem;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;

import com.bitctrl.lib.eclipse.actions.AbstractCopyTableAction;

/**
 * Action zum exportieren der selektierten Tabelle nach CSV oder HTML.
 * 
 * @author BitCtrl Systems GmbH, Hoesel
 * @version $Id: ExportGridAction.java,v 1.3 2014/04/17 15:51:11 Pittner Exp $
 */
public class ExportGridAction extends AbstractCopyTableAction {

	/**
	 * Konstante fr den Zeileumbruch.
	 */
	private static final String LINEBREAK = "\r\n";

	/**
	 * Der Titeltext fr den Export.
	 */
	private static String title = "";

	/**
	 * Konstante fr den HTML Header.
	 */
	private static final String HTML_HEADER_START = "<html xmlns=\"http://www.w3.org/1999/xhtml\"><head><meta http-equiv=\"content-type\" content=\"text/html;charset=ISO-8859-1\"/><title>";

	/**
	 * Konstante fr das Ende des HTML Headers und den Body.
	 */
	private static final String HTML_HEADER_ENDE = "</title></head><body>";

	/**
	 * Konstante fr das Ende des Body.
	 */
	private static final String HTML_FOOTER = "</body></html>";

	/**
	 * Enthlt die zur exportierenden Daten.
	 */
	private final Grid grid;

	/**
	 * Erzeugt eines neue Aktion fr den Export eines {@link Grid}.
	 * 
	 * @param grid
	 *            Das {@link Grid} dessen Inhalt exportiert werden soll.
	 */
	public ExportGridAction(final Grid grid) {
		this.grid = grid;
		setToolTipText("Exportiert die selektierten Zeilen in der Tabelle nach CSV oder HTML");
	}

	@Override
	public void run() {
		if (grid != null && !grid.isDisposed()) {
			exportToFile(grid.getDisplay());
		}
	}

	/**
	 * Setzt den Titeltext, der im Header des Exports angezeigt wird.
	 * 
	 * @param neuerTitle
	 *            der Titel.
	 */
	public void setTitleText(final String neuerTitle) {
		title = neuerTitle != null ? neuerTitle : "";
	}

	@Override
	protected String getTextForSelection(final MainTag main) {
		final StringBuffer text = new StringBuffer();
		text.append(main.getBegin());
		text.append(main.head.getBegin());
		final GridColumnGroup[] columnGroups = grid.getColumnGroups();
		if (null != columnGroups && columnGroups.length > 0) {
			text.append(main.head.row.getBegin());
			if (grid.isRowHeaderVisible()) {
				text.append(main.head.row.data.getBegin());
				text.append(main.head.row.data.getEnd());
			}
			for (final GridColumnGroup group : columnGroups) {
				final int cols = group.getColumns().length;
				text.append(main.head.row.data.getBeginColspan(cols));
				final String content = group.getText();
				text.append(main.mask(content));
				text.append(main.head.row.data.getEndColspan(cols));
			}
			text.append(main.head.row.getEnd());
		}
		final GridColumn[] columns = grid.getColumns();
		text.append(main.head.row.getBegin());
		if (grid.isRowHeaderVisible()) {
			text.append(main.head.row.data.getBegin());
			text.append(main.head.row.data.getEnd());
		}
		for (final GridColumn column : columns) {
			text.append(main.head.row.data.getBegin());
			final String content = column.getText();
			text.append(main.mask(content));
			text.append(main.head.row.data.getEnd());
		}
		text.append(main.head.row.getEnd());
		text.append(main.head.getEnd());

		text.append(main.body.getBegin());
		GridItem[] rows = grid.getSelection();
		if (null == rows || 0 == rows.length) {
			rows = grid.getItems();
		}
		for (final GridItem row2 : rows) {
			text.append(main.body.row.getBegin());

			if (grid.isRowHeaderVisible()) {
				text.append(main.head.row.data.getBegin());
				text.append(main.mask(row2.getHeaderText()));
				text.append(main.head.row.data.getEnd());
			}
			for (int col = 0; col < columns.length; col++) {
				text.append(main.body.row.data.getBegin());
				final String content = row2.getText(col);
				text.append(main.mask(content));
				text.append(main.body.row.data.getEnd());
			}
			text.append(main.body.row.getEnd());
		}
		text.append(main.body.getEnd());
		text.append(main.getEnd());

		return text.toString();
	}

	/**
	 * Exportiert die Verkehrsdaten ein eine Datei.
	 * 
	 * @param display
	 *            Display, mit dessen Shell, der {@link FileDialog} geffnet
	 *            wird.
	 */
	protected void exportToFile(final Display display) {
		final FileDialog fd = new FileDialog(display.getActiveShell(), SWT.SAVE);
		fd.setFilterExtensions(new String[] { ".csv", ".html" });
		final String newFilePath = fd.open();
		if (null != newFilePath) {

			try {
				final File f = new File(newFilePath);

				if (f.exists()) {
					// berschreiben??
					if (MessageDialog
							.openQuestion(
									display.getActiveShell(),
									"Die Datei " + newFilePath
											+ " existiert bereits",
									"Die Datei "
											+ newFilePath
											+ " existiert bereits. Mchten Sie diese berschreiben?")) {
						f.delete();
					} else {
						// nicht berschreiben
						return;
					}

				}

				final FileOutputStream fo = new FileOutputStream(f);
				final OutputStreamWriter ouw = new OutputStreamWriter(fo,
						"ISO-8859-1");
				String text = "";
				if (newFilePath.endsWith("csv".toLowerCase())) {
					text = getTextForSelection(CSV);
					ouw.write("#" + title + LINEBREAK + text);
				} else if (newFilePath.endsWith("html".toLowerCase())) {
					text = getTextForSelection(HTML);
					ouw.write(HTML_HEADER_START + title + HTML_HEADER_ENDE
							+ text + HTML_FOOTER);
				} else {
					text = getTextForSelection(TEXT);
					ouw.write(title + LINEBREAK + text);
				}

				ouw.close();
				fo.close();
			} catch (final FileNotFoundException e) {
				e.printStackTrace();
			} catch (final IOException e) {
				e.printStackTrace();
			}
		}
	}
}
