/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/* Segment 13 Bedienung und Visualisierung, Plug-In Streckenprofil 
 * Copyright (C) 2009-2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information: 
 * BitCtrl Systems GmbH 
 * Weienfelser Strae 67 
 * 04229 Leipzig 
 * Phone: +49 341-490670 
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.streckenprofil.sts.a8.actions;

import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.TreePath;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.ui.IViewActionDelegate;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.PartInitException;

import de.bsvrz.buv.plugin.streckenprofil.sts.a8.editor.StreckenprofilEditor;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.editor.StreckenprofilEditorInput;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.StreckenprofilEditorModel;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungenWaehler.EinstellungsArt;

/**
 * Action zum ffnen des Streckenprofileditors, aus dem Kontextmenu des
 * StreckenprofilNavigators.
 * 
 * @author BitCtrl Systems GmbH, Hoesel
 * @version $Id: StreckenprofilOeffnenAction.java,v 1.3 2014/04/17 15:51:11 Pittner Exp $
 */
public class StreckenprofilOeffnenAction implements IViewActionDelegate {

	/**
	 * Der {@link IViewPart}, wird zum ffnen eines Streckenprofil - Editors
	 * bentigt.
	 */
	private IViewPart viewPart;

	/**
	 * Akutelle Selektion, die nach Streckenprofilen durchsucht wird.
	 */
	private ISelection aktuelleAuswahl;

	@Override
	public void init(final IViewPart view) {
		viewPart = view;

	}

	@Override
	public void run(final IAction action) {
		if (viewPart != null) {
			if (aktuelleAuswahl instanceof TreeSelection) {
				final Object firstSelection = ((TreeSelection) aktuelleAuswahl)
						.getFirstElement();
				final TreePath[] treePaths = ((TreeSelection) aktuelleAuswahl)
						.getPathsFor(firstSelection);
				if (firstSelection instanceof StreckenprofilEditorModel
						&& treePaths != null && treePaths.length > 0) {

					try {
						final Object parentObj = treePaths[0].getFirstSegment();
						final StreckenprofilEditorInput input = new StreckenprofilEditorInput();
						input
								.setModel((StreckenprofilEditorModel) firstSelection);
						if (parentObj instanceof EinstellungsArt) {
							input
									.setEinstellungsArt((EinstellungsArt) parentObj);
						}

						viewPart.getSite().getPage().openEditor(input,
								StreckenprofilEditor.EDITOR_ID);
					} catch (final PartInitException e) {
						e.printStackTrace();
					}
				}
			}
		}

	}

	@Override
	public void selectionChanged(final IAction action,
			final ISelection selection) {
		aktuelleAuswahl = selection;
	}

}
