/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/* Segment 13 Bedienung und Visualisierung, Plug-In Streckenprofil 
 * Copyright (C) 2009-2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information: 
 * BitCtrl Systems GmbH 
 * Weienfelser Strae 67 
 * 04229 Leipzig 
 * Phone: +49 341-490670 
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.streckenprofil.sts.a8.editor.pages;

import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.contexts.IContextActivation;
import org.eclipse.ui.contexts.IContextService;
import org.eclipse.ui.forms.editor.FormEditor;

import com.bitctrl.lib.eclipse.editors.FormPageWithBorders;

import de.bsvrz.buv.plugin.streckenprofil.sts.a8.StreckenprofilUtil;
import de.bsvrz.buv.rw.basislib.seiteneinstellungen.druck.vorschau.IDruckvorschau;
import de.bsvrz.sys.funclib.bitctrl.modell.tmvewsimulationglobal.objekte.OfflineSimulation;

/**
 * Abstrakte Basisklasse fr Streckenprofil-Editor
 * {@link org.eclipse.ui.forms.editor.FormPag}s, die den UI Context
 * {@link StreckenprofilUtil#STRECKENPROFIL_CHART_KONTEXT} aktivieren und
 * deaktieren mchten.
 * 
 * @author BitCtrl Systems GmbH, Hoesel
 * @version $Id: AbstractStreckenProfilEditorFormPage.java,v 1.3 2014/04/17 15:51:13 Pittner Exp $
 */
public abstract class AbstractStreckenProfilEditorFormPage extends
		FormPageWithBorders implements IDruckvorschau {

	/**
	 * Der Kontext, in dem die {@link org.eclipse.ui.forms.editor.FormPag}
	 * arbeitet. Wird fr den ShortCut <Strg+E> bentigt.
	 */
	private IContextActivation editorContext;

	/**
	 * Erzeugt eines neue {@link AbstractStreckenProfilEditorFormPage}.
	 * 
	 * @param editor
	 *            der Parent Editor.
	 * @param id
	 *            die ID der Form - Page.
	 * @param title
	 *            der Titel der {@link org.eclipse.ui.forms.editor.FormPag}.
	 */
	public AbstractStreckenProfilEditorFormPage(final FormEditor editor,
			final String id, final String title) {
		super(editor, id, title);
	}

	/**
	 * Erzeugt eines neue {@link AbstractStreckenProfilEditorFormPage}.
	 * 
	 * @param id
	 *            die ID der Form - Page.
	 * @param title
	 *            der Titel der {@link org.eclipse.ui.forms.editor.FormPag}.
	 */
	public AbstractStreckenProfilEditorFormPage(final String id,
			final String title) {
		super(id, title);
	}

	@Override
	public void setActive(final boolean active) {
		super.setActive(active);
		final IContextService cs = (IContextService) PlatformUI.getWorkbench()
				.getAdapter(IContextService.class);
		if (active) {
			editorContext = cs
					.activateContext(StreckenprofilUtil.STRECKENPROFIL_CHART_KONTEXT);
		} else {
			if (editorContext != null) {
				cs.deactivateContext(editorContext);
			}
		}
	}

	/**
	 * Setzt die Offline - Simulation fr diese Page.
	 * 
	 * @param simulation
	 *            Die Offline Simulation der Page, oder <code>null</code>, wenn
	 *            Live Daten empfangen werden sollen.
	 */
	public abstract void setSimulation(OfflineSimulation simulation);

}
