/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/* Segment 13 Bedienung und Visualisierung, Plug-In Streckenprofil 
 * Copyright (C) 2009-2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information: 
 * BitCtrl Systems GmbH 
 * Weienfelser Strae 67 
 * 04229 Leipzig 
 * Phone: +49 341-490670 
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.streckenprofil.sts.a8.editor.pages;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.birt.chart.model.attribute.Position;
import org.eclipse.core.databinding.DataBindingContext;
import org.eclipse.core.databinding.observable.ChangeEvent;
import org.eclipse.core.databinding.observable.IChangeListener;
import org.eclipse.core.databinding.observable.list.WritableList;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.databinding.EMFObservables;
import org.eclipse.emf.ecore.util.EContentAdapter;
import org.eclipse.jface.databinding.swt.SWTObservables;
import org.eclipse.jface.databinding.viewers.ObservableListContentProvider;
import org.eclipse.jface.databinding.viewers.ViewersObservables;
import org.eclipse.jface.dialogs.IMessageProvider;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.nebula.jface.tablecomboviewer.TableComboViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Scale;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Widget;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.forms.editor.FormEditor;
import org.eclipse.ui.forms.widgets.ColumnLayout;
import org.eclipse.ui.forms.widgets.ColumnLayoutData;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.ScrolledForm;
import org.eclipse.ui.forms.widgets.Section;
import org.eclipse.ui.views.properties.ComboBoxLabelProvider;

import com.bitctrl.lib.eclipse.editors.FormPageWithBorders;
import com.bitctrl.lib.eclipse.resources.WidgetImageRegistry;

import de.bsvrz.buv.plugin.streckenprofil.sts.a8.Activator;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.Modell;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.chart.AbstractStreckenprofilBuilder;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.editor.LinePropertiesTable;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.editor.StreckenprofilEditor;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.AchsenBeschreibung;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.DiagrammEigenschaften;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.LinienEigenschaften;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.SeriesType;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.StreckenprofilPackage;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.TypObjektBezeichnung;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.TypPrognoseTyp;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.TypVerkehrsDaten;

/**
 * {@link FormPageWithBorders} zum Einstellen der grafischen Eigenschaften des
 * Streckenprofils.
 * 
 * @author BitCtrl Systems GmbH, Hoesel
 * @version $Id: StreckenprofilEditorGrafischeEigenschaften.java,v 1.8 2014/04/28 13:02:06 Pittner Exp $
 */
public class StreckenprofilEditorGrafischeEigenschaften extends
		FormPageWithBorders {

	/**
	 * {@link LabelProvider} fr die Positionierung der Y-Achsen zum Chart.
	 * 
	 * @author BitCtrl Systems GmbH, christian
	 * @version $Id: StreckenprofilEditorGrafischeEigenschaften.java,v 1.8 2014/04/28 13:02:06 Pittner Exp $
	 */
	private class AxisSideLabelProvider extends LabelProvider {

		/**
		 * Mit diesem Widget wird ein mgliches Icon verbunden, dass dann beim
		 * disposen des Widgets auch aufgerumt wird.
		 */
		private final Widget widget;

		/**
		 * Konstruktor.
		 * 
		 * @param widget
		 *            Mit diesem Widget wird ein mgliches Icon verknpft, das
		 *            dann beim dispose des Widgets auch mit aufgerumt wird.
		 */
		public AxisSideLabelProvider(final Widget widget) {
			super();
			this.widget = widget;

		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see
		 * org.eclipse.jface.viewers.LabelProvider#getImage(java.lang.Object)
		 */
		@Override
		public Image getImage(final Object element) {
			ImageDescriptor result = null;
			if (element instanceof Position) {
				final Position type = (Position) element;
				switch (type) {
				case RIGHT_LITERAL:
					result = Activator.getDefault().getImageDescriptor(
							Activator.ACHSENPOSITION_RECHTS_ICON_MARKER_URL);
					break;
				case LEFT_LITERAL:
					result = Activator.getDefault().getImageDescriptor(
							Activator.ACHSENPOSITION_LINKS_ICON_MARKER_URL);
					break;
				default:
					result = null;
				}
			}
			final Image img = WidgetImageRegistry.getInstance(widget).getImage(
					result);
			return img;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see
		 * org.eclipse.jface.viewers.LabelProvider#getText(java.lang.Object)
		 */
		@Override
		public String getText(final Object element) {
			String result = super.getText(element);
			if (element instanceof Position) {
				final Position type = (Position) element;
				switch (type) {
				case RIGHT_LITERAL:
					result = "rechts";
					break;
				case LEFT_LITERAL:
					result = "links";
					break;
				default:
					result = super.getText(element);
				}
			}
			return result;
		}
	}

	/**
	 * Das angezeigte Streckenprofil.
	 */
	private final Modell modell;

	/**
	 * Der Databinding Kontext dieser FormPage.
	 */
	private final DataBindingContext dbc;

	/**
	 * Das dirty - Flag. Zeigt an, ob der Editor speichern kann (
	 * <code>true</code>).
	 */
	private boolean dirty;

	/**
	 * Lokaler {@link EContentAdapter}, der auf alle nderungen des Linientyps
	 * der {@link LinienEigenschaften} reagiert.
	 */
	private final EContentAdapter linienTypAdapter = new EContentAdapter() {
		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.emf.ecore.util.EContentAdapter#notifyChanged
		 * (org.eclipse.emf.common.notify.Notification)
		 */
		@Override
		public void notifyChanged(final Notification notification) {
			super.notifyChanged(notification);
			final Object feature = notification.getFeature();
			if ((StreckenprofilPackage.Literals.LINIEN_EIGENSCHAFTEN__SERIES_TYPE
					.equals(feature))) {
				ueberpruefungLinienEigenschaften();
			}
		}
	};

	/**
	 * Konstruktor fr eine FormPage, auf der die grafischen Eigenschaften eines
	 * Streckenprofil gendert werden knnen.
	 * 
	 * @param editor
	 *            der {@link FormEditor}, zu welchem diese FormPage gehrt.
	 * @param modell
	 *            das Streckenprofil.
	 */
	public StreckenprofilEditorGrafischeEigenschaften(final FormEditor editor,
			final Modell modell) {
		super(editor, StreckenprofilEditorGrafischeEigenschaften.class
				.getName(), "Eigenschaften");
		this.modell = modell;
		dbc = new DataBindingContext();
		
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.bitctrl.lib.eclipse.editors.FormPageWithBorders#createFormContent
	 * (org.eclipse.ui.forms.IManagedForm)
	 */
	@Override
	protected void createFormContent(final IManagedForm managedForm) {
		super.createFormContent(managedForm);
		final ScrolledForm form = managedForm.getForm();
		form.setText(getTitle());
		final Composite formBody = form.getForm().getBody();
		formBody.setLayout(new GridLayout(1, false));
		final FormToolkit toolkit = managedForm.getToolkit();
		createLinesSection(formBody, toolkit);
//		createYAxisSection(formBody, toolkit);
		createLinePropertiesSection(formBody, toolkit);
//		createXAxisZoom(formBody, toolkit);

		createAlternativesSection(formBody, toolkit);
		modell.getEMFModell().eAdapters().add(linienTypAdapter);
		// Online-Hilfe hinzufgen
		PlatformUI
				.getWorkbench()
				.getHelpSystem()
				.setHelp(
						getManagedForm().getForm(),
						Activator.PLUGIN_ID
								+ "."
								+ StreckenprofilEditorGrafischeEigenschaften.class
										.getSimpleName());
	}

	/**
	 * In dieser {@link Section}, wird die Skalierung der X-Achse bestimmt.
	 * 
	 * @param formBody
	 *            das Parent {@link Composite}.
	 * @param toolkit
	 *            das FormToolkit.
	 */
	private void createXAxisZoom(final Composite formBody,
			final FormToolkit toolkit) {
		final String title = "Skalierung X - Achse";
		final Section section = toolkit.createSection(formBody,
				defaultSectionStyle | Section.DESCRIPTION);
		section.setDescription("Durch die Skalierung der X - Achse kann die Gesamtbreite des Diagramms bestimmt werden, die dieses auf dem Bildschirm bentigt. ");
		section.setText(title);
		final Composite client = toolkit.createComposite(section, SWT.NONE);
		GridDataFactory.fillDefaults().grab(true, false).applyTo(section);
		toolkit.paintBordersFor(client);
		section.setClient(client);
		client.setLayout(new GridLayout(3, false));

		final Scale skalierungsScale = new Scale(client, SWT.HORIZONTAL);
		GridDataFactory.fillDefaults().hint(500, SWT.DEFAULT)
				.applyTo(skalierungsScale);
		skalierungsScale.setMinimum(700);
		skalierungsScale.setMaximum(32768);
		skalierungsScale.setIncrement(10);
		skalierungsScale.setPageIncrement(1000);
		skalierungsScale.setToolTipText("Skalierung der X-Achse");

		final Label labelSpinner = new Label(client, SWT.NONE);
		labelSpinner.setText("Pixel\t=");
		final Spinner scaleValue = new Spinner(client, SWT.BORDER);
		skalierungsScale.setSelection((int) modell.getChartProperties()
				.getSkalierungXAchse());
		scaleValue.setValues(skalierungsScale.getSelection(), 700, 32768, 0,
				10, 100);

		scaleValue.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(final SelectionEvent e) {
				final int value = ((Spinner) e.widget).getSelection();
				if (skalierungsScale.getSelection() != value) {
					skalierungsScale.setSelection(value);
					modell.getChartProperties().setSkalierungXAchse(value);
				}
			}
		});

		scaleValue.addModifyListener(new ModifyListener() {

			@Override
			public void modifyText(final ModifyEvent e) {
				final int value = ((Spinner) e.widget).getSelection();
				if (scaleValue.getSelection() != value) {
					modell.getChartProperties().setSkalierungXAchse(value);
				}
			}
		});

		skalierungsScale.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(final SelectionEvent e) {
				final int value = ((Scale) e.widget).getSelection();
				if (scaleValue.getSelection() != value) {
					scaleValue.setSelection(value);
					modell.getChartProperties().setSkalierungXAchse(value);
				}
			}
		});

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.forms.editor.FormPage#setFocus()
	 */
	@Override
	public void setFocus() {
		getManagedForm().getForm().setFocus();
	}

	/**
	 * Erzeugt eine {@link Section}, in der die
	 * {@link de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.LinienEigenschaften} des
	 * Modell bestimmt werden.
	 * 
	 * @param formBody
	 *            das Parent {@link Composite}.
	 * @param toolkit
	 *            das FormToolkit.
	 */
	private void createLinePropertiesSection(final Composite formBody,
			final FormToolkit toolkit) {
		LinePropertiesTable.createDiagrammComponents(formBody, toolkit, modell,
				true);
		formBody.layout(true, true);
		getManagedForm().reflow(true);
	}

	/**
	 * Die {@link Section}, in der ausgewhlt werden kann, welche Fahrstreifen
	 * oder ob der MQ betrachtet werden soll und welche Verkehrsdaten
	 * (vKfz,qKfz,...) verwendet werden.
	 * 
	 * @param formBody
	 *            das Parent {@link Composite}.
	 * @param toolkit
	 *            das FormToolkit.
	 */
	private void createLinesSection(final Composite formBody,
			final FormToolkit toolkit) {
		final String title = "Beschilderung anzeigen";
		final Section section = toolkit.createSection(formBody,
				defaultSectionStyle);
		section.setText(title);
		final Composite client = toolkit.createComposite(section, SWT.NONE);
		toolkit.paintBordersFor(client);
		section.setClient(client);
		client.setLayout(new GridLayout());

		final Button gsbStatLinienAnzeigenButton = toolkit.createButton(client,
				"Geschwindigkeitsbeschrnkungen (stat.) als roten Balken anzeigen",
				SWT.CHECK);

		dbc.bindValue(
				SWTObservables.observeSelection(gsbStatLinienAnzeigenButton),
				EMFObservables.observeValue(
						modell.getChartProperties(),
						StreckenprofilPackage.Literals.DIAGRAMM_EIGENSCHAFTEN__GESCHWINDIGKEITS_LIMIT_LINIE_STAT));
		
		final Button gsbDynLinienAnzeigenButton = toolkit.createButton(client,
		    "Geschwindigkeitsbeschrnkungen (dyn.) als roten Balken anzeigen",
		    SWT.CHECK);
		
		dbc.bindValue(
		    SWTObservables.observeSelection(gsbDynLinienAnzeigenButton),
		    EMFObservables.observeValue(
		        modell.getChartProperties(),
		        StreckenprofilPackage.Literals.DIAGRAMM_EIGENSCHAFTEN__GESCHWINDIGKEITS_LIMIT_LINIE_DYN));

		final Button geschwIconsAnzeigenButton = toolkit.createButton(client,
				"Statische Beschilderung anzeigen", SWT.CHECK);
		dbc.bindValue(
				SWTObservables.observeSelection(geschwIconsAnzeigenButton),
				EMFObservables.observeValue(
						modell.getChartProperties(),
						StreckenprofilPackage.Literals.DIAGRAMM_EIGENSCHAFTEN__GESCHWINDIGKEITS_LIMIT_ICONS));

//		final Button geschwIconsAnzeigenQuerschnitteButton = toolkit
//				.createButton(
//						client,
//						"Geschwindigkeitsbeschrnkungen an Anzeigequerschnitten anzeigen",
//						SWT.CHECK);
//		dbc.bindValue(
//				SWTObservables
//						.observeSelection(geschwIconsAnzeigenQuerschnitteButton),
//				EMFObservables.observeValue(
//						modell.getChartProperties(),
//						StreckenprofilPackage.Literals.DIAGRAMM_EIGENSCHAFTEN__ANZEIGE_QUERSCHNITTE_ICONS));

	}
	
	/**
	 * Die {@link Section}, in der ausgewhlt werden kann, welche Fahrstreifen
	 * oder ob der MQ betrachtet werden soll und welche Verkehrsdaten
	 * (vKfz,qKfz,...) verwendet werden.
	 * 
	 * @param formBody
	 *            das Parent {@link Composite}.
	 * @param toolkit
	 *            das FormToolkit.
	 */
	private void createAlternativesSection(final Composite formBody,
	    final FormToolkit toolkit) {
	  final String title = "Objektbezeichnung Streckenzug";
	  final Section section = toolkit.createSection(formBody,
	      defaultSectionStyle);
	  section.setText(title);
	  final Composite client = toolkit.createComposite(section, SWT.NONE);
	  toolkit.paintBordersFor(client);
	  section.setClient(client);
	  client.setLayout(new ColumnLayout());
	  
	  final ComboViewer prognoseTypenCombo = new ComboViewer(
	      client, SWT.READ_ONLY);
    prognoseTypenCombo.getControl().setLayoutData(new ColumnLayoutData());
    final ObservableListContentProvider prognoseTypenProvider = new ObservableListContentProvider();
    prognoseTypenCombo.setContentProvider(prognoseTypenProvider);
    prognoseTypenCombo.setLabelProvider(new LabelProvider());
    //String[] ss = new String[]{"Name", "Index 0", "Index 1", "Index 2", "Index 3", "Index 4", "Index 5", "Index 6", "Index 7", "Index 8", "Index 9", "keine"};
    //prognoseTypenCombo.setInput(new WritableList(Arrays.asList(ss), String.class));
    
    prognoseTypenCombo.setInput(new WritableList(Arrays
        .asList(TypObjektBezeichnung.values()), TypObjektBezeichnung.class));

    // initialisieren
    prognoseTypenCombo.setSelection(new StructuredSelection(modell
        .getChartProperties().getTypPrognoseTyp()));
    dbc.bindValue(
        ViewersObservables.observeSingleSelection(prognoseTypenCombo),
        EMFObservables.observeValue(
            modell.getChartProperties(),
            StreckenprofilPackage.Literals.DIAGRAMM_EIGENSCHAFTEN__TYP_OBJEKT_BEZEICHNUNG));
    

    // initialisieren
//    prognoseTypenCombo.setSelection(new StructuredSelection(ss[0]));
//     
//    SWTObservables.observeSelection(prognoseTypenCombo.getCombo())
//    .addChangeListener(new IChangeListener() {
//     
//     @Override
//     public void handleChange(ChangeEvent event)
//     {
//       // TODO Auto-generated method stub      
//       int selectionIndex = prognoseTypenCombo.getCombo().getSelectionIndex();
//       ISelection selection = prognoseTypenCombo.getSelection();
//       if (selection instanceof StructuredSelection)
//       {
//        StructuredSelection ss = (StructuredSelection)selection;
//        Object firstElement = ss.getFirstElement();
//        if (firstElement instanceof String)
//        {
//          String s = (String)firstElement;
//          int index = 0;
//          if (s.equals("Name")){
//            index = -1;
//          }
//          else if (s.equals("keine")){
//           index = -2; 
//          }else{
//            index = selectionIndex-1;
//          }
//          FormEditor editor = getEditor();
//          if (editor instanceof StreckenprofilEditor)
//          {
//            StreckenprofilEditor editorStreckenprofil = (StreckenprofilEditor)editor;
//            editorStreckenprofil.setAlternativeObjektbezeichnung(index);
//          }          
//        }        
//      }
//     }
//   });
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.forms.editor.FormPage#isDirty()
	 */
	@Override
	public boolean isDirty() {
		return dirty;
	}

	/**
	 * Setzt das lokale Dirty - Flag.
	 * 
	 * @param newDirtyFlag
	 *            <code>true</code>, wenn der Editor-Inhalt sich gendert hat
	 *            und gespeichert werden kann, sonst <code>false</code>.
	 */
	private void setDirty(final boolean newDirtyFlag) {
		dirty = newDirtyFlag;
		firePropertyChange(PROP_DIRTY);
		if (getManagedForm() != null) {
			getManagedForm().dirtyStateChanged();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.ui.forms.editor.FormPage#doSave(org.eclipse.core.runtime.
	 * IProgressMonitor)
	 */
	@Override
	public void doSave(final IProgressMonitor monitor) {
		super.doSave(monitor);
		setDirty(false);
	}

	/**
	 * Erzeugt die {@link Section}, in der die Eigenschaften der Y Achsen
	 * festgelegt werden.
	 * 
	 * @param formBody
	 *            das Parent {@link Composite}.
	 * @param toolkit
	 *            das FormToolkit.
	 */
	private void createYAxisSection(final Composite formBody,
			final FormToolkit toolkit) {
		final DiagrammEigenschaften puaGanglinien = modell.getChartProperties();

		final List<AchsenBeschreibung> achsenListe = new ArrayList<AchsenBeschreibung>();
		achsenListe.add(puaGanglinien.getYAxisFahrezeugdichte());
		achsenListe.add(puaGanglinien.getYAxisLKWAnteil());
		achsenListe.add(puaGanglinien.getYAxisTypAnzahl());
		achsenListe.add(puaGanglinien.getYAxisTypeGeschwindigkeit());
		achsenListe.add(puaGanglinien.getYAxisBemessungsverkehrsdichte());
		achsenListe.add(puaGanglinien.getYAxisBemessungsverkehrsstaerke());

		final Map<AchsenBeschreibung, String> achsenNamen = new HashMap<AchsenBeschreibung, String>();
		achsenNamen.put(puaGanglinien.getYAxisFahrezeugdichte(),
				"Fahrzeugdichte [Fz/km]");
		achsenNamen.put(puaGanglinien.getYAxisBemessungsverkehrsdichte(),
				"Bemessungsverkehrsdichte [Pkw/km]");
		achsenNamen.put(puaGanglinien.getYAxisBemessungsverkehrsstaerke(),
				"Bemessungsverkehrsstrke [Pkw/h]");
		achsenNamen.put(puaGanglinien.getYAxisLKWAnteil(), "LKW Anteil [%]");
		achsenNamen.put(puaGanglinien.getYAxisTypAnzahl(),
				"Anzahl [Fz / Intervall]");
		achsenNamen.put(puaGanglinien.getYAxisTypeGeschwindigkeit(),
				"Geschwindigkeit [km / h]");

		final Composite client = newSectionAndClient(formBody, toolkit,
				"Y-Skalen",
				new GridData(SWT.FILL, SWT.FILL, false, false, 1, 1), false);
		final Composite rightClient;
		client.setLayout(new GridLayout(3, false));
		final TableViewer axesTable = new TableViewer(client);
		axesTable.getTable().setLayoutData(
				new GridData(SWT.FILL, SWT.FILL, true, false, 1, 2));
		axesTable.setContentProvider(new ArrayContentProvider());
		axesTable.setLabelProvider(new LabelProvider() {

			/*
			 * (non-Javadoc)
			 * 
			 * @see org.eclipse.jface.viewers.LabelProvider#getImage(java.lang
			 * .Object)
			 */
			@Override
			public Image getImage(final Object element) {
				ImageDescriptor result = null;
				if (element instanceof AchsenBeschreibung) {
					final Position type = ((AchsenBeschreibung) element)
							.getAchsenposition();
					switch (type) {
					case RIGHT_LITERAL:
						// result = Activator
						// .getDefault()
						// .getImageDescriptor(
						// Activator.ACHSENPOSITION_RECHTS_ICON_MARKER_URL);
						// break;
					case LEFT_LITERAL:
						result = Activator.getDefault().getImageDescriptor(
								Activator.ACHSENPOSITION_LINKS_ICON_MARKER_URL);
						break;
					default:
						result = null;
					}
				}
				final Image img = WidgetImageRegistry.getInstance(
						axesTable.getTable()).getImage(result);
				return img;
			}

			/*
			 * (non-Javadoc)
			 * 
			 * @see org.eclipse.jface.viewers.LabelProvider#getText(java.lang
			 * .Object)
			 */
			@Override
			public String getText(final Object element) {

				if (element instanceof AchsenBeschreibung) {
					return achsenNamen.get(element);
				}
				return super.getText(element);
			}

		});
		axesTable.setInput(achsenListe);

		rightClient = newPartAndClient(client, toolkit, "Skaleneinstellungen",
				new GridData(SWT.FILL, SWT.FILL, true, false, 1, 2), false);

		createAxisProperties(rightClient, toolkit, axesTable);

		// Default Selection
		axesTable.setSelection(new StructuredSelection(achsenListe.get(0)));
	}

	/**
	 * Erzeugt {@link Widget}s zum Einstellen der Achsen - Eigenschaften. Dabei
	 * wird die Selektion der bergebenen Tabelle als aktuell betrachtete Y
	 * Achse verwendet.
	 * 
	 * @param comp
	 *            das Parent {@link Composite}.
	 * @param toolkit
	 *            das FormToolkit.
	 * @param table
	 *            Tabelle, deren aktuelle Selektion bearbeitet wird.
	 */
	private void createAxisProperties(final Composite comp,
			final FormToolkit toolkit, final TableViewer table) {

		final GridLayout layout = new GridLayout(5, false);
		layout.marginHeight = 0;
		layout.marginWidth = 0;
		layout.horizontalSpacing = 10;
		comp.setLayout(layout);

		toolkit.createLabel(comp, "Diagrammseite", SWT.NONE);
		toolkit.createLabel(comp, "Minimum", SWT.NONE);
		toolkit.createLabel(comp, "Maximum", SWT.NONE);
		toolkit.createLabel(comp, "Hilfslinien", SWT.NONE);
		toolkit.createLabel(comp, "Schrittweite", SWT.NONE);

		// Diagrammseite
		final TableComboViewer v = new TableComboViewer(comp, SWT.BORDER
				| SWT.READ_ONLY);
		v.setContentProvider(new ArrayContentProvider());

		v.setInput(new Position[] { Position.LEFT_LITERAL,
				Position.RIGHT_LITERAL });
		v.setLabelProvider(new AxisSideLabelProvider(v.getTableCombo()
				.getTable()));
		v.getTableCombo().setBackground(null);
		v.getTableCombo().setVisibleItemCount(2);
		v.addSelectionChangedListener(new ISelectionChangedListener() {
			@Override
			public void selectionChanged(final SelectionChangedEvent event) {
				final ISelection s = event.getSelection();
				if (s instanceof StructuredSelection) {
					final Object firstObj = ((StructuredSelection) s)
							.getFirstElement();
					if (firstObj instanceof Position) {
						final ISelection achsenAusahl = table.getSelection();
						if (achsenAusahl instanceof StructuredSelection) {
							final Object ausgewaehlteAchse = ((StructuredSelection) achsenAusahl)
									.getFirstElement();
							if (ausgewaehlteAchse instanceof AchsenBeschreibung) {
								((AchsenBeschreibung) ausgewaehlteAchse)
										.setAchsenposition((Position) firstObj);
							}
						}
					}
				}

			}
		});

		// Minimum
		final Spinner minSpinner = new Spinner(comp, SWT.BORDER);
		minSpinner
				.setValues(0, Integer.MIN_VALUE, Integer.MAX_VALUE, 0, 1, 100);
		minSpinner.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(final SelectionEvent e) {
				final int value = minSpinner.getSelection();
				final ISelection achsenAusahl = table.getSelection();
				if (achsenAusahl instanceof StructuredSelection) {
					final Object ausgewaehlteAchse = ((StructuredSelection) achsenAusahl)
							.getFirstElement();
					if (ausgewaehlteAchse instanceof AchsenBeschreibung) {
						((AchsenBeschreibung) ausgewaehlteAchse)
								.setMinimum(value);
					}
				}

			}
		});

		// maxSpinner
		final Spinner maxSpinner = new Spinner(comp, SWT.BORDER);
		maxSpinner
				.setValues(0, Integer.MIN_VALUE, Integer.MAX_VALUE, 0, 1, 100);
		maxSpinner.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(final SelectionEvent e) {
				final int value = maxSpinner.getSelection();
				final ISelection achsenAusahl = table.getSelection();
				if (achsenAusahl instanceof StructuredSelection) {
					final Object ausgewaehlteAchse = ((StructuredSelection) achsenAusahl)
							.getFirstElement();
					if (ausgewaehlteAchse instanceof AchsenBeschreibung) {
						((AchsenBeschreibung) ausgewaehlteAchse)
								.setMaximum(value);
					}
				}
			}

		});

		// hilfsLinien
		final Button hilfslinienButton = toolkit.createButton(comp, "",
				SWT.CHECK);
		hilfslinienButton.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(final SelectionEvent e) {
				final boolean value = hilfslinienButton.getSelection();
				final ISelection achsenAusahl = table.getSelection();
				if (achsenAusahl instanceof StructuredSelection) {
					final Object ausgewaehlteAchse = ((StructuredSelection) achsenAusahl)
							.getFirstElement();
					if (ausgewaehlteAchse instanceof AchsenBeschreibung) {
						((AchsenBeschreibung) ausgewaehlteAchse)
								.setHilfslinien(value);
					}
				}
			}
		});

		// Schrittweite
		final Spinner schritteSpinner = new Spinner(comp, SWT.BORDER);
		schritteSpinner.setValues(1, 0, Integer.MAX_VALUE, 0, 1, 100);
		schritteSpinner.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(final SelectionEvent e) {
				final int value = schritteSpinner.getSelection();
				final ISelection achsenAusahl = table.getSelection();
				if (achsenAusahl instanceof StructuredSelection) {
					final Object ausgewaehlteAchse = ((StructuredSelection) achsenAusahl)
							.getFirstElement();
					if (ausgewaehlteAchse instanceof AchsenBeschreibung) {
						((AchsenBeschreibung) ausgewaehlteAchse)
								.setSchritte(value);
					}
				}
			}

		});

		table.addSelectionChangedListener(new ISelectionChangedListener() {
			@Override
			public void selectionChanged(final SelectionChangedEvent event) {
				final ISelection s = event.getSelection();
				if (s instanceof StructuredSelection) {
					final Object firstObj = ((StructuredSelection) s)
							.getFirstElement();
					if (firstObj instanceof AchsenBeschreibung) {
						v.setSelection(new StructuredSelection(
								((AchsenBeschreibung) firstObj)
										.getAchsenposition()));
						minSpinner
								.setSelection((int) ((AchsenBeschreibung) firstObj)
										.getMinimum());
						maxSpinner
								.setSelection((int) ((AchsenBeschreibung) firstObj)
										.getMaximum());
						hilfslinienButton
								.setSelection(((AchsenBeschreibung) firstObj)
										.isHilfslinien());
						schritteSpinner
								.setSelection(((AchsenBeschreibung) firstObj)
										.getSchritte());

					}
				}
			}
		});

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.forms.editor.FormPage#dispose()
	 */
	@Override
	public void dispose() {
		if (linienTypAdapter != null) {
			modell.getEMFModell().eAdapters().remove(linienTypAdapter);
		}
		if (dbc != null) {
			dbc.dispose();
		}
		super.dispose();
	}

	/**
	 * Die Methode berprft die {@link LinienEigenschaften} des Streckenprofils
	 * und gibt eines entsprechende Fehlermeldung aus, wenn z.B. die
	 * Balken-Darstellung mit der Liniendarstellung gemischt wird und damit das
	 * Chart nicht erstellt werden kann.
	 */
	private void ueberpruefungLinienEigenschaften() {

		final Map<String, List<LinienEigenschaften>> typEigenschaftenMap = new HashMap<String, List<LinienEigenschaften>>();
		for (final LinienEigenschaften leigenschaft : modell
				.getLineProperties()) {
			final TypVerkehrsDaten datenTyp = leigenschaft
					.getTypVerkehrsDaten();
			String key = null;
			if (TypVerkehrsDaten.QKFZ.equals(datenTyp)
					|| TypVerkehrsDaten.QLKW.equals(datenTyp)
					|| TypVerkehrsDaten.QPKW.equals(datenTyp)) {
				key = AbstractStreckenprofilBuilder.ANZAHL_FZ_INTERVALL;
			} else if (TypVerkehrsDaten.VKFZ.equals(datenTyp)
					|| TypVerkehrsDaten.VLKW.equals(datenTyp)
					|| TypVerkehrsDaten.VPKW.equals(datenTyp)) {
				key = AbstractStreckenprofilBuilder.GESCHWINDIGKEIT_KM_H;
			} else if ((TypVerkehrsDaten.KKFZ.equals(datenTyp)
					|| TypVerkehrsDaten.KLKW.equals(datenTyp) || TypVerkehrsDaten.KPKW
					.equals(datenTyp))) {
				key = AbstractStreckenprofilBuilder.FAHRZEUGDICHTE_FZ_KM;
			} else if (TypVerkehrsDaten.ALKW.equals(datenTyp)) {
				key = AbstractStreckenprofilBuilder.LKW_ANTEIL;

			} else if (TypVerkehrsDaten.KB.equals(datenTyp)) {
				key = AbstractStreckenprofilBuilder.BEMESSUNGSVERKEHRSDICHTE_PKW_KM;
			} else if (TypVerkehrsDaten.QB.equals(datenTyp)) {
				key = AbstractStreckenprofilBuilder.BEMESSUNGSVERKEHRSSTAERKE_PKW_H;
			}
			if (!typEigenschaftenMap.containsKey(key)) {
				typEigenschaftenMap.put(key,
						new ArrayList<LinienEigenschaften>());
			}
			typEigenschaftenMap.get(key).add(leigenschaft);
		}

		for (final String typ : typEigenschaftenMap.keySet()) {
			boolean enthaeltGestapelteBalken = false;
			boolean enthaeltLinienBzwBalken = false;
			for (final LinienEigenschaften leingeschaft : typEigenschaftenMap
					.get(typ)) {
				if (SeriesType.BAR_STACKED.equals(leingeschaft.getSeriesType())) {
					enthaeltGestapelteBalken = true;
				} else {
					enthaeltLinienBzwBalken = true;
				}
			}
			if (enthaeltGestapelteBalken && enthaeltLinienBzwBalken) {
				getManagedForm()
						.getMessageManager()
						.addMessage(
								typ,
								"Fr die Darstellung von Daten fr die Y-Achse \""
										+ typ
										+ "\" wurden sowohl \"gestapelte Balken\" wie auch \"Balken\" oder \"Linien\" gewhlt.\n Dadurch kann das Diagramm nicht gezeichnet werden. Bitte whlen sie entweder \"gestapelte Balken\" oder \"Balken\" und \"Linien\" fr Datenreihen einer Y-Achse aus.",
								null, IMessageProvider.ERROR);
			} else {
				getManagedForm().getMessageManager().removeMessage(typ);
			}
		}

	}
}
