/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/* Segment 13 Bedienung und Visualisierung, Plug-In Streckenprofil 
 * Copyright (C) 2009-2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information: 
 * BitCtrl Systems GmbH 
 * Weienfelser Strae 67 
 * 04229 Leipzig 
 * Phone: +49 341-490670 
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.streckenprofil.sts.a8.editor.simulation;

import java.util.logging.Level;
import java.util.logging.Logger;

import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;

import de.bsvrz.buv.plugin.streckenprofil.sts.a8.Activator;
import de.bsvrz.dav.daf.main.config.DynamicObject;
import de.bsvrz.dav.daf.main.config.InvalidationListener;
import de.bsvrz.dav.daf.main.config.MutableSet;
import de.bsvrz.sys.funclib.bitctrl.modell.AnmeldeException;
import de.bsvrz.sys.funclib.bitctrl.modell.Aspekt;
import de.bsvrz.sys.funclib.bitctrl.modell.DatensendeException;
import de.bsvrz.sys.funclib.bitctrl.modell.att.Zeitstempel;
import de.bsvrz.sys.funclib.bitctrl.modell.systemmodellaoe.objekte.AutarkeOrganisationsEinheit;
import de.bsvrz.sys.funclib.bitctrl.modell.tmvewsimulationglobal.attribute.AtlLoescheSimulationAnfrage;
import de.bsvrz.sys.funclib.bitctrl.modell.tmvewsimulationglobal.attribute.AttSimulationsZustand;
import de.bsvrz.sys.funclib.bitctrl.modell.tmvewsimulationglobal.konfigurationsdaten.KdSimulationsEigenschaften;
import de.bsvrz.sys.funclib.bitctrl.modell.tmvewsimulationglobal.objekte.OfflineSimulation;
import de.bsvrz.sys.funclib.bitctrl.modell.tmvewsimulationglobal.objekte.SimulationsStrecke;
import de.bsvrz.sys.funclib.bitctrl.modell.tmvewsimulationglobal.onlinedaten.OdSimulationVerwaltungAnfrage;
import de.bsvrz.sys.funclib.bitctrl.modell.tmvewsimulationglobal.onlinedaten.OdSimulationsSteuerungOffline;
import de.bsvrz.sys.funclib.bitctrl.modell.util.KappichModellUtil;
import de.bsvrz.sys.funclib.dynobj.DynObjektException;
import de.bsvrz.sys.funclib.dynobj.DynamischeObjekte;

/**
 * Job, welcher eine Simulation anhlt und ein Dynamisches Objekt vom Typ
 * <code>typ.simulationsStrecke</code> aus der Menge der SimulationsStrecken an
 * der AOE entfernt und es dann invalidiert. Irgendwelche Timingprobleme (z.B.
 * was passiert, wenn sich der
 * {@link SimulationsStreckenUebersichtContentProvider} versucht, von den
 * Parameterdaten abzumelden, nachdem das Objekt bereits invalidiert ist) sind
 * hier nicht bercksichtigt.
 * 
 * @author BitCtrl Systems GmbH, Albrecht Uhlmann/ Christian Hoesel
 * @version $Id: SimulationBeendenUndVernichtenJob.java,v 1.3 2014/04/17 15:51:15 Pittner Exp $
 * 
 */
public class SimulationBeendenUndVernichtenJob extends Job implements
		InvalidationListener {

	/**
	 * Die zu lschende Simulation.
	 */
	private final OfflineSimulation simulation;

	/**
	 * Die zu lschende {@link SimulationsStrecke}.
	 */
	private final SimulationsStrecke strecke;

	/**
	 * Kennzeichnung des {@link Job}s fr die Simulationsverwaltung.
	 */
	private final String simuZeichen = "StreckenprofilOfflineSimulation_"
			+ System.currentTimeMillis();

	/**
	 * Konstruktor erstellt nur den Namen des Jobs und merkt sich die zu
	 * lschende Simulationsstrecke.
	 * 
	 * @param simulation
	 *            die zu {@link OfflineSimulation}.
	 */
	public SimulationBeendenUndVernichtenJob(final OfflineSimulation simulation) {
		super("Simulation " + " '" + simulation.getName() + "' " + " beenden");
		setRule(new SingleJobOfTypeSchedulingRule());
		Assert.isNotNull(simulation, "Ungltige Simulationsobjekt bergeben.");
		this.simulation = simulation;

		strecke = simulation.getKdSimulationsEigenschaften().getDatum(
				KdSimulationsEigenschaften.Aspekte.Eigenschaften)
				.getSimulationsStreckenReferenz();

		((DynamicObject) simulation.getSystemObject())
				.addListenerForInvalidation(this);

	}

	@Override
	protected IStatus run(final IProgressMonitor monitor) {

		monitor.beginTask("Beende Streckenprofil-Offline-Simulation...",
				IProgressMonitor.UNKNOWN);
		IStatus status = Status.OK_STATUS;

		try {
			final Aspekt asp = OdSimulationsSteuerungOffline.Aspekte.Vorgabe;
			final OdSimulationsSteuerungOffline simSteuerung = simulation
					.getOdSimulationsSteuerungOffline();
			simSteuerung.anmeldenSender(asp);
			final OdSimulationsSteuerungOffline.Daten datum = simSteuerung
					.createDatum();
			final long wartezeit = 500L;
			int i = 0;

			while (!AttSimulationsZustand.ZUSTAND_3_STOP.equals(simSteuerung
					.getDatum(OdSimulationsSteuerungOffline.Aspekte.Zustand)
					.getSimulationsZustand())
					&& !AttSimulationsZustand.ZUSTAND_4_GELOESCHT
							.equals(simSteuerung
									.getDatum(
											OdSimulationsSteuerungOffline.Aspekte.Zustand)
									.getSimulationsZustand())) {
				if (monitor.isCanceled()) {
					return Status.CANCEL_STATUS;
				}
				Thread.sleep(wartezeit);
				i++;
				datum
						.setSimulationsZustand(AttSimulationsZustand.ZUSTAND_3_STOP);
				simSteuerung.sendeDatum(asp, datum);
			}

			while (!AttSimulationsZustand.ZUSTAND_4_GELOESCHT
					.equals(simSteuerung.getDatum(
							OdSimulationsSteuerungOffline.Aspekte.Zustand)
							.getSimulationsZustand())) {
				if (monitor.isCanceled()) {
					return Status.CANCEL_STATUS;
				}
				Thread.sleep(wartezeit);
				i++;
				datum
						.setSimulationsZustand(AttSimulationsZustand.ZUSTAND_4_GELOESCHT);
				simSteuerung.sendeDatum(asp, datum);
			}
			simSteuerung.abmeldenSender(asp);
			final AutarkeOrganisationsEinheit sv = KappichModellUtil
					.getAOE(Activator.getDefault().getObjektFactory());
			final OdSimulationVerwaltungAnfrage anfrage = sv
					.getOdSimulationVerwaltungAnfrage();
			anfrage
					.anmeldenSender(OdSimulationVerwaltungAnfrage.Aspekte.Anfrage);
			final OdSimulationVerwaltungAnfrage.Daten datum2 = anfrage
					.createDatum();
			final AtlLoescheSimulationAnfrage loeschAnfrage = new AtlLoescheSimulationAnfrage();
			loeschAnfrage.setReferenzZuLoeschendeSimulation(simulation);
			datum2.getLoescheSimulationAnfrage().add(loeschAnfrage);
			datum2.setAbsenderId(KappichModellUtil.getApplikation(Activator
					.getDefault().getObjektFactory()));
			datum2.setAbsenderZeichen(simuZeichen);
			datum2.dSetZeitstempel(new Zeitstempel());
			anfrage.sendeDatum(OdSimulationVerwaltungAnfrage.Aspekte.Anfrage,
					datum2);
			anfrage
					.abmeldenSender(OdSimulationVerwaltungAnfrage.Aspekte.Anfrage);
			status = Status.OK_STATUS;
		} catch (final AnmeldeException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		} catch (final DatensendeException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (final InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		monitor.done();
		return status;
	}

	@Override
	public void invalidObject(final DynamicObject dynamicObject) {
		if (simulation.getSystemObject().equals(dynamicObject)) {
			dynamicObject.removeListenerForInvalidation(this);
			if (strecke != null) {
				try {
					final DynamischeObjekte dynObjektVerwaltung = DynamischeObjekte
							.getInstanz(Activator.getDefault()
									.getObjektFactory().getDav());

					final DynamicObject dynobjekt = (DynamicObject) strecke
							.getSystemObject();
					final MutableSet mengeSimulationsStrecken = Activator
							.getDefault().getObjektFactory().getDav()
							.getLocalConfigurationAuthority().getMutableSet(
									"SimulationsStrecken");

					final MutableSet mengeSimulationen = Activator.getDefault()
							.getObjektFactory().getDav()
							.getLocalConfigurationAuthority().getMutableSet(
									"Simulationen");

					dynObjektVerwaltung
							.bereinigeMenge(mengeSimulationsStrecken);
					dynObjektVerwaltung.bereinigeMenge(mengeSimulationen);
					final boolean result = dynObjektVerwaltung
							.entferneObjektAusMenge(dynobjekt,
									mengeSimulationsStrecken, true);

					if (!result) {
						final String msg = "Zeichenketten.PLUGIN_SIM_FEHLER_DYNOBJ_LOESCHEN";
						Logger.getLogger(getClass().getName()).log(
								Level.WARNING, msg);
						// status = new Status(IStatus.ERROR,
						// Activator.PLUGIN_ID,
						// msg);
					}
				} catch (final DynObjektException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}

			}
		}

	}
}
