/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/* Segment 13 Bedienung und Visualisierung, Plug-In Streckenprofil 
 * Copyright (C) 2009-2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information: 
 * BitCtrl Systems GmbH 
 * Weienfelser Strae 67 
 * 04229 Leipzig 
 * Phone: +49 341-490670 
 * mailto: info@bitctrl.de
 */
package de.bsvrz.buv.plugin.streckenprofil.sts.a8.export;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.jface.viewers.BaseLabelProvider;
import org.eclipse.jface.viewers.CellLabelProvider;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.nebula.jface.gridviewer.GridTableViewer;
import org.eclipse.nebula.jface.gridviewer.GridViewerColumn;
import org.eclipse.nebula.widgets.grid.GridCellRenderer;
import org.eclipse.nebula.widgets.grid.GridColumn;
import org.eclipse.nebula.widgets.grid.GridColumnGroup;
import org.eclipse.nebula.widgets.grid.GridItem;
import org.eclipse.nebula.widgets.grid.internal.DefaultCellRenderer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.printing.Printer;
import org.eclipse.swt.widgets.Composite;

import com.bitctrl.util.ITwoDimensionalMap;

/**
 * Kapselt den {@link GridTableViewer} und ermglicht die typsichere Verwendung
 * eines Grids.
 * 
 * @param <R>
 *            "Row" - der Typ der Zeilenobjekte
 * @param <C>
 *            "Column" - der Typ der Spaltenobjekte
 * @param <D>
 *            "Data" - der Typ der Datenobjekte
 * 
 * @author BitCtrl Systems GmbH, Enrico Schnepel
 * @version $Id: MatrixGrid.java,v 1.3 2014/04/17 15:51:15 Pittner Exp $
 */
@SuppressWarnings("restriction")
public class MatrixGrid<R, C, D> extends GridTableViewer {
	/**
	 * Hilfsklasse fr die Konfiguration der Spalten im {@link MatrixGrid}.
	 * 
	 * @author BitCtrl Systems GmbH, Enrico Schnepel
	 * @version $Id: MatrixGrid.java,v 1.3 2014/04/17 15:51:15 Pittner Exp $
	 */
	public abstract static class ColumnAdvisor<C> {
		/**
		 * Wird vom Framework aufgerufen und dient zum Zwischenspeichern des
		 * Spalten-Objektes.
		 * 
		 * @param c
		 *            das Spalten-Objekt
		 */
		public abstract void setColumnObject(C c);

		/**
		 * Ermittelt gemeinsamen den Text fr gruppierte Spalten. Spalten, die
		 * den gemeinsamen Gruppentext bekommen, werden automatisch zu einer
		 * GRuppe zusammengefasst. <b>null</b> wird verwendet, wenn fr die
		 * Spalte keine Gruppierung erfolgen soll.
		 * 
		 * @return der Spalten-Gruppen-Kopf-Text (default: <b>null</b> => keine
		 *         Gruppierung fr diese Spalte)
		 */
		public String getGroupColumnText() {
			return null;
		}

		/**
		 * Ermittelt den Text fr den Spaltenkopf. <b>null</b> wird verwendet,
		 * um den Spaltenkopf ber die toString()-Methode durch das Framework
		 * ermitteln zu lassen.
		 * 
		 * @return der Spalten-Kopf-Text (default: <b>null</b> =>
		 *         [Column].toString())
		 */
		public String getColumnText() {
			return null;
		}

		/**
		 * Wie soll die Spalte ausgerichtet sein?
		 * 
		 * @return {@link SWT#LEFT}, {@link SWT#RIGHT} oder {@link SWT#CENTER}
		 */
		public int getJustification() {
			return SWT.LEFT;
		}
	}

	/**
	 * Der Labelprovider, mit dem die Zeilenkpfe per toString() aus den
	 * Zeilenobjekten ermittelt werden.
	 * 
	 * @author BitCtrl Systems GmbH, Enrico Schnepel
	 * @version $Id: MatrixGrid.java,v 1.3 2014/04/17 15:51:15 Pittner Exp $
	 */
	private final class MatrixRowHeaderLabelProvider extends CellLabelProvider {
		@Override
		public void update(final ViewerCell cell) {
			cell.setText(cell.getElement().toString());
		}
	}

	/**
	 * Factory-Klasse, mit der essentielle Objekte fr das {@link MatrixGrid}
	 * erstellt werden.
	 * 
	 * @author BitCtrl Systems GmbH, Enrico Schnepel
	 * @version $Id: MatrixGrid.java,v 1.3 2014/04/17 15:51:15 Pittner Exp $
	 */
	public abstract static class MatrixGridFactoryHelper<R, C, D> {

		/**
		 * Wenn diese Methode nicht berschrieben wird, dann wird immer
		 * toString() aufden einzelnen Zell-Objekten aufgerufen.
		 * 
		 * @param c
		 *            das Spalten-Objekt
		 * @param input
		 *            die 2D-Map
		 * @return der Zell-Renderer (default: null)
		 */
		public GridCellRenderer createCellRenderer(final C c,
				final ITwoDimensionalMap<R, C, D> input) {
			return null;
		}

		/**
		 * Erstellt den {@link ITableLabelProvider} fr ein {@link MatrixGrid}.
		 * 
		 * @param grid
		 *            das {@link MatrixGrid}
		 * @return der {@link ITableLabelProvider}
		 */
		public abstract ITableLabelProvider createTableLabelProvider(
				MatrixGrid<R, C, D> grid);

		/**
		 * Erstellt einen {@link ColumnAdvisor}.
		 * 
		 * @return der {@link ColumnAdvisor}
		 */
		public ColumnAdvisor<C> getColumnAdvisor() {
			return null;
		}
	}

	/**
	 * Zell-Renderer fr auf {@link ITwoDimensionalMap}s aufbauenden
	 * {@link MatrixGrid}s.
	 * 
	 * @param <R>
	 *            "Row" - der Typ der Zeilenobjekte
	 * @param <C>
	 *            "Column" - der Typ der Spaltenobjekte
	 * @param <D>
	 *            "Data" - der Typ der Datenobjekte
	 * 
	 * @author BitCtrl Systems GmbH, Enrico Schnepel
	 * @version $Id: MatrixGrid.java,v 1.3 2014/04/17 15:51:15 Pittner Exp $
	 */
	public abstract static class MatrixGridColumnRenderer<R, C, D> extends
			DefaultCellRenderer {
		/**
		 * Die {@link ITwoDimensionalMap}, in welcher die Daten der Tabelle
		 * hinterlegt sind.
		 */
		private final ITwoDimensionalMap<R, C, D> input;

		/**
		 * Konstruktor.
		 * 
		 * @param index
		 *            das Spaltenobjekt
		 * @param input
		 *            die 2D-Map
		 */
		public MatrixGridColumnRenderer(final C index,
				final ITwoDimensionalMap<R, C, D> input) {
			this.index = index;
			this.input = input;
		}

		/**
		 * Der Index der Spalte, die dieser Renderer malt.
		 */
		private final C index;

		@SuppressWarnings("unchecked")
		@Override
		public final void paint(final GC gc, final Object value) {
			if (!(gc.getDevice() instanceof Printer)) {
				super.paint(gc, value);
			}
			final GridItem item = (GridItem) value;
			final D d = input.get((R) item.getData(), index);
			paintCell(gc, getBounds(), d);
		}

		/**
		 * Malt eine Zelle neu.
		 * 
		 * @param gc
		 *            der GraphicsContext.
		 * @param bounds
		 *            die Begrenzung der Zelle.
		 * @param d
		 *            der Inhalt der Zelle.
		 */
		abstract void paintCell(GC gc, Rectangle bounds, D d);
	}

	/**
	 * Liste aller Spalter des {@link MatrixGrid}.
	 */
	private final List<GridViewerColumn> columns = new ArrayList<GridViewerColumn>();

	/**
	 * Die bei der Konfiguration des {@link MatrixGrid} assistierende Factory.
	 */
	private final MatrixGridFactoryHelper<R, C, D> factory;

	/**
	 * Schlssel zur Speicherung des Spaltenschlssels via
	 * {@link org.eclipse.swt.widgets.Item#setData(String, Object)
	 * Item.setData(String, Object)}.
	 */
	public static final String COLUMN_KEY = "COLUMN_KEY";

	/**
	 * Der Zell-Labelprovider.
	 */
	private final CellLabelProvider cellLabelProvider = new CellLabelProvider() {
		@SuppressWarnings("unchecked")
		@Override
		public void update(final ViewerCell cell) {
			final MatrixLabelProvider<R, C, D> labelProvider;
			labelProvider = (MatrixLabelProvider<R, C, D>) getLabelProvider();
			cell.setText(labelProvider.getColumnText(cell.getElement(), cell
					.getColumnIndex()));
			cell.setImage(labelProvider.getColumnImage(cell.getElement(), cell
					.getColumnIndex()));
		}
	};

	/**
	 * {@link IStructuredContentProvider} welcher die Matrix aus einer
	 * {@link ITwoDimensionalMap} befllt.
	 * 
	 * @author BitCtrl Systems GmbH, Enrico Schnepel
	 * @version $Id: MatrixGrid.java,v 1.3 2014/04/17 15:51:15 Pittner Exp $
	 */
	public class MatrixGridContentProvider implements
			IStructuredContentProvider {

		/**
		 * Das zu befllende {@link MatrixGrid}.
		 */
		private MatrixGrid<R, C, D> mg;

		@Override
		@SuppressWarnings("unchecked")
		public void inputChanged(final Viewer viewer, final Object oldInput,
				final Object newInput) {
			dispose();
			if (null != newInput) {

				final ColumnAdvisor<C> columnAdvisor = factory
						.getColumnAdvisor();
				mg = (MatrixGrid) viewer;
				final ITwoDimensionalMap<R, C, D> input = (ITwoDimensionalMap<R, C, D>) newInput;
				int i = 0;
				final Map<String, GridColumnGroup> mappings = new HashMap<String, GridColumnGroup>();
				for (final C c : input.getColumnKeys()) {
					GridViewerColumn col = null;
					String columnText = null;
					if (null != columnAdvisor) {
						columnAdvisor.setColumnObject(c);
						final String groupText = columnAdvisor
								.getGroupColumnText();
						if (null != groupText) {
							GridColumnGroup group = mappings.get(groupText);
							if (null == group) {
								group = new GridColumnGroup(mg.getGrid(),
										SWT.NONE);
								group.setText(groupText);
								if (groupText.contains("\n")) {
									group.setHeaderWordWrap(true);
								}
								mappings.put(groupText, group);
							}
							col = new GridViewerColumn(
									mg,
									new GridColumn(
											group,
											columnAdvisor.getJustification()
													& (SWT.LEFT | SWT.RIGHT | SWT.CENTER)));
						}
						columnText = columnAdvisor.getColumnText();
					}
					if (null == col) {
						col = new GridViewerColumn(mg, new GridColumn(mg
								.getGrid(), SWT.NONE));
					}
					col.getColumn().setData(COLUMN_KEY, c);
					if (null == columnText) {
						columnText = c.toString();
					}
					col.getColumn().setText(columnText);
					if (columnText.contains("\n")) {
						col.getColumn().setHeaderWordWrap(true);
						col.getColumn().setWidth(50);
					}
					final GridCellRenderer renderer = factory
							.createCellRenderer(c, input);
					if (renderer != null) {
						col.getColumn().setCellRenderer(renderer);
					}
					col.setLabelProvider(cellLabelProvider);
					col.getColumn().pack();
					columns.add(col);
					i++;
				}
			}
		}

		@Override
		@SuppressWarnings("unchecked")
		public Object[] getElements(final Object inputElement) {
			final ITwoDimensionalMap<R, C, D> input = (ITwoDimensionalMap<R, C, D>) inputElement;

			return input.getRowKeys().toArray();
		}

		@Override
		public void dispose() {
			if (null != mg) {
				for (final GridColumn c : mg.getGrid().getColumns()) {
					c.dispose();
				}
				for (final GridColumnGroup c : mg.getGrid().getColumnGroups()) {
					c.dispose();
				}
				mg.getGrid().removeAll();
			}
		}
	}

	/**
	 * Abstrakte Klasse zur Implementierung eines
	 * {@link org.eclipse.jface.viewers.LabelProvider LabelProvider}s fr auf
	 * {@link ITwoDimensionalMap}s aufbauenden {@link MatrixGrid}s.
	 * 
	 * @param <R>
	 *            "Row" - der Typ der Zeilenobjekte
	 * @param <C>
	 *            "Column" - der Typ der Spaltenobjekte
	 * @param <D>
	 *            "Data" - der Typ der Datenobjekte
	 * 
	 * @author BitCtrl Systems GmbH, Enrico Schnepel
	 * @version $Id: MatrixGrid.java,v 1.3 2014/04/17 15:51:15 Pittner Exp $
	 */
	public abstract static class MatrixLabelProvider<R, C, D> extends
			BaseLabelProvider implements ITableLabelProvider {

		/**
		 * Das {@link MatrixGrid}.
		 */
		private final MatrixGrid<R, C, D> grid;

		/**
		 * Konstruktor.
		 * 
		 * @param grid
		 *            das {@link MatrixGrid} zu diesem
		 *            {@link org.eclipse.jface.viewers.LabelProvider
		 *            LabelProvider}
		 */
		public MatrixLabelProvider(final MatrixGrid<R, C, D> grid) {
			this.grid = grid;
		}

		@SuppressWarnings("unchecked")
		@Override
		public final Image getColumnImage(final Object element,
				final int columnIndex) {
			final C c = getColumnKey(columnIndex);
			return getCellImage((R) element, c, getCellElement(element, c));
		}

		@SuppressWarnings("unchecked")
		@Override
		public final String getColumnText(final Object element,
				final int columnIndex) {
			final C c = getColumnKey(columnIndex);
			return getCellText((R) element, c, getCellElement(element, c));
		}

		/**
		 * Liefert das Zell-Objekt, fr ein Objekt in einer Spalte.
		 * 
		 * @param element
		 *            der Objekt, das Dargestellt werden soll.
		 * @param c
		 *            die Spalte.
		 * @return Das Zell-Objekt.
		 */
		@SuppressWarnings("unchecked")
		private D getCellElement(final Object element, final C c) {
			return grid.getInput().get((R) element, c);
		}

		/**
		 * Liefert die Spalte mit entsprechendem Index.
		 * 
		 * @param columnIndex
		 *            der Index, der gesuchten Spalte.
		 * @return die Spalte.
		 */
		@SuppressWarnings("unchecked")
		private C getColumnKey(final int columnIndex) {
			try {
				return (C) grid.getGrid().getColumns()[columnIndex]
						.getData(COLUMN_KEY);
			} catch (final NullPointerException e) {
				return null;
			}
		}

		/**
		 * Muss implementiert werden und gibt das Bild fr eine bestimmte Zelle
		 * zurck.
		 * 
		 * @param row
		 *            die Zeile
		 * @param column
		 *            die Spalte
		 * @param data
		 *            das Datenelement
		 * @return das Bild
		 */
		protected abstract Image getCellImage(R row, C column, D data);

		/**
		 * Muss implementiert werden und gibt den Text fr eine bestimmte Zelle
		 * zurck.
		 * 
		 * @param row
		 *            die Zeile
		 * @param column
		 *            die Spalte
		 * @param data
		 *            das Datenelement
		 * @return der Zellentext
		 */
		protected abstract String getCellText(R row, C column, D data);
	}

	/**
	 * Konstruktor.
	 * 
	 * @param parent
	 *            das bergeordnete UI-Element
	 * @param style
	 *            der SWT-Style
	 * @param factory
	 *            die Hilfs-Factory.
	 */
	public MatrixGrid(final Composite parent, final int style,
			final MatrixGridFactoryHelper<R, C, D> factory) {
		super(parent, style);
		this.factory = factory;
		getGrid().setHeaderVisible(true);
		getGrid().setRowHeaderVisible(true);
		setRowHeaderLabelProvider(new MatrixRowHeaderLabelProvider());
		setContentProvider(new MatrixGridContentProvider());
		super.setLabelProvider(factory.createTableLabelProvider(this));
	}

	/**
	 * Setzt die Eingabe auf die bergebene {@link ITwoDimensionalMap}.
	 * 
	 * @param input
	 *            die 2D-Map.
	 */
	public void setInput(final ITwoDimensionalMap<R, C, D> input) {
		super.setInput(input);
	}

	@SuppressWarnings("unchecked")
	@Override
	public ITwoDimensionalMap<R, C, D> getInput() {
		return (ITwoDimensionalMap<R, C, D>) super.getInput();
	}

	/**
	 * Gibt die bei der Konfiguration assistierende Factory zurck.
	 * 
	 * @return die Factory
	 */
	public MatrixGridFactoryHelper<R, C, D> getFactory() {
		return factory;
	}

	/**
	 * Gibt die Columns des Grids zurck.
	 * 
	 * @return die Columns
	 */
	public GridViewerColumn[] getColumns() {
		return columns.toArray(new GridViewerColumn[columns.size()]);
	}
}
