/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.bsvrz.buv.plugin.streckenprofil.sts.a8.histdata;

import java.io.IOException;

import org.eclipse.core.runtime.Assert;

import de.bsvrz.dav.daf.main.archive.ArchiveData;

/**
 * Datenpuffer, der jeweils den aktuellen Wert eines Satzes von Archiv-Ergenisdatenstmen speichert
 * 
 * @author Sans
 *
 */
public class ArchivDatenPuffer
{
  
  private ArchiveData[] daten;
  private IArchivAnfrage archivAnfrage;
  
  /**
   * 
   * @param archivAnfrage
   * @throws InterruptedException
   * @throws IOException
   */
  public ArchivDatenPuffer(IArchivAnfrage archivAnfrage) throws InterruptedException, IOException {
    
    this.archivAnfrage = archivAnfrage;
    daten = new ArchiveData[archivAnfrage.getErgebnis().length];
    
    for (int i = 0; i < daten.length; i++) {
        daten[i] = archivAnfrage.getErgebnis()[i].take();
    }  
  }

  /**
   * Bestimmt die Lnge des Datenpuffers
   * 
   * @return Lnge des Datenpuffers
   */
  public int getLaenge() {
    return daten.length;
  }

  /**
   * Holt aktuellen Datensatz eines Ergebnis-Streams, ohne ihn aus dem Puffer zu enfernen
   * 
   * @param index Index des Streams
   * @return aktuelles Datum des Streams - null, falls der Stream keine Daten mehr liefert
   */
  public ArchiveData get(int index) {
    Assert.isTrue(index >=0 && index < getLaenge(), "Index muss zwischen 0 und " + getLaenge() + " liegen");
    
    return daten[index];
  }
  
  /**
   * Holt aktuellen Datensatz eines Ergebnis-Streams, und entfernt ihn aus dem Puffer - es wird der 
   * nchste Datensatz aus dem Ergebnisstream in den Datenpuffer geladen
   * 
   * @param index Index des Streams
   * @return aktuelles Datum des Streams - null, falls der Stream keine Daten mehr liefert
   * @throws IOException 
   * @throws InterruptedException  
   */
  public ArchiveData take(int index) throws InterruptedException, IOException {
    Assert.isTrue(index >=0 && index < getLaenge(), "Index muss zwischen 0 und " + getLaenge() + " liegen");
    
    ArchiveData result = daten[index];
    
    if (result != null) {
      daten[index] = archivAnfrage.getErgebnis()[index].take();
    }
    
    return result;
  }
  
  /**
   * Bestimmt den Index der Daten mit dem minimalen Archivzeitstempel innerhalb des Datenpuffers
   * 
   * @return Index der Daten mit dem minimalen Archivzeitstempel oder -1, falls kein Stream mehr Daten liefert
   */
  public int getIndexMinimalerArchivZeitstempel() {
    
    int result = -1;
    
    for (int i = 0; i < daten.length; i++) {
      if (daten[i] != null) {
        if (result == -1 || daten[i].getArchiveTime() < daten[result].getArchiveTime()) {
          result = i;
        }  
      }
    }
    
    return result;
  }
}
