/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.bsvrz.buv.plugin.streckenprofil.sts.a8.wizard;

import java.io.File;
import java.io.FileInputStream;
import java.util.Map;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.Resource.Diagnostic;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchWizard;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.plugin.streckenprofil.sts.a8.Activator;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.editor.StreckenprofilEditor;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.editor.StreckenprofilEditorInput;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.StreckenprofilEditorModel;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.wizardpages.StreckenprofilDateImportWizardPage;

/**
 * Wizard zum Importieren von {@link StreckenprofilEditorModel}.
 * 
 * @author BitCtrl Systems GmbH, Hoesel
 * @version $Id: StreckenprofilDateiImportWizard.java,v 1.3 2014/04/17 15:51:20 Pittner Exp $
 */
public class StreckenprofilDateiImportWizard extends Wizard implements
		IWorkbenchWizard {

	private StreckenprofilDateImportWizardPage importPage;

	public StreckenprofilDateiImportWizard() {
		setWindowTitle("Streckenprofil importieren");
	}

	@Override
	public void addPages() {
		importPage = new StreckenprofilDateImportWizardPage();
		addPage(importPage);
	}

	@Override
	public boolean performFinish() {
		boolean result = true;
		final String fileName = importPage.getFilesource();
		if (fileName == null || fileName.isEmpty()) {
			final String meldung = "Es wurde keine gltiger Dateiname als Quelldatei angegeben!";
			Activator
					.getDefault()
					.getLog()
					.log(new Status(IStatus.ERROR, Activator.PLUGIN_ID, meldung));
			MessageDialog.openError(getShell(),
					"Streckenprofil kann nicht importiert werden", meldung);
			result = false;
		} else {

			final Resource.Factory.Registry reg = Resource.Factory.Registry.INSTANCE;
			final Map<String, Object> m = reg.getExtensionToFactoryMap();
			Object object = m.get(Activator.FILE_EXTENSION);
			if (null == object) {
				object = new XMIResourceFactoryImpl();
				m.put(Activator.FILE_EXTENSION, object);
			}

			Resource resource = null;
			try {
				final File feile = new File(fileName);
				final FileInputStream is = new FileInputStream(feile);

				final ResourceSet resSet = new ResourceSetImpl();
				resource = resSet.createResource(URI
						.createURI("bitctrl://import."
								+ Activator.FILE_EXTENSION));
				resource.load(is, null);

			} catch (final Exception ex) {
				MessageDialog.openError(getShell(), "Datei " + fileName
						+ " kann nicht gelesen werden.",
						ex.getLocalizedMessage());
				Activator
						.getDefault()
						.getLog()
						.log(new Status(IStatus.ERROR, Activator.PLUGIN_ID, ex
								.getLocalizedMessage()));
			} finally {
				if (resource != null) {
					for (final Diagnostic d : resource.getErrors()) {
						Activator
								.getDefault()
								.getLog()
								.log(new Status(IStatus.ERROR,
										Activator.PLUGIN_ID, d.getMessage()));
					}
					for (final Diagnostic d : resource.getWarnings()) {
						Activator
								.getDefault()
								.getLog()
								.log(new Status(IStatus.WARNING,
										Activator.PLUGIN_ID, d.getMessage()));
					}
					final EList<EObject> objListe = resource.getContents();

					try {
						final EObject erstesObject = objListe.iterator().next();
						if (erstesObject instanceof StreckenprofilEditorModel) {
							final StreckenprofilEditorInput input = new StreckenprofilEditorInput();
							input.setModel((StreckenprofilEditorModel) erstesObject);

							final StreckenprofilEditor editor = (StreckenprofilEditor) PlatformUI
									.getWorkbench()
									.getActiveWorkbenchWindow()
									.getActivePage()
									.openEditor(input,
											StreckenprofilEditor.EDITOR_ID);
						}
					} catch (final Exception e) {
						Activator
								.getDefault()
								.getLog()
								.log(new Status(
										IStatus.ERROR,
										Activator.PLUGIN_ID,
										"Die Datei konnte nicht gelesen werden",
										e));

					}
				} else {
					Activator
							.getDefault()
							.getLog()
							.log(new Status(IStatus.ERROR, Activator.PLUGIN_ID,
									"Kein Resource-Provider fr die Deserialisation gefunden!"));
				}
			}
		}
		return result;
	}

	@Override
	public void init(final IWorkbench workbench,
			final IStructuredSelection selection) {
		// TODO Auto-generated method stub

	}
}
