/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.ursacheneinheit.naessewarnung;

/**
 * Enum zur Kodierung der Nssestufen
 * @author ObertM
 */
public enum NaesseStufe
{
//	/**
//	 * Nssestufe konnte nicht ermittelt werden.
//	 */
//  NICHT_ERMITTELBAR           ("nicht ermittelbar",           -1),
//  
//  /**
//   * Nssestufe darf bei den aktuellen FBZ und NS Zustnden nicht ermittelt werden
//   */
//  DARF_NICHT_ERMITTELT_WERDEN ("Darf nicht ermittelt werden", -2),

  /**
   * Nicht verwendet (wird bei Parameterdialog bentigt)
   */
  NV                          ("Nicht verwendet",             -3),

  /**
   * Fahrbahn ist trocken.
   */
  TROCKEN                     ("Trocken",                      0),
  
  /**
   * Fahrbahnzustand ist nass1.
   */
  NASS1                       ("Nass 1",                       1),
  
  /**
   * Fahrbahnzustand ist nass2.
   */
  NASS2                       ("Nass 2",                       2),
  
  /**
   * Fahrbahnzustand ist nass3.
   */
  NASS3                       ("Nass 3",                       3),
  
  /**
   * Fahrbahnzustand ist nass4.
   */
  NASS4                       ("Nass 4",                       4),
  
  /**
   * Platzregen.
   */
  PLATZREGEN                  ("Platzregen",                   5);
  
  /**
   * Kennung
   */
  private String _kennung;
  
  /**
   * Kodierung im DaV
   */
  private int _kodierung;
  
  /**
   * Konstruktor der Klasse
   * @param kennung Kennung
   * @param kodierung Kodierung
   */
  NaesseStufe (String kennung, int kodierung)
  {
  	_kennung = kennung;
  	
  	_kodierung = kodierung;
  }
 
  /**
   * Methode liefert die Nssestufe zu einer Kodierung
   * @param kodierung Kodierung
   * @return Nssestufe
   */
  public static NaesseStufe getStufe (int kodierung)
  {
  	for (NaesseStufe s : NaesseStufe.values())
  		if (s.getKodierung() == kodierung)
  			return s;
  	
  	return null;
  }

  /**
   * Methode liefert die Nssestufe zu einer Kennung
   * @param kennung Kennung
   * @return Nssestufe
   */
  public static NaesseStufe getStufe (String kennung)
  {
  	for (NaesseStufe s : NaesseStufe.values())
  		if (s.getKennung().equals(kennung))
  			return s;
  	
  	return null;
  }

	/**
   * Methode liefert kennung zurck
   * @return kennung
   */
  public String getKennung()
  {
  	return _kennung;
  }

	/**
   * Methode liefert kodierung zurck
   * @return kodierung
   */
  public int getKodierung()
  {
  	return _kodierung;
  }
}
