/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.ursacheneinheit.sbaueparam;

import java.util.Observable;
import java.util.Observer;

import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Spinner;

import de.bsvrz.buv.plugin.ursacheneinheit.zeitdauer.MyCompositeZeitDauer;

/**
 * TabellenModell zum TabelViewer
 * 
 * @author Krause
 *
 */
public class TableModel extends Observable implements Observer
{ 
  private Composite _attribut;
  private Composite _wert;
  
  public TableModel(Composite attribut, Composite wert)
  {
    this._attribut = attribut;   
    this._wert = wert;    
  }
  
  public Composite getAttribut()
  {
    return _attribut;
  }
 
  public Composite getWert()
  {
    return _wert;
  }
  
  public void setWert( Composite wert )
  {
    this._wert = wert;
  }
  
  public String getSelectedWert()
  {
    Control control = _wert.getChildren()[0];
    if ( control instanceof Spinner )
    {
      Spinner sp = (Spinner)control;
      int spSel = sp.getSelection();
      return String.valueOf(spSel);
    }
    else if ( control instanceof Combo )
    {
      Combo cb = (Combo)control;
      String cbSel = cb.getItem( cb.getSelectionIndex() );
      return cbSel;
    }
    else if ( control instanceof MyCompositeZeitDauer )
    {      
        MyCompositeZeitDauer czr = (MyCompositeZeitDauer)control;
        Long l = czr.getZeitDauer();        
        return l.toString();        
    }
    else if ( control instanceof Label )
    {
      Label im = (Label)control;
      String imSel = (String)im.getText();
      return imSel;
    }
    else
      return "";
  }

  public void setSelectedWert(String selWert)
  {
    Control control = _wert.getChildren()[0];
    if ( control instanceof Spinner )
    {
      Spinner sp = (Spinner)control;
      try
      {
        sp.setSelection(Integer.valueOf( selWert ));
      }
      catch (NumberFormatException e)
      {
        e.printStackTrace();
      }
    }
    else if ( control instanceof Combo )
    {
      Combo cb = (Combo)control;
      for (int k = 0; k < cb.getItemCount(); k++)
      {
        if (selWert.equals(cb.getItem( k )))
          cb.select(k);      
      }
    }
    else if ( control instanceof MyCompositeZeitDauer )
    {
      MyCompositeZeitDauer mcz = (MyCompositeZeitDauer)control;
      
      Long l = new Long(selWert);
      
      mcz.setZeitDauerInBezugAufAnfangsZeitpunkt(l);
    
    }
    
      

  }

  @Override
  public void update(Observable o, Object arg)
  {
    fireEvent();
  }

  public void fireEvent()
  {
    setChanged();
    notifyObservers();
  }

}
