/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.ursacheneinheit.seitenstreifenfreigabebelegung;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;

import de.bsvrz.buv.plugin.ursacheneinheit.tools.Tools;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.ReferenceValue;

/**
 * Darstellung des Fensters zur Parameternderung mit den Messquerschnitten und den Algorithmus spezifischen
 * Parametern
 * @author ObertM
 */
public class CompositeParameterGesamt extends Composite
{
	/**
	 * Schalter fr Debugausgaben
	 */
  private static final boolean _isDebug = false;
  
  /**
   * Ergebnisdaten vom DaV fr die Attributgruppe Messquerschnitte
   */
  private ResultData _resultDataMq;

  /**
   * Ergebnisdaten vom DaV fr die Attributgruppe Algorithmus spezifische Parameter
   */
  private ResultData _resultDataAlg;

  /**
   * Liste mit den mglichen Messquerschnitte
   */
  private List<String> _listeMoeglicherObjekte;
  
  /**
   * Hauptcomposite
   */
  private Composite _composite;

  /**
   * Comparator fr die Messquerschnitte
   */
  private MyComparator _comp = new MyComparator();
  
  /**
   * Gruppe mit den Messquerschnitten
   */
  private Group _groupMessQuerschnitte;

  /**
   * Compoiste mit den Messquerschnitten
   */
  private Composite _compositeMessQuerschnitte = null;

  /**
   * Composite mit den Buttons zum Verschieben der Messquerschnitte
   */
  private Composite _composite1 = null;

  /**
   * Liste mit den mglichen Messquerschnitten
   */
  private org.eclipse.swt.widgets.List _list1 = null;

  /**
   * Liste mit den angewhlten Messquerschnitten
   */
  private org.eclipse.swt.widgets.List _list2 = null;
  
  /**
   * Button zum Hinzufgen eines Messquerschnitts
   */
  private Button _addButton = null;

  /**
   * Button zum Entfernen eines Messquerschnitts
   */
  private Button _subButton = null;

	/**
	 * Compoiste mit den Algorithmus spezifischen Parametern
	 */
	private CompositeParameterAlgorithmus[] _compositeAlg;

	/**
	 * Merker ob die Versorgung der Messquerschnitte angeboten wird oder nicht
	 */
	private boolean _auswahlMQ;

	private Group _groupParameter;

	/**
	 * Konstruktor der Klasse
	 * @param parent Aufrufendes Composite
	 * @param style Stil
	 * @param resultDataMq Ergebnis Messquerschnitte
	 * @param resultDataAlg Ergebnis Algorithmusspezifische Parameter
	 * @param listMoeglicheMessQuerschnitte Liste mit den mglichen Messquerschnitten
	 * @param auswahlMQ Merker ob die Versorgung der Messquerschnitte angeboten wird oder nicht
	 */
  CompositeParameterGesamt(Composite parent, int style, ResultData resultDataMq, ResultData resultDataAlg, List<String> listMoeglicheMessQuerschnitte, boolean auswahlMQ)
  {
    super(parent, style);
    
    this._resultDataMq = resultDataMq;
    this._resultDataAlg = resultDataAlg;
    this._auswahlMQ = auswahlMQ;
    
    this._listeMoeglicherObjekte = listMoeglicheMessQuerschnitte;
    initialize();
  }
  
  /**
   * Fhrt die Basis-Initialisierung des Composites durch
   */
  private void initialize()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.heightHint = -1;
    gridData1.widthHint = -1;
    gridData1.horizontalIndent = 0;
    gridData1.verticalAlignment = GridData.FILL;
    setLayout(new GridLayout(1, false));
    setLayoutData(gridData1);
    GridLayout gridLayout = new GridLayout();
    gridLayout.makeColumnsEqualWidth = false;
    gridLayout.numColumns = 1;
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.heightHint = -1;
    gridData2.widthHint = -1;
    gridData2.horizontalIndent = 0;
    gridData2.verticalAlignment = GridData.FILL;

    _composite = new Composite(this, SWT.NONE);
    _composite.setLayoutData(gridData2);
    _composite.setLayout(gridLayout);

    if (_auswahlMQ)
      createMessquerschnittAuswahl();
    
    createCompositeTab();
    
  	_compositeAlg = new CompositeParameterAlgorithmus [1];

    updateData ();
  }

  /**
   * Updatefunktion zur bernahme von neuen Daten aus dem DaV
	 * @param resultDataMq Ergebnis Messquerschnitte
	 * @param resultDataAlg Ergebnis Algorithmusspezifische Parameter
   */
  public synchronized void update (ResultData resultDataMq, ResultData resultDataAlg)
  {
    this._resultDataMq = resultDataMq;
    this._resultDataAlg = resultDataAlg;

    updateData ();
  }
  
  /**
   * Updatefunktion zur bernahme von neuen Daten aus dem DaV
   * @param resultDataAlg Ergebnis Algorithmusspezifische Parameter
   */
  public synchronized void updateDataAlg (ResultData resultDataAlg)
  {
    this._resultDataAlg = resultDataAlg;

    updataDataAlgorithmus ();
  }
  
  /**
   * Updatefunktion zum Aktualisieren der Daten
   */
  private void updateData ()
  {
    if (_resultDataAlg != null)
    {
    	updataDataAlgorithmus ();
    }

    if (_resultDataMq != null)
    {
    	if (_auswahlMQ)
        updateDataMessQuerschnitt();
    }
  }
  
  /**
   * Updatefunktion fr die Algorithmus spezifischen Parameter
   */
  private void updataDataAlgorithmus()
  {
    Composite c = (Composite) _groupParameter;
    Control[] children = c.getChildren();
    CompositeParameterAlgorithmus compPar = null;

    Data data = null;
    if (_resultDataAlg.hasData())
      data = _resultDataAlg.getData();
    
    if ( children.length == 0 )
    {
      compPar = new CompositeParameterAlgorithmus ( c, SWT.None, _resultDataAlg, data );
      _compositeAlg [0] = compPar;
      
      compPar.layout( true );
      c.layout( true );
      
      compPar.getTvParameter().refresh();
    }
    else
    {
      compPar = (CompositeParameterAlgorithmus) children[0];
      compPar.update ( _resultDataAlg, data );
      
      compPar.layout( true );
      c.layout( true );

      compPar.getTvParameter().refresh();
    }
  }

  /**
   * Updatefunktion ber die Messquerschnitte
   */
	private void updateDataMessQuerschnitt()
  {
		_list1.removeAll();
    for ( String str : _listeMoeglicherObjekte )
    {
      _list1.add( str );
    }
    
    String[] items = _list1.getItems();
    java.util.Arrays.sort( items, _comp );
    _list1.setItems( items );

    String[] items2 = _list2.getItems();
    java.util.Arrays.sort( items2, _comp );
    _list2.setItems( items2 );

    if ( _resultDataMq != null)
    {
      if ( _resultDataMq.hasData() )
      {
      	Data data = _resultDataMq.getData();
      	
        Array arrayMQ = data.getArray( "MessQuerschnitt" );
        for (int k = 0; k < arrayMQ.getLength(); k++)
        {
          String text = "";
          ReferenceValue rV = arrayMQ.getReferenceValue( k );
          if ( rV != null )
            text = Tools.getName(rV.getSystemObject());

          _list1.remove( text );
          _list2.add( text );

        }
      }
    }
  }

  /**
   * Methode erteugt die Darstellung der Auswahl der Messquerschnitte
   * 
   */
  private void createMessquerschnittAuswahl()
  {
    FillLayout fillLayout2 = new FillLayout();
    fillLayout2.type = org.eclipse.swt.SWT.HORIZONTAL;
    
    RowLayout rowLayout3 = new RowLayout();
    rowLayout3.type = org.eclipse.swt.SWT.VERTICAL;
    rowLayout3.justify = false;
    rowLayout3.wrap = false;
    rowLayout3.pack = false;
    rowLayout3.spacing = 25;
    rowLayout3.fill = false;
    
    GridData gridData22 = new GridData();
    gridData22.horizontalAlignment = GridData.CENTER;
    gridData22.grabExcessVerticalSpace = false;
    gridData22.grabExcessHorizontalSpace = false;
    gridData22.verticalAlignment = GridData.CENTER;

    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = false;
    gridData2.heightHint = 110;
    gridData2.widthHint = -1;
    gridData2.horizontalIndent = 0;
    gridData2.verticalAlignment = GridData.CENTER;

    _groupMessQuerschnitte = new Group(_composite, SWT.NONE);
    _groupMessQuerschnitte.setText("Messquerschnitte");
    _groupMessQuerschnitte.setLayout(fillLayout2);
    _groupMessQuerschnitte.setLayoutData(gridData2);
    
    GridData gridData41 = new GridData();
    gridData41.horizontalAlignment = GridData.FILL;
    gridData41.grabExcessHorizontalSpace = true;
    gridData41.grabExcessVerticalSpace = true;
    gridData41.verticalAlignment = GridData.FILL;
    
    GridData gridData21 = new GridData();
    gridData21.horizontalAlignment = GridData.CENTER;
    gridData21.verticalAlignment = GridData.CENTER;
    
    GridData gridData12 = new GridData();
    gridData12.horizontalAlignment = GridData.CENTER;
    gridData12.verticalAlignment = GridData.CENTER;
    
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.FILL;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.verticalAlignment = GridData.FILL;
    gridData3.grabExcessVerticalSpace = true;

    //GridLayout mit Anzahl Spalten
    GridLayout gridLayout = new GridLayout();
    gridLayout.numColumns = 4;
    gridLayout.marginHeight = 5;
    gridLayout.marginWidth = 5;
    
    _compositeMessQuerschnitte = new Composite(_groupMessQuerschnitte, SWT.NONE);
    Label label = new Label(_compositeMessQuerschnitte, SWT.NONE);
    label.setText("mgliche Messquerschnitte");
    label.setLayoutData(gridData12);
    _compositeMessQuerschnitte.setLayout(gridLayout);
    new Label(_compositeMessQuerschnitte, SWT.NONE);
    
    Label label1 = new Label(_compositeMessQuerschnitte, SWT.NONE);
    label1.setText("zugeordnete Messquerschnitte");
    label1.setLayoutData(gridData21);
    new Label(_compositeMessQuerschnitte, SWT.NONE);
 
    _list1 = new org.eclipse.swt.widgets.List(_compositeMessQuerschnitte, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL);
    _list1.setLayoutData(gridData3);

    _composite1 = new Composite(_compositeMessQuerschnitte, SWT.NONE);
    _composite1.setLayoutData(gridData22);
    _composite1.setLayout(rowLayout3);
    _addButton = new Button(_composite1, SWT.NONE);
    _addButton.setText("<");
    _addButton.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        String[] selection = _list2.getSelection();
        for (String string : selection)
        {
          _list1.add(string);
          _list2.remove(string);
        }

        String[] items = _list1.getItems();
        java.util.Arrays.sort(items, _comp);
        _list1.setItems(items);

        String[] items2 = _list2.getItems();
        java.util.Arrays.sort(items2, _comp);
        _list2.setItems(items2);

      }
    });
    _subButton = new Button(_composite1, SWT.NONE);
    _subButton.setText(">");
    _subButton.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        String[] selection = _list1.getSelection();
        for (String string : selection)
        {
          _list2.add(string);
          _list1.remove(string);
        }

        String[] items = _list2.getItems();
        java.util.Arrays.sort(items, _comp);
        _list2.setItems(items);

        String[] items2 = _list1.getItems();
        java.util.Arrays.sort(items2, _comp);
        _list1.setItems(items2);

      }
    });
    _list2 = new org.eclipse.swt.widgets.List(_compositeMessQuerschnitte, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL);
    _list2.setLayoutData(gridData41);

    _list1.setSize(_list1.getSize().x, 100);
  }

  /**
   * Interne Klasse zum Vergleich
   * 
   * @author Krause
   * 
   */
  private class MyComparator implements Comparator<String>
  {

    @Override
    public int compare(String o1, String o2)
    {
      int ret = 0;
      
      //String-Sortierung
      if ( o1 != null )
        ret = o1.compareTo( o2 );
     
//      //Integer-Sortierung
//      Integer i1 = new Integer(o1);
//      Integer i2 = new Integer(o2);
//
//      if (i1 < i2)
//        ret = -1;
//      else if (i1 == i2)
//        ret = 0;
//      else if (i1 > i2)
//        ret = 1;
      
      return ret;
      
    }

  }

  /**
   * Methode zum Anlegen des Composite fr die Tabelle
   */
  private void createCompositeTab()
  {
    GridData gridDataGroup = new GridData();
    gridDataGroup.horizontalAlignment = GridData.FILL;
    gridDataGroup.grabExcessHorizontalSpace = true;
    gridDataGroup.grabExcessVerticalSpace = true;
    gridDataGroup.verticalAlignment = GridData.FILL;

  	GridData gridDataTabFolder = new GridData();
    gridDataTabFolder.horizontalAlignment = GridData.FILL;
    gridDataTabFolder.grabExcessHorizontalSpace = true;
    gridDataTabFolder.grabExcessVerticalSpace = true;
    gridDataTabFolder.verticalAlignment = GridData.FILL;

    _groupParameter = new Group(_composite, SWT.NONE);
    _groupParameter.setText("Parameter");
    _groupParameter.setLayout(new GridLayout(1, false));
    _groupParameter.setLayoutData(gridDataGroup);
  }

  public void setList1( List<String> list )
  {
    if ( list != null && list.size() > 0 )
    {
      _list1.removeAll();
      for (String str : list)
      {
        _list1.add( str );
      }
    }
  }
  
  /**
   * Methode liefert die Liste der noch mglichen Messquerschnitte
   * @return Liste der noch mglichen Messquerschnitte
   */
  public org.eclipse.swt.widgets.List getList1()
  {
    return _list1;
  }

  /**
   * Methode liefert die Liste der angewhlten Messquerschnitte
   * @return Liste der angewhlten Messquerschnitte
   */
  public org.eclipse.swt.widgets.List getList2()
  {
    return _list2;
  }

  /**
   * Methode liefert die einzelnen Composite fr die Algorithmus spezifischen Parameter zurck
   * @return Composites
   */
  public CompositeParameterAlgorithmus[] getCompositeAlg()
  {
  	return _compositeAlg;
  }
  
}
