/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.ursacheneinheit.verwaltung;

import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Die Klasse {@link EintragSystemObjekt} reprsentiert die Datenstze fr SystemObjekte, die 
 * in der Liste im Dialog {@link SystemObjektTabelleComp} dargestellt werden. <br>
 * Implementiert das Interface {@link Cloneable}.
 * 
 * @author Krause
 * 
 */
public class EintragSystemObjekt implements Cloneable
{
  /**
   * Name
   */
  private String _name = null;

  /**
   * Info
   */
  private String _info = null;

  /**
   * SystemObject
   */
  private SystemObject _systemObjekt = null;

  /**
   * Konstruktor zur Erzeugung eines Datensatzes zum SystemObjekt.
   * 
   * @param name
   *          Name des SystemObjektes.
   * @param info
   *          Info zum SystemObjekt.
   */
  public EintragSystemObjekt(String name, String info, SystemObject systemObjekt)
  {
    this._name = name;
    this._info = info;
    this._systemObjekt = systemObjekt;
  }

  /**
   * Liefert den Namen des SystemObjektes.
   * 
   * @return Name des SystemObjektes.
   */
  public String getName()
  {
    return _name;
  }

  /**
   * Setzt den Namen des SystemObjektes.
   * 
   * @param name
   *          Name des SystemObjektes.
   */
  public void setName(String name)
  {
    _name = name;
  }

  /**
   * Liefert die Info zum SystemObjekt.
   * 
   * @return Info zum SystemObjekt.
   */
  public String getInfo()
  {
    return _info;
  }

  /**
   * Setzt die Info zum SystemObjekt.
   * 
   * @param info
   *          Info zum SystemObjekt.
   */
  public void setInfo(String info)
  {
    _info = info;
  }

  /**
   * Liefert das SystemObjekt zum SystemObjekt.
   * 
   * @return SystemObjekt zum SystemObjekt.
   */
  public SystemObject getSystemObjekt()
  {
    return _systemObjekt;
  }

  /**
   * Setzt das SystemObjekt zum SystemObjekt.
   * 
   * @param systemObjekt
   *          SystemObjekt zum SystemObjekt.
   */
  public void setSystemObjekt(SystemObject systemObjekt)
  {
    _systemObjekt = systemObjekt;
  }

  /**
   * Liefert einen Clone der Instanz dieser Klasse.
   * 
   * @return Clone der Instanz dieser Klasse.
   */
  public EintragSystemObjekt clone()
  {
    EintragSystemObjekt eintrag = new EintragSystemObjekt(_name, _info, _systemObjekt);
    return eintrag;
  }
  
  /**
   * Liefert die Gleichheit zu einem anderen Objekt
   * 
   * @param obj
   *          Objekt zum Vergleich
   */
  @Override
  public boolean equals(Object obj)
  {
    boolean flagge = false;
    EintragSystemObjekt eintrag = (EintragSystemObjekt)obj;
    if (_systemObjekt.equals(eintrag.getSystemObjekt()))
      flagge = true;
    return flagge;

  }

}
