/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.ursacheneinheit.verwaltung;


import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.PlatformUI;

//import de.bsvrz.buv.plugin.automatikprogramme.verwaltung.VerwaltungsSicht.BaseComparator;
//import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Die Klasse {@link SystemObjektTabelleComp} reprsentiert das Composite zur Darstellung 
 * einer Tabelle von SystemObjekten zu einem bestimmten ObjektTyp 
 * und den Button Neu... sowie Lschen fr diese Tabelle.  
 * Die Tabelle enthlt die Eigenschaften Name und Info der SystemObjekte.
 * 
 * @author Krause
 */
public class SystemObjektTabelleComp extends Composite
{
  /**
   * Der Debugger
   */
//  private Debug _debug = Debug.getLogger();

  private String _dialogTitel = null;

  private String _bezEingabe = null;

  private String _objektTyp = null;

  private Composite _composite = null;

  private Composite _compositeTabelle = null;

  private Composite _compositeRechts = null;

  private Button buttonNeu = null;

  private Button buttonBearbeiten = null;

  /**
   * TableViewer fr SystemObjekte des Objekttyps
   */
  private TableViewer _tvTeilSystemObjekte = null;

  /**
   * Liste der SystemObjektEintrge
   */
  private List<EintragSystemObjekt> _lEintragSystemObjekte = null;

  /**
   * Speicher fr ein SystemObjekt.
   */
  private EintragSystemObjekt _eintragSystemObjekt;

  /**
   * Konstruktor zur Erzeugung eines Composites zur Darstellung einer Tabelle
   * von Systemobjekten mit den Eigenschaften Name und Info
   * 
   * @param parent
   *          Eltern-Group, das fr die Darstellung benutzt werden soll.
   * @param dialogTitel
   *          Titel des SystemObjektDefinitionsDialoges
   * @param bezEingabe
   *          Bezeichnung des Eingabefeldes des SystemObjektDefinitionsDialoges
   * @param objektTyp
   *          Objekttyp der SystemObjekte, die im SystemObjektDefinitionsDialog
   *          angezeigt werden sollen
   */

  public SystemObjektTabelleComp( Group parent, String dialogTitel, String bezEingabe, String objektTyp )
  {
    super( parent, SWT.NONE );
    _composite = this;

    _dialogTitel = dialogTitel;
    _bezEingabe = bezEingabe;
    _objektTyp = objektTyp;

    _lEintragSystemObjekte = new ArrayList<EintragSystemObjekt>();

    init();

  }

  /**
   * This method initializes _composite
   */
  private void init()
  {
    GridLayout gridLayout = new GridLayout();
    gridLayout.numColumns = 2;
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    _composite.setLayoutData( gridData );
    createCompositeTabelleVk();
    _composite.setLayout( gridLayout );
    createCompositeRechts();
  }

  /**
   * This method initializes _compositeTabelle
   * 
   */
  private void createCompositeTabelleVk()
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    _compositeTabelle = new Composite( _composite, SWT.NONE );
    _compositeTabelle.setLayout( new FillLayout() );
    _compositeTabelle.setLayoutData( gridData );
    erzeugeTabelle();
  }

  /**
   * This method initializes _compositeRechts
   * 
   */
  private void createCompositeRechts()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.verticalAlignment = GridData.CENTER;
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.verticalAlignment = GridData.CENTER;
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.FILL;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.grabExcessVerticalSpace = true;
    gridData3.verticalAlignment = GridData.FILL;
    _compositeRechts = new Composite( _composite, SWT.NONE );
    _compositeRechts.setLayout( new GridLayout() );
    _compositeRechts.setLayoutData( gridData3 );
    buttonNeu = new Button( _compositeRechts, SWT.NONE );
    buttonNeu.setText( "Neu..." );
    buttonNeu.setLayoutData( gridData2 );
    buttonNeu.addMouseListener( new MouseAdapter()
    {

      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );

        SystemObjektDefinitionsDialog systemObjektDefinitionsDialog = new SystemObjektDefinitionsDialog( PlatformUI
            .getWorkbench().getDisplay().getActiveShell() );
        EintragSystemObjekt eintrag = systemObjektDefinitionsDialog.oeffne( _dialogTitel, _bezEingabe, _objektTyp );

        if ( eintrag != null )
        {
          if ( !_lEintragSystemObjekte.contains( eintrag ) )
            _lEintragSystemObjekte.add( eintrag );

          _tvTeilSystemObjekte.setInput( _lEintragSystemObjekte );
        }
      }

    } );

    buttonBearbeiten = new Button( _compositeRechts, SWT.NONE );
    buttonBearbeiten.setText( "Lschen" );
    buttonBearbeiten.setLayoutData( gridData1 );
    buttonBearbeiten.addMouseListener( new MouseAdapter()
    {

      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );

        if ( _eintragSystemObjekt != null )
        {

          _lEintragSystemObjekte.remove( _eintragSystemObjekt );

          _tvTeilSystemObjekte.setInput( _lEintragSystemObjekte );

        }
        else
        {
          MessageBox mb = new MessageBox( new Shell( Display.getCurrent() ), SWT.ICON_INFORMATION | SWT.OK );
          mb.setText( "Fehler" );
          mb.setMessage( "Es muss ein EintragUrsachenEinheit ausgewhlt werden!" );
          mb.open();
          return;
        }

      }

    } );
  }

  private void erzeugeTabelle()
  {
    _tvTeilSystemObjekte = new TableViewer( _compositeTabelle, SWT.BORDER | SWT.FULL_SELECTION | SWT.H_SCROLL
        | SWT.V_SCROLL );

    final TableColumn tc0 = new TableColumn( _tvTeilSystemObjekte.getTable(), SWT.LEFT | SWT.Move );
    tc0.setText( "Name" );
    tc0.setWidth( 180 );
    tc0.setMoveable( true );
    tc0.addListener( SWT.Selection, new Listener()
    {

      private boolean sortFlag = false;

      public void handleEvent( Event event )
      {
        sortFlag = !sortFlag;
        Table table = _tvTeilSystemObjekte.getTable();

//        BaseComparator[][] comparables = new BaseComparator[table.getColumnCount()][table.getItemCount()];

        for (int i = 0; i < table.getItemCount(); i++)
        {
          TableItem item = table.getItem( i );

          String[] strings = new String[table.getColumnCount()];
          for (int j = 0; j < table.getColumnCount(); j++)
          {
            String text = item.getText( j );
            strings[j] = text;
          }

          // comparables[colPos][i] = new StringComparator(strings, i, colPos);
        }

        // sortTable(table, table.getItems(), colPos, comparables, sortFlag);
      }

    } );

    TableColumn tc1 = new TableColumn( _tvTeilSystemObjekte.getTable(), SWT.LEFT | SWT.Move );
    tc1.setText( "Info" );
    tc1.setWidth( 180 );
    tc1.setMoveable( true );
    tc1.addListener( SWT.Selection, new Listener()
    {

      private boolean sortFlag = false;

      public void handleEvent( Event event )
      {
        sortFlag = !sortFlag;
        Table table = _tvTeilSystemObjekte.getTable();

//        BaseComparator[][] comparables = new BaseComparator[table.getColumnCount()][table.getItemCount()];

        for (int i = 0; i < table.getItemCount(); i++)
        {
          TableItem item = table.getItem( i );

          String[] strings = new String[table.getColumnCount()];
          for (int j = 0; j < table.getColumnCount(); j++)
          {
            String text = item.getText( j );
            strings[j] = text;
          }

          // comparables[colPos][i] = new StringComparator(strings, i, colPos);
        }

        // sortTable(table, table.getItems(), colPos, comparables, sortFlag);
      }

    } );

    _tvTeilSystemObjekte.getTable().setHeaderVisible( true );
    _tvTeilSystemObjekte.getTable().setLinesVisible( true );

    _tvTeilSystemObjekte.setContentProvider( new TabelleInhaltsVersorger() );
    _tvTeilSystemObjekte.setLabelProvider( new TabelleDarstellungsVersorger() );

    _tvTeilSystemObjekte.addSelectionChangedListener( new ISelectionChangedListener()
    {
      public void selectionChanged( SelectionChangedEvent event )
      {
        IStructuredSelection selection = (IStructuredSelection) event.getSelection();
        Object firstElement = selection.getFirstElement();

        if ( firstElement instanceof EintragSystemObjekt )
        {
          _eintragSystemObjekt = (EintragSystemObjekt) firstElement;
        }
        else
          _eintragSystemObjekt = null;

      }

    } );

  }

  /**
   * Liefert den TableViewer der SystemObjektTabelle
   * 
   * @return TableViewer der SystemObjektTabelle
   */
  public TableViewer getTableViewer()
  {
    return _tvTeilSystemObjekte;
  }

  /**
   * Liefert die Liste der SystemObjektEintrge
   * 
   * @return Liste der SystemObjektEintrge
   */
  public List<EintragSystemObjekt> getListeEintraege()
  {
    return _lEintragSystemObjekte;
  }
  
  /**
   * Setzt die Liste der SystemObjektEintrge
   * 
   * @param eintragSystemObjekte
   *          Liste der SystemObjektEintrge
   */
  public void setListeEintraege(List<EintragSystemObjekt> eintragSystemObjekte)
  {
    _lEintragSystemObjekte = eintragSystemObjekte;
  }
  
  private class TabelleInhaltsVersorger implements IStructuredContentProvider
  {
    public Object[] getElements( Object inputElement )
    {

      List list = (List) inputElement;

      Object[] objects = new Object[list.size()];

      for (int i = 0; i < list.size(); i++)
      {
        objects[i] = list.get( i );

      }

      return objects;

    }

    public void dispose()
    {
    }

    public void inputChanged( Viewer viewer, Object oldInput, Object newInput )
    {
    }

  }

  private class TabelleDarstellungsVersorger implements ITableLabelProvider
  {

    public Image getColumnImage( Object element, int columnIndex )
    {
      return null;
    }

    public String getColumnText( Object object, int columnIndex )
    {
      String returnString = null;

      switch (columnIndex)
        {
        case 0:
          returnString = ((EintragSystemObjekt) object).getName();
          break;
        case 1:
          returnString = ((EintragSystemObjekt) object).getInfo();
          break;

        default:
          returnString = "";
        }

      return returnString;
    }

    public void addListener( ILabelProviderListener listener )
    {
    }

    public void dispose()
    {
    }

    public boolean isLabelProperty( Object element, String property )
    {
      return false;
    }

    public void removeListener( ILabelProviderListener listener )
    {
    }

  }

}
