/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.ursacheneinheit.verwaltung;

import java.util.List;

import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.plugin.ursacheneinheit.Activator;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * In diesem Dialog sind die aktuell definierten Eintrge zu den Automatikprogrammen dargestellt. 
 * Fr jeden EintragUrsachenEinheit ist die Eigenschaft Name und Pid dargestellt.
 * 
 * <br>
 * <br>
 * Die Klasse erfllt die technische Anforderungen des Segments Bedienung und Visualisierung (BUV): <br>
 * <ul>
 * <li>TAnf-Vi-BW-C3-9</li>
 * </ul>
 * 
 * @author Krause
 * 
 */
public class VerwaltungsSicht extends ViewPart implements IUrsachenEinheitListener
{

  public static final String ID = "de.bsvrz.buv.plugin.automatikprogramme.verwaltung.verwaltungsSicht";

  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();

  /**
   * Verbindung zum DaV
   */
  private DaVKommunikation _daVKommunikation = null;

  private Composite _composite = null;
  
  private CTabFolder _tabFolder = null;

  private Composite _compositeBottom = null;

  private Button buttonSchliessen = null;

  private Button buttonHilfe = null;
  
  private ConfigurationObject _selKonfigObj= null;

  @Override
  public void createPartControl(Composite parent)
  {
   
    _composite = new Composite(parent, SWT.NONE);
    _composite.setLayout(new GridLayout());

    //Fr die F1 Taste
    PlatformUI.getWorkbench().getHelpSystem().setHelp( _composite, Activator.PLUGIN_ID + ".verwaltungUrsachenEinheit" );
       
    // Darstellung aufbauen
    createCompositeTab();
    createCompositeBot();

    _daVKommunikation = DaVKommunikation.getInstanz();

    _daVKommunikation.hinzufuegenUrsachenEinheitListener( this );
    
    // Objekte fr die Reiter ermitteln
    List<SystemObject> allKonfigObjekte = _daVKommunikation.getAllKonfigObjekte();
       
    // Reiter erstellen
    erzeugeTabItems( allKonfigObjekte );

  }

  @Override
  public void setFocus()
  {
    //Fr die F1 Taste
    if (_composite != null)
      _composite.setFocus();
  }
  
  @Override
  public void dispose()
  {

    _daVKommunikation.abmelden( _selKonfigObj );
    _daVKommunikation.entferneUrsachenEinheitListener(this);
    
    super.dispose();    
    
  }

  /**
   * Schliet den Verwaltungsdialog {@link VerwaltungsSicht}.
   * 
   */
  public void aktionAbbrechen()
  {
    getViewSite().getPage().hideView(VerwaltungsSicht.this);
  }

  /**
   * Zeigt die Hilfe zum Verwaltungsdialog {@link VerwaltungsSicht} an.
   * 
   */
  private void aktionHilfe()
  {
    String contextId = Activator.PLUGIN_ID + ".verwaltungUrsachenEinheit";
    _debug.fine("ContextID: " + contextId);
    PlatformUI.getWorkbench().getHelpSystem().displayHelp(contextId);
  }

  /**
   * This method initializes _tabFolder (Bereich ber der ButtonLeiste)
   * 
   */
  private void createCompositeTab()
  {
    
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    gridData.verticalAlignment = GridData.FILL;

    _tabFolder = new CTabFolder(_composite, SWT.NONE);
    _tabFolder.setLayout(new GridLayout());
    _tabFolder.setLayoutData(gridData);
    _tabFolder.addSelectionListener(new SelectionListener() {

      @Override
      public void widgetDefaultSelected(SelectionEvent e)
      {
        // TODO Auto-generated method stub
      }

      @Override
      public void widgetSelected(SelectionEvent e)
      {       
        CTabFolder folder = (CTabFolder)e.getSource();        
        CTabItem itemSel = folder.getSelection();
        
        Composite cSel = (Composite)itemSel.getControl();
        Control[] children = cSel.getChildren();
        
        VerwaltungsTabelleComp tabComp = null;
        
        if ( children.length == 0 )
        {
          GridData gridData = new GridData();
          gridData.horizontalAlignment = GridData.FILL;
          gridData.grabExcessHorizontalSpace = true;
          gridData.grabExcessVerticalSpace = true;
          gridData.verticalAlignment = GridData.FILL;
          
          _selKonfigObj = (ConfigurationObject)folder.getData( itemSel.getText() );

          tabComp = new VerwaltungsTabelleComp( cSel, SWT.NONE, _selKonfigObj );
          tabComp.setLayout( new GridLayout() );
          tabComp.setLayoutData( gridData );
        }
        else
        {
          tabComp = (VerwaltungsTabelleComp)cSel.getChildren()[0];      
          TableViewer tViewer = tabComp.getTvUrsachenEinheit();
          tViewer.refresh();
        }
      }

    });
  }
  
  /**
   * Reiter fr die Liste der Objekte erstellen
   * 
   * @param list
   */
  public void erzeugeTabItems( List<SystemObject> list )
  {

    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.verticalAlignment = GridData.FILL;
    
    // Sortierung zu Beginn nicht gewnscht, da Tabelle ber 
    // die Spaltenberschriften sortiert werden
    
//    Map<String, SystemObject> treeMap = new TreeMap<String, SystemObject>();
//    for (SystemObject so : list)
//    {
//      treeMap.put(so.getName(), so);
//    }
//
//    List<SystemObject> sortList = new ArrayList<SystemObject>();
//    sortList.addAll(treeMap.values());
    
    for (SystemObject so : list)
    {
      if ( _selKonfigObj == null )
        _selKonfigObj = (ConfigurationObject) so;

      CTabItem tabItem = new CTabItem( _tabFolder, SWT.NONE );

      String tabName = so.getName();
      tabItem.setText( tabName );
      tabItem.setData( tabName, so );

      Composite c1 = new Composite( _tabFolder, SWT.NONE );
      c1.setLayout( new GridLayout() );
      c1.setLayoutData( gridData2 );

      tabItem.setControl( c1 );

    }
    _tabFolder.setSelection( 0 );
    
    //da durch setSelection in diesem Fall der SelectionListener nicht aktiv wird, muss er selektierte Tab hier gesetzt werden
    CTabItem itemSel = _tabFolder.getSelection();
    Composite cSel = (Composite)itemSel.getControl();
    Control[] children = cSel.getChildren();
    
    if ( children.length == 0 )
    {
      String soName = itemSel.getText();
      VerwaltungsTabelleComp tabComp = new VerwaltungsTabelleComp( cSel, SWT.NONE, (SystemObject)itemSel.getData( soName ) );
      tabComp.setLayout( new GridLayout() );
      tabComp.setLayoutData( gridData2 );
    }
    
  }
  
  /**
   * This method initializes _compositeBottom
   * 
   */
  private void createCompositeBot()
  {
    RowLayout rowLayout = new RowLayout();
    rowLayout.pack = false;
    rowLayout.fill = false;
    rowLayout.wrap = false;
    GridData gridData4 = new GridData();
    gridData4.horizontalAlignment = GridData.END;
    gridData4.verticalAlignment = GridData.CENTER;
    _compositeBottom = new Composite(_composite, SWT.NONE);
    _compositeBottom.setLayoutData(gridData4);
    _compositeBottom.setLayout(rowLayout);
    buttonSchliessen = new Button(_compositeBottom, SWT.NONE);
    buttonSchliessen.setText("Schlieen");
    buttonSchliessen.addMouseListener(new org.eclipse.swt.events.MouseAdapter() {
      public void mouseDown(org.eclipse.swt.events.MouseEvent e)
      {
        //dispose wird ausgefhrt
        aktionAbbrechen();
      }
    });

    buttonHilfe = new Button(_compositeBottom, SWT.NONE);
    buttonHilfe.setText("Hilfe");
    buttonHilfe.addMouseListener(new org.eclipse.swt.events.MouseAdapter() {
      public void mouseDown(org.eclipse.swt.events.MouseEvent e)
      {

        aktionHilfe();
      }
    });
  }


  @Override
  public void ursachenEinheitenEingang(ListeUrsachenEinheiten e)
  {
//    System.out.println("Tabelle fllen");
    
    CTabItem itemSel = _tabFolder.getSelection();
    Composite cSel = (Composite) itemSel.getControl();
    VerwaltungsTabelleComp compParSel = (VerwaltungsTabelleComp) cSel.getChildren()[0];
        
    compParSel.bestimmeGefilterteUrsachenEinheiten();
  }
  
}
