package de.bsvrz.sba.plugin.actions.parameter.verwaltung;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.sba.plugin.actions.Activator;
import de.bsvrz.sba.plugin.actions.parameter.IParameterAction;

/**
 * Liefert eine bersicht ber alle "Actions", die ber den Erweiterungspunkt
 * <code>de.bsvrz.sba.plugin.action.parameterAction</code> contributed wurden.
 * 
 * @author HofmannJ
 * 
 */
public class ViewParameterActionRegistry extends ViewPart
{

	/**
	 * Table zur Darstellung der Tabelle
	 */
	private Table _tableParameter = null;
	
	/**
	 * Tableviewer
	 */
	private TableViewer _tvParameter = null;

	@Override
	public void createPartControl(Composite parent)
	{
		Composite top = new Composite(parent, SWT.NONE);

		createParameterTable(top);
		createToolBarActions();

		LinkedList<ParameterActionRegistryElement> parameterActionElements = ParameterActionRegistry
				.getInstance().getParameterActionElements();

		_tvParameter.setInput(parameterActionElements);

		/*
		 * Layout
		 */
		GridLayout glTop = new GridLayout(1, false);
		top.setLayout(glTop);

		GridData gdTable = new GridData();
		gdTable.grabExcessHorizontalSpace = true;
		gdTable.horizontalAlignment = GridData.FILL;
		gdTable.grabExcessVerticalSpace = true;
		gdTable.verticalAlignment = GridData.FILL;
		_tableParameter.setLayoutData(gdTable);
	}

	/**
	 * Erzeugt die {@link Table} und den {@link TableViewer}, mit deren Hilfe
	 * sich die registrierten Actions darstellen lassen.
	 * 
	 * @param parent
	 *          Eltern-Composite, auf dessen Basis die Darstellung erfolgen soll.
	 * @return Liefert die erzeugte Tabelle zurck.
	 */
	private Table createParameterTable(Composite parent)
	{
		_tableParameter = new Table(parent, SWT.FULL_SELECTION | SWT.MULTI
				| SWT.H_SCROLL | SWT.V_SCROLL);
		_tableParameter.setHeaderVisible(true);
		_tableParameter.setLinesVisible(true);

		TableColumn tcId = new TableColumn(_tableParameter, SWT.LEFT);
		tcId.setText("ID:");
		tcId.setWidth(150);

		TableColumn tcPlugin = new TableColumn(_tableParameter, SWT.LEFT);
		tcPlugin.setText("Von Plug-In:");
		tcPlugin.setWidth(150);

		TableColumn tcUsers = new TableColumn(_tableParameter, SWT.LEFT);
		tcUsers.setText("User IDs:");
		tcUsers.setWidth(300);
		
		TableColumn tcParameter = new TableColumn(_tableParameter, SWT.LEFT);
		tcParameter.setText("Parameter:");
		tcParameter.setWidth(400);

		_tvParameter = new TableViewer(_tableParameter);
		_tvParameter.setContentProvider(new CP());
		_tvParameter.setLabelProvider(new LP());
		_tvParameter.setSorter(new ViewerSorter());

		return _tableParameter;
	}

	/**
	 * Erzeugt die Actions fr die Toolbar dieser View
	 */
	private void createToolBarActions()
	{
		IToolBarManager tbm = getViewSite().getActionBars().getToolBarManager();

		Action actionRunSelection = new Action()
		{
			public void run()
			{
				actionRunSelectedAction();
			}
		};
		actionRunSelection.setText("Ausfhren");
		actionRunSelection
				.setToolTipText("Fhrt die Action hinter dem selekierten Element aus.");
		actionRunSelection.setImageDescriptor(Activator
				.getImageDescriptor("icons/exec_16x16.png"));

		tbm.add(actionRunSelection);
	}

	/**
	 * Fhrt die Action hinter einem Element im TableViewer aus, sofern vorhanden.
	 */
	private void actionRunSelectedAction()
	{
		ISelection selection = _tvParameter.getSelection();
		if (selection instanceof IStructuredSelection)
		{
			IStructuredSelection ss = (IStructuredSelection) selection;
			Object firstElement = ss.getFirstElement();
			if (firstElement != null)
			{
				if (firstElement instanceof ParameterActionRegistryElement)
				{
					ParameterActionRegistryElement ele = (ParameterActionRegistryElement) firstElement;
					IParameterAction parameterAction = ele.getParameterAction();
					if (parameterAction != null)
					{
						MessageBox mb = new MessageBox(PlatformUI.getWorkbench().getDisplay().getActiveShell(), SWT.ICON_INFORMATION | SWT.OK);
						mb.setText("Keine Funktion");
						mb.setMessage("Hinter dieser Action ist z.Zt. keine Funktion hinterlegt.");
						mb.open();
					}
				}
			}
		}
	}

	/**
	 * Content Provider fr den TabelViewer.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class CP implements IStructuredContentProvider
	{
		@Override
		public Object[] getElements(Object inputElement)
		{
			Object[] re = null;

			if (inputElement instanceof List)
			{
				@SuppressWarnings("unchecked")
				List list = (List) inputElement;
				re = list.toArray();
			}

			return re;
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
		{
		}

	}

	/**
	 * Labelprovider fr den Tabelviewer.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class LP implements ITableLabelProvider
	{

		@Override
		public Image getColumnImage(Object element, int columnIndex)
		{
			return null;
		}

		@Override
		public String getColumnText(Object element, int columnIndex)
		{
			String re = null;

			if (element != null)
			{
				if (element instanceof ParameterActionRegistryElement)
				{
					ParameterActionRegistryElement par = (ParameterActionRegistryElement) element;

					if (columnIndex == 0)
					{
						re = par.getParameterAction().getParameterActionId();
					}
					else if (columnIndex == 1)
					{
						re = par.getContributor();
					}
					else if (columnIndex == 2)
					{
						re = "";
						Iterator<String> it = par.getUserIds().iterator();
						while (it.hasNext())
						{
							re = re + "[" + it.next() + "]";
							if (it.hasNext())
							{
								re = re + ", ";
							}
						}
					}
					else if(columnIndex == 3)
					{
						IParameterAction parameterAction = par.getParameterAction();
						if(parameterAction != null)
						{							
							LinkedList<String> parameterIds = parameterAction.getParameterIds();
							if((parameterIds != null) && (parameterIds.size() > 0))
							{
								re = "";
								Iterator<String> itParId = parameterIds.iterator();
								while(itParId.hasNext())
								{
									String parameterId = itParId.next();
									re = re + "[ID: " + parameterId + "]";
									if(itParId.hasNext())
									{
										re = re + ", ";
									}
								}
							}							
							else
							{
								re = "Keine Parameter vorhanden!";
							}
						}
					}
					else
					{
						re = "UNDEFINIERT";
					}
				}
			}
			else
			{
				re = "NULL";
			}

			return re;
		}

		@Override
		public void addListener(ILabelProviderListener listener)
		{
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public boolean isLabelProperty(Object element, String property)
		{
			return false;
		}

		@Override
		public void removeListener(ILabelProviderListener listener)
		{
		}

	}

	@Override
	public void setFocus()
	{
	}

}
