/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.sba.plugin.dynobj.configurationAreas;

import java.util.Collection;
import java.util.LinkedList;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.TextArray;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.ObjectTimeSpecification;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

/**
 * Liefert eine Liste der im DaV vorhandenen Konfigurationsbereiche.
 * 
 * @author HofmannJ
 * 
 */
public class ConfigurationAreas
{
	/**
	 * Verbindung zum DaV
	 */
	private ClientDavInterface _connection = null;

	/**
	 * Konstruktor.
	 * 
	 * @param connection
	 *          Verbindung zum DaV.
	 */
	public ConfigurationAreas(ClientDavInterface connection)
	{
		this._connection = connection;
	}

	/**
	 * Liefert eine Liste der Konfigurationsbereiche, welche im DaV vorhanden
	 * sind.
	 * 
	 * @return Liste der Konfigurationsbereiche
	 */
	public LinkedList<SystemObject> getKonfigurationsBereiche()
	{
		LinkedList<SystemObject> re = new LinkedList<SystemObject>();

		if (_connection != null)
		{
			DataModel dataModel = _connection.getDataModel();
			SystemObjectType type = dataModel.getType("typ.konfigurationsBereich");

			LinkedList<SystemObjectType> types = new LinkedList<SystemObjectType>();
			types.add(type);

			Collection<SystemObject> systemObjects = dataModel.getObjects(null,
					types, ObjectTimeSpecification.valid());

			re.addAll(systemObjects);
		}

		return re;
	}

	/**
	 * Bestimmt den Default-Konfigurationsbereich der Autarken
	 * Organisationseinheit.
	 * 
	 * @return Default-Konfigurationsbereich der Autarken Organisationseinheit
	 *         oder <code>null</code>.
	 */
	public String bestimmeDefaultBereich()
	{
		Data data = _connection.getDataModel().getConfigurationAuthority()
				.getConfigurationData(
						_connection.getDataModel().getAttributeGroup(
								"atg.konfigurationsVerantwortlicherEigenschaften"));

		if (data != null)
		{
			TextArray array = data.getTextArray("defaultBereich");

			if (array.getLength() == 1)
			{
				return _connection.getDataModel().getObject(array.getText(0)).getPid();
			}
		}

		return null;
	}
}
