/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.sba.plugin.dynobj.configurationAreas.dialogs;

import java.util.Collection;
import java.util.LinkedList;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.dynobj.configurationAreas.compsites.CompositeConfigurationAreaChooser;
import de.bsvrz.sba.plugin.dynobj.configurationAreas.compsites.EnumSelection;
import de.bsvrz.sba.plugin.sonstiges.shell.ShellPosition;

/**
 * Dialog zur Auswahl von Konfigurationsbereichen. Es werden alle im DaV
 * verfgbaren Konfigurationsbereiche dargestellt.
 * 
 * @author HofmannJ
 * 
 */
public class DialogConfigurationAreaChooser extends Dialog
{
	/**
	 * Verbindung zum DaV
	 */
	private ClientDavInterface _connection = null;

	/**
	 * Liste der {@link SystemObject}s der selektierten Konfigurationsbereiche
	 */
	private LinkedList<SystemObject> _result = null;

	/**
	 * Obere Gruppe zur Darstellung der Konfigurationsbereiche
	 */
	private Group _groupUp = null;

	/**
	 * Untere Gruppe
	 */
	private Composite _compositeDown = null;

	/**
	 * Konfigurationsbereichswhler
	 */
	private CompositeConfigurationAreaChooser _compositeConfigurationAreaChooser = null;

	/**
	 * Shell
	 */
	private Shell _shell = null;

	/**
	 * Konstruktor
	 * 
	 * @param parent
	 *          Eltern-{@link Shell}, auf deren Basis die Darstellung erfolgen
	 *          soll.
	 * @param style
	 *          SWT-Stil
	 * @param connection
	 *          Verbindung zum DaV
	 */
	public DialogConfigurationAreaChooser(Shell parent, int style,
			ClientDavInterface connection)
	{
		super(parent, style);
		this._connection = connection;
	}

	/**
	 * Konstruktor
	 * 
	 * @param parent
	 *          Eltern-{@link Shell}, auf deren Basis die Darstellung erfolgen
	 *          soll.
	 * @param connection
	 *          Verbindung zum DaV
	 */
	public DialogConfigurationAreaChooser(Shell parent,
			ClientDavInterface connection)
	{
		this(parent, 0, connection);
	}

	/**
	 * ffnet die Darstellung. bernimmt eine Liste von Strings mit den Pid der
	 * KonfigurationsBereiche, welche vorselektiert werden sollen.
	 * 
	 * @param pidConfigurationAreaSystemObjectPids
	 *          Pids der {@link SystemObject}s der KonfigurationsBereiche, welche
	 *          vorselektiert werden sollen.
	 * @return Liefert eine Liste der {@link SystemObject}s der selektierten
	 *         Konfigurationsbereiche. Diese Liste ist leer, wenn keine
	 *         KonfigurationsBereiche selektiert wurden. Der Rckgabewert ist
	 *         <code>null</code>, wenn der Dialog mit Abbrechen beendet wurde.
	 * @param posAtMouse
	 *          Wenn <code>true</code>, dann wird der Dialog beim Maus-Cursor
	 *          positioniert. Der max. Bildschirmbereich wird nicht berschritten.
	 *          Bei <code>false</code> wird der Dialog auf dem aktuellen
	 *          {@link Display} zentriert dargestellt.
	 */
	public LinkedList<SystemObject> open(
			Collection<String> pidConfigurationAreaSystemObjectPids,
			boolean posAtMouse)
	{
		Shell parent = getParent();
		_shell = new Shell(parent, SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
		_shell.setText("Konfigurationsbereichs Whler");
		_shell.setSize(400, 550);

		if (posAtMouse)
		{
			ShellPosition.positionShellAtMouseCursor(_shell);
		}
		else
		{
			_shell.setBounds(ShellPosition.getBoundsChildCenteredOnParent(PlatformUI
					.getWorkbench().getActiveWorkbenchWindow().getShell().getBounds(),
					_shell.getBounds()));
		}

		/*
		 * Fllen
		 */
		createGroupUp(_shell);
		createCompositeDown(_shell);

		/*
		 * Layout
		 */
		_shell.setLayout(new GridLayout(1, false));

		GridData gdUp = new GridData();
		gdUp.grabExcessHorizontalSpace = true;
		gdUp.horizontalAlignment = GridData.FILL;
		gdUp.grabExcessVerticalSpace = true;
		gdUp.verticalAlignment = GridData.FILL;
		_groupUp.setLayoutData(gdUp);

		GridData gdDown = new GridData();
		gdDown.grabExcessHorizontalSpace = true;
		gdDown.horizontalAlignment = GridData.END;
		_compositeDown.setLayoutData(gdDown);

		/*
		 * Vorauswahl
		 */
		if ((pidConfigurationAreaSystemObjectPids != null)
				&& (pidConfigurationAreaSystemObjectPids.size() > 0))
		{
			if ((pidConfigurationAreaSystemObjectPids != null)
					&& (_compositeConfigurationAreaChooser != null))
			{
				_compositeConfigurationAreaChooser
						.setSelectedConfigurationAreasPids(pidConfigurationAreaSystemObjectPids);
			}
		}

		/*
		 * ffnen des Dialogs
		 */
		_shell.open();
		Display display = parent.getDisplay();
		while (!_shell.isDisposed())
		{
			if (!display.readAndDispatch())
				display.sleep();
		}
		return _result;
	}

	/**
	 * Erzeugt die obere Gruppe zur Darstellung der {@link SystemObject}s der
	 * Konfigurationsbereich.
	 * 
	 * @param parent
	 *          Eltern-{@link Composite}
	 * @return Obere Gruppe
	 */
	private Group createGroupUp(Composite parent)
	{
		if (_groupUp == null)
		{
			_groupUp = new Group(parent, SWT.NONE);
			_groupUp.setText("Konfigurationsbereichsauswahl:");

			_compositeConfigurationAreaChooser = new CompositeConfigurationAreaChooser(
					_groupUp, SWT.NONE, _connection, EnumSelection.MULTI);
			/*
			 * Layout
			 */
			GridLayout glUp = new GridLayout(1, false);
			_groupUp.setLayout(glUp);

			GridData gdConfArea = new GridData();
			gdConfArea.grabExcessHorizontalSpace = true;
			gdConfArea.horizontalAlignment = GridData.FILL;
			gdConfArea.grabExcessVerticalSpace = true;
			gdConfArea.verticalAlignment = GridData.FILL;
			_compositeConfigurationAreaChooser.setLayoutData(gdConfArea);
		}

		return _groupUp;
	}

	/**
	 * Erzeugt das untere {@link Composite} mit den Buttons OK und Abbrechen.
	 * 
	 * @param parent
	 *          Eltern-{@link Composite}
	 * @return Unteres {@link Composite}
	 */
	private Composite createCompositeDown(Composite parent)
	{
		if (_compositeDown == null)
		{
			_compositeDown = new Composite(parent, SWT.NONE);

			Button buttonOk = new Button(_compositeDown, SWT.PUSH);
			buttonOk.setText("OK");
			buttonOk.addSelectionListener(new SelectionAdapter()
			{
				public void widgetSelected(SelectionEvent e)
				{
					System.out.println("OK");
					actionOk();
				}
			});

			Button buttonCancel = new Button(_compositeDown, SWT.PUSH);
			buttonCancel.setText("Abbrechen");
			buttonCancel.addSelectionListener(new SelectionAdapter()
			{
				public void widgetSelected(SelectionEvent e)
				{
					System.out.println("Abbrechen");
					actionCancel();
				}
			});

			/*
			 * Layout
			 */
			RowLayout rlDown = new RowLayout();
			rlDown.pack = false;
			_compositeDown.setLayout(rlDown);
		}

		return _compositeDown;
	}

	/**
	 * Aktion hinter dem OK-Button
	 */
	private void actionOk()
	{
		if (_compositeConfigurationAreaChooser != null)
		{
			_result = _compositeConfigurationAreaChooser
					.getSelectedConfigurationAreasSystemObjects();
		}

		_shell.dispose();
	}

	/**
	 * Aktion hinter dem Abbrechen-Button
	 */
	private void actionCancel()
	{
		_result = null;
		_shell.dispose();
	}

}
