/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.sba.plugin.dynobj.preferences;

import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.jface.preference.PreferenceStore;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.sba.plugin.dynobj.Activator;
import de.bsvrz.sba.plugin.dynobj.configurationAreas.ConfigurationAreaControler;
import de.bsvrz.sba.plugin.dynobj.configurationAreas.ConfigurationAreaDirectAccessPreferenceStore;
import de.bsvrz.sba.plugin.dynobj.configurationAreas.ConfigurationAreaModel;
import de.bsvrz.sba.plugin.dynobj.configurationAreas.compsites.CompositeConfigurationAreaModelEditor;

/**
 * {@link PreferencePage} zur Festlegung einer fr das Rahmenwerk gltigen
 * Default-Zuordnung von Objekttypen zu Konfigurationsbereichen.<br>
 * <br>
 * Eine Abfrage der Zuordnungen ist fr den Entwickler mit Hilfe der Klasse
 * {@link ConfigurationAreaDirectAccessPreferenceStore} mglich.
 * 
 * @author HofmannJ
 * 
 */
public class PreferencePageObjTypeConfArea extends PreferencePage implements
		IWorkbenchPreferencePage
{
	/**
	 * ID der PreferencePage
	 */
	public static final String ID = "de.bsvrz.sba.plugin.dynobj.pageObjTypeConfArea";

	/**
	 * Editor-Composite
	 */
	private CompositeConfigurationAreaModelEditor _cEdit = null;

	/**
	 * Konstruktor. Setzt den zu verwendenden {@link PreferenceStore}.
	 */
	public PreferencePageObjTypeConfArea()
	{
		setPreferenceStore(Activator.getDefault().getPreferenceStore());
		setDescription("Zuordnung von Objekttypen zu Konfigurationsbereichen.");
	}

	@Override
	protected Control createContents(Composite parent)
	{
		Composite top = new Composite(parent, SWT.NONE);

		// ConfigurationAreaModel model = new ConfigurationAreaModel();
		// model.add("pid.ObjTyp", "pid.KonfBereich");
		// model.add("pid.blubObjTyp", "pid.blubConfArea");
		// model.resetDirtyFlag();

		ConfigurationAreaControler cont = new ConfigurationAreaControler();
		cont.readFromPreferenceStore(getPreferenceStore());
		ConfigurationAreaModel model = cont.getModel();
		model.resetDirtyFlag();

		_cEdit = new CompositeConfigurationAreaModelEditor(top, SWT.NONE, model,
				getPreferenceStore());
		/*
		 * Layout
		 */
		GridLayout glTop = new GridLayout(1, false);
		top.setLayout(glTop);

		GridData gdTop = new GridData();
		gdTop.grabExcessHorizontalSpace = true;
		gdTop.horizontalAlignment = GridData.FILL;
		gdTop.grabExcessVerticalSpace = true;
		gdTop.verticalAlignment = GridData.FILL;
		top.setLayoutData(gdTop);

		GridData gdEdit = new GridData();
		gdEdit.grabExcessHorizontalSpace = true;
		gdEdit.horizontalAlignment = GridData.FILL;
		gdEdit.grabExcessVerticalSpace = true;
		gdEdit.verticalAlignment = GridData.FILL;
		_cEdit.setLayoutData(gdEdit);

		return null;
	}

	/**
	 * Sichert die getroffenen Einstellungen im PreferenceStore
	 */
	private void actionSave()
	{
		if (_cEdit != null)
		{
			_cEdit.exportToPrefStore();
		}
	}

	@Override
	protected void performApply()
	{
		actionSave();
	}

	@Override
	public boolean performOk()
	{
		performApply();
		return super.performOk();
	}

	@Override
	public boolean performCancel()
	{
		return true;
	}

	@Override
	protected void performDefaults()
	{
		// actionRestoreDefaults();
		// updateApplyButton();
		MessageBox mb = new MessageBox(PlatformUI.getWorkbench().getDisplay()
				.getActiveShell(), SWT.ICON_INFORMATION | SWT.OK);
		mb.setText("Keine Default-Einstellungen verfgbar.");
		mb
				.setMessage("Es ist fr die Zuordnung von Objekttypen zu Konfigurationsbereichen keine Defaulteinstellung verfgbar.");
		mb.open();
	}
	
	@Override
	public void performHelp()
	{
		String helpContextId = Activator.PLUGIN_ID + ".dynobj";
		PlatformUI.getWorkbench().getHelpSystem().displayHelp(helpContextId);
	}

	@Override
	public void init(IWorkbench workbench)
	{
	}

	public void dispose()
	{
		super.dispose();
		_cEdit.dispose();
	}
}
