/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.sba.plugin.navigator.navigatorModell;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.navigator.navigatorModell.INavigatorElement.STATUS;

/**
 * Klasse der Elemente des Navigators
 * 
 * @author Krause
 *
 */
public class NavigatorElementUrsachenEinheit extends NavigatorElement
{ 
  /**
   * Bildpfad zum Bild, das im Baum angezeigt wird
   */
  private String m_imagePath;

  //--------------------------------------------------------------------- 
  /**
   * Konstruktor fr ein Element des Navigators
   * 
   * @param parent
   *          Elternteil
   * @param name
   *          angezeigter Name
   * @param kategorie
   *          Kategorie des NavigatorElements, entspricht dem Teil des Filenames
   *          bis zum "_"
   * @param listeObjekte
   *          Liste der zugeordneten Systemobjekte
   */
  public NavigatorElementUrsachenEinheit(Object parent, String name, KATEGORIE kategorie, LinkedList<SystemObject> listeObjekte)
  {
//	super(parent, name, kategorie, listeObjekte);
	
	super();
	  
	m_parent = parent;
	m_name = name;
	m_ausgeklappt = false;
	m_childrenList = new ArrayList<Object>();
	if ( listeObjekte == null )
	  m_listeObjekte = new LinkedList<SystemObject>();
	else
	  m_listeObjekte = listeObjekte;
	    
	if ( parent != null )
	{
	  ((INavigatorElement) parent).addChild( this );
	}
	    
	if (kategorie != null)
	{
	  m_kategorie = kategorie;
	}

    // Status "unbekannt" vorbelegen
	if (name != null)
	  setStatus(STATUS.unbekannt);
  }
    
  /**
   * Setzt den Status und das entsprechende Bild
   * sowie den Status des Elternteils und die Aktualisierung des Teilbaums
   * 
   * @param status    
   *          Status des NavigatorElementes
   * @param IMAGEORDNER 
   *          Ordner in dem sich die Images befinden
   * @param m_kategorie 
   *          Kategorie des NavigatorElements also Bildart, welche verwendet werden soll         
   */
  @Override
  public boolean setStatus(STATUS status)
  {
    // Bild zum Status entsprechend setzen
    String imagefile = null;
    
    if (status == null)
    {
      imagefile = IMAGEORDNER + "UrsachenEinheitUnbekannt.png";
    }
    else
    {
      if (status == STATUS.unbekannt)
      {
        imagefile = IMAGEORDNER + "UrsachenEinheitUnbekannt.png";
      }
      else if (status == STATUS.aktiv_automatik)
      {
        imagefile = IMAGEORDNER + "UrsachenEinheitAutoAktiv.png";
      }
      else if (status == STATUS.inaktiv_automatik)
      {
        imagefile = IMAGEORDNER + "UrsachenEinheitAutoInaktiv.png";
      }
      else if (status == STATUS.aktiv_manuell)
      {
        imagefile = IMAGEORDNER + "UrsachenEinheitManuellAktiv.png";
      }
      else if (status == STATUS.inaktiv_manuell)
      {
        imagefile = IMAGEORDNER + "UrsachenEinheitManuellInaktiv.png";
      }
    }
    
    if (m_debug)
      System.out.println(imagefile);

    if ((imagefile != null) && (!imagefile.equals("")))
      setImagePath(imagefile);
        
    upd(this);
    
    return true;
  }

  //------------------------------------------------------------
  // Datenverteiler
  //------------------------------------------------------------
  /**
   * Status aus Datenverteiler ermitteln
   */
  @Override
  public void update( ResultData[] results )
  {
    for (ResultData resultData : results)
    {
      if (resultData.getDataDescription().getAttributeGroup().getPid().equals("atg.sswSbaEinzelsituationVorstufe"))
      {       
        if (resultData.hasData())
        {        
          boolean statusAktiv = resultData.getData().getScaledValue("Status").getValueText().equals("Ja");
          boolean statusAutomatik = resultData.getData().getScaledValue("StatusBercksichtigung").getValueText().equals("automatik");
          
          if (m_debug)
            System.out.println("\nAktiv: " + statusAktiv + " Automatik: " + statusAutomatik);
          
          if (statusAktiv && statusAutomatik)
          {
            setStatus(STATUS.aktiv_automatik);
          }
          else if (!statusAktiv && statusAutomatik)
          {
            setStatus(STATUS.inaktiv_automatik);
          }
          else if (statusAktiv && !statusAutomatik)
          {
            setStatus(STATUS.aktiv_manuell);
          }
          else if (!statusAktiv && !statusAutomatik)
          {
            setStatus(STATUS.inaktiv_manuell);
          }
          else
          {
            setStatus(STATUS.unbekannt);
          }
        }
        else
        {
          setStatus(STATUS.unbekannt);
        }
      }
    }
  }
}
