/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.sba.plugin.navigator.navigatorView;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindungEreignis;
import de.bsvrz.buv.rw.rw.dav.IDaVVerbindungZuhoerer;
import de.bsvrz.buv.rw.rw.dav.IDaVVerbindungsAbbauWarten;
import de.bsvrz.sba.plugin.navigator.navigatorModell.INavigatorElement;
import de.bsvrz.sba.plugin.navigator.navigatorModell.NavigatorElement;

/**
 * Job-Klasse, liefert die Daten aus dem Datenverteiler
 * fr die Baumstruktur und startet den Progressmonitor.
 * 
 * @author Krause
 * 
 */
public class LadeBaumDaten extends Job
{
  /**
   * Membervariable mit der einzigen Instanz der Klasse
   */
  private static LadeBaumDaten m_instance = null;
  
  /**
   * Flag ob der Job noch luft
   */
  private boolean m_laufeJob = false;
  
  /**
   * Wurzel-Element des Navigator-Baums
   */
  public INavigatorElement m_root;
  
  /**
   * Listener fr die Benachtichtigung, wenn der Ladevorgang abgeschlossen ist
   */ 
  private List<ILadeDatenListener> m_angemeldeteListener = new LinkedList<ILadeDatenListener>();
  
  /**
   * Listener auf die Datenverteilerverbindung
   */
  private IDaVVerbindungZuhoerer m_listenerDaVVerbindung;
  
  /**
   * Listener auf den Abbau der Datenverteilerverbindung
   */
  public IDaVVerbindungsAbbauWarten m_listenerDaVVerbindungsabbauWarten;

  /**
   * Konstruktor
   */
  private LadeBaumDaten()
  {
    super("NavigatorDaten vorladen");
    registriereDaVListener();
  }
  
  /**
   * Methode zum Auslesen der einzigen Instanz der Klasse
   * @return Instanz der Klasse
   */
  public static LadeBaumDaten getInstance ()
  {
    if (m_instance == null)
      m_instance = new LadeBaumDaten();
    
    return m_instance;
  }
  
  @Override
  protected IStatus run( IProgressMonitor monitor )
  {
    m_laufeJob = true;
    
    monitor.beginTask( "Lese Daten fr den Navigator ein ...", 22 );
    try
    {
      NavigatorDaten navDaten = new NavigatorDaten(monitor);
      m_root = navDaten.getNavigatorRoot(); 
      
      if (m_root == null)
        System.err.println("LadeBaumDaten:run: m_root=null <--schwerwiegender Fehler aus den NavigatorDaten--" );
      
      if (monitor.isCanceled())
        return Status.CANCEL_STATUS;
      monitor.worked( 1 );
    }
    catch (Exception e)
    {
      new NavigatorElement(m_root, "Verbindung zu den Daten wurde unterbrochen", null, null);
    }
    finally
    {
      monitor.done();
      entferneDaVListener();

      m_laufeJob = false;
      
      benachrichtigeListener(); 
    }
          
    return Status.OK_STATUS;
  }

  /**
   * Registrieren der DaVListener
   */
  private void registriereDaVListener()
  {
    m_listenerDaVVerbindung = new IDaVVerbindungZuhoerer()
    {

      @Override
      public void verbindungAbgebaut( DaVVerbindungEreignis e )
      {
        System.out.println(getClass().getName() + ":: Verbindung abgebaut!");
        //keine Aktivitt
      }

      @Override
      public void verbindungAufgebaut( DaVVerbindungEreignis e )
      {
        System.out.println("Verbindung aufgebaut!");
        
        //Job starten, wenn dieser nicht bereits luft
        if (!m_laufeJob)
          schedule();
      }    

    };
    DaVVerbindung.getInstanz().hinzufuegenZuhoerer( m_listenerDaVVerbindung );
    
    m_listenerDaVVerbindungsabbauWarten = new IDaVVerbindungsAbbauWarten()
    {

      @Override
      public boolean DaVVerbindungsAbbauOk()
      {
        if (m_listenerDaVVerbindung != null)
        {
          DaVVerbindung.getInstanz().entferneZuhoerer( m_listenerDaVVerbindung );
          m_listenerDaVVerbindung = null;
        }
        if (m_listenerDaVVerbindung == null)
          return true;
        else
          return false;
      }     
    
    };
    DaVVerbindung.getInstanz().hinzufuegenVerbindungsabbauWarten( m_listenerDaVVerbindungsabbauWarten );
  }
  
  /**
   * Registrieren der DaVListener
   */
  private void entferneDaVListener()
  {
    if (m_listenerDaVVerbindung != null)
    {
      DaVVerbindung.getInstanz().entferneZuhoerer( m_listenerDaVVerbindung );
      m_listenerDaVVerbindung = null;
    }
    if (m_listenerDaVVerbindungsabbauWarten != null)
    {
      DaVVerbindung.getInstanz().entfernenVerbindungsabbauWarten( m_listenerDaVVerbindungsabbauWarten );
      m_listenerDaVVerbindungsabbauWarten = null;
    }      
  }

  /**
   * Methode um den Job zu Starten
   * 
   * @return  erfolgreich(true)
   */
  public void start()
  {
    if (m_laufeJob)
    {
      //Job luft noch
      cancelJob();
    }

    //Job starten
    schedule();
  }
  
  /**
   * Methode um den Job zu Unterbrechen
   * 
   * @return  erfolgreich(true)
   */
  public boolean cancelJob()
  {
    m_laufeJob = false;
    return super.cancel();
  }

  /**
   * Job ist noch aktiv
   * 
   * @return aktiv(true), deaktiv(false)
   */
  public boolean isAktiv()
  {
    return m_laufeJob;
  }

  /**
   * Daten sind bereits geladen
   * 
   * @return geladen(true), noch nicht geladen(false)
   */
  public boolean isDatenGeladen()
  {
    //Job luft noch
    if (isAktiv())
      return false;
    else
    {
      //Daten vohanden
      if (m_root == null)
        return false;
      else
        return true;    
    }
  }
  
  /**
   * Get-Methode fr das Wurzel-Element des Baumes
   * @return
   */
  public INavigatorElement getRoot()
  {
    return m_root;
  }
  
  /**
   * Methode zum Abmelden der Listener
   */
  public void dispose()
  {
    if (m_listenerDaVVerbindungsabbauWarten != null)
    {
      System.out.println("VerbindungsabbauWarten entfernen!");
      DaVVerbindung.getInstanz().entfernenVerbindungsabbauWarten( m_listenerDaVVerbindungsabbauWarten );
      m_listenerDaVVerbindungsabbauWarten = null;
    }
  }
  
  //----------------------------------------------------------------------------------------------
  // Listener Funktionalitten
  //----------------------------------------------------------------------------------------------

  /**
   * Methode zum Hinzufgen eines Listeners
   * @param listener Listener der hinzugefgt werden soll
   */
  public void addListener (ILadeDatenListener listener)
  {
    m_angemeldeteListener.add( listener );
  }
 
  /**
   * Methode zum Entfernen eines Listeners
   * @param listener Listener der entfernt werden soll
   */
  public void removeListener (ILadeDatenListener listener)
  {
    m_angemeldeteListener.remove( listener );
  }
 
  /**
   * Methode mit der die Klasse alle bei ihr angemeldeten Listener
   * ber die nderung der Selektion informiert.
   * @param objekte Objekte aus der Listbox
   */
  private void benachrichtigeListener ()
  {   
   // zu bergebende Daten eintragen 
   Iterator<ILadeDatenListener> it = m_angemeldeteListener.iterator();
   while (it.hasNext())
   {
     ILadeDatenListener l = it.next();
     l.ladenFertig(m_root);
   }
  }

}
