/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Adolf-Dambach-Strae
 * 76571 Gaggenau
 * Phone: +49-7225-64-01
 * Fax: +49-7225-64-300
 * mailto: info@els.dambach.de
 */

package de.bsvrz.sba.plugin.parameter.steuerungskern.konfig;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.Iterator;
import java.util.Vector;
import java.util.Map.Entry;
import java.util.concurrent.ConcurrentHashMap;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.ReferenceArray;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dav.DaVKommunikationSBA;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dav.SendeParameter;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.AnzeigeElementTyp;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.AnzeigeTypEigenschaften;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.DefaultEinstellungen;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.EAnzeigeTypen;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.ProjektSpezifischeEinstellungen;

/**
 * Klasse beinhaltet die Parameter ber die speziellen WBVZ-Inhalte 'dunkel', 'beliebig', 'beibehalten'
 * und 'don't care' definiert werden. 
 * @author ObertM
 *
 */
public class VerwaltungAnzeigeTypMitWVZInhalt
{
	/**
	 * Einzige Instanz der Klasse
	 */
  private static VerwaltungAnzeigeTypMitWVZInhalt _instance;

  /**
   * Verwaltung der Listener
   */
  private PropertyChangeSupport m_pcs = new PropertyChangeSupport (this);
  
  /**
   * Pid der angeforderten Attributgruppe
   */
  public static String _pidAtg = "atg.sswSbaAnzeigeTypMitWVZInhalt";

  /**
   * Definitionen der WVZ-Inhalte
   */
  private ConcurrentHashMap<EAnzeigeTypen, AnzeigeTypEigenschaften> _definitionen = new ConcurrentHashMap<EAnzeigeTypen, AnzeigeTypEigenschaften>();

  /**
   * Pid der SBA
   */
  private String _pidSBA;

  /**
   * Privater Konstruktor der Klasse
   */
  private VerwaltungAnzeigeTypMitWVZInhalt ()
  {
  	_pidSBA = ProjektSpezifischeEinstellungen.getPidSba();
    
    /*
     * Daten lesen ber DaVKommunikation
     */
  	final DaVKommunikationSBA kom = DaVKommunikationSBA.getInstanz();
  	kom.addDatenArt(_pidSBA, _pidAtg, "asp.parameterSoll", ReceiverRole.receiver());
  	
  	kom.addPropertyChangeListener(new PropertyChangeListener()
		{
			@Override
			public void propertyChange(PropertyChangeEvent evt)
			{
				if (evt.getPropertyName().equals(_pidAtg))
				{
					ResultData resultData = kom.getData(_pidAtg);
					if (resultData != null)
						auswerteAttributgruppe(resultData);
				}
			}
		});

		ResultData resultData = kom.getData(_pidAtg);
		if (resultData != null)
			auswerteAttributgruppe(resultData);
		
		pruefeAufDefaultWerte ();
  }
  
  /**
   * Methode prft ob der Parameter im DaV vorhanden und versorgt ist. Wenn nicht werden, sofern
   * vorhanden, Defaultdaten fr den Parametersatz erzeugt und in den DaV geschrieben. 
   */
  private void pruefeAufDefaultWerte()
  {
  	if (_definitionen.size() == 0)
  	{
  		_definitionen.clear();
  		
      System.err.println("Parameter " + _pidAtg + " fr " + _pidSBA + " nicht definiert. Es wird ein" +
      		"Defaultdatensatz erzeugt !" + ProjektSpezifischeEinstellungen.getInstance().getSba());
      
			for (AnzeigeTypEigenschaften eigenschaften : DefaultEinstellungen.getDefaultAnzeigeTypen())
				_definitionen.put(eigenschaften.getTyp(), eigenschaften);
			
			speichereInDaV();
  	}
  }

  /**
   * Methode zum Speichern der Parameter im DaV
   */
  private void speichereInDaV ()
  {
	  Data data = getData();
	  
    if (data != null)
    {
      SendeParameter sende = new SendeParameter(_pidAtg);
      sende.sendeParameter(getData());
    }
  }
  
	/**
   * Methode liefert die Singleton Instanz der Klasse
   * @return Singleton Instanz der Klasse
   */
  public static VerwaltungAnzeigeTypMitWVZInhalt getInstance ()
  {
    if (_instance == null)
    {
      synchronized (VerwaltungAnzeigeTypMitWVZInhalt.class)
      {
        if (_instance == null)
        {
          _instance = new VerwaltungAnzeigeTypMitWVZInhalt ();
        }
      }
    }
    
    return _instance;
  }
  
  /**
   * Methode fgt einen Listener der Klasse hinzu
   * @param pcl Listern der Klasse PropertyChangeListener
   */
  public void addPropertyChangeListener (PropertyChangeListener pcl)
  {
    m_pcs.addPropertyChangeListener( pcl );    
  }
  
  /**
   * Methode entfernt einen Listener aus der Klasse
   * @param pcl Listern der Klasse PropertyChangeListener
   */
  public void removePropertyChangeListener (PropertyChangeListener pcl)
  {
    m_pcs.removePropertyChangeListener( pcl );    
  }

  /**
   * Methode benachrichtigt die Listener
   */
  private void benachrichtigeListener ()
  {
    m_pcs.firePropertyChange( "Aktualisierung", null, null );
  }

  /**
   * Methode zum Auswerten der empfangenen Attributgruppen
   * @param results empfangenen Attributgruppen
   */
  public void auswerteAttributgruppen (ResultData[] results)
  {
    int anzahl = results.length;

    // Daten aus Eingangsqueue bearbeiten      
    
    for (int i = 0; i < anzahl; i++)
    {
      ResultData dat = results[i];
  
      if (dat.getDataDescription().getAttributeGroup().getPid().equals(_pidAtg))
      {
        auswerteAttributgruppe (dat);
        
        benachrichtigeListener();
      }
    }
  }
  
  /**
   * Methode zum Auswerten der Attributgruppe 'atg.sswSbaAnzeigeTypMitWVZInhalt'
   * @param dat empfangender Datensatz
   */
  private void auswerteAttributgruppe( ResultData result )
  {
  	_definitionen.clear();
  	
    if (result.hasData())
    {
    	Data data = result.getData();
    	
    	Array a1 = data.getArray("Definitionen");
    	for (int i=0; i<a1.getLength(); ++i)
    	{
    		Data aktuell = a1.getItem(i);

    		int tmp = aktuell.getUnscaledValue("AnzeigeTyp").intValue();
    		EAnzeigeTypen typ = EAnzeigeTypen.getTyp(tmp);

  		  SystemObject soTyp = aktuell.getReferenceValue("SystemObjekt").getSystemObject();

    		if (typ != null && soTyp != null)
    		{
//    			System.out
//              .println("VerwaltungAnzeigeTypMitWVZInhalt.auswerteAttributgruppe() " + soTyp.getPid());
    			
          AnzeigeTypEigenschaften eigenschaften = new AnzeigeTypEigenschaften ();
          eigenschaften.setTyp(typ);

          _definitionen.put(typ, eigenschaften);
          
          eigenschaften.setAnzeigeTyp (soTyp);
          
    			SystemObject soDefault = aktuell.getReferenceValue("Defaultbild").getSystemObject();
    			eigenschaften.setDefaultBild(soDefault);
    			
    			Vector<SystemObject> v = new Vector<SystemObject>();
          
    			ReferenceArray a2 = aktuell.getReferenceArray("Inhalte");
    			for (int j=0; j<a2.getLength(); ++j)
    			{
    				SystemObject so = a2.getSystemObject(j);
    				
//    				if (so.getPid().equals("wvzInhalt.uz.a2.tlsNurLkwUndKraftomnibus"))
//    					continue;
    				
    				if (so != null)
    					if (!v.contains(so))
    						v.add(so);
    			}

    			eigenschaften.setInhalte(v);
    		}
    	}
    }
  }
	
  /**
   * Methode zum Speichern der Parameter "Abhngigkeiten" im DaV
   * @param abhaengigkeiten Abhngigkeiten
   */
  public Data erzeugeAttributgruppe ()
  {
	  ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
    if (connection == null)
      return null;

    // Attributgruppe zusammensetzen und versenden
      
    AttributeGroup atg = (AttributeGroup) connection.getDataModel().getAttributeGroup(_pidAtg);
    Data data = connection.createData(atg );
    
    Array a1 = data.getArray("Definitionen");
    
    a1.setLength(_definitionen.size());
    
    int i = 0;
    Iterator<Entry<EAnzeigeTypen, AnzeigeTypEigenschaften>> it = _definitionen.entrySet().iterator();
    while (it.hasNext())
    {
      Data aktuell = a1.getItem(i++);
      
      Entry<EAnzeigeTypen, AnzeigeTypEigenschaften> entry = it.next();
      
      EAnzeigeTypen typ = entry.getKey();
      AnzeigeTypEigenschaften eigenschaft = entry.getValue();
      
      if (typ != null)
      {
      	aktuell.getUnscaledValue("AnzeigeTyp").set(typ.getKodieerungImDaV());
      	
      	aktuell.getReferenceValue("SystemObjekt").setSystemObject(eigenschaft.getAnzeigeTyp());
      	
      	aktuell.getReferenceValue("Defaultbild").setSystemObject(eigenschaft.getDefaultBild());
      	
      	Vector<SystemObject> inhalte = eigenschaft.getInhalte();
      	
      	ReferenceArray a2 = aktuell.getReferenceArray("Inhalte");
      	a2.setLength(inhalte.size());

      	for (int j=0; j<inhalte.size(); ++j)
      		a2.getReferenceValue(j).setSystemObject(inhalte.get(j));
      }
    }
    
    return data;
  }

  /**
   * Methode liefert die Parameter als Attributgruppe fr den DaV
   * @return Attributgruppe
   */
  public Data getData ()
  {
  	return erzeugeAttributgruppe();
  }
  
  /**
   * Methode liefert einen Vektor mit den versorgten Anzeigetypen 
   * @return Vektor mit den versorgten Anzeigetypen
   */
  public Vector<EAnzeigeTypen> getAnzeigeTypen ()
  {
  	Vector<EAnzeigeTypen> v = new Vector<EAnzeigeTypen>();
  	
  	for (EAnzeigeTypen eat : _definitionen.keySet())
    	v.add(eat);
  	
  	return v;
  }

  /**
   * Methode liefert einen Vektor mit den Eigenschaften der einzelnen Anzeigetypen
   * @return Vektor mit Eigenschaften
   */
	public Vector<AnzeigeTypEigenschaften> getEigenschaften()
  {
  	Vector<AnzeigeTypEigenschaften> v = new Vector<AnzeigeTypEigenschaften>();
  	for (AnzeigeTypEigenschaften eate : _definitionen.values())
    	v.add(eate);
  	
  	return v;
  }

	/**
	 * Methode zum Setzen der Parameter
	 * @param parameter Parameter
	 */
	public void setEigenschaften (Vector<AnzeigeTypEigenschaften> parameter)
  {
		_definitionen.clear();
		
		for (AnzeigeTypEigenschaften ate : parameter)
			if ((ate != null) && (ate.getTyp() != null))
		    _definitionen.put(ate.getTyp(), ate);
  }

  /**
   * Methode liefert die Eigenschaften zu einem bestimmten Anzeigetyp
   * @param typ Anzeigetyp
   * @return Eigenschaften dieses Typs oder null
   */
	public AnzeigeTypEigenschaften getEigenschaft (AnzeigeElementTyp typ)
  {
		if (_definitionen.containsKey(typ.getTyp()))
  	  return _definitionen.get(typ.getTyp());
		
  	return null;
  }
	
	public EAnzeigeTypen getAnzeigeTyp (SystemObject soWvzTyp)
	{
	  Iterator<Entry<EAnzeigeTypen, AnzeigeTypEigenschaften>> it =  _definitionen.entrySet().iterator();
	  while (it.hasNext())
	  {
	  	Entry<EAnzeigeTypen, AnzeigeTypEigenschaften> entry = it.next();
	  	
	  	EAnzeigeTypen typ = entry.getKey();
	  	AnzeigeTypEigenschaften ate = entry.getValue();
	  	
	  	if (ate.getAnzeigeTyp() == soWvzTyp)
	  		return typ;
	  }
  
	  return null;
	}
}
