package de.bsvrz.sba.plugin.parameter.steuerungskern.konfig;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;
import java.util.Map.Entry;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.EAnzeigeQuerschnittTyp;

/**
 * Klasse verwaltet die Zuordnung der Anzeigequerschnitttypen zu den Anzeigequerschnitten
 * @author ObertM
 */
public class VerwaltungAnzeigeTypen
{
	/**
	 * Einzige Instanz der Klasse
	 */
  private static VerwaltungAnzeigeTypen _instance;

  private HashMap<String, EAnzeigeQuerschnittTyp> _zuordnung = new HashMap<String, EAnzeigeQuerschnittTyp>();

	/**
   * Privater Konstruktor der Klasse
   */
	private VerwaltungAnzeigeTypen ()
	{
		bestimmeTypen ();
	}

	/**
   * Methode liefert die Singleton Instanz der Klasse
   * @return Singleton Instanz der Klasse
   */
  public static VerwaltungAnzeigeTypen getInstance ()
  {
    if (_instance == null)
    {
      synchronized (VerwaltungAnzeigeTypen.class)
      {
        if (_instance == null)
        {
          _instance = new VerwaltungAnzeigeTypen ();
        }
      }
    }
    
    return _instance;
  }

	/**
	 * Methode zum Bestimmen der Anzeigetypen
	 */
	private void bestimmeTypen()
  {
		HashMap<SystemObject, AnzeigeQuerschnittMitTyp> zuordnung = VerwaltungMoeglicheAnzeigeQuerschnitte.getInstance().getZuordnung();
		
		Iterator<Entry<SystemObject, AnzeigeQuerschnittMitTyp>> it = zuordnung.entrySet().iterator();
		while (it.hasNext())
		{
			Entry<SystemObject, AnzeigeQuerschnittMitTyp> entry = it.next();
			
			SystemObject so = entry.getKey();
			EAnzeigeQuerschnittTyp typ = entry.getValue().getTyp();

			if (so != null)
			  _zuordnung.put(so.getPid(), typ);
		}
  }

	/**
	 * Methode liefert den Anzeigetyp zu einem Anzeigequerschnitt
	 * @param pidAnzeigeQuerschnitt Pid des Anzeigequerschnitts
	 * @return Anzeigetyp des Anzeigequerschnitts
	 */
  public EAnzeigeQuerschnittTyp getTyp (String pidAnzeigeQuerschnitt)
  {
  	return _zuordnung.get(pidAnzeigeQuerschnitt);
  }

  /**
	 * Methode liefert den Anzeigetyp zu einem Anzeigequerschnitt
	 * @param anzeigeQuerschnitt Anzeigequerschnitt
	 * @return Anzeigetyp des Anzeigequerschnitts
	 */
  public EAnzeigeQuerschnittTyp getTyp (SystemObject anzeigeQuerschnitt)
  {
  	return getTyp(anzeigeQuerschnitt.getPid());
  }
  
  /**
   * Methode liefert eine Liste aller bentigten Anzeigequerschnitttypen
   * @return Liste aller bentigten Anzeigequerschnitttypen
   */
  public EAnzeigeQuerschnittTyp[] getTypen ()
  {
    List<EAnzeigeQuerschnittTyp> l = new ArrayList<EAnzeigeQuerschnittTyp>();	
  
		Iterator<Entry<String, EAnzeigeQuerschnittTyp>> it = _zuordnung.entrySet().iterator();
		while (it.hasNext())
		{
			Entry<String, EAnzeigeQuerschnittTyp> entry = it.next();
			
			EAnzeigeQuerschnittTyp typ = entry.getValue();
			
			if (!l.contains(typ))
				l.add (typ);
		}
  	
  	return l.toArray(new EAnzeigeQuerschnittTyp [l.size()]);
  }

  /**
	 * Methode prft ob es zu einem Anzeigequerschnittyp mindestens einen AQ gibt
	 * @param so Systemobjekt 
	 * @return Info ob verwendet
	 */
	public boolean isAnzeigeTypInVerwendung(EAnzeigeQuerschnittTyp typ)
  {
		Vector<SystemObject> aqs = Konfig.getInstance().getAnzeigeQuerschnitte();
		for (SystemObject so : aqs)
			if (getTyp(so) == typ)
				return true;
			
		return false;
  }
}
