package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.bedienung;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;
import java.util.Map.Entry;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.AnzeigeQuerschnittMitTyp;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.DefaultEinstellungen;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.EAnzeigeQuerschnittTyp;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;

/**
 * Composite zum Versorgen der Parameter Bedienung
 * @author ObertM
 *
 */
public class CompositeMoeglicheAnzeigeQuerschnitte extends Composite
{
	/**
	 * Hashmap mit den mglichen Objekten
	 */
	private HashMap<SystemObject, AnzeigeQuerschnittMitTyp> _objekte = new HashMap<SystemObject, AnzeigeQuerschnittMitTyp>();

	/**
	 * Parameter Anzeigequerschnitte
	 */
	private List<AnzeigeQuerschnittMitTyp> _parameter;

	private CompositeZuordnungMitTypAuswahl _zuordnung;

	private Button _buttonTypUebernehmen;

	private Combo _combo;

  /**
   * Konstruktor der Klasse
   * @param parent Parent Composite
   * @param style Stil
   * @param parameter aktuelle Parameter
   */
	CompositeMoeglicheAnzeigeQuerschnitte(Composite parent, int style, ParameterBedienung parameter) 
  {
    super(parent, style);
    
    /*
     * Kopie der Parameter anlegen
     */
    _parameter = new ArrayList<AnzeigeQuerschnittMitTyp>();
    for (AnzeigeQuerschnittMitTyp aqmt : parameter.getMoeglicheAnzeigeQuerschnitte())
    	_parameter.add(aqmt.getCopy());
   
    bestimmeMoeglicheObjekte();
    
    initialize();
    
    aktualisiereEingabeElemente();
  }

	/**
   * Fhrt die Basis-Initialisierung des Composites durch
   */
  private void initialize()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.horizontalIndent = 0;
    gridData1.verticalAlignment = GridData.FILL;

    setLayout(new GridLayout(1, false));
    setLayoutData(gridData1);

		GridData gridDataCombo = new GridData();
		gridDataCombo.horizontalAlignment = GridData.FILL;
		gridDataCombo.grabExcessHorizontalSpace = true;

    /*
     * Composite mit der Zuordnung der WVZ-Anzeigen
     */
		GridData gridDataZuordnung = new GridData();
		gridDataZuordnung.horizontalAlignment = GridData.FILL;
		gridDataZuordnung.grabExcessHorizontalSpace = true;
		gridDataZuordnung.verticalAlignment = GridData.FILL;
		gridDataZuordnung.grabExcessVerticalSpace = true;

  	_zuordnung = new CompositeZuordnungMitTypAuswahl (this, SWT.BORDER, "mgliche Anzeigequerschnitte:", "zugeordnete Anzeigequerschnitte:");
  	_zuordnung.setLayoutData(gridDataZuordnung);
    
  	_combo                = _zuordnung.getCombo();
  	_buttonTypUebernehmen = _zuordnung.getButtonTypUebernehmen();
  	
  	_buttonTypUebernehmen.addMouseListener(new MouseAdapter()
		{
			@Override
			public void mouseDown(MouseEvent e)
			{
				super.mouseDown(e);

				aendereTyp ();
			}
		});

  	
  	/*
  	 * Composite mit dem Defaultbutton 
  	 */
  	createCompositeDefaultButton(this);
  }

	/**
	 * Methode erzeugt das Composite mit dem Default Button
	 */
	private void createCompositeDefaultButton(Composite parent)
	{
		GridData gridData1 = new GridData();
		gridData1.horizontalAlignment = GridData.FILL;
		gridData1.grabExcessHorizontalSpace = true;
		
		Composite composite = new Composite(parent, SWT.BORDER);
		composite.setLayout(new GridLayout(1, false));
		composite.setLayoutData(gridData1);

		Button buttonDefault = new Button(composite, SWT.NONE);
		
		buttonDefault.setText("Defaultwerte");
		buttonDefault.addMouseListener(new MouseAdapter()
		{
			@Override
			public void mouseDown(MouseEvent e)
			{
				super.mouseDown(e);
				
				defaultWerte();
			}
		});
	}

	/**
	 * Methode definiert die Defaultwerte
	 */
	private void defaultWerte ()
	{
		_parameter.clear();
			
		ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
		DataModel model = connection.getDataModel();
		
    HashMap<String, EAnzeigeQuerschnittTyp> querschnitte = DefaultEinstellungen.getDefaultAnzeigeQuerschnitte();
    
    Iterator<Entry<String, EAnzeigeQuerschnittTyp>> it = querschnitte.entrySet().iterator();
    while (it.hasNext())
    {
      Entry<String, EAnzeigeQuerschnittTyp> entry = it.next();	
      
      String pid = entry.getKey();
      EAnzeigeQuerschnittTyp typ = entry.getValue();

      SystemObject soAq = model.getObject(pid);
      
    	if (soAq != null)
    	{
    		if (_objekte.containsKey(soAq))
    		{
    			AnzeigeQuerschnittMitTyp aqmt = _objekte.get(soAq);
        	
    			aqmt.setTyp(typ);
       		_parameter.add(aqmt);
    		}
    	}
    }
    
    aktualisiereEingabeElemente();
	}

  /**
   * Methode bestimmt die im Datenverteiler definierten Objekte
   */
	private void bestimmeMoeglicheObjekte()
  {
		java.util.List<SystemObject> objekte = Tools.getSystemObjekte("typ.anzeigeQuerschnitt");
    for (SystemObject so : objekte)
    {
    	boolean gefunden = false;
    	for (AnzeigeQuerschnittMitTyp aqt : _parameter)
    	{
    		if (aqt.getAnzeigeQuerschnitt() == so)
    		{
    			gefunden = true;
    			break;
    		}
    	}
    	
    	if (!gefunden)
    	  _objekte.put(so, new AnzeigeQuerschnittMitTyp (so, EAnzeigeQuerschnittTyp.UNDEFINIERT));
    }
  }

	/**
   * Methode fgt die Anzeigequerschnitte hinzu
   */
  private void aktualisiereEingabeElemente ()
  {
  	_zuordnung.getMoeglicheObjekte().clear();
  	_zuordnung.getZugewieseneObjekte().clear();
  	
  	for (AnzeigeQuerschnittMitTyp so : _objekte.values())
  		_zuordnung.getMoeglicheObjekte().add(so);
  	
  	for (AnzeigeQuerschnittMitTyp so : _parameter)
  	{
  		_zuordnung.getZugewieseneObjekte().add(so);
  		_zuordnung.getMoeglicheObjekte().remove(so);
  	}

  	_zuordnung.refresh ();
  }  

  /**
   * Methode liefert die aktuell eingestellten Parameter
   * @return aktuell eingestellten Parameter
   */
	public Vector<AnzeigeQuerschnittMitTyp>  getParameter()
  {
    Vector<AnzeigeQuerschnittMitTyp> objekte = new Vector<AnzeigeQuerschnittMitTyp>();
    
    for (Object so : _zuordnung.getZugewieseneObjekte())
    	objekte.add((AnzeigeQuerschnittMitTyp) so);
    
  	return objekte;
  }

	/**
	 * Methode aktualisiert die View
	 */
	public void refresh()
  {
  }
	
	protected void aendereTyp()
  {
		/*
		 * Ausgewhlten Anzeigetyp bestimmen
		 */
		int index = _combo.getSelectionIndex();
		if (index == -1)
			return;
		
		EAnzeigeQuerschnittTyp typ = EAnzeigeQuerschnittTyp.values()[index];

		Vector<Object> selektiert = _zuordnung.getSelectedObjekteRechts();
		for (Object obj1 : selektiert)
		{
      if (obj1 instanceof AnzeigeQuerschnittMitTyp)
      {
      	AnzeigeQuerschnittMitTyp as = (AnzeigeQuerschnittMitTyp) obj1;
      	as.setTyp(typ);
      }
    }
		
		/*
		 * nderung 05.06.14 MO
		 * Vor nderung des Typs mssen die sich aktuell auf der rechten Seite befindenden
		 * Anzeigequerschnitte wieder eingelesen werden. Sonst passiert es, dass ein AQ
		 * der gerade erst auf die rechte Seite kopiert wurde bei einer Typ-nderung wieder
		 * nach links wandert !
		 */
  	_parameter = getParameter();  

		aktualisiereEingabeElemente();
  }
}
