package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.bedienung;

import java.util.HashMap;
import java.util.TreeMap;
import java.util.Vector;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.ESpezialWVZInhalt;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.Konfig;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.DefaultEinstellungen;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.IProjektEinstellungen;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;

/**
 * Composite zum Versorgen der Parameter Bedienung
 * 
 * @author ObertM
 * 
 */
public class CompositeSpezialWVZInhalte extends Composite
{
	/**
	 * Parameter Spezial WVZ-Inhalte
	 */
	private HashMap<ESpezialWVZInhalt, SystemObject> _parameter = null;

	/**
	 * Vektor mit allen mglichen WVZ-Inhalten
	 */
	private Vector<SystemObject> _wvzInhalte = new Vector<SystemObject>();

	private Combo[] _combo;

	private ESpezialWVZInhalt[] _inhalt;

	/**
	 * Konstruktor der Klasse
	 * 
	 * @param parent
	 *          Parent Composite
	 * @param style
	 *          Stil
	 * @param parameter
	 *          aktuelle Parameter
	 */
	CompositeSpezialWVZInhalte(Composite parent, int style,
	    ParameterBedienung parameter)
	{
		super(parent, style);

		_parameter = parameter.getSpezialWVZInhalte();

		bestimmeWVZInhalte();

		initialize();
	}

	/**
	 * Fhrt die Basis-Initialisierung des Composites durch
	 */
	private void initialize()
	{
		GridData gridData1 = new GridData();
		
		gridData1.horizontalAlignment = GridData.FILL;
		gridData1.verticalAlignment = GridData.FILL;
		gridData1.grabExcessHorizontalSpace = true;
		gridData1.grabExcessVerticalSpace = true;

		setLayout    (new GridLayout(1, false));
		setLayoutData(gridData1);

		createCompositeSpezialWVZInhalte (this);
		
		createCompositeButtons (this);
		
		aktualisiereParameter();
	}

	/**
	 * Methode legt ein Composite an in dem die Parameter versorgt werden knnen
	 * 
	 * @param parent
	 *          Parent Composite
	 */
	private void createCompositeSpezialWVZInhalte(Composite parent)
	{
		GridData gridData1 = new GridData();
		gridData1.horizontalAlignment = GridData.FILL;
		gridData1.verticalAlignment = GridData.FILL;
		gridData1.grabExcessHorizontalSpace = true;
		gridData1.grabExcessVerticalSpace = true;
		
		Composite composite = new Composite(parent, SWT.BORDER);
		composite.setLayout(new GridLayout(1, false));
		composite.setLayoutData(gridData1);

		int anzahl = ESpezialWVZInhalt.values().length;

		_combo = new Combo[anzahl];
		_inhalt = new ESpezialWVZInhalt[anzahl];

		for (int i = 0; i < anzahl; ++i)
			erzeugeCompositeWVZInhalt(composite, i);
	}

	private void erzeugeCompositeWVZInhalt (Composite parent, int index)
	{
		GridData gd1 = new GridData();
		gd1.horizontalAlignment = GridData.FILL;
		gd1.grabExcessHorizontalSpace = true;
		gd1.grabExcessVerticalSpace = true;
		gd1.verticalAlignment = GridData.FILL;

		Composite comp1 = new Composite(parent, SWT.BORDER);
		comp1.setBackground(parent.getBackground());
		comp1.setBackgroundMode(SWT.INHERIT_DEFAULT);
		comp1.setLayoutData(gd1);
		comp1.setLayout(new GridLayout());

		GridData gdTextItem = new GridData();
		gdTextItem.horizontalAlignment = GridData.FILL;
		gdTextItem.grabExcessHorizontalSpace = true;
		gdTextItem.verticalAlignment = GridData.BEGINNING;

		GridData gdComboItem = new GridData();
		gdComboItem.horizontalAlignment = GridData.FILL;

		_inhalt[index] = ESpezialWVZInhalt.values()[index];

		Label dummy = new Label(comp1, SWT.LEFT);
		dummy.setText(_inhalt[index].toString());
		dummy.setLayoutData(gdTextItem);

		_combo[index] = new Combo(comp1, SWT.BORDER);
		_combo[index].setLayoutData(gdComboItem);

		String[] sa = new String[_wvzInhalte.size()];
		for (int j = 0; j < _wvzInhalte.size(); ++j)
			sa[j] = _wvzInhalte.get(j).getName();

		_combo[index].setItems(sa);
	}
	
	/**
	 * Methode erzeugt das Composite mit den Buttons
	 */
	private void createCompositeButtons(Composite parent)
	{
		GridData gridData1 = new GridData();
		gridData1.horizontalAlignment = GridData.FILL;
		gridData1.grabExcessHorizontalSpace = true;
		
		Composite composite = new Composite(parent, SWT.BORDER);
		composite.setLayout(new GridLayout(1, false));
		composite.setLayoutData(gridData1);

		GridData gdComboItem = new GridData();
		gdComboItem.horizontalAlignment = GridData.END;

		Button buttonDefault = new Button(composite, SWT.NONE);
		
		buttonDefault.setText("Defaultwerte");
		buttonDefault.addMouseListener(new MouseAdapter()
		{
			@Override
			public void mouseDown(MouseEvent e)
			{
				super.mouseDown(e);
				
				defaultWerte();
			}
		});
	}

	private void defaultWerte ()
	{
		_parameter.clear();
		
		IProjektEinstellungen ipe = DefaultEinstellungen
		    .getProjektEinstellungen();
		
		if (ipe != null)
		{
			SystemObject so = Konfig.getInstance().getSystemObject(ipe.getPidDunkel());
			_parameter.put(ESpezialWVZInhalt.Dunkel, so);
			
			so = Konfig.getInstance().getSystemObject(ipe.getPidBeibehalten());
			_parameter.put(ESpezialWVZInhalt.Beibehalten, so);
			
			so = Konfig.getInstance().getSystemObject(ipe.getPidBeliebig());
			_parameter.put(ESpezialWVZInhalt.Beliebig, so);
			
			so = Konfig.getInstance().getSystemObject(ipe.getPidDontCare());
			_parameter.put(ESpezialWVZInhalt.DontCare, so);
		}
		
		aktualisiereParameter();
	}
	
	/**
	 * Methode aktualisiert die Anzeige der Parameter
	 */
	private void aktualisiereParameter ()
	{
		for (int i=0; i<_combo.length; ++i)
		{
			int indexInhalt = -1;
			SystemObject aktuell = _parameter.get(_inhalt[i]);
			for (int j = 0; j < _wvzInhalte.size(); ++j)
				if (_wvzInhalte.get(j) == aktuell)
				{
					indexInhalt = j;
					break;
				}
		
			_combo[i].select(indexInhalt);
		}
	}
	
	/**
	 * Methode liefert die aktuell eingestellten Parameter
	 * 
	 * @return aktuell eingestellten Parameter
	 */
	public HashMap<ESpezialWVZInhalt, SystemObject> getParameter()
	{
		return leseAktuelleEinstellungen();
	}

	private HashMap<ESpezialWVZInhalt, SystemObject> leseAktuelleEinstellungen()
	{
		HashMap<ESpezialWVZInhalt, SystemObject> aktuell = new HashMap<ESpezialWVZInhalt, SystemObject>();

		for (int i = 0; i < _combo.length; ++i)
		{
			int index = _combo[i].getSelectionIndex();

			aktuell.put(_inhalt[i], _wvzInhalte.get(index));
		}

		return aktuell;
	}

	/**
	 * Methode aktualisiert die View
	 */
	public void refresh()
	{
	}

	/**
	 * Methode liefert eine Bezeichnung fr einen Aspekt. Die Bezeichnung setzt
	 * sich zusammen aus Name + Pid
	 * 
	 * @param objekt
	 *          Systemobjekt des Aspekts
	 * @return Bezeichnung
	 */
	private String getBezeichnung(SystemObject objekt)
	{
		return objekt.getName() + " (" + objekt.getPid() + ")";
	}
	
	/**
	 * Methode bestimmt die im DaV definierten Aspekte
	 */
	private void bestimmeWVZInhalte()
	{
		TreeMap<String, SystemObject> sorter = new TreeMap<String, SystemObject>();

		java.util.List<SystemObject> objekte = Tools
		    .getSystemObjekte("typ.wvzInhalt");
		for (SystemObject so : objekte)
		{
			sorter.put(getBezeichnung(so), so);
		}

		for (SystemObject a : sorter.values())
			_wvzInhalte.add(a);
	}

}
