/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Adolf-Dambach-Strae
 * 76571 Gaggenau
 * Phone: +49-7225-64-01
 * Fax: +49-7225-64-300
 * mailto: info@els.dambach.de
 */

package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.entfernungsangaben;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;

/**
 * Klasse realisiert eine Stufe fr die Treppenfunktion der Entfernungsangaben.
 * Eine Stufe besteht dabei aus einem unteren und einem oberen Grenzwert fr diese
 * Stufe. 
 * @version $Revision: 1.1 $ / $Date: 2013/06/05 14:17:02 $ / ($Author: ObertM $)
 * @author Dambach-Werke GmbH
 * @author Matthias Obert
 */
public class Stufe
{
  /**
   * Unterer Grenzwert fr diese Stufe (Entfernungsangabe)
   */
  private long m_von;
  
  /**
   * Oberer Grenzwert fr diese Stufe (Entfernungsangabe)
   */
  private long m_bis;
  
  /**
   * WVZ-Inhalt dieser Stufe (typ.wvzInhalt aus TeilmodellAnzeigenGlobal (kb.tmAnzeigenGlobal))
   */
  private WVZInhalt m_wvzInhalt = null;
  
  /**
   * Parameterloser Konstruktor der Klasse
   */
  public Stufe ()
  {
    
  }
  
  /**
   * Konstruktor der Klasse
   * @param von Von
   * @param bis Bis
   * @param inhalt WVZ Inhalt
   */
  public Stufe (long von, long bis, WVZInhalt inhalt)
  {
  	m_von = von;
  	
  	m_bis = bis;
  	
  	m_wvzInhalt = inhalt;
  }

  /**
   * Methode liefert die Variable m_von zurck
   * @return Variable m_von
   */
  public long getVon()
  {
    return m_von;
  }

  /**
   * Methode setzt die Variable m_von
   * @param von Wert auf den die Variable m_von gesetzt werden soll
   */
  public void setVon( long von )
  {
    m_von = von;
  }

  /**
   * Methode liefert die Variable m_bis zurck
   * @return Variable m_bis
   */
  public long getBis()
  {
    return m_bis;
  }

  /**
   * Methode setzt die Variable m_bis
   * @param bis Wert auf den die Variable m_bis gesetzt werden soll
   */
  public void setBis( long bis )
  {
    m_bis = bis;
  }

  /**
   * Methode zum Auswerten einer Attributliste 'atl.sswSbaEntferungsAngabeStufe'
   * @param data Empfangene Daten vom DaV
   */
  public void auswerteAttributgruppe( Data data )
  {
    m_von = data.getUnscaledValue( "UntereEntfernung" ).longValue();
    
    m_bis = data.getUnscaledValue( "ObereEntfernung" ).longValue();
    
    SystemObject so = data.getReferenceValue( "AnzeigeInhalt" ).getSystemObject();
    
    m_wvzInhalt = new WVZInhalt(so.getPid());
  }

  /**
   * Methode zum Ausgeben der Klasse als String
   */
  @Override
  public String toString ()
  {
    String s = "";
    
    s = "Von: " + getVon() + "m bis " + getBis() + "m --> ";
    
    if (getWvzInhalt() != null)
      s += getWvzInhalt().toString();
    else 
      s += null;
    
    s += "\n";
    
    return s;
  }

  /**
   * Methode liefert die Mitte der Stufe
   * @return Mitte der Stufe
   */
  public long getMitteDerStufe()
  {
    long differenz = getBis() - getVon();
    long mitte = getVon() + differenz / 2;
    
    return mitte;
  }

  /**
   * Methode liefert die Variable m_wvzInhalt zurck
   * @return Variable m_wvzInhalt
   */
  public WVZInhalt getWvzInhalt()
  {
    return m_wvzInhalt;
  }

  /**
   * Methode setzt die Variable m_wvzInhalt
   * @param wvzInhalt Wert auf den die Variable m_wvzInhalt gesetzt werden soll
   */
  public void setWvzInhalt( WVZInhalt wvzInhalt )
  {
    m_wvzInhalt = wvzInhalt;
  }
}
