package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.lueckenfuellen;

import java.util.List;
import java.util.Vector;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.Richtung.ArbeitsRichtungVN;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.monotonie.RegelMonotonie;

/**
 * Klasse enthlt ein Feld (Liste) mit Parametern
 * @author       Dambach Werke GmbH
 */
public class ListeMitRegeln
{
  /**
   * Liste mit den Parametern
   */
  private Vector<RegelLueckenFuellen> m_parameter = new Vector<RegelLueckenFuellen> ();

  /**
   * Parameterloser Konstruktor der Klasse
   */
  public ListeMitRegeln ()
  {
  }
  
  /**
   * Methode liefert die Variable parameter zurck
   * @return Variable parameter
   */
  public Vector<RegelLueckenFuellen> getParameter()
  {
    Vector<RegelLueckenFuellen> v = new Vector<RegelLueckenFuellen>();
    synchronized (m_parameter)
    {
      for (RegelLueckenFuellen r : m_parameter)
        v.add( r );
    }
    return v;
  }

  /**
   * Methode zum Auswerten der Regeln der Attributliste 'atl.sswSbaLckenFllenProAnzeigeTyp'
   * @param data Daten vom DaV
   */
  public void auswerteAttributgruppe (Data data, Aspect schaltgrundAllgemein)
  {
    Array a = data.getArray( "Regeln" );
    int anzahl = a.getLength();
    
    synchronized (m_parameter)
    {
      m_parameter.clear();
      for (int x = 0; x < anzahl; ++x)
      {
        RegelLueckenFuellen r = new RegelLueckenFuellen ();
        r.auswerteAttributgruppe( a.getItem( x ), schaltgrundAllgemein );
        
        m_parameter.add( r );
      }
    }
  }

  /**
   * Methode fgt eine Regel hinzu
   * @param x Anzeigeinhalt der aktuell betrachteten Anzeige 
   * @param xyz Anzeigeinhalt des Vorgnger/Nachfolgers
   * @param uvw Anzeigeinhalt des Vorgngers des Vorgnger/Nachfolger des Nachfolgers
   * @param a Angeforderter Anzeigeinhalt
   * @param richtung Richtung in der die Regel arbeitet
   * @param schaltgrund Schaltgrund unter dem diese Regel publiziert wird
   */
  public void addRegel (WVZInhalt x, List<WVZInhalt> xyz, List<WVZInhalt> uvw, WVZInhalt a, ArbeitsRichtungVN richtung, Aspect schaltgrund )
  {
    synchronized (m_parameter)
    {
      RegelLueckenFuellen r = new RegelLueckenFuellen ();
      
      r.setSchaltgrundRegel( schaltgrund );
      r.setAnzeigenInhalt( x );
      r.setAnzeigenInhaltVorgaengerNachfolger( xyz );
      r.setAnzeigenInhaltVorVorgaengerNachNachfolger( uvw );
      r.setAngeforderterAnzeigenInhalt( a );
      r.setRichtung( richtung );
      
      m_parameter.add( r );
    }
  }

  /**
   * Methode liefert den Inahlt der Klasse als String
   */
  public String toString ()
  {
    String s = "";
    
    synchronized (m_parameter)
    {
      s += "Regeln:\n";
      for (RegelLueckenFuellen r : m_parameter)
        s += r.toString() + "\n";
    }
    
    return s;
  }

  /**
   * Methode zum bernehmen der Parameter
   * @param parameter zu bernehmende Parameter
   */
  public void setParameter(Vector<RegelLueckenFuellen> parameter)
  {
    synchronized (m_parameter)
    {
      m_parameter.clear();
      
      for (RegelLueckenFuellen r : parameter)
         m_parameter.add(r);
    } 
  }
}
