package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.universal;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Vector;

import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Spinner;

import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Darstellung eines Composite mit unterschiedlichen Foldern fr die einzelnen
 * Anzeigetypen
 */
public class CompositeRegeln extends Composite
{  
	/**
	 * Aktuelle Parameter
	 */
  private ListeMitRegeln _aktuelleParameter;

  /**
   * Instanz mit den Foldern
   */
  private CTabFolder _tabFolder = null;

  /**
   * Pid des auslsenden Objekts
   */
	private String _pid;

	private Composite _composite;

	private int _anzahlRegeln;
	
	/**
	 * Compoiste mit den Algorithmus spezifischen Parametern
	 */
	private CompositeParameterUniversal[] _compositeAlg;

	/**
	 * Spinner Anzahl Regeln
	 */
	private Spinner _spinnerAnzahlRegeln;
	
  /**
   * Liste mit den unterschiedlichen Parametergruppen
   */
	private List<String> _list = new ArrayList<String>();


	/**
	 * Konstuktor der Klasse
	 * @param parent ausfrufendes Composite
	 * @param style Stil
	 * @param aktuell aktuelle Parameter
	 * @param pid Pid des Objekts mit den Parametern
	 */
  public CompositeRegeln (Composite parent, int style, ListeMitRegeln aktuell, String pid)
  {
    super (parent, style);
    
    _pid = pid;
    
    _aktuelleParameter = aktuell;
    
    initialize();
  }

  /**
   * Fhrt die Basis-Initialisierung des Composites durch
   */
  private void initialize()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.heightHint = -1;
    gridData1.widthHint = -1;
    gridData1.horizontalIndent = 0;
    gridData1.verticalAlignment = GridData.FILL;
    
    setLayout(new GridLayout(1, false));
    setLayoutData(gridData1);
    
    _composite = new Composite(this, SWT.NONE);
    _composite.setLayout(new GridLayout(1, false));
    _composite.setLayoutData(gridData1);
  	
    createSpinnerAnzahlRegeln();
    
    createCompositeTab();
    
    int anzahl = bestimmeAnzahlFahrstreifen();
    erzeugeBenoetigteReiter(anzahl);


//    erzeugeTabItems (anzeigeTypen);
    
    /*
     * Daten einblenden
     */
    
//    for (SystemObject so : anzeigeTypen)
//      verarbeiteDaten(so);
    
  }

  /**
   * Methode erteugt die Darstellung der Auswahl der Messquerschnitte
   * 
   */
  private void createSpinnerAnzahlRegeln()
  {
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = false;

    Group groupFahrstreifen = new Group(_composite, SWT.NONE);
    groupFahrstreifen.setText("Anzahl Regeln");
    groupFahrstreifen.setLayout(new GridLayout(1, false));
    groupFahrstreifen.setLayoutData(gridData2);

    Composite compositeSpinnerFahrstreifen = new Composite(groupFahrstreifen, SWT.NONE);
    compositeSpinnerFahrstreifen.setLayout(new GridLayout(1, false));

  	_spinnerAnzahlRegeln = new Spinner(compositeSpinnerFahrstreifen, SWT.BORDER);
  	_spinnerAnzahlRegeln.setSelection(_anzahlRegeln);
  	
  	_spinnerAnzahlRegeln.addSelectionListener(new SelectionListener()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				if (e.getSource() instanceof Spinner)
				{
					Spinner sp = (Spinner) e.getSource();
					int selection = sp.getSelection();
					
					erzeugeBenoetigteReiter (selection);
				}
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent e)
			{
			}
		});
  }

  /**
   * Methode legt die bentigte Anzahl der Reiter an
   * @param anzahl Anzahl bentigter Reiter (= Anzahl Regeln)
   */
  private void erzeugeBenoetigteReiter (int anzahl)
  {
  	if (anzahl == _anzahlRegeln)
  		return;
  	
  	/*
  	 * Beim ersten Anlegen der Tabelle wird Reiter 0 markiert, bei nderung der Anzahl der
  	 * Reiter ber den Spinner wird der letzten Reiter markiert.
  	 */
  	int indexAnwahl = 0;
  	if (_compositeAlg != null)
  		indexAnwahl = anzahl - 1;

		if (_spinnerAnzahlRegeln != null)
			_spinnerAnzahlRegeln.setSelection(anzahl);
		
		/*
		 * Namen der Reiter anpassen
		 */
		_list.clear();
    for (int i=0; i<anzahl; ++i)
      _list.add ("Regel " + (i+1));
    
    /*
     * Bisherige Composite temp. speichern
     */
		Vector<CompositeParameterUniversal> tmp = new Vector<CompositeParameterUniversal>();
		if (_compositeAlg != null)
			for (CompositeParameterUniversal cpa : _compositeAlg)
				tmp.add(cpa);

		/*
		 * Nicht bentigte Reiter entfernen
		 */
	  CTabItem[] items = _tabFolder.getItems();
	  for (int i=items.length - 1; i >= anzahl; i--)
		  items [i].dispose();

		/*
		 * Feld mit den Composites erweitern bzw. reduzieren
		 */
  	_compositeAlg = new CompositeParameterUniversal [_list.size()];
  	for (int i=0; i<tmp.size() && i<anzahl; ++i)
  		_compositeAlg [i] = tmp.get(i);
		
		/*
		 * Reiter hinzufgen
		 */
		if (anzahl > _anzahlRegeln)
		{
		  for (int i=_anzahlRegeln; i<anzahl; ++i)
		  {
		  	erzeugeTabItem(_list.get(i));
		  	updataDataAlgorithmus (i);
		  }
		}
    
	  _anzahlRegeln = anzahl;

	  if (indexAnwahl >= 0)
  	  _tabFolder.setSelection( indexAnwahl );

    _tabFolder.redraw();
  }

  private void erzeugeTabItem (String name)
  { 	
    String so = name;
    
    CTabItem tabItem = new CTabItem( _tabFolder, SWT.NONE );

    String tabName = so;
    tabItem.setText( tabName );
    tabItem.setData( tabName, so );

    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.verticalAlignment = GridData.FILL;

    Composite c1 = new Composite( _tabFolder, SWT.NONE );
    c1.setLayout( new GridLayout() );
    c1.setLayoutData( gridData2 );

    tabItem.setControl( c1 );
  }

  /**
   * Updatefunktion fr die Algorithmus spezifischen Parameter
   */
  private void updataDataAlgorithmus (int indexReiter)
  {
		CTabItem[] items = _tabFolder.getItems();
    CTabItem item = items [indexReiter];
    
    if (item != null )
    {
      Composite c = (Composite) item.getControl();
      Control[] children = c.getChildren();
      CompositeParameterUniversal compPar = null;

      RegelUniversal para = null;
      
      if (_aktuelleParameter.getRegeln().size() > indexReiter)
        para = _aktuelleParameter.getRegeln().get(indexReiter);
      else
      {
      	para = new RegelUniversal();
      	para.setSchaltgrundRegel(VerwaltungParaUniversal.getInstance().getSchaltgrundAllgemein());
      }
      
      if ( children.length == 0 )
      {
        compPar = new CompositeParameterUniversal( c, SWT.None, para );
        _compositeAlg [indexReiter] = compPar;
        
        compPar.layout( true );
        c.layout( true );
        
        compPar.refresh();
      }
      else
      {
        compPar = (CompositeParameterUniversal) children[0];
        compPar.update ( para );
//        
//        compPar.layout( true );
//        c.layout( true );
//
//        compPar.getTvParameter().refresh();
      }
    }
  }

  /**
   * Methode bestimmt die Anzahl der Fahrstreifen aus dem Ergebnis des DaV
   * @return Anzahl Fahrstreifen
   */
  private int bestimmeAnzahlFahrstreifen ()
  {
  	int anzahl = _aktuelleParameter.getRegeln().size();
  	
    return anzahl;
  }
    
  /**
   * Methode legt das Composite mit den Foldern an
   */
  private void createCompositeTab()
  {
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.FILL;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.grabExcessVerticalSpace = true;
    gridData3.verticalAlignment = GridData.FILL;

    _tabFolder = new CTabFolder( _composite, SWT.NONE );
    _tabFolder.setLayout( new GridLayout() );
    _tabFolder.setLayoutData( gridData3 );

    _tabFolder.addSelectionListener( new SelectionListener()
    {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        CTabFolder folder = (CTabFolder) e.getSource();
        CTabItem itemSel = folder.getSelection();

        aktualisiereFolder(itemSel);
      }
      
      @Override
      public void widgetDefaultSelected(SelectionEvent e)
      {
      }
    });
  }

  /**
   * Methode zum Aktualisieren aller Folder
   */
  public void aktualisiereAlleFolder ()
  {
    CTabItem[] items = _tabFolder.getItems();
    for (CTabItem itemSel : items)
    	aktualisiereFolder(itemSel);
  }

  /**
   * Methode zum Aktualisieren eines Folder
   * @param itemSel selektierter Folder
   */
  public void aktualisiereFolder (CTabItem itemSel)
  {
    Composite cSel = (Composite) itemSel.getControl();

    if ( cSel.getChildren() != null && cSel.getChildren().length > 0 )
    {
      CompositeParameterUniversal compParSel = (CompositeParameterUniversal) cSel.getChildren()[0];
      compParSel.refresh();
    }
  }
  
  /**
   * Methode erzeugt die Tabs fr die einzelnen Anzeigetypen
   * @param anzeigeTypen Anzeigetypen
   */
  public void erzeugeTabItems( Vector<SystemObject> anzeigeTypen )
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    
    for (SystemObject so : anzeigeTypen)
    {
      CTabItem tabItem = new CTabItem( _tabFolder, SWT.NONE );
      tabItem.setText( so.getName() );
      tabItem.setData( so.getName(), so.getPid() );

      Composite c1 = new Composite( _tabFolder, SWT.NONE );
      c1.setLayout( new GridLayout() );
      c1.setLayoutData( gridData );

      tabItem.setControl( c1 );

    }
    _tabFolder.setSelection( 0 );
  }
  
  /**
   * Methode liefert die Tabellenviewer der einzlnen Tabellen	
   * @return Tabellenviewer der einzlnen Tabellen
   */
	public HashMap<String, TableViewer> getTableViewer ()
	{
		HashMap<String, TableViewer> tableViewer = new HashMap<String, TableViewer>();

		CTabItem[] items = _tabFolder.getItems();
	  for (CTabItem item : items)
	  {
	    Composite c = (Composite)item.getControl();
	    CompositeParameterUniversal compPar = (CompositeParameterUniversal) c.getChildren()[0];
	    
//	    TableViewer viewer = compPar.getTvAnzeige();
//      String pid = (String)item.getData(item.getText());
//      
//      tableViewer.put(pid, viewer);
	  }
		
		return tableViewer;
	}

	/**
	 * Methode zum bernehmen der Tabellenviewer der einzelnen Tabellen
	 * @param tableViewer zu bernehmende Viewer
	 */
	public void setTableViewer (HashMap<String, TableViewer> tableViewer)
	{
//		CTabItem[] items = _tabFolder.getItems();
//	  for (CTabItem item : items)
//	  {
//	    Composite c = (Composite)item.getControl();
//	    CompositeParameterUniversal compPar = (CompositeParameterUniversal) c.getChildren()[0];
//	    
//      String pid = (String)item.getData(item.getText());
//	    TableViewer viewer = tableViewer.get(pid);
//	    
//	    if (viewer != null)
//	    	compPar.setTvAnzeige(viewer);
//	  }
	}

	public ListeMitRegeln getParameter()
  {
		ListeMitRegeln parameter = new ListeMitRegeln();
		
  	/*
  	 * Daten der einzelnen Tabs einlesen
  	 */

	  CTabItem[] items = _tabFolder.getItems();
	  for (CTabItem item : items)
	  {
	    Composite c = (Composite)item.getControl();
	    CompositeParameterUniversal compPar = (CompositeParameterUniversal) c.getChildren()[0];

	    RegelUniversal regel = compPar.getParameter();
	    parameter.addRegel(regel);
	  }
	  
	  return parameter;
  }
}
