/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Swarco Traffic Systems GmbH
 * Adolf-Dambach-Strae
 * 76571 Gaggenau
 * Phone: +49-7225-64-01
 * Fax: +49-7225-64-300
 * mailto: info@els.dambach.de
 */

package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.universal;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.Vector;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.ReferenceArray;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.Richtung.ArbeitsRichtungQL;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.Richtung.ArbeitsRichtungVN;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dav.DaVKommunikationSBA;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.Konfig;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.universal.eigenschaften.UniversalEigenschaft1;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.universal.eigenschaften.UniversalEigenschaft10;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.universal.eigenschaften.UniversalEigenschaft11;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.universal.eigenschaften.UniversalEigenschaft2;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.universal.eigenschaften.UniversalEigenschaft3;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.universal.eigenschaften.UniversalEigenschaft4;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.universal.eigenschaften.UniversalEigenschaft5;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.universal.eigenschaften.UniversalEigenschaft6;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.universal.eigenschaften.UniversalEigenschaft7;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.universal.eigenschaften.UniversalEigenschaft8;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.universal.eigenschaften.UniversalEigenschaft9;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.universal.eigenschaften.UniversalEigenschaften;

/**
 * Klasse enthlt die Parameter fr die Regelklasse "Lngsabgleichs- und Querabgleichsregeln", Regel "Universal".<br>
 * Regel:<br>
 * Eine Reihe von Anzeigen, welche die folgenden Eigenschaften aufweist (whlbar je Anzeige und Anzeigenverknpfung):<br> 
 * Anzeigeinhalt ist "X" = ja / nein / beliebig<br>
 * Anzeigeinhalt des Vorintervalls ist "Y" = ja / nein / beliebig<br>
 * Hat keinen Vorgnger = ja / nein / beliebig<br>
 * Hat keinen Nachfolger = ja / nein / beliebig<br>
 * Ist vom WZGtyp "A" = ja / nein / beliebig<br>
 * Ist die Anzeige "4711" = ja / nein / beliebig<br>
 * Verweist auf Fahrstreifentyp "F" = ja / nein / beliebig<br>
 * Verweist auf die Fahrstreifen "4712" = ja / nein / beliebig<br<
 * Anzeige ausgefallen = ja / nein / beliebig<br>
 * WVZinhalt ["U", "V", "W"] ausgefallen = ja / nein / beliebig<br>
 * Anzahl Fahrstreifen AQ [ErsteAnzeigeDer Reihe] ist "n" = ja / nein / beliebig<br>
 * wird ersetzt durch die Reihe von Inhalten "1", "2", "3".<br> 
 * Dabei muss whlbar sein, dass der vorige Inhalt beibehalten wird, d.h. an einzelnen Anzeigen keine Ersetzung stattfindet. 
 * Alternativ dazu ist bei allen Anzeigen vom WZGTyp "B" auf allen beteiligten AQ der Inhalt "Y" anzufordern.
 * @version $Revision: 1.1 $ / $Date: 2013/06/25 09:27:08 $ / ($Author: ObertM $)
 * @author Swarco Traffic Systems GmbH
 * @author Matthias Obert
 *
 */
public class VerwaltungParaUniversal
{
  /**
   * Debug Ausgabe
   */
  private boolean _debug = false;

  /**
   * Pid der angeforderten Attributgruppe
   */
  private String _pidAtg = "atg.sswSbaUniversal";

  /**
   * Allgemeiner Schaltgrund fr die Regelklasse
   */
  private Aspect m_schaltgrundAllgemein;
  
  /**
   * Parametersatz fr die Anlage
   */
  private ListeMitRegeln m_parameterAnlage = new ListeMitRegeln();
  
  /**
   * Anlage fr die die SWE arbeitet
   */
  private String m_anlage;

  /**
   * Verwaltung der Listener
   */
  private PropertyChangeSupport m_pcs = new PropertyChangeSupport (this);

  /**
   * Mutex fr den synchronisierten Zugriff auf die Datenstruktur
   */
  private Object m_mutex = new Object();

  /**
   * Einzige Instanz der Klasse
   */
  private static VerwaltungParaUniversal _instance = null;

  /**
   * Pid des Defaultaspekts
   */
  public static String _pidDefaultAspekt = "asp.sswSbaUniversal";
  
  /**
   * Pid der SBA
   */
  private String _pidSBA;

  /**
   * Privater Konstruktor der Klasse
   */
  private VerwaltungParaUniversal ()
  {
  	_pidSBA = Konfig.getInstance().getPidSBA();
    
    m_schaltgrundAllgemein = Konfig.getInstance().getAspekt(_pidDefaultAspekt);
    
    /*
     * Datrn lesen ber DaVKommunikation
     */
  	final DaVKommunikationSBA kom = DaVKommunikationSBA.getInstanz();
  	kom.addDatenArt(_pidSBA, _pidAtg, "asp.parameterSoll", ReceiverRole.receiver());
  	
  	kom.addPropertyChangeListener(new PropertyChangeListener()
		{
			@Override
			public void propertyChange(PropertyChangeEvent evt)
			{
				if (evt.getPropertyName().equals(_pidAtg))
				{
					ResultData resultData = kom.getData(_pidAtg);
					if (resultData != null)
						auswerteAttributgruppe(resultData);
				}
			}
		});

		ResultData resultData = kom.getData(_pidAtg);
		if (resultData != null)
			auswerteAttributgruppe(resultData);
  }

  /**
   * Methode liefert die Singleton Instanz der Klasse
   * @return Singleton Instanz der Klasse
   */
  public static VerwaltungParaUniversal getInstance ()
  {
    if (_instance == null)
    {
      synchronized (VerwaltungParaUniversal.class)
      {
        if (_instance == null)
        {
          _instance = new VerwaltungParaUniversal ();
        }
      }
    }
    
    return _instance;
  }

  /**
   * Methode fgt einen Listener der Klasse hinzu
   * @param pcl Listern der Klasse PropertyChangeListener
   */
  public void addPropertyChangeListener (PropertyChangeListener pcl)
  {
    m_pcs.addPropertyChangeListener( pcl );    
  }
  
  /**
   * Methode entfernt einen Listener aus der Klasse
   * @param pcl Listern der Klasse PropertyChangeListener
   */
  public void removePropertyChangeListener (PropertyChangeListener pcl)
  {
    m_pcs.removePropertyChangeListener( pcl );    
  }

  /**
   * Methode benachrichtigt die Listener
   */
  private void benachrichtigeListener ()
  {
    m_pcs.firePropertyChange( "Aktualisierung", null, null );
    
    if (_debug)
      System.out.println(this.toString());
  }

  


  /**
   * Methode zum Auswerten der empfangenen Attributgruppen
   * @param results empfangenen Attributgruppen
   */
  public void auswerteAttributgruppen (ResultData[] results)
  {
    int anzahl = results.length;

    // Daten aus Eingangsqueue bearbeiten      
    
    for (int i = 0; i < anzahl; i++)
    {
      ResultData dat = results[i];
  
      if (dat.getDataDescription().getAttributeGroup().getPid().equals(_pidAtg))
      {
        auswerteAttributgruppe (dat);
        
        benachrichtigeListener();
      }
    }
  }
  
  /**
   * Methode zum Auswerten der Attributgruppe 'atg.sswSbaUniversal'
   * @param dat empfangender Datensatz
   */
  private void auswerteAttributgruppe( ResultData dat )
  {
    synchronized (m_mutex)
    {
      if (!dat.hasData())
      {
        m_schaltgrundAllgemein = null;
      }
      else
      {
        Data data = dat.getData();
          
        m_schaltgrundAllgemein = (Aspect) data.getReferenceValue( "SchaltgrundAllgemein" ).getSystemObject();
        
        Array a = data.getArray( "AnlageRegeln" );
        auswerteAttributgruppeAnlage( a );
      }
      
      m_schaltgrundAllgemein = Konfig.getInstance().getAspekt(_pidDefaultAspekt);
    }
  }

  /**
   * Methode zum Auswerten der fr die gesamte Anlage geltenden Parameter
   * @param array Array mit Parametern
   */
  private void auswerteAttributgruppeAnlage (Array array)
  {
    m_parameterAnlage.auswerteAttributgruppe( array, m_schaltgrundAllgemein );
  }

  /**
   * Methode fgt eine Regel fr die allgemeinen Parameter der Anlage hinzu
   * @param richtungVN Arbeitet Regel in Vorgnger- oder Nachfolgerrichtung 
   * @param richtungQL Arbeitet Regel in Quer- oder Lngsrichtung
   * @param anzeigen Reihe von Anzeigen welche die folgenden Eigenschaften aufweist
   * @param eigenschaften Eigenschaften fr die Regel
   * @param angeforderteInhalte angeforderte Inhalte der Anzeigen
   * @param alternativerAnzeigenTyp Anzeigentyp fr Alterntivschaltung
   * @param alternativerAnzeigenInhalt alternativer Anzeigeninhalt
   * @param schaltgrund Spezieller Schaltgrund fr die Regel
   */
  public void addRegel (
                         ArbeitsRichtungVN richtungVN,
                         ArbeitsRichtungQL richtungQL,
                         Vector<SystemObject> anzeigen,
                         Vector<UniversalEigenschaften> eigenschaften,
                         Vector<WVZInhalt> angeforderteInhalte,
                         SystemObject alternativerAnzeigenTyp,
                         WVZInhalt alternativerAnzeigenInhalt,
                         Aspect schaltgrund) 
   {
     m_parameterAnlage.addRegel( richtungVN, 
                                 richtungQL, 
                                 anzeigen, 
                                 eigenschaften, 
                                 angeforderteInhalte, 
                                 alternativerAnzeigenTyp, 
                                 alternativerAnzeigenInhalt, 
                                 schaltgrund );
   }
  
  /**
   * Methode liefert den Inhalt der Klasse als String
   */
  public String toString ()
  {
    String s = "";
    
    
    s += "Parameter Universal\n";
    s += "===================\n";
    
    s += "Parameter Anlage:\n";
    s += "=================\n";
    s += m_parameterAnlage.toString();

    return s;
  }

  /**
   * Methode liefert die Variable m_parameterAnlage zurck
   * @return Variable m_parameterAnlage
   */
  public ListeMitRegeln getParameterAnlage()
  {
    return m_parameterAnlage;
  }

  /**
   * Methode setzt die Variable m_parameterAnlage
   * @param parameterAnlage Wert auf den die Variable m_parameterAnlage gesetzt werden soll
   */
  public void setParameterAnlage( ListeMitRegeln parameterAnlage )
  {
    m_parameterAnlage = parameterAnlage;
  }

  /**
   * Methode zum Erzeugen einer Attributgruppe mit den Parametern
   * @return Attributgruppe mit den Parametern
   */
  public Data erzeugeAttributgruppe ()
  {
    ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();

    if (connection == null)
      return null;

    String pidAtg = _pidAtg;
    
    // Attributgruppe zusammensetzen und versenden
      
    AttributeGroup atg = (AttributeGroup) connection.getDataModel().getAttributeGroup(pidAtg);
    Data data = connection.createData(atg );

    data.getReferenceValue( "SchaltgrundAllgemein" ).setSystemObject( getSchaltgrundAllgemein() );
    
    Array anlageRegeln = data.getArray( "AnlageRegeln" );
    erzeugeAnlageRegeln (anlageRegeln);

    return data;
  }

  /**
   * Methode schreibt die Anlageregeln in ein Array einer Attributgruppe
   * @param array Array fr die Regeln
   */
  private void erzeugeAnlageRegeln( Array array )
  {
    Vector<RegelUniversal> regeln = getParameterAnlage().getRegeln();
    array.setLength( regeln.size() );
    
    for (int i=0; i<regeln.size(); ++i)
      erzeugeRegeln ( array.getItem( i ), regeln.get( i ) );
  }

  /**
   * Methode schreibt eine Regel in ein Datenfeld
   * @param data Datenfeld
   * @param regel Regel
   */
  private void erzeugeRegeln( Data data, RegelUniversal regel )
  {
    data.getReferenceValue( "SchaltgrundRegel" ).setSystemObject( regel.getSchaltgrundRegel() );
    
    data.getUnscaledValue ( "VorgngerNachfolger" ).set (regel.getRichtungVN().getKodierung());

    data.getUnscaledValue ( "QuerLngsRichtung" ).set (regel.getRichtungQL().getKodierung());

    /*
     * Anzeigen
     */

    int anzahlAnzeigen = regel.getAnzeigen().size();

    ReferenceArray a = data.getReferenceArray( "Anzeigen" );
    a.setLength( anzahlAnzeigen );
    
    for (int i=0; i<anzahlAnzeigen; ++i)
      a.getReferenceValue( i ).setSystemObject( regel.getAnzeigen().get( i ) );
    
    /*
     * Eigenschaften ...
     */

    int anzahlEigenschaften = regel.getEigenschaften().size();
    
    Array eigenschaften = data.getArray( "Eigenschaften" );
    eigenschaften.setLength( anzahlEigenschaften );
    
    for (int i=0; i<anzahlEigenschaften; ++i)
      erzeugeEigenschaften (eigenschaften.getItem( i ), regel.getEigenschaften().get( i ));
      
    /*
     * Angeforderte Inhalt
     */
    
    int anzahlAngeforderteInhalte = regel.getAngeforderteInhalte().size();
    
    Array angeforderteInhalte = data.getArray( "AngeforderteInhalte" );
    angeforderteInhalte.setLength( anzahlAngeforderteInhalte );
    
    for (int i=0; i<anzahlAngeforderteInhalte; ++i)
      regel.getAngeforderteInhalte().get( i ).erzeugeAttributgruppe( angeforderteInhalte.getItem( i ) );
    
    /*
     * Alternativer Anzeigetyp
     */
    
    data.getReferenceValue( "AlternativerAnzeigeTyp" ).setSystemObject( regel.getAlternativerAnzeigenTyp() );
    
    /*
     * Alternativer Anzeigeinhalt
     */
    
    regel.getAlternativerAnzeigenInhalt().erzeugeAttributgruppe( data.getItem( "AlternativerAnzeigenInhalt" ) );
    
  }

  private void erzeugeEigenschaften( Data data, UniversalEigenschaften eigenschaften )
  {
    /*
     * ... Anzeigeinhalt
     */
    
    Data data1 = data.getItem( "AnzeigeInhalt" );
    UniversalEigenschaft1 eigenschaft1 = eigenschaften.getEigenschaftAnzeigenInhalt();
    
    eigenschaft1.getAnzeigenInhalt().erzeugeAttributgruppe( data1.getItem( "Anzeigeinhalt" ));
    data1.getTextValue( "Kriterium" ).setText( eigenschaft1.getVergleich().getKennung() );

    /*
     * ... Anzeigeinhalt Vorintervall
     */
    
    Data data2 = data.getItem( "AnzeigeInhaltVorIntervall" );
    UniversalEigenschaft2 eigenschaft2 = eigenschaften.getEigenschaftAnzeigenInhaltVorIntervall();
    
    eigenschaft2.getAnzeigenInhalt().erzeugeAttributgruppe( data2.getItem( "Anzeigeinhalt" ));
    data2.getTextValue( "Kriterium" ).setText( eigenschaft2.getVergleich().getKennung() );
    
    /*
     * ... Vorgnger
     */
    
    Data data3 = data.getItem( "Vorgnger" );
    UniversalEigenschaft3 eigenschaft3 = eigenschaften.getEigenschaftVorgaenger();
    
    data3.getUnscaledValue( "Art" ).set(  eigenschaft3.getArt().getKodierung());
    data3.getTextValue( "Kriterium" ).setText( eigenschaft3.getVergleich().getKennung() );

    /*
     * ... Nachfolger
     */
    
    Data data4 = data.getItem( "Nachfolger" );
    UniversalEigenschaft4 eigenschaft4 = eigenschaften.getEigenschaftNachfolger();
    
    data4.getUnscaledValue( "Art" ).set(  eigenschaft4.getArt().getKodierung());
    data4.getTextValue( "Kriterium" ).setText( eigenschaft4.getVergleich().getKennung() );

    /*
     * ... WVZG Typ
     */
    
    Data data5 = data.getItem( "WZGTyp" );
    UniversalEigenschaft5 eigenschaft5 = eigenschaften.getEigenschaftWzgTyp();
    
    data5.getReferenceValue( "WZGTyp" ).setSystemObject( eigenschaft5.getWzgTyp() );
    data5.getTextValue( "Kriterium" ).setText( eigenschaft5.getVergleich().getKennung() );

    /*
     * ... Anzeige
     */
    
    Data data6 = data.getItem( "Anzeige" );
    UniversalEigenschaft6 eigenschaft6 = eigenschaften.getEigenschaftAnzeige();
    
    data6.getReferenceValue( "Anzeige" ).setSystemObject( eigenschaft6.getAnzeige() );
    data6.getTextValue( "Kriterium" ).setText( eigenschaft6.getVergleich().getKennung() );

    /*
     * ... Fahrstreifentyp
     */
    
    Data data7 = data.getItem( "FahrstreifenTyp" );
    UniversalEigenschaft7 eigenschaft7 = eigenschaften.getEigenschaftFahrstreifenTyp();
    
    data7.getUnscaledValue( "FahrstreifenTyp" ).set ( eigenschaft7.getFahrstreifenLage().getKodierung() );
    data7.getTextValue( "Kriterium" ).setText( eigenschaft7.getVergleich().getKennung() );

    /*
     * ... Fahrstreifen
     */
    
    Data data8 = data.getItem( "Fahrstreifen" );
    UniversalEigenschaft8 eigenschaft8 = eigenschaften.getEigenschaftFahrstreifen();
    
    data8.getReferenceValue( "Fahrstreifen" ).setSystemObject( eigenschaft8.getFahrstreifen()  );
    data8.getTextValue( "Kriterium" ).setText( eigenschaft8.getVergleich().getKennung() );

    /*
     * ... AusfallAnzeige
     */
    
    Data data9 = data.getItem( "AusfallAnzeige" );
    UniversalEigenschaft9 eigenschaft9 = eigenschaften.getEigenschaftAnzeigeAusgefallen();
    
    data9.getTextValue( "Kriterium" ).setText( eigenschaft9.getVergleich().getKennung() );

    /*
     * ... Ausfall WVZ Inhalt
     */
    
    Data data10 = data.getItem( "AusfallWVZInhalt" );
    UniversalEigenschaft10 eigenschaft10 = eigenschaften.getEigenschaftInhaltAusgefallen();
    
    int anzahl10 = eigenschaft10.getInhalte().size();
    
    Array a10 = data10.getArray( "WVZInhalt" );
    a10.setLength( anzahl10 );
    
    for (int i=0; i<anzahl10; ++i)
      eigenschaft10.getInhalte().get( i ).erzeugeAttributgruppe( a10.getItem( i ) );
    
    data10.getTextValue( "Kriterium" ).setText( eigenschaft10.getVergleich().getKennung() );

    /*
     * ... Anzahl Fahrstreifen
     */
    
    Data data11 = data.getItem( "AnzahlFahrstreifen" );
    UniversalEigenschaft11 eigenschaft11 = eigenschaften.getEigenschaftAnzahlFahrstreifen();
    
    data11.getUnscaledValue( "Anzahl" ).set ( eigenschaft11.getAnzahlFahrstreifen() );
    data11.getTextValue( "Kriterium" ).setText( eigenschaft11.getVergleich().getKennung() );
  }

	public Data getData()
  {
	  return erzeugeAttributgruppe();
  }

	/**
   * Methode liefert schaltgrundAllgemein zurck
   * @return schaltgrundAllgemein
   */
  public Aspect getSchaltgrundAllgemein()
  {
  	return m_schaltgrundAllgemein;
  }
}
