/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Adolf-Dambach-Strae
 * 76571 Gaggenau
 * Phone: +49-7225-64-01
 * Fax: +49-7225-64-300
 * mailto: info@els.dambach.de
 */

package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.zusammenfassungEnde;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;
import java.util.Map.Entry;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.Richtung.ArbeitsRichtungVN;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dav.DaVKommunikationSBA;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.Konfig;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.AnzeigeElementTyp;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;

/**
 * Klasse enthlt die Parameter fr die Regelklasse "Lngsabgleichs- und Querabgleichsregeln", Regel "Monotonie Kreuz/Pfeil".<br>
 * @version $Revision: 1.1 $ / $Date: 2013/09/26 12:17:27 $ / ($Author: ObertM $)
 * @author Swarco Traffic Systems GmbH
 * @author Matthias Obert
 *
 */
public class VerwaltungParaZusammenfassungEnde
{
	/**
	 * Einzige Instanz der Klasse
	 */
  private static VerwaltungParaZusammenfassungEnde _instance;

  /**
   * Debug Ausgabe
   */
  private boolean _debug = false;

  /**
   * Pid der angeforderten Attributgruppe
   */
  public static String _pidAtg = "atg.sswSbaZusammenfassungAufhebung";
  
  /**
   * Allgemeiner Schaltgrund fr die Regelklasse
   */
  private Aspect m_schaltgrundAllgemein;
  
  /**
   * Merker ob die regel angewendet werden soll
   */
  private boolean m_regelAnwenden;

  /**
   * Parametersatz fr die Anlage
   */
  private ListeMitRegeln m_parameterAnlage = new ListeMitRegeln();
  
  /**
   * Parametersatz pro Anzeigequerschnitt
   */
  private HashMap<SystemObject, ListeMitRegeln> m_parameterProAnzeigeQuerschnitt = new HashMap<SystemObject, ListeMitRegeln>();
  
  /**
   * Verwaltung der Listener
   */
  private PropertyChangeSupport m_pcs = new PropertyChangeSupport (this);

  /**
   * Mutex fr den synchronisierten Zugriff auf die Datenstruktur
   */
  private Object m_mutex = new Object();

  /**
   * Pid der Anlage
   */
  private String m_anlage;
  
  /**
   * Pid des Defaultaspekts
   */
  public static String _pidDefaultAspekt = "asp.sswSbaZusammenfassungEnde";
  
  /**
   * Pid der SBA
   */
  private String _pidSBA;

  /**
   * Privater Konstruktor der Klasse
   */
  private VerwaltungParaZusammenfassungEnde ()
  {
  	_pidSBA = Konfig.getInstance().getPidSBA();
    
    m_schaltgrundAllgemein = Konfig.getInstance().getAspekt(_pidDefaultAspekt);
    
    /*
     * Daten lesen ber DaVKommunikation
     */
  	final DaVKommunikationSBA kom = DaVKommunikationSBA.getInstanz();
  	kom.addDatenArt(_pidSBA, _pidAtg, "asp.parameterSoll", ReceiverRole.receiver());
  	
  	kom.addPropertyChangeListener(new PropertyChangeListener()
		{
			@Override
			public void propertyChange(PropertyChangeEvent evt)
			{
				if (evt.getPropertyName().equals(_pidAtg))
				{
					ResultData resultData = kom.getData(_pidAtg);
					if (resultData != null)
						auswerteAttributgruppe(resultData);
				}
			}
		});

		ResultData resultData = kom.getData(_pidAtg);
		if (resultData != null)
			auswerteAttributgruppe(resultData);
  }
  
  /**
   * Methode liefert die Singleton Instanz der Klasse
   * @return Singleton Instanz der Klasse
   */
  public static VerwaltungParaZusammenfassungEnde getInstance ()
  {
    if (_instance == null)
    {
      synchronized (VerwaltungParaZusammenfassungEnde.class)
      {
        if (_instance == null)
        {
          _instance = new VerwaltungParaZusammenfassungEnde ();
        }
      }
    }
    
    return _instance;
  }


  /**
   * Methode liefert den Schaltgrund fr die Regelklasse
   * @return Schaltgrund fr die Regelklasse
   */
  public Aspect getSchaltgrundAllgemein()
  {
    return m_schaltgrundAllgemein;
  }

  /**
   * Methode setzt des Schaltgrund fr dei Regelklasse
   * @param schaltgrund Schaltgrund fr dei Regelklasse
   */
  public void setSchaltgrundAllgemein( Aspect schaltgrund )
  {
    m_schaltgrundAllgemein = schaltgrund;
  }
  /**
   * Methode fgt einen Listener der Klasse hinzu
   * @param pcl Listern der Klasse PropertyChangeListener
   */
  public void addPropertyChangeListener (PropertyChangeListener pcl)
  {
    m_pcs.addPropertyChangeListener( pcl );    
  }
  
  /**
   * Methode entfernt einen Listener aus der Klasse
   * @param pcl Listern der Klasse PropertyChangeListener
   */
  public void removePropertyChangeListener (PropertyChangeListener pcl)
  {
    m_pcs.removePropertyChangeListener( pcl );    
  }

  /**
   * Methode benachrichtigt die Listener
   */
  private void benachrichtigeListener ()
  {
    m_pcs.firePropertyChange( "Aktualisierung", null, null );
    
    if (_debug)
      System.out.println(toString());
  }

  /**
   * Methode zum Auswerten der empfangenen Attributgruppen
   * @param results empfangenen Attributgruppen
   */
  public void auswerteAttributgruppen (ResultData[] results)
  {
    int anzahl = results.length;

    // Daten aus Eingangsqueue bearbeiten      
    
    for (int i = 0; i < anzahl; i++)
    {
      ResultData dat = results[i];
  
      if (dat.getDataDescription().getAttributeGroup().getPid().equals(_pidAtg))
      {
        auswerteAttributgruppe (dat);
        
        benachrichtigeListener();
      }
    }
  }
  
  /**
   * Methode zum Auswerten der Attributgruppe 'atg.sswSbaLckenFllen'
   * @param dat empfangender Datensatz
   */
  private void auswerteAttributgruppe( ResultData dat )
  {
    synchronized (m_mutex)
    {
      if (!dat.hasData())
      {
        m_schaltgrundAllgemein = null;
        m_regelAnwenden = false;
        m_parameterAnlage.clear();
        m_parameterProAnzeigeQuerschnitt.clear();
      }
      else
      {
        Data data = dat.getData();
          
        String jaNein = data.getItem( "RegelAnwenden" ).asTextValue().getText();
        m_regelAnwenden =  jaNein.equalsIgnoreCase( "Ja" );

        m_schaltgrundAllgemein = (Aspect) data.getReferenceValue( "SchaltgrundAllgemein" ).getSystemObject();
        
        Array a = data.getArray( "AnlageRegeln" );
        auswerteAttributgruppeAnlage( a );
        
        a = data.getArray( "AnzeigeQuerschnitt" );
        auswerteAttributgruppeQuerschnitt (a);
      }
      
      m_schaltgrundAllgemein = Konfig.getInstance().getAspekt(_pidDefaultAspekt);
    }
  }

  /**
   * Methode zum Auswerten der fr die gesamte Anlage geltenden Parameter
   * @param array Array mit Parametern
   */
  private void auswerteAttributgruppeAnlage (Array array)
  {
    m_parameterAnlage.auswerteAttributgruppe( array, getSchaltgrundAllgemein() );
  }

  /**
   * Methode zum Auswerten der Anzeigequerschnitt spezifischen Parameter
   * @param array Array mit Parametern fr die Anzeigequerschnitte
   */
  private void auswerteAttributgruppeQuerschnitt( Array array )
  {
    int anzahl = array.getLength(); 
    
    synchronized (m_parameterProAnzeigeQuerschnitt)
    {
      m_parameterProAnzeigeQuerschnitt.clear();
      for (int x = 0; x < anzahl; ++x)
      {
        Data d1 = array.getItem( x );
        
        SystemObject anzeigeQuerschnitt = d1.getReferenceValue( "AnzeigeQuerschnitt" ).getSystemObject();
        Array a = d1.getArray( "AnzeigeQuerschnittRegeln" );
        
        ListeMitRegeln regeln = new ListeMitRegeln();
        regeln.auswerteAttributgruppe( a, getSchaltgrundAllgemein() );

        m_parameterProAnzeigeQuerschnitt.put( anzeigeQuerschnitt, regeln );
      }
    }
  }

  /**
   * Methode fgt eine Regel hinzhu
   * @param anzeigenInhaltA Liste mit mglichen Inhalten fr Typ A
   * @param anzeigenInhaltB Liste mit mglichen Inhalten fr Typ B
   * @param angeforderterInhaltA zu schaltender Inhalt fr Typ A
   * @param angeforderterInhaltB zu schaltender Inhalt fr Typ B
   * @param prioritaetA Prioritt fr Typ A
   * @param prioritaetB Prioritt fr Typ B
   * @param schaltgrundRegel Schaltgrund
   */
  public void addRegelAllgemein( List<WVZInhalt> anzeigenInhaltA, List<WVZInhalt> anzeigenInhaltB, WVZInhalt angeforderterInhaltA, WVZInhalt angeforderterInhaltB, int prioritaetA, int prioritaetB, Aspect schaltgrundRegel )
  {
  	RegelZusammenfassungEnde r = new RegelZusammenfassungEnde();
  	
		r.setSchaltgrundRegel(schaltgrundRegel);
  	r.setAnzeigenInhaltA(anzeigenInhaltA);
  	r.setAnzeigenInhaltB(anzeigenInhaltB);
  	r.setAngeforderterInhaltA(angeforderterInhaltA);
  	r.setAngeforderterInhaltB(angeforderterInhaltB);
  	r.setPrioritaetA(prioritaetA);
  	r.setPrioritaetB(prioritaetB);
  	
  	m_parameterAnlage.addRegel(r);
  }
  
  /**
   * Methode liefert den Inhalt der Klasse als String
   */
  public String toString ()
  {
    String s = "";

    s += "Parameter Zusammenfassung Aufhebung\n";
    s += "===================================\n";

    s += "Parameter Anlage:\n";
    s += "=================\n";
    s += m_parameterAnlage.toString();

    s += "Parameter pro Anzeigequerschnitt:\n";
    s += "=================================\n";
    Iterator<Entry<SystemObject, ListeMitRegeln>> it = m_parameterProAnzeigeQuerschnitt.entrySet().iterator();
    while (it.hasNext())
    {
      Map.Entry<SystemObject, ListeMitRegeln> entry = it.next();

      SystemObject anzeigeQuerschnitt = (SystemObject) entry.getKey();

      s += Tools.getObjektName( anzeigeQuerschnitt ) + "\n";
      s += entry.getValue().toString();
    }

    return s;
  }

  /**
   * Methode zum Erzeugen einer Attributgruppe mit den Parametern
   * @return Attributgruppe mit den Parametern
   */
  public Data erzeugeAttributgruppe ()
  {
	  ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
    if (connection == null)
      return null;

    bereinigeParameter();
    
    String pidAtg = _pidAtg;
    
    // Attributgruppe zusammensetzen und versenden
      
    // Attributgruppe zusammensetzen und versenden
    
    AttributeGroup atg = (AttributeGroup) connection.getDataModel().getAttributeGroup(pidAtg);
    Data data = connection.createData(atg );

    data.getTextValue ( "RegelAnwenden" ).setText( isRegelAnwenden() ? "Ja" : "Nein" );
    
    data.getReferenceValue( "SchaltgrundAllgemein" ).setSystemObject( getSchaltgrundAllgemein() );
    
    Array anlageRegeln = data.getArray( "AnlageRegeln" );
    erzeugeAnlageRegeln (anlageRegeln);

    Array anzeigeQuerschnitt = data.getArray( "AnzeigeQuerschnitt" );
    erzeugeAnzeigeQuerschnittRegeln (anzeigeQuerschnitt);
    
    return data;
  }

  /**
   * Methode lscht leere Parameterfelder. Dies ist notwendig, da sonst die Steuerung Problem
   * mit der Bestimmung der richtigen Parameterstze hat
   */
  private void bereinigeParameter()
  {
  	bereinigeRegelProAnzeigeQuerschnitt (getParameterProAnzeigeQuerschnitt());
  }
  
  /**
   * Methode bereinigt die Parameter. Dabei werden Anzeigequerschnitte fr die keine Regeln definiert sind
   * aus dem Parametersatz entfern.
   * @param hashMap zu bereinigende Regeln
   */
  private void bereinigeRegelProAnzeigeQuerschnitt (HashMap<SystemObject, ListeMitRegeln> hashMap)
  {
  	List<SystemObject> zuLoeschen = new ArrayList<SystemObject>();
  	
	  Iterator<Entry<SystemObject, ListeMitRegeln>> it = hashMap.entrySet().iterator();
	  while (it.hasNext())
	  {
	  	Entry<SystemObject,ListeMitRegeln> entry = it.next();
	  	
	  	SystemObject anzeigeQuerschnitt = entry.getKey();
	  	ListeMitRegeln regeln = entry.getValue();
	  	
	  	if (regeln.getParameter().size() == 0)
	  		zuLoeschen.add (anzeigeQuerschnitt);
	  }

	  for (SystemObject so : zuLoeschen)
	  	hashMap.remove(so);
  }

  /**
   * Methode schreibt die Anlageregeln in ein Array einer Attributgruppe
   * @param array Array fr die Regeln
   */
  private void erzeugeAnlageRegeln( Array array )
  {
    erzeugeRegeln( array, getParameterAnlage() );
  }

  /**
   * Methode schreibt die Anzeigequerschnittregeln in ein Array einer
   * Attributgruppe
   * 
   * @param array
   *          Array fr die Regeln
   */
  private void erzeugeAnzeigeQuerschnittRegeln( Array array )
  {
    int anzahlAQ = m_parameterProAnzeigeQuerschnitt.size();

    array.setLength( anzahlAQ );

    int i = 0;
    Iterator<Entry<SystemObject, ListeMitRegeln>> it = m_parameterProAnzeigeQuerschnitt.entrySet().iterator();
    while (it.hasNext())
    {
      Entry<SystemObject, ListeMitRegeln> entry = it.next();

      SystemObject anzeigeQuerschnitt = entry.getKey();
      ListeMitRegeln regeln = entry.getValue();

      Data dataAQ = array.getItem( i++ );

      dataAQ.getReferenceValue( "AnzeigeQuerschnitt" ).setSystemObject( anzeigeQuerschnitt );
      Array arrayRegeln = dataAQ.getArray( "AnzeigeQuerschnittRegeln" );

      erzeugeRegeln( arrayRegeln, regeln );
    }
  }

  /**
   * Methode schreibt die Regeln pro Anzeigetyp in ein Array
   * 
   * @param array
   *          Array in das die Regeln geschrieben werden solle
   * @param rpat
   *          Regeln
   */
  private void erzeugeRegeln( Array array, ListeMitRegeln lmr )
  {
    Vector<RegelZusammenfassungEnde> parameter = lmr.getParameter();

    int anzahl = parameter.size();
    array.setLength( anzahl );

    for (int i = 0; i < anzahl; ++i)
    {
      Data d = array.getItem( i );

      RegelZusammenfassungEnde rv = parameter.get( i );

      d.getReferenceValue( "SchaltgrundRegel" ).setSystemObject( rv.getSchaltgrundRegel() );
      d.getReferenceValue( "AnzeigeTypA" ).setSystemObject( rv.getAnzeigeTypA() );
      d.getReferenceValue( "AnzeigeTypB" ).setSystemObject( rv.getAnzeigeTypB() );
      d.getUnscaledValue( "PriorittA" ).set( rv.getPrioritaetA() );
      d.getUnscaledValue( "PriorittB" ).set( rv.getPrioritaetB() );

      List<WVZInhalt> inhaltA = rv.getAnzeigenInhaltA();
      Array a = d.getArray( "AnzeigeinhaltTypA" );
      a.setLength( inhaltA.size() );
      for (int k = 0; k < inhaltA.size(); ++k)
      {
        WVZInhalt wi = inhaltA.get( k );
        wi.erzeugeAttributgruppe( a.getItem( k ) );
      }

      List<WVZInhalt> inhaltB = rv.getAnzeigenInhaltB();
      a = d.getArray( "AnzeigeinhaltTypB" );
      a.setLength( inhaltB.size() );
      for (int k = 0; k < inhaltB.size(); ++k)
      {
        WVZInhalt wi = inhaltB.get( k );
        wi.erzeugeAttributgruppe( a.getItem( k ) );
      }

      WVZInhalt anfoA = rv.getAngeforderterInhaltA();
      anfoA.erzeugeAttributgruppe( d.getItem( "AngeforderterAnzeigeInhaltA" ) );

      WVZInhalt anfoB = rv.getAngeforderterInhaltB();
      anfoB.erzeugeAttributgruppe( d.getItem( "AngeforderterAnzeigeInhaltB" ) );
    }
  }

  /**
   * Methode liefert die Variable m_parameterAnlage zurck
   * @return Variable m_parameterAnlage
   */
  public ListeMitRegeln getParameterAnlage()
  {
    return m_parameterAnlage;
  }

  /**
   * Methode liefert die Variable m_parameterProAnzeigeQuerschnitt zurck
   * @return Variable m_parameterProAnzeigeQuerschnitt
   */
  public HashMap<SystemObject, ListeMitRegeln> getParameterProAnzeigeQuerschnitt()
  {
    return m_parameterProAnzeigeQuerschnitt;
  }

  /**
   * Methode bernimmt die Parameter fr eine bestimmte Pid
   * @param pid Pid
   * @param regeln Regeln
   */
	public void setParameter(String pid, ListeMitRegeln regeln)
  {
    ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();

    /*
     * Parameter Anlage
     */
    if (pid.equals(_pidSBA))
    {
      m_parameterAnlage = regeln;
    }
    else if (connection != null)
    {
    	/*
    	 * Prfen ob Anzeigequerschnitt oder Anzeige
    	 */
    	SystemObject so = connection.getDataModel().getObject(pid);
    	if (so != null)
    	{
    		if (so.getType().getPid().equals("typ.anzeigeQuerschnitt"))
    			m_parameterProAnzeigeQuerschnitt.put(so, regeln);
    	}
    }
    
    benachrichtigeListener();
  }

  /**
   * Methode liefert die Parameter als Attributgruppe fr den DaV
   * @return Attributgruppe
   */
  public Data getData ()
  {
  	return erzeugeAttributgruppe();
  }

  /**
   * Methode liefert die Parameter zu einer bestimmten Pid
   * @param pid Pid des gesuchten Objekts, dies kann die Pid der SBA, die Pid eines Anzeigequerschnitts oder
   * die Pid einer Anzeige sein.
   * @return Parameter zu der Pid
   */
	public ListeMitRegeln getParameter(String pid)
  {
    /*
     * Parameter Anlage
     */
    if (pid.equals(_pidSBA))
      return getParameterAnlage();
    
    /*
     * Parameter Anzeigequerschnitt
     */
    for (SystemObject so : m_parameterProAnzeigeQuerschnitt.keySet())
    {
      if (pid.equals(so.getPid()))
        return m_parameterProAnzeigeQuerschnitt.get(so);
    }
    
    /*
     * Parametersatz noch nicht definiert
     */
   
    return new ListeMitRegeln();
  }

	/**
   * Methode liefert regelAnwenden zurck
   * @return regelAnwenden
   */
  public boolean isRegelAnwenden()
  {
  	return m_regelAnwenden;
  }

	/**
   * Methode setzt die Variable regelAnwenden
   * @param regelAnwenden zu bernehmender Wert fr regelAnwenden
   */
  public void setRegelAnwenden(boolean regelAnwenden)
  {
  	m_regelAnwenden = regelAnwenden;
  }
}
