package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.notwendigeschaltungen.absolutnotwendigeschaltung;

import java.util.ArrayList;
import java.util.List;
import java.util.Observable;
import java.util.Observer;
import java.util.Vector;

import org.eclipse.jface.viewers.CellLabelProvider;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.TableEditor;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;

import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.Konfig;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;

/**
 * Darstellung des Fensters zur Versorgung der Parameter
 */
public class CompositeParameterAbsolutNotwendigeSchaltung extends Composite
{
	private final String _anwenden = "Regelteil anwenden";

	private final String _nichtAnwenden = "Regelteil nicht anwenden";

	/**
	 * Aktuelles Composite
	 */
  private Composite _composite;

  /**
   * Composite der Tabelle
   */
  private Composite _compositeTabelle;

  /**
   * Gruppe mit den Regeln
   */
  private Group _groupRegeln;

  /**
   * Tabelviewer der Tabelle
   */
  private TableViewer _tvAnzeige;

  /**
   * Aktuelle Parameter
   */
  private RegelAbsolutNotwendigeSchaltung _aktuelleParameter;

  /**
   * Composite fr die Buttons
   */
	private Composite _compositeButtons;

	/**
	 * Button Hinzufgen
	 */
	private Button _buttonHinzufuegen;

	/**
	 * Button Lschen
	 */
	private Button _buttonLoeschen;

	/**
	 * Tabellenmodell
	 */
	private TableModel[] _tableModel;

	/**
	 * Pid des Objekts dessen Parameter verndert werden sollen
	 */
	private String _pidObjekt;
	
  /**
   * Konstruktor der Klasse
   * @param parent Parent Composite
   * @param style Stil des Fenstern
   * @param aktuell Aktuelle Parameter
   * @param pidAnzeigeTyp Pid des Anzeigetyps
   */
  CompositeParameterAbsolutNotwendigeSchaltung (Composite parent, int style, RegelAbsolutNotwendigeSchaltung aktuell, String pidObjekt)
  {
    super (parent, style);
    
    this._aktuelleParameter = erzeugeKopie(aktuell);
    
    this._pidObjekt = pidObjekt;
    
    initialize();
  }

  /**
   * Methode legt eine Arbeitskopie der akutellen Daten an
   * @param aktuell aktuelle Daten
   * @return Arbeitskopie
   */
  private RegelAbsolutNotwendigeSchaltung erzeugeKopie(RegelAbsolutNotwendigeSchaltung aktuell)
  {
	  RegelAbsolutNotwendigeSchaltung kopie = new RegelAbsolutNotwendigeSchaltung();
	 
	  if (aktuell != null)
	  {
	    kopie.setRegelAnwenden(aktuell.isRegelAnwenden());
	    kopie.setSchaltgrundRegel(aktuell.getSchaltgrundRegel());
	  }
	  else
	  {
	  	kopie.setSchaltgrundRegel(VerwaltungParaAbsolutNotwendigeSchaltung.getInstance().getSchaltgrundAllgemein());
	  	kopie.setRegelAnwenden(true);
	  }

	  return kopie;
  }

  /**
   * Fhrt die Basis-Initialisierung des Composites durch
   */
  private void initialize()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.heightHint = -1;
    gridData1.widthHint = -1;
    gridData1.horizontalIndent = 0;
    gridData1.verticalAlignment = GridData.FILL;
    
    setLayout(new GridLayout(1, false));
    setLayoutData(gridData1);
    
    _composite = new Composite(this, SWT.NONE);
    _composite.setLayout(new GridLayout(1, false));
    _composite.setLayoutData(gridData1);
 
    erzeugeKommentar();
    
    //Initialisierungen der Arrays
    
    List<TableModel[]> inputList = new ArrayList<TableModel[]>();
    
    erzeugeGruppeRegeln();
    
    if (!Konfig.getInstance().getPidSBA().equals(_pidObjekt))
      createButtons();
    
    Table table = _tvAnzeige.getTable();
    _tableModel = createModel1(table);
    
    if ( _tableModel != null )
      _tvAnzeige.setInput(_tableModel); 
    
    updateData(_aktuelleParameter);
  }

  /**
   * Methode blendet einen Kommentar zu dem Parametersatz ein
   */
  private void erzeugeKommentar ()
  {
  	String kommentar = "Notwendige Schaltung:\n\n" + 
  	"Die Regel reagiert darauf, dass ein notwendiger Inhalt physikalisch nicht darstellbar ist.\n\n" +
    "Wenn angefordeter Bildinhalt eines WVZ-Inhalts = \"Notwendig\"\n" +
    "und angefordeter Inhalt eines WVZ-Inhalts = \"nicht darstellbar\"\n" +
    "dann nchstes alternatives Schaltbild anfordern.\n\n";
  	
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    
    Group groupKommentar = new Group (_composite, SWT.NONE);
    groupKommentar.setLayout(new FillLayout());
    groupKommentar.setLayoutData(gridData1);

//    groupKommentar.setText("Erklrung");
    
    Label text = new Label (groupKommentar, SWT.NONE);
    text.setText(kommentar);
  }

  /**
   * Methode bernimmt die Daten und zeigt diese in der Tabelle an
   */
  private void updateData (RegelAbsolutNotwendigeSchaltung parameter)
  {
  	int anzahlParameter = 1;
  	
  	if (parameter == null)
  		anzahlParameter = 0;
  	
  	erzeugeTabellenModell (anzahlParameter);
  	
  	if (anzahlParameter == 1)
  	{
	  	TableModel tm = _tableModel [0];    	
	    RegelAbsolutNotwendigeSchaltung aktuell = parameter;
	    	
	  	/*
	  	 * Schaltgrund
	  	 */
	  	
	  	if (aktuell.getSchaltgrundRegel() == null)
	  	{
	  	  tm.setSelectedSchaltgrund("");
	  	}
	  	else
	  	{
		    tm.setSelectedSchaltgrund(aktuell.getSchaltgrundRegel().getName());
	  	}
	    	
	  	/*
	  	 * Regelteil anwenden
	  	 */
	  	
		  if (aktuell.isRegelAnwenden())
	  	  tm.setSelectedAnwenden(_anwenden);
	  	else
	  		tm.setSelectedAnwenden(_nichtAnwenden);
  	}
  }

  /**
   * Methode erzeugt ein Tabellenmodell einer bestimmten Gre. Dabei werden bereits vorhandene
   * Composite in der Tabelle weiter verwendet
   * @param anzahlParameter Gre des erzeugten Tabellenmodells
   */
  private void erzeugeTabellenModell (int anzahlParameter)
  {
  	/*
  	 * Hat sich die Gre gendert
  	 */
  	if (_tableModel.length != anzahlParameter)
  	{
  		/*
  		 * Tabelle erweitern
  		 */
  		if (_tableModel.length < anzahlParameter)
  		{
  		  Vector<TableModel> v = new Vector<TableModel>();
  		  
  		  for (int i=0; i<_tableModel.length; ++i)
  		  	v.add(_tableModel [i]);
  		  
  		  for (int i=_tableModel.length; i<anzahlParameter; ++i)
  		  	v.add (erzeugeZeileInTabelle(_tvAnzeige.getTable()));
  		  
  		  _tableModel = new TableModel [anzahlParameter];
  		  for (int i=0; i<v.size(); ++i)
  		  	_tableModel [i] = v.get(i);
  		}
  		/*
  		 * Tabelle reduzieren
  		 */
  		else if (_tableModel.length > anzahlParameter)
  		{
  			/*
  			 * Nicht mehr bentigte Elemente lschen
  			 */
  			
  			for (int i = _tableModel.length - 1; i > anzahlParameter - 1; i--)
  			{
  				_tableModel [i].getAnwenden().dispose();
  				_tableModel [i].getSchaltgrund().dispose();
  			}
  				
  		  Vector<TableModel> v = new Vector<TableModel>();
  		  for (int i=0; i<anzahlParameter; ++i)
  		  	v.add(_tableModel [i]);

  		  _tableModel = new TableModel [anzahlParameter];
  		  for (int i=0; i<v.size(); ++i)
  		  	_tableModel [i] = v.get(i);
  		}
  		
  	  _tvAnzeige.setInput(_tableModel);
  	  _tvAnzeige.getTable().layout(true);
  	}
  }
  
  /**
   * Methode erzeugt die Gruppe mit den Regeln
   */
  private void erzeugeGruppeRegeln()
  {
    FillLayout fillLayout1 = new FillLayout();
    fillLayout1.marginHeight = 5;
    fillLayout1.marginWidth = 5;
    fillLayout1.type = org.eclipse.swt.SWT.HORIZONTAL;
    
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.verticalAlignment = GridData.FILL;
    
    _groupRegeln = new Group(_composite, SWT.NONE);
    _groupRegeln.setText("Regeln");
    
    _groupRegeln.setLayout(fillLayout1);
    _groupRegeln.setLayoutData(gridData2);

    erzeugeTabelle();
  }

	/**
	 * Methode erzeugt das Composite mit der Tabelle
	 */
  private void erzeugeTabelle ()
  {
    FillLayout fillLayout1 = new FillLayout();
    fillLayout1.type = org.eclipse.swt.SWT.VERTICAL;
    
    _compositeTabelle = new Composite(_groupRegeln, SWT.None);
    _compositeTabelle.setLayout(fillLayout1);
    
    _tvAnzeige = new TableViewer(_compositeTabelle, SWT.HIDE_SELECTION | SWT.H_SCROLL | SWT.V_SCROLL);
    _tvAnzeige.setContentProvider(new TableContentProvider());

    TableViewerColumn column1 = new TableViewerColumn(_tvAnzeige, SWT.NONE);
    column1.getColumn().setWidth(200);
    column1.getColumn().setText("Schaltgrund");
    column1.setLabelProvider(new MyCellLabelProvider());

    TableViewerColumn column2 = new TableViewerColumn(_tvAnzeige, SWT.NONE);
    column2.getColumn().setWidth(200);
    column2.getColumn().setText("Regelteil anwenden");
    column2.setLabelProvider(new MyCellLabelProvider());

    // resize the row height using a MeasureItem listener
    Listener listener = new Listener() {
      @Override
      public void handleEvent(Event event)
      {
        switch (event.type)
        {
          case SWT.MeasureItem:
            event.height = 40;
            break;

          default:
        }

      }
    };
    
    _tvAnzeige.getTable().addListener(SWT.MeasureItem, listener);
    _tvAnzeige.getTable().setLinesVisible(true);
    _tvAnzeige.getTable().setHeaderVisible(true);
  }

  /**
   * Methode erzeugt das Composite mit den Buttons
   */
  private void createButtons()
  {
    RowLayout rowLayout = new RowLayout();
    rowLayout.pack = false;
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.END;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.verticalAlignment = GridData.CENTER;
    
    _compositeButtons = new Composite( _composite, SWT.NONE );
    _compositeButtons.setLayoutData( gridData3 );
    _compositeButtons.setLayout( rowLayout );

    _buttonLoeschen = new Button( _compositeButtons, SWT.NONE );
    _buttonLoeschen.setText( "Regel loeschen" );
    _buttonLoeschen.addMouseListener( new MouseAdapter()
    {
      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        aktionRegelLoeschen();
      }

    } );
  }
  
  /**
   * Methode zum Lschen der Regeln
   */
	private void aktionRegelLoeschen()
  {
    _aktuelleParameter = null;
    updateData(_aktuelleParameter);
  }


  /**
   * Klasse realsiert den TableContentProvider fr die Tabelle
   * @author ObertM
   */
  private class TableContentProvider implements IStructuredContentProvider, Observer
  {
  	/**
  	 * Tabelviewer
  	 */
    private TableViewer tableViewer;

    /**
     * Methode liefert die Elemente der Tabelle
     */
    public Object[] getElements(Object inputElement)
    {
			Object[] re = null;

			if (inputElement instanceof TableModel [])
			{
        re = (TableModel[]) inputElement;
			}
			
			return re;
    }

    /**
     * Dispose
     */
    public void dispose()
    {
    }

    /**
     * Methode wird aufgerufen wenn sich der Inhalt der Tabelle gendert hat
     */
    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
    {
      tableViewer = (TableViewer)viewer;

      if (oldInput != null)
      {
        TableModel[] tms = (TableModel[])newInput;

        if (tms != null)
        {
          for (TableModel tm : tms)
            tm.deleteObserver(this);
        }
      }
      
      if (newInput != null)
      {
        TableModel[] tms = (TableModel[])newInput;
        if (tms != null)
        {
          for (TableModel tm : tms)
            tm.addObserver(this);
        }
      }
    }

    @Override
    public void update(Observable arg0, Object arg1)
    {
      if (tableViewer != null)
      {
        tableViewer.refresh();
      }
    }
  }

  /**
   * Erzeugt ein TabellenModell fr den Inhalt der Tabellen
   * @param table
   * @return TabellenModell
   */
  private TableModel[] createModel1 (Table table)
  {
    int tabLength = getAnzahlParameter();
    int elIndex   = 0;
    
    TableModel[] elements = new TableModel [tabLength];
    
    if (elements.length == 0)
    	return elements;

    for (int x=0; x<getAnzahlParameter(); ++x)
    {
	    elements[elIndex] = erzeugeZeileInTabelle(table);
	    elIndex++;
    }
    
    return elements;
  }

  /**
   * Methode erzeugt ein Tabellenmodell fr eine Zeile in der Tabelle
   * @param table Tabelle 
   * @return Tabellenmodell fr eine Zeile
   */
  private TableModel erzeugeZeileInTabelle (Table table)
  {
    /*
     * Layout festlegen
     */
  	
    GridData gd0 = new GridData();
    gd0.horizontalAlignment = GridData.CENTER;
    gd0.verticalAlignment = GridData.CENTER;
    gd0.grabExcessHorizontalSpace = true;
    gd0.grabExcessVerticalSpace = true;

    GridData gd1 = new GridData();
    gd1.horizontalAlignment = GridData.CENTER;
    gd1.verticalAlignment = GridData.CENTER;
    gd1.grabExcessHorizontalSpace = true;
    gd1.grabExcessVerticalSpace = true;

    GridData gdTableItem = new GridData();
    gdTableItem.horizontalAlignment = GridData.FILL;
    gdTableItem.grabExcessHorizontalSpace = true;
    gdTableItem.verticalAlignment = GridData.CENTER;

    /*
     * Combobox Schaltgrund    
     */
    Composite comp0 = new Composite( table, SWT.NONE );
    comp0.setBackground( table.getBackground() );
    comp0.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp0.setLayoutData( gd0 );
    comp0.setLayout( new GridLayout() );
    
    Combo combo0 = new Combo( comp0, SWT.BORDER );
    combo0.setLayoutData( gdTableItem );
    
    Vector<SystemObject> aspekte = Tools.getAspekte();
    String [] sa = new String [aspekte.size()];
    for (int i=0; i<aspekte.size(); ++i)
      sa [i] = aspekte.get(i).getName();
    
    combo0.setItems(sa);
    combo0.select(0);

    /*
     * Combobox Regelteil anwenden    
     */
    Composite comp1 = new Composite( table, SWT.NONE );
    comp1.setBackground( table.getBackground() );
    comp1.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp1.setLayoutData( gd1 );
    comp1.setLayout( new GridLayout() );
    
    Combo combo1 = new Combo( comp1, SWT.BORDER );
    combo1.setLayoutData( gdTableItem );
    
    String [] sa1 = new String [2];
    sa1 [0] = _anwenden;
    sa1 [1] = _nichtAnwenden;
    
    combo1.setItems(sa1);
    combo1.select(0);

    return new TableModel( comp0, comp1);
  }
  
  /**
   * Methode liefert die Anzahl der Parameter
   * @return Anzahl der Parameter
   */
  public int getAnzahlParameter()
  {
    return 1;
  }
  
  /**
   * Methode liefert den Tabelviewer
   * @return
   */
  public TableViewer getTvAnzeige()
  {
    return _tvAnzeige;
  }

  /**
   * Renderer fr die Tabelle
   * @author ObertM
   */
  private class MyCellLabelProvider extends CellLabelProvider 
  {
    @Override
    public void update(ViewerCell cell)
    {
      TableItem item = (TableItem)cell.getItem();

//      if (item.getData("EDITOR") != null)
//      {
//        TableEditor editor = (TableEditor)item.getData("EDITOR");
//        editor.dispose();
//      }

      int cnt = cell.getColumnIndex();

      TableModel model = (TableModel)item.getData();
      Composite comp = null;
      switch (cnt)
      {
        case 0:
          comp = model.getSchaltgrund();
          break;
          
        case 1:
          comp = model.getAnwenden();
          break;
          
        default:
      }
      
      TableEditor editor = new TableEditor(item.getParent());
      editor.grabHorizontal = true;
      editor.setEditor(comp, item, cnt);

    }
  }

  /**
   * Methode liefert die eingestellten Parameter
   * @return eingestellte Parameter
   */
	public RegelAbsolutNotwendigeSchaltung getParameter()
  {
 		return leseAktuelleEinstellungen();
  }
		
	private RegelAbsolutNotwendigeSchaltung leseAktuelleEinstellungen ()
	{
		if (_tableModel.length == 0)
			return null;
		
    RegelAbsolutNotwendigeSchaltung regel = new RegelAbsolutNotwendigeSchaltung();		

    TableModel tm = _tableModel [0];
      
    /*
     * Schaltgrund
     */
    
    String nameSchaltgrund = tm.getSelectedSchaltgrund();
    Aspect schaltgrund = Tools.getAspekt(nameSchaltgrund);
      
    /*
     * Anwenden
     */
    boolean anwenden = false;
    String tmp = tm.getSelectedAnwenden();
    anwenden = tmp.equals(_anwenden);

    regel.setSchaltgrundRegel(schaltgrund);
    regel.setRegelAnwenden(anwenden);

		return regel;
  }

	/**
	 * Methode bernimmt die Daten eines anderen Folders
	 * @param tvAnzeige Tableviewer der zu bernhmenden Tabelle
	 */
	public void setTvAnzeige (TableViewer tvAnzeige)
	{
    TableModel[] modelSet = (TableModel[]) tvAnzeige.getInput();

    RegelAbsolutNotwendigeSchaltung regel = new RegelAbsolutNotwendigeSchaltung();
    
    if (modelSet.length == 0)
    {
    	_aktuelleParameter = null;
    }
    else
    {
	    /*
	     * Schaltgrund
	     */
	    
	    String nameSchaltgrund = modelSet [0].getSelectedSchaltgrund();
	    Aspect schaltgrund = Tools.getAspekt(nameSchaltgrund);
	      
	    /*
	     * Anwenden
	     */
	    boolean anwenden = false;
	    String tmp = modelSet [0].getSelectedAnwenden();
	    anwenden = tmp.equals(_anwenden);
	
	    _aktuelleParameter.setSchaltgrundRegel(schaltgrund);
	    _aktuelleParameter.setRegelAnwenden(anwenden);
    }

    updateData(_aktuelleParameter);
  }
	
  /**
   * Methode zum Aktualisieren aller Folder
   */
  public void aktualisiereFolder ()
  {
    TableViewer tvParameter = getTvAnzeige();          
    tvParameter.refresh();
  }
}
