package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.prio;

import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;

/**
 * Klasse enthlt eine Regel fr die Priorisierung. Eine Regel besteht dabei aus einem WVZ Inhalt,
 * eine Prioritt und einem Aspekt unter dem die Prioritt gilt. Die Defaultprioritt hat den Aspekt null.
 * @author   Dambach Werke GmbH
 */
class RegelPriorisierungNeu
{
	/**
	 * WVZ Inhalt fr den die Regel gelten soll
	 */
	private WVZInhalt _wvzInhalt;
	
  /**
   * Prioritt
   */
  private int _prioritaet = -1;
  
  /**
   * Schaltgrund
   */
  private Aspect _schaltgrund = null;

  /**
   * Konstruktor der Klasse
   * @param inhalt WVZ-Inhalt
   * @param prioritaet Prioritt
   * @param schaltgrund Schaltgrund
   */
	public RegelPriorisierungNeu(WVZInhalt inhalt, int prioritaet, Aspect schaltgrund)
  {
		_wvzInhalt = inhalt;
		
		_prioritaet = prioritaet;
		
		_schaltgrund = schaltgrund;
  }

  /**
   * Konstruktor der Klasse
   * @param inhalt WVZ-Inhalt
   * @param prioritaet Prioritt
   */
	public RegelPriorisierungNeu(WVZInhalt inhalt, int prioritaet)
  {
		_wvzInhalt = inhalt;
		
		_prioritaet = prioritaet;
		
		_schaltgrund = null;
  }

	/**
   * Methode liefert wvzInhalt zurck
   * @return wvzInhalt
   */
  public WVZInhalt getWvzInhalt()
  {
  	return _wvzInhalt;
  }

	/**
   * Methode setzt die Variable wvzInhalt
   * @param wvzInhalt zu bernehmender Wert fr wvzInhalt
   */
  public void setWvzInhalt(WVZInhalt wvzInhalt)
  {
  	_wvzInhalt = wvzInhalt;
  }

	/**
   * Methode liefert prioritaet zurck
   * @return prioritaet
   */
  public int getPrioritaet()
  {
  	return _prioritaet;
  }

	/**
   * Methode setzt die Variable prioritaet
   * @param prioritaet zu bernehmender Wert fr prioritaet
   */
  public void setPrioritaet(int prioritaet)
  {
  	_prioritaet = prioritaet;
  }

	/**
   * Methode liefert schaltgrund zurck
   * @return schaltgrund
   */
  public Aspect getSchaltgrund()
  {
  	return _schaltgrund;
  }

	/**
   * Methode setzt die Variable schaltgrund
   * @param schaltgrund zu bernehmender Wert fr schaltgrund
   */
  public void setSchaltgrund(Aspect schaltgrund)
  {
  	_schaltgrund = schaltgrund;
  }
  
  @Override
  public String toString ()
  {
  	String s = "";
  	
  	s += "Inhalt: " + getWvzInhalt().toString() + "\n";
  	
  	if (getSchaltgrund() != null)
      s += "Schaltgrund: " + getSchaltgrund().getPid() + "\n";
    s += "Prio: " + getPrioritaet() + "\n";
  	
  	return s;
  }
}
