package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.prio;

import java.util.Observable;
import java.util.Observer;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Spinner;

/**
 * Tabellen Modell fr die Parameter Trichterung
 * @author ObertM
 *
 */
public class TableModel extends Observable implements Observer, Comparable<TableModel>
{
	/**
	 * Merker zum Lschen der Zeile
	 */
	private Composite _loeschen;
	
	/**
	 * Schaltgrund
	 */
  private Composite _schaltgrund;
  
  /**
   * Parameter X
   */
  private Composite _x;
  
  /**
   * Prioritt
   */
  private Composite _prioritaet;
  
  /**
   * Konstruktor der Klasse
   * @param loeschen Loeschen
   * @param schaltgrund Schaltgrund
   * @param x X
   * @param prio Prioritt
   */
  public TableModel(Composite loeschen, Composite x, Composite schaltgrund, Composite prio)
  {
  	this._loeschen = loeschen;
    this._schaltgrund = schaltgrund;
    this._x = x;    
    this._prioritaet = prio;
  }

  public String getSelectedLoeschen()
  {
    return getSelectedWert(_loeschen);    
  }

  public int getSelectedPrioritaet()
  {
    Control control = _prioritaet.getChildren()[0];
    if ( control instanceof Spinner )
    {
      Spinner sp = (Spinner)control;
      int intPrio = sp.getSelection();
      return intPrio;
    }
    else
      return 0;
  }

  public String getSelectedSchaltgrund ()
  {
    return getSelectedWert(_schaltgrund);    
  }

  public String getSelectedX ()
  {
    return getSelectedWert(_x);    
  }

  public void setSelectedPrioritaet(String selWert)
  {
    setSelectedWert(_prioritaet, selWert, null);
  }

  public void setSelectedLoeschen(String selWert)
  {
    setSelectedWert(_loeschen, selWert, null);
  }

  public void setSelectedSchaltgrund(String selWert)
  {
    setSelectedWert(_schaltgrund, selWert, null);
  }

  public void setSelectedX(Image selImage)
  {
    setSelectedWert(_x, null, selImage);
  }

  public String getSelectedWert(Composite c)
  {
    Control control = c.getChildren()[0];
    if ( control instanceof Spinner )
    {
      Spinner sp = (Spinner)control;
      int spSel = sp.getSelection();
      return String.valueOf(spSel);
    }
    else if ( control instanceof Combo )
    {
      Combo cb = (Combo)control;
      String  cbSel = null;
      
      try
      {
        cbSel = cb.getItem( cb.getSelectionIndex() );
      }
      catch (Exception e) 
      {
			}
      return cbSel;
    }
    else if ( control instanceof Button )
    {
      if ( (control.getStyle() & SWT.CHECK) != 0 )
      {
        Button ch = (Button)control;
        boolean chSel = ch.getSelection();
        return String.valueOf(chSel);
      }
      else
        return "";
    }
    else if ( control instanceof Label )
    {
      Label im = (Label)control;
      String imSel = (String)im.getData();
      return imSel;
    }
    else
      return "";
  }

  public void setSelectedWert(Composite c, String selWert, Image selImage)
  {
    Control control = c.getChildren()[0];
    if ( control instanceof Spinner )
    {
      Spinner sp = (Spinner)control;
      try
      {
        sp.setSelection(Integer.valueOf( selWert ));
      }
      catch (NumberFormatException e)
      {
        e.printStackTrace();
      }
    }
    else if ( control instanceof Combo )
    {
      Combo cb = (Combo)control;
      for (int k = 0; k < cb.getItemCount(); k++)
      {
        if (selWert.equals(cb.getItem( k )))
          cb.select(k);      
      }
    }
    else if ( control instanceof Button )
    {
      if ( (control.getStyle() & SWT.CHECK) != 0 )
      {
        Button ch = (Button)control;
        ch.setSelection( Boolean.valueOf( selWert ) );
      }
    }
    else if ( control instanceof Label )
    {
        Label im = (Label)control;
        im.setData( selWert );
        im.setImage( selImage ); 
    }
  }

  @Override
  public void update(Observable o, Object arg)
  {
    fireEvent();
  }

  public void fireEvent()
  {
    setChanged();
    notifyObservers();
  }

	/**
   * Methode liefert schaltgrund zurck
   * @return schaltgrund
   */
  public Composite getSchaltgrund()
  {
  	return _schaltgrund;
  }

	/**
   * Methode setzt die Variable schaltgrund
   * @param schaltgrund zu bernehmender Wert fr schaltgrund
   */
  public void setSchaltgrund(Composite schaltgrund)
  {
  	_schaltgrund = schaltgrund;
  }

	/**
   * Methode liefert x zurck
   * @return x
   */
  public Composite getX()
  {
  	return _x;
  }

	/**
   * Methode setzt die Variable x
   * @param x zu bernehmender Wert fr x
   */
  public void setX(Composite x)
  {
  	_x = x;
  }

	/**
   * Methode liefert loeschen zurck
   * @return loeschen
   */
  public Composite getLoeschen()
  {
  	return _loeschen;
  }

	/**
   * Methode liefert prioritaet zurck
   * @return prioritaet
   */
  public Composite getPrioritaet()
  {
  	return _prioritaet;
  }

	/**
   * Methode setzt die Variable prioritaet
   * @param prioritaet zu bernehmender Wert fr prioritaet
   */
  public void setPrioritaet(Composite prioritaet)
  {
  	_prioritaet = prioritaet;
  }

	/**
   * Methode setzt die Variable loeschen
   * @param loeschen zu bernehmender Wert fr loeschen
   */
  public void setLoeschen(Composite loeschen)
  {
  	_loeschen = loeschen;
  }
  
  @Override
  public int compareTo( TableModel arg )
  {
    if( getSelectedPrioritaet() < arg.getSelectedPrioritaet() )
      return -1;
  if( getSelectedPrioritaet() > arg.getSelectedPrioritaet() )
      return 1;

    return 0;
  }

}
