package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.steuerungskern;

import java.util.ArrayList;
import java.util.List;
import java.util.Observable;
import java.util.Observer;

import org.eclipse.jface.viewers.CellLabelProvider;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.TableEditor;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;

/**
 * Darstellung des Fensters zur Parameternderung ohne ButtonLeiste
 * 
 * @author Krause
 *
 */
public class CompositeIterationen extends Composite
{  
  /**
   * Parametergruppen der Attributgruppe des DAV
   */
  private static final String[] _itemGroups = 
  {
    "Iterationen",  "Zeitraster",
  };

  /**
   * ParameterElemente des DAV, die als Bild angezeigt werden
   * 
   * der 1.Index entpricht dabei dem Index der zugehrigen Parametergruppe
   */
  private static final String[][] _parItemImage = {
    {}, {},
  };
  
  /**
   * ParameterElemente des DAV, die als Spinner angezeigt werden
   * 
   * der 1.Index entpricht dabei dem Index der zugehrigen Parametergruppe
   */
  private static final String[][] _parItemSpin = {
    {"Iterationen"}, {"Zeitraster", "Offset"}
  };

  /**
   * ParameterElemente des DAV, die als Combo-Box angezeigt werden
   * 
   * der 1.Index entpricht dabei dem Index der zugehrigen Parametergruppe
   */
  private static final String[][] _parItemCombo = {
    {}, {},
  };

  /**
   * ParameterElemente des DAV, die als Check-Box angezeigt werden 
   * 
   * der 1.Index entpricht dabei dem Index der zugehrigen Parametergruppe
   */
  private static final String[][] _parItemCheck = {
    {}, {},
  };
  
  private Composite _composite;

  //Die Anzahl dieser Elemente entspricht der Anzahl der Parametergruppen
  private Composite[] _composite1;

  private Group[] _groupFilterDef;

  private TableViewer[] _tvAnzeige;

  /**
   * Anzahl Iterationen
   */
	private long _anzahlIterationen;

	/**
	 * Zeitraster
	 */
	private long _zeitraster;

	/**
	 * Offset
	 */
	private long _offset;
  
	
  CompositeIterationen(Composite parent, int style, ParameterSteuerungsKern parameter) 
  {
    super(parent, style);
   
    _anzahlIterationen = parameter.getAnzahlIterationen();
    
    _zeitraster = parameter.getParaZeitraster().getZeitraster();
    
    _offset = parameter.getParaZeitraster().getOffset();
    
    initialize();
  }

	/**
   * Fhrt die Basis-Initialisierung des Composites durch
   */
  private void initialize()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.heightHint = -1;
    gridData1.widthHint = -1;
    gridData1.horizontalIndent = 0;
    gridData1.verticalAlignment = GridData.FILL;
    setLayout(new GridLayout(1, false));
    setLayoutData(gridData1);
    
    _composite = new Composite(this, SWT.NONE);
    _composite.setLayout(new GridLayout(1, false));
    _composite.setLayoutData(gridData1);
 
    //Initialisierungen der Arrays
    List<TableModel[]> inputList = new ArrayList<TableModel[]>();
    _groupFilterDef = new Group[_itemGroups.length];
    _composite1 = new Composite[_itemGroups.length];
    _tvAnzeige = new TableViewer[_itemGroups.length];
    
    for (int i = 0; i < _itemGroups.length; i++)
    {
//      System.out.println(_itemGroups[i]);
      createGroupDefinition(i);
      
      Table table = _tvAnzeige[i].getTable();
      TableModel[] input = createModel1(i, table);
      
      if ( input != null )
      {
        _tvAnzeige[i].setInput(input); 
        inputList.add( input );
      }
    }
    
    updateData(inputList);
  }

  private void updateData( List<TableModel[]> modelList )
  {
    /*
     *  alle TableModelle durchlaufen
     */
  	
    for (int i = 0; i < modelList.size(); i++)
    {
      //Elemente des TabellenModells durchlaufen
      
      for (TableModel tm : modelList.get( i ))
      {
        Label lb = (Label) tm.getAttribut().getChildren()[0];

        /*
         * Spinner?
         */
        for (int j = 0; j < _parItemSpin[i].length; j++)
        {
        	/*
        	 * Anzahl Iterationen
        	 */
          if ( lb.getText().equals( _parItemSpin[0][0] ) )
          {
            Spinner sp = (Spinner) tm.getWert().getChildren()[0];
            sp.setSelection( (int) _anzahlIterationen );
          }
          
        	/*
        	 * Zeitraster
        	 */
          if ( lb.getText().equals( _parItemSpin[1][0] ) )
          {
            Spinner sp = (Spinner) tm.getWert().getChildren()[0];
            sp.setSelection( (int) _zeitraster );
          }

        	/*
        	 * Offset
        	 */
          if ( lb.getText().equals( _parItemSpin[1][1] ) )
          {
            Spinner sp = (Spinner) tm.getWert().getChildren()[0];
            sp.setSelection( (int) _offset );
          }

        }
      }
    }
  }

  /**
   * This method initializes createGroupDefinition
   * 
   */
  private void createGroupDefinition(int index)
  {
    FillLayout fillLayout1 = new FillLayout();
    fillLayout1.marginHeight = 5;
    fillLayout1.marginWidth = 5;
    fillLayout1.type = org.eclipse.swt.SWT.HORIZONTAL;
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.verticalAlignment = GridData.FILL;
    //Gre der 3.Gruppe fest setzen, damit diese grer wird
    if (index == 2)
      gridData2.heightHint = 350;
    _groupFilterDef[index] = new Group(_composite, SWT.NONE);
    if (_itemGroups[index] != null )
      _groupFilterDef[index].setText(_itemGroups[index]);
    _groupFilterDef[index].setLayout(fillLayout1);
    _groupFilterDef[index].setLayoutData(gridData2);

    erzeugeTabelle(index);

  }

  private void erzeugeTabelle(int index)
  {
    FillLayout fillLayout1 = new FillLayout();
    fillLayout1.type = org.eclipse.swt.SWT.VERTICAL;
    _composite1[index] = new Composite(_groupFilterDef[index], SWT.None);
    _composite1[index].setLayout(fillLayout1);
    _tvAnzeige[index] = new TableViewer(_composite1[index], SWT.HIDE_SELECTION | SWT.H_SCROLL | SWT.V_SCROLL);
    _tvAnzeige[index].setContentProvider(new TableContentProvider());
   
    TableViewerColumn column1 = new TableViewerColumn(_tvAnzeige[index], SWT.NONE);
    column1.getColumn().setWidth(225);
    column1.getColumn().setText("Attribut");
    column1.setLabelProvider(new MyCellLabelProvider());

    TableViewerColumn column2 = new TableViewerColumn(_tvAnzeige[index], SWT.NONE);
    column2.getColumn().setWidth(80);
    column2.getColumn().setText("Wert");
    column2.setLabelProvider(new MyCellLabelProvider());

    // resize the row height using a MeasureItem listener
    Listener listener = new Listener() {
      @Override
      public void handleEvent(Event event)
      {
        switch (event.type)
        {
          case SWT.MeasureItem:
            event.height = 30;
            break;

          default:
        }

      }

    };
    _tvAnzeige[index].getTable().addListener(SWT.MeasureItem, listener);
    _tvAnzeige[index].getTable().setLinesVisible(true);
    _tvAnzeige[index].getTable().setHeaderVisible(true);
  }


  private class TableContentProvider implements IStructuredContentProvider, Observer
  {
    private TableViewer tableViewer;

    public Object[] getElements(Object inputElement)
    {
      return (TableModel[])inputElement;      
    }

    public void dispose()
    {

    }

    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
    {
      tableViewer = (TableViewer)viewer;

      if (oldInput != null)
      {
        TableModel[] tms = (TableModel[])newInput;

        if (tms != null)
        {
          for (TableModel tm : tms)
          {
            tm.deleteObserver(this);
          }

        }

      }
      
     
      if (newInput != null)
      {
        TableModel[] tms = (TableModel[])newInput;
        if (tms != null)
        {
          for (TableModel tm : tms)
          {
            tm.addObserver(this);
          }
        }

      }

    }

    @Override
    public void update(Observable arg0, Object arg1)
    {

      if (tableViewer != null)
      {
        tableViewer.refresh();

      }
    }
  }

  /**
   * Erzeugt ein TabellenModell fr den Inhalt der Tabellen
   * 
   * @param table
   * @return TabellenModell
   */
  private TableModel[] createModel1(int index, Table table)
  {
    int tabLength = _parItemImage[index].length + _parItemCombo[index].length + _parItemSpin[index].length + _parItemCheck[index].length;
    
    TableModel[] elements = new TableModel[tabLength];

    //Layout festlegen
    GridData gd0 = new GridData();
    gd0.horizontalAlignment = GridData.CENTER;
    gd0.verticalAlignment = GridData.CENTER;
    gd0.grabExcessHorizontalSpace = true;
    gd0.grabExcessVerticalSpace = true;  
    GridData gd1 = new GridData();
    gd1.horizontalAlignment = GridData.CENTER;
    gd1.verticalAlignment = GridData.END;
    gd1.grabExcessHorizontalSpace = true;
    gd1.grabExcessVerticalSpace = true;
    //zum Ausfllen der Zelle
    GridData gdTableItem = new GridData();
    gdTableItem.horizontalAlignment = GridData.FILL;
    gdTableItem.grabExcessHorizontalSpace = true;
    gdTableItem.verticalAlignment = GridData.CENTER;
    
    int elIndex = 0;

    //Image-Label erstellen
    for (int i = 0; i < _parItemImage[index].length; i++)
    {
    }
    
    //Comboboxen erstellen
    for (int i = 0; i < _parItemCombo[index].length; i++)
    {
      Composite comp0 = new Composite( table, SWT.NONE );
      comp0.setBackground( table.getBackground() );
      comp0.setBackgroundMode( SWT.INHERIT_DEFAULT );
      comp0.setLayoutData( gd0 );
      comp0.setLayout( new GridLayout() );
      Label label = new Label( comp0, SWT.NONE );
      label.setText( _parItemCombo[index][i] );
//    System.out.println("Combo("+index+"," + i+"): " +_parItemCombo[index][i] );
      
      Composite comp1 = new Composite( table, SWT.NONE );
      comp1.setBackground( table.getBackground() );
      comp1.setBackgroundMode( SWT.INHERIT_DEFAULT );
      comp1.setLayoutData( gd1 );
      comp1.setLayout( new GridLayout() );
      Combo combo = new Combo( comp1, SWT.BORDER );
      combo.setLayoutData( gdTableItem );
      
      elements[elIndex] = new TableModel( comp0, comp1 );
      elIndex++;
    }
    
    //Spinner erstellen
    
    for (int i = 0; i < _parItemSpin[index].length; i++)
    {
      Composite comp0 = new Composite( table, SWT.NONE );
      comp0.setBackground( table.getBackground() );
      comp0.setBackgroundMode( SWT.INHERIT_DEFAULT );
      comp0.setLayoutData( gd0 );
      comp0.setLayout( new GridLayout() );
      
      Label label = new Label( comp0, SWT.NONE );
      label.setText( _parItemSpin[index][i] );
      
      Composite comp1 = new Composite( table, SWT.NONE );
      comp1.setBackground( table.getBackground() );
      comp1.setBackgroundMode( SWT.INHERIT_DEFAULT );
      comp1.setLayoutData( gd1 );
      comp1.setLayout( new GridLayout() );
      
      Spinner spinner = new Spinner( comp1, SWT.BORDER );

      /*
       * Anzahl Iterationen
       */
      if (label.getText().equals(_parItemSpin [0][0]))
      {
	      spinner.setMinimum(0);
	      spinner.setMaximum(10000);
	      spinner.setDigits(0);
	      
	      String toolTip = "Anzahl erlaubter Iterationen des Steuerungskerns. Zulssig von  " + spinner.getMinimum() 
	                       + " bis " + spinner.getMaximum() + " Iterationen";
	     
	      label.setToolTipText(toolTip);
	      spinner.setToolTipText(toolTip);
      }

      /*
       * Zeitraster
       */
      if (label.getText().equals(_parItemSpin [1][0]))
      {
	      spinner.setMinimum(0);
	      spinner.setMaximum(86400);
	      spinner.setDigits(0);
	      
	      String toolTip = "Zeitraster in Sekunden in denen der Steuerungskern arbeitet. Zulssig von  " + spinner.getMinimum() 
	                       + " bis " + spinner.getMaximum() + " Sekunden";
	     
	      label.setToolTipText(toolTip);
	      spinner.setToolTipText(toolTip);
      }

      /*
       * Offset
       */
      if (label.getText().equals(_parItemSpin [1][1]))
      {
	      spinner.setMinimum(0);
	      spinner.setMaximum(Integer.MAX_VALUE);
	      spinner.setDigits(0);
	      
	      String toolTip = "Offset in Millisekunden in denen der Steuerungskern startet. Zulssig von  " + spinner.getMinimum() 
	                       + " bis " + spinner.getMaximum() + " Millisekunden";
	     
	      label.setToolTipText(toolTip);
	      spinner.setToolTipText(toolTip);
      }

      spinner.setLayoutData( gdTableItem );
      
      elements[elIndex] = new TableModel( comp0, comp1 );
      elIndex++;
    }

    //Checkboxen erstellen
    for (int i = 0; i < _parItemCheck[index].length; i++)
    {
      Composite comp0 = new Composite( table, SWT.NONE );
      comp0.setBackground( table.getBackground() );
      comp0.setBackgroundMode( SWT.INHERIT_DEFAULT );
      comp0.setLayoutData( gd0 );
      comp0.setLayout( new GridLayout() );
      Label label = new Label( comp0, SWT.NONE );
      label.setText( _parItemCheck[index][i] );
//    System.out.println("Check("+index+"," + i+"): " +_parItemCheck[index][i] );
      
      Composite comp1 = new Composite( table, SWT.NONE );
      comp1.setBackground( table.getBackground() );
      comp1.setBackgroundMode( SWT.INHERIT_DEFAULT );
      comp1.setLayoutData( gd1 );
      comp1.setLayout( new GridLayout() );
      new Button( comp1, SWT.CHECK );
      
      elements[elIndex] = new TableModel( comp0, comp1 );
      elIndex++;
    }
    
    return elements;
  }

  /**
   * Anzahl der vorhandenen Gruppen
   * @return
   */
  public int getAnzahlGruppen()
  {
    return _itemGroups.length;
  }

  public String[] getGruppen()
  {
    return _itemGroups;
  }

  /**
   * Anzahl der vorhandenen Items je Gruppe
   * 
   * @param index   Index der Gruppe
   * @return
   */
  public int getAnzahlItems(int index)
  {
    return _parItemImage[index].length + _parItemCombo[index].length + _parItemSpin[index].length + _parItemCheck[index].length;
  }
  
  public TableModel[] getItems(int index)
  {
    return (TableModel[])_tvAnzeige[index].getInput();
  }
  
  /**
   * Anzahl der vorhandenen TabelViewer
   * @return
   */
  public int getAnzahlTvAnzeige()
  {
    return _tvAnzeige.length;
  }
  
  public TableViewer getTvAnzeige(int index)
  {
    return _tvAnzeige[index];
  }
  
  public void setTvAnzeige( int index, TableViewer tvSet )
  {
    setTvAnzeige( _tvAnzeige[index], tvSet );
  }
  
  
  private void setTvAnzeige( TableViewer tvAnzeige, TableViewer tvSet )
  {
    TableModel[] modelSet = (TableModel[]) tvSet.getInput();
    TableModel[] models1 = (TableModel[]) tvAnzeige.getInput();

    if ( models1.length == modelSet.length )
    {
      for (TableModel tm : models1)
      {
        Label lb = (Label) tm.getAttribut().getChildren()[0];

        String wert = "";
        Image image = null;
        for (TableModel tmSet : modelSet)
        {
          Label lbSet = (Label) tmSet.getAttribut().getChildren()[0];

          if ( lb.getText().equals( lbSet.getText() ) )
          {
            wert = tmSet.getSelectedWert();
            Object c = tmSet.getWert().getChildren()[0];
            if ( c instanceof Label )
            {
              image = ((Label)c).getImage();
            }
          }
        }
        tm.setSelectedWert( wert, image );
      }
    }
  }

  private class MyCellLabelProvider extends CellLabelProvider
  {

    @Override
    public void update(ViewerCell cell)
    {
      TableItem item = (TableItem)cell.getItem();

      if (item.getData("EDITOR") != null)
      {
        TableEditor editor = (TableEditor)item.getData("EDITOR");
        editor.dispose();
      }

      int cnt = cell.getColumnIndex();

      TableModel model = (TableModel)item.getData();
      Composite comp = null;
      switch (cnt)
      {
        case 0:
          comp = model.getAttribut();
          break;
        case 1:
          comp = model.getWert();
          break;
        default:
      }
      
      TableEditor editor = new TableEditor(item.getParent());
      editor.grabHorizontal = true;
      editor.setEditor(comp, item, cnt);

    }
  }

  /**
   * Methode liefert die aktuell eingestellten Parameter
   * @return aktuell eingestellten Parameter
   */
	public ParameterSteuerungsKern getParameter()
  {
	  return leseAktuelleEinstellungen();
  }
 
	private ParameterSteuerungsKern leseAktuelleEinstellungen ()
	{
    ParameterSteuerungsKern para = new ParameterSteuerungsKern();
    
  	long anzahlIterationen = leseAnzIterationen ((TableModel[]) _tvAnzeige [0].getInput());
  	para.setAnzahlIterationen(anzahlIterationen);
  	
  	leseWerte ((TableModel[]) _tvAnzeige [1].getInput(), para.getParaZeitraster());
  	
  	return para;
  }

	/**
	 * Methode liest die Anzahl der Iterationen
	 * @param input Tabellenmodell
	 * @return Anzahl Iterationen
	 */
	private long leseAnzIterationen (TableModel[] input)
	{
		long anzIterationen = 0;
		
    for (TableModel tm : input)
    {
      Label lb = (Label) tm.getAttribut().getChildren()[0];
      
      if (lb.getText().equals(_parItemSpin [0][0]))
      {
        Spinner sp = (Spinner) tm.getWert().getChildren()[0];
        anzIterationen = sp.getSelection();
      }
    }
    
    return anzIterationen;
  }
	
	/**
	 * Methode liest die Parameter
	 * @param input Tabellenmodell
	 * @param para Parametersatz
	 */
	private void leseWerte (TableModel[] input, ParaZeitraster para)
	{
		long zeitraster = 0;
		long offset     = 0;
		
    for (TableModel tm : input)
    {
      Label lb = (Label) tm.getAttribut().getChildren()[0];
      
      if (lb.getText().equals(_parItemSpin [1][0]))
      {
        Spinner sp = (Spinner) tm.getWert().getChildren()[0];
        zeitraster = sp.getSelection();
      }
      
      if (lb.getText().equals(_parItemSpin [1][1]))
      {
        Spinner sp = (Spinner) tm.getWert().getChildren()[0];
        offset = sp.getSelection();
      }
    }
    
    para.setZeitraster(zeitraster);
    para.setOffset(offset);
    
    return;
  }

}
