package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.steuerungskern;

import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.Activator;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.Konfig;
import de.bsvrz.sys.funclib.debug.Debug;
import de.koelnloevenich.dif.basislib.urlasserdialog.UrlasserDialog;

/**
 * View zum ndern der Parameter "Steuerungskern"
 */
public class ViewParameterSteuerungskern extends ViewPart 
{
  public static final String ID = "de.bsvrz.sba.plugin.parameter.steuerungskern.view.ViewParameterSteuerungskern";
  
  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();

  /**
   * Folder fr die Tabs
   */
  private CTabFolder _tabFolder = null;

  /**
   * Composite
   */
  private Composite _composite = null;

  /**
   * Composite mit den Buttons
   */
  private Composite _compositeBottom;

  /**
   * Button "Fr alle bernehmen"
   */
  private Button buttonAlle;
  
  /**
   * Button "Speichern"
   */
  private Button buttonOk;

  /**
   * Button "bernehmen"  
   */
  private Button buttonUebernehemen;

  /**
   * Button "Abbrechen"
   */
  private Button buttonAbbrechen;

  /**
   * Button "Hilfe"
   */
  private Button buttonHilfe;

  /**
   * Memento
   */
  private IMemento _memento = null;

  /**
   * Zweite ID des Views
   */
  private String _secondViewId;

  /**
   * Verbindung zum DaV
   */
  private ClientDavInterface _connection;

  /**
   * Anzuzeigende Objekte
   */
  private List<String> _objekte = new ArrayList<String>();

  /**
   * Listener
   */
  private PropertyChangeListener m_listener;
  
  @Override
  public void createPartControl( Composite parent )
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessVerticalSpace = true;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    
    GridLayout gridLayout = new GridLayout();
    gridLayout.numColumns = 1;
    
    _composite = new Composite( parent, SWT.NONE );
    _composite.setLayout( gridLayout );
    _composite.setLayoutData( gridData );
    
    //Fr die F1 Taste
    
    PlatformUI.getWorkbench().getHelpSystem().setHelp( _composite, Activator.PLUGIN_ID + ".parameter" );   
    
    _connection = DaVVerbindung.getInstanz().getDaVVerbindung();
    
    // Darstellung aufbauen
    
    createCompositeTab();
    createCompositeButtons();

    restoreState();

   _objekte.clear();

   
    if ( _secondViewId.contains( ";" ) )
    {

      String[] split = _secondViewId.split( ";" );

      for (String s : split)
      {
        _objekte.add( s );
      }
    }
    else
    {
    	if (_connection != null)
    	{
        _objekte.add( _secondViewId );
    	}
    }

    erzeugeTabItems( _objekte );
    
    verarbeiteDaten();
  }

  @Override
  public void setFocus()
  {
    if (_composite != null)
      _composite.setFocus();
  }

  /**
   * Methode erzeugt das Composite mit den Foldern
   */
  private void createCompositeTab()
  {
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.FILL;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.grabExcessVerticalSpace = true;
    gridData3.verticalAlignment = GridData.FILL;

    _tabFolder = new CTabFolder( _composite, SWT.NONE );
    _tabFolder.setLayout( new GridLayout() );
    _tabFolder.setLayoutData( gridData3 );
    
    _tabFolder.addSelectionListener( new SelectionListener()
		{
      @Override
      public void widgetSelected( SelectionEvent e )
      {

        CTabFolder folder = (CTabFolder) e.getSource();
        CTabItem itemSel = folder.getSelection();

        Composite cSel = (Composite) itemSel.getControl();

        if ( cSel.getChildren() != null && cSel.getChildren().length > 0 )
        {
        	CompositeParameterAuswahl compParSel = (CompositeParameterAuswahl) cSel.getChildren()[0];
        	compParSel.aktualisiereAlleFolder();
        }
      }

			@Override
      public void widgetDefaultSelected(SelectionEvent e)
      {
      }
    } );
  }

  /**
   * Methode erzeugt die einzelnen Folder
   * @param list List mit Objekten fr die Folder angelegt werden
   */
  public void erzeugeTabItems( List<String> list )
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    
    for (String so : list)
    {
      CTabItem tabItem = new CTabItem( _tabFolder, SWT.NONE );
      tabItem.setText( so );
      tabItem.setData( so, so );

      Composite c1 = new Composite( _tabFolder, SWT.NONE );
      c1.setLayout( new GridLayout() );
      c1.setLayoutData( gridData );

      tabItem.setControl( c1 );

    }
    _tabFolder.setSelection( 0 );
  }

  /**
   * Methode erzugt das Composite mit den Buttons
   */
  private void createCompositeButtons()
  {
    RowLayout rowLayout = new RowLayout();
    rowLayout.pack = false;
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.END;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.verticalAlignment = GridData.CENTER;
    
    _compositeBottom = new Composite( _composite, SWT.NONE );
    _compositeBottom.setLayoutData( gridData3 );
    _compositeBottom.setLayout( rowLayout );

//    buttonAlle = new Button( _compositeBottom, SWT.NONE );
//    buttonAlle.setText( "Alle setzen" );
//    buttonAlle.setToolTipText( "Einstellungen bei allen ausgewhlten gleichartigen AQs setzen" );
//    buttonAlle.addMouseListener( new MouseAdapter()
//    {
//      @Override
//      public void mouseDown( MouseEvent e )
//      {
//        super.mouseDown( e );
//        aktionAlle();
//      }
//    } );
   
    buttonOk = new Button( _compositeBottom, SWT.NONE );
    buttonOk.setText( "Ok" );
    buttonOk.addMouseListener( new MouseAdapter()
    {
      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        aktionSchalten();
      }
    } );

    buttonUebernehemen = new Button( _compositeBottom, SWT.NONE );
    buttonUebernehemen.setText( "bernehmen" );
    buttonUebernehemen.addMouseListener( new MouseAdapter()
    {
      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        aktionUebernehmen();
      }
    } );

    buttonAbbrechen = new Button( _compositeBottom, SWT.NONE );
    buttonAbbrechen.setText( "Abbrechen" );
    buttonAbbrechen.addMouseListener( new MouseAdapter()
    {

      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        dispose();
      }

    } );
    buttonHilfe = new Button( _compositeBottom, SWT.NONE );
    buttonHilfe.setText( "Hilfe" );
    buttonHilfe.addMouseListener( new MouseAdapter()
    {

      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        aktionHilfe();
      }

    } );
  }

  /**
   * Zeigt die Hilfe an.
   */
  protected void aktionHilfe()
  {
    String contextId = Activator.PLUGIN_ID + ".parameter";
    _debug.fine("ContextID: " + contextId);
    PlatformUI.getWorkbench().getHelpSystem().displayHelp(contextId);
  }
  
  /**
   * Methode bernimmt die eingestellten Parameter fr alle Tabs. Hierzu werden die Tabellenviewer des
   * angewhlten Tab an die anderen Tabs bergeben.
   */
  protected void aktionAlle()
  {
    /*
     * aktueller Reiter
     */
    CTabItem itemSel = _tabFolder.getSelection();

    Composite compSel = null;

    if ( itemSel == null )
      compSel = (Composite) _tabFolder.getItems()[0].getControl();
    else
      compSel = (Composite) itemSel.getControl();

    if ( compSel.getChildren() != null )
    {
    	CompositeParameterAuswahl compParSel = (CompositeParameterAuswahl) compSel.getChildren()[0];

    	HashMap<String, TableViewer> modelle = compParSel.getTableViewer();
    	
      CTabItem[] items = _tabFolder.getItems();
      for (CTabItem item : items)
      {
        if ( !item.equals( itemSel ) )
        {
          Composite c = (Composite) item.getControl();
  
          CompositeParameterAuswahl compPar = (CompositeParameterAuswahl) c.getChildren()[0];
          compPar.setTableViewer(modelle);
        }
      }
    }
    else
      System.out.println("Kein selektierter Tab!!!");
  }

  /**
   * bernehmen der Parameter und Beenden des Views
   */
  protected void aktionSchalten()
  {
    if ( speichernDerParameterInDaV() )
    {
      dispose();
    }
    else
    {
      MessageBox mb = new MessageBox( new Shell( Display.getCurrent() ), SWT.ICON_INFORMATION | SWT.OK );
      mb.setText( "Fehler Urlasser" );
      mb.setMessage( "Objekt konnte nicht parametriert werden: " + "" );
      mb.open();
      return;
    }

  }

  /**
   * bernehmen der Parameter
   */
  protected void aktionUebernehmen()
  {
    if ( speichernDerParameterInDaV() )
    {
      // keine weiteren Aktionen
    }
    else
    {
      MessageBox mb = new MessageBox( new Shell( Display.getCurrent() ), SWT.ICON_INFORMATION | SWT.OK );
      mb.setText( "Fehler Urlasser" );
      mb.setMessage( "Objekt konnte nicht parametriert werden: " + "" );
      mb.open();
      return;
    }

  }

  @Override
  public void init( IViewSite site, IMemento memento ) throws PartInitException
  {
    _memento = memento;
    _secondViewId = site.getSecondaryId();
    super.init( site, memento );
  }

  private void restoreState()
  {
    if ( _memento == null )
    {
      return;
    }

    _secondViewId = _memento.getString( "pidMemento" );

  }

  @Override
  public void saveState( IMemento memento )
  {
    memento.putString( "pidMemento", _secondViewId );
    super.saveState( memento );
  }

  /**
   * Methode speichert die Parameter im DaV
   * @return
   */
  private Boolean speichernDerParameterInDaV()
  {   
  	Boolean schliessenOk = false;
  	
  	/*
  	 * Daten der einzelnen Tabs einlesen
  	 */

	  CTabItem[] items = _tabFolder.getItems();
	  for (CTabItem item : items)
	  {
	    Composite c = (Composite)item.getControl();
	    CompositeParameterAuswahl compPar = (CompositeParameterAuswahl) c.getChildren()[0];

      String pid = (String)item.getData(item.getText());

	    ParameterSteuerungsKern parameter = compPar.getParameter();

	    /*
	     * Parameter auf die einzelnen Parameterarten verteilen
	     */
	    
	    VerwaltungParaIterationen.getInstance().setAnzIterationen(pid, parameter.getAnzahlIterationen());

	    VerwaltungParaZeitraster.getInstance().setZeitraster(pid, parameter.getParaZeitraster());

	    VerwaltungParaAspekte.getInstance().setParameter(pid, parameter.getParaAspekte());
	    
	    VerwaltungParaAnzeigen.getInstance().setParameter(pid, parameter.getParaAnzeigen());
	  }

	  /*
	   * Attributgruppe zusammenstellen und an den DaV senden
	   */

    List<ResultData> list = new ArrayList<ResultData>();

    /*
     * Interationen
     */
	  Data data = VerwaltungParaIterationen.getInstance().getData();
    if (data != null)
    {
      SystemObject sysObj = _connection.getDataModel().getObject(Konfig.getInstance().getPidSBA());
      AttributeGroup atg  = _connection.getDataModel().getAttributeGroup ("atg.sswSbaIterationen");
      Aspect asp = _connection.getDataModel().getAspect("asp.parameterVorgabe");
      DataDescription dd = new DataDescription(atg, asp);

      ResultData result = new ResultData( sysObj, dd, new Date ().getTime(), data );
      list.add(result);
    }

    /*
     * Zeitraster
     */
	  Data data1 = VerwaltungParaZeitraster.getInstance().getData();
    if (data1 != null)
    {
      SystemObject sysObj = _connection.getDataModel().getObject(Konfig.getInstance().getPidSBA());
      AttributeGroup atg  = _connection.getDataModel().getAttributeGroup ("atg.sswSbaZeitRaster");
      Aspect asp = _connection.getDataModel().getAspect("asp.parameterVorgabe");
      DataDescription dd = new DataDescription(atg, asp);

      ResultData result = new ResultData( sysObj, dd, new Date ().getTime(), data1 );
      list.add(result);
    }

    /*
     * Aspekte
     */
	  Data data2 = VerwaltungParaAspekte.getInstance().getData();
    if (data2 != null)
    {
      SystemObject sysObj = _connection.getDataModel().getObject(Konfig.getInstance().getPidSBA());
      AttributeGroup atg  = _connection.getDataModel().getAttributeGroup ("atg.sswSbaZuBearbeitendeAspekte");
      Aspect asp = _connection.getDataModel().getAspect("asp.parameterVorgabe");
      DataDescription dd = new DataDescription(atg, asp);

      ResultData result = new ResultData( sysObj, dd, new Date ().getTime(), data2 );
      list.add(result);
    }

    /*
     * Anzeigen
     */
	  Data data3 = VerwaltungParaAnzeigen.getInstance().getData();
    if (data3 != null)
    {
      SystemObject sysObj = _connection.getDataModel().getObject(Konfig.getInstance().getPidSBA());
      AttributeGroup atg  = _connection.getDataModel().getAttributeGroup ("atg.sswSbaZuErmittelndeAnzeigen");
      Aspect asp = _connection.getDataModel().getAspect("asp.parameterVorgabe");
      DataDescription dd = new DataDescription(atg, asp);

      ResultData result = new ResultData( sysObj, dd, new Date ().getTime(), data3 );
      list.add(result);
    }


    if (list.size() > 0)
    {
      UrlasserDialog urlasserDialog = new UrlasserDialog( Display.getCurrent().getActiveShell() );
      schliessenOk = (urlasserDialog.anzeigen( list )).getEingabeOk();
    }

    return schliessenOk;
  }

  @Override
  public void dispose()
  {
    super.dispose();

//    VerwaltungParaZusammenfassen.getInstance().removePropertyChangeListener(m_listener);
    IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
    if ( page != null )
    {
      page.hideView( this );
    }
  }

  /**
   * Methode aktualisiert die einzelnen Tabs
   */
  private void verarbeiteDaten()
  {
    for (String so : _objekte)
    {
      verarbeiteDaten (so);        
    }
    
//    if ( _tabFolder.getItemCount() <= 1 )
//      buttonAlle.setEnabled( false );
//    else
//      buttonAlle.setEnabled( true );
  }
  
  /**
   * Methode aktualisiert einen Tab
   * @param instanz Instanz des Steuerungskerns dessen Parameter versorgt werden sollen
   */
  private void verarbeiteDaten (String instanz)
  {
    CTabItem[] items = _tabFolder.getItems();
    
    // Tab zum Systemobject ermitteln
    
    CTabItem item = null;
    for (CTabItem ti : items)
    {
      String pid = (String)ti.getData(ti.getText());

      if (pid.equals(instanz))
      {
        item = ti;
        break;
      }
    }

    // Tab gefunden
    
    if (item != null)
    {
    	ParameterSteuerungsKern para = bestimmeParameter(instanz);
      
      Composite c = (Composite)item.getControl();
      
      Control[] children = c.getChildren();

      // Tab neu anlegen, wenn noch leer
      
      if (children.length == 0)
      {
      	CompositeParameterAuswahl compPar = new CompositeParameterAuswahl( c, SWT.None, para, (String) instanz );
        compPar.layout( true );
        c.layout( true );
      }
    }
  }
  
  /**
   * Methode fasst die Parameter des Steuerungskerns, die ja aus mehreren Attributgruppe bestehen, in einer
   * Klasse zusammen. 
   * @param instanz Instanz des Steuerungskerns
   * @return Parameter der Instanz des Steuerungskerns
   */
  private ParameterSteuerungsKern bestimmeParameter (String instanz)
  {
    ParameterSteuerungsKern para = new ParameterSteuerungsKern();
    
    long anzahlIterationen = VerwaltungParaIterationen.getInstance().getAnzahlIterationen(instanz);
    para.setAnzahlIterationen(anzahlIterationen);

    para.setParaZeitraster(VerwaltungParaZeitraster.getInstance().getParameter(instanz));
    
    para.setParaAspekte(VerwaltungParaAspekte.getInstance().getParameter(instanz));

    para.setParaAnzeigen(VerwaltungParaAnzeigen.getInstance().getParameter(instanz));

    return para;
  }
}
