package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.verriegelung.spezielleVerriegelungsMatrix;

import java.util.List;
import java.util.Vector;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Aspect;

/**
 * Klasse enthlt ein Feld (Liste) mit Parametern
 * @author       Swarco Traffic Systems GmbH
 */
public class ListeMitRegeln
{
  /**
   * Liste mit den Parametern
   */
  private Vector<RegelSpezielleVerriegelungsMatrix> m_parameter = new Vector<RegelSpezielleVerriegelungsMatrix> ();

  /**
   * Parameterloser Konstruktor der Klasse
   */
  public ListeMitRegeln ()
  {
  }
  
  /**
   * Methode liefert die Variable parameter zurck
   * @return Variable parameter
   */
  public Vector<RegelSpezielleVerriegelungsMatrix> getParameter()
  {
    Vector<RegelSpezielleVerriegelungsMatrix> v = new Vector<RegelSpezielleVerriegelungsMatrix>();
    synchronized (m_parameter)
    {
      for (RegelSpezielleVerriegelungsMatrix r : m_parameter)
        v.add( r );
    }
    return v;
  }

  /**
   * Methode setzt die Variable parameter
   * @param parameter Wert fr parameter
   */
  public void setParameter(Vector<RegelSpezielleVerriegelungsMatrix> parameter)
  {
    synchronized (m_parameter)
    {
      m_parameter = parameter;
    } 
  }

  /**
   * Methode zum Auswerten der Regeln der Attributliste 'atl.sswSbaSpezielleVerriegelungsMatrixProAnzeigeTyp'
   * @param data Daten vom DaV
   */
  public void auswerteAttributgruppe (Data data, Aspect schaltgrundAllgemein)
  {
    Array a = data.getArray( "AnlageRegeln" );
    int anzahl = a.getLength();
    
    synchronized (m_parameter)
    {
      m_parameter.clear();
      for (int x = 0; x < anzahl; ++x)
      {
        RegelSpezielleVerriegelungsMatrix r = new RegelSpezielleVerriegelungsMatrix ();
        r.auswerteAttributgruppe( a.getItem( x ), schaltgrundAllgemein );
        
        m_parameter.add( r );
      }
    }
  }

  /**
   * Methode fgt eine Regel hinzu
   * @param verriegelungen Liste mit den zu verriegelnden Anzeigen
   * @param schaltgrund Schaltgrund
   */
  public void addRegel (List<Verriegelung> verriegelungen, Aspect schaltgrund )
  {
    synchronized (m_parameter)
    {
      RegelSpezielleVerriegelungsMatrix r = new RegelSpezielleVerriegelungsMatrix ();
      
      r.setSchaltgrundRegel( schaltgrund );
      r.setVerriegelungen( verriegelungen );
      
      m_parameter.add( r );
    }
  }

  /**
   * Methode liefert den Inahlt der Klasse als String
   */
  public String toString ()
  {
    String s = "";
    
    synchronized (m_parameter)
    {
      s += "Regeln:\n";
      int i = 1;
      for (RegelSpezielleVerriegelungsMatrix r : m_parameter)
      { 
        s += "Regel: " + (i++) + "\n";
        s += r.toString() + "\n";
      }
    }
    
    return s;
  }

	public void addRegel(RegelSpezielleVerriegelungsMatrix rsvm)
  {
		synchronized (m_parameter)
    {
	    m_parameter.add (rsvm);
    }
  }
}
