/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Adolf-Dambach-Strae
 * 76571 Gaggenau
 * Phone: +49-7225-64-01
 * Fax: +49-7225-64-300
 * mailto: info@els.dambach.de
 */

package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.vorgaengerNachfolger;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;
import java.util.Map.Entry;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.ReferenceArray;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.VorgaengerNachfolgerArt;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.Richtung.ArbeitsRichtungVN;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.Konfig;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;

/**
 * Klasse beinhaltet die Parameter Vorgnger - Nachfolger auf Anzeigenbasis.
 * Die Vorgnger / Nachfolger werden dabei in Quer- und Lngsrichtung gespeichert.
 * @version $Revision: 1.3 $ / $Date: 2013/06/28 09:45:14 $ / ($Author: ObertM $)
 * @author Dambach-Werke GmbH
 * @author Matthias Obert
 */
public class ParameterVorgaengerNachfolger
{
	/**
   * Speichert die Vorgnger / Nachfolgerbeziehung in einer Hashmap
   * Als Key dient die Anzeige, als Value die Zuordnung
   */
  private HashMap<SystemObject, VorgaengerNachfolger> m_nachbarn = new HashMap<SystemObject, VorgaengerNachfolger>();
  
  /**
   * Verwaltung der Listener
   */
  private PropertyChangeSupport m_pcs = new PropertyChangeSupport (this);

	/**
	 * Pid der Attributgruppe
	 */
	private String _pidAtg = "atg.sswSbaVorgngerNachfolgerBeziehung";
  
  /**
   * Privater Konstruktor der Klasse
   */
  public ParameterVorgaengerNachfolger ()
  {
  }
  
  /**
   * Methode liefert die Vorgnger/Nachfolger Beziehung fr eine Anzeige zurck
   * @param anzeige Anzeige fr die die Beziehung geliefert werden soll
   * @return Vorgnger/Nachfolger Beziehung sofern definiert, sonst null
   */
  public VorgaengerNachfolger getVorgaengerNachfolger (SystemObject anzeige)
  {
    synchronized (m_nachbarn)
    {
      if (m_nachbarn.containsKey( anzeige ))
      {
        VorgaengerNachfolger beziehung = m_nachbarn.get( anzeige );
        return beziehung.getKopie ();
      }
    }
    
    return null;
  }
  
  /**
   * Methode liefert die Vorgaenger einer Anzeige in Querrichtung
   * @param anzeige Anzeige
   * @param art Art der Vorgnger/Nachfolger
   * @return Vorgaenger
   */
  public SystemObject getVorgaengerInQuerRichtung (SystemObject anzeige, VorgaengerNachfolgerArt art)
  {
    VorgaengerNachfolger vn = getVorgaengerNachfolger( anzeige );
    if (vn != null)
    {
      if (art == VorgaengerNachfolgerArt.DIREKT)
        return vn.getQuerRichtung().getDirekterVorgaenger();

      if (art == VorgaengerNachfolgerArt.GLEICHERTYP)
        return vn.getQuerRichtung().getVorgaengerGleicherTyp();
    }
    
    return null;
  }
  
  /**
   * Methode liefert die direkten Nachfolger einer Anzeige in Querrichtung
   * @param anzeige Anzeige
   * @param art Art der Vorgnger/Nachfolger
   * @return Nachfolger
   */
  public SystemObject[] getNachfolgerInQuerRichtung (SystemObject anzeige, VorgaengerNachfolgerArt art)
  {
    VorgaengerNachfolger vn = getVorgaengerNachfolger( anzeige );
    if (vn != null)
    {
      if (art == VorgaengerNachfolgerArt.DIREKT)
        return vn.getQuerRichtung().getDirekterNachfolger();

      if (art == VorgaengerNachfolgerArt.GLEICHERTYP)
        return vn.getQuerRichtung().getNachfolgerGleicherTyp();
    }
    
    return null;
  }
  
  /**
   * Methode liefert die Vorgaenger einer Anzeige in Lngsrichtung
   * @param anzeige Anzeige
   * @param art Art der Vorgnger/Nachfolger
   * @return Vorgaenger
   */
  public SystemObject getVorgaengerInLaengsRichtung (SystemObject anzeige, VorgaengerNachfolgerArt art)
  {
    VorgaengerNachfolger vn = getVorgaengerNachfolger( anzeige );
    if (vn != null)
    {
      if (art == VorgaengerNachfolgerArt.DIREKT)
        return vn.getLaengsRichtung().getDirekterVorgaenger();

      if (art == VorgaengerNachfolgerArt.GLEICHERTYP)
        return vn.getLaengsRichtung().getVorgaengerGleicherTyp();
    }
    
    return null;
  }
  
  /**
   * Methode liefert die direkten Nachfolger einer Anzeige in Lngsrichtung
   * @param anzeige Anzeige
   * @param art Art der Vorgnger/Nachfolger
   * @return Nachfolger
   */
  public SystemObject[] getNachfolgerInLaengsRichtung (SystemObject anzeige, VorgaengerNachfolgerArt art)
  {
    VorgaengerNachfolger vn = getVorgaengerNachfolger( anzeige );
    if (vn != null)
    {
      if (art == VorgaengerNachfolgerArt.DIREKT)
        return vn.getLaengsRichtung().getDirekterNachfolger();

      if (art == VorgaengerNachfolgerArt.GLEICHERTYP)
        return vn.getLaengsRichtung().getNachfolgerGleicherTyp();
    }
    
    return null;
  }
  
  /**
   * Methode fgt einen Listener der Klasse hinzu
   * @param pcl Listern der Klasse PropertyChangeListener
   */
  public void addPropertyChangeListener (PropertyChangeListener pcl)
  {
    m_pcs.addPropertyChangeListener( pcl );    
  }
  
  /**
   * Methode entfernt einen Listener aus der Klasse
   * @param pcl Listern der Klasse PropertyChangeListener
   */
  public void removePropertyChangeListener (PropertyChangeListener pcl)
  {
    m_pcs.removePropertyChangeListener( pcl );    
  }

  /**
   * Methode benachrichtigt die Listener
   */
  private void benachrichtigeListener ()
  {
    m_pcs.firePropertyChange( "Aktualisierung", null, null );
  }

  /**
   * Methode zum Auswerten der Attributgruppe 'atg.sswSbaVorgngerNachfolgerBeziehung'
   * @param dat empfangender Datensatz
   */
  public void auswerteAttributgruppe( ResultData dat )
  {
    synchronized (m_nachbarn)
    {
      m_nachbarn.clear();
      
      if (dat.hasData())
      {
        Array a = dat.getData().getArray( "Beziehungen" );
        int anzahl = a.getLength();
        
        for (int x = 0; x < anzahl; ++x)
          auswerteAttributgruppeFuerEineAnzeige( a.getItem( x ) );
      }
    }
    
    benachrichtigeListener();
  }

  /**
   * Methode zum Auswerten der Attributgruppe 'atg.sswSbaVorgngerNachfolgerBeziehung' fr
   * eine Anzeige
   * @param data empfangender Datensatz
   */
  private void auswerteAttributgruppeFuerEineAnzeige ( Data data )
  {
    SystemObject anzeige = data.getReferenceValue( "Anzeige" ).getSystemObject();
    
    VorgaengerNachfolger nachbarn = null;
    
    if (!m_nachbarn.containsKey( anzeige ))
    {
      nachbarn = new VorgaengerNachfolger();
      m_nachbarn.put( anzeige, nachbarn );
    }
    
    nachbarn = m_nachbarn.get( anzeige );
    nachbarn.auswerteAttributgruppe( data );
  }
  
  /**
   * Methode setzt den direkten Vorgnger einer Anzeige in Lngsrichtung
   * @param anzeige Anzeige
   * @param vorgaenger Direkter Vorgnger der Anzeige
   */
  public void setDirektenVorgaengerLaengsrichtung (SystemObject anzeige, SystemObject vorgaenger)
  {
    synchronized (m_nachbarn)
    {
      VorgaengerNachfolger nachbarn = getNachbarn( anzeige );
      nachbarn.getLaengsRichtung().setDirekterVorgaenger( vorgaenger );
    }
  }

  /**
   * Methode setzt die direkten Nachfolger einer Anzeige in Lngsrichtung
   * @param anzeige Anzeige
   * @param nachfolger Direkte Nachfolger der Anzeige
   */
  public void setDirekteNachfolgerLaengsrichtung (SystemObject anzeige, SystemObject[] nachfolger)
  {
    synchronized (m_nachbarn)
    {
      VorgaengerNachfolger nachbarn = getNachbarn( anzeige );
      nachbarn.getLaengsRichtung().setDirekterNachfolger( nachfolger );
    }
  }

  /**
   * Methode setzt den direkten Vorgnger einer Anzeige in Querrichtung
   * @param anzeige Anzeige
   * @param vorgaenger Direkter Vorgnger der Anzeige
   */
  public void setDirektenVorgaengerQuerrichtung (SystemObject anzeige, SystemObject vorgaenger)
  {
    synchronized (m_nachbarn)
    {
      VorgaengerNachfolger nachbarn = getNachbarn( anzeige );
      nachbarn.getQuerRichtung().setDirekterVorgaenger( vorgaenger );
    }
  }

  /**
   * Methode setzt die direkten Nachfolger einer Anzeige in Querrichtung
   * @param anzeige Anzeige
   * @param nachfolger Direkte Nachfolger der Anzeige
   */
  public void setDirekteNachfolgerQuerrichtung (SystemObject anzeige, SystemObject[] nachfolger)
  {
    synchronized (m_nachbarn)
    {
      VorgaengerNachfolger nachbarn = getNachbarn( anzeige );
      nachbarn.getQuerRichtung().setDirekterNachfolger( nachfolger );
    }
  }

  /**
   * Methode liefeert die Nachbarbeziehung einer Anzeige
   * @param anzeige Anzeige deren Nachbarn gesucht werden
   * @return Nachbarn der Anzeige
   */
  private VorgaengerNachfolger getNachbarn (SystemObject anzeige)
  {
    synchronized (m_nachbarn)
    {
      if (m_nachbarn.containsKey( anzeige ))
        return m_nachbarn.get( anzeige );
      
      VorgaengerNachfolger vn = new VorgaengerNachfolger ();
      m_nachbarn.put( anzeige, vn );
      
      return vn;
    }
  }

  /**
   * Methode setzt den Vorgnger gleichen Typs einer Anzeige in Lngsrichtung
   * @param anzeige Anzeige
   * @param vorgaenger Vorgnger gleichen Typs der Anzeige
   */
  public void setVorgaengerGleichenTypsLaengsrichtung (SystemObject anzeige, SystemObject vorgaenger)
  {
    synchronized (m_nachbarn)
    {
      VorgaengerNachfolger nachbarn = getNachbarn( anzeige );
      nachbarn.getLaengsRichtung().setVorgaengerGleicherTyp( vorgaenger );
    }
  }

  /**
   * Methode setzt die Nachfolger gleichen Typs einer Anzeige in Lngsrichtung
   * @param anzeige Anzeige
   * @param nachfolger Nachfolger gleichen Typs der Anzeige
   */
  public void setNachfolgerGleichenTypsLaengsrichtung (SystemObject anzeige, SystemObject[] nachfolger)
  {
    synchronized (m_nachbarn)
    {
      VorgaengerNachfolger nachbarn = getNachbarn( anzeige );
      nachbarn.getLaengsRichtung().setNachfolgerGleicherTyp( nachfolger );
    }
  }

  /**
   * Methode setzt den Vorgnger gleichen Typs einer Anzeige in Querrichtung
   * @param anzeige Anzeige
   * @param vorgaenger Vorgnger gleichen Typs der Anzeige
   */
  public void setVorgaengerGleichenTypsQuerrichtung (SystemObject anzeige, SystemObject vorgaenger)
  {
    synchronized (m_nachbarn)
    {
      VorgaengerNachfolger nachbarn = getNachbarn( anzeige );
      nachbarn.getQuerRichtung().setVorgaengerGleicherTyp( vorgaenger );
    }
  }

  /**
   * Methode setzt die Nachfolger gleichen Typs einer Anzeige in Querrichtung
   * @param anzeige Anzeige
   * @param nachfolger Nachfolger gleichen Typs der Anzeige
   */
  public void setNachfolgerGleichenTypsQuerrichtung (SystemObject anzeige, SystemObject[] nachfolger)
  {
    synchronized (m_nachbarn)
    {
      VorgaengerNachfolger nachbarn = getNachbarn( anzeige );
      nachbarn.getQuerRichtung().setNachfolgerGleicherTyp( nachfolger );
    }
  }

  /**
   * Methode liefert den Inhalt der Klasse als String
   */
  public String toString ()
  {
    String s = "";
    
    synchronized (m_nachbarn)
    {
      Iterator<Entry<SystemObject, VorgaengerNachfolger>> it = m_nachbarn.entrySet().iterator();
      while (it.hasNext())
      {
        Map.Entry<SystemObject, VorgaengerNachfolger> entry = it.next();
        
        SystemObject anzeige = (SystemObject) entry.getKey();
        VorgaengerNachfolger vg = (VorgaengerNachfolger) entry.getValue();
        
        s += "++++++++++++++++++++++++++++++++++++++++++++++++++++\n";
        s += "Anzeige: " + Tools.getObjektName(  anzeige ) + "\n";
        s += vg.toString() + "\n";
      }
    }
    
    return s;
  }

  /**
   * Methode liefert die direkten Vorgnger/Nachfolger in Lngsrichtung einer Anzeige.
   * @param anzeige Ausgangspunkt
   * @param richtung Richtung Vorgnger/Nachfolger
   * @param anzeigeTyp gesuchter Anzeigetyp (wenn null dann keine Bercksichtigung)
   * @param art Welche Vorgnger/Nachfolger sollen gesucht werden
   * @return Liste mit Vorgngern/Nachfolgern
   */
  public Vector<SystemObject> getVorgaengerNachfolgerInLaengsRichtung ( SystemObject anzeige, ArbeitsRichtungVN richtung, SystemObject anzeigeTyp, VorgaengerNachfolgerArt art )
  {
    return getVorgaengerNachfolgerInLaengsRichtung( anzeige, richtung, 99, anzeigeTyp, art );
  }
  
  /**
   * Methode liefert die direkten Vorgnger/Nachfolger in Lngsrichtung einer Anzeige bis zu einer bestimmten Ebene.
   * @param anzeige Ausgangspunkt
   * @param richtung Richtung Vorgnger/Nachfolger
   * @param ebene Ebene bis zu der gesucht werden soll
   * @param anzeigeTyp gesuchter Anzeigetyp (wenn null dann keine Bercksichtigung)
   * @param art Welche Vorgnger/Nachfolger sollen gesucht werden
   * @return Liste mit Vorgngern/Nachfolgern
   */
  public Vector<SystemObject> getVorgaengerNachfolgerInLaengsRichtung ( SystemObject anzeige, ArbeitsRichtungVN richtung, int ebene, SystemObject anzeigeTyp, VorgaengerNachfolgerArt art )
  {
//    System.out.println(this.getClass() + " ---> getDirekteVorgaengerNachfolger " + Tools.getObjektName( anzeige ) + " Ebene = " + ebene);

//    ParameterVorgaengerNachfolger para = this;
    
    List<SystemObject> anzeigen               = new ArrayList<SystemObject>();
    List<SystemObject> zuBetrachtendeAnzeigen = new ArrayList<SystemObject>();
    List<SystemObject> anzeigenEinerEbene     = new ArrayList<SystemObject>();

    // Ausgangsbasis auslsende Anzeige
    
    anzeigenEinerEbene.add( anzeige );
    
    // Loop ber die Ebenen um die Anzeigen zu bestimmen
    
    for (int e = 0; e < ebene; ++e)
    {
//      System.out.println(this.getClass() + " Ebene = " + e );
      
      for (SystemObject so : anzeigenEinerEbene )
      {
//        System.out.println(this.getClass() + " ---> SO: " + Tools.getObjektName( so ));
        
        SystemObject[] vorgaengerNachfolger = bestimmeVorgaengerNachfolgerInLaengsRichtung( so, richtung, art );
        for (int i=0; i<vorgaengerNachfolger.length; ++i)
        {
          zuBetrachtendeAnzeigen.add( vorgaengerNachfolger [i] );
        }
      }
      
      anzeigen.addAll( zuBetrachtendeAnzeigen );
      
      anzeigenEinerEbene.clear();
      anzeigenEinerEbene.addAll( zuBetrachtendeAnzeigen );
      zuBetrachtendeAnzeigen.clear();
    }
    
    // Anzeigen filtern die dem Anzeigetyp nicht entsprechen
    
    Vector<SystemObject> ergebnis = new Vector<SystemObject>();
    for (SystemObject so : anzeigen)
      if ((anzeigeTyp == null) || (Konfig.getInstance().getWzgTypVonAnzeige( so.getPid() ) == anzeigeTyp))
      {
//        System.out.println(this.getClass() + " ----> " + Tools.getObjektName( so ));
        ergebnis.add( so );
      }
    
    return ergebnis;
  }

  /**
   * Methode liefert die direkten Vorgnger/Nachfolger in Querrichtung einer Anzeige bis zu einer bestimmten Ebene.
   * @param anzeige Ausgangspunkt
   * @param richtung Richtung Vorgnger/Nachfolger
   * @param anzeigeTyp gesuchter Anzeigetyp (wenn null dann keine Bercksichtigung)
   * @param art Welche Vorgnger/Nachfolger sollen gesucht werden
   * @return Liste mit Vorgngern/Nachfolgern
   */
  public Vector<SystemObject> getVorgaengerNachfolgerInQuerRichtung ( SystemObject anzeige, ArbeitsRichtungVN richtung, SystemObject anzeigeTyp, VorgaengerNachfolgerArt art )
  {
    return getVorgaengerNachfolgerInQuerRichtung( anzeige, richtung, 99, anzeigeTyp, art );
  }
  
  /**
   * Methode liefert die direkten Vorgnger/Nachfolger in Querrichtung einer Anzeige bis zu einer bestimmten Ebene.
   * @param anzeige Ausgangspunkt
   * @param richtung Richtung Vorgnger/Nachfolger
   * @param ebene Ebene bis zu der gesucht werden soll
   * @param anzeigeTyp gesuchter Anzeigetyp (wenn null dann keine Bercksichtigung)
   * @param art Welche Vorgnger/Nachfolger sollen gesucht werden
   * @return Liste mit Vorgngern/Nachfolgern
   */
  public Vector<SystemObject> getVorgaengerNachfolgerInQuerRichtung ( SystemObject anzeige, ArbeitsRichtungVN richtung, int ebene, SystemObject anzeigeTyp, VorgaengerNachfolgerArt art )
  {
//    System.out.println(this.getClass() + " ---> getDirekteVorgaengerNachfolger " + Tools.getObjektName( anzeige ) + " Ebene = " + ebene);

//    ParameterVorgaengerNachfolger para = this;
    
    List<SystemObject> anzeigen               = new ArrayList<SystemObject>();
    List<SystemObject> zuBetrachtendeAnzeigen = new ArrayList<SystemObject>();
    List<SystemObject> anzeigenEinerEbene     = new ArrayList<SystemObject>();

    // Ausgangsbasis auslsende Anzeige
    
    anzeigenEinerEbene.add( anzeige );
    
    // Loop ber die Ebenen um die Anzeigen zu bestimmen
    
    for (int e = 0; e < ebene; ++e)
    {
//      System.out.println(this.getClass() + " Ebene = " + e );
      
      for (SystemObject so : anzeigenEinerEbene )
      {
//        System.out.println(this.getClass() + " ---> SO: " + Tools.getObjektName( so ));
        
        SystemObject[] vorgaengerNachfolger = bestimmeVorgaengerNachfolgerInQuerRichtung( so, richtung, art);
        for (int i=0; i<vorgaengerNachfolger.length; ++i)
        {
          zuBetrachtendeAnzeigen.add( vorgaengerNachfolger [i] );
        }
      }
      
      anzeigen.addAll( zuBetrachtendeAnzeigen );
      
      anzeigenEinerEbene.clear();
      anzeigenEinerEbene.addAll( zuBetrachtendeAnzeigen );
      zuBetrachtendeAnzeigen.clear();
    }
    
    // Anzeigen filtern die dem Anzeigetyp nicht entsprechen
    
    Vector<SystemObject> ergebnis = new Vector<SystemObject>();
    for (SystemObject so : anzeigen)
      if ((anzeigeTyp == null) || (Konfig.getInstance().getWzgTypVonAnzeige( so.getPid() ) == anzeigeTyp))
      {
//        System.out.println(this.getClass() + " ----> " + Tools.getObjektName( so ));
        ergebnis.add( so );
      }
    
    return ergebnis;
  }

  /**
   * Methode liefert die Vorgnger Nachfolger einer Anzeige in der n. Ebene
   * @param anzeige aktuelle Anzeige 
   * @param richtung Richtung Vorgnger/Nachfolger
   * @param ebene Ebene deren Vorgnger/Nachfolger bestimmt werden sollen
   * @param art Art der Vorgnger/Nachfolger Beziehung (direkt, gleicher Typ)
   * @return Liste mit Anzeigen
   */
  public List<SystemObject> getVorgaengerNachfolgerInLaengsrichtungInEbene ( SystemObject anzeige, ArbeitsRichtungVN richtung, int ebene, VorgaengerNachfolgerArt art )
  {
    List<SystemObject> zuBetrachtendeAnzeigen = new ArrayList<SystemObject>();
    List<SystemObject> anzeigenEinerEbene     = new ArrayList<SystemObject>();

    // Ausgangsbasis auslsende Anzeige
    
    anzeigenEinerEbene.add( anzeige );
    
    // Loop ber die Ebenen um die Anzeigen zu bestimmen
    
//    System.out.println( "ParameterVorgaengerNachfolger.getVorgaengerNachfolgerInEbene()" );
//    System.out.println( Tools.getObjektName( anzeige ));
    
    for (int e = 0; e < ebene; ++e)
    {
      for (SystemObject so : anzeigenEinerEbene )
      {
//        System.out.println("Ebene " + e + ": " + Tools.getObjektName( so ));
        
        List<SystemObject> vorgaengerNachfolger = getVorgaengerNachfolgerInLaengsRichtung( so, richtung, 1, null, art );
        zuBetrachtendeAnzeigen.addAll( vorgaengerNachfolger );
      }
      
      anzeigenEinerEbene.clear();
      anzeigenEinerEbene.addAll( zuBetrachtendeAnzeigen );
      zuBetrachtendeAnzeigen.clear();
    }
      
    return anzeigenEinerEbene;
  }

  /**
   * Methode liefert aus der Vorgnger/Nachfolger Beziehung den gesuchten Vorgnger bzw. die
   * gesuchten Nachfolger in Lngsrichtung. Als bergabeparameter werden die Parameter und die 
   * die Anzeige fr die Vorgnger/Nachfolger bestimmt werden sollen, bergeben.
   * @param anzeige Anzeige fr die Vorgnger/Nachfolger bestimmt werden sollen
   * @param richtung Richtung in die die Regel arbeitet
   * @param art Welche Vorgnger/Nachfolger sollen gesucht werden
   * @return Feld mit Vorgngern/Nachfolgern
   */
  public SystemObject [] bestimmeVorgaengerNachfolgerInLaengsRichtung (SystemObject anzeige, ArbeitsRichtungVN richtung, VorgaengerNachfolgerArt art)
  {
    SystemObject[] vorgaengerNachfolger = new SystemObject [0];

    VorgaengerNachfolger vn = getVorgaengerNachfolger( anzeige );
    
    if (vn != null)
    {
      if (richtung == ArbeitsRichtungVN.VORGAENGER)
      {
        if (art == VorgaengerNachfolgerArt.GLEICHERTYP)
        {
          if (vn.getLaengsRichtung().getVorgaengerGleicherTyp() != null)
          {
            vorgaengerNachfolger = new SystemObject [1];
            vorgaengerNachfolger [0] = vn.getLaengsRichtung().getVorgaengerGleicherTyp();
          }
        }
  
        if (art == VorgaengerNachfolgerArt.DIREKT)
        {
          if (vn.getLaengsRichtung().getDirekterVorgaenger() != null)
          {
            vorgaengerNachfolger = new SystemObject [1];
            vorgaengerNachfolger [0] = vn.getLaengsRichtung().getDirekterVorgaenger();
          }
        }
  
      }
      else
      {
        if (art == VorgaengerNachfolgerArt.GLEICHERTYP)
          vorgaengerNachfolger = vn.getLaengsRichtung().getNachfolgerGleicherTyp();
  
        if (art == VorgaengerNachfolgerArt.DIREKT)
          vorgaengerNachfolger = vn.getLaengsRichtung().getDirekterNachfolger();
      }
    }
    
    return vorgaengerNachfolger;
  }
  
  /**
   * Methode liefert aus der Vorgnger/Nachfolger Beziehung den gesuchten Vorgnger bzw. die
   * gesuchten Nachfolger in Lngsrichtung. Als bergabeparameter werden die Parameter und die 
   * die Anzeige fr die Vorgnger/Nachfolger bestimmt werden sollen, bergeben.
   * @param anzeige Anzeige fr die Vorgnger/Nachfolger bestimmt werden sollen
   * @param richtung Richtung in die die Regel arbeitet
   * @param art Welche Vorgnger/Nachfolger sollen gesucht werden
   * @return Feld mit Vorgngern/Nachfolgern
   */
  public SystemObject [] bestimmeVorgaengerNachfolgerInQuerRichtung (SystemObject anzeige, ArbeitsRichtungVN richtung, VorgaengerNachfolgerArt art)
  {
    SystemObject[] vorgaengerNachfolger = new SystemObject [0];

    VorgaengerNachfolger vn = getVorgaengerNachfolger( anzeige );
    
    if (vn == null)
      return vorgaengerNachfolger;
  
    if (richtung == ArbeitsRichtungVN.VORGAENGER)
    {
      if (art == VorgaengerNachfolgerArt.GLEICHERTYP)
      {
        if (vn.getQuerRichtung().getVorgaengerGleicherTyp() != null)
        {
          vorgaengerNachfolger = new SystemObject [1];
          vorgaengerNachfolger [0] = vn.getQuerRichtung().getVorgaengerGleicherTyp();
        }
      }

      if (art == VorgaengerNachfolgerArt.DIREKT)
      {
        if (vn.getQuerRichtung().getDirekterVorgaenger() != null)
        {
          vorgaengerNachfolger = new SystemObject [1];
          vorgaengerNachfolger [0] = vn.getQuerRichtung().getDirekterVorgaenger();
        }
      }

    }
    else
    {
      if (art == VorgaengerNachfolgerArt.GLEICHERTYP)
        vorgaengerNachfolger = vn.getQuerRichtung().getNachfolgerGleicherTyp();

      if (art == VorgaengerNachfolgerArt.DIREKT)
        vorgaengerNachfolger = vn.getQuerRichtung().getDirekterNachfolger();
    }
    
    return vorgaengerNachfolger;
  }

  /**
   * Methode zum Erzeugen einer Attributgruppe mit den Parametern
   * @return Attributgruppe mit den Parametern
   */
  public Data erzeugeAttributgruppe()
  {
	  ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
		
    if (connection == null)
      return null;

    // Attributgruppe zusammensetzen und versenden
      
    AttributeGroup atg = (AttributeGroup) connection.getDataModel().getAttributeGroup(_pidAtg);
    Data data = connection.createData(atg );

    synchronized (m_nachbarn)
    {
      Iterator<Entry<SystemObject, VorgaengerNachfolger>> it = m_nachbarn.entrySet().iterator();
      
      Array beziehungen = data.getArray( "Beziehungen" );
      beziehungen.setLength( m_nachbarn.size() );
        
      int i = 0;
      while (it.hasNext())
      {
        Entry<SystemObject, VorgaengerNachfolger> entry = it.next();
        
        SystemObject anzeige = entry.getKey();
        VorgaengerNachfolger parameter = entry.getValue();

        Data aktuell = beziehungen.getItem( i++ );
        
        aktuell.getReferenceValue( "Anzeige" ).setSystemObject( anzeige );
        
        /*
         * Parameter in Lngsrichtung
         */
        Data dataLaengs = aktuell.getItem( "Lngsrichtung" );
        NachbarAnzeigen paraLaengs = parameter.getLaengsRichtung();
        
        erzeugeNachbarAnzeigen( dataLaengs, paraLaengs );
        
        /*
         * Parameter in Querrichtung
         */
        Data dataQuer = aktuell.getItem( "Querrichtung" );
        NachbarAnzeigen paraQuer = parameter.getQuerRichtung();
        
        erzeugeNachbarAnzeigen( dataQuer, paraQuer );

      }
    }

    return data;
  }
  
  /**
   * Methode speichert die Nachbaranzeigen in der Attributgruppe
   * @param data Element in der Attributgruppe
   * @param para zu speicherende Parameter
   */
  private void erzeugeNachbarAnzeigen (Data data, NachbarAnzeigen para)
  {
    /*
     * Direkte Vorgnger
     */
    data.getReferenceValue( "DirekterVorgnger" ).setSystemObject( para.getDirekterVorgaenger() );

    /*
     * Direkte Nachfolger
     */
    SystemObject[] dnf = para.getDirekterNachfolger();
    ReferenceArray direkteNachfolger = data.getReferenceArray(  "DirekteNachfolger" );
    direkteNachfolger.setLength( dnf.length );
    for (int j=0; j<dnf.length; ++j)
      direkteNachfolger.getReferenceValue( j ).setSystemObject( dnf [j] );
      
    /*
     * Vorgnger gleichen Typs
     */
    data.getReferenceValue( "VorgngerGleichenTyps" ).setSystemObject( para.getVorgaengerGleicherTyp() );

    /*
     * Nachfolger gleichen Typs
     */
    SystemObject[] nfgt = para.getNachfolgerGleicherTyp();
    ReferenceArray nachfolgerGleichenTyps = data.getReferenceArray(  "NachfolgerGleichenTyps" );
    nachfolgerGleichenTyps.setLength( nfgt.length );
    for (int j=0; j<nfgt.length; ++j)
      nachfolgerGleichenTyps.getReferenceValue( j ).setSystemObject( nfgt [j] );
  }

  /**
   * Methode liefert die Parameter als Attributgruppe fr den DaV
   * @return Attributgruppe
   */
  public Data getData ()
  {
  	return erzeugeAttributgruppe();
  }
 
  /**
   * Methode leifert die Querbeziehung einen Anzeigequerschnitts auf Basis des Querabgleichs
   * @param soAQ Systemobejkt des Anzeigequerschnitts
   * @return Feld mit Anzeigen von links nach rechts und von oben nach unten
   */
  public SystemObject [] getQuerbeziehung (SystemObject soAQ)
  {
  	SystemObject [] anzeigen = new SystemObject [0];
  	
  	// Alle Anzeigen die zu dem AQ gehren
  	
  	SystemObject[] tmp = Tools.getSystemObjekteAnzeigen(soAQ);
  	
  	// Die Anzeige suchen, die keinen direkten Vorgnger mehr hat (= linke Anzeige)
  	
  	SystemObject anfang = null;
  	for (SystemObject so : tmp)
  	{
  		if (getVorgaengerInQuerRichtung(so, VorgaengerNachfolgerArt.DIREKT) == null)
  		{
  			anfang = so;
  			break;
  		}
  	}
  	
  	if (anfang != null)
  	{
  		Vector<SystemObject> reihenfolge = new Vector<SystemObject>();
  		reihenfolge.add(anfang);
  		
  		SystemObject zuBearbeiten = anfang;
  		
  		do
  		{
  		  SystemObject[] nachfolger = getNachfolgerInQuerRichtung(zuBearbeiten, VorgaengerNachfolgerArt.DIREKT);
  		  if (nachfolger.length > 0)
  		  {
  		  	zuBearbeiten = nachfolger [0];
  		  	reihenfolge.add(zuBearbeiten);
  		  }
  		  else
  		  	zuBearbeiten = null;
  		}
  		while (zuBearbeiten != null);

  		anzeigen = reihenfolge.toArray(new SystemObject [reihenfolge.size()]);
  	}
  	
  	return anzeigen;
  }

  /**
   * Methode liefert eine Kopie der Instanz
   * @return Kopie der Instanz
   */
  public ParameterVorgaengerNachfolger getKopie ()
  {
  	ParameterVorgaengerNachfolger kopie = new ParameterVorgaengerNachfolger();
  	
  	synchronized (m_nachbarn)
    {
	  	Iterator<Entry<SystemObject, VorgaengerNachfolger>> it = m_nachbarn.entrySet().iterator();
	  	while (it.hasNext())
	  	{
	  		Entry<SystemObject, VorgaengerNachfolger> entry = it.next();
	  		
	  		SystemObject         so = entry.getKey();
	  		VorgaengerNachfolger vn = entry.getValue();
	  		
	  		kopie.getNachbarn().put(so, vn.getKopie());
	  	}
    }
  	
  	return kopie;
  }

	/**
   * Methode liefert nachbarn zurck
   * @return nachbarn
   */
  public HashMap<SystemObject, VorgaengerNachfolger> getNachbarn()
  {
  	return m_nachbarn;
  }

	/**
   * Methode setzt die Variable nachbarn
   * @param nachbarn zu bernehmender Wert fr nachbarn
   */
  public void setNachbarn(HashMap<SystemObject, VorgaengerNachfolger> nachbarn)
  {
  	m_nachbarn = nachbarn;
  }
}
