package de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen;

import java.util.List;
import java.util.Vector;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.Konfig;

/**
 * Klasse speichert die Eigenschaften eines Anzeigetyps
 * @author ObertM
 *
 */
public class AnzeigeTypEigenschaften
{
	/**
	 * Type des Anzeigetyps
	 */
  private EAnzeigeTypen _typ;
  
  /**
   * SystemObjekt des Anzeigetyps
   */
  private SystemObject _anzeigeTyp;
  
  /**
   * des Defaultbilds (WvzInhalt) des Anzeigetyps
   */
  private SystemObject _defaultBild;
  
  /**
   * Liste mit den WVZ-Inhalten
   */
  private Vector<SystemObject> _inhalte = new Vector<SystemObject>();
  
  /**
   * Konstruktor der Klasse
   * @param typ Anzeigtyp
   * @param pidAnzeigeTyp Pid des Systemobjekts zu diesem Typ
   * @param pidDefaultBild Pid des Defaultbildes
   */
	public AnzeigeTypEigenschaften(EAnzeigeTypen typ, String pidAnzeigeTyp, String pidDefaultBild)
  {
		_typ = typ;
		
		_anzeigeTyp  = Konfig.getInstance().getSystemObject(pidAnzeigeTyp);
		_defaultBild = Konfig.getInstance().getSystemObject(pidDefaultBild);
  }

	/**
	 * Konstruktor der Klasse
	 */
	public AnzeigeTypEigenschaften()
  {
  }

	public AnzeigeTypEigenschaften(EAnzeigeTypen typ, String pidAnzeigeTyp, String pidDefaultBild, 
			List<String> pidWvzInhalte)
  {
		_typ = typ;
		
		_anzeigeTyp  = Konfig.getInstance().getSystemObject(pidAnzeigeTyp);
		_defaultBild = Konfig.getInstance().getSystemObject(pidDefaultBild);
		
		for (String s : pidWvzInhalte)
		{
			SystemObject so = Konfig.getInstance().getSystemObject(s);
			
			if (so == null)
				System.err.println("Kein Systemobjekt zu Pid " + s + " gefunden");
			
			if ((so != null) && (!_inhalte.contains(so)))
		    _inhalte.add(so);
		}
  }

	public AnzeigeTypEigenschaften(EAnzeigeTypen typ)
  {
		_typ = typ;
  }

	/**
   * Methode liefert typ zurck
   * @return typ
   */
  public EAnzeigeTypen getTyp()
  {
  	return _typ;
  }

	/**
   * Methode setzt die Variable typ
   * @param typ zu bernehmender Wert fr typ
   */
  public void setTyp(EAnzeigeTypen typ)
  {
  	_typ = typ;
  }

	/**
   * Methode liefert anzeigeTyp zurck
   * @return anzeigeTyp
   */
  public SystemObject getAnzeigeTyp()
  {
  	return _anzeigeTyp;
  }

	/**
   * Methode setzt die Variable anzeigeTyp
   * @param anzeigeTyp zu bernehmender Wert fr anzeigeTyp
   */
  public void setAnzeigeTyp(SystemObject anzeigeTyp)
  {
  	_anzeigeTyp = anzeigeTyp;
  }

	/**
   * Methode liefert defaultBild zurck
   * @return defaultBild
   */
  public SystemObject getDefaultBild()
  {
  	return _defaultBild;
  }

	/**
   * Methode setzt die Variable defaultBild
   * @param defaultBild zu bernehmender Wert fr defaultBild
   */
  public void setDefaultBild(SystemObject defaultBild)
  {
  	_defaultBild = defaultBild;
  }

	/**
   * Methode liefert inhalte zurck
   * @return inhalte
   */
  public Vector<SystemObject> getInhalte()
  {
  	return _inhalte;
  }

	/**
   * Methode setzt die Variable inhalte
   * @param inhalte zu bernehmender Wert fr inhalte
   */
  public void setInhalte(Vector<SystemObject> inhalte)
  {
  	_inhalte = inhalte;
  }

  /**
   * Methode liefert eine Kopie der Instanz
   * @return Kopie der Instanz
   */
	public AnzeigeTypEigenschaften getCopy()
  {
		AnzeigeTypEigenschaften copy = new AnzeigeTypEigenschaften();
		
		copy.setAnzeigeTyp(getAnzeigeTyp());
		copy.setDefaultBild(getDefaultBild());
		copy.setTyp(getTyp());
		
		Vector<SystemObject> v = new Vector<SystemObject>();
		v.addAll(getInhalte());
		copy.setInhalte(v);
		
	  return copy;
  }
}
