/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.sba.plugin.sonstiges.davstatus;

import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindungEreignis;
import de.bsvrz.buv.rw.rw.dav.IDaVVerbindungZuhoerer;
import de.bsvrz.buv.rw.rw.dav.IDaVVerbindungsAbbauWarten;
import de.bsvrz.sba.plugin.sonstiges.Activator;

/**
 * Liefert den aktuellen Status der Verbindung zum Datenverteiler, inklusive
 * <ul>
 * <li>Datenverteiler IP-Adresse</li>
 * <li>Datenverteiler Port</li>
 * <li>Datenverteiler Benutzer</li>
 * </ul>
 * 
 * @author HofmannJ
 * 
 */
public class ViewDaVStatus extends ViewPart
{

	public static final String ID = "de.bsvrz.sba.plugin.sonstiges.davstatus.ViewDaVStatus";
	private Composite _top = null;
	private Group _groupDaVStatus = null;
	private Label _labelDaVStatus = null;
	private Label _textDaVStatus = null;

	/**
	 * Speichert die Bilder fr die Verbindung
	 */
	private ImageRegistry _ir = new ImageRegistry();
	private Composite _compositeInfo = null;
	private Label _labelDaVIp = null;
	private Label _labelDaVPort = null;
	private Label _labelDaVBenutzer = null;
	private Text _textDaVIp = null;
	private Text _textDaVPort = null;
	private Text _textDaVBenutzer = null;
	private String _ip = null;
	private int _port = -1;
	private String _benutzer = null;

	/**
	 * Listener fr den Status der DaV-Verbindung
	 */
	private IDaVVerbindungZuhoerer _listenerDaVVerbindung;

	/**
	 * Listener fr das Warten auf den Verbindungsabbau
	 */
	private IDaVVerbindungsAbbauWarten _listenerVerbindungsabbauWarten;

	@Override
	public void createPartControl(Composite parent)
	{
		_top = new Composite(parent, SWT.NONE);
		_top.setLayout(new GridLayout());
		createGroupDaVStatus();

		/*
		 * ImageRegistry versorgen
		 */
		_ir.put("online", Activator
				.getImageDescriptor("icons/davstatus/Ort_GRUEN_96x96.png"));
		_ir.put("offline", Activator
				.getImageDescriptor("icons/davstatus/Ort_SW_96x96.png"));

		/*
		 * Basisinitialisierung
		 */
		setStatusInitial();

		/*
		 * Registrieren der Listener 
		 */
		registriereDaVListener();
	}

	@Override
	public void setFocus()
	{
		setStatusInitial();

	}

	/**
	 * This method initializes _groupDaVStatus
	 * 
	 */
	private void createGroupDaVStatus()
	{
		GridData gridDataLabelDaVStatus = new GridData();
		gridDataLabelDaVStatus.grabExcessHorizontalSpace = true;
		gridDataLabelDaVStatus.verticalAlignment = GridData.CENTER;
		gridDataLabelDaVStatus.horizontalAlignment = GridData.CENTER;
		GridData gridDataTextDaVStatus = new GridData();
		gridDataTextDaVStatus.horizontalAlignment = GridData.CENTER;
		gridDataTextDaVStatus.grabExcessHorizontalSpace = true;
		gridDataTextDaVStatus.verticalAlignment = GridData.CENTER;
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 1;
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = GridData.FILL;
		_groupDaVStatus = new Group(_top, SWT.NONE);
		_groupDaVStatus.setLayoutData(gridData);
		_groupDaVStatus.setLayout(gridLayout);
		_groupDaVStatus.setText("DaV Status:");
		_labelDaVStatus = new Label(_groupDaVStatus, SWT.NONE);
		// _labelDaVStatus.setText(" ");
		_labelDaVStatus.setLayoutData(gridDataLabelDaVStatus);
		_textDaVStatus = new Label(_groupDaVStatus, SWT.NONE);
		_textDaVStatus.setLayoutData(gridDataTextDaVStatus);
		createCompositeInfo();
	}

	/**
	 * This method initializes _compositeInfo
	 * 
	 */
	private void createCompositeInfo()
	{
		GridData gridDataTextDaVBenutzer = new GridData();
		gridDataTextDaVBenutzer.horizontalAlignment = GridData.FILL;
		gridDataTextDaVBenutzer.grabExcessHorizontalSpace = true;
		gridDataTextDaVBenutzer.verticalAlignment = GridData.CENTER;
		GridData gridDataTextDaVPort = new GridData();
		gridDataTextDaVPort.horizontalAlignment = GridData.FILL;
		gridDataTextDaVPort.grabExcessHorizontalSpace = true;
		gridDataTextDaVPort.verticalAlignment = GridData.CENTER;
		GridData gridDataTextDaVIp = new GridData();
		gridDataTextDaVIp.horizontalAlignment = GridData.FILL;
		gridDataTextDaVIp.grabExcessHorizontalSpace = true;
		gridDataTextDaVIp.verticalAlignment = GridData.CENTER;
		GridLayout gridLayoutCompositeInfo = new GridLayout();
		gridLayoutCompositeInfo.numColumns = 2;
		GridData gridDataCompositeInfo = new GridData();
		gridDataCompositeInfo.horizontalAlignment = GridData.FILL;
		gridDataCompositeInfo.grabExcessHorizontalSpace = true;
		gridDataCompositeInfo.grabExcessVerticalSpace = false;
		gridDataCompositeInfo.verticalAlignment = GridData.CENTER;
		_compositeInfo = new Composite(_groupDaVStatus, SWT.NONE);
		_compositeInfo.setLayoutData(gridDataCompositeInfo);
		_compositeInfo.setLayout(gridLayoutCompositeInfo);
		_labelDaVIp = new Label(_compositeInfo, SWT.NONE);
		_labelDaVIp.setText("IP:");
		_textDaVIp = new Text(_compositeInfo, SWT.BORDER);
		_textDaVIp.setEditable(false);
		_textDaVIp.setLayoutData(gridDataTextDaVIp);
		_labelDaVPort = new Label(_compositeInfo, SWT.NONE);
		_labelDaVPort.setText("Port:");
		_textDaVPort = new Text(_compositeInfo, SWT.BORDER);
		_textDaVPort.setEditable(false);
		_textDaVPort.setLayoutData(gridDataTextDaVPort);
		_labelDaVBenutzer = new Label(_compositeInfo, SWT.NONE);
		_labelDaVBenutzer.setText("Benutzer:");
		_textDaVBenutzer = new Text(_compositeInfo, SWT.BORDER);
		_textDaVBenutzer.setEditable(false);
		_textDaVBenutzer.setLayoutData(gridDataTextDaVBenutzer);
	}

	/**
	 * Setzt den initialien Status Online oder Offline
	 */
	private void setStatusInitial()
	{
		DaVVerbindung dav = DaVVerbindung.getInstanz();
		if (dav.istDaVVerbindungAktiv())
		{
			setStatusOnline(dav.getDaVIp(), dav.getDaVPort(), dav.getDaVBenutzer());
		} else
		{
			setStatusOffline();
		}
	}

	/**
	 * Setzt den Online- oder Offlinestatus.
	 * 
	 * @param ip
	 *          IP-Adresse des DaV
	 * @param port
	 *          Port des DaV
	 * @param benutzer
	 *          Benutzer der DaV-Verbindung
	 */
	private void setStatusOnline(String ip, int port, String benutzer)
	{
		this._ip = ip;
		this._port = port;
		this._benutzer = benutzer;

		Runnable run = new Runnable()
		{

			@Override
			public void run()
			{
				_compositeInfo.setVisible(true);
				_labelDaVStatus.setImage(_ir.get("online"));
				_textDaVStatus.setText("Online");
				_textDaVStatus.getParent().layout();
				_textDaVIp.setText(_ip);
				_textDaVPort.setText("" + _port);
				_textDaVBenutzer.setText(_benutzer);
			}

		};

		PlatformUI.getWorkbench().getDisplay().syncExec(run);

	}

	/**
	 * Setzt den Status auf Offline.
	 */
	private void setStatusOffline()
	{
		Runnable run = new Runnable()
		{

			@Override
			public void run()
			{
				if (!_labelDaVStatus.isDisposed())
				{
					_labelDaVStatus.setImage(_ir.get("offline"));
				}

				if (!_textDaVStatus.isDisposed())
				{
					_textDaVStatus.setText("Offline");
					_textDaVStatus.getParent().layout();
				}

				if (!_compositeInfo.isDisposed())
				{
					_compositeInfo.setVisible(false);
				}

			}
		};

		// System.out.println("PlatformUI.getWorkbench().getDisplay()=>" +
		// PlatformUI.getWorkbench().getDisplay());

		PlatformUI.getWorkbench().getDisplay().asyncExec(run);

	}

	/**
	 * Registiert die Listener auf die DaV-Verbindung. Ein
	 * {@link IDaVVerbindungZuhoerer} hrt dabei auf die Verbindung zum DaV und
	 * setzt den Status. Durch einen {@link IDaVVerbindungsAbbauWarten}-Listener
	 * wird der {@link IDaVVerbindungZuhoerer} von der Liste der Zuhrer entfernt.
	 * <br>
	 * In der dispose-Methode dieser Klasse kann er nicht entfernt werden, da zu
	 * dem Zeitpunkt wo diese Mehtode aufgerufen - beim Beenden des Rahmenwerkes -
	 * auch alle {@link IDaVVerbindungZuhoerer} darber informiert werden, dass
	 * gerade die Verbindung abgebaut wird.<br>
	 * Der {@link IDaVVerbindungsAbbauWarten}-Listener kann wieder in der
	 * dispose-Methode entfernt werden.
	 */
	private void registriereDaVListener()
	{
		/*
		 * IDaVVerbindungsZuhoerer
		 */

		_listenerDaVVerbindung = new IDaVVerbindungZuhoerer()
		{

			@Override
			public void verbindungAbgebaut(DaVVerbindungEreignis e)
			{
				setStatusOffline();
			}

			@Override
			public void verbindungAufgebaut(DaVVerbindungEreignis e)
			{
				setStatusOnline(e.getDaVIpAdresse(), e.getDaVPort(), e.getDaVBenutzer());
			}

		};

		DaVVerbindung.getInstanz().hinzufuegenZuhoerer(_listenerDaVVerbindung);

	}

	public void dispose()
	{
		// System.out.println("DISPOSE............................");

		/*
		 * Verbindungsabbau Warten
		 */
		_listenerVerbindungsabbauWarten = new IDaVVerbindungsAbbauWarten()
		{

			@Override
			public boolean DaVVerbindungsAbbauOk()
			{
				System.out.println("DaVStatus DaVVerbindungsAbbauOk");

				if (_listenerDaVVerbindung != null)
				{
					DaVVerbindung.getInstanz().entferneZuhoerer(_listenerDaVVerbindung);
					_listenerDaVVerbindung = null;
				}

				if (_listenerDaVVerbindung == null)
				{
					return true;
				} else
				{
					return false;
				}
			}

		};

		DaVVerbindung.getInstanz().hinzufuegenVerbindungsabbauWarten(
				_listenerVerbindungsabbauWarten);

		if (_listenerDaVVerbindung != null)
		{
			DaVVerbindung.getInstanz().entferneZuhoerer(_listenerDaVVerbindung);
		}

		super.dispose();

		DaVVerbindung.getInstanz().entfernenVerbindungsabbauWarten(
				_listenerVerbindungsabbauWarten);

		// System.out.println("DISPOSE BEENDET!");
	}

}
