/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.sba.plugin.sonstiges.statusleiste;

import de.bsvrz.buv.rw.rw.statusleiste.xml.datenstruktur.EreignisStatusLeisteGeaendert;
import de.bsvrz.buv.rw.rw.statusleiste.xml.datenstruktur.IZuhoererStatusLeisteAenderung;
import de.bsvrz.buv.rw.rw.statusleiste.xml.datenstruktur.StatusLeisteElement;
import de.bsvrz.buv.rw.rw.statusleiste.xml.datenstruktur.StatusLeisteVerwaltung;


/**
 * Zentrale Verwaltungsklasse zum Anpassen der Rahmenwerk-Menueleiste
 * 
 * @author Pittner
 * 
 */
public class StatusLineModifier implements IZuhoererStatusLeisteAenderung
{

  /**
   * Singleton-Instanz
   */
  private static StatusLineModifier _instanz = null;

  /**
   * Liefert die Singletoninstanz.
   * 
   * @return Instanz dieser Klasse.
   */
  public static StatusLineModifier getInstanz()
  {
    if (_instanz == null)
    {
      _instanz = new StatusLineModifier();
    }
    return _instanz;
  }

  /**
   * Konstruktor, setzt die Verbindung zum DaV.
   * 
   * @param connection
   *          Verbindung zum DaV.
   */
  private StatusLineModifier()
  {
     registriereSymbolleistenElemente();
  }
  
  
  /**
   * 
   */
  public void anmeldenAufMenuLadeListener(){    
    StatusLeisteVerwaltung.getInstanz().hinzufuegenZuhoerer(this);
  }
  
  /**
   * 
   */
  public void abmeldenAufMenuLadeListener(){
    StatusLeisteVerwaltung.getInstanz().entferneZuhoerer(this);
  }
   
  
  /**
   * Fhrt die Registrierung von Symbolleistenelementen durch
   */
  private void registriereSymbolleistenElemente()
  {    
    String idDavItem = "sl.davelement";
    SymbolleisteDavItem sdi = new SymbolleisteDavItem(idDavItem);
    StatusLeisteElement sleDav = new StatusLeisteElement(idDavItem, "DaVelement", sdi);
    String idArchivItem = "sl.archivelement";
    SymbolleisteArchivItem sai = new SymbolleisteArchivItem(idArchivItem);
    StatusLeisteElement sleArchiv = new StatusLeisteElement(idArchivItem, "Archivelement", sai);

    try
    {
      StatusLeisteVerwaltung.getInstanz().hinzufuegenElement(sleDav);
      StatusLeisteVerwaltung.getInstanz().hinzufuegenElement(sleArchiv);
    } catch (IllegalArgumentException e)
    {
      e.printStackTrace();
    }
  }

  @Override
  public void ausfuehren(EreignisStatusLeisteGeaendert e)
  {    
//    System.out.println("StatusLineModifier.ausfuehren()" + e);    
  }
  

 

 

}
