package de.bsvrz.sba.plugin.zlt.dav;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.Collection;
import java.util.Collections;
import java.util.LinkedList;
import java.util.concurrent.ConcurrentHashMap;

import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.zlt.Defs;

public class VerwaltungZltProgrammaktivierungZustaende
{
	private ConcurrentHashMap<SystemObject, ZltGrouped> _mapCombined = new ConcurrentHashMap<SystemObject, ZltGrouped>();
	private ConcurrentHashMap<SystemObject, ZltProgrammaktivierung> _mapDatenProgrammaktivierung = new ConcurrentHashMap<SystemObject, ZltProgrammaktivierung>();

	/**
	 * Wird aufgerufen, wenn eine Programmaktivierung hinzugefgt / gendert
	 * wurde.
	 */
	public static final String PROP_PUT_ZLT_PROGRAMMAKTIVIERUNG = "PUT_ZLT_PROGRAMMAKTIVIERUNG";

	public static final String PROP_UPDATE_ZLT_PROGRAMMAKTIVIERUNG = "UPDATE_ZLT_PROGRAMMAKTIVIERUNG";

	/**
	 * Wird abgefeuert, wenn ein Listener initial ber alle Programmaktivierungen
	 * informiert wird.
	 */
	public static final String PROP_ZLT_PROGRAMMAKTIVIERUNGEN_INIT = "ZLT_PROGRAMMAKTIVIERUNGEN_INIT";

	/**
	 * Wird abgefeuert, wenn ein ProgrammaktivierungZustand hinzugefgt/gendert
	 * wird.
	 */
	public static final String PROP_PUT_ZLT_PROGRAMMAKTIVIERUNG_ZUSTAND = "PUT_ZLT_PROGRAMMAKTIVIERUNG_ZUSTAND";

	public static final String PROP_UPDATE_ZLT_PROGRAMMAKTIVIERUNG_ZUSTAND = "UPDATE_ZLT_PROGRAMMAKTIVIERUNG_ZUSTAND";

	/**
	 * Wird abgefeuert, wenn ein Listener initial ber alle
	 * Programmaktivierungszustnde informiert wird.
	 */
	public static final String PROP_ZLT_PROGRAMMAKTIVIERUNGEN_ZUSTAENDE_INIT = "ZLT_PROGRAMMAKTIVIERUNGEN_ZUSTAENDE_INIT";

	public static final String PROP_ZLT_GROUPED_PUT = "ZLT_GROUPED_PUT";
	/**
	 * Wird abgefeuert, wenn eine Aktualisierung der Daten statt fand.
	 */
	public static final String PROP_ZLT_GROUPED_UPDATE = "ZLT_GROUPED_UPDATE";

	/**
	 * Wird ausgelst um die Listener mit einem Array der gruppierten Daten zu
	 * informieren.
	 */
	public static final String PROP_ZLT_GROUPED_INIT = "ZLT_GROUPED_INIT";

	private PropertyChangeSupport _pcs = new PropertyChangeSupport(this);





	public void putZltProgrammaktivierung(ResultData result)
	{
		if ((result != null) && (result.getObject() != null))
		{
			AttributeGroup atg = result.getDataDescription().getAttributeGroup();
			if (atg.getPid().equals(Defs.ATG_TLS_ZLT_PROGRAMMAKTIVIERUNG))
			{
				ZltProgrammaktivierung zltProgrammaktivierung = getZltProgrammaktivierung(result
						.getObject());
				if (zltProgrammaktivierung == null)
				{
					zltProgrammaktivierung = new ZltProgrammaktivierung(result);
				} else
				{
					zltProgrammaktivierung.initFromResult(result);
				}
				putZltProgrammaktivierung(zltProgrammaktivierung);
			}
		}
	}

	public void putZltProgrammaktivierung(
			ZltProgrammaktivierung programmaktivierung)
	{
		if ((programmaktivierung != null) && (programmaktivierung.getSo() != null))
		{
			if (programmaktivierung == _mapDatenProgrammaktivierung
					.get(programmaktivierung.getSo()))
			{
				_pcs.firePropertyChange(PROP_UPDATE_ZLT_PROGRAMMAKTIVIERUNG, null,
						programmaktivierung);
			} else
			{
				_mapDatenProgrammaktivierung.put(programmaktivierung.getSo(),
						programmaktivierung);
				_pcs.firePropertyChange(PROP_PUT_ZLT_PROGRAMMAKTIVIERUNG, null,
						programmaktivierung);
			}

			ZltGrouped zltGrouped = getZltGrouped(programmaktivierung.getSo());
			zltGrouped.setProgrammaktivierung(programmaktivierung);

			_pcs.firePropertyChange(PROP_ZLT_GROUPED_UPDATE, null, zltGrouped);
		}
	}

	public ZltProgrammaktivierung getZltProgrammaktivierung(SystemObject so)
	{
		ZltProgrammaktivierung re = null;

		if (so != null)
		{
			re = _mapDatenProgrammaktivierung.get(so);
		}

		return re;
	}

	public ZltProgrammaktivierung[] getZltProgrammaktivierungen()
	{
		LinkedList<ZltProgrammaktivierung> list = new LinkedList<ZltProgrammaktivierung>();

		Collection<ZltProgrammaktivierung> values = _mapDatenProgrammaktivierung
				.values();
		for (ZltProgrammaktivierung zltProgrammaktivierung : values)
		{
			list.add(zltProgrammaktivierung);
		}

		if (list.size() > 1)
		{
			Collections.sort(list, new CompAbstractAtgData());
		}

		return list.toArray(new ZltProgrammaktivierung[list.size()]);
	}

	private ZltGrouped getZltGrouped(SystemObject so)
	{
		ZltGrouped re = null;

		if (so != null)
		{
			ZltGrouped zltGrouped = _mapCombined.get(so);
			if (zltGrouped == null)
			{
				zltGrouped = new ZltGrouped(so);
				_mapCombined.put(so, zltGrouped);

				_pcs.firePropertyChange(PROP_ZLT_GROUPED_PUT, null, zltGrouped);
			}

			re = zltGrouped;
		}

		return re;
	}

	/**
	 * Liefer alle verfgbaren Gruppierungen von Programmaktivierungen sowie
	 * ProgrammaktivierungZustnden.
	 * 
	 * @return Array mit den {@link ZltGrouped}-Elementen. Kann leer, aber nicht
	 *         <code>null</code> sein.
	 */
	public ZltGrouped[] getZltGroups()
	{
		LinkedList<ZltGrouped> list = new LinkedList<ZltGrouped>();

		list.addAll(_mapCombined.values());

		Collections.sort(list);

		return list.toArray(new ZltGrouped[list.size()]);
	}

	public void addListener(PropertyChangeListener listener)
	{
		if (listener != null)
		{
			_pcs.addPropertyChangeListener(listener);

			/*
			 * ber die Gruppierten Datenstze informieren
			 */
			listener.propertyChange(new PropertyChangeEvent(
					VerwaltungZltProgrammaktivierungZustaende.this,
					PROP_ZLT_GROUPED_INIT, null, getZltGroups()));

			/*
			 * ber die Programmaktivierung informieren
			 */
			listener.propertyChange(new PropertyChangeEvent(
					VerwaltungZltProgrammaktivierungZustaende.this,
					PROP_ZLT_PROGRAMMAKTIVIERUNGEN_INIT, null,
					getZltProgrammaktivierungen()));

		}
	}

	public void removeListener(PropertyChangeListener listener)
	{
		_pcs.removePropertyChangeListener(listener);
	}

}
