package de.bsvrz.sba.plugin.zlt.dav;

import java.util.Date;

import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Liefert ein Gruppierungselement fr die Zustammenfassung von fr Daten gemt ATG
 * <code>atg.tlsZltProgrammaktivierung</code> sowie ATG <code>atg.tlsZltProgrammaktivierungZustand</code> des Typs
 * <code>typ.deZltProgrammaktivierung</code>.
 * 
 * @author Swarco Traffic Systems GmbH
 * 
 */
public class ZltGrouped implements Comparable<ZltGrouped>
{
  private SystemObject _so;

  private ZltProgrammaktivierung _programmaktivierung;

  public ZltGrouped(SystemObject so)
  {
    this._so = so;
  }

  public SystemObject getSo()
  {
    return _so;
  }

  public void setSo(SystemObject so)
  {
    _so = so;
  }

  public ZltProgrammaktivierung getProgrammaktivierung()
  {
    return _programmaktivierung;
  }

  public void setProgrammaktivierung(ZltProgrammaktivierung programmaktivierung)
  {
    _programmaktivierung = programmaktivierung;
  }

  /**
   * Liefert die Programmnummer aus ZltProgrammaktivierung.
   * 
   * @return Programmnummer oder <code>null</code>, falls nicht verfgbar.
   */
  public Long getProgrammnummer()
  {
    Long re = null;

    if (getProgrammaktivierung() != null)
    {
      re = getProgrammaktivierung().getProgrammNummer();
    }

    return re;
  }

  /**
   * Liefert den Status aus ZltProgrammaktivierung.
   * 
   * @return Status oder <code>null</code>, falls nicht verfgbar.
   */
  public String getStatus()
  {
    String re = null;
    if (getProgrammaktivierung() != null)
    {
      re = getProgrammaktivierung().getStatus();
    }
    return re;
  }

  /**
   * Liefert das Krzel aus der ZltProgrammaktivierung.
   * 
   * @return Krzel oder <code>null</code>, falls nicht verfgbar.
   */
  public String getKuerzel()
  {
    String re = null;

    if (getProgrammaktivierung() != null)
    {
      re = getProgrammaktivierung().getKuerzel();
    }

    return re;
  }

  /**
   * Liefert den aktuellsten Zeitstempel von ZltProgrammaktivierungZustand oder ZltProgrammaktivierung.
   * 
   * @return Aktueller Zeitstempel oder <code>null</code>, falls nicht verfgbar.
   */
  public Date getZeitstempel()
  {
    Date ts = null;

    if (getProgrammaktivierung() != null)
    {
      /*
       * Wenn verfgbar den Zeitstempel aus der Programmaktivierung verwenden.
       */
      ts = getProgrammaktivierung().getZeitstempel();
    }

    return ts;
  }

  @Override
  public String toString()
  {
    return "ZltGrouped [getKuerzel()=" + getKuerzel() + ", getProgrammnummer()=" + getProgrammnummer() + ", getSo()="
        + getSo() + ", getStatus()=" + getStatus() + ", getZeitstempel()=" + getZeitstempel() + "]";
  }

  @Override
  public int compareTo(ZltGrouped o)
  {
    int re = 0;

    if (o != null)
    {
      if ((this.getSo() != null) && (o.getSo() != null))
      {
        re = this.getSo().getNameOrPidOrId().compareTo(o.getSo().getNameOrPidOrId());
      }
      else if (this.getSo() != null)
      {
        re = 1;
      }
      else if (o.getSo() != null)
      {
        re = -1;
      }

    }
    else
    {
      re = 1;
    }

    return re;
  }

}
