/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.koelnloevenich.dif.basislib.urlasserdialog;

import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Monitor;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.rw.dav.BenutzerAngemeldet;
import de.bsvrz.buv.rw.rw.dav.UrlasserAngemeldet;
import de.bsvrz.dav.daf.main.ResultData;
import de.koelnloevenich.dif.basislib.Activator;

/**
 * Diese Klasse liefert den Dialog zum Erfassen des Urlassers. <br>
 * Die erfassten Daten werden in der Klasse {@link UrlasserAngemeldet}
 * gespeichert. <br>
 * Die Verarbeitung der Daten erfolgt ber das {@link UrlasserObjekt}.
 * 
 * @version $Revision: 1.2 $ / $Date: 2014/05/16 08:11:21 $ / ($Author:
 *          HofmannJ $)
 * 
 * @author HofmannJ
 * 
 */
public class UrlasserDialog extends Dialog
{
  public final static int SHOW_PROGRESS_IN_STATUSBAR = 1;
  public final static int DISABLE_CONTROLS_ON_OK = 2;
  public final static int INPUT_OK_ON_CANCEL = 4;
  
  public final static int NEW_URLASSER_1 = 
    SHOW_PROGRESS_IN_STATUSBAR | DISABLE_CONTROLS_ON_OK | INPUT_OK_ON_CANCEL;
  
  
	private Shell _shell = null;
	private Composite _compositeEingaben = null;
	private Composite _compositeKontrollen = null;
	private Composite _compositeStatus = null;
	private Button _schaltflaecheOk = null;
	private Button _schaltflaecheAbbrechen = null;
	private Button _schaltflaecheHilfe = null;
	private Label _labelBenutzer = null;
	private Label _labelPasswort = null;
	private Label _labelUrsache = null;
	private Label _labelVeranlasser = null;
	private Text _textBenutzer = null;
	private Text _textPasswort = null;
	private Text _textVeranlasser = null;
	private Text _textFeldUrsache = null;
	
	

	/**
	 * Speichert den Benutzernamen fr einen spteren Zugriff, wenn sich alle
	 * Widgets dieses Dialogs im Zustand disposed befinden.
	 */
	private String _strBenutzer = null;

	/**
	 * Speichert das Passwort des Benutzers fr einen spteren Zugriff, wenn sich
	 * alle Widgets dieses Dialogs im Zustand disposed befinden.
	 */
	private String _strPasswort = null;

	/**
	 * Speichert den Veranlasser fr einen spteren Zugriff, wenn sich alle
	 * Widgets dieses Dialogs im Zustand disposed befinden.
	 */
	private String _strVeranlasser = null;

	/**
	 * Speichert die Ursache fr einen spteren Zugriff, wenn sich alle Widgets
	 * dieses Dialogs im Zustand disposed befinden.
	 */
	private String _strUrsache = null;

	/**
	 * Label zum Erfassen des Status.
	 */
	@SuppressWarnings("unused")
	private Label _labelStatus = null;

	/**
	 * Eingabe ok
	 */
	private boolean _eingabeOk = false;

	/**
	 * UrlasserObjekt, mit allen Verbindungsdaten
	 */
	private UrlasserObjekt _urlasserObjekt = null;

	/**
	 * Zu versendende Daten
	 */
	private Collection<ResultData> _resultDatas = null;

	/**
	 * Speichert Informationen zu Fehlern, die zurckgegeben werden sollen.
	 */
	private String _fehlerMeldung = null;
	
	 /**
   * Flag fr das Anzeigen des Fortschritts in der Statuszeile
   */
  public int flags; 
  
  private boolean _cancelled = false;

	/**
   * Konstrukor. bernimmt eine {@link Shell} als Eltern-Shell.
   * 
   * @param parent
   */
  public UrlasserDialog(Shell parent)
  {
    this(parent, 0);
  }
  
  /**
   * Konstrukor. bernimmt eine {@link Shell} als Eltern-Shell und zustzlichen Flags
   * 
   * @param parent
   * @param flags
   */
  public UrlasserDialog(Shell parent, int flags)
  {
    super(parent, SWT.NONE);
    this.flags = flags;
  }

	/**
	 * Diese Methode initialisiert die Shell dieses Dialogs und bernimmt die zu
	 * versendenden Daten.
	 * 
	 * @param resultDatas
	 *          Zu versendende {@link ResultData}s. Knnen z.B. in einer
	 *          {@link LinkedList} geliefert werden. Im {@link ResultData} muss
	 *          der korrekte Aspekt fr das Versenden in der DataDescription
	 *          eingetragen sein.
	 * @return Liefert in einer {@link UrlasserDialogInfo}-Instanz Informationen
	 *         dazu, ob der Dialog mit Abbrechen oder OK beendet wurde sowie die
	 *         in den Feldern eingegebenen Strings.
	 */
	public UrlasserDialogInfo anzeigen(Collection<ResultData> resultDatas)
	{
		_resultDatas = resultDatas;

		UrlasserInfo info = UrlasserFactory.getInstanz().getInfo();		
		erzeugeDarstellung(info);
				
		if (info.getBenutzer().length() > 0 && !_textPasswort.isDisposed())
      _textPasswort.setFocus();
    		 
		return new UrlasserDialogInfo(_eingabeOk, info.getBenutzer(), info.getPasswort(),
		    info.getUrsache(), info.getVeranlasser(), _fehlerMeldung);
	}

	/**
	 * Diese Methode initialisiert die Shell dieses Dialogs und bernimmt die zu
	 * versendenden Daten.
	 * 
	 * @param resultDatas
	 *          Zu versendende {@link ResultData}s. Knnen z.B. in einer
	 *          {@link LinkedList} geliefert werden. Im {@link ResultData} muss
	 *          der korrekte Aspekt fr das Versenden in der DataDescription
	 *          eingetragen sein.
	 * @param benutzer
	 *          Name des Benutzers, mit dem das Feld Benutzer vorbelegt werden
	 *          soll.
	 * @param passwort
	 *          Passwort, mit dem das Feld Passwort vorbelegt werden soll.
	 * @param ursache
	 *          Ursache, mit der das Feld Ursache vorbelegt werden soll.
	 * @param veranlasser
	 *          Veranlasser, mit dem das Feld Veranlasser vorbelegt werden soll.
	 * @return Liefert in einer {@link UrlasserDialogInfo}-Instanz Informationen
	 *         dazu, ob der Dialog mit Abbrechen oder OK beendet wurde sowie die
	 *         in den Feldern eingegebenen Strings.
	 */
	public UrlasserDialogInfo anzeigen(Collection<ResultData> resultDatas,
			String benutzer, String passwort, String ursache, String veranlasser)
	{
		_resultDatas = resultDatas;
		erzeugeDarstellung();

		_textBenutzer.setText(benutzer);
		_textPasswort.setText(passwort);		
		_textFeldUrsache.setText(ursache);
		_textVeranlasser.setText(veranlasser);

		return new UrlasserDialogInfo(_eingabeOk, _strBenutzer, _strPasswort,
				_strUrsache, _strVeranlasser, _fehlerMeldung);
	}

	/**
	 * Erzeugt die Darstellung
	 */
	private void erzeugeDarstellung()
	{
		_shell = new Shell(getParent(), SWT.TITLE | SWT.BORDER
				| SWT.APPLICATION_MODAL);
		_shell.setText("Urlasser Information");
		_shell.setSize(new Point(400, 250));
		_shell.setLayout(new GridLayout());

		erzeugeCompositeEingaben();
		erzeugeCompositeKontrollen();
		erzeugeCompositeStatus();
	
		Control[] tabListeEingabe = new Control[] { _textBenutzer, _textPasswort,
		    _textFeldUrsache, _textVeranlasser }; 
		_compositeEingaben.setTabList(tabListeEingabe);
		
		Control[] tabListeKontrolle = new Control[] { _schaltflaecheOk,
				_schaltflaecheAbbrechen, _schaltflaecheHilfe };
		_compositeKontrollen.setTabList(tabListeKontrolle);

		Control[] tabListeShell = new Control[] { _compositeEingaben,
				_compositeKontrollen };
		_shell.setTabList(tabListeShell);
	 
		_shell.open();

		Display display = getParent().getDisplay();

		/*
		 * Dialog zentrieren
		 */
		Monitor primary = display.getPrimaryMonitor();
		Rectangle bounds = primary.getBounds();
		Rectangle rect = _shell.getBounds();
		int x = bounds.x + (bounds.width - rect.width) / 2;
		int y = bounds.y + (bounds.height - rect.height) / 2;
		_shell.setLocation(x, y);
		
//		String name = BenutzerAngemeldet.getInstanz().getName();
//	    if (name != null){
//	      _textBenutzer.setText(name);
//	      _textPasswort.setFocus();
//	    }
       
		while (!_shell.isDisposed())
			if (!display.readAndDispatch())
				display.sleep();
	}
	
	/**
	 * Erzeugt die Darstellung
	 */
	private void erzeugeDarstellung(UrlasserInfo info)
	{
	  _shell = new Shell(getParent(), SWT.TITLE | SWT.BORDER
	      | SWT.APPLICATION_MODAL);
	  _shell.setText("Urlasser Information");
	  _shell.setSize(new Point(400, 250));
	  _shell.setLayout(new GridLayout());
	  
	  erzeugeCompositeEingaben(info);
	  erzeugeCompositeKontrollen();
	  erzeugeCompositeStatus();
	  
	  Control[] tabListeEingabe = new Control[] { _textBenutzer, _textPasswort,
	      _textFeldUrsache, _textVeranlasser }; 
	  _compositeEingaben.setTabList(tabListeEingabe);
	  
	  Control[] tabListeKontrolle = new Control[] { _schaltflaecheOk,
	      _schaltflaecheAbbrechen, _schaltflaecheHilfe };
	  _compositeKontrollen.setTabList(tabListeKontrolle);
	  
	  Control[] tabListeShell = new Control[] { _compositeEingaben,
	      _compositeKontrollen };
	  
	  _shell.setTabList(tabListeShell);
	  
	  _shell.open();
	  
	  Display display = getParent().getDisplay();
	  
	  /*
	   * Dialog zentrieren
	   */
	  Monitor primary = display.getPrimaryMonitor();
	  Rectangle bounds = primary.getBounds();
	  Rectangle rect = _shell.getBounds();
	  int x = bounds.x + (bounds.width - rect.width) / 2;
	  int y = bounds.y + (bounds.height - rect.height) / 2;
	  _shell.setLocation(x, y);
	  
//	  String name = BenutzerAngemeldet.getInstanz().getName();
//	  if (name != null){
//	    _textBenutzer.setText(name);
//	    _textPasswort.setFocus();
//	  }
	  
	  while (!_shell.isDisposed())
	    if (!display.readAndDispatch())
	      display.sleep();
	}

	/**
	 * Erzeugt das Composite zum Erfassen der Eingaben bzgl. des Benutzers.
	 * 
	 */
	private void erzeugeCompositeEingaben()
	{
		GridData gridData1 = new GridData();
		gridData1.horizontalAlignment = GridData.BEGINNING;
		gridData1.verticalAlignment = GridData.BEGINNING;
		GridData gridDataTextUrsache = new GridData();
		gridDataTextUrsache.horizontalAlignment = GridData.FILL;
		gridDataTextUrsache.grabExcessHorizontalSpace = true;
		gridDataTextUrsache.grabExcessVerticalSpace = true;
		gridDataTextUrsache.verticalAlignment = GridData.FILL;
		GridData gridDataTextVeranlasser = new GridData();
		gridDataTextVeranlasser.horizontalAlignment = GridData.FILL;
		gridDataTextVeranlasser.grabExcessHorizontalSpace = true;
		gridDataTextVeranlasser.verticalAlignment = GridData.CENTER;
		GridData gridDataTextPasswort = new GridData();
		gridDataTextPasswort.horizontalAlignment = GridData.FILL;
		gridDataTextPasswort.grabExcessHorizontalSpace = true;
		gridDataTextPasswort.verticalAlignment = GridData.CENTER;
		GridData gridDataTextBenutzer = new GridData();
		gridDataTextBenutzer.horizontalAlignment = GridData.FILL;
		gridDataTextBenutzer.grabExcessHorizontalSpace = true;
		gridDataTextBenutzer.verticalAlignment = GridData.CENTER;
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		GridData gridDataEingaben = new GridData();
		gridDataEingaben.horizontalAlignment = GridData.FILL;
		gridDataEingaben.grabExcessHorizontalSpace = true;
		gridDataEingaben.grabExcessVerticalSpace = true;
		gridDataEingaben.verticalAlignment = GridData.FILL;
		_compositeEingaben = new Composite(_shell, SWT.NONE);
		_compositeEingaben.setLayoutData(gridDataEingaben);
		_compositeEingaben.setLayout(gridLayout);

		/*
		 * Benutzer
		 */
		_labelBenutzer = new Label(_compositeEingaben, SWT.NONE);
		_labelBenutzer.setText("Benutzer:");		
		
		_textBenutzer = new Text(_compositeEingaben, SWT.BORDER);		
		_textBenutzer.setLayoutData(gridDataTextBenutzer);	
		_textBenutzer.addListener(SWT.DefaultSelection, new Listener()
		{
			public void handleEvent(Event e)
			{
				aktionOk();
			}
		});
		
	

		/*
		 * Passwort
		 */
		_labelPasswort = new Label(_compositeEingaben, SWT.NONE);
		_labelPasswort.setText("Passwort:");
		_textPasswort = new Text(_compositeEingaben, SWT.BORDER | SWT.PASSWORD);
		_textPasswort.setLayoutData(gridDataTextPasswort);

		/*
		 * Aktion hinter dem Passwort Feld
		 */
		_textPasswort.addListener(SWT.DefaultSelection, new Listener()
		{
			public void handleEvent(Event e)
			{
				aktionOk();
			}
		});
		
		/*
		 * Ursache
		 */
		_labelUrsache = new Label(_compositeEingaben, SWT.NONE);
		_labelUrsache.setText("Ursache:");
		_labelUrsache.setLayoutData(gridData1);
		_textFeldUrsache = new Text(_compositeEingaben, SWT.MULTI | SWT.WRAP
				| SWT.V_SCROLL);
		_textFeldUrsache.setLayoutData(gridDataTextUrsache);
		_textFeldUrsache.addListener(SWT.DefaultSelection, new Listener()
		{
			public void handleEvent(Event e)
			{
				aktionOk();
			}
		});

		/*
		 * Veranlasser
		 */
		_labelVeranlasser = new Label(_compositeEingaben, SWT.NONE);
		_labelVeranlasser.setText("Veranlasser:");
		_textVeranlasser = new Text(_compositeEingaben, SWT.BORDER);
		_textVeranlasser.setLayoutData(gridDataTextVeranlasser);
		_textVeranlasser.addListener(SWT.DefaultSelection, new Listener()
		{
			public void handleEvent(Event e)
			{
				aktionOk();
			}
		});

		/*
		 * Modify Listener zur Erfassung des Feldes Benutzer
		 */
		ModifyListener mlBenutzer = new ModifyListener()
		{
			@Override
			public void modifyText(ModifyEvent e)
			{
				_strBenutzer = _textBenutzer.getText();
			}
		};
		_textBenutzer.addModifyListener(mlBenutzer);

		/*
		 * Modify Listener zur Erfassung des Feldes Passwort
		 */
		ModifyListener mlPasswort = new ModifyListener()
		{
			@Override
			public void modifyText(ModifyEvent e)
			{
				_strPasswort = _textPasswort.getText();
			}
		};
		_textPasswort.addModifyListener(mlPasswort);

		/*
		 * Modify Listener zur Erfassung des Feldes Ursache
		 */
		ModifyListener mlUrsache = new ModifyListener()
		{
			@Override
			public void modifyText(ModifyEvent e)
			{
				_strUrsache = _textFeldUrsache.getText();
			}
		};
		_textFeldUrsache.addModifyListener(mlUrsache);

		/*
		 * Modify Listener zur Erfassung des Feldes Veranlasser
		 */
		ModifyListener mlVeranlasser = new ModifyListener()
		{
			@Override
			public void modifyText(ModifyEvent e)
			{
				_strVeranlasser = _textVeranlasser.getText();
			}
		};
		_textVeranlasser.addModifyListener(mlVeranlasser);
	
	}
	
	/**
	 * Erzeugt das Composite zum Erfassen der Eingaben bzgl. des Benutzers.
	 * 
	 */
	private void erzeugeCompositeEingaben(UrlasserInfo info)
	{
	  GridData gridData1 = new GridData();
	  gridData1.horizontalAlignment = GridData.BEGINNING;
	  gridData1.verticalAlignment = GridData.BEGINNING;
	  GridData gridDataTextUrsache = new GridData();
	  gridDataTextUrsache.horizontalAlignment = GridData.FILL;
	  gridDataTextUrsache.grabExcessHorizontalSpace = true;
	  gridDataTextUrsache.grabExcessVerticalSpace = true;
	  gridDataTextUrsache.verticalAlignment = GridData.FILL;
	  GridData gridDataTextVeranlasser = new GridData();
	  gridDataTextVeranlasser.horizontalAlignment = GridData.FILL;
	  gridDataTextVeranlasser.grabExcessHorizontalSpace = true;
	  gridDataTextVeranlasser.verticalAlignment = GridData.CENTER;
	  GridData gridDataTextPasswort = new GridData();
	  gridDataTextPasswort.horizontalAlignment = GridData.FILL;
	  gridDataTextPasswort.grabExcessHorizontalSpace = true;
	  gridDataTextPasswort.verticalAlignment = GridData.CENTER;
	  GridData gridDataTextBenutzer = new GridData();
	  gridDataTextBenutzer.horizontalAlignment = GridData.FILL;
	  gridDataTextBenutzer.grabExcessHorizontalSpace = true;
	  gridDataTextBenutzer.verticalAlignment = GridData.CENTER;
	  GridLayout gridLayout = new GridLayout();
	  gridLayout.numColumns = 2;
	  GridData gridDataEingaben = new GridData();
	  gridDataEingaben.horizontalAlignment = GridData.FILL;
	  gridDataEingaben.grabExcessHorizontalSpace = true;
	  gridDataEingaben.grabExcessVerticalSpace = true;
	  gridDataEingaben.verticalAlignment = GridData.FILL;
	  _compositeEingaben = new Composite(_shell, SWT.NONE);
	  _compositeEingaben.setLayoutData(gridDataEingaben);
	  _compositeEingaben.setLayout(gridLayout);
	  
	  /*
	   * Benutzer
	   */
	  _labelBenutzer = new Label(_compositeEingaben, SWT.NONE);
	  _labelBenutzer.setText("Benutzer:");		
	  
	  _textBenutzer = new Text(_compositeEingaben, SWT.BORDER);		
	  _textBenutzer.setLayoutData(gridDataTextBenutzer);	
	  _textBenutzer.setText(info.getBenutzer());	
	  _textBenutzer.addListener(SWT.DefaultSelection, new Listener()
	  {
	    public void handleEvent(Event e)
	    {
	      aktionOk();
	    }
	  });
	  
	  
	  
	  /*
	   * Passwort
	   */
	  _labelPasswort = new Label(_compositeEingaben, SWT.NONE);
	  _labelPasswort.setText("Passwort:");
	  _textPasswort = new Text(_compositeEingaben, SWT.BORDER | SWT.PASSWORD);
	  _textPasswort.setLayoutData(gridDataTextPasswort);
	 
//	  _textPasswort.setText(info.getPasswort());
//	  _textPasswort.setFocus();
	  /*
	   * Aktion hinter dem Passwort Feld
	   */
	  _textPasswort.addListener(SWT.DefaultSelection, new Listener()
	  {
	    public void handleEvent(Event e)
	    {
	      aktionOk();
	    }
	  });
	  
	  if (!_textBenutzer.isDisposed() && !_textPasswort.isDisposed() && _textBenutzer.getText().length() > 0)
      _textPasswort.setFocus();
	  
	  /*
	   * Ursache
	   */
	  _labelUrsache = new Label(_compositeEingaben, SWT.NONE);
	  _labelUrsache.setText("Ursache:");
	  _labelUrsache.setLayoutData(gridData1);
	  _textFeldUrsache = new Text(_compositeEingaben, SWT.MULTI | SWT.WRAP
	      | SWT.V_SCROLL);
	  _textFeldUrsache.setLayoutData(gridDataTextUrsache);
//	  _textFeldUrsache.setText(info.getUrsache());
	  _textFeldUrsache.addListener(SWT.DefaultSelection, new Listener()
	  {
	    public void handleEvent(Event e)
	    {
	      aktionOk();
	    }
	  });
	  
	  /*
	   * Veranlasser
	   */
	  _labelVeranlasser = new Label(_compositeEingaben, SWT.NONE);
	  _labelVeranlasser.setText("Veranlasser:");
	  _textVeranlasser = new Text(_compositeEingaben, SWT.BORDER);
	  _textVeranlasser.setLayoutData(gridDataTextVeranlasser);
//	  _textVeranlasser.setText(info.getVeranlasser());
	  _textVeranlasser.addListener(SWT.DefaultSelection, new Listener()
	  {
	    public void handleEvent(Event e)
	    {
	      aktionOk();
	    }
	  });
	  
	  /*
	   * Modify Listener zur Erfassung des Feldes Benutzer
	   */
	  ModifyListener mlBenutzer = new ModifyListener()
	  {
	    @Override
	    public void modifyText(ModifyEvent e)
	    {
	      _strBenutzer = _textBenutzer.getText();
	    }
	  };
	  _textBenutzer.addModifyListener(mlBenutzer);
	  
	  /*
	   * Modify Listener zur Erfassung des Feldes Passwort
	   */
	  ModifyListener mlPasswort = new ModifyListener()
	  {
	    @Override
	    public void modifyText(ModifyEvent e)
	    {
	      _strPasswort = _textPasswort.getText();
	    }
	  };
	  _textPasswort.addModifyListener(mlPasswort);
	  
	  /*
	   * Modify Listener zur Erfassung des Feldes Ursache
	   */
	  ModifyListener mlUrsache = new ModifyListener()
	  {
	    @Override
	    public void modifyText(ModifyEvent e)
	    {
	      _strUrsache = _textFeldUrsache.getText();
	    }
	  };
	  _textFeldUrsache.addModifyListener(mlUrsache);
	  
	  /*
	   * Modify Listener zur Erfassung des Feldes Veranlasser
	   */
	  ModifyListener mlVeranlasser = new ModifyListener()
	  {
	    @Override
	    public void modifyText(ModifyEvent e)
	    {
	      _strVeranlasser = _textVeranlasser.getText();
	    }
	  };
	  _textVeranlasser.addModifyListener(mlVeranlasser);
	  
	}

	/**
	 * Erzeugt ein {@link UrlasserObjekt} und initialisiert es mit den Daten
	 * dieses Dialogs.
	 * 
	 * @return {@link UrlasserObjekt}
	 */
	protected UrlasserObjekt erzeugeUrlasserObjekt()
	{
		if (_urlasserObjekt == null)
		{
			_urlasserObjekt = new UrlasserObjekt(this);
		}

		return _urlasserObjekt;
	}
	
	private void enableControls(boolean enabled) {
    _textBenutzer.setEnabled(enabled);
    _textPasswort.setEnabled(enabled);
    _textVeranlasser.setEnabled(enabled);
    _textFeldUrsache.setEnabled(enabled);
    _schaltflaecheOk.setEnabled(enabled);
  }

	/**
	 * Aktionen hinter der Schaltflche OK
	 */
	private void aktionOk()
	{
		erzeugeUrlasserObjekt();

		/*
		 * UrlasserDialog mit frischen Eintrgen aus diesem Dialog initialisieren!
		 */
		if (_urlasserObjekt != null)
		{
			_urlasserObjekt.reInit(this);
		}

		if (_urlasserObjekt.baueDaVVerbindungAuf())
		{
			LinkedList<ResultData> sendeDaten = _urlasserObjekt
					.sendeDaten(_resultDatas);

			if (sendeDaten.size() > 0)
			{
			  if (!_shell.isDisposed()) {
          if ((flags & DISABLE_CONTROLS_ON_OK) == DISABLE_CONTROLS_ON_OK) {
            enableControls(true);  
          }

        }
        
        String meldung = getClass().getName() + "::aktionOk " + "Von "
        + _resultDatas.size() + " Datenstzen konnten " + sendeDaten.size()
        + " Datenstze nicht versendet werden (Siehe Error Log)";

        generiereMeldungFehler("Datenstze nicht versendbar", meldung);

				Iterator<ResultData> it = sendeDaten.iterator();
				while (it.hasNext())
				{
					ResultData resultData = it.next();

					/*
					 * Zurckzugebende Fehlermeldung erzeugen
					 */
					_fehlerMeldung = getClass().getName()
							+ " - Datensatz konnte nicht versendet werden => "
							+ resultData.toString();
				}

				_eingabeOk = false;
			}
			else
			{
			  _eingabeOk = !_cancelled;
        
        if (!_shell.isDisposed()) {
        
          UrlasserInfo info = new UrlasserInfo(getBenutzer(), getPasswort(), getUrsache(), getVeranlasser());       
//        System.out.println("Speichern: " + info);               
          UrlasserFactory.getInstanz().setInfo(info);
          _shell.dispose();  
        }
			}
		}
		else
		{
			/*
			 * Zurckzugebende Fehlermeldung erzeugen
			 */
			_fehlerMeldung = getClass().getName()
					+ "::aktionOK Das UrlasserObjekt konnte keine DaV-Verbindung "
					+ "aufbauen! Daten wurden nicht gesendet!";
		}

	}

	/**
	 * Aktionen hinter der Schaltflche Abbrechen
	 */
	private void aktionAbbrechen()
	{
	  if ((flags & INPUT_OK_ON_CANCEL) == INPUT_OK_ON_CANCEL) {
      _cancelled = true;
      _eingabeOk = true;
    }
    else {
      _eingabeOk = false;  
    }

		_shell.dispose();
	}

	/**
	 * Aktionen hinter der Schaltflche Hilfe
	 */
	private void aktionHilfe()
	{
		String helpContextId = Activator.PLUGIN_ID + ".urlasserdialog";
		PlatformUI.getWorkbench().getHelpSystem().displayHelp(helpContextId);
	}

	/**
	 * Erzeugt das Composite fr die Steurungs-Buttons.
	 * 
	 */
	private void erzeugeCompositeKontrollen()
	{
		GridData gridDataSchaltflaecheHilfe = new GridData();
		gridDataSchaltflaecheHilfe.horizontalAlignment = GridData.FILL;
		gridDataSchaltflaecheHilfe.grabExcessHorizontalSpace = true;
		gridDataSchaltflaecheHilfe.horizontalIndent = 4;
		gridDataSchaltflaecheHilfe.verticalAlignment = GridData.CENTER;
		GridData gridDataSchaltflaecheAbbrechen = new GridData();
		gridDataSchaltflaecheAbbrechen.horizontalAlignment = GridData.FILL;
		gridDataSchaltflaecheAbbrechen.grabExcessHorizontalSpace = true;
		gridDataSchaltflaecheAbbrechen.horizontalIndent = 4;
		gridDataSchaltflaecheAbbrechen.verticalAlignment = GridData.CENTER;
		GridData gridDataSchaltflaecheOk = new GridData();
		gridDataSchaltflaecheOk.horizontalAlignment = GridData.FILL;
		gridDataSchaltflaecheOk.grabExcessHorizontalSpace = true;
		gridDataSchaltflaecheOk.horizontalIndent = 4;
		gridDataSchaltflaecheOk.verticalAlignment = GridData.CENTER;
		GridLayout gridLayoutKontrollen = new GridLayout();
		gridLayoutKontrollen.numColumns = 3;
		gridLayoutKontrollen.makeColumnsEqualWidth = true;
		GridData gridDataKontrollen = new GridData();
		gridDataKontrollen.horizontalAlignment = GridData.FILL;
		gridDataKontrollen.grabExcessHorizontalSpace = true;
		gridDataKontrollen.verticalAlignment = GridData.FILL;
		_compositeKontrollen = new Composite(_shell, SWT.NONE);
		_compositeKontrollen.setLayoutData(gridDataKontrollen);
		_compositeKontrollen.setLayout(gridLayoutKontrollen);
		_schaltflaecheOk = new Button(_compositeKontrollen, SWT.NONE);
		_schaltflaecheOk.setText("OK");
		_schaltflaecheOk.setLayoutData(gridDataSchaltflaecheOk);
		_schaltflaecheAbbrechen = new Button(_compositeKontrollen, SWT.NONE);
		_schaltflaecheAbbrechen.setText("Abbrechen");
		_schaltflaecheAbbrechen.setLayoutData(gridDataSchaltflaecheAbbrechen);
		_schaltflaecheHilfe = new Button(_compositeKontrollen, SWT.NONE);
		_schaltflaecheHilfe.setText("Hilfe");
		_schaltflaecheHilfe.setLayoutData(gridDataSchaltflaecheHilfe);

		_schaltflaecheOk.addMouseListener(new MouseAdapter()
		{

			public void mouseDown(MouseEvent e)
			{
				if (e.button == 1)
				{
					aktionOk();
				}
//				else if (e.button == 3)
//				{
//					_textBenutzer.setText("Tester");
//					_textPasswort.setText("geheim");
//					_textFeldUrsache.setText("Eine Ursache halt...");
//					_textVeranlasser.setText("Der Veranlasser...");
//					aktionOk();
//				}
			}
		});

		// _schaltflaecheOk.addSelectionListener(new SelectionAdapter()
		// {
		// public void widgetSelected(SelectionEvent e)
		// {
		// aktionOk();
		// }
		// });

		_schaltflaecheAbbrechen.addMouseListener(new MouseAdapter()
		{
			public void mouseDown(MouseEvent e)
			{
				aktionAbbrechen();
			}
		});

		_schaltflaecheAbbrechen.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				aktionAbbrechen();
			}
		});

		_schaltflaecheHilfe.addMouseListener(new MouseAdapter()
		{
			public void mouseDown(MouseEvent e)
			{
				aktionHilfe();
			}
		});

		_schaltflaecheHilfe.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				aktionHilfe();
			}
		});
	}

	/**
	 * Erzeugt das Composite zur Darstellung von Statusinformationen.
	 * 
	 */
  private void erzeugeCompositeStatus()
  {
    _compositeStatus = new Composite(_shell, SWT.NONE);
//    _labelStatus = new Label(_compositeStatus,  SWT.NONE);
    
//    RowLayout layout = new RowLayout();
//    layout.fill = false;
//    layout.justify = true;
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.FILL;

    GridLayout layout = new GridLayout(1, false);
    layout.marginHeight = 0;
    layout.marginWidth = 0;
    _compositeStatus.setLayoutData(gridData);
    _compositeStatus.setLayout(layout);
    
    _labelStatus = new Label(_compositeStatus,  SWT.CENTER);
    
    GridData labelData = new GridData();
    labelData.horizontalAlignment = GridData.FILL;
    labelData.grabExcessHorizontalSpace = true;
    labelData.verticalAlignment = GridData.CENTER;
    _labelStatus.setLayoutData(labelData);

  }
  
  public Label getLabelStatus() {
    return _labelStatus;
  }
  
  public boolean isCancelled() {
    return _cancelled;
  }

	/**
	 * Liefert den Benutzer.
	 * 
	 * @return Benutzer
	 */
	protected String getBenutzer()
	{
		return _textBenutzer.getText();
	}

	/**
	 * Liefert das Passwort.
	 * 
	 * @return Passwort
	 */
	protected String getPasswort()
	{
		return _textPasswort.getText();
	}

	/**
	 * Liefert die Ursache.
	 * 
	 * @return Ursache
	 */
	protected String getUrsache()
	{
		return _textFeldUrsache.getText();
	}

	/**
	 * Liefert den Veranlasser.
	 * 
	 * @return Veranlasser
	 */
	protected String getVeranlasser()
	{
		return _textVeranlasser.getText();
	}

	/**
	 * Generiert eine grafische Fehlermeldung und zeigt diese an.
	 * 
	 * @param titel
	 *          Titel der Fehlermeldung
	 * @param meldung
	 *          Mitteilung der Fehlermeldung
	 */
	private void generiereMeldungFehler(String titel, String meldung)
	{
		MessageBox mb = new MessageBox(PlatformUI.getWorkbench().getDisplay()
				.getActiveShell(), SWT.OK | SWT.ICON_ERROR);
		mb.setText(titel);
		mb.setMessage(meldung);
		mb.open();
	}

	/**
	 * Diese Klasse wird von der anzeigen-Methode der Klasse
	 * {@link UrlasserDialog} zurckgeliefert und enthlt Informationen zu den
	 * Eingaben, die der Anwender im {@link UrlasserDialog} gettigt hat. Dazu
	 * zhlen:
	 * <ul>
	 * <li>Wurde OK oder Abbrechen angeklickt?</li>
	 * <li>Der eingegebene Benutzername.</li>
	 * <li>Das eingegebene Passwort.</li>
	 * <li>Die eingegebene Ursache.</li>
	 * <li>Der eingegebene Veranlasser.</li>
	 * </ul>
	 * 
	 * @author HofmannJ
	 * 
	 */
	public class UrlasserDialogInfo
	{
		/**
		 * Flag fr ok oder abbrechen
		 */
		private boolean _eingabeOk = false;

		/**
		 * Angegebener Benutzername
		 */
		private String _benutzer = null;

		/**
		 * Eingegbenes Passwort
		 */
		private String _passwort = null;

		/**
		 * Angegebene Ursache
		 */
		private String _ursache = null;

		/**
		 * Angegebener Veranlasser
		 */
		private String _veranlasser = null;

		/**
		 * Speichert Informationen, warum eine Datensatz nicht versendet werden
		 * konnt.
		 */
		private String _fehlerInfo = null;

		/**
		 * Konstruktor. bernimmt die nachfolgenden Parameter:
		 * 
		 * @param eingabeOk
		 *          Wurde OK oder Abbrechen geklickt?
		 * @param benutzer
		 *          Benutzername
		 * @param passwort
		 *          Passwort des Benutzers
		 * @param ursache
		 *          Ursache
		 * @param veranlasser
		 *          Veranlasser
		 * @param fehlerInfo
		 *          Fehlerinformationen.
		 */
		public UrlasserDialogInfo(boolean eingabeOk, String benutzer,
				String passwort, String ursache, String veranlasser, String fehlerInfo)
		{
			this._eingabeOk = eingabeOk;
			this._benutzer = benutzer;
			this._passwort = passwort;
			this._ursache = ursache;
			this._veranlasser = veranlasser;
			this._fehlerInfo = fehlerInfo;
		}

		/**
		 * Prft, ob der Dialog mit OK oder Abbrechen beendet wurde.
		 * 
		 * @return Liefert <code>true</code>, wenn der Dialog mit OK beendet
		 *         wurde und <code>false</code>, wenn der Dialog mit Abbrechen
		 *         beendet wurde.
		 */
		public boolean getEingabeOk()
		{
			return _eingabeOk;
		}

		/**
		 * Liefert den Benutzernamen, der im Dialog eingegeben wurde.
		 * 
		 * @return Benutzername
		 */
		public String getBenutzeer()
		{
			return _benutzer;
		}

		/**
		 * Liefert das Passwort, welches im Dialog eingegeben wurde.
		 * 
		 * @return Passwort
		 */
		public String getPasswort()
		{
			return _passwort;
		}

		/**
		 * Liefert die Ursache, die im Dialog eingegben wurde.
		 * 
		 * @return Ursache
		 */
		public String getUrsache()
		{
			return _ursache;
		}

		/**
		 * Liefert den Veranlasser, der im Dialog eingegeben wurde.
		 * 
		 * @return Veranlasser
		 */
		public String getVeranlasser()
		{
			return _veranlasser;
		}

		/**
		 * Liefert Informationen, warum ein Datensatz nicht versendet werden konnte.
		 * 
		 * @return Fehlerinformationen
		 */
		public String getFehlerInfo()
		{
			return _fehlerInfo;
		}

	}
	
	

}
