/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.clientueberw;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map.Entry;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.DynamicObject;
import de.bsvrz.dav.daf.main.config.DynamicObjectType;
import de.bsvrz.dav.daf.main.config.InvalidationListener;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.DynamicObjectType.DynamicObjectCreatedListener;

/**
 * Liefert eine bersicht aller registrierten Applikationen des jeweiligen
 * Datenverteilers. <br>
 * <br>
 * <b>Lesen:</b><br>
 * <ul>
 * <li>TYP: <code>typ.applikation</code></li>
 * </ul>
 * 
 * @author HofmannJ
 * 
 */
public class DaVClientUeberwachung
{
	private static DaVClientUeberwachung _instance = null;

	private ClientDavInterface _connection = null;

	/**
	 * Typ der dynamischen Objekte
	 */
	private DynamicObjectType _dot = null;

	private InvalidationListener _invalidationListener;

	private DynamicObjectCreatedListener _dynamicObjectCreatedListener;

	private HashMap<Long, ApplikationKapsel> _map = new HashMap<Long, ApplikationKapsel>();

	private PropertyChangeSupport _pcs = new PropertyChangeSupport(this);

	/**
	 * INIT-Event der gesendet wird, wenn nach Anmeldung eines Listeners eine neue
	 * Applikation hinzugefgt wird. Der New-Value ist ein Array vom Typ
	 * {@link ApplikationKapsel}.
	 */
	public static final String PROP_INIT = "PROP_INIT";

	/**
	 * Event der gesendet wird, wenn eine Applikation hinzukommt. Der New-Value
	 * ist ein Objekt vom Typ {@link ApplikationKapsel}.
	 */
	public static final String PROP_ADDED = "PROP_ADDED";

	/**
	 * Event der gesendet wird, wenn ein Applikation wegfllt. Der Old-Value
	 * entspricht einem Objekt vom Typ {@link ApplikationKapsel} (der entfernten
	 * Applikation), der New-Value ist <code>null</code>.
	 */
	public static final String PROP_REMOVED = "PROP_REMOVED";

	private DaVClientUeberwachung()
	{
		getDaVConnection();
		init();
	}

	public static DaVClientUeberwachung getInstance()
	{
		if (_instance == null)
		{
			synchronized (DaVClientUeberwachung.class)
			{
				if (_instance == null)
				{
					_instance = new DaVClientUeberwachung();
				}
			}
		}

		return _instance;
	}

	private void init()
	{
		if (_connection != null)
		{
			_invalidationListener = new InvalidationListener()
			{
				@Override
				public void invalidObject(DynamicObject dynamicObject)
				{
					if (dynamicObject != null)
					{
						// removeApplikation(dynamicObject.getId());
						removeApplikation(new ApplikationKapsel(dynamicObject));
					}
				}
			};
			_dynamicObjectCreatedListener = new DynamicObjectCreatedListener()
			{
				@Override
				public void objectCreated(DynamicObject createdObject)
				{
					createdObject.addListenerForInvalidation(_invalidationListener);
					ApplikationKapsel app = new ApplikationKapsel(createdObject);
					putApplikation(app);
				}
			};

			DataModel dataModel = _connection.getDataModel();
			_dot = (DynamicObjectType) dataModel.getType("typ.applikation");
			_dot.addObjectCreationListener(_dynamicObjectCreatedListener);

			List<SystemObject> objects = _dot.getObjects();
			Iterator<SystemObject> it = objects.iterator();
			while (it.hasNext())
			{

				DynamicObject dynObj = (DynamicObject) it.next();
				if (dynObj.isValid())
				{
					dynObj.addListenerForInvalidation(_invalidationListener);
					ApplikationKapsel app = new ApplikationKapsel(dynObj);
					putApplikation(app);
				}
			}

		}
	}

	public void putApplikation(ApplikationKapsel app)
	{

		if ((app != null) && (app.getId() > -1))
		{
			ApplikationKapsel oldValue = null;
			ApplikationKapsel newValue = app;
			synchronized (_map)
			{
				oldValue = _map.get(app.getId());
				_map.put(app.getId(), app);
			}

			_pcs.firePropertyChange(PROP_ADDED, oldValue, newValue);
		}
	}

	public ApplikationKapsel removeApplikation(ApplikationKapsel app)
	{
		ApplikationKapsel removed = null;

		if ((app != null) && (app.getId() > -1))
		{
			removed = removeApplikation(app.getId());
		}

		if (removed != null)
		{
			_pcs.firePropertyChange(PROP_REMOVED, removed, null);
		}

		return removed;
	}

	public ApplikationKapsel removeApplikation(long id)
	{
		ApplikationKapsel removed = null;

		synchronized (_map)
		{
			removed = _map.remove(id);
		}

		return removed;
	}

	public ApplikationKapsel getApplikation(String pid)
	{
		ApplikationKapsel app = null;

		synchronized (_map)
		{
			app = _map.get(pid);
		}

		return app;
	}

	public ApplikationKapsel[] getApplikationen()
	{
		LinkedList<ApplikationKapsel> lApps = new LinkedList<ApplikationKapsel>();

		synchronized (_map)
		{
			Iterator<Entry<Long, ApplikationKapsel>> it = _map.entrySet().iterator();

			while (it.hasNext())
			{
				Entry<Long, ApplikationKapsel> entry = it.next();
				if (entry.getValue() != null)
				{
					lApps.add(entry.getValue());
				}
			}
		}

		return lApps.toArray(new ApplikationKapsel[lApps.size()]);
	}

	public boolean existiertApplikationClientId(String ClientId)
	{
		boolean re = false;

		if (ClientId != null)
		{
			ApplikationKapsel[] aApplikationen = getApplikationen();
			for (ApplikationKapsel applikationKapsel : aApplikationen)
			{
				if (applikationKapsel.getClientId() != null)
				{
					if (applikationKapsel.getClientId().equals(ClientId))
					{
						re = true;
						break;
					}
				}
			}
		}

		return re;
	}

	public class ApplikationKapsel
	{
		public DynamicObject _dynamicObject = null;

		public ApplikationKapsel(DynamicObject dynamicObject)
		{
			this._dynamicObject = dynamicObject;
		}

		public DynamicObject getDynamicObject()
		{
			return _dynamicObject;
		}

		public long getId()
		{
			long id = -1;

			if (getDynamicObject() != null)
			{
				return getDynamicObject().getId();
			}

			return id;
		}

		public String getClientId()
		{
			String re = null;

			if (getDynamicObject() != null)
			{
				re = getDynamicObject().getName();
			}

			return re;
		}

		@Override
		public String toString()
		{
			StringBuffer sb = new StringBuffer();

			if (getDynamicObject() != null)
			{
				sb.append(getDynamicObject().getNameOrPidOrId());
			} else
			{
				sb.append("-");
			}

			return sb.toString();
		}

	}

	/**
	 * Listener anmelden, der ber die am DaV angemeldete Applikationen informiert
	 * wird.
	 * 
	 * @param pcl
	 */
	public void addListener(PropertyChangeListener pcl)
	{
		if (pcl != null)
		{
			_pcs.addPropertyChangeListener(pcl);

			ApplikationKapsel[] aApps = getApplikationen();
			pcl.propertyChange(new PropertyChangeEvent(this, PROP_INIT, null, aApps));
		} else
		{
			System.err.println(this.getClass().getName()
					+ "::addListener => Angegebener Listener ist null!");
		}
	}

	/**
	 * Listener abmelden, der dann nich mehr ber die am DaV angemeldeten
	 * Applikationen informiert wird.
	 * 
	 * @param pcl
	 */
	public void removeListener(PropertyChangeListener pcl)
	{
		_pcs.removePropertyChangeListener(pcl);
	}

	private ClientDavInterface getDaVConnection()
	{
		if (_connection == null)
		{
			_connection = DaVVerbindung.getInstanz().getDaVVerbindung();
		}

		return _connection;
	}

	public static void main(String[] args)
	{
		new DaVClientUeberwachung();
	}

	@Override
	public String toString()
	{
		StringBuffer sb = new StringBuffer();

		ApplikationKapsel[] aApps = getApplikationen();

		sb.append("Anzahl Applikationen: ");
		sb.append(aApps.length);
		sb.append("\n");

		int idx = 0;
		for (ApplikationKapsel applikationKapsel : aApps)
		{
			sb.append("Applikation #");
			sb.append(idx);
			sb.append(": ");
			sb.append(applikationKapsel.toString());
			sb.append("\n");

			idx++;
		}

		return sb.toString();
	}

}
