/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dav;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.HashMap;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;

import de.sts.buv.plugin.workflow.tsf.Activator;
import de.sts.buv.plugin.workflow.tsf.PreferenceInit;
import de.sts.buv.plugin.workflow.tsf.dlg.DialogWFAbfrage;
import de.sts.buv.plugin.workflow.tsf.dlg.DialogWFEinfach;
import de.sts.buv.plugin.workflow.tsf.dlg.ViewWFEinfach;

/**
 * Hngt sich auf {@link GUIModelTSFWorkflow} und erzeugt bzw. beschickt die
 * entsprechenden Dialoge mit den Informationen aus den Workflows.
 * 
 * @author HofmannJ
 * 
 */
public class GUIManager_WorkflowDialog implements PropertyChangeListener
{
	/**
	 * Flag zur Anzeige, dass die Darstellung als VIEW und nicht als Dialog
	 * erfolgen soll.
	 */
	private static final boolean USE_VIEWS = true;

	private GUIModelTSFWorkflow _guiModelTSFWorkflow = null;

	/**
	 * Key = (TSFAbschnittReferenz, Referenz auf ein Objekt des Typs
	 * <code>typ.tsfAbschnitt</code>), Value = {@link DialogWFEinfach}.
	 */
	private HashMap<String, DialogWFEinfach> _mapDlgEinfach = new HashMap<String, DialogWFEinfach>();

	public GUIManager_WorkflowDialog(GUIModelTSFWorkflow guiModelTSFWorkflow)
	{
		if (guiModelTSFWorkflow != null)
		{
			this._guiModelTSFWorkflow = guiModelTSFWorkflow;
			_guiModelTSFWorkflow.addListener(this);
		}
	}

	@Override
	public void propertyChange(PropertyChangeEvent evt)
	{
		if ((evt != null) && (evt.getPropertyName() != null))
		{
			String propName = evt.getPropertyName();

			if (propName.equals(GUIModelTSFWorkflow.PROP_INIT))
			{
				// Initialisierung
				if (evt.getNewValue() != null)
				{
					if (evt.getNewValue() instanceof M_TSFWorkflow[])
					{
						M_TSFWorkflow[] aWF = (M_TSFWorkflow[]) evt.getNewValue();

						for (M_TSFWorkflow mTSFWorkflow : aWF)
						{
							genFillDlgWF(mTSFWorkflow);
						}
					}
				}

			} else if (propName.equals(GUIModelTSFWorkflow.PROP_ADDED))
			{
				// ADDED
				M_TSFWorkflow wfOld = null;
				M_TSFWorkflow wfNew = null;

				if ((evt.getOldValue() != null)
						&& (evt.getOldValue() instanceof M_TSFWorkflow))
				{
					wfOld = (M_TSFWorkflow) evt.getOldValue();
				}

				if ((evt.getNewValue() != null)
						&& (evt.getNewValue() instanceof M_TSFWorkflow))
				{
					wfNew = (M_TSFWorkflow) evt.getNewValue();
				}

				if (wfNew != null)
				{
					genFillDlgWF(wfNew);
				}

			} else if (propName.equals(GUIModelTSFWorkflow.PROP_REMOVED))
			{
	       System.out.println("GUIManager_WorkflowDialog.propertyChange(REMOVE)");
				// REMOVED
				M_TSFWorkflow wfRemoved = null;
				if ((evt.getOldValue() != null)
						&& (evt.getOldValue() instanceof M_TSFWorkflow))
				{
					wfRemoved = (M_TSFWorkflow) evt.getOldValue();
				}

				if ((wfRemoved != null) && (wfRemoved.getSo() != null))
				{
					M_TSFWorkflowAbschnitt[] aAbschnitte = wfRemoved.getAbschnitte();
					for (M_TSFWorkflowAbschnitt tsfWorkflowAbschnitt : aAbschnitte)
					{
						if ((tsfWorkflowAbschnitt != null)
								&& (tsfWorkflowAbschnitt.getAbschnitt() != null))
						{
							/*
							 * Dialoge Schlieen
							 */
							if (!USE_VIEWS)
							{
								DialogWFEinfach dlgEinfachRemoved = null;
								synchronized (_mapDlgEinfach)
								{
									dlgEinfachRemoved = _mapDlgEinfach
											.remove(tsfWorkflowAbschnitt.getAbschnitt().getPid());
								}

								if (dlgEinfachRemoved != null)
								{
									final DialogWFEinfach fDlgEinfachRemoved = dlgEinfachRemoved;

									Runnable run = new Runnable()
									{
										@Override
										public void run()
										{
											fDlgEinfachRemoved.dispose();
										}
									};
									PlatformUI.getWorkbench().getDisplay().asyncExec(run);
								}
							}

							/*
							 * Views schlieen
							 */
							if (USE_VIEWS)
							{
								// TODO JH
								final String pidAbschnitt = tsfWorkflowAbschnitt.getAbschnitt()
										.getPid();

								Runnable run = new Runnable()
								{

									@Override
									public void run()
									{
									  System.out.println("Schliesse0");
										ViewWFEinfach.schliesseViewWFEinfach(pidAbschnitt);
									}
								};
								PlatformUI.getWorkbench().getDisplay().asyncExec(run);
							}
						}
					}

				}
			}
		}
	}

	/**
	 * Zeigt den zum Workflow passenden Dialog an.
	 * 
	 * @param workflow
	 */
	public void genFillDlgWF(final M_TSFWorkflow workflow)
	{
		if ((workflow != null) && (workflow.getSo() != null))
		{

			M_TSFWorkflowAbschnitt[] aAbschnitte = workflow.getAbschnitte();
			for (M_TSFWorkflowAbschnitt tsfWorkflowAbschnitt : aAbschnitte)
			{
				final M_TSFWorkflowAbschnitt fTsfWorkflowAbschnitt = tsfWorkflowAbschnitt;
				 
				boolean istBestimmtesProgrammFuerAbschnittErreicht = Defs
						.istBestimmtesProgrammFuerAbschnittErreicht(tsfWorkflowAbschnitt);
				boolean istSteuerungFuerAbschnittInBearbeitung = Defs.pruefeObSteuerungFuerAbschnittInBearbeitung(tsfWorkflowAbschnitt);
				
				if (!istBestimmtesProgrammFuerAbschnittErreicht
						&& !istSteuerungFuerAbschnittInBearbeitung)
				{
					if ((tsfWorkflowAbschnitt != null)
							&& (tsfWorkflowAbschnitt.getAbschnitt() != null))
					{

						/*
						 * Dialog Einfach suchen und wenn nicht auffindbar erstellen
						 */
						DialogWFEinfach dialogWFEinfach = null;

						if (!USE_VIEWS)
						{
							synchronized (_mapDlgEinfach)
							{
								dialogWFEinfach = _mapDlgEinfach.get(tsfWorkflowAbschnitt
										.getAbschnitt().getPid());
							}
						}

						if (dialogWFEinfach != null)
						{
							/*
							 * Vorhandenen einfachen Dialog erzeugen und ffnen
							 */
							final DialogWFEinfach fDialogWFEinfach = dialogWFEinfach;
							Runnable run = new Runnable()
							{
								@Override
								public void run()
								{
									/*
									 * Lt. Rolf schickt dieser ein Abbrechen, wenn der Workflow
									 * mittels Ablehenen erfolgreich abgelehnt wurde und das
									 * Fenster geschlossen werden kann, falls offen.
									 */
									if (fTsfWorkflowAbschnitt.getAktion() != E_TSFBefehl.Abbrechen)
									{
										// Daten setzen
										fDialogWFEinfach.setData(fTsfWorkflowAbschnitt);

										// Fenster aktivieren
										fDialogWFEinfach.getParent().setActive();
									} else
									{
										/*
										 * Abbrechen empfangen als Besttigung der Ablehnen-Aktion.
										 * Fenster schlieen.
										 */
										removeDialogEinfach(fDialogWFEinfach);
										fDialogWFEinfach.dispose();
									}
								}
							};
							PlatformUI.getWorkbench().getDisplay().asyncExec(run);
						} else
						{
							/*
							 * Lt. Rolf schickt dieser ein Abbrechen, wenn der Workflow
							 * mittels Ablehenen erfolgreich abgelehnt wurde und das Fenster
							 * geschlossen werden kann und nicht mehr geffnet werden soll.
							 */
							if (!USE_VIEWS)
							{
								if (fTsfWorkflowAbschnitt.getAktion() != E_TSFBefehl.Abbrechen)
								{
									/*
									 * Neuen einfachen Dialog erzeugen und ffnen.
									 */
									Runnable run = new Runnable()
									{

										@Override
										public void run()
										{											
											Shell shell = new Shell(SWT.SHELL_TRIM);
											DialogWFEinfach rDialogWFEinfach = new DialogWFEinfach(
													shell, GUIManager_WorkflowDialog.this);

											synchronized (_mapDlgEinfach)
											{
												_mapDlgEinfach.put(fTsfWorkflowAbschnitt.getAbschnitt()
														.getPid(), rDialogWFEinfach);
											}

											rDialogWFEinfach.oeffne(fTsfWorkflowAbschnitt);
										}

									};
									PlatformUI.getWorkbench().getDisplay().asyncExec(run);
								}
							}
						}

						// TODO JH
						if (USE_VIEWS)
						{
							final String pidAbschnitt = fTsfWorkflowAbschnitt.getAbschnitt()
									.getPid();
							Runnable run = null;
							if (fTsfWorkflowAbschnitt.getAktion() != E_TSFBefehl.Abbrechen)
							{
								run = new Runnable()
								{
									@Override
									public void run()
									{									  
									  boolean bSchaltEmpfehlungInaktiv = Activator.getDefault().getPreferenceStore().getBoolean(PreferenceInit.PROP_SCHALTEMPFEHLUNG_INAKTIV);									 									  									  
									  if (!bSchaltEmpfehlungInaktiv){
									    ViewWFEinfach viewWFEinfach = ViewWFEinfach
									    .zeigeViewWFEinfach(pidAbschnitt);
									    if (viewWFEinfach != null)
									      viewWFEinfach.setData(fTsfWorkflowAbschnitt);
									    else{
									      System.out.println("Workflow->ViewEinfach ist null!" + pidAbschnitt);
									    }
									  }
									}
								};
							} else
							{
								run = new Runnable()
								{

									@Override
									public void run()
									{
									  System.out.println("---> Schliesse1");
										ViewWFEinfach.schliesseViewWFEinfach(pidAbschnitt);
									}
								};
							}

							if (run != null)
							{
								PlatformUI.getWorkbench().getDisplay().asyncExec(run);
							}
						}
					}
				} else
				{
					/*
					 * Dialog schlieen
					 */
					if (!USE_VIEWS)
					{
						DialogWFEinfach dialogWFEinfach = null;
						synchronized (_mapDlgEinfach)
						{
							dialogWFEinfach = _mapDlgEinfach.get(tsfWorkflowAbschnitt
									.getAbschnitt().getPid());
						}

						if (dialogWFEinfach != null)
						{
							// Disposen
							final DialogWFEinfach fDialogWFEinfach = dialogWFEinfach;
							Runnable run = new Runnable()
							{
								@Override
								public void run()
								{
									fDialogWFEinfach.dispose();
								}
							};
							PlatformUI.getWorkbench().getDisplay().asyncExec(run);

							// Von Map entfernen.
							removeDialogEinfach(dialogWFEinfach);
						}
					}

					if (USE_VIEWS)
					{
						// TODO JH
						/*
						 * Views schlieen
						 */
						final String pidAbschnitt = fTsfWorkflowAbschnitt.getAbschnitt()
								.getPid();
						Runnable run = new Runnable()
						{

							@Override
							public void run()
							{
							  System.out.println("---> Schliesse2");
								ViewWFEinfach.schliesseViewWFEinfach(pidAbschnitt);
							}
						};
						PlatformUI.getWorkbench().getDisplay().asyncExec(run);
					}
				}
			}

		}
	}

	public void removeDialog(DialogWFAbfrage dialog)
	{
		// if ((dialog != null) && (dialog.getData() != null)
		// && (dialog.getData().getSo() != null))
		// {
		// SystemObject so = dialog.getData().getSo();
		// String pid = so.getPid();
		// synchronized (_mapDlg)
		// {
		// _mapDlg.remove(pid);
		// }
		// }
	}

	public void removeDialogEinfach(DialogWFEinfach dialogEinfach)
	{
		if ((dialogEinfach != null) && (dialogEinfach.getData() != null)
				&& (dialogEinfach.getData().getAbschnitt() != null))
		{
			String pidAbschnitt = dialogEinfach.getData().getAbschnitt().getPid();
			synchronized (_mapDlgEinfach)
			{
				_mapDlgEinfach.remove(pidAbschnitt);
			}
		}
	}

	// protected DialogWFAbfrage[] getAllDialogs()
	// {
	// LinkedList<DialogWFAbfrage> lDlgs = new LinkedList<DialogWFAbfrage>();
	//
	// synchronized (_mapDlg)
	// {
	// Iterator<Entry<String, DialogWFAbfrage>> it = _mapDlg.entrySet()
	// .iterator();
	// while (it.hasNext())
	// {
	// Entry<String, DialogWFAbfrage> entry = it.next();
	// lDlgs.add(entry.getValue());
	// }
	// }
	//
	// return lDlgs.toArray(new DialogWFAbfrage[lDlgs.size()]);
	// }

}
