/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dav;

import java.util.LinkedList;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.SystemObject;

public class M_TSFAbschnittDiagnoseDaten
{
	/**
	 * SystemObjekt typ.tsfAbschnitt
	 */
	private SystemObject _so = null;

	/**
	 * Feld <code>ZeitbereichKeineFreigabe</code>, JaNein
	 */
	private Boolean _zeitbereichKeineFreigabe = null;

	/**
	 * Feld <code>PrioZeitbereichKeineFreigabe</code>
	 */
	private Long _prioZeitbereichKeineFreigabe = null;

	/**
	 * Feld <code>ZeitbereichFreigabe</code>, JaNein
	 */
	private Boolean _zeitbereichFreigabe = null;

	/**
	 * Feld <code>PrioZeitbereichFreigabe</code>.
	 */
	private Long _prioZeitbereichFreigabe = null;

	/**
	 * Feld <code>ZeitbereichFreigabeDurchAutomatik</code>, JaNein
	 */
	private Boolean _zeitbereichFreigabeDurchAutomatik = null;

	/**
	 * Feld <code>PrioZeitbereichFreigabeDurchAutomatik</code>
	 */
	private Long _prioZeitbereichFreigabeDurchAutomatik = null;

	/**
	 * Feld <code>TSFSollZustand</code>, Typ <code>att.tsfDiagnoseZustand</code>.
	 */
	private E_TSFDiagnoseZustand _tsfSollZustand = null;

	/**
	 * Feld <code>TSFIstZustand</code>, Typ <code>att.tsfDiagnoseZustand</code>.
	 */
	private E_TSFDiagnoseZustand _tsfIstZustand = null;

	/**
	 * Feld Messquerschnitte, variables Array fr Daten entsprechend
	 * <code>atl.tsfMQDiagnoseDaten</code>.
	 */
	private LinkedList<M_TSFMQDiagnoseDaten> _lDiagnoseDaten = new LinkedList<M_TSFMQDiagnoseDaten>();

	public M_TSFAbschnittDiagnoseDaten(ResultData resultData)
	{
		if ((resultData != null) && (resultData.hasData()))
		{
			setSo(resultData.getObject());

			Data data = resultData.getData();

			// ZeitbereichKeineFreigabe
			int iZeitbereichKeineFreigabe = data.getUnscaledValue(
					"ZeitbereichKeineFreigabe").intValue();
			if (iZeitbereichKeineFreigabe == 1)
			{
				setZeitbereichKeineFreigabe(new Boolean(true));
			} else
			{
				setZeitbereichKeineFreigabe(new Boolean(false));
			}

			// PrioZeitbereichKeineFreigabe
			long lPrioZeitbereichKeineFreigabe = data.getUnscaledValue(
					"PrioZeitbereichKeineFreigabe").longValue();
			setPrioZeitbereichKeineFreigabe(lPrioZeitbereichKeineFreigabe);

			// ZeitbereichFreigabe
			int iZeitbereichFreigabe = data.getUnscaledValue("ZeitbereichFreigabe")
					.intValue();
			if (iZeitbereichFreigabe == 1)
			{
				setZeitbereichFreigabe(true);
			} else
			{
				setZeitbereichFreigabe(false);
			}

			// PrioZeitbereichFreigabe
			long lPrioZeitbereichFreigabe = data.getUnscaledValue(
					"PrioZeitbereichFreigabe").longValue();
			setPrioZeitbereichFreigabe(lPrioZeitbereichFreigabe);

			// ZeitbereichFreigabeDurchAutomatik
			int iZeitbereichFreigabeDurchAutomatik = data.getUnscaledValue(
					"ZeitbereichFreigabeDurchAutomatik").intValue();
			if (iZeitbereichFreigabeDurchAutomatik == 1)
			{
				setZeitbereichFreigabeDurchAutomatik(true);
			} else
			{
				setZeitbereichFreigabeDurchAutomatik(false);
			}

			// PrioZeitbereichFreigabeDurchAutomatik
			long lPrioZeitbereichFreigabeDurchAutomatik = data.getUnscaledValue(
					"PrioZeitbereichFreigabeDurchAutomatik").longValue();
			setPrioZeitbereichFreigabeDurchAutomatik(lPrioZeitbereichFreigabeDurchAutomatik);

			// TSFSollZustand
			int iTSFSollZustand = data.getUnscaledValue("TSFSollZustand").intValue();
			E_TSFDiagnoseZustand tsfSollZustand = E_TSFDiagnoseZustand
					.getBefehl(iTSFSollZustand);
			setTsfSollZustand(tsfSollZustand);

			// TSFIstZustand
			int iTSFIstZustand = data.getUnscaledValue("TSFIstZustand").intValue();
			E_TSFDiagnoseZustand tsfIstZustand = E_TSFDiagnoseZustand
					.getBefehl(iTSFIstZustand);
			setTsfIstZustand(tsfIstZustand);

			// Array MessQuerschnitte
			Array aMQ = data.getArray("MessQuerschnitte");
			for (int i = 0; i < aMQ.getLength(); i++)
			{
				Data dataMQ = aMQ.getItem(i);
				M_TSFMQDiagnoseDaten diagnoseDaten = new M_TSFMQDiagnoseDaten(dataMQ);
				addDiagnoseDaten(diagnoseDaten);
			}

		}
	}

	public SystemObject getSo()
	{
		return _so;
	}

	public void setSo(SystemObject so)
	{
		_so = so;
	}

	public Boolean getZeitbereichKeineFreigabe()
	{
		return _zeitbereichKeineFreigabe;
	}

	public void setZeitbereichKeineFreigabe(Boolean zeitbereichKeineFreigabe)
	{
		_zeitbereichKeineFreigabe = zeitbereichKeineFreigabe;
	}

	public Long getPrioZeitbereichKeineFreigabe()
	{
		return _prioZeitbereichKeineFreigabe;
	}

	public void setPrioZeitbereichKeineFreigabe(Long prioZeitbereichKeineFreigabe)
	{
		_prioZeitbereichKeineFreigabe = prioZeitbereichKeineFreigabe;
	}

	public Boolean getZeitbereichFreigabe()
	{
		return _zeitbereichFreigabe;
	}

	public void setZeitbereichFreigabe(Boolean zeitbereichFreigabe)
	{
		_zeitbereichFreigabe = zeitbereichFreigabe;
	}

	public Long getPrioZeitbereichFreigabe()
	{
		return _prioZeitbereichFreigabe;
	}

	public void setPrioZeitbereichFreigabe(Long prioZeitbereichFreigabe)
	{
		_prioZeitbereichFreigabe = prioZeitbereichFreigabe;
	}

	public Boolean getZeitbereichFreigabeDurchAutomatik()
	{
		return _zeitbereichFreigabeDurchAutomatik;
	}

	public void setZeitbereichFreigabeDurchAutomatik(
			Boolean zeitbereichFreigabeDurchAutomatik)
	{
		_zeitbereichFreigabeDurchAutomatik = zeitbereichFreigabeDurchAutomatik;
	}

	public Long getPrioZeitbereichFreigabeDurchAutomatik()
	{
		return _prioZeitbereichFreigabeDurchAutomatik;
	}

	public void setPrioZeitbereichFreigabeDurchAutomatik(
			Long prioZeitbereichFreigabeDurchAutomatik)
	{
		_prioZeitbereichFreigabeDurchAutomatik = prioZeitbereichFreigabeDurchAutomatik;
	}

	public E_TSFDiagnoseZustand getTsfSollZustand()
	{
		return _tsfSollZustand;
	}

	public void setTsfSollZustand(E_TSFDiagnoseZustand tsfSollZustand)
	{
		_tsfSollZustand = tsfSollZustand;
	}

	public E_TSFDiagnoseZustand getTsfIstZustand()
	{
		return _tsfIstZustand;
	}

	public void setTsfIstZustand(E_TSFDiagnoseZustand tsfIstZustand)
	{
		_tsfIstZustand = tsfIstZustand;
	}

	public void addDiagnoseDaten(M_TSFMQDiagnoseDaten diagnoseDaten)
	{
		synchronized (_lDiagnoseDaten)
		{
			_lDiagnoseDaten.add(diagnoseDaten);
		}
	}

	public boolean removeDiagnoseDaten(M_TSFMQDiagnoseDaten diagnoseDaten)
	{
		boolean entfernt = false;

		synchronized (_lDiagnoseDaten)
		{
			entfernt = _lDiagnoseDaten.remove(diagnoseDaten);
		}

		return entfernt;
	}

	public M_TSFMQDiagnoseDaten[] getDiagnoseDaten()
	{
		M_TSFMQDiagnoseDaten[] re = null;

		synchronized (_lDiagnoseDaten)
		{
			re = _lDiagnoseDaten.toArray(new M_TSFMQDiagnoseDaten[_lDiagnoseDaten
					.size()]);
		}

		return re;
	}

}
