/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dav;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Umsetzung von ATL <code>atl.tsfMQDiagnoseDaten</code>, Diagnosedaten pro
 * Messquerschnitt.
 * 
 * @author HofmannJ
 * 
 */
public class M_TSFMQDiagnoseDaten
{
	/**
	 * Feld <code></code>, Referenz auf ein Objekt vom Typ
	 * <code>typ.messQuerschnitt</code>.
	 */
	private SystemObject _messQuerschnitt = null;

	/**
	 * Feld <code>Empfehlung</code>.
	 */
	private E_TSFDiagnoseZustand _empfehlung = null;

	/**
	 * Feld <code>Parameter</code>, Parameter des Messquerschnitts.
	 */
	private M_TSFMQDiagnoseParameter _parameter = null;

	/**
	 * Feld <code>Werte</code>, Werte des Messquerschnitts.
	 */
	private M_TSFMQDiagnoseWerte _werte = null;

	public M_TSFMQDiagnoseDaten(Data data)
	{
		if (data != null)
		{
			// MessQuerschnitt - Referenz auf MQ
			SystemObject soMQ = data.getReferenceValue("MessQuerschnitt")
					.getSystemObject();
			setMessQuerschnitt(soMQ);

			// Empfehlung
			int iEmpfehlung = data.getUnscaledValue("Empfehlung").intValue();
			E_TSFDiagnoseZustand empfehlung = E_TSFDiagnoseZustand
					.getBefehl(iEmpfehlung);
			setEmpfehlung(empfehlung);

			Data dataParameter = data.getItem("Parameter");
			M_TSFMQDiagnoseParameter parameter = new M_TSFMQDiagnoseParameter(
					dataParameter);
			setParameter(parameter);

			Data dataWerte = data.getItem("Werte");
			M_TSFMQDiagnoseWerte werte = new M_TSFMQDiagnoseWerte(dataWerte);
			setWerte(werte);
		}
	}

	public SystemObject getMessQuerschnitt()
	{
		return _messQuerschnitt;
	}

	public void setMessQuerschnitt(SystemObject messQuerschnitt)
	{
		_messQuerschnitt = messQuerschnitt;
	}

	public E_TSFDiagnoseZustand getEmpfehlung()
	{
		return _empfehlung;
	}

	public void setEmpfehlung(E_TSFDiagnoseZustand empfehlung)
	{
		_empfehlung = empfehlung;
	}

	public M_TSFMQDiagnoseParameter getParameter()
	{
		return _parameter;
	}

	public void setParameter(M_TSFMQDiagnoseParameter parameter)
	{
		_parameter = parameter;
	}

	public M_TSFMQDiagnoseWerte getWerte()
	{
		return _werte;
	}

	public void setWerte(M_TSFMQDiagnoseWerte werte)
	{
		_werte = werte;
	}

}
