/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dav;

import de.bsvrz.dav.daf.main.Data;

/**
 * Umsetzung von <code>atl.tsfMQDiagnoseParameter</code>, Parameter des
 * Messquerschnitts.
 * 
 * @author HofmannJ
 * 
 */
public class M_TSFMQDiagnoseParameter
{
	/**
	 * Feld <code> QBEin</code>, Einschaltkriterium Prognose QB. Ganze Zahl
	 * Positiv.
	 */
	private Long _qbEin = null;

	/**
	 * Feld <code>VKfzEin</code>, att.geschwindigkeit. Ganze Zahl Positiv, 0-254
	 * km/h. Einschaltkriterium Prognose Geschwindigkeit.
	 */
	private Integer _vKfzEin = null;

	/**
	 * Feld <code>AnzahlMessZyklenEin</code>. Anzahl von Messzyklen ber die die
	 * Einschaltbedingung erfllt sein muss bevor eine Schaltempfehlung abgeleitet
	 * wird. Ganze Zahl.
	 */
	private Long _anzahlMessZyklenEin = null;

	/**
	 * Feld <code>AnzahlMessZyklenAus</code>. Anzahl von Messzyklen ber die die
	 * Ausschaltbedingung erfllt sein muss bevor eine Schaltempfehlung abgeleitet
	 * wird. Ganze Zahl.
	 */
	private Long _anzahlMessZyklenAus = null;

	public M_TSFMQDiagnoseParameter(Data data)
	{
		if (data != null)
		{
			// QBEin
			setQbEin(data.getUnscaledValue("QBEin").longValue());

			// VKfzEin
			setvKfzEin(data.getUnscaledValue("VKfzEin").intValue());

			// AnzahlMessZyklenEin
			setAnzahlMessZyklenEin(data.getUnscaledValue("AnzahlMessZyklenEin")
					.longValue());

			// AnzahlMessZyklenAus
			setAnzahlMessZyklenAus(data.getUnscaledValue("AnzahlMessZyklenAus")
					.longValue());
		}
	}

	public Long getQbEin()
	{
		return _qbEin;
	}

	public void setQbEin(Long qbEin)
	{
		_qbEin = qbEin;
	}

	public Integer getvKfzEin()
	{
		return _vKfzEin;
	}

	public void setvKfzEin(Integer vKfzEin)
	{
		_vKfzEin = vKfzEin;
	}

	public Long getAnzahlMessZyklenEin()
	{
		return _anzahlMessZyklenEin;
	}

	public void setAnzahlMessZyklenEin(Long anzahlMessZyklenEin)
	{
		_anzahlMessZyklenEin = anzahlMessZyklenEin;
	}

	public Long getAnzahlMessZyklenAus()
	{
		return _anzahlMessZyklenAus;
	}

	public void setAnzahlMessZyklenAus(Long anzahlMessZyklenAus)
	{
		_anzahlMessZyklenAus = anzahlMessZyklenAus;
	}

}
