/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dav;

import java.util.Date;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Umsetzung von <code>atg.tsfZustandArgoscan</code>.
 * 
 * @author HofmannJ
 * 
 */
public class M_TSFZustandArgoscan implements I_TSFZustand
{
	/**
	 * SystemObjekt typ.tsfAbschnitt
	 */
	private SystemObject _so;
	
	
	private Date _timestamp;

	/**
	 * Zustand, Definition mglicher Zustnde des Argoscan.
	 */
	private E_TSFArgoscanZustand _zustand;

	/**
	 * MakroEreignis, Makro/Ereignis das mit dem Zustand verbunden ist.
	 */
	private Long _makroEreignis;

	public M_TSFZustandArgoscan()
	{
	}

	public M_TSFZustandArgoscan(ResultData rd)
	{
		init(rd);
	}

	private void init(ResultData rd)
	{
		if ((rd != null) && (rd.getObject() != null) && (rd.hasData()))
		{
			setSo(rd.getObject());
			
			setTimestamp(new Date(rd.getDataTime()));			

			Data data = rd.getData();

			// Zustand
			setZustand(E_TSFArgoscanZustand.getZustand(data
					.getUnscaledValue("Zustand").intValue()));

			// MakroEreignis
			setMakroEreignis(new Long(data.getUnscaledValue("MakroEreignis")
					.longValue()));

		}
	}

	public SystemObject getSo()
	{
		return _so;
	}

	public void setSo(SystemObject so)
	{
		_so = so;
	}

	public E_TSFArgoscanZustand getZustand()
	{
		return _zustand;
	}

	public void setZustand(E_TSFArgoscanZustand zustand)
	{
		_zustand = zustand;
	}

	public Long getMakroEreignis()
	{
		return _makroEreignis;
	}

	public void setMakroEreignis(Long makroEreignis)
	{
		_makroEreignis = makroEreignis;
	}

	@Override
	public String toString()
	{
		StringBuffer sb = new StringBuffer();

		sb.append("TSFAbschnitt=");
		if (getSo() != null)
		{
			sb.append(getSo());
		} else
		{
			sb.append("-");
		}

		sb.append(", Zustand=");
		if (getZustand() != null)
		{
			sb.append(getZustand().toString());
		} else
		{
			sb.append("-");
		}

		sb.append(", MakroEreignis=");
		if (getMakroEreignis() != null)
		{
			sb.append(getMakroEreignis());
		} else
		{
			sb.append("-");
		}

		return sb.toString();
	}

	@Override
	public Date getTimestamp()
	{
		return _timestamp;
	}
	
	
	public void setTimestamp(Date timestamp)
	{
		this._timestamp = timestamp;
	}

}
