/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dlg;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

/**
 * Mit Hilfe dieser Klasse knnen sich Listener darber informieren lassen, dass
 * das Weiterschalten eines Bereichs abgebrochen wurde.
 * 
 * @author HofmannJ
 * 
 */
public class InfoWeiterschaltenAbgebrochen
{
	private static InfoWeiterschaltenAbgebrochen _instance = null;
	private PropertyChangeSupport _pcs = new PropertyChangeSupport(this);
	public static final String PROP_SCHALTEN_ABSCHNITT_ABGEBROCHEN = "SCHALTEN_ABSCHNITT_ABGEBROCHEN";

	private InfoWeiterschaltenAbgebrochen()
	{

	}

	public static InfoWeiterschaltenAbgebrochen getInstance()
	{
		if (_instance == null)
		{
			synchronized (InfoWeiterschaltenAbgebrochen.class)
			{
				if (_instance == null)
				{
					_instance = new InfoWeiterschaltenAbgebrochen();
				}
			}
		}

		return _instance;
	}

	public void addListener(PropertyChangeListener pcl)
	{
		_pcs.addPropertyChangeListener(pcl);
	}

	public void removeListener(PropertyChangeListener pcl)
	{
		_pcs.removePropertyChangeListener(pcl);
	}

	public void benachrichtigeUeberAbgebrochenesWeiterschnalten(
			String pidAbschnitt)
	{
		final AbgebrochenesWeiterschaltenAbschnitt aAbs = new AbgebrochenesWeiterschaltenAbschnitt(
				pidAbschnitt);

		_pcs.firePropertyChange(PROP_SCHALTEN_ABSCHNITT_ABGEBROCHEN, null, aAbs);
	}

	/**
	 * Wrapper-Klasse, liefert die Info fr welchen Abschnitt das Weiterschalten
	 * abgebrochen wurde.
	 * 
	 * @author HofmannJ
	 * 
	 */
	public class AbgebrochenesWeiterschaltenAbschnitt
	{
		private String _pidAbschnitt = null;

		/**
		 * Konstruktor.
		 * 
		 * @param pidAbschnitt
		 *          Pid des Abschnitts , fr den das Weiterschalten abgebrochen
		 *          wurde.
		 */
		public AbgebrochenesWeiterschaltenAbschnitt(String pidAbschnitt)
		{
			this._pidAbschnitt = pidAbschnitt;
		}

		/**
		 * Liefert die Pid des Abschnitts, fr den das Weiterschalten abgebrochen
		 * wurde.
		 * 
		 * @return
		 */
		public String getPidAbschnitt()
		{
			return _pidAbschnitt;
		}

		@Override
		public String toString()
		{
			StringBuffer sb = new StringBuffer();

			sb.append("Weiterschalten Abgebrochen fr Abschnitt ");
			sb.append(getPidAbschnitt());

			return sb.toString();
		}

	}
}
