/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dlg;

import java.util.HashMap;

import org.eclipse.swt.graphics.Point;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;

/**
 * Liefert die Position von Dialogen. Folgende Dialoge werden z.Zt. untersttzt:
 * <ul>
 * <li>Schalten</li>
 * <li>Weiterschalten</li>
 * <li>Schaltempfehlung</li>
 * </ul>
 * 
 * @author HofmannJ
 * 
 */
public class StaticDialogLocation
{
	/**
	 * Singleton-Instanz dieser Klasse.
	 */
	private static StaticDialogLocation _instance = null;

	/**
	 * Key = Dialog-Typ, Value=[Map Key = Pid Dialog, Value = Point location
	 * Dialog].<br>
	 * Als Key fr den Dialog-Typ kann der vollqualifizierte Klassenpfad verwendet
	 * werden.
	 */
	private HashMap<String, HashMap<String, Point>> _map = new HashMap<String, HashMap<String, Point>>();

	/**
	 * Privater Konstruktor, legt die Positionen initial fest.
	 */
	private StaticDialogLocation()
	{
//		/*
//		 * Positionen der Fenster von Schaltempfehlungen.
//		 */
//		HashMap<String, Point> mapWFSchaltEmpfehlung = new HashMap<String, Point>();
//		mapWFSchaltEmpfehlung.put(
//				"bw.uz.A8.sba_stgt.tsf.abschnitt1.W", new Point(10, 10));
//		mapWFSchaltEmpfehlung.put(
//				"bw.uz.A8.sba_stgt.tsf.abschnitt2.O", new Point(10, 370));
// 		synchronized (_map)
//		{
//			_map.put(DialogWFEinfach.class.getName(), mapWFSchaltEmpfehlung);
//		}
 		
 		
 		IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
 		Point dwLocation = dw.getShell().getLocation();

		/*
		 * Positionen der Fenster von Schaltempfehlungen VIEW
		 */
		HashMap<String, Point> mapViewWFSchaltEmpfehlung = new HashMap<String, Point>();
		mapViewWFSchaltEmpfehlung.put(
				"bw.uz.A8.sba_stgt.tsf.abschnitt1.W", new Point(dwLocation.x + 10, dwLocation.y + 70));
		mapViewWFSchaltEmpfehlung.put(
				"bw.uz.A8.sba_stgt.tsf.abschnitt2.O", new Point(dwLocation.x + 10, dwLocation.y + 500));
		synchronized (_map)
		{
			_map.put(ViewWFEinfach.class.getName(), mapViewWFSchaltEmpfehlung);
		}

//		/*
//		 * Weiterschalten
//		 */
//		HashMap<String, Point> mapWeiterschalten = new HashMap<String, Point>();
//		mapWeiterschalten.put(
////				"bw.uz.A8.sba_stgt.tsf.abschnitt1.W", new Point(10, 10));
//		    "bw.uz.A8.sba_stgt.tsf.abschnitt1.W", new Point(dwLocation.x + 10, dwLocation.y + 10));
//		mapWeiterschalten.put(
////				"bw.uz.A8.sba_stgt.tsf.abschnitt2.O", new Point(10, 370));	 
//		    "bw.uz.A8.sba_stgt.tsf.abschnitt2.O", new Point(dwLocation.x + 10, dwLocation.y + 370));	 
//		synchronized (_map)
//		{
//			_map.put(DialogWeiterschalten.class.getName(), mapWeiterschalten);
//		}

		/*
		 * Weiterschalten als View
		 */
		HashMap<String, Point> mapWeiterschaltenView = new HashMap<String, Point>();
		mapWeiterschaltenView.put(
//				"bw.uz.A8.sba_stgt.tsf.abschnitt1.W", new Point(10, 10));
		    "bw.uz.A8.sba_stgt.tsf.abschnitt1.W", new Point(dwLocation.x + 10, dwLocation.y + 70));
		mapWeiterschaltenView.put(
//				"bw.uz.A8.sba_stgt.tsf.abschnitt2.O", new Point(10, 370));
		    "bw.uz.A8.sba_stgt.tsf.abschnitt2.O", new Point(dwLocation.x + 10, dwLocation.y + 500));
		synchronized (_map)
		{
			_map.put(ViewWeiterschalten.class.getName(), mapWeiterschaltenView);
		}

		/*
		 * Dialog Schalten
		 */
		HashMap<String, Point> mapDlgSchalten = new HashMap<String, Point>();
		mapDlgSchalten.put("bw.uz.A8.sba_stgt.tsf.abschnitt1.W",
//				new Point(10, 10));
		    new Point(dwLocation.x + 10, dwLocation.y + 10));
		mapDlgSchalten.put("bw.uz.A8.sba_stgt.tsf.abschnitt2.O",
//				new Point(10, 370)); 
		    new Point(dwLocation.x + 10, dwLocation.y + 370)); 
		synchronized (_map)
		{
			_map.put(DialogSchalten.class.getName(), mapDlgSchalten);
		}
	}

	/**
	 * Liefert die Singleton-Instanz dieser Klasse.
	 * 
	 * @return Singleton-Instanz dieser Klasse.
	 */
	public static StaticDialogLocation getInstance()
	{
		if (_instance == null)
		{
			synchronized (StaticDialogLocation.class)
			{
				if (_instance == null)
				{
					_instance = new StaticDialogLocation();
				}
			}
		}

		return _instance;
	}

	/**
	 * Liefert die Location fr einen Dialog als {@link Point}.
	 * 
	 * @param dlgTyp
	 *          Typ des Dialogs.
	 * @param dlgPid
	 *          Pid des Dialogs.
	 * @return Liefert die Location des Dialoges als {@link Point} oder
	 *         <code>null</code>, falls nicht verfgbar.
	 */
	public Point getDlgLoc(String dlgTyp, String dlgPid)
	{
		Point re = null;

		if ((dlgTyp != null) && (dlgPid != null))
		{
			synchronized (_map)
			{
				HashMap<String, Point> mapForDlgTyp = _map.get(dlgTyp);
				if (mapForDlgTyp != null)
				{
					re = mapForDlgTyp.get(dlgPid);
				}
			}
		}

		return re;
	}

}
