/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dlg;

import org.eclipse.core.runtime.ILog;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;

import de.sts.buv.plugin.workflow.tsf.Activator;

public class ViewTools
{
	/**
	 * Durchsucht alle der Workbench bekannten ViewReferenzes mit View- und
	 * Secondary-Id.
	 * 
	 * @param viewId
	 *          ID der View.
	 * @param sndId
	 *          Secondary-ID der View (darf nicht <code>null</code> sein!).
	 * @return Liefert die {@link IViewReference} auf die gefundene View oder
	 *         <code>null</code>, falls eine solche View nicht existiert.
	 */
	public static IViewReference searchViewReference(String viewId, String sndId)
	{
		IViewReference foundRef = null;

		if ((viewId != null) && (sndId != null))
		{
			IWorkbenchWindow[] aWBW = PlatformUI.getWorkbench().getWorkbenchWindows();
			for (IWorkbenchWindow workbenchWindow : aWBW)
			{
				IWorkbenchPage[] aPages = workbenchWindow.getPages();
				for (IWorkbenchPage workbenchPage : aPages)
				{
					IViewReference[] aViewReferences = workbenchPage.getViewReferences();
					for (IViewReference viewReference : aViewReferences)
					{
						String pruefeViewId = viewReference.getId();
						String pruefeSecId = viewReference.getSecondaryId();

						if ((pruefeViewId.equals(viewId)) && (pruefeSecId != null)
								&& (pruefeSecId.equals(sndId)))
						{
							foundRef = viewReference;
							break;
						}

						if (foundRef != null)
						{
							break;
						}
					}
					if (foundRef != null)
					{
						break;
					}
				}
				if (foundRef != null)
				{
					break;
				}
			}
		}

		return foundRef;
	}
	
	
	/**
	 * Loggen in die Error Log von Eclipse.
	 * 
	 * @param msg
	 *          Zu loggende Meldung.
	 * @param serverity
	 *          Entsprechend {@link IStatus#WARNING}, {@link IStatus#ERROR} etc.
	 * @param t
	 *          {@link Throwable} wenn Verfgbar.
	 */
	public static void log(String msg, int serverity, Throwable t)
	{
		ILog log = Activator.getDefault().getLog();

		if (t == null)
		{
			log.log(new Status(serverity, Activator.PLUGIN_ID, msg));
		} else
		{
			log.log(new Status(serverity, Activator.PLUGIN_ID, msg, t));
		}
	}
}
