/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dlg;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.TreeMap;
import java.util.Map.Entry;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.TableEditor;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.sts.buv.plugin.workflow.tsf.Activator;
import de.sts.buv.plugin.workflow.tsf.clientueberw.DaVClientUeberwachung;
import de.sts.buv.plugin.workflow.tsf.clientueberw.DaVClientUeberwachung.ApplikationKapsel;
import de.sts.buv.plugin.workflow.tsf.dav.DaVKomTSFWeiterSchalten;
import de.sts.buv.plugin.workflow.tsf.dav.GUIModelTSFWeiterSchalten;
import de.sts.buv.plugin.workflow.tsf.dav.M_TSFWeiterSchalten;

/**
 * In dieser View werden alle Aufforderungen zum Weiterschalten von Abschnitten
 * dargestellt. Diese sind ber folgenden DaV-Zugriff verfgbar:
 * <ul>
 * <li>Typ: <code>TSFAbschnitt (typ.tsfAbschnitt)</code></li>
 * <li>Atg: <code>TSFWeiterSchalten (atg.tsfWeiterSchalten)</code></li>
 * <li>Asp: <code>TSFZustand (asp.tsfZustand)</code></li>
 *</ul>
 * 
 * @author HofmannJ
 * 
 */
public class ViewWeiterschaltenUebersicht extends ViewPart
{
	public static final String ID = "de.sts.buv.plugin.workflow.tsf.dlg.ViewWeiterschaltenUebersicht";

	/**
	 * PCL fr die Daten von {@link GUIModelTSFWeiterSchalten}
	 */
	private PropertyChangeListener _pclWeiterSchalten = null;

	/**
	 * PCL fr die bersicht der am Client angemeldeten Applikationen.
	 */
	private PropertyChangeListener _pclClientUeberw = null;

	/**
	 * Speichert die Kapseln mit den Weiterschaltenaufforderungen.
	 */
	private TreeMap<String, KapselWeiterSchalten> _map = new TreeMap<String, KapselWeiterSchalten>();

	private TableViewer _tv;

	private ButtonSelectionListener _buttonSelectionListener;

	/**
	 * Liste der Buttons, die vor einer Aktualisierung der Tabelle gecleared
	 * werden muss.
	 */
	private LinkedList<Button> _lButtons = new LinkedList<Button>();
	
	private Composite _parent;

	@Override
	public void createPartControl(Composite parent)
	{
		_parent = parent;
		
		 //Fr die F1 Taste
	    PlatformUI.getWorkbench().getHelpSystem().setHelp( parent, Activator.PLUGIN_ID + ".tsfWeiterschaltenAufforderungen" );
	    
		Group top = new Group(parent, SWT.NONE);
		top.setText("bersicht Weiterschalten");

		createTable(top);

		top.setLayout(new FillLayout());

		initListeners();

		getViewSite().getActionBars().getToolBarManager().add(
				getActionUebernehmen());
	}

	private void createTable(Composite parent)
	{
		Table table = new Table(parent, SWT.FULL_SELECTION | SWT.SINGLE
				| SWT.H_SCROLL | SWT.V_SCROLL);
		table.setHeaderVisible(true);
		table.setLinesVisible(true);

		TableColumn tcAbschnitt = new TableColumn(table, SWT.LEFT);
		tcAbschnitt.setText("Abschnitt:");
		tcAbschnitt.setWidth(200);

		TableColumn tcClient = new TableColumn(table, SWT.LEFT);
		tcClient.setText("Rechner:");
		tcClient.setWidth(150);

		TableColumn tcClientAktiv = new TableColumn(table, SWT.LEFT);
		tcClientAktiv.setText("Rechner Aktiv:");
		tcClientAktiv.setWidth(100);

		TableColumn tcUebernehmen = new TableColumn(table, SWT.LEFT);
		tcUebernehmen.setText("bernehmen");
		tcUebernehmen.setWidth(100);

		_tv = new TableViewer(table);
		_tv.setContentProvider(new CP());
		_tv.setLabelProvider(new LP());

		_tv.getTable().addListener(SWT.MeasureItem, new Listener()
		{
			public void handleEvent(Event event)
			{
				// height cannot be per row so simply set
				event.height = 30;
			}
		});

		verankereKontextMenu();
	}

	private void setData(KapselWeiterSchalten[] aKapselnWeiterSchalten)
	{
		synchronized (_lButtons)
		{
			Iterator<Button> it = _lButtons.iterator();
			while (it.hasNext())
			{
				Button button = it.next();
				button.removeSelectionListener(getButtonSelectionListener());
				button.dispose();
			}
			_lButtons.clear();
		}

		if (_tv != null)
		{
			_tv.setInput(aKapselnWeiterSchalten);

			if (aKapselnWeiterSchalten.length > 0)
			{
				TableItem[] items = _tv.getTable().getItems();

				for (int i = 0; i < items.length; i += 1)
				{
					TableEditor editor = new TableEditor(_tv.getTable());
					Button button = new Button(_tv.getTable(), SWT.PUSH);
					synchronized (_lButtons)
					{
						_lButtons.add(button);
					}
					button.setText("bernehmen");
					button.pack();

					editor.grabVertical = true;
					editor.minimumWidth = button.getSize().x;
					editor.horizontalAlignment = SWT.CENTER;
					editor.setEditor(button, items[i], 3);

					if ((items[i] != null) && (items[i].getData() != null)
							&& (items[i].getData() instanceof KapselWeiterSchalten))
					{
						KapselWeiterSchalten kapselWeiterSchaltenUebernehmen = (KapselWeiterSchalten) items[i]
								.getData();
						button.setData(kapselWeiterSchaltenUebernehmen);

						button.addSelectionListener(getButtonSelectionListener());
					}
				}
			}
		}
	}

	/**
	 * Aktualisiert die Darstellung des TableViewers.
	 */
	private void aktTableViewer()
	{
		if ((_tv != null) && (_tv.getContentProvider() != null))
		{

			Runnable run = new Runnable()
			{
				@Override
				public void run()
				{
					// _tv.setInput(getKapseln());
					setData(getKapseln());
				}
			};

			PlatformUI.getWorkbench().getDisplay().asyncExec(run);

		}
	}

	/**
	 * Verankert das Kontextmen an der Tabelle Weiterschalten.
	 * 
	 */
	private void verankereKontextMenu()
	{
		MenuManager menuMgr = new MenuManager("#PopupMenu");
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener()
		{
			public void menuAboutToShow(IMenuManager manager)
			{
				fuelleKontextMenu(manager);
			}
		});

		Menu menu = menuMgr.createContextMenu(_tv.getControl());
		_tv.getControl().setMenu(menu);
		getSite().registerContextMenu(menuMgr, _tv);
	}

	/**
	 * Fllt das Kontextmen, welches an der Benutzertabelle angehngt ist.
	 * Kontextsensitiv kann das Men mehr oder weniger Eintrge umfassen.
	 * 
	 * @param manager
	 *          {@link IMenuManager}, der fr die Darstellung des Kontextmens
	 *          zustndig ist.
	 */
	private void fuelleKontextMenu(IMenuManager manager)
	{
		manager.add(getActionUebernehmen());

		manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
	}

	private IAction getActionUebernehmen()
	{
		IAction aUebernehmen = new Action("bernehmen")
		{

			@Override
			public void run()
			{
				aktionUebernehmen();
			}

		};

		aUebernehmen.setId(ID + ".uebernehmen");
		aUebernehmen.setImageDescriptor(Activator
				.getImageDescriptor("icons/reload_16x16.png"));
		aUebernehmen
				.setToolTipText("bernimmt ber eine fr einen anderen Client bestimmten Weiterschalten-Aufforderung die Kontrolle.");

		return aUebernehmen;
	}

	private void aktionUebernehmen()
	{
		if (_tv != null)
		{
			ISelection selection = _tv.getSelection();
			if (selection instanceof StructuredSelection)
			{
				StructuredSelection ss = (StructuredSelection) selection;

				Object firstElement = ss.getFirstElement();
				if (firstElement != null)
				{

					if (firstElement instanceof KapselWeiterSchalten)
					{
						KapselWeiterSchalten kapsel = (KapselWeiterSchalten) firstElement;
						aktionUebernehmen(kapsel);
					}
				}
			}
		}
	}

	private void aktionUebernehmen(
			KapselWeiterSchalten kapselWeiterSchaltenUebernehmen)
	{
		M_TSFWeiterSchalten weiterSchaltenUebernehmen = kapselWeiterSchaltenUebernehmen
				.getWeiterSchalten();
		M_TSFWeiterSchalten duplicate = weiterSchaltenUebernehmen.duplicate(true);
		DaVKomTSFWeiterSchalten.getInstance().getModel().putTSFWeiterSchalten(
				duplicate);
	}

	public void initListeners()
	{
		DaVKomTSFWeiterSchalten.getInstance().getModel().addListener(
				getPCLWeiterSchalten());
		DaVClientUeberwachung.getInstance().addListener(getPCLClientUeberwachung());

		vglAppKapseln();
	}

	/**
	 * Liefert den {@link PropertyChangeListener} fr die Daten von
	 * {@link GUIModelTSFWeiterSchalten}.
	 * 
	 * @return {@link PropertyChangeListener}.
	 */
	public PropertyChangeListener getPCLWeiterSchalten()
	{
		if (_pclWeiterSchalten == null)
		{
			_pclWeiterSchalten = new PropertyChangeListener()
			{

				@Override
				public void propertyChange(PropertyChangeEvent evt)
				{
					if ((evt != null) && (evt.getPropertyName() != null))
					{
						if (evt.getPropertyName().equals(
								GUIModelTSFWeiterSchalten.PROP_INIT))
						{
							if ((evt.getNewValue() != null)
									&& (evt.getNewValue() instanceof M_TSFWeiterSchalten[]))
							{
								M_TSFWeiterSchalten[] aInitialeDaten = (M_TSFWeiterSchalten[]) evt
										.getNewValue();

								putKapseln(aInitialeDaten);

							}
						} else if (evt.getPropertyName().equals(
								GUIModelTSFWeiterSchalten.PROP_ADDED))
						{
							if ((evt.getNewValue() != null)
									&& (evt.getNewValue() instanceof M_TSFWeiterSchalten))
							{
								putKapsel((M_TSFWeiterSchalten) evt.getNewValue());
							}
						} else if (evt.getPropertyName().equals(
								GUIModelTSFWeiterSchalten.PROP_REMOVED))
						{
							if ((evt.getOldValue() != null)
									&& (evt.getOldValue() instanceof M_TSFWeiterSchalten))
							{
								M_TSFWeiterSchalten weiterSchalten = (M_TSFWeiterSchalten) evt
										.getOldValue();
								removeKapsel(weiterSchalten);
							}
						}
					}
				}
			};
		}

		return _pclWeiterSchalten;
	}

	public PropertyChangeListener getPCLClientUeberwachung()
	{
		if (_pclClientUeberw == null)
		{
			_pclClientUeberw = new PropertyChangeListener()
			{

				@Override
				public void propertyChange(PropertyChangeEvent evt)
				{
					if ((evt != null) && (evt.getPropertyName() != null))
					{
						vglAppKapseln();
					}
				}
			};
		}

		return _pclClientUeberw;
	}

	/**
	 * Kapsel mit der Information der WeiterschaltenAufforderung
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class KapselWeiterSchalten
	{
		/**
		 * Der WeiterSchalten-Befehl, um den es eigentlich geht.
		 */
		private M_TSFWeiterSchalten _weiterSchalten = null;

		/**
		 * Flag das anzeigt, ob das System der ID nach aktiv ist oder nicht.
		 */
		private boolean _systemAktiv = true;

		public KapselWeiterSchalten(M_TSFWeiterSchalten weiterSchalten)
		{
			this._weiterSchalten = weiterSchalten;
		}

		public M_TSFWeiterSchalten getWeiterSchalten()
		{
			return _weiterSchalten;
		}

		public String getPid()
		{
			String re = null;

			M_TSFWeiterSchalten weiterSchalten = getWeiterSchalten();
			if (weiterSchalten != null)
			{
				if (weiterSchalten.getSo() != null)
				{
					re = weiterSchalten.getSo().getPid();
				}
			}

			return re;
		}

		public String getClientId()
		{
			String re = null;

			M_TSFWeiterSchalten weiterSchalten = getWeiterSchalten();
			if ((weiterSchalten != null) && (weiterSchalten.getClientId() != null))
			{
				re = weiterSchalten.getClientId();
			}

			return re;
		}

		public boolean isSystemAktiv()
		{
			return _systemAktiv;
		}

		public void setSystemAktiv(boolean systemAktiv)
		{
			this._systemAktiv = systemAktiv;
		}

		@Override
		public String toString()
		{
			StringBuffer sb = new StringBuffer();

			M_TSFWeiterSchalten weiterSchalten = getWeiterSchalten();
			if (weiterSchalten != null)
			{
				sb.append(weiterSchalten.toString());
			} else
			{
				sb.append("-");
			}

			return sb.toString();
		}
	}

	private void putKapseln(KapselWeiterSchalten[] aKapseln)
	{
		if ((aKapseln != null) && (aKapseln.length > 0))
		{
			for (KapselWeiterSchalten kapsel : aKapseln)
			{
				if (kapsel != null)
				{
					synchronized (_map)
					{
						_map.put(kapsel.getPid(), kapsel);
					}
				}
			}

			vglAppKapseln();
			aktTableViewer();
		}
	}

	private void putKapsel(M_TSFWeiterSchalten weiterSchalten)
	{
		if (weiterSchalten != null)
		{
			putKapseln(new M_TSFWeiterSchalten[] { weiterSchalten });
		}
	}

	private void putKapseln(M_TSFWeiterSchalten[] aWeiterSchalten)
	{
		if ((aWeiterSchalten != null) && (aWeiterSchalten.length > 0))
		{
			LinkedList<KapselWeiterSchalten> lKapseln = new LinkedList<KapselWeiterSchalten>();

			for (M_TSFWeiterSchalten weiterSchalten : aWeiterSchalten)
			{
				KapselWeiterSchalten kapsel = new KapselWeiterSchalten(weiterSchalten);
				lKapseln.add(kapsel);
			}

			putKapseln(lKapseln.toArray(new KapselWeiterSchalten[lKapseln.size()]));
		}
	}

	private void removeKapsel(M_TSFWeiterSchalten weiterSchalten)
	{
		if (weiterSchalten != null)
		{
			KapselWeiterSchalten kapselToRemove = new KapselWeiterSchalten(
					weiterSchalten);
			removeKapsel(kapselToRemove);
		}
	}

	private KapselWeiterSchalten removeKapsel(KapselWeiterSchalten kapsel)
	{
		KapselWeiterSchalten removed = null;

		if (kapsel != null)
		{
			removed = removeKapsel(kapsel.getPid());
		}

		return removed;
	}

	private KapselWeiterSchalten removeKapsel(String pid)
	{
		KapselWeiterSchalten removed = null;
		synchronized (_map)
		{
			removed = _map.remove(pid);
		}

		if (removed != null)
		{
			aktTableViewer();
		}

		return removed;
	}

	private KapselWeiterSchalten[] getKapseln()
	{
		LinkedList<KapselWeiterSchalten> lKapseln = new LinkedList<KapselWeiterSchalten>();

		synchronized (_map)
		{
			Iterator<Entry<String, KapselWeiterSchalten>> it = _map.entrySet()
					.iterator();
			while (it.hasNext())
			{
				Entry<String, KapselWeiterSchalten> entry = it.next();
				if (entry.getValue() != null)
				{
					lKapseln.add(entry.getValue());
				}
			}
		}

		return lKapseln.toArray(new KapselWeiterSchalten[lKapseln.size()]);
	}

	private void vglAppKapseln()
	{
		KapselWeiterSchalten[] kapseln = getKapseln();

		// Kapseln zurcksetzen
		for (KapselWeiterSchalten kapsel : kapseln)
		{
			kapsel.setSystemAktiv(false);
		}

		// Kapseln prfen
		ApplikationKapsel[] apps = DaVClientUeberwachung.getInstance()
				.getApplikationen();
		for (ApplikationKapsel appKapsel : apps)
		{
			if ((appKapsel != null) && (appKapsel.getClientId() != null))
			{
				for (KapselWeiterSchalten kapsel : kapseln)
				{
					if ((kapsel != null) && (kapsel.getClientId() != null))
					{
						if (appKapsel.getClientId().equals(kapsel.getClientId()))
						{
							kapsel.setSystemAktiv(true);
						}
					}
				}
			}
		}

		// Aktualisieren
		aktTableViewer();
	}

	private ButtonSelectionListener getButtonSelectionListener()
	{
		if (_buttonSelectionListener == null)
		{
			_buttonSelectionListener = new ButtonSelectionListener();
		}

		return _buttonSelectionListener;
	}

	private class ButtonSelectionListener implements SelectionListener
	{

		@Override
		public void widgetDefaultSelected(SelectionEvent e)
		{
		}

		@Override
		public void widgetSelected(SelectionEvent e)
		{
			Object data = e.widget.getData();

			// if ((data != null) && (data instanceof M_TSFWorkflowAbschnitt))
			// {
			// M_TSFWorkflowAbschnitt workflowAbschnitt = (M_TSFWorkflowAbschnitt)
			// data;
			//
			// M_TSFAbschnittBefehl befehl = new M_TSFAbschnittBefehl(Defs
			// .getClientId(), workflowAbschnitt.getAbschnitt(),
			// E_TSFBefehl.Freigabe, E_TSFProgrammZustand.Undefiniert);
			// DaVKomTSFAbschnittBefehl.getInstance().sendeAbschnittBefehl(befehl);
			// }
			if ((data != null) && (data instanceof KapselWeiterSchalten))
			{
				KapselWeiterSchalten kapselWeiterSchalten = (KapselWeiterSchalten) data;
				aktionUebernehmen(kapselWeiterSchalten);
			}
		}

	}

	private class CP implements IStructuredContentProvider
	{

		@Override
		public Object[] getElements(Object inputElement)
		{
			Object[] re = null;

			if (inputElement instanceof KapselWeiterSchalten[])
			{
				KapselWeiterSchalten[] aKapseln = (KapselWeiterSchalten[]) inputElement;
				re = aKapseln;
			}

			return re;
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
		{
		}

	}

	private class LP implements ITableLabelProvider
	{

		@Override
		public Image getColumnImage(Object element, int columnIndex)
		{
			Image re = null;

//			if (element instanceof KapselWeiterSchalten)
//			{
//				KapselWeiterSchalten kapsel = (KapselWeiterSchalten) element;
//				switch (columnIndex)
//				{
//				case 2:
//					if (kapsel.isSystemAktiv())
//					{
//						re = Activator.getImageDescriptor("icons/online_16x16.png").createImage();
//					} else
//					{
//						re = Activator.getImageDescriptor("icons/offline_16x16.png").createImage();
//
//					}
//					break;
//				}
//			}

			return re;
		}

		@Override
		public String getColumnText(Object element, int columnIndex)
		{
			String re = null;

			if (element instanceof KapselWeiterSchalten)
			{
				KapselWeiterSchalten kapsel = (KapselWeiterSchalten) element;

				switch (columnIndex)
				{
				case 0:
					re = kapsel.toString();
					break;
				case 1:
					re = kapsel.getClientId();
					break;
				case 2:
					if (kapsel.isSystemAktiv())
					{
						re = "Ja";
					} else
					{
						re = "Nein";
					}
					break;

				default:
					break;
				}
			}

			return re;
		}

		@Override
		public void addListener(ILabelProviderListener listener)
		{
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public boolean isLabelProperty(Object element, String property)
		{
			return false;
		}

		@Override
		public void removeListener(ILabelProviderListener listener)
		{
		}

	}

	@Override
	public void dispose()
	{
		super.dispose();

		// Listener fr Weiterschalten abmelden.
		DaVKomTSFWeiterSchalten.getInstance().getModel().removeListener(
				getPCLWeiterSchalten());
	}

//	@Override
//	public void setFocus()
//	{
//	}
	
	@Override
	public void setFocus()
	{
	  //Fr die F1 Taste
	  if (_parent != null)
	    _parent.setFocus();
	}

}
