/*  Copyright (C) 2004 by Friederich Kupzog Elektronik & Software
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    
	Author: Friederich Kupzog  
	fkmk@kupzog.de
	www.kupzog.de/fkmk
*/
 
 
package de.kupzog.ktools.kprint.examples;
import org.eclipse.swt.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.printing.*;

import de.kupzog.ktools.kprint.boxes.*;
import de.kupzog.ktools.kprint.gui.IconSource;
import de.kupzog.ktools.kprint.gui.PrintPreview;

/**
 * This example shows the use of KPrint classes
 * to create a form that can be filled by hand 
 * once printed.
 * 
 * @author Friederich Kupzog
 *
 */
public class FormExample {
	private Display d;
	
	public FormExample() {

		d=new Display();


		// create the document
		PDocument doc = new PDocument("Form");
		
		// put text on it
		PTextBox t;
		
		t = new PTextBox(doc);
		t.setText("Title");
		new PHSpace(doc, PBox.POS_RIGHT, 0.3);
		createBoxRow(doc, 4);
		
		new PHSpace(doc, PBox.POS_RIGHT, 0.3);
		t = new PTextBox(doc, PBox.POS_RIGHT);
		t.setText("Initials");
		new PHSpace(doc, PBox.POS_RIGHT, 0.3);
		createBoxRow(doc, 4);

		new PHSpace(doc, PBox.POS_RIGHT, 0.3);
		t = new PTextBox(doc, PBox.POS_RIGHT);
		t.setText("First Name");
		new PHSpace(doc, PBox.POS_RIGHT, 0.3);
		createBoxRow(doc, 19);
		
		new PVSpace(doc, 0.2);

		t = new PTextBox(doc);
		t.setText("Surname");
		new PHSpace(doc, PBox.POS_RIGHT, 0.3);
		createBoxRow(doc, 35);
		
		new PVSpace(doc, 0.2);

		t = new PTextBox(doc);
		t.setText("Maiden name/former surname");
		new PHSpace(doc, PBox.POS_RIGHT, 0.3);
		createBoxRow(doc, 27);
		

	
		
		// open the print preview on this document
		PrintPreview pr = new PrintPreview(null, "Test", IconSource.getImage("print"), doc);
		pr.open();
		
		// dispose the document in the end
		d.dispose();
	}

	private void createBoxRow(PDocument doc, int length) {
		createFirstBox(doc);
		for (int i = 0; i < length-1; i++) {
			createBox(doc);
		}
	}
	
	private PBox createFirstBox(PDocument parent)
	{
		// create a box with fixed width of 0.4 cm
		PLittleTextBox result = new PLittleTextBox(parent, PBox.POS_RIGHT, 0, 0.4);
		result.setText(" "); // neccessary because otherwise the box would have no content
							 // whick leads to a height of 0.
		
		// set border lines all around
		result.getBoxStyle().lines = new double[] {0.01, 0.01, 0.01, 0.01};
		
		return result;
	}

	private PBox createBox(PDocument parent)
	{
		// create a box with fixed width of 0.4 cm
		PLittleTextBox result = new PLittleTextBox(parent, PBox.POS_RIGHT, 0, 0.4);
		result.setText(" "); // neccessary because otherwise the box would have no content
		 // whick leads to a height of 0.

		
		// set border lines all around except on the left
		result.getBoxStyle().lines = new double[] {0.01, 0.01, 0.01, 0.00};
		
		return result;
	}



	/**
	 * This function would print the document witout the print preview.
	 * @param doc
	 */
	public void print(PDocument doc)
	{
		PrintDialog dialog = new PrintDialog(null, SWT.BORDER);
		PrinterData data = dialog.open();
		if (data == null) return;
		if (data.printToFile) {
			data.fileName = "print.out"; // you probably want to ask the user for a filename
		}
		
		Printer printer = new Printer(data);
		GC gc = new GC(printer);
		PBox.setParameters(gc, printer, printer.getDPI(), 100);
		if (printer.startJob("DoSys Druckauftrag")) 
		{
			printer.startPage();
			doc.layout();
			doc.draw(1);
			printer.endJob();
		}
		gc.dispose();

	}

	public static void main(String[] args) {
		new FormExample();
	}
}